/* This program read the hex file, find if the position lines, the lat and lon are very
 * close, if yes, then remove the position line which is from the new(from cdrom) and replace
 * it with one from old(/a1 real time production recent directory).
 *
 * @author: Xiangdong Xia
 * @date: Jan 26,27, 2006
 */



import java.io.*;
import java.util.*;


public class CleanHex{
	static String inputCDRomHexFileName;
	static String inputOldHexFileName;
	static String outputHexFileName;
	static List hexLineList;//from CDRom Hex file
	static List oldHexLineList;//from /a1 production Hex file
	static boolean debug=true;
	
	public static void main(String args[]){
		if(args.length!=3){
			System.out.println("Usage: java CleanHex inputCDRomHexFileName inputOldHexFileName outputHexFileName");
			System.exit(1);
		}
		inputCDRomHexFileName=args[0];
		inputOldHexFileName=args[1];
		outputHexFileName=args[2];
		CleanHex ch=new CleanHex();		
		hexLineList=ch.readCDRomHexFile(inputCDRomHexFileName);		
		oldHexLineList=ch.readOldHexFile(inputOldHexFileName);		
		ch.cleanPositionLines();
		ch.writeOutputHexFile(outputHexFileName);
	}
	
	/* read the inputCDRomHexFileName, read each line and put into hexLineList
	 * the HexLine is a class to parse each Hex line
	 */
	public List readCDRomHexFile(String inputCDRomHexFileName){
     String str;
   	 List hexLineList=new ArrayList();
   	 HexLine hl;
   	 FileReader fr=null;
   	 BufferedReader in=null;
   	 try{
	     fr=new FileReader(inputCDRomHexFileName);
         in= new BufferedReader(fr);
         
         while ((str= in.readLine()) != null){
         	//System.out.println(str);
         	hl=new HexLine(str);
         	hexLineList.add(hl);         	     
         }
         in.close();
      	 fr.close();
      }catch(java.io.IOException e) {
        e.printStackTrace();
      }catch(Exception ex){    	
    	System.out.println(ex.getMessage());
        System.exit(2);   
      }
     return hexLineList;
	}
	
	
	/* read the inputOldHexFileName, read each line and put into oldHexLineList	 
	 * the oldHexLineList will only contain position lines
	 */
	public List readOldHexFile(String inputOldHexFileName){
     String str;
   	 List hexLineList=new ArrayList();
   	 HexLine hl;
   	 FileReader fr=null;
   	 BufferedReader in=null;
   	 try{
	     fr=new FileReader(inputOldHexFileName);
         in= new BufferedReader(fr);
         
         while ((str= in.readLine()) != null){         	
           hl=new HexLine(str);	
           if(hl.getHexData().equals("0000000000000000000000000000000000000000000000000000000000000000")){
           	//it is a position line
           	//if(debug) System.out.println(str);
         	hexLineList.add(hl);         	              	
           }	         	
         }
         
         in.close();
      	 fr.close();
      }catch(java.io.IOException e) {
        e.printStackTrace();
      }catch(Exception ex){    	
    	System.out.println(ex.getMessage());
        System.exit(2);   
      }
     return hexLineList;
	}
	
	/* Clean the position lines if the lat and lon are too close to each other
	 * 
	 */
	private void cleanPositionLines(){
		double latNew,latOld,lonNew,lonOld,epson=0.00000001;
		long timeNew,timeOld;
		String satNew,satOld;
		HexLine hlNew=null;
		HexLine hlOld=null;
		
		for(int i=0;i<hexLineList.size();i++){
			hlNew=(HexLine)(hexLineList.get(i));
			if(hlNew.isPositionLine()){
              latNew=Double.parseDouble(hlNew.getLat());
			  lonNew=Double.parseDouble(hlNew.getLon());
			  timeNew=hlNew.getTime();
			  satNew=hlNew.getSat();
			  	
			  for(int j=0;j<oldHexLineList.size();j++){		  	  
			  	  hlOld=(HexLine)(oldHexLineList.get(j));				  	  				  
				  latOld=Double.parseDouble(hlOld.getLat());
				  lonOld=Double.parseDouble(hlOld.getLon());
				  timeOld=hlOld.getTime();
			      satOld=hlOld.getSat();
			      
			      			      
			      //update hexLineList
			      if(timeNew==timeOld && satNew.equals(satOld)){		      
			        boolean positionEqual=((hlNew.getLat()).equals(hlOld.getLat())) && ((hlNew.getLon()).equals(hlOld.getLon()));
			        boolean tooClose=(!positionEqual) && ((Math.abs(latNew-latOld)-0.001)<epson && (Math.abs(lonNew-lonOld)-0.001)<epson||
			                         ((Math.abs(latNew-latOld)<epson) && (Math.abs(lonNew-lonOld)-0.001)<epson)||
			                         ((Math.abs(latNew-latOld)-0.001<epson) && (Math.abs(lonNew-lonOld)<epson)));			        
			        if(tooClose){			         
			        HexLine updatedHexLine=new HexLine();
			        
			        if(debug){
			        	System.out.println("========================================");			        
				        System.out.println("old position line from /a1:");
				        System.out.println(hlOld.getHexLine());
				        System.out.println("new position line from cdrom:");
				        System.out.println(hlNew.getHexLine());
				        System.out.println("positionEqual="+positionEqual);
			            System.out.println("tooClose="+tooClose);
				        System.out.println("old will replace new for this position line in new cdrom hex file");
				        System.out.println("========================================");
			        }			        			        
		        	updatedHexLine.setHexLine(hlOld.getHexLine());
		        	hexLineList.set(i,updatedHexLine);			      				        
		        	}
			      }
		     }//end of for(j	
		  	}
	    }//end of for(i
	}    
	    private void writeOutputHexFile(String outputHexFileName){
	    	FileWriter fw=null;
	    	PrintWriter pw=null;
	    	HexLine hl=null;
	    	String hexString=null;
	        try{
		       fw=new FileWriter(outputHexFileName);
               pw=new PrintWriter(fw);
               
               //write the contents of the updated hexLineList to output file
               for(int i=0;i<hexLineList.size();i++){
               	  hl=(HexLine)hexLineList.get(i);
               	  hexString=hl.getHexLine();
               	  pw.println(hexString);
               }
               pw.close();
               fw.close();
            }catch(IOException ioe){
   	           ioe.printStackTrace();
               System.exit(2);    	
            }
	    	
	    }
	
}
