/*  GetHexFileNamesFromPrv.java
 *  
 *  Extract hex filenames from a particular PRV file, first find the unique transmission IDs,
 *  then find its corresponding aomlID based on the floatno_argoid_list, finally construct
 *  the hex filenames based on aomlID and transmissionID.
 *  
 *  Exit status:
 *  0--success
 *  1--command line error
 *  2--I/O related error
 *  3--more than one aomlID are associated with the same transmissionID
 *  
 *  Usage:
 *  java GetHexFileNamesFromPrv inputPrvFileName floatno_argoid_list hexFileNamesListFile
 *  @author: Xiangdong Xia
 *  @date:   Feb-1-2006
 */


import java.util.*;
import java.io.*;
import java.util.regex.*;
import java.text.*;

public class GetHexFileNamesFromPrv{
	static String inputPrvFileName, floatno_argoid_list, hexFileNamesListFile;
	Set transmissionIDSet=new TreeSet();
	List hexFileNameList=new ArrayList();
	static String programNum;
	List aomlIDList;
	List transmissionIDList;
	
	public static void main(String args[]){
	    if(args.length!=3){
	    	System.out.println("Usage:java GetHexFileNamesFromPrv inputPrvFileName floatno_argoid_list hexFileNamesListFile");
	    	System.exit(1);
	    }
	    inputPrvFileName=args[0];
	    floatno_argoid_list=args[1];
	    hexFileNamesListFile=args[2];
	    	
		GetHexFileNamesFromPrv ghf=new GetHexFileNamesFromPrv();
		programNum=ghf.getProgramNum();
		ghf.readFloatNoArgoidList();
		ghf.getTransmissionIDs();
		ghf.getHexFileNames();
		ghf.writeOutputFile();
	}
	
	private String getProgramNum(){
	   File f=new File(inputPrvFileName);
	   String fileName=f.getName();	
	   StringTokenizer st = new StringTokenizer(fileName,".");
	   String firstPart=null;
	   String programNum=null;
	   
	   while (st.hasMoreTokens()) {
		  firstPart=st.nextToken();		  		  
		  break;
	   }
	   if(firstPart!=null){
	   	 programNum=firstPart.substring(3);//skip "prv"
	   }
	   return programNum;  
	}
	
	/* get unique TransmissionIDs from a prv file
	 */
	private void getTransmissionIDs(){
   	 FileReader fr=null;
   	 BufferedReader in=null;
   	 String str=null;
   	 String transmissionID=null;
   	 String dummy=null;
   	 try{
	     fr=new FileReader(inputPrvFileName);
         in= new BufferedReader(fr);
         
         while ((str= in.readLine()) != null){         	           
           if(str.startsWith(programNum+" ")){
           	  
           	  StringTokenizer st = new StringTokenizer(str);
           	  while (st.hasMoreTokens()) {
		         dummy=st.nextToken();
		         transmissionID=st.nextToken().trim();
		         break;
		      }              
           	  
         	  transmissionIDSet.add(transmissionID);         	              	
           }	         	
         }
         
         in.close();
      	 fr.close();
      }catch(java.io.IOException e) {
        e.printStackTrace();
      }catch(Exception ex){    	
    	System.out.println(ex.getMessage());
        System.exit(2);   
      }
		
	}
	
	private void getHexFileNames(){
	  List transmissionIDSetList=new ArrayList();
	  transmissionIDSetList.addAll(transmissionIDSet);	
	  String transID=null;
	  String aomlID=null;
	  String hexFileName=null;
	  
	  for(int i=0;i<transmissionIDSetList.size();i++){
	    transID=(String)(transmissionIDSetList.get(i));
	    aomlID=findAOMLID(transID);
	    if(aomlID!=null){	    
	      hexFileName=aomlID+"_"+transID+".hex";
	      hexFileNameList.add(hexFileName);
	    }  
	  }		  
	}
	
	/* find aomlID from transmissionID, if more than one aomlID found for the same
	 * transmissionID, report error and exit
	 */
	private String findAOMLID(String transmissionID){
		int count=0;
		String aomlID=null;
		for(int i=0;i<transmissionIDList.size();i++){
			if(transmissionIDList.get(i).equals(transmissionID)){
				aomlID=(String)(aomlIDList.get(i));
				//System.out.println("the transmission ID="+transmissionID+" aomlID="+aomlID);
				count++;
			}
		}
		if(count>1) {
			System.out.println("more than one AOML IDs are associated with the same transmission ID");			
			System.exit(3);
		}
		return aomlID;
	}
	/* read floatno_argoid_list
	 */
	
	private void readFloatNoArgoidList(){
   	 FileReader fr=null;
   	 BufferedReader in=null;
   	 String str=null;
   	 String transmissionID=null;
   	 String aomlID=null;
     aomlIDList=new ArrayList();
     transmissionIDList=new ArrayList();
     
   	 try{
	     fr=new FileReader(floatno_argoid_list);
         in= new BufferedReader(fr);
         
         while ((str= in.readLine()) != null){         	           
           	  StringTokenizer st = new StringTokenizer(str);
           	  while (st.hasMoreTokens()) {
		         aomlID=st.nextToken().trim();
		         transmissionID=st.nextToken().trim();
		         aomlIDList.add(aomlID);
		         transmissionIDList.add(transmissionID);
		         break;
		      }                         	           	  
         }         
         in.close();
      	 fr.close();
      	       	 
      }catch(java.io.IOException e) {
        e.printStackTrace();
      }catch(Exception ex){    	
    	System.out.println(ex.getMessage());
        System.exit(2);   
      }
		
	}
	
	private void writeOutputFile(){
	    	FileWriter fw=null;
	    	PrintWriter pw=null;	    	
	    	
	        try{
		       fw=new FileWriter(hexFileNamesListFile);
               pw=new PrintWriter(fw);
               
               //write the contents of the updated hexLineList to output file
               for(int i=0;i<hexFileNameList.size();i++){               	  
               	  pw.println(hexFileNameList.get(i));
               }
               pw.close();
               fw.close();
            }catch(IOException ioe){
   	           ioe.printStackTrace();
               System.exit(2);    	
            }	    	
	 }
	  	
	
	
}
