#include <stdio.h>

/*------------------------------------------------------------------------*/
/* function to compute the compensated phase difference                   */
/*------------------------------------------------------------------------*/
/**
   This function computes the compensated phase difference of the Aanderaa
   optode according to the following equation.

      \begin{verbatim}
      DPhase = B0 + B1*(BPhase  + B2*(BPhase  + B3*BPhase))
      \end{verbatim}

      \begin{verbatim}
      input:
         BPhase......The BPhase as reported by the optode.
         PhaseCoef...The 4-vector of coefficients supplied by Aanderaa.

      output:
         This function returns the compensated phase difference.
      \end{verbatim}
*/

float swiftpoly(float x, float coeff[],int n);

float OptodeDPhase(float BPhase, float *PhaseCoef)
{
   int debug=0;

   /* compute the compensated phase difference */
   float DPhase = swiftpoly(BPhase,PhaseCoef,3);
   if (debug==1) printf ("\nOptodeDphase: PhaseCoef = %e %e %e %e\n",PhaseCoef[0],PhaseCoef[1],PhaseCoef[2],PhaseCoef[3]);
   if (debug==1) printf ("OptodeDphase: BPhase = %f, DPhase = %f\n", BPhase, DPhase);
   return DPhase;
}
