/*****************************************************************
  Revision Date: 01-Nov-11 (ER - PTS1 triplet (diagnostic) moved from end of 
                                  drift to start of ascent in 2004
                 28-Oct-11 (ER -Fixed bug calculating BIT PRESSURE AFTER RESET OFFSET
                 27-Oct-11 (ER -Fixed bug calculating CHANGE IN PRESSURE FIRST SEEK
                 25-Oct-11 (ER -Fixed bug calculating of MOTOR CURRENT OUT TO ASCEND,
                                MOTOR CURRENT AT END OF ASCENT
                 20-Oct-11 (ER -Fixed bug calculating PRESSURE AT END OF ASCENT 
                 12-Oct-10 (ER -For some SOLO_TS12, psurfgain is used instead 
                                of pgain)
                 26-AUG-10 (ER -Replace parameter name 'START SURFACE PRESSURE (DBAR)'
                                 with 'SURFACE PRES BEFORE RESET(PREV CYCLE)' 
                               -Replace parameter name 
                                 'START SURFACE PRES AFTER RESET (DBAR)'
                                 with 'SURFACE PRES AFTER RESET(PREV CYCLE)' 
                 30-Dec-09 (ER - change function get_meta_file_name to adapt to
                                 variable length transmission ids. 
                 20-Dec-07 (CS - fix temperature overflow correction and simplify
                                 salinity overflow correction
                 22-May-07 (XXD -start surface pressure added)
                 13-Jun-05 (XXD -change the pressure computation for bin=1)
                 13-May-04 (XXD- drift salinity use different sgain from profile)
                 09-Aug-02 (CS - As for TS9 tgain (sgain) have to be
                                 devided by 4 in calculation of some
                                 header T (S) -> bug of SOLO_TS12
                                 changed compute_sio_pressure, since
                                 TS12 has the pressure at the bottom
                                 of a bin -> different p-equation)
                 06-Aug-02 (DB - started with CNT2PHY_TS2)

  cnt2phy_ts12
  ===========
  Converts count to physical units for Type I floats

******************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256  //Maximum length of each scan read from *.str
//#define value_col 40    //Number of characters for start of line to value column

char* strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

FILE *input_file, *out_file, *error_file, *trace_file, *meta_file, *list_file;

char PRGNAME[] ="cnt2phy_ts12";
char buffer[512];

struct sio_cals { double pgain;
                  double psurfgain;
                  double poff;
                  double tgain;
                  double toff;
                  double sgain;
                  double soff;
                  double pmp_gain;
                  double pmp_off;
                  double cpugain;
                  double cpuoff;
                  double blok;
                  double av1;
                  double av2;
                  double pb1;
                  double pb2;
                };

union u_cals { struct sio_cals s;
               double c[sizeof(struct sio_cals)/sizeof(double)];
             };

struct solo_ts_meta_struct { int AOML_number;
                             int WMO_id;
                             int ARGOS_id;
                             float cycle_time;
                             float start_time;
                             union u_cals cals;};

/******************************************************************************
  Computes vacuum for TS12 SOLO floats in inches Hg
******************************************************************************/
float compute_vacuum (float v_counts)
{
	return (float)(0.1 * v_counts);
}

/******************************************************************************
  Computes battery voltages for TS12 SOLO floats
******************************************************************************/
float compute_voltage (float v_counts)
{
	return (float)(0.1 * v_counts);
}

/******************************************************************************
  Computes electrical current for TS12 SOLO floats in milliamps
******************************************************************************/
float compute_current (float i_counts)
{
	return (float)(1.0 * i_counts);
}

/******************************************************************************
  Computes salinity for SIO salinity floats from raw salinity counts
******************************************************************************/
float compute_sio_salinity (struct solo_ts_meta_struct *s, float s_counts)
{
	return (float)((s->cals.s.sgain * s_counts + s->cals.s.soff)*0.001);
}

/******************************************************************************
  Routine for computing temperature for SIO floats
******************************************************************************/
float compute_sio_temperature (struct solo_ts_meta_struct *s, float t_counts)
{
	return (float)((s->cals.s.tgain * t_counts + s->cals.s.toff)*0.001);
}

/******************************************************************************
  Routine for computing pressure for SIO floats
******************************************************************************/
float compute_sio_pressure (struct solo_ts_meta_struct *s, int p_counts)
{
 int i;
 float p;
 
 if(p_counts==1)
          p=(s->cals.s.blok+5)/2;
 else if (p_counts <= s->cals.s.pb1){
	 i = (int) (p_counts*s->cals.s.blok -s->cals.s.blok/2 );
	 p=(float)i;
 }	 
 else if (p_counts <= s->cals.s.pb2){
	 i = (int) ((p_counts - s->cals.s.pb1)*s->cals.s.av1*s->cals.s.blok +
	      s->cals.s.pb1*s->cals.s.blok -s->cals.s.av1*s->cals.s.blok/2 );
	 p=(float)i;     
 }	      
 else{
	 i = (int) ((p_counts - s->cals.s.pb2)*s->cals.s.av2*s->cals.s.blok +
	     (s->cals.s.pb2 - s->cals.s.pb1)*s->cals.s.av1*s->cals.s.blok +
		  s->cals.s.pb1*s->cals.s.blok -s->cals.s.av2*s->cals.s.blok/2 );
         p=(float)i; 		  
 }
 return p;
}


/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char *strupr(char* s)
{
  char* p = s;

  while (*p) {
    if (islower((int)*p)) 
      *p = toupper((int)*p);

    ++p;
  }

  return s;
}

/******************************************************************************
  Converts data from counts to physical units
******************************************************************************/
int convert_sio_ctd_data(struct solo_ts_meta_struct *s, char *output_name)
{
//  int i1=0,i2=0,i3=0,i4=0,ip[128],it[128],ic[128],iflg[128];
//TEST CS
  int i1=0,i2=0,i3=0,i4=0,ip[128],iflg[128];
  float it[128],ic[128];
  float a1,a2,a3,a2sav;
  char *ptr_phy_filename,col_space[]="   0",*ptr;

  if ((out_file = fopen(output_name,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file," CONVERT_TYPE12_DATA: error opening output file %s\n",
               output_name);

    if (error_file != NULL)
      fprintf (error_file," CONVERT_TYPE2_DATA: error opening output file %s\n",
               output_name);

    return 1;
  }

  if (trace_file != NULL)
    fprintf (trace_file," CONVERT_TYPE12_DATA: output file %s successfully opened",output_name);

  printf ("\nOutput file %s successfully opened",output_name);


  // Write Output Profile Physical Unit File Name to List File
  // Check if last occurence of '/' in buffer is NULL
  if ((ptr_phy_filename = strrchr(output_name, '/')) == NULL)
  {
    printf ("\nCan't get phy file basename %s", output_name);

    if (trace_file != NULL)
      fprintf (trace_file,
               "\n%s : Can't get phy file basename %s", PRGNAME, output_name);

    if (error_file != NULL)
      fprintf (error_file,
               "\n%s: Can't get phy file basename %s", PRGNAME, output_name);

    return 1;
  } // endif: ptr_phy_filename

  ptr_phy_filename++; // Increment file name pointer
  fprintf (list_file,"%s\n", ptr_phy_filename); // Write file name to list file


  // Reads a line from input_file into buff and check if NULL
  while (fgets(buffer,sizeof(buffer),input_file) != NULL)
  {
    if (strstr(strupr(buffer),"DRIFT TEMPERATURE") != NULL)
    {
        if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
           return(1);
        a1=compute_sio_temperature(s,(float) i1);
        a2=compute_sio_temperature(s,(float) i2);

        sprintf (buffer,"DRIFT TEMPERATURE (DEG C)               %-6.3f %-6.3f\n",
        a1,a2);
        fwrite (strupr(buffer),1,strlen(buffer),out_file);
        continue;
    }

    if (strstr(buffer,"P,T,S") != NULL)
    {
      if (sscanf(&buffer[40],"%d %d %d",&i1,&i2,&i3) != 3)
         return(1);

      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
// have to divide counts by 4 here (bug of SOLO_TS12)
      a2=compute_sio_temperature(s,((float) i2)/4);
// have to divide counts by 4 here (bug of SOLO_TS12)
      a3=compute_sio_salinity(s,((float) i3)/4);
      while (a3>41)
      {
         if (trace_file != NULL)
//            fprintf (trace_file,"\n A) salinity > 41, subtract 1024 from counts\n");
         i3=i3-1024;
// have to divide counts by 4 here (bug of SOLO_TS12)
         a3=compute_sio_salinity(s,((float) i3)/4);
      }

      if ((ptr = strstr(buffer,"(COUNTS)")) == NULL)
          buffer[40] = '\0';
      else
          *ptr = '\0';
      fprintf (out_file,"%-40s",buffer);
      sprintf (buffer,"%-6.1f %-6.3f %-6.3f\n",a1,a2,a3);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"DRIFT SALINITY") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
        return(1);
// have to divide counts by 4 here (bug of SOLO_TS12)
      
      a1=compute_sio_salinity(s,((float) i1)/4);
      a2=compute_sio_salinity(s,((float) i2)/4);
      
//      printf("i1=%d i2=%d a1=%f a2=%f\n",i1,i2,a1,a2);
      
      while (a1>41)
      {
         if (trace_file != NULL)
//            fprintf (trace_file,"\n B) salinity > 41, subtract 1024 from counts\n");
         i1=i1-1024;
// have to divide counts by 4 here (bug of SOLO_TS12)
         a1=compute_sio_salinity(s,((float) i1)/4);
      }
      while (a2>41)
      {
         if (trace_file != NULL)
//            fprintf (trace_file,"\n C) salinity > 41, subtract 1024 from counts\n");
         i2=i2-1024;
// have to divide counts by 4 here (bug of SOLO_TS12)	 
         a2=compute_sio_salinity(s,((float) i2)/4);
      }

      sprintf (buffer,"DRIFT SALINITY (PSU)                    %-6.3f %-6.3f\n",
      a1,a2);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"SURF PRES BEF RESET(PREV CYC)(COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      if (s->cals.s.psurfgain > 0.001 ) {
        a1 = (float) (s->cals.s.psurfgain*i1 + s->cals.s.poff);
      }
      else {
        a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      }
      if (a1 < 0.0 && a1 > -0.05) {
        a1 = 0.0;
      }
      sprintf (buffer,"SURFACE PRES BEFORE RESET(PREV CYCLE)   %-6.1f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"SURF PRES AFTER RESET(PREV CYC)(COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      if (s->cals.s.psurfgain > 0.001 ) {
        a1 = (float) (s->cals.s.psurfgain*i1 + s->cals.s.poff);
      }
      else {
        a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      }
      if (a1 < 0.0 && a1 > -0.05) {
        a1 = 0.0;
      }
      sprintf (buffer,"SURFACE PRES AFTER RESET(PREV CYCLE)    %-6.1f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"BIT PRESSURE AFTER RESET OFFSET (CNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      a1 = ((float)i1)/50.0-10.0;
      if (a1 < 0.0 && a1 > -0.05) {
        a1 = 0.0;
      }                
      sprintf (buffer,"BIT PRESSURE AFTER RESET OFFSET (DBAR)  %-6.2f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }


    if (strstr(buffer,"PRESSURE AT END OF ASCENT (COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      if (s->cals.s.psurfgain > 0.001 ) {
        a1 = (float) (s->cals.s.psurfgain*i1 + s->cals.s.poff);
      }
      else {
        a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      }
      if (a1 < 0.0 && a1 > -0.05) {
        a1 = 0.0;
      }               
      sprintf (buffer,"PRESSURE AT END OF ASCENT (DBAR)        %-6.1f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }


    if ((strstr(buffer,"CHANGE IN PRESSURE FIRST SEEK") != NULL) )
    {
      i3 = sscanf(&buffer[40],"%d %d",&i1,&i2);

      if (((ptr = strstr(buffer,"(COUNTS)")) != NULL) || ((ptr = strstr(buffer,"(CNTS)")) != NULL))
      {
        *ptr = '\0';
        strcat(buffer,"(DBAR)");
      }
      else
        buffer[40] = '\0';

      fprintf (out_file,"%-40s",buffer);
      a1 = (float) (s->cals.s.pgain*i1 );
      if (i3 == 1)
        fprintf (out_file,"%-6.1f\n",a1);
      else if (i3 == 2)
      {
        a2 = (float) (s->cals.s.pgain*i2 );
        fprintf (out_file,"%-6.1f %-6.1f\n",a1,a2);
      }
      continue;
    }


    if ((strstr(buffer,"PRES") != NULL) && (strstr(buffer,"AIR") == NULL))
    {
      i3 = sscanf(&buffer[40],"%d %d",&i1,&i2);

      if (((ptr = strstr(buffer,"(COUNTS)")) != NULL) || ((ptr = strstr(buffer,"(CNTS)")) != NULL))
      {
        *ptr = '\0';
        strcat(buffer,"(DBAR)");
      }
      else
        buffer[40] = '\0';

      fprintf (out_file,"%-40s",buffer);
      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      if (i3 == 1)
        fprintf (out_file,"%-6.1f\n",a1);
      else if (i3 == 2)
      {
        a2 = (float) (s->cals.s.pgain*i2 + s->cals.s.poff);
        fprintf (out_file,"%-6.1f %-6.1f\n",a1,a2);
      }
      continue;
    }

   
    if (strstr(buffer,"TIME ") != NULL)
    {
      i3 = sscanf(&buffer[40],"%d %d",&i1,&i2);

      if (strstr(buffer,"FALL") != NULL)
        i1 *= 100;
      if (strstr(buffer,"BLADDERS") != NULL)
        i1 *= 100;
      if (strstr(buffer,"START") != NULL)
        i1 *= 10;

      if ((ptr = strstr(buffer,"(COUNTS)")) == NULL)
        buffer[40] = '\0';
      else
      {
        *ptr = '\0';
        strcat(buffer,"(SECONDS)");
      }
      fprintf (out_file,"%-40s",buffer);
      if (i3 == 1)
        fprintf (out_file,"%d.0\n",i1);
      continue;
    }

    if (strstr(buffer,"VACUUM") != NULL)
    {
      i3 = sscanf(&buffer[40],"%d %d",&i1,&i2);

      if ((ptr = strstr(buffer,"(COUNTS)")) == NULL)
          buffer[40] = '\0';
      else
      {
          *ptr = '\0';
          strcat(buffer,"(IN HG)");
      }
      fprintf (out_file,"%-40s",buffer);
      if (i3 == 1)
        sprintf (buffer,"%-6.1f\n",compute_vacuum((float)i1));
      else if (i3 == 2)
        sprintf (buffer,"%-6.1f %-6.1f\n",
            compute_vacuum((float)i1),compute_vacuum((float)i2));
      else
          return 1;
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"BATTERY") != NULL)
    {
      if (sscanf(&buffer[40],"%d",&i1) != 1)
        return(1);

      if ((ptr = strstr(buffer,"(COUNTS)")) == NULL)
          buffer[40] = '\0';
      else
      {
          *ptr = '\0';
          strcat(buffer,"(VOLTS)");
      }
      fprintf (out_file,"%-40s",buffer);
      sprintf (buffer,"%-6.1f\n",compute_voltage((float)i1));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if ( (strstr(buffer,"MOTOR CURRENT AT END OF ASCENT")!=NULL) ||
         (strstr(buffer,"MOTOR CURRENT OUT TO ASCEND")!=NULL) )
    {
      if (sscanf(&buffer[40],"%d",&i1) != 1)
        return(1);

      if ((ptr = strstr(buffer,"(COUNTS)")) == NULL)
          buffer[40] = '\0';
      else
      {
          *ptr = '\0';
          strcat(buffer,"(MA)");
      }
      fprintf (out_file,"%-40s",buffer);
      sprintf (buffer,"%-6.1f\n",(float)i1 * 10 );
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }


    if (strstr(buffer,"CURRENT") != NULL)
    {
      if (sscanf(&buffer[40],"%d",&i1) != 1)
        return(1);

      if ((ptr = strstr(buffer,"(COUNTS)")) == NULL)
          buffer[40] = '\0';
      else
      {
          *ptr = '\0';
          strcat(buffer,"(MA)");
      }
      fprintf (out_file,"%-40s",buffer);
      sprintf (buffer,"%-6.1f\n",compute_current((float)i1));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"COLUMNS") != NULL)
    {
      fprintf (out_file,"NUMBER OF COLUMNS                       4\n");
      fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
      fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
      fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
      fprintf (out_file,"4. COLUMN                               CHECKSUM FLAG\n");
      rewind(meta_file);

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
        if (strstr(buffer,"================") != NULL)
          break;

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
      {
        if (strstr(buffer,"================") != NULL)
		{
          fprintf (out_file,"==========================================================\n");
          break;
		}
        fwrite (strupr(buffer),1,strlen(buffer),out_file);
      }

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
        if (strstr(buffer,"LATITUDE") != NULL)
          break;

      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
      {
        if (strstr(buffer,"================") != NULL)
		{
          fprintf (out_file,"==========================================================\n");
          break;
		}
        fwrite (strupr(buffer),1,strlen(buffer),out_file);
//        fwrite (strupr(buffer),1,strlen(buffer),out_file);

//        if (strstr(buffer,"================") != NULL)
//          break;
      }

      i1 = 0;

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
      {
        if (sscanf(buffer,"%d %f %f %d",&ip[i1],&it[i1],&ic[i1],&iflg[i1]) == 4)
          i1++;
      }
// START - NEW - CS
// fix temperature overflow correction and simplify salinity overflow correction
      for (i2=0; i2<i1-1; i2++)
      {
        while (abs(it[i2+1]-it[i2]) > 2048 && it[i2+1]>it[i2])
        {
          if (trace_file != NULL)
          {
//              fprintf (trace_file,"\n temperature difference >2048 counts, subtract 4096 from counts\n");
//              fprintf (trace_file,"%d %f %f\n",i2,it[i2],it[i2+1]);
          }
          it[i2+1]=it[i2+1]-4096;
        }
        while (abs(ic[i2+1]-ic[i2]) > 512 && ic[i2+1]>ic[i2])
        {
          if (trace_file != NULL)
		  {
//              fprintf (trace_file,"\n salinity difference >512 counts, subtract 1024 from counts\n");
//              fprintf (trace_file,"%d %f %f\n",i2,ic[i2],ic[i2+1]);
		  }
          ic[i2+1]=ic[i2+1]-1024;
        }
      }
      for (i2=0; i2<i1; i2++)
      {
        a2=compute_sio_salinity(s,ic[i2]);
        sprintf (buffer,"%6.1f   %6.3f   %6.3f   %1d\n",
                 compute_sio_pressure(s,ip[i2]),a1=compute_sio_temperature(s,it[i2]),
                 a2,iflg[i2]);
        fwrite (buffer,1,strlen(buffer),out_file);
      }
// END - NEW - CS
/* START - OLD - CS
      a2sav=39;
      for (i2=0; i2<i1; i2++)
      {
        a2=compute_sio_salinity(s,ic[i2]);
        while (a2>a2sav+3)
        {
          if (trace_file != NULL)
              fprintf (trace_file,"\n D) salinity > prev+3, subtract 1024 from counts\n");
          ic[i2]=ic[i2]-1024;
          a2=compute_sio_salinity(s,ic[i2]);
        }
        a2sav=a2;
        if (a2sav<36)
           a2sav=39;
        sprintf (buffer,"%6.1f   %6.3f   %6.3f   %1d\n",
                 compute_sio_pressure(s,ip[i2]),a1=compute_sio_temperature(s,it[i2]),
                 a2,iflg[i2]);
        fwrite (buffer,1,strlen(buffer),out_file);
      }
END - OLD - CS */
    }
    else
      fprintf (out_file,buffer);

    continue;
  }

  return 0;
}


/******************************************************************************
  Assumes str_name = *.cnt, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
  char *ptr,tmp_buff[80];

  if ((ptr = strrchr(strupr(str_name),'_')) == NULL)
    strcat(str_name,".meta");
  else
  {
    *ptr = '\0';
    sprintf (tmp_buff,"%s", str_name );
//  sprintf (tmp_buff,"%s",ptr-10);
    sprintf (str_name,"%s.meta",tmp_buff);
  }

  return 0;
}


/******************************************************************************
  Reads meta_file
******************************************************************************/
int read_solo_ts_meta_file (char *meta_name, struct solo_ts_meta_struct *meta_struct)
{
  int i,j;
  char *ptr;
  char *cal_keys[] = {"PGAIN=", "PSURFGAIN=", "POFF=","TGAIN=", "TOFF=","SGAIN=", "SOFF=", 
                      "PMP_GAIN=", "PMP_OFF=", "CPUGAIN=", "CPUOFF=",  
                      "BLOK=", "AV1=", "AV2=", "PB1=", "PB2=", "CDIV=", "\0"};
  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_time = (float) 0.0;
  printf("\nOpening meta file %s",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("\nError opening meta file %s",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (meta_file != NULL)
    fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
        meta_struct->AOML_number = 0;

      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;

      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;

      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof(ptr+j);

      i++;
    }
  }

  return 0;
}

/******************************************************************************
  'main' function
******************************************************************************/
int main (int argc, char *argv[])
{
  char *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy,
       *ptr_path_log, *ptr_path_list,
       pathname_input_filename[161]="",
       pathname_trace[161]="", pathname_error[161]="";

  char input_filename[80],output_directory[256],
       msg[11]="", title[11]="", *ptr;

  struct solo_ts_meta_struct s;

  unsigned long t;
  int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates counts files for SOLO_ts12 float");
  printf("%s\n", title);

// Input Paramenters
// -----------------
  if (argc < 6)
  {
    printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
    printf("\n");
    printf("cnt_infile:    counts input file name\n");
    printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
    printf("trace_outile:  output file name for generated trace file\n");
    printf("error_outfile: output file name for generated error file\n");
    printf("list_outfile:  output file name for generated list file\n");
    return 1;
  }

// Input Counts File Name
// ----------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
  if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
  sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val ;
  }

  printf ("%s: Opening trace file %s ", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[3]);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> trace file opened\n", PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// ERROR File
// ----------
  sprintf (buffer,"%s",argv[4]);
  printf ("%s: Opening error file %s ", PRGNAME, buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> error file opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s ", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n",
             PRGNAME, buffer);

// Open Input Counts File
// ----------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n",
            PRGNAME);
    if (trace_file != NULL)
      fprintf (trace_file,
               "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,
               "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
    printf ("\nERROR (%s): Opening input file %s",
            PRGNAME, pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file, "ERROR (%s): Opening input file %s\n",
               PRGNAME, pathname_input_filename);
    if (error_file != NULL)
      fprintf (error_file, "ERROR (%s): Opening input file %s\n",
                PRGNAME, pathname_input_filename);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
  }
  else
  {
    printf ("%s: Input file %s successfully opened\n",
            PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file,
              "%s: Input file %s successfully opened\n",
              PRGNAME, pathname_input_filename);

// META File
// ---------
    if (get_meta_file_name (input_filename) != 0)
    {
      printf ("ERROR (%s): Finding meta file name", PRGNAME);
      if (error_file != NULL)
      fprintf (error_file,
               "\nERROR (%s): Finding meta file name - returned name %s\n",
               PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "\nERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      ret_val = 1;
    }
    else
    {                                             // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);
        if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                 PRGNAME);

        if (error_file != NULL)
          fprintf (error_file,
                   "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                   PRGNAME);

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);
        if (error_file != NULL)
          fclose (error_file);

        ret_val = 1;
      } // endif  ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_solo_ts_meta_file (pathname_input_filename, &s) != 0)
      {
        printf ("\nERROR (%s): Reading meta file", PRGNAME);
        if (trace_file != NULL)
          fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        if (error_file != NULL)
          fprintf (error_file, "ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        ret_val = 1;
      }
      else
      {
        if (trace_file != NULL)
          fprintf (trace_file, "%s: Meta file %s successfully read\n",
                   PRGNAME, pathname_input_filename);
          printf ("\n%s: Meta file %s successfully read\n", 
                  PRGNAME, pathname_input_filename);


// Convert Counts to Physical Units
// --------------------------------
        sprintf (input_filename,"%s%s",output_directory,argv[1]);

        if ((ptr = strrchr(input_filename,'.')) == NULL)
          strcat (input_filename,".phy");
        else
          sprintf(ptr,".phy");

        printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);

        if (convert_sio_ctd_data(&s, input_filename) == 0)
        {
          printf ("\n%s: CONVERT_TYPE12_DATA  successful\n", PRGNAME);
          if (trace_file != NULL)
            fprintf (trace_file, "%s: CONVERT_TYPE12_DATA successful\n", PRGNAME);
        }
        else
        {
          printf ("\nERROR (%s): CONVERT_TYPE12_DATA\n", PRGNAME);
          if (trace_file != NULL)
            fprintf (trace_file, "ERROR (%s): CONVERT_TYPE12_DATA\n",
                     PRGNAME);
          if (error_file != NULL)
            fprintf (error_file, "ERROR (%s): CONVERT_TYPE12_DATA\n",
                     PRGNAME);
          ret_val = 1;
        } // endif: convert_sio_ctd_data 
      } // endif: read_solo_ts_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);
  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
   fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
   fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
    fclose (list_file);

  return ret_val;
}
