/******************************************************************************   
   08-Dec-2011   ER - Change parameter SURFACE PRESSURE (DBAR) with
                      SURFACE PRES OFFSET APPLIED 1CBAR(DBAR)
  15-Apr-2009   ER - change the way the pressure is calculated (function
                     compute_pressure)
  14-Sep-2007   RS - turn off the debugg (set to 0)
  13-Sep-2007   CS & LL - eliminated many bugs - can now generate best profiles
                     (for FSI and SBE) - pressure bin correction, ...
  15-Aug-2007   CS - eliminated bug with adding Pinit to profile (for FSI and SBE)
                     changed     if(npts%9==1 && Pinit>0)
                     to          if(npts%9==1 && Pinit>PNew_vector[0])
                     eliminated second call of compute_pressure (was executed as
                     alternative to calc_fsi_p & added multiple 'crazy' levels to
                     profile)
  13-Aug-2007   CS - eliminated segmentation fault problem for profile length 0
                     fixed profile reversal problem after calc_fsi_p (profile has
                     to be upside-down before going into correctOverflow)
  10-Aug-2007   CS - corrected pressure binning (method from June-19-2007 had a
                       bug)
  01-Aug-2007   XXD&CS-shifting of the pressure by the value of the surface pressure
                CS - where ????
  22-Jun-2007   XXD/CS- FSI calc_fsi_p function added to get the right pressure column
                i.e. shifting of the pressure to start at bottom pressure
                (bug fix for FSI floats)
  19-Jun-2007   XXD- SBE no shift by 1/2 bin anymore  
                     SBE shallowest bin pressure starts from dp1  
  16-Apr-2007   XXD- add Pinit,Tinit,etc on top when npts mod 9==1 (if(npts%9==1 && Pinit>0))
  31-Oct-2005   CS - seems like fill value can also be 5000 for T and 1000
                     for S -> have to skip these values during overflow
                     correction

  07-Mar-2005   XXD-convert ITS-90 temperature to ITS-68 temperature to get compute salinity(FSI floats only)
   
   cnt2phy_ts16.cpp
   ============
   Converts the counts ARGOS data for temperature/salinity/pressure
   revision Date: 05-Apr-04
   revision: 1)For SBE floats, if count T[0]=5000 and S[0]=1000,then skip this first message block 
               when printing to phy file
             2)Add overflow correction algorithm, convert Breck Owens matlab overflow correction code
               to C++ to handle the overflow problem.   
             3)Add check to see if number of levels in cnt file is larger than the largest possible bin
               number, if it is, then truncate the cnt vector to be the length of largest possible bin number    
   Author: Xiangdong Xia
   Date:   June 2003
   Note: adjust pressure to average neighboring pressure values
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <vector>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <iostream>
//added new
#include <algorithm>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256   //Maximum length of each scan read from *.str
using namespace std;

char *strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */
int nbrk ( char *original, int *a, int n );

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

int debug=0;//1 for debug, 0 for operation
char PRGNAME[] ="cnt2phy_ts16";
char buffer[512];

struct aoml_cals {double pgain;
                  double poff;
                  double tgain;
                  double toff;
                  //for FSI
                  double cgain;                           
                  double coff;
                  //for SBE
                  double sgain;
                  double soff;
                                    
                  double p1;
                  double p2;
                  double p3;
                  double dp1;
                  double dp2;
                  double dp3;
                  double cpugain;
                  double cpuoff;
                  double auxgain;
                  double auxoff;                   
                  };

union u_cals {struct aoml_cals s;
              double c[sizeof(struct aoml_cals)/sizeof(double)];
             };

struct ts16_meta_struct {
              int AOML_number;
              int WMO_id;
              int ARGOS_id;
              int wmo_instrument;
              int wmo_recorder;
              float cycle_time;
              int exp_no;
              char float_type[40];
              char PI[60];
              float launch_time;
              float start_cycle_time;
              char hex_id[20];//newly added hexadecimal ARGOS ID
              
              int year;
              int month;
              int day;
              int hour;
              int minute;
              float lat;
              float latm;
              float lon;
              float lonm;  
                          
              union u_cals cals;
                         };

long ymd2julday(int, int, int);
//correct overflow problem,use breck owen's matlab algorithm
void correctOverflow(struct ts16_meta_struct *s,vector<float> PNew,vector<int>& i2_vector,vector<int>& i3_vector);//added new
//truncate counts to make sure the number of levels is <=largest possible bin number
void truncateCounts(struct ts16_meta_struct *s,vector<int>& i1_vector,vector<int>& i2_vector,vector<int>& i3_vector,vector<int>& i4_vector);
void calc_fsi_p(vector<float>& P_vector,float P_bot, float P_top, int nbin, float P1, float P2, float avg1, float avg2, float avg3);
/**********************************************************************************
                  Convert lower case characters to upper case 
***********************************************************************************/

char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
      input[i] = toupper(input[i]);
   return input;
}

/******************************************************************************
  This function is used by FSI floats only.SBE floats report salinity directly.
  convert conductivity to salinity standard function,convert ITS-90 temperature to ITS-68 temperature
  to use the conductivity to salinity conversion equation.
******************************************************************************/
float salinity (float c, float t, float p)
{  
   double t2,t3,t4,r,r1,r3,r4,r5,rp,rt,ft,d,ds,s;
   double t68;
  
  //cout<<"t="<<t<<" p="<<p<<" c="<<c;
/*
  if ((c>65.0) || (c<0.0) || (t>45.0) || (t<-3.0) || (p>12000.0) || (p<-20.0))
    return (0.0);
*/  
   if ((c<0.0) || (t<-3.0) || (p<-20.0))
      return (0.0);
   t68=t*1.00024;  
   t2 = t68 * t68;
   t3 = t2 * t68;
   t4 = t3 * t68;
   r  = c/42.914;
   rt = .6766097 + .0200564 * t68 + .0001104259 * t2
        - .00000069698 * t3 + 1.0031E-09 * t4;
   d  = 1 + .03426 * t68 + .0004464 * t2 + (.4215 - .003107 * t68) * r;
   rp = 1 + p * (.0000207 - 6.37E-10 * p + 3.989E-15 * p * p) / d;
   rt = r / rp / rt;
   ft = (t68 - 15) / (1 + .0162 * (t68 - 15));
   r1 = sqrt(rt);
   r3 = rt * r1;
   r4 = rt * rt;
   r5 = r4 * r1;
   ds = ft * (.0005 - .0056 * r1 - .0066 * rt -
              .0375 * r3 + .0636 * r4 - .0144 * r5);
   s = .008 - .1692 * r1 + 25.3851 * rt + 14.0941 * r3
       - 7.0261 * r4 + 2.7081 * r5 + ds;
   //cout<<" s="<<s<<endl;    
  return ((float)s);
}

/**********************************************************************************
               Convert temperature count to physical units
***********************************************************************************/

float compute_temperature (struct ts16_meta_struct *s, int t_counts)
{
   float result=(t_counts*s->cals.s.tgain + s->cals.s.toff);
//   cout<<" t_cnt= "<<t_counts<<"   t_phy="<<result<<endl;    
   return result; 
}

/**********************************************************************************
               Convert temperature count to physical units
               (for header part drift data only,set offset to 0)
***********************************************************************************/

float compute_temperature_d(struct ts16_meta_struct *s, int t_counts)
{
   float result=t_counts*s->cals.s.tgain;
   return result; 
}

/**********************************************************************************
                Convert pressure count to physical units(for drift data)
***********************************************************************************/


float compute_drift_pressure (struct ts16_meta_struct *s, int p_counts)
{
   float p;
   p=s->cals.s.pgain*p_counts+s->cals.s.poff;
   return(p);
}

/**********************************************************************************
                Convert pressure count to physical units(for drift data)
                (for header part drift data, set offset to 0,this is special for 
                 the floats right now WHOI had,later computer_drift_pressure should be called)
***********************************************************************************/


float compute_drift_pressure_d(struct ts16_meta_struct *s, int p_counts)
{
   float p;
   p=s->cals.s.pgain*p_counts;
   if(debug==1) printf("here pgain=%f,pcounts=%d,p=%f\n",s->cals.s.pgain,p_counts,p);
   return(p);
}


/**********************************************************************************
     Convert pressure count to physical units(for profile data) based on bin number
     
     As the float goes deeper, the pressure changes faster from bin to bin.
     Thus,
     from the surface to pressure 1 (p1) the press varies dp1 from bin to bin
     from p1 to p2 the pressure varies dp2 from bin to bin
     from p2 to p3 the pressure varies dp3 from bin to bin
     beyond p3 a warning will be issued.
***********************************************************************************/
float compute_pressure (struct ts16_meta_struct *s, int bin_num)
{

   bool in_deep_trouble;   
   float pressure;  // pressure to be calculated.

   in_deep_trouble = false;
   pressure = 0;
   for(int i=0; i<=bin_num; i++){
     if (pressure < s->cals.s.p1) {  // previous pressure <= p1
       pressure = pressure + s->cals.s.dp1  ;  
     }
     else if (pressure < s->cals.s.p2 ) { // previous pressure <= p2 
       pressure = pressure + s->cals.s.dp2;  
     }
     else if (pressure <= s->cals.s.p3 ) { // previous pressure <= p3 
       pressure = pressure + s->cals.s.dp3;  
     }
     else {
       pressure = pressure + s->cals.s.dp3;  
       in_deep_trouble = true;       
     }
   }

   if (in_deep_trouble) {
     if (trace_file != NULL)
         fprintf (trace_file,"warning in %s: float went deeper than %f dbar: max = %f dbar\n",PRGNAME,s->cals.s.p3,pressure);
     if (error_file != NULL)
         fprintf (error_file,"warning in %s: float went deeper than %f dbar: max = %f dbar\n",PRGNAME,s->cals.s.p3,pressure);
   }

   if(debug==1) {
     printf("bin_num = %d, pressure = %f\n",bin_num,pressure);
   }

   return(pressure);
}

/******************************************************************************
  convert conductivity, specific for TS16 float  FSI type
******************************************************************************/

float compute_conductivity (struct ts16_meta_struct *s, int s_counts, int t_counts)
{
   float t,c;
   t=compute_temperature(s,t_counts);
   c= s_counts*s->cals.s.cgain + s->cals.s.coff + t;//calibration equation 
   return(c);
}

/******************************************************************************
  convert conductivity, specific for TS16 float  FSI type
  (set offset to 0, this is special for WHOI floats right now, later compute_conductivity function
   should be used)
******************************************************************************/

float compute_conductivity_d (struct ts16_meta_struct *s, int s_counts, int t_counts)
{
   float t,c;
   t=compute_temperature_d(s,t_counts);
   c= s_counts*s->cals.s.cgain;//calibration equation 
   return(c);
}


/******************************************************************************
  convert conductivity to salinity specific for TS16 float based on pressure physical values
  for FSI type
  For SBE type,just use the original calibration equation
  Note,for header part,salinity computation should be based on real pressure,not bin number!!!
  Note: pressure here is physical value, not count value!!! Be careful!!!
******************************************************************************/


float compute_salinity_p (struct ts16_meta_struct *s, float c, float t, float p,int s_counts)
{
   float sa; 
   if(s->wmo_instrument==852){//FOR FSI FLOAT
      sa=salinity(c,t,p);
   }else{//FOR SBE FLOAT
      sa=s_counts*s->cals.s.sgain + s->cals.s.soff;
   }
   return(sa);
}
/**********************************************************************************
                Convert CPU voltage count to physical units
***********************************************************************************/


float compute_cpu (struct ts16_meta_struct *s, int cpu_counts)
{
   float cpu;
   cpu=s->cals.s.cpugain*cpu_counts+s->cals.s.cpuoff;
   return(cpu);
}

/**********************************************************************************
                Convert AUX voltage count to physical units
***********************************************************************************/


float compute_aux (struct ts16_meta_struct *s, int aux_counts)
{
   float aux;
   aux=s->cals.s.auxgain*aux_counts+s->cals.s.auxoff;
   return(aux);
}


/***************************************************
Convert year,month,day to julian day
*****************************************************/

long ymd2julday(int year, int month, int day)
{
   int  ic, iy;
   long julday;

   if ( month > 2 )
      month = month - 3;
   else if ( month < 3 )
   {
      month = month + 9;
      year = year - 1;
   }

   ic = (int) floor((double) (year/100) );
   iy = year - 100*ic;

// printf("ic: %d iy: %d\n", ic, iy);
 
   julday =   (long) floor((double)(146097*ic)/4) 
             + (long) floor((double)(1461*iy)/4)
             + (long) floor((double)(153*month+2)/5) 
             + day + 1721119;
   julday = julday - 2440000;
   return julday;
}



/**********************************************************************************
                Compute julian date from 1997
***********************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
   int    yr, mo, dy, day_from_97;
   long   passed_jd, jd_xx0101, jd_970101;
   double d_jd;

   yr = yp;
   mo = mp;
   dy = dp;

   if (yr <= 15)
      yr += 2000;
   else if (yr < 1900)
      yr += 1900;

   if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
      return (0.0);

   passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year)
   dy = 1;
   mo = 1;
   jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
   yr = 1997;
   jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
   day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs,
                                                // mins, secs)
   d_jd = day_from_97;
   d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
   return (d_jd);                                // and adds this to d_jd
}


/**********************************************************************************
                Convert Counts Data to Physical Data File(Key part)
                for drift pressure, use pgain,poff to compute physical pressure
                for drift temperature,use tgain,toff
                for profile pressure, use p1,p2,p3,dp1,dp2,dp3 to compute physical pressure
                for profile temperature, use tgain,toff
                for FSI profile salinity and drift salinity, use standard conductivity conversion routine 
                for SBE profile salinity and drift salinity, use calibration equation 
                this routine requires conductivity counts,temperature counts,and pressure counts
***********************************************************************************/


int convert_data(struct ts16_meta_struct *s, char *output_name)
{
   int  i1=0,i2=0,i3=0,i4=0,iflg=0,breaks=0,j=0,jj=0;
 //new added variables in order to call the compute_salinity function since it need several input parameters
   int ip=0,it=0,ic=0,is=0;//initial pressure,temperature,conductivity,salinity 
   int fdp=0,fdt=0,fdc=0,fds=0;//first half drift pressure,temperature,conductivity,salinity
   int sdp=0,sdt=0,sdc=0,sds=0;//second half drift pressure,temperature,conductivity,salinity
   int dp[10],dt[10],ds[10];//store drift P,T,S values 
   int vcpu,vaux;//voltage cpu,aux counts
   int skip_message=0;//added new for SBE floats
 
   char *ptr_phy_filename;
   char buffer1[512];//used for distinguish from buffer[512] in drift part
   vector<float> P_vector,PNew_vector,T_vector,C_vector,S_vector;
   vector<int> Flag_vector;
   vector<int> i1_vector,i2_vector,i3_vector,i4_vector;
   int flag_data=0;
   float Pinit=0.0,Tinit=0.0,Cinit=0.0,Sinit=0.0;//physical values of initial p,t,c,s
   float Psurface=0.0;//surface pressure
   int npts=0;

// add flagp for FSI which shifts the pressure
   int flagp = 0;

   if ((out_file = fopen(output_name,"w")) == NULL)
   {
      if (trace_file != NULL)
         fprintf (trace_file," CONVERT_DATA: error opening output file %s\n",
         output_name);
      if (error_file != NULL)
         fprintf (error_file," CONVERT_DATA: error opening output file %s\n",
         output_name);
      return 1;
   }
 
 
   if (trace_file != NULL)
      fprintf (trace_file," CONVERT_DATA: output file %s successfully opened\n",output_name);

   printf ("\nOutput file %s successfully opened\n", output_name);

// Write Output Profile Physical Unit File Name to List File
// ---------------------------------------------------------
   if ( (ptr_phy_filename = strrchr(output_name, '/')) == NULL)
   {
      printf ("\nCan't get phy file basename %s", output_name);
      if (trace_file != NULL)
         fprintf (trace_file,
                "\n%s : Can't get phy file basename %s", PRGNAME, output_name);
       if (error_file != NULL)
          fprintf (error_file,
                 "\n%s: Can't get phy file basename %s", PRGNAME, output_name);
       return 1;
   } // endif: ptr_phy_filename
 
   ptr_phy_filename++;
   fprintf (list_file,"%s\n", ptr_phy_filename);

   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
   {
      if (strstr(strupr(buffer),"INITIAL PRESSURE (COUNTS)") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %d",&ip) != 1)
            ip = 0;
         if(debug==1) printf("Here:ip=%d\n",ip);
         Pinit=compute_drift_pressure(s,ip);
         sprintf (buffer, "INITIAL PRESSURE (DBAR)                 %-6.1f\n",
         Pinit);   
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }

      if (strstr(strupr(buffer),"INITIAL TEMPERATURE") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %d",&it) != 1)
            it = 0;
         Tinit=compute_temperature(s,it); 
         sprintf (buffer, "INITIAL TEMPERATURE (DEG C)             %-6.3f\n",
         Tinit);
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(buffer,"INITIAL CONDUCTIVITY") != NULL)
      {//FOR FSI FLOAT
         if (sscanf(buffer,"%*s %*s %*s %d",&ic) != 1)
            ic = 0;
         Cinit=compute_conductivity(s,ic,it);
         sprintf (buffer,"INITIAL CONDUCTIVITY (MS/CM)            %-6.3f\n",
          Cinit);
         fwrite (buffer,1,strlen(buffer),out_file);
         Sinit=compute_salinity_p(s,Cinit,Tinit,Pinit,0);
         sprintf (buffer,"INITIAL SALINITY (PSU)                  %-6.3f\n",
          Sinit);
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
      if (strstr(buffer,"INITIAL SALINITY") != NULL)
      {//FOR SBE FLOAT
         if (sscanf(buffer,"%*s %*s %*s %d",&is) != 1)
            is = 0;
         Sinit=compute_salinity_p(s,0,0,0,is);  
         sprintf (buffer,"INITIAL SALINITY (PSU)                  %-6.3f\n",
         Sinit);
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(strupr(buffer),"NUMBER OF POINTS") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %d",&npts) != 1)
            npts = 0;   
         sprintf (buffer,"NUMBER OF POINTS                        %d\n",
         npts);   
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }

      if (strstr(strupr(buffer),"SURFACE PRES OFFSET APPLIED 1CBAR(CNTS)") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&ip) != 1)
            ip = 0;   
         Psurface=(float)compute_drift_pressure(s,ip);
         sprintf (buffer,"SURFACE PRES OFFSET APPLIED 1CBAR(DBAR) %-6.1f\n",
                          Psurface);   
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
    
      if (strstr(strupr(buffer),"START OF DRIFT PRESSURE (COUNTS)") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fdp) != 1)
            fdp = 0;
         sprintf (buffer, "START OF DRIFT PRESSURE (DBAR)          %-6.1f\n",
         compute_drift_pressure_d(s,fdp));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(strupr(buffer),"START OF DRIFT TEMPERATURE (COUNTS)") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fdt) != 1)
            fdt = 0;
         sprintf (buffer, "START OF DRIFT TEMPERATURE (DEG C)      %-6.3f\n",
         compute_temperature_d(s,fdt));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }

      if (strstr(buffer,"START OF DRIFT CONDUCTIVITY (COUNTS)") != NULL)
      {//FOR FSI FLOAT
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fdc) != 1)
            fdc = 0;
         sprintf (buffer,"START OF DRIFT CONDUCTIVITY (MS/CM)     %-6.3f\n",
         compute_conductivity_d(s,fdc,fdt));
         fwrite (buffer,1,strlen(buffer),out_file);
         sprintf (buffer, "START OF DRIFT SALINITY (PSU)           %-6.3f\n",
         compute_salinity_p(s,compute_conductivity_d(s,fdc,fdt),compute_temperature_d(s,fdt),compute_drift_pressure_d(s,fdp),0));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }

      if (strstr(buffer,"START OF DRIFT SALINITY (COUNTS)") != NULL)
      {//FOR SBE FLOAT
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&fds) != 1)
            fds = 0;
         sprintf (buffer, "START OF DRIFT SALINITY (PSU)           %-6.3f\n",
         compute_salinity_p(s,0,0,0,fds));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
    
      if (strstr(strupr(buffer),"END OF DRIFT PRESSURE (COUNTS)") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sdp) != 1)
            sdp = 0;
         sprintf (buffer, "END OF DRIFT PRESSURE (DBAR)            %-6.1f\n",   
         compute_drift_pressure_d(s,sdp));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(strupr(buffer),"END OF DRIFT TEMPERATURE (COUNTS)") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sdt) != 1)
            sdt = 0;
         sprintf (buffer, "END OF DRIFT TEMPERATUR (DEG C)         %-6.3f\n",
         compute_temperature_d(s,sdt));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
 
      if (strstr(buffer,"END OF DRIFT CONDUCTIVITY (COUNTS)") != NULL)
      {//FOR FSI FLOAT
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sdc) != 1)
            sdc = 0;
         sprintf (buffer,"END OF DRIFT CONDUCTIVITY (MS/CM)       %-6.3f\n",
         compute_conductivity_d(s,sdc,sdt));
         fwrite (buffer,1,strlen(buffer),out_file);
         sprintf (buffer,"END OF DRIFT SALINITY (PSU)             %-6.3f\n",   
         compute_salinity_p(s,compute_conductivity_d(s,sdc,sdt),compute_temperature_d(s,sdt),compute_drift_pressure_d(s,sdp),0));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }

      if (strstr(buffer,"END OF DRIFT SALINITY (COUNTS)") != NULL)
      {//FOR SBE FLOAT
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&sds) != 1)
            sds = 0;
         sprintf (buffer,"END OF DRIFT SALINITY (PSU)             %-6.3f\n",
         compute_salinity_p(s,0,0,0,sds));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(buffer,"VOLTAGE OF CPU") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %d",&vcpu) != 1)
            vcpu = 0;
         sprintf (buffer,"VOLTAGE OF CPU (VOLT)                   %-6.1f\n",
         compute_cpu(s,vcpu));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(buffer,"VOLTAGE OF AUX") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %d",&vaux) != 1)
            vaux = 0;
         sprintf (buffer,"VOLTAGE OF AUX (VOLT)                   %-6.1f\n",
         compute_aux(s,vaux));
         fwrite (buffer,1,strlen(buffer),out_file);
         continue;
      }
  
      if (strstr(buffer,"DRIFT PRESSURE (COUNTS)") != NULL)
      {
         sprintf (buffer1,"DRIFT PRESSURE (DBAR)                   ");
         fwrite (buffer1,1,strlen(buffer1),out_file);
         breaks=nbrk(buffer,dp,10);
         for(j=0;j<breaks;j++){
            if(dp[0]==99999){
               sprintf(buffer1,"%-6.1f ",99999.); 
               fwrite (buffer1,1,strlen(buffer1),out_file);    
               break;
            }
            sprintf(buffer1,"%-6.1f ",compute_drift_pressure_d(s,dp[j])); 
            fwrite (buffer1,1,strlen(buffer1),out_file);
         }
         sprintf(buffer1,"\n");    
         fwrite (buffer1,1,strlen(buffer1),out_file);
         continue;
      }
  
      if (strstr(buffer,"DRIFT TEMPERATURE (COUNTS)") != NULL)
      {
         sprintf (buffer1,"DRIFT TEMPERATURE (DEG C)               ");
         fwrite (buffer1,1,strlen(buffer1),out_file);
         breaks=nbrk(buffer,dt,10);
         for(j=0;j<breaks;j++){
            if(dt[0]==99999){
               sprintf(buffer1,"%-6.1f ",99999.); 
               fwrite (buffer1,1,strlen(buffer1),out_file);    
               break;
            }
            sprintf(buffer1,"%-6.3f ",compute_temperature_d(s,dt[j])); 
            fwrite (buffer1,1,strlen(buffer1),out_file);
         }
         sprintf(buffer1,"\n");    
         fwrite (buffer1,1,strlen(buffer1),out_file);
         continue;
      }
  
      if (strstr(buffer,"DRIFT CONDUCTIVITY (COUNTS)") != NULL)
      {//FOR FSI FLOAT
         breaks=nbrk(buffer,ds,10);
         sprintf (buffer1,"DRIFT CONDUCTIVITY (MS/CM)              ");
         fwrite (buffer1,1,strlen(buffer1),out_file);
         for(j=0;j<breaks;j++){
            if(ds[0]==99999){
               sprintf(buffer1,"%-6.1f ",99999.); 
               fwrite (buffer1,1,strlen(buffer1),out_file);    
               break;
            }
            sprintf(buffer1,"%-6.3f ",compute_conductivity_d(s,ds[j],dt[j])); 
            fwrite (buffer1,1,strlen(buffer1),out_file);
         }
         sprintf(buffer1,"\n");    
         fwrite (buffer1,1,strlen(buffer1),out_file);
         sprintf (buffer1,"DRIFT SALINITY (PSU)                    ");
         fwrite (buffer1,1,strlen(buffer1),out_file);
         for(j=0;j<breaks;j++){
            if(ds[0]==99999){
               sprintf(buffer1,"%-6.1f ",99999.); 
               fwrite (buffer1,1,strlen(buffer1),out_file);    
               break;
            }
            sprintf(buffer1,"%-6.3f ",compute_salinity_p(s,
            compute_conductivity_d(s,ds[j],dt[j]),
            compute_temperature_d(s,dt[j]),
            compute_drift_pressure_d(s,dp[j]),0)); 
            fwrite (buffer1,1,strlen(buffer1),out_file);
         }
         sprintf(buffer1,"\n");    
         fwrite (buffer1,1,strlen(buffer1),out_file);
         continue;
      }
      if (strstr(buffer,"DRIFT SALINITY (COUNTS)") != NULL)
      {//FOR SBE FLOAT
         breaks=nbrk(buffer,ds,10);
         sprintf (buffer1,"DRIFT SALINITY (PSU)                    ");
         fwrite (buffer1,1,strlen(buffer1),out_file);
         for(j=0;j<breaks;j++){
            if(ds[0]==99999){
               sprintf(buffer1,"%-6.1f ",99999.); 
               fwrite (buffer1,1,strlen(buffer1),out_file);    
               break;
            }
            sprintf(buffer1,"%-6.3f ",compute_salinity_p(s,0,0,0,ds[j])); 
            fwrite (buffer1,1,strlen(buffer1),out_file);
         }
         sprintf(buffer1,"\n"); 
         fwrite (buffer1,1,strlen(buffer1),out_file);
         continue;
      }
  
      if ((strstr(buffer,"COLUMNS") != NULL) || (strstr(buffer,"COLUMNS")))
      {
         if (strstr(strupr(buffer),"COLUMNS") != NULL)
         {
            if(s->wmo_instrument==852){//FOR FSI FLOAT
               fprintf (out_file,"NUMBER OF COLUMNS                       5\n");
               fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
               fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
               fprintf (out_file,"3. COLUMN                               CONDUCTIVITY \n");
               fprintf (out_file,"4. COLUMN                               SALINITY (PSU)\n");
               fprintf (out_file,"5. COLUMN                               CHECKSUM FLAG\n");
            }else{//FOR SBE FLOAT
               fprintf (out_file,"NUMBER OF COLUMNS                       4\n");
               fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
               fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
               fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
               fprintf (out_file,"4. COLUMN                               CHECKSUM FLAG\n");
            }
         }
         rewind(meta_file);
         while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
            if (strstr(buffer,"================") != NULL)
               break;
         while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
         {
            if (strstr(buffer,"================") != NULL)
             {
                sprintf (buffer,"==========================================================\n");
               fwrite (buffer,1,strlen(buffer),out_file);
               break;
             }
            fwrite (strupr(buffer),1,strlen(buffer),out_file);
         }
         while (fgets(buffer,sizeof(buffer),input_file) != NULL)
            if (strstr(buffer,"LATITUDE") != NULL)
               break;
         fwrite (buffer,1,strlen(buffer),out_file);
         while (fgets(buffer,sizeof(buffer),input_file) != NULL)
         {
            fwrite (buffer,1,strlen(buffer),out_file);
            if (strstr(buffer,"================") != NULL)
               break;
         }
         iflg = 1;
         continue;
      }
//      if(debug==1) cout<<"iflg "<<iflg<<endl;
//      if(debug==1) cout<<"buffer "<<buffer<<endl;

      if (iflg == 1)
      {
         if (sscanf(buffer,"%d %d %d %d",&i1,&i2,&i3,&i4) == 4)
         {
            if (i1==0 & i2==0 & i3==0 & i4==0)  // no data
            { 
//CS - no good              flag_data=0;//no data
              i1_vector.push_back(i1);i2_vector.push_back(i2);i3_vector.push_back(i3);i4_vector.push_back(i4);           
            }
            else
            { 
              flag_data=1;//has data
              i1_vector.push_back(i1);i2_vector.push_back(i2);i3_vector.push_back(i3);i4_vector.push_back(i4);       
            }
         }
      }
      else
         fprintf (out_file,buffer);
   } // while (fgets(buffer,sizeof(buffer),input_file) != NULL) (??? - correct CS)

   if(debug==1) cout<<"flag_data "<<flag_data<<endl;

//decode profile
   if(flag_data==1){ // moved here (from below)  - CS
      if(debug==1) {
        printf( "ip=%d it=%d ic=%d is=%d\n", ip,it,ic,is);
        printf( "P=%6.1f   T=%6.3f   C=%6.3f   S=%6.3f\n",Pinit,Tinit,Cinit,Sinit);  
      }

/*added new
 Add check to see if number of levels in cnt file is larger than the largest possible bin
 number, if it is, then truncate the cnt vector to be the length of largest possible bin number      
*/

//    truncateCounts(s,i1_vector,i2_vector,i3_vector,i4_vector);    

//get original pressure values(no averaging performed)
   if(i1_vector.size()!=0){ 
      if(debug==1) cout<<"compute_pressure 1"<<endl;
      for(jj=0;jj<i1_vector.size();jj++){
         P_vector.push_back(compute_pressure(s,i1_vector[jj]));
         if(debug==1) cout<<"P_vector[jj]="<<P_vector[jj]<<endl;
      } 
   }
 
   if(debug==1) cout<<"P_vector.size()="<<P_vector.size()<<" wmo instrument="<<s->wmo_instrument<<endl;

   if(s->wmo_instrument==851){//for SBE, no shift by 1/2 bin anymore  
      for(jj=0;jj<P_vector.size();jj++){
         PNew_vector.push_back(P_vector[jj]);
      }
   }else{//FSI pressure
      if(P_vector.size()!=0){
         for(jj=0;jj<P_vector.size();jj++){
            PNew_vector.push_back(P_vector[jj]);
         }
      }
   } // end if(s->wmo_instrument==851)
   
   if(debug==1) cout<<"PNew_vector.size()="<<PNew_vector.size()<<endl;

//fix the overflow problem for T and C/S counts first before actually doing calibrations.
//use Breck Owen's matlab algorithm,translated into C++ here by Xiangdong Xia
  
   if(debug==1) cout<<"correct overflow  here2"<<endl;
   if(debug==1) {
      for(jj=0;jj<P_vector.size();jj++){
         cout<<"A        P_vector[jj]="<<P_vector[jj]<<"  i2_vector[jj]="<<i2_vector[jj]
         <<"  i3_vector[jj]="<<i3_vector[jj]<<"  i4_vector[jj]="<<i4_vector[jj]<<endl;
      }
   }
   correctOverflow(s,PNew_vector,i2_vector,i3_vector);
   if(debug==1) {
      for(jj=0;jj<PNew_vector.size();jj++){
         cout<<"B        PNew_vector[jj]="<<PNew_vector[jj]<<"  i2_vector[jj]="<<i2_vector[jj]
         <<"  i3_vector[jj]="<<i3_vector[jj]<<"  i4_vector[jj]="<<i4_vector[jj]<<endl;
      }
   }

   if(s->wmo_instrument==852){//for FSI
//shifting of the pressure to start at bottom pressure (FSI only, bug fix for these floats)
      if(debug==1) cout<<"Pinit = "<<Pinit<<"   PNew_vector[0] = "<<PNew_vector[0]<<endl;
      if(debug==1) cout<<"Tinit = "<<Tinit<<"   i2_vector[0] = "<<compute_temperature(s,i2_vector[0])<<endl;
//add condition of comparing Tinit          
      if(Pinit>0 && Pinit>PNew_vector[0] && Tinit == compute_temperature(s,i2_vector[0]))    {
         if(debug==1) cout<<"calc_fsi_p   here1"<<endl;
//CS - misunderstanding         npts=npts+1;
//add condition when npts is very large            
         if( npts > PNew_vector.size()+1 ) {
             npts = PNew_vector.size()+1;
             if(debug==1) cout<<"npts   "<<npts<<endl;
         }
         calc_fsi_p(PNew_vector,Pinit,Psurface,npts,s->cals.s.p1,s->cals.s.p2,s->cals.s.dp1,s->cals.s.dp2,s->cals.s.dp3); 
         reverse(PNew_vector.begin(),PNew_vector.end()); // CS make pressure vector upside down again
//CS - misunderstanding         flagp = 1;
      }  // end Pinit>0 && Pinit>PNew_vector[0]) && ....
   } // end if(s->wmo_instrument==852)
   if(debug==1) cout<<"P_vector.size()="<<P_vector.size()<<"PNew_vector.size()="<<PNew_vector.size()<<endl;

   int i2New_vector[PNew_vector.size()],i3New_vector[PNew_vector.size()],i4New_vector[PNew_vector.size()];
// add condition to switch temperature, conductivity, and flag, and set the initial value
   if(flagp==1){
// will never be executed (fixed the misunderstanding above -> flagp never 1 ...)
      if( Tinit != compute_temperature(s,i2_vector[0]) )         {
         if(debug==1) printf("Here3!!!!\n");
         for(jj=PNew_vector.size()-2;jj>=0;jj--){                
            i2New_vector[jj+1] = i2_vector[jj];
            i3New_vector[jj+1] = i3_vector[jj];
            i4New_vector[jj+1] = i4_vector[jj];
         }
         i2New_vector[0] = it;
         i3New_vector[0] = ic;
         i4New_vector[0] =  1;
      }
   }
   else{
      if(debug==1) printf("Here4!!!!\n");
      for(jj=PNew_vector.size()-1;jj>=0;jj--){                
         i2New_vector[jj] = i2_vector[jj];
         i3New_vector[jj] = i3_vector[jj];
         i4New_vector[jj] = i4_vector[jj];
      }
   }

   if(debug==1) {
      for(jj=0;jj<PNew_vector.size();jj++){
         cout<<"C        PNew_vector[jj]="<<PNew_vector[jj]<<"  i2New_vector[jj]="<<i2New_vector[jj]
         <<"  i3New_vector[jj]="<<i3New_vector[jj]<<"  i4New_vector[jj]="<<i4New_vector[jj]<<endl;
      }
   }

   //get T,C,S,Flag vector
   if(s->wmo_instrument==852){//FOR FSI FLOAT
      for(jj=0;jj<PNew_vector.size();jj++){
         T_vector.push_back(compute_temperature(s,i2New_vector[jj]));
         C_vector.push_back(compute_conductivity(s,i3New_vector[jj],i2New_vector[jj]));
//get salinity based on pressure physical value,not bin number since pressure averaged should be used
         S_vector.push_back(compute_salinity_p(s,compute_conductivity(s,i3New_vector[jj],i2New_vector[jj]),
                          compute_temperature(s,i2New_vector[jj]),
                          PNew_vector[jj],0)); 

//reset P,T,C,S=0 if in cnt file, T=C=0, they are missing messages. 
         if(i2New_vector[jj]==0 && i3New_vector[jj]==0){
            PNew_vector[jj]=0;
            T_vector[jj]=0;
            C_vector[jj]=0;
            S_vector[jj]=0;
            i4New_vector[jj]=0;
         }
         Flag_vector.push_back(i4New_vector[jj]);
      }
   }else{//FOR SBE FLOAT
      for(jj=0;jj<PNew_vector.size();jj++){
         T_vector.push_back(compute_temperature(s,i2New_vector[jj]));
         //get salinity purely based on calibration equation for SBE float since salinity counts are available
         S_vector.push_back(compute_salinity_p(s,0,0,0,i3New_vector[jj]));        
         //reset P,T,C,S=0 if in cnt file, T=C=0, they are missing messages. 
         if(i2New_vector[jj]==0 && i3New_vector[jj]==0){
            PNew_vector[jj]=0;
            T_vector[jj]=0;
            S_vector[jj]=0;
            i4New_vector[jj]=0;
         }
         Flag_vector.push_back(i4New_vector[jj]);    
      }
//added new for SBE floats, if T[0] cnt=5000 and S[0] cnt=1000,then skip the first message block
//note the profile depth decreases in the cnt/phy file

      skip_message=0;
// CS - needed to eliminate segmentation fault problem for profile length 0 (Aug 13, 2007):
      if ( PNew_vector.size()>=9 ) {
         if(i2New_vector[PNew_vector.size()-1]==5000 && i3New_vector[PNew_vector.size()-1]==1000){
            skip_message=9;
         }
      }
//if skip_message is 9, then we need to skip the first 9 values(first message block), so PNew,T,S,Flag
//vector will be shorten
      if(skip_message==9){
         int newsize=PNew_vector.size()-skip_message;
         float*  pnew= new float[newsize];
         float*  tnew= new float[newsize];
         float*  snew= new float[newsize];
         int* flagnew= new int[newsize];
         //delete 9 times at the end of the vector P,T,S,Flag
         for(jj=0;jj<newsize;jj++){
             pnew[jj]=PNew_vector[jj];tnew[jj]=T_vector[jj];snew[jj]=S_vector[jj];flagnew[jj]=Flag_vector[jj];
         }
         //reset and refill PNew,T,S,Flag vectors
         PNew_vector.clear();T_vector.clear();S_vector.clear();Flag_vector.clear();
         for(jj=0;jj<newsize;jj++){
            PNew_vector.push_back(pnew[jj]);
            T_vector.push_back(tnew[jj]);
            S_vector.push_back(snew[jj]);
            Flag_vector.push_back(flagnew[jj]);
         }
         delete pnew;
         delete tnew;
         delete snew;
         delete flagnew;
      }//end of if(skip_message=9)
   }//end of SBE float
   
     
   if(debug==1) cout<<"PNew_vector.size()="<<PNew_vector.size()<<" T_vector.size()="<<T_vector.size()<<endl;

         
//print data to phy file
//   if(flag_data==1){  // in wrong place CS
      if(s->wmo_instrument==852){//FOR FSI FLOAT          
      //print the Pinit,Tinit,Cinit,Sinit on the top
         if(debug==1) cout<<"Pinit="<<Pinit<<", PNew_vector[0]="<<PNew_vector[0]<<endl;
         if(debug==1) printf("P=%6.1f   T=%6.3f   C=%6.3f   S=%6.3f   F=%1d\n",PNew_vector[0],T_vector[0],C_vector[0],S_vector[0],Flag_vector[0]);     
         if(debug==1) printf("P=%6.1f   T=%6.3f   C=%6.3f   S=%6.3f   F=%1d\n",PNew_vector[PNew_vector.size()-1],T_vector[PNew_vector.size()-1],C_vector[PNew_vector.size()-1],S_vector[PNew_vector.size()-1],Flag_vector[PNew_vector.size()-1]);

//OLD -CS        if(npts%9==1 && Pinit>PNew_vector[0]) fprintf (out_file,"%6.1f   %6.3f   %6.3f   %6.3f   %1d\n",Pinit,Tinit,Cinit,Sinit,1);     
        for(jj=0;jj<PNew_vector.size();jj++){
            if( PNew_vector[jj]< 4000 ) //add condition
              fprintf (out_file,"%6.1f   %6.3f   %6.3f   %6.3f   %1d\n",PNew_vector[jj],T_vector[jj],C_vector[jj],S_vector[jj],Flag_vector[jj]);
         }      
      }
      else{//FOR SBE FLOAT
         //print the Pinit,Tinit,Sinit on the top
         if(debug==1) cout<<"Pinit="<<Pinit<<", PNew_vector[0]="<<PNew_vector[0]<<endl;
         if(npts%9==1 && Pinit>PNew_vector[0]) {
            if( Pinit < 4000 )          //add condition   
              fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",Pinit,Tinit,Sinit,1);   
         }  
         for(jj=0;jj<PNew_vector.size();jj++){
            if( PNew_vector[jj]< 4000 )  //add condition
              fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",PNew_vector[jj],T_vector[jj],S_vector[jj],Flag_vector[jj]);
         }
      }
  } //  if(flag_data==1)
  
  
  if (trace_file != NULL)
     fprintf (trace_file," CONVERT_DATA:%s completed successfully\n",output_name);
  return 0;

}

/**********************************************************************************
                Get Meta Data File Name
***********************************************************************************/


int get_meta_file_name (char *str_name)
{
   char *ptr,tmp_buff[80];
   int len;

// Assume str_name = *.hex, returns *.meta in str_name

   if ((ptr = strrchr(str_name,'_')) == NULL)    // get pointer to last '_'
      strcat(str_name,".meta");                    
   else
   {
      sprintf (tmp_buff,"%s\0", ptr);
      len = strlen(str_name) - strlen(tmp_buff);
      strncpy(tmp_buff, str_name, len);
      tmp_buff[len] = '\0';
      sprintf (str_name,"%s.meta",tmp_buff);
   }
   return 0;
}


/**********************************************************************************
                Read Meta Data File
***********************************************************************************/


int read_meta_file (char *meta_name, struct ts16_meta_struct *meta_struct)
{
   int i,j,syear,smonth,sday,shour,smin;
   float x,lat,latm,lon,lonm;
   int lyear,lmonth,lday,lhour,lmin;
   char *ptr;
   char buffer[512];
  
   char manufacturer[20];
   char fsi[]="FSI";
   char sbe[]="SBE";
  
   char *cal_keys[] = {"PGAIN=", "POFF=","TGAIN=", "TOFF=","CGAIN=", "COFF=","SGAIN=", "SOFF=",
     "P1=", "P2=","P3=","DP1=", "DP2=","DP3=","CPUGAIN=","CPUOFF=","AUXGAIN=","AUXOFF=","\0"};

   meta_struct->cycle_time = (float) 0.0;
   meta_struct->start_cycle_time = (float) 0.0;

   printf("\nOpening meta file!!!!!! %s\n",meta_name);

   if ((meta_file = fopen(meta_name,"r")) == NULL)
   {
      printf ("Error opening meta file!!!!!! %s\n",meta_name);
      if (error_file != NULL)
         fprintf (error_file," ERROR opening meta file %s\n",meta_name);
      return(1);
   }
 
   if (trace_file != NULL)
      fprintf (meta_file," Meta file %s successfully opened\n",meta_name);
 
// Reads a line from meta_file into buffer and checks if NULL
   while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
   {
// Convert buffer to upper case, return a pointer to the first
// occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
      if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
      {
// Read chars from buffer and store them in AOML_number inside
// structure.
//
// Note: * means the field does not need to be included in vector of
// variable names to write to
         if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
             meta_struct->AOML_number = 0;
         continue;
      }
      if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
            meta_struct->WMO_id = 0;
         continue;
      }
      if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
            meta_struct->ARGOS_id = 0;
         continue;
      }
      if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
      {
         sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x);
         meta_struct->start_cycle_time += (float) (x/24.0);
         continue;
      }
      if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
            meta_struct->wmo_instrument = 0;
         continue;
      }
//newly added sensor manufacturer 
      if (strstr(strupr(buffer),"TEMPERATURE SENSOR MANUFACTURER") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %s",&manufacturer) != 1)
            sprintf (manufacturer,"\0");
         continue;
      }
      if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
            meta_struct->wmo_recorder = 0;
         continue;
      }
// DOWN TIME was changed to CYCLE TIME in meta files (CS)
      if (strstr(strupr(buffer),"CYCLE TIME") != NULL)
      {
          sscanf(buffer,"%*s %*s %*s %f",&x);
          meta_struct->cycle_time = x;
          continue;
      }
// UP TIME was changed to SURFACE TIME in meta files (CS)
      if (strstr(strupr(buffer),"SURFACE TIME") != NULL)
      {
         sscanf(buffer,"%*s %*s %*s %f",&x);
// (CS) now obsolete     meta_struct->cycle_time += (float) (x/24.0);
          meta_struct->start_cycle_time -= (float) (x/24.0);
          continue;
      }
// start_cycle_time = FIRST DOWN TIME - SURFACE TIME + START TIME
//            ^                 ^
//            6 hours           1 day     dfr97 days
      if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) 
      {
          sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
               &sday,&smonth,&syear,&shour,&smin);
          meta_struct->start_cycle_time += (float) (dfr97(syear,smonth,sday,shour,smin,0));
          continue;
      }
      if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
      {
          if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
             meta_struct->exp_no = 0;
          continue;
      }
//newly added hexadecimal ARGOS ID field 
      if (strstr(strupr(buffer),"HEXADECIMAL ARGOS ID") != NULL)
      {
         if (sscanf(buffer,"%*s %*s %*s %s",&meta_struct->hex_id) != 1)
            sprintf (meta_struct->hex_id,"\0");
         continue;
      }
      if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
      {
         if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
         {
            sprintf (meta_struct->float_type,"\0");
            continue;
         }
      }
      if (strstr(strupr(buffer),"PI     ") != NULL)
      {
         if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
         {
            sprintf (meta_struct->PI,"\0");
            continue;
         }
         if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
            sprintf (meta_struct->PI,"%s",ptr);
         if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
            sprintf (ptr,"\0");
         continue;
      }
      if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) 
      {
         sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
              &lday,&lmonth,&lyear,&lhour,&lmin);
         meta_struct->launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0));
         meta_struct->year=lyear;
         meta_struct->month=lmonth;
         meta_struct->day=lday;
         meta_struct->hour=lhour;
         meta_struct->minute=lmin;
         continue;
      }
      if ((strstr(strupr(buffer),"LAUNCH POSITION") != NULL) && (strstr(strupr(buffer),"LONM") != NULL)) 
      {
         sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d %d %d %d",&lat,&latm,&lon,&lonm);
         meta_struct->lat=lat;
         meta_struct->latm=latm;
         meta_struct->lon=lon;
         meta_struct->lonm=lonm;
         continue;
      }
      i = 0;
      while ((j = strlen(cal_keys[i])) > 1)
      {
// Get pointer to 1st occurence of cal_keys[i] in buffer
         if ((ptr = (char *)strstr(buffer,cal_keys[i])) != NULL){
           meta_struct->cals.c[i] = atof((const char *)(ptr+j));   
         } 
         i++;
      }
   } //while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
//check if the wmo_instrument type is consistent with the manufacturer, if not, exit with status 1
   if(((meta_struct->wmo_instrument==852) && (strcmp(manufacturer,fsi)!=0)) ||
      ((meta_struct->wmo_instrument==851) && (strcmp(manufacturer,sbe)!=0))   ) 
   {
      printf("manufacturer=%s,instrument type=%d\n",manufacturer,meta_struct->wmo_instrument);
      printf ("Error in meta file %s, manufacturer is not consistent with WMO instrument type\n",meta_name);
      if (error_file != NULL)
         fprintf (error_file,"Error in meta file %s, manufacturer is not consistent with WMO instrument type\n",meta_name);
      return(1);
   }  
   if (debug==1) printf("meta pgain=%f\n",meta_struct->cals.s.pgain);
   return 0;
}

/****************************************************************************
  char buffer[]="ABC DEF GHJ 23 345 67 890   76567"; 

  Get number of words from original and convert words into numbers, 
  save it to array a
  
****************************************************************************/

int nbrk ( char *original, int *a, int n )
{
   int i = 0; 
   char *temp, *p; 
/* Make a copy of original because strtok modifies its argument */ 
   temp =(char*) malloc ( strlen ( original ) * sizeof (*temp) + 1 ); 
   strcpy ( temp, original ); 
/* Find first token */ 
   p = strtok ( temp, " " ); 
/* Loop while there are more tokens AND places left in a */ 
   while ( p != NULL && i < n ) { 
/* Assumes if the first cell is a digit then the whole token is a number */ 
   if ( isdigit ( *p ) ) 
/* Convert the string to an int and assign it to a[i], increment i */ 
      a[i++] = (int)strtol ( p, NULL, 10 ); 

/* Find the next token in the same string */ 
      p = strtok ( NULL, " " ); 
   } 

/* Always free your malloc'd memory */ 
   delete temp; 

/* Return the elements filled in a */ 
   return i; 
} 


int main (int argc, char *argv[])
{

   char     *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy, 
            *ptr_path_log, *ptr_path_list,
            pathname_input_filename[161]="",
            pathname_trace[161]="", pathname_error[161]="";
   char     input_filename[80],output_directory[256], 
            msg[11]="", title[11]="", *ptr;
   struct   ts16_meta_struct ms;
                                                      // long t;
   long t;
   int      ret_val = 0;

// Program Name
// ------------
   printf("Entering Program: %s\n", PRGNAME);
   strcpy(title, "Program calibrates counts files for TS16 float");
   printf("%s\n", title);

// Input Arguments 
// ---------------
   if (argc < 6)
   {
      printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
      printf("\n");
      printf("cnt_infile:    counts input file name\n");
      printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
      printf("trace_outile:  output directory for generated trace file\n");
      printf("error_outfile: output directory for generated error file\n");
      printf("list_outfile:  output directory for generated list file\n");
      return 1;
   }

// Input Counts File Name 
// ----------------------
   printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
   sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
   if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
   {
      printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return ret_val;
   }
   printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
   sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
   if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
   {
      printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return ret_val ;
   }
   printf ("%s: Opening trace file %s ", PRGNAME, buffer);
   sprintf (buffer,"%s",argv[3]);
   if ((trace_file = fopen(buffer,"a+")) == NULL)
   {
      printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return ret_val;
   }
   printf("%s: OK -> trace file opened\n", PRGNAME);
   if (trace_file != NULL)
   {
      t = time(NULL);
      fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
   }

// ERROR File
// ----------
   sprintf (buffer,"%s",argv[4]);
   printf ("%s: Opening error file %s ", PRGNAME, buffer);
   if ((error_file = fopen(buffer,"a+")) == NULL)
   {
      printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
      if (trace_file != NULL)
       fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return ret_val;
   }
   printf("%s: OK -> error file opened\n", PRGNAME);
   if (trace_file != NULL)
      fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
   if (error_file != NULL)
   {
      t = time(NULL);
      fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
   }

// LIST File
// ---------
   if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
   {
      printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
             PRGNAME);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return ret_val;
   }
   sprintf (buffer,"%s",argv[5]);
   printf ("%s: Opening list file %s ", PRGNAME, buffer);
   if ((list_file = fopen(buffer,"a+")) == NULL)
   {
      printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    
      if (trace_file != NULL)
         fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
                  PRGNAME, buffer);
      if (error_file != NULL)
         fprintf (error_file,"ERROR (%s): Opening list file %s\n",
                  PRGNAME, buffer);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return ret_val;
   }
   printf("%s: OK -> list file opened\n", PRGNAME);
   if (trace_file != NULL)
      fprintf (trace_file,"%s: List file %s successfully opened\n", 
               PRGNAME, buffer);

// Open Input Counts File
// ----------------------
   if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
   {
       printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n", PRGNAME);
      if (trace_file != NULL)
         fprintf (trace_file,
                  "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
                  PRGNAME);
      if (error_file != NULL)
         fprintf (error_file,
                  "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
                  PRGNAME);
      if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);
      if (error_file != NULL)
          fclose (error_file);
      t = time(NULL);
      printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
      ret_val = 1;
      return(ret_val);
   }
   sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

   if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
   {                                                 // Failure: could not open cnt file 
      printf ("\nERROR (%s): Opening input file %s", 
              PRGNAME, pathname_input_filename);
    
       if (trace_file != NULL)
          fprintf (trace_file, "ERROR (%s): Opening input file %s\n", 
                   PRGNAME, pathname_input_filename);
       if (error_file != NULL)
          fprintf (error_file, "ERROR (%s): Opening input file %s\n", 
                   PRGNAME, pathname_input_filename);
       t = time(NULL);
       printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
       ret_val = 1;
       return ret_val;
   }
   else                                              // Success: cnt file opened
   {
      printf ("%s: Input file %s successfully opened\n", 
              PRGNAME, pathname_input_filename);
      if (trace_file != NULL)
         fprintf (trace_file,
                  "%s: Input file %s successfully opened\n", 
                  PRGNAME, pathname_input_filename);
// META File
// ---------
       if (get_meta_file_name (input_filename) != 0)
       {                                             // Failure: did not get meta filename     
          printf ("ERROR (%s): Finding meta file name", PRGNAME);
          if (error_file != NULL)
           fprintf (error_file,
                    "\nERROR (%s): Finding meta file name - returned name %s\n",
                    PRGNAME, input_filename);
          if (trace_file != NULL)
           fprintf (trace_file,
                    "\nERROR (%s): Finding meta file name - returned name %s\n",
                    PRGNAME, input_filename);
          ret_val = 1;
       }
       else
       {                                             // Success: got meta filename 
          if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
          {
             printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME); 
             if (trace_file != NULL) 
                fprintf (trace_file,
                         "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                         PRGNAME);
             if (error_file != NULL)
                fprintf (error_file,
                         "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                         PRGNAME);
             if (trace_file != NULL)                  // Close Log files
                fclose (trace_file);
             if (error_file != NULL)
                fclose (error_file);
             ret_val = 1;
          } // endif  ptr_path_meta
          sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);
          if (read_meta_file (pathname_input_filename, &ms) != 0)
          {                                         // Failure: couln't read meta_file
             printf ("\nERROR (%s): Reading meta file", PRGNAME);
             if (trace_file != NULL)
              fprintf (trace_file,"ERROR (%s): Reading meta file %s\n", 
                       PRGNAME, pathname_input_filename);
             if (error_file != NULL)
              fprintf (error_file, "ERROR (%s): Reading meta file %s\n", 
                       PRGNAME, pathname_input_filename);
             ret_val = 1;
          }
          else
          {  
             if (trace_file != NULL)
                fprintf (trace_file, "%s: Meta file %s successfully read\n", 
                         PRGNAME, pathname_input_filename);
             printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename);
// Convert Counts to Physical Units 
// --------------------------------
             sprintf (input_filename,"%s%s", output_directory, argv[1]); // !! using input storage
             if ( (ptr = strrchr(input_filename,'.')) == NULL)
                strcat (input_filename,".phy");
             else
                sprintf(ptr,".phy");                             // ???
                printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);
             if (convert_data(&ms, input_filename) == 0)
             {
                printf ("\n%s: CONVERT_DATA  successful\n", PRGNAME);
                if (trace_file != NULL)
                   fprintf (trace_file, "%s: CONVERT_DATA successful\n", PRGNAME);
             }
             else
             { 
                printf ("\nERROR (%s): CONVERT_DATA\n");
                if (trace_file != NULL)
                  fprintf (trace_file, "ERROR (%s): CONVERT_DATA\n", 
                           PRGNAME);
                if (error_file != NULL)
                  fprintf (error_file, "ERROR (%s): CONVERT_DATA\n", 
                           PRGNAME);
                ret_val = 1;
             } // endif: convert_data
          } // endif: read_meta_file
       } // endif: get_meta_file_name
   } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
   t = time(NULL);
   if ( ret_val == 0)
      strcpy(msg, "finished");
   else
      strcpy(msg, "terminated");
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   if (trace_file != NULL)
      fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));
  
   if (error_file != NULL)
      fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
   if (input_file != NULL)
      fclose (input_file);
   if (trace_file != NULL)
      fclose (trace_file);
   if (error_file != NULL)
      fclose (error_file);
   if (list_file != NULL)
      fclose (list_file);

   return ret_val;
}



//correct overflow problem for T and C/S counts,use breck owen's matlab algorithm
void correctOverflow(struct ts16_meta_struct *s,vector<float> PNew,vector<int>& i2_vector,vector<int>& i3_vector){
   vector<int> TCount;
   vector<int> SCount;

   vector<float> grad;
   vector<float> tst;
//use same variable names as Breck Owen's.
   int pmess,nprof,in,adj;
//int frst=999; //if find mininum is used
   int frst=0;   //if find maximum is used
   float T1,S1;
 
   if(debug==1) cout<<"Entering correctOverflow function"<<endl;
//since the depth is decreasing in original PNew vector, we need to reverse it to do overflow correction,
//finally at the end of this function, we reverse again to make the depth decreasing again.
   reverse(i2_vector.begin(),i2_vector.end());
   reverse(i3_vector.begin(),i3_vector.end());
   reverse(PNew.begin(),PNew.end());
   if(debug==1) cout<<"reverse pressures"<<endl;

//make a copy of i2 to T ,i3 to S count vector and reset i2,i3 vectors
   TCount.assign(i2_vector.begin(),i2_vector.end());i2_vector.clear();
   SCount.assign(i3_vector.begin(),i3_vector.end());i3_vector.clear();
   if(debug==1) cout<<"TCount size="<<TCount.size()<<" SCount size="<<SCount.size()<<endl;
//number of levels
   nprof=TCount.size();
   if(debug==1){
      for (int jj=0;jj<nprof;jj++){
         cout<<"1        PNew[jj]="<<PNew[jj]<<endl;
      }
   }

//index of first messy bad point suspected
   pmess=8;
//for j=9,18,27,...
   for(int j=pmess+1;j<nprof;j=j+(pmess+1))
   {
      if(j+1>=nprof) break;
//if the first message block's T and S are both 0, skip the overflow correction for this message block, go to next message
      if(TCount[pmess]==0 && SCount[pmess]==0) continue;
//if T and S are both 0, skip the overflow correction for this message block, go to next message
      if(TCount[j]==0 && SCount[j]==0) continue;

//CS: seems like fill value can also be 5000 for T and 1000 for S
    //if T=5000 and S=1000, skip the overflow correction for this message block, go to next message

//cout<<"T(j)= "<<TCount[j]<<"  S(j)= "<<SCount[j]<<endl;
      if(TCount[j]==5000 && SCount[j]==1000) continue;
    
//if T[9]>T[18]>T[27] does not satisfy, then skip correction for index 19 block
//only test this from the second message to the next to last message block,note: pmess+1=9
      if((j-pmess-1>pmess) && (j<nprof-2*(pmess+1)) && (TCount[j]>TCount[j-pmess-1] || TCount[j]<TCount[j+pmess+1])) continue;           
    
      T1=TCount[j]-(PNew[j]-PNew[j-1])*(TCount[j+1]-TCount[j])/(PNew[j+1]-PNew[j]);
      if(debug==1) cout<<"j= "<<j<<"  T1= "<<T1<<"  T(j-1)= "<<TCount[j-1]<<endl;
      if(fabs(T1-TCount[j-1])>4096){
         adj=((int)((TCount[j]-TCount[j-1])/4096))*4096;  
      //get gradient values,grad =diff(T(j-pmess:j-1))./diff(P(j-pmess:j-1)); matlab version
         for(int i=0;i<pmess;i++){
           grad.push_back((TCount[j-pmess+i]-TCount[j-pmess+i-1])/(PNew[j-pmess+i]-PNew[j-pmess+i-1]));            
           if(debug==1) cout<<"T grad="<<grad[i]<<endl;
         }
         TCount[j-1] = TCount[j-1]+adj;
//tst = grad*(TCount[j]-TCount[j-1])/(PNew[j]-PNew[j-1]);matlab version   
         for(int i=0;i<pmess;i++){
           tst.push_back(grad[i]*(TCount[j]-TCount[j-1])/(PNew[j]-PNew[j-1]));     
         }
     // tst will indicate where gradient changes sign,find the largest index which has sign(tst)<0
     //matlab version: frst = max(find(sign(tst)<0))+1;
         for(int i=0;i<pmess;i++){
           if(tst[i]<0 && i>frst) frst=i;//get maximum,frst first initialize to 0
           //if(tst[i]<0 && i<frst) frst=i;//get minimum, frst first initialize to 999
         }
         frst=frst+1;      
         if(debug==1) cout<<"T frst="<<frst<<endl;
         for(int i=frst;i<=pmess-1;i++){
            in=j-pmess+i-1;
            TCount[in]=TCount[in]+adj;
         }             
      }//end of if(fabs(T1-TCount[j-1])>4096)
      if(debug==1) cout<<"j= "<<j<<"T(j-1)= "<<TCount[j-1]<<endl;
//reset grad and tst vectors and frst
      grad.clear();tst.clear();
      frst=0; //if find maximum is used
      //frst=999; //if find minimum is used
      S1=SCount[j]-(PNew[j]-PNew[j-1])*(SCount[j+1]-SCount[j])/(PNew[j+1]-PNew[j]);
      if(fabs(S1-SCount[j-1])>1024 && (s->wmo_instrument==851)){//SBE,salinity correction
         adj=((int)((SCount[j]-SCount[j-1])/1024))*1024;  
//get gradient values,grad =diff(S(j-pmess:j-1))./diff(P(j-pmess:j-1)); matlab version
         for(int i=0;i<pmess;i++){
            grad.push_back((SCount[j-pmess+i]-SCount[j-pmess+i-1])/(PNew[j-pmess+i]-PNew[j-pmess+i-1]));            
            if(debug==1) cout<<"S grad="<<grad[i]<<endl;
         }
         SCount[j-1] = SCount[j-1]+adj;
//tst = grad*(SCount[j]-SCount[j-1])/(PNew[j]-PNew[j-1]);matlab version   
         for(int i=0;i<pmess;i++){
            tst.push_back(grad[i]*(SCount[j]-SCount[j-1])/(PNew[j]-PNew[j-1]));     
         }
// tst will indicate where gradient changes sign,find the largest index which has sign(tst)<0
//matlab version: frst = max(find(sign(tst)<0))+1;
         for(int i=0;i<pmess;i++){
           if(tst[i]<0 && i>frst) frst=i;//get maximum,frst first initialize to 0
           //if(tst[i]<0 && i<frst) frst=i;//get minimum, frst first initialize to 999
         }
         frst=frst+1;      
         if(debug==1) cout<<"S frst="<<frst<<endl;
         for(int i=frst;i<=pmess-1;i++){
            in=j-pmess+i-1;
            SCount[in]=SCount[in]+adj;
         }             
      }//end of if(fabs(S1-SCount[j-1])>1024) for SBE floats
      if(fabs(S1-SCount[j-1])>1024 && (s->wmo_instrument==852)){//FSI,conductivity correction            
         for(int i=j;i>=j-pmess+1;i--){
           adj=((int)(ceil(((double)(SCount[i]-SCount[i-1]-0.01)*1.0/1024.0))))*1024;        
           in=i-1;
           SCount[in]=SCount[in]+adj;
           if(debug==1)  cout<<"i="<<i<<" adj="<<adj<<"  in="<<in<<" SCount[in]="<<SCount[in]<<endl;
         }             
      }//end of if(fabs(S1-SCount[j-1])>1024)  
   }//end of first for loop
//make a copy of T to i2 ,S to i3 count vector
   i2_vector.assign(TCount.begin(),TCount.end());
   i3_vector.assign(SCount.begin(),SCount.end());
   reverse(i2_vector.begin(),i2_vector.end());
   reverse(i3_vector.begin(),i3_vector.end());
   reverse(PNew.begin(),PNew.end());
   if(debug==1) cout<<"reverse pressures again -> back to normal"<<endl;
//reset TCount,SCount vectors
   TCount.clear();
   SCount.clear();
}//end of function

/*added new
   truncateCounts
   Add check to see if number of levels in cnt file is larger than the largest possible bin
   number, if it is, then truncate the cnt vector to be the length of largest possible bin number      
*/
void truncateCounts(struct ts16_meta_struct *s,vector<int>& i1_vector,vector<int>& i2_vector,vector<int>& i3_vector,vector<int>& i4_vector){
   int j1,j2,j3; 
   int largest_bin_num;
//get largest possible bin number
   j1=(int)(s->cals.s.p1/s->cals.s.dp1);
   j2=(int)((s->cals.s.p2-s->cals.s.p1)/s->cals.s.dp2+j1);
   j3=(int)((s->cals.s.p3-s->cals.s.p2)/s->cals.s.dp3+j2);
   largest_bin_num=j3; //index begins from 0
//reverse the vectors so the depth increases now
   reverse(i1_vector.begin(),i1_vector.end());
   reverse(i2_vector.begin(),i2_vector.end());
   reverse(i3_vector.begin(),i3_vector.end());
   reverse(i4_vector.begin(),i4_vector.end());
   if(i1_vector.size()>largest_bin_num){//truncate the profile length to be largest_bin_num long          
       int newsize=largest_bin_num;
       int*  pnew= new int[newsize];
       int*  tnew= new int[newsize];
       int*  snew= new int[newsize];
       int* flagnew= new int[newsize];
       if (trace_file != NULL) fprintf (trace_file,"=> profile length is larger than largest bin number,truncate it now!\n");
       for(int jj=0;jj<newsize;jj++){
           pnew[jj]=i1_vector[jj];tnew[jj]=i2_vector[jj];snew[jj]=i3_vector[jj];flagnew[jj]=i4_vector[jj];
       }
//reset and refill i1,i2,i3,i4 vectors
       i1_vector.clear();i2_vector.clear();i3_vector.clear();i4_vector.clear();
       for(int jj=0;jj<newsize;jj++){
          i1_vector.push_back(pnew[jj]);
          i2_vector.push_back(tnew[jj]);
          i3_vector.push_back(snew[jj]);
          i4_vector.push_back(flagnew[jj]);
       }
       delete pnew;
       delete tnew;
       delete snew;
       delete flagnew;
   }//end of if
 
//reverse the vectors so the depth decreases now
   reverse(i1_vector.begin(),i1_vector.end());
   reverse(i2_vector.begin(),i2_vector.end());
   reverse(i3_vector.begin(),i3_vector.end());
   reverse(i4_vector.begin(),i4_vector.end());
   
}

void calc_fsi_p(vector<float>& P_Vector,float P_bot, float P_top, int nbin, float P1, float P2, float avg1, float avg2, float avg3){

   float Ptest, dp,dp1; 
// start at bottom of profile
   Ptest = P_bot;
   P_Vector.assign(nbin,(float)0.0);
// find bin interval for bottom bin
   if(Ptest + avg1/2+1 < P1){
      dp = avg1;    
   }else if(Ptest + avg2/2+1 < P2){
      dp = avg2;
   }else{
      dp = avg3;
   }
// fill in values from engineering data
   if(debug==1) cout<<"nbin"<<nbin<<endl;
   if(debug==1) cout<<"before ---- P_Vector[nbin-1] "<<P_Vector[nbin-1]<<endl;
   if(debug==1) cout<<"before ---- P_Vector[0] "<<P_Vector[0]<<endl;
   P_Vector[nbin-1] = P_bot;
// p_top is not reliable, but attempt to use it is at the end of this function
//   P_Vector[0] = P_top;
   if(debug==1) cout<<"after ---- P_Vector[nbin-1] "<<P_Vector[nbin-1]<<endl;
   if(debug==1) cout<<"after ---- P_Vector[0] "<<P_Vector[0]<<endl;
// get values for intervening bins
// n>=0 if we don't assign p_top
//   for(int n=nbin-2;n>=1;n--){
   for(int n=nbin-2;n>=0;n--){  
       dp1 = dp/2; // pressure change to top of last bin
// bin interval for next bin set by value at top of last bin
       Ptest = P_Vector[n+1]-dp1;
// find bin interval for this bin
       if(Ptest < P1){
          dp = avg1;
       }else if(Ptest < P2){
          dp = avg2;
       }else{
          dp = avg3;
       }
       // interval between bin centers
       dp1 = dp1 +dp/2;
       // pressure at center of bin 
       P_Vector[n] = P_Vector[n+1] - dp1;
       if(debug==1) cout<<"n "<<n<<"P_Vector[n] "<<P_Vector[n]<<endl;
   }
   if(P_top>0 && P_Vector[1]>P_top) {
     P_Vector[0] = P_top;
   }
// add condition of surface pressure 
   if( P_top>0 && P_top<P_Vector[1] )
       P_Vector[0] = P_top;

}
