/*****************************************************************
   Revision Date: 14-Jan-11 (ER -If ROM version=5.23 then surface press parameters are:
                                 SURFACE PRESSURE (DBAR) and
                                 SURFACE PRES START OF DESCENT (DBAR) 
                 25-Aug-10 (ER - Replace parameter name 
                                 'PRESSURE AT START OF DESCENT (DECIBARS)' with
                                 'SURFACE PRES AFTER RESET(PREV CYCLE)' 
                 23-Aug-10 (ER - Replace parameter name 'START SURFACE PRESSURE (DBAR)'
                                 with 'SURF PRES BEF RESET(PREV CYC)(COUNTS)' 
                 30-Dec-09 (ER - change function get_meta_file_name to adapt to
                                 variable length transmission ids.
  				   01-Jun-05 (CS&XXD -change the pressure computation for bin=1)
                 12-Aug-03 (XX&CS-diagnostics part,if counts are 0,do not need to use
		                          calibration equation,set the phy output to 0)
                 09-Aug-02 (CS - replaced hardwired 2 by tgain/4 (sgain/4) in
                                 calculation of some header T (S)
                                 needed as correction for a bug of SOLO_TS9)
                 05-Mar-02 (JZ - replaced call to function 'compute_sio_pressure'
                            with correct computation.
                 29-Aug-01 (CS - changed salinity computation subtract 1024
                                 from counts if s>41)
                 09-Nov-00 (RS - Bitterman updates( diff1109.lis)
                 31-Aug-00 (JZ)
                 29-Aug-00 (JZ - rename ARGOLIST_DIR)
                 01-Aug-00 (RS - eliminate redundant stm & add program name
                                to stdout and trace/error output)
                 07-07-00 (JZ)
  cnt2phy_TS9
  ===========
  Converts count to physical units for Type TS9 floats

******************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256  //Maximum length of each scan read from *.str

char* strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

FILE *input_file, *out_file, *error_file, *trace_file, *meta_file, *list_file;

char PRGNAME[] ="cnt2phy_ts9";
char buffer[512];

int debug=0;//1 debug,0 no debug

struct sio_cals { double pgain;
                  double poff;
                  double tgain;
                  double toff;
                  double sgain;
                  double soff;
                  double pmp_gain;
                  double pmp_off;
                  double cpugain;
                  double cpuoff;
                  double blok;
                  double av1;
                  double av2;
                  double pb1;
                  double pb2;
                };

union u_cals { struct sio_cals s;
               double c[sizeof(struct sio_cals)/sizeof(double)];
             };

struct solo_ts_meta_struct { int AOML_number;
                             int WMO_id;
                             int ARGOS_id;
                             float cycle_time;
                             float start_time;
                             union u_cals cals;};

/******************************************************************************
  Computes salinity for SIO salinity floats from raw salinity counts
******************************************************************************/
//float compute_sio_salinity (struct solo_ts_meta_struct *s, int s_counts)
//TEST CS
float compute_sio_salinity (struct solo_ts_meta_struct *s, float s_counts)
{
 return (float)((s->cals.s.sgain * s_counts + s->cals.s.soff)*0.001);
}

/******************************************************************************
  Routine for computing temperature for SIO floats
******************************************************************************/
float compute_sio_temperature (struct solo_ts_meta_struct *s, float t_counts)
{
 return (float)((s->cals.s.tgain * t_counts + s->cals.s.toff)*0.001);
}

/******************************************************************************
  Routine for computing pressure for SIO floats
******************************************************************************/
float compute_sio_pressure (struct solo_ts_meta_struct *s, int p_counts)
{
 int i;
 float p;
 if(p_counts==1)
    p=(s->cals.s.blok*1.5+5)/2;
 else if (p_counts <= s->cals.s.pb1){
  i = (int) (p_counts*s->cals.s.blok);
  p=(float)i;
 }
 else if (p_counts <= s->cals.s.pb2){
  i = (int) ((p_counts - s->cals.s.pb1)*s->cals.s.av1*s->cals.s.blok + s->cals.s.pb1*s->cals.s.blok);
  p=(float)i;
 }
 else{
  i = (int) ((p_counts - s->cals.s.pb2)*s->cals.s.av2*s->cals.s.blok +
      (s->cals.s.pb2 - s->cals.s.pb1)*s->cals.s.av1*s->cals.s.blok +
    s->cals.s.pb1*s->cals.s.blok);
  p=(float)i;
}

 return p;
}
/******************************************************************************
  Routine for computing pressure for SIO floats
******************************************************************************/
float compute_sio_air_pressure (struct solo_ts_meta_struct *s, int p_counts)
{
 return (float) p_counts;
}

/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char *strupr(char* s)
{
  char* p = s;

  while (*p)
	 {
    if (islower((int)*p))
      *p = toupper((int)*p);

    ++p;
  }

  return s;
}

/******************************************************************************
  Converts data from counts to physical units
******************************************************************************/
int convert_sio_ctd_data(struct solo_ts_meta_struct *s, char *output_name)
{
  int i1=0,i2=0,i3=0,i4=0,ip[128],iflg[128];
  float it[128],ic[128];
  float a1,a2,a3,a2sav;
  char *ptr_phy_filename,col_space[]="   0";
  char *aa1;

  if ((out_file = fopen(output_name,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file," CONVERT_TYPE9_DATA: error opening output file %s\n",
               output_name);

    if (error_file != NULL)
      fprintf (error_file," CONVERT_TYPE9_DATA: error opening output file %s\n",
               output_name);

    return 1;
  }

  if (trace_file != NULL)
    fprintf (trace_file," CONVERT_TYPE9_DATA: output file %s successfully opened\n",output_name);

  printf ("Output file %s successfully opened\n",output_name);


  // Write Output Profile Physical Unit File Name to List File
  // Check if last occurence of '/' in buffer is NULL
  if ((ptr_phy_filename = strrchr(output_name, '/')) == NULL)
  {
    printf ("\nCan't get phy file basename %s", output_name);

    if (trace_file != NULL)
      fprintf (trace_file,
               "\n%s : Can't get phy file basename %s", PRGNAME, output_name);

    if (error_file != NULL)
      fprintf (error_file,
               "\n%s: Can't get phy file basename %s", PRGNAME, output_name);

    return 1;
  } // endif: ptr_phy_filename

  ptr_phy_filename++; // Increment file name pointer
  fprintf (list_file,"%s\n", ptr_phy_filename); // Write file name to list file

  // Reads a line from input_file into buff and check if NULL
  while (fgets(buffer,sizeof(buffer),input_file) != NULL)
  {
    if (strstr(strupr(buffer),"DRIFT TEMPERATURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
        return(1);

      sprintf (buffer,"DRIFT TEMPERATURE (DEG C)               %-7.3f %-7.3f\n",
      compute_sio_temperature(s,(float) i1),compute_sio_temperature(s,(float) i2));
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"DRIFT PRESSURE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
        return(1);


      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      a2 = (float) (s->cals.s.pgain*i2 + s->cals.s.poff);
      sprintf (buffer,"DRIFT PRESSURE (DBAR)                   %-+6.1f %-+6.1f\n",a1,a2);


      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"DRIFT SALINITY (8-MSB") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %d",&i1) != 1)
        return(1);

      a2=compute_sio_salinity(s,(float) i1);
      while (a2>41)
      {
         if (trace_file != NULL)
            fprintf (trace_file,"\n A) salinity > 41, subtract 1024 from counts\n");
         i1=i1-1024;
         a2=compute_sio_salinity(s,(float) i1);
      }

      sprintf (buffer,"DRIFT SALINITY (8-MSB, PSU)             %-7.3f\n",
        a2);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"SURF PRES BEF RESET(PREV CYC)(COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      sprintf (buffer,"SURFACE PRES BEFORE RESET(PREV CYCLE)   %-6.1f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      fflush(out_file);
      continue;
    }

    if (strstr(buffer,"SURFACE PRESSURE (COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        return(1);
      a1 = (float) (s->cals.s.pgain*i1 + s->cals.s.poff);
      sprintf (buffer,"SURFACE PRESSURE (DBAR)                 %-6.1f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      fflush(out_file);
      continue;
    }


    if (strstr(buffer,"CPU") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        return(1);
      if(i1==0){a1=0.0;}
      else{
      a1 = (float) (s->cals.s.cpugain*i1 + s->cals.s.cpuoff);
      }
      sprintf (buffer,"BATTERY CPU (VOLT)                      %-5.2f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"PUMP") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&i1) != 1)
        return(1);
      if(i1==0){a1=0.0;}
      else{
      a1 = (float) (s->cals.s.pmp_gain*i1 + s->cals.s.pmp_off);
      }
      sprintf (buffer,"BATTERY PUMP (VOLT)                     %-5.2f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"P,T,S BEFORE ASCENT") != NULL)
    {
        if (sscanf(buffer,"%*s %*s %*s %*s %d %d %d",&i1,&i2,&i3) != 3)
        return(1);

      a1 = (float)(s->cals.s.pgain*(float)i1 + s->cals.s.poff);
// have to divide counts by 4 here (bug of SOLO_TS9)
      a3=compute_sio_salinity(s,((float) i3)/4);
//      a3=compute_sio_salinity(s,(float) i3);
      while (a3>41)
      {
         if (trace_file != NULL)
            fprintf (trace_file,"\n B) salinity > 41, subtract 1024 from counts\n");
         i3=i3-1024;
// have to divide counts by 4 here (bug of SOLO_TS9)
         a3=
compute_sio_salinity(s,((float) i3)/4);
//         a3=compute_sio_salinity(s,(float) i3);
      }

// have to divide counts by 4 here (bug of SOLO_TS9)
      a2=
compute_sio_temperature(s,((float) i2)/4);
//      a2 = compute_sio_temperature(s,(float) i2);
      if(i1==0) a1=0.0;
      if(i2==0) a2=0.0;
      if(i3==0) a3=0.0;
      sprintf (buffer,"P,T,S BEFORE ASCENT (DB,DEG C,PSU)      %-7.3f %-7.3f %-7.3f\n",a1,a2,a3);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"P,T,S +TWO SECONDS") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %d %d %d",&i1,&i2,&i3) != 3)
        return(1);

      a1 = (float)(s->cals.s.pgain*(float)i1 + s->cals.s.poff);
// have to divide sgain by 4 here (bug of SOLO_TS9)
// have to divide counts by 4 here (bug of SOLO_TS9)
      a3=
compute_sio_salinity(s,((float) i3)/4);
//      a3=compute_sio_salinity(s,(float) i3);
      while (a3>41)
      {
         if (trace_file != NULL)
            fprintf (trace_file,"\n C) salinity > 41, subtract 1024 from counts\n");
         i3=i3-1024;
// have to divide counts by 4 here (bug of SOLO_TS9)
         a3=
compute_sio_salinity(s,((float) i3)/4);
//         a3=compute_sio_salinity(s,(float) i3);
      }

// have to divide counts by 4 here (bug of SOLO_TS9)
      a2=
compute_sio_temperature(s,((float) i2)/4);
//      a2 = compute_sio_temperature(s,(float) i2);

      if(i1==0) a1=0.0;
      if(i2==0) a2=0.0;
      if(i3==0) a3=0.0;

      sprintf (buffer,"P,T,S +TWO SECONDS (DB,DEG C,PSU)       %-7.3f %-7.3f %-7.3f\n",a1,a2,a3);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"P,T,S +FOUR SECONDS") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %d %d %d",&i1,&i2,&i3) != 3)
        return(1);

      a1 = (float)(s->cals.s.pgain*(float)i1 + s->cals.s.poff);
// have to divide counts by 4 here (bug of SOLO_TS9)
      a3=
compute_sio_salinity(s,((float) i3)/4);
//      a3=compute_sio_salinity(s,(float) i3);
      while (a3>41)
      {
         if (trace_file != NULL)
            fprintf (trace_file,"\n D) salinity > 41, subtract 1024 from counts\n");
         i3=i3-1024;
// have to divide counts by 4 here (bug of SOLO_TS9)
         a3=
compute_sio_salinity(s,((float) i3)/4);
//         a3=compute_sio_salinity(s,(float) i3);
      }

// have to divide counts by 4 here (bug of SOLO_TS9)
      a2=
compute_sio_temperature(s,((float) i2)/4);
//      a2 = compute_sio_temperature(s,(float) i2);

      if(i1==0) a1=0.0;
      if(i2==0) a2=0.0;
      if(i3==0) a3=0.0;

      sprintf (buffer,"P,T,S +FOUR SECONDS (DB,DEG C,PSU)      %-7.3f %-7.3f %-7.3f\n",a1,a2,a3);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"AIR PRESSURE END OF LAST SURFACE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);

      a1 = compute_sio_air_pressure (s,i1);
      sprintf (buffer,"AIR PRESSURE END OF LAST SURFACE        %-7.3f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"AIR PRESSURE START OF LAST SURFACE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);

      a1 = compute_sio_air_pressure (s,i1);
      sprintf (buffer,"AIR PRESSURE START OF LAST SURFACE      %-7.3f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

//  if (strstr(buffer,"PRESSURE AT START OF DESCENT") != NULL)
    if (strstr(buffer,"SURF PRES AFTER RESET(PREV CYC)(COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      a1 = (float)(i1 * s->cals.s.pgain + s->cals.s.poff);
      if(i1==0) a1=0.0;
//    sprintf (buffer,"PRESSURE AT START OF DESCENT (DECIBARS) %-7.3f\n",a1);
      sprintf (buffer,"SURFACE PRES AFTER RESET(PREV CYCLE)    %-7.3f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"SURFACE PRES START OF DESCENT (COUNTS)") != NULL)
    {
    printf( "Inside SURFACE PRES START OF DESCENT (COUNTS)" );
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);
      a1 = (float)(i1 * s->cals.s.pgain + s->cals.s.poff);
      if(i1==0) a1=0.0;
      sprintf (buffer,"SURFACE PRES START OF DESCENT (DBAR)    %-7.3f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"PRESSURE AT END OF DESCENT") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);

//      a1 = compute_sio_pressure (s,i1);
      a1 = (float)(i1 * s->cals.s.pgain + s->cals.s.poff);
      if(i1==0) a1=0.0;
      sprintf (buffer,"PRESSURE AT END OF DESCENT (DECIBARS)   %-7.3f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"PRESSURE AT END OF ASCENT") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&i1) != 1)
        return(1);

//      a1 = compute_sio_pressure (s,i1);
      a1 = (float)(i1 * s->cals.s.pgain + s->cals.s.poff);
      if(i1==0) a1=0.0;
      sprintf (buffer,"PRESSURE AT END OF ASCENT (DECIBARS)    %-7.3f\n",a1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"DRIFT SALINITY (COUNTS)") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d %d",&i1,&i2) != 2)
        return(1);

      a1=compute_sio_salinity(s,(float) i1);
      a2=compute_sio_salinity(s,(float) i2);
      if(debug>0) printf("drift salinity i1=%d,i2=%d,a1=%f,a2=%f\n",i1,i2,a1,a2);

      while (a1>41)
      {
         if (trace_file != NULL)
            fprintf (trace_file,"\n E) salinity > 41, subtract 1024 from counts\n");
         i1=i1-1024;
         a1=compute_sio_salinity(s,(float) i1);
      }
      while (a2>41)
      {
         if (trace_file != NULL)
            fprintf (trace_file,"\n F) salinity > 41, subtract 1024 from counts\n");
         i2=i2-1024;
         a2=compute_sio_salinity(s,(float) i2);
      }
      if(i1==0) a1=0.0;
      if(i2==0) a2=0.0;
      sprintf (buffer,"DRIFT SALINITY (PSU)                    %-7.3f %-7.3f\n",
        a1,a2);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      continue;
    }

    if (strstr(buffer,"COLUMNS") != NULL)
    {
      fprintf (out_file,"NUMBER OF COLUMNS                       4\n");
      fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
      fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
      fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
      fprintf (out_file,"4. COLUMN                               CHECKSUM FLAG\n");
      rewind(meta_file);

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
        if (strstr(buffer,"================") != NULL)
          break;

      while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
      {
        if (strstr(buffer,"================") != NULL)
        {
          fprintf (out_file,"==========================================================\n");
          break;
        }
        fwrite (strupr(buffer),1,strlen(buffer),out_file);
      }

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
        if (strstr(buffer,"LATITUDE") != NULL)
          break;

      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
      {
        if (strstr(buffer,"================") != NULL)
        {
          fprintf (out_file,"==========================================================\n");
          break;
        }
        fwrite (strupr(buffer),1,strlen(buffer),out_file);
//        fwrite (strupr(buffer),1,strlen(buffer),out_file);

//        if (strstr(buffer,"================") != NULL)
//          break;
      }

      i1 = 0;

      while (fgets(buffer,sizeof(buffer),input_file) != NULL)
      {
        if (sscanf(buffer,"%d %f %f %d",&ip[i1],&it[i1],&ic[i1],&iflg[i1]) == 4)
          i1++;
      }
      a2sav=39;
      for (i2=0; i2<i1; i2++)
      {
        a2=compute_sio_salinity(s,ic[i2]);
        while (a2>a2sav+2)
        {
          if (trace_file != NULL)
              fprintf (trace_file,"\n G) salinity > prev+2, subtract 1024 from counts\n");
          ic[i2]=ic[i2]-1024;
          a2=compute_sio_salinity(s,ic[i2]);
        }
        a2sav=a2;
        if (a2sav<36)
           a2sav=39;

        sprintf (buffer,"%6.1f   %6.3f   %6.3f   %1d\n",
                 compute_sio_pressure(s,ip[i2]),
                 a1=compute_sio_temperature(s,it[i2]),
                 a2,iflg[i2]);

        fwrite (buffer,1,strlen(buffer),out_file);
      }
    }
    else
      fprintf (out_file,buffer);

    continue;
  }

  return 0;
}


/******************************************************************************
  Assumes str_name = *.cnt, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
  char *ptr,tmp_buff[80];

  if ((ptr = strrchr(strupr(str_name),'_')) == NULL)
    strcat(str_name,".meta");
  else
  {
    *ptr = '\0';
    sprintf (tmp_buff,"%s", str_name );
//  sprintf (tmp_buff,"%s",ptr-10);
    sprintf (str_name,"%s.meta",tmp_buff);
  }

  return 0;
}


/******************************************************************************
  Reads meta_file
******************************************************************************/
int read_solo_ts_meta_file (char *meta_name, struct solo_ts_meta_struct *meta_struct)
{
  int i,j;
  char *ptr;
  char *cal_keys[] = {"PGAIN=", "POFF=","TGAIN=", "TOFF=","SGAIN=", "SOFF=",
                      "PMP_GAIN=", "PMP_OFF=", "CPUGAIN=", "CPUOFF=",
                      "BLOK=", "AV1=", "AV2=", "PB1=", "PB2=", "CDIV=", "\0"};
  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_time = (float) 0.0;
  printf("\nOpening meta file %s",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("\nError opening meta file %s",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (meta_file != NULL)
    fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
        meta_struct->AOML_number = 0;

      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;

      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;

      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof(ptr+j);

      i++;
    }
  }

  return 0;
}

/******************************************************************************
  'main' function
******************************************************************************/
int main (int argc, char *argv[])
{
  char *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy,
       *ptr_path_log, *ptr_path_list,
       pathname_input_filename[161]="",
       pathname_trace[161]="", pathname_error[161]="";

  char input_filename[80],output_directory[256],
       msg[11]="", title[11]="", *ptr;

  struct solo_ts_meta_struct s;

  unsigned long t;
  int      ret_val = 0;

// Program Name
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program calibrates counts files for SOLO_TS9 float");
  printf("%s\n", title);

// Input Paramenters
// -----------------
  if (argc < 6)
  {
    printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
    printf("\n");
    printf("cnt_infile:    counts input file name\n");
    printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

// Input Counts File Name
// ----------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Output directory path: defaults to PHYFILES_DIR
// -----------------------------------------------
  if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
  sprintf (output_directory,"%s/",argv[2]);

// TRACE File
// ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val ;
  }

  printf ("%s: Opening trace file %s ", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[3]);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> trace file opened\n", PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// ERROR File
// ----------
  sprintf (buffer,"%s",argv[4]);
  printf ("%s: Opening error file %s ", PRGNAME, buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> error file opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening list file %s ", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n",
              PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n",
             PRGNAME, buffer);

// Open Input Counts File
// ----------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n",
            PRGNAME);
    if (trace_file != NULL)
      fprintf (trace_file,
               "ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,
               "ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
    printf ("\nERROR (%s): Opening input file %s",
            PRGNAME, pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file, "ERROR (%s): Opening input file %s\n",
               PRGNAME, pathname_input_filename);
    if (error_file != NULL)
      fprintf (error_file, "ERROR (%s): Opening input file %s\n",
                PRGNAME, pathname_input_filename);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
  }
  else
  {
    printf ("%s: Input file %s successfully opened\n",
            PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file,
              "%s: Input file %s successfully opened\n",
              PRGNAME, pathname_input_filename);

// META File
// ---------
    if (get_meta_file_name (input_filename) != 0)
    {
      printf ("ERROR (%s): Finding meta file name", PRGNAME);
      if (error_file != NULL)
      fprintf (error_file,
               "\nERROR (%s): Finding meta file name - returned name %s\n",
               PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "\nERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      ret_val = 1;
    }
    else
    {                                             // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);
        if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                 PRGNAME);

        if (error_file != NULL)
          fprintf (error_file,
                   "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                   PRGNAME);

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);
        if (error_file != NULL)
          fclose (error_file);

        ret_val = 1;
      } // endif  ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_solo_ts_meta_file (pathname_input_filename, &s) != 0)
      {
        printf ("\nERROR (%s): Reading meta file", PRGNAME);
        if (trace_file != NULL)
          fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        if (error_file != NULL)
          fprintf (error_file, "ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        ret_val = 1;
      }
      else
      {
        if (trace_file != NULL)
          fprintf (trace_file, "%s: Meta file %s successfully read\n",
                   PRGNAME, pathname_input_filename);
          printf ("\n%s: Meta file %s successfully read\n",
                  PRGNAME, pathname_input_filename);


// Convert Counts to Physical Units
// --------------------------------
        sprintf (input_filename,"%s%s",output_directory,argv[1]);

        if ((ptr = strrchr(input_filename,'.')) == NULL)
          strcat (input_filename,".phy");
        else
          sprintf(ptr,".phy");

        printf ("%s: Output Path & File Name - %s\n", PRGNAME,input_filename);

        if (convert_sio_ctd_data(&s, input_filename) == 0)
        {
          printf ("\n%s: CONVERT_TYPE4_DATA  successful\n", PRGNAME);
          if (trace_file != NULL)
            fprintf (trace_file, "%s: CONVERT_TYPE4_DATA successful\n", PRGNAME);
        }
        else
        {
          printf ("\nERROR (%s): CONVERT_TYPE4_DATA\n", PRGNAME);
          if (trace_file != NULL)
            fprintf (trace_file, "ERROR (%s): CONVERT_TYPE4_DATA\n",
                     PRGNAME);
          if (error_file != NULL)
            fprintf (error_file, "ERROR (%s): CONVERT_TYPE4_DATA\n",
                     PRGNAME);
          ret_val = 1;
        } // endif: convert_sio_ctd_data
      } // endif: read_solo_ts_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);
  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
   fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
   fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
    fclose (list_file);

  return ret_val;
}
