/******************************************************************************
   Revision Date: 21-JUN-04 (XXD&CS - do not allow negative profile numbers
                            (bug fix); do not try to write a data file if
                            no lines are available (j=n_scans=0), in
                            get_type1_data)
                  07-OCT-03 (XX-correct profile_no if it is not
                            continuous,follow ts3 practice)
                  15-AUG-03 (XX&CS-add exit status 2 when frame1 missing problem happens 
                             in some but not all profile cycles)   
                  20-NOV-01 (CS - write abs(s->WMO_id) not s->WMO_id)
                  31-May-01 (JZ) - Changed 'BATTERY CPU' to 'BATTERY' 
                  18-DEC-00 (CS - Make sure to use messages received only once - Test!)
                  20-Nov-00 (DB - change uppercase in HEADER &
                             RS - fix return from get_type1_data)  
                  31-Aug-00 (JZ) 
                  10-Aug-00 (RS - rename ARGOLIST_DIR)
                  27-Jul-00 (RS - eliminate redundant stm & add program name
                                  to stdout and trace/error output)
                  29-Jun-00 (RS - replaced juldat with ymd2julday)
                  20-Jun-00 (RS - change LOG file name)
                  14-Jun-00 (RS - update get_raw_data, most_likely_frame
                                  get_meta_file_name, remove prescan_data,
                                  read_type1_meta_file 
                  09-Jun-00 (JZ)
                  03-Feb-00 (RS - fixed union and added "strupr" function) 

   hex2cnt_t1.c
   ============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   PALACE floats PALACE_T1 manufactured in 1997 by Webb Research Cooperation
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans        2000   //Maximum scans for each profile
#define buff_size         256   //Maximum length of each scan read from *.hex
#define max_positions     100   //Maximum number of positions per profile
#define min_frame_no        1   //Minimun frame number
#define frames_per_message 20   //Number of frames per message
#define good_bad_position  43   //Location of status byte in *.hex records
#define start_of_hex_data  63   //Start position of data in *.hex records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0

int frame1_problem_count=0,total_cycles_count=0;//added new.

char* strupr(char* ptr_to_string_to_be_upper_cased); 

FILE *input_file, *out_file, *error_file, *trace_file, *meta_file, *list_file;

char PRGNAME[] ="hex2cnt_t1";
char buffer[512];

struct aoml1_cals { double R1;
                    double R2;
                    double R3;
                    double R4;
                    double T1;
                    double T2;
                    double T3;
                    double pslope;
                    double poff;
                    double vslope;
                    double voff;
                  };

union u_cals { struct aoml1_cals s;
               double c[sizeof(struct aoml1_cals)/sizeof(double)];
             };

struct type1_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
         int wmo_instrument;
         int wmo_recorder;
         int exp_no;
         unsigned char float_type[40];
         unsigned char PI[60];
                           float ARGOS_rep_rate;
                           float cycle_time;
                           float start_cycle_time;
                           float start_time;
			   float start_of_cycles;//added Start_time+delay_of_start+down_time			   			   
                           union u_cals cals;
                         };

// Raw data structure used in decoding Temperature/Pressure (T/P)
// float data.  Part of RAW_DATA_RECORD described below
struct data_fmt { unsigned char controller_sn;
                  unsigned char profile_no;
                  unsigned char profile_length;
                  unsigned char termination_byte;
                  unsigned char bottom_temp[2];
                  unsigned char bottom_pressure;
                  unsigned char battery_volts;
                  unsigned char surface_pressure;
                  unsigned char temperature[115][2]; // 115 is # of press/temp pairs
                  unsigned char unused;};

// Equivalence of structure with linear array.  Linear array
// contains raw data byte minus first two bytes (check sum and
// frame number) of each frame.  Read T/P data format description 
union data_size { unsigned char data_bytes[frames_per_message][30];
                  struct data_fmt d_fmt;
                };

// Intermediate structure containing T/P raw data and positions 
struct raw_data_record { int    jday;
                         double jd[max_positions];
                         float  latitude[max_positions];
                         float  longitude[max_positions];
                         char   date_time[max_positions][32];
                         int    n_xmits[max_positions];
                         char   sat_name[max_positions];
                         char   loc_class[max_positions];
                         union  data_size data;
                       };

// Final structure containig T/P data and positions 
struct output_record { int    float_id;
                       int    profile_no;
                       int    jd;
                       int    status[frames_per_message];
                       int    controller_sn;
                       int    profile_length;
                       int    termination_byte;
                       int    bottom_temperature;
                       int    bottom_pressure;
                       int    surface_pressure;
                       int    battery_voltage;
                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       int    press_temp[115][2];
                     };

long ymd2julday (int, int, int);

/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char  *strupr(char* s)
{
  char* p = s;
  while (*p) {
    if (islower((int)*p)) *p = toupper((int)*p);
    ++p;
  }
  return s;
}

/******************************************************************************
  Computes days from 1997
******************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
  int    yr, mo, dy, day_from_97;
  long   passed_jd, jd_xx0101, jd_970101;
  double d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
   yr += 2000;
  else if (yr < 1900)
   yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
   return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year) 
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
  day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs, 
                                                // mins, secs)
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
  return (d_jd);                                // and adds this to d_jd
}

/******************************************************************************
   
******************************************************************************/
int data_flag1(int bin_no, int *status_word)
{
  int i;
  int ranges[frames_per_message][2] = {0,10,10,25,25,40,40,55,55,70,70,85,85,100,100,115};

  for (i=0; i<frames_per_message; i++)
    if ((bin_no >= ranges[i][0]) && (bin_no <= ranges[i][1]))
      return status_word[i];

  return BAD;
}

/******************************************************************************
  Writes output data to disk for T/P floats.
  Profile 0 is transmitted when float is first turned on.
  It has no temperature or pressure data. 
******************************************************************************/
int write_to_disk (char *dir_name, struct type1_meta_struct *s, struct output_record *x)
{
  int  i,j,id;
  char *ptr_cnt_filename;

  id = x->float_id;

  if ((x->status[0]) != GOOD) //Must have received first scan
  {
    if (trace_file != NULL)
      fprintf (trace_file,"First scan not received from float %05d - no data written\n",x->float_id);

    if (error_file != NULL)
      fprintf (error_file,"First scan not received from float %05d - no data written\n",x->float_id);

    printf ("First scan not received from float %05d - no data written\n",x->float_id);
    return(1);
  }

  sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
  printf ("Opening output file %s...",buffer);

  if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
  {
    printf ("ERROR\n");

    if (trace_file != NULL)
      fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);

    if (error_file != NULL)
      fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);

    return(1);
  }

  if (trace_file != NULL)
    fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);
  printf ("OK\n");

  // Write Output Profile Count File Name to List File

  // Check if last occurence of '/' in buffer is NULL
  if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
  {
    printf ("\nCan't get cnt file basename %s", buffer);

    if (trace_file != NULL)
      fprintf (trace_file, "\n%s : Can't get cnt file basename %s", PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file, "\n%s: Can't get cnt file basename %s", PRGNAME, buffer);

    return 1;
  } // endif: ptr_cnt_filename

  ptr_cnt_filename++;
  fprintf (list_file,"%s\n", ptr_cnt_filename); // Count File Name

  sprintf (buffer,
    "INTERNAL ID NUMBER                      %04d\n"
    "WMO ID NUMBER                           %05d\n"
    "TRANSMISSION ID NUMBER                  %05d\n"
    "PROFILE NUMBER                          %d\n",
    s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);

  fwrite (strupr(buffer),1,strlen(buffer),out_file); // Write buffer to file

  sprintf (buffer,
    "INSTRUMENT TYPE                         %s\n",s->float_type);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
    "PROFILE TERMINATION FLAG                %d\n"
    "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
    "WMO RECORDER TYPE (TABLE 4770)          %d\n"
    "ARGOS PROGRAM NUMBER                    %d\n",
    x->termination_byte,s->wmo_instrument,s->wmo_recorder,s->exp_no);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
    "PI                                      %s\n",s->PI);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  if (x->profile_no > 0)
  {
  sprintf (buffer,
    "DRIFT TEMPERATURE (COUNTS)              %-05d\n"
    "DRIFT PRESSURE (COUNTS)                 %-05d\n"
    "START SURFACE PRESSURE (COUNTS)         %-05d\n"
    "BATTERY (COUNTS)                        %-05d\n",
    x->bottom_temperature,x->bottom_pressure,x->surface_pressure,
    x->battery_voltage);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

// JZ if (x->profile_no > 0)
// JZ {
    sprintf (buffer,
      "NUMBER OF COLUMNS                       3\n"
      "1. COLUMN                               PRESSURE (COUNTS)\n"
      "2. COLUMN                               TEMPERATURE (COUNTS)\n"
      "3. COLUMN                               CHECKSUM FLAG\n"
      "==========================================================\n");

    fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
  else
  {
    sprintf (buffer,
      "==========================================================\n");
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
  i = 0;
  sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  //           "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  while (x->position_jd[i] > 0.0)
  {
    sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
      x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
      x->loc_class[i]);
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
    i++;
  }

  fprintf (out_file,"==========================================================\n");

  // Write depth, temperature, and salinity to disk 
  if (x->profile_no > 0)
  {
    for (i=0; i<115; i++)
    {
      if (x->press_temp[i][0] < 0)
        break;
   if ((j = data_flag1(i,x->status)) == NEVER_RECEIVED)
          sprintf (buffer," %5d  %5d  %1d\n",0,0,0);
 else
          sprintf (buffer," %5d  %5d  %1d\n",
            x->press_temp[i][0],x->press_temp[i][1],j);

        fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
  }

  if (out_file != NULL)
    fclose (out_file);   // Close output file

  return(0);
}

/******************************************************************************
  Finds the most likely frame for the T/P floats
  Returns index of most common frame, -1 for error.

  The outer loop reads each line of the passed buffer and finds the ones with
  a frame number equal to the passed frame number.

  For each line found with the same frame number, the inner loop is executed.

  The inner loop reads each line of the passed buffer, finds the ones with
  the same frame number as the one of the line above, and compares the two lines.
 
  This information is used to determine which buffer line with the given frame
  number is the most common.

09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
******************************************************************************/
int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_only)
{
  int i,j,k,l,matches = 0;
  int num_matches,most_likely = -1;

  if (n_scans == 0)
    return(-1);

  // Reads each line of buffer and finds those with frame number equal
  // to the passed frame number 
  for (i=0; i<n_scans; i++)
  {
    sscanf(&buffer[i][start_of_hex_data+2],"%2X",&k); // Read frame # from ith line
                                                      // of buffer into k 
 
    if ((buffer[i][good_bad_position] != '0') && good_only) // 0=good CRC, 1=bad CRC
      continue;  // Ignore if status byte=bad and good_only is true (skip current iteration)
    // Ignore if frame number (k) from buffer line is not equal to the frame
    // number passed (frame_num)
    if (k != frame_num)
      continue;

    num_matches = 0;

    // For each frame number of each line read with the outer loop, read all lines
    // of buffer to find those with the same frame number
    for (j=0; j<n_scans; j++)
    {
      sscanf(&buffer[j][start_of_hex_data+2],"%2X",&l); // Read frame # from jth line
                                                        // of buffer into l
      
      // Check if the frame number l from the jth line is different from the frame
      // number k of ith line 
      if (k != l)
        continue; // If the frame numbers are different, skip this line

      if ((buffer[j][good_bad_position] != '0') && good_only)
        continue;  // Ignore if status byte=bad and good_only is true

      // If frame numbers are equal compare jth line with ith line to see if they
      // are identical. If so, then increment num_matches
      if (strcmp(&buffer[i][start_of_hex_data],&buffer[j][start_of_hex_data]) == 0)
        num_matches++; // Hex data the same => increment match count
    }

    if (num_matches > matches) // Is this the most matches?
    {
      matches = num_matches; // Yes - save match count
      most_likely = i;  // Save record number (which line)
    }
// inserted by Claudia. Does it work???
    if ((num_matches==0) && (matches==0)) // Make sure to use messages only received once
    {
      most_likely=i;
    }

  }

  return(most_likely); // Return the frame # which was found the most often
}

/******************************************************************************
  Does the actual reading of the headers and data.

  If possible, would like to keep the most common frame out of all frames with
  a good CRC. However, if there are no frames with a good CRC, then keep the
  most common frame.

09334 1997 08 24 15 24 14 +00.000 +000.000 0 041 J 0 069516551 F506FB11C711B111
91115C11541153115211501145113010DF10B2108E103E0E

  Calls: most_likely_frame, dfr97
******************************************************************************/
int get_raw_data (struct output_record *y, char ** buffer, int n_scans)
{
  struct raw_data_record x;
  int i,j,k,n_frames = frames_per_message,most_common[frames_per_message];
  int id,iy,imo,ida,ih,im,is,g,n_l,jd_old=0,n_pos=0;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32];

  char nodata[] = "0000000000000000000000000000000000000000000000000000000000000000";

  //added new
  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
    y->status[i] = NEVER_RECEIVED;

    // Initialize array in structure
    for (j=0; j<30; j++)
      x.data.data_bytes[i][j] = 0;

    // Finds most common frame # (good CRC only)
    if ((j = most_likely_frame (buffer, n_scans, i+1, 1)) >= 0)
      most_common[i] = j;
    // Finds most common frame # (any CRC)
    else if ((most_likely_frame (buffer, n_scans, i+1, 0)) >= 0)
      most_common[i] = j;
    else 
      most_common[i] = -1;
  }

  if (most_common[0] < 0)
  {
   //added new
   frame1_problem_count++;

    if (trace_file != NULL)
      fprintf (trace_file," Can't decode message without frame 1\n");

    if (error_file != NULL)
      fprintf (error_file," Can't decode message without frame 1\n");

//   exit(9);	//Can't decode message without frame 1 - error return
   return(-1);	//Can't decode message without frame 1 - error return
  }

  for (j=1; j<max_positions; j++)
    x.jd[j] = -1.0;

  for (i=0; i<n_scans; i++)
  {
    // Read chars from buffer into C variables
    if (sscanf(buffer[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
        &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
      continue;

    // Read third and fourth characters from hex_data, convert from hex to decimal and
    // skip if > frames_per_message.
    frames=0;
    for (l=0; l<2 ; l++)
    {
       ch=*(hex_data+l+2);
       switch (ch)
       {
         case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
         case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
         case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
         case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
         case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
         case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
         default : frms[0]=ch; frms[1]='\0'; break;
       }
       frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
    }
    if (frames > frames_per_message)
    {
      continue;
    }

    // Read in position (lat/lon)
    x.latitude[n_pos] = lat;
    x.longitude[n_pos] = lon;
    x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is);
    sprintf(x.date_time[n_pos],
    "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
    x.loc_class[n_pos] = l_c;
    x.sat_name[n_pos] = sat;
    x.n_xmits[n_pos] = (int) floor((n_l-1)/8);

    // Check to see if a header line (i.e., data block is zero)
    if (strcmp(hex_data, nodata) == 0)
    {
      n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
      continue;
    }
    // If data block, then position is zero and data block is not zero.
    // Set lat to 99.9999, lon to 999.999 and read 1st time for profile.
    x.latitude[n_pos] = (float) 99.999;
    x.longitude[n_pos] = (float) 999.999;
    x.n_xmits[n_pos] = 0;
    n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

    // Read data block
    for (j=0; j<32; j++)
      sscanf((const char *) &hex_data[2*j],"%2x",&z[j]); // Read each hex pair into z

    if ((z[1] - min_frame_no < 0) ||
        (z[1] - min_frame_no > frames_per_message - 1 + min_frame_no))
          continue;

    if (i !=  (most_common[z[1] - min_frame_no])) 
      continue;

    if (z[1] == 1) // Check if 1st frame
    {
      if (z[4] <= 208)  // If z[4]=number of data bytes is <= 208, there are 7 frames
        n_frames = 7;

      if (z[4] <= 178)
        n_frames = 6;

      if (z[4] <= 148)
        n_frames = 5;

      if (z[4] <= 118)
        n_frames = 4;

      if (z[4] <= 88)
        n_frames = 3;

      if (z[4] <= 58)
        n_frames = 2;

      if (z[4] <= 28)   // includes profile length byte!!
        n_frames = 1;
    }

    y->status[z[1] - 1] = (g == 0) ? GOOD : BAD; // g: 0=good CRC, 1=bad CRC

    for (k=0; k<30; k++)
      x.data.data_bytes[z[1]-1][k] = z[k+2]; // remove CRC and frame number
  }

  x.jday = (int) floor(x.jd[0]);
  printf ("\nT/P Float: profile number %1d status = ",(int)x.data.d_fmt.profile_no);

  for (j=(frames_per_message-1); j>=0; j--)
    printf ("%1d",y->status[j]);

  printf (" (0=NOT RCVD,1=OK,4=BAD)\n");

  y->float_id = id;
  y->profile_no = x.data.d_fmt.profile_no;
  y->jd = x.jday;
  y->controller_sn = x.data.d_fmt.controller_sn;
  y->profile_length = x.data.d_fmt.profile_length;
  y->termination_byte = x.data.d_fmt.termination_byte;

  y->bottom_temperature = (256*x.data.d_fmt.bottom_temp[0] +
    x.data.d_fmt.bottom_temp[1]);

  y->surface_pressure = (x.data.d_fmt.surface_pressure);
  y->bottom_pressure = (x.data.d_fmt.bottom_pressure);
  y->battery_voltage = (x.data.d_fmt.battery_volts);

  for (j=0; j<max_positions; j++)
  {
    y->latitude[j] = x.latitude[j];
    y->longitude[j] = x.longitude[j];
    y->position_jd[j] = x.jd[j];
    y->n_xmits[j] = x.n_xmits[j];
    y->sat_name[j] = x.sat_name[j];
    y->loc_class[j] = x.loc_class[j];
    strncpy(y->date_time[j],x.date_time[j],24);
  }

  k = x.data.d_fmt.bottom_pressure;

  for (j = 0; j < ((x.data.d_fmt.profile_length) - 6)/2; j++)
  {
    if (j >= 115)
      break;

    y->press_temp[j][1] = (256*x.data.d_fmt.temperature[j][0] +
      x.data.d_fmt.temperature[j][1]);
    k = (--k <= 29) ? k : k-1;
    y->press_temp[j][0] = k;

    if (j < 114)
      y->press_temp[j+1][0] = -1;
  }

  return(y->profile_no);
}

/******************************************************************************
  Reads the data 

  Calls: ymd2julday, get_raw_data, write_to_disk 
******************************************************************************/ 
int get_type1_data (int np, struct type1_meta_struct *s,char *out_dir)
{
  struct output_record y;
  char **buff;
  int i,j, ret_val;
  int iy=0,imo,ida,ih,im,is;
  int n_prof=0,jd_old=0,jd;
  int write_to_disk_error;
  int corrected_prof_no;
  
  if ((buff = (void *) malloc(max_scans*(sizeof(void *)))) != NULL) // Allocate memory
  {
    for (i=0; i<max_scans; i++)
      if ((buff[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)
        iy = 1;
  }
  else
    iy = 1;

  if (iy == 1)
  {
    printf ("\n\nError allocating data buffer arrays\n");

    if (trace_file != NULL)
      fprintf (trace_file," GET_TYPE1_DATA: Error allocating data buffer arrays\n");

    if (error_file != NULL)
      fprintf (error_file," GET_TYPE1_DATA: Error allocating data buffer arrays\n");

    return(1);
  }

  if (trace_file != NULL)
    fprintf (trace_file," GET_TYPE1_DATA: Arrays successfully allocated\n");

  // Get the data
  j=0;

  // Reads a line from input_file into buff and check if NULL
  // (Data accumulates in buff until a new profile is read)
  while (fgets(buff[j],buff_size,input_file) != NULL)
  {
    // Read chars from buff, store them in variables and check if
    // exactly 7 were read
    if(sscanf(buff[j],"%d %d %d %d %d %d %d",
                      &i,&iy,&imo,&ida,&ih,&im,&is) != 7)
      continue; // Skip current iteration and continue with next iteration of while 

 if (dfr97 (iy,imo,ida,ih,im,is) < s->start_time)
  continue; //Skip scans before start time
    jd = ymd2julday( iy, imo, ida ) + 1; // Days from 1/1/1997 (ignoring hrs, mins, secs)


    // Check if new profile (since data collected every 10 days) and
    // not 1st profile
    if ((abs(jd - jd_old) > 2) && (j > 0))
    {
      // Check if you want to include all profiles or just the last one. If so,
      // then read/write profile each time. Otherwise, do not read/write profile.
      // The last profile will be written when the loop exits.
      if (np > 0) // np=1 => all profiles, np=0 => last profile only
      {
        n_prof = get_raw_data (&y,buff,j);
        //added profile_no correction
        if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
        {
	 //changed here
         corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
         if(corrected_prof_no>0) y.profile_no=corrected_prof_no;

         fprintf (trace_file," GET_TYPE1_DATA: transmitted profile no. = %d",y.profile_no);
         fprintf (trace_file," computed = %d - using computed value\n",
         y.profile_no);
        }
	
        if (y.profile_no >= 0 && n_prof!=-1)
          write_to_disk (out_dir,s,&y);
      }

      sprintf (buff[0],"%s",buff[j]); // write  buff[j] to buff[0], since new profile
      j = 0; // Reset to 0 in preparation for next profile (>2 days later)
    }

    jd_old = jd; // set to days currently read  

    // Increment j and check if maximum scans exceeded
    if (++j >= max_scans) // buff[0],...,buff[max_scans-1] 
    {
      if (trace_file != NULL)
        fprintf (trace_file," Maximum scans for profile exceeded\n");

      if (error_file != NULL)
        fprintf (error_file," Maximum scans for profile exceeded\n");

      exit (1);
    }
  } // while
 if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
 { // Read/write last profile 
  n_prof = get_raw_data (&y,buff,j);
  //added profile_no correction
  if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
  {
         //changed here
         corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
         if(corrected_prof_no>0) y.profile_no=corrected_prof_no;
	 
         fprintf (trace_file," GET_TYPE1_DATA: transmitted profile no. = %d",y.profile_no);
         fprintf (trace_file," computed = %d - using computed value\n",
         y.profile_no);
  }
  
 if (y.profile_no >=0 && n_prof!=-1)
  {
    if (trace_file != NULL)
      printf ("GET_type1_DATA completed successfully\n");
    fprintf (trace_file," GET_type1_DATA completed successfully\n");
    write_to_disk_error=write_to_disk (out_dir,s,&y);
    ret_val = write_to_disk_error;
  }else
  {
    printf ("GET_type1_DATA n_prof < 0\n");
    if (trace_file != NULL)
      fprintf (trace_file," GET_type1_DATA n_prof < 0 OR Frame 1 Problem: %d \n",n_prof);
    ret_val = 1;
  }
  }else
  {
    write_to_disk_error =1; // no data to store -> something may be wrong
  } // if (j > 0)
  free (buff);

  if (trace_file != NULL)
    fprintf (trace_file," GET_TYPE1_DATA completed successfully\n");

  return(ret_val);
}

/******************************************************************************
  Assumes str_name = *.hex, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
 char *ptr; 

 // Check if pointer to 1st occurrence of .hex in str_name is NULL
  if ((ptr = strstr(str_name,".hex"))!= NULL)
    sprintf (ptr,".meta");
  else
    strcat(str_name,".meta");

  return 0;
}

/******************************************************************************
  Reads meta_file

  Calls: strupr  
******************************************************************************/
int read_type1_meta_file (char *meta_name, struct type1_meta_struct *meta_struct)
{
  int i,j,syear,smonth,sday,shour,smin,x;
  char *ptr;
  float x1;//added  
  char *cal_keys[] = {"R1=","R2=","R3=","R4=","T1=","T2=","T3=",
                      "PSLOPE=","POFF=","VSLOPE=","VOFF=","\0"};
 
  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_cycle_time = (float) 0.0;
  meta_struct->start_time = (float) 0.0;
 //added
  meta_struct->start_of_cycles = (float) 0.0; 

  for (i=0; i<(sizeof(struct aoml1_cals)/sizeof(double)); i++)
    meta_struct->cals.c[i] = 0.0;

  printf("\nOpening meta file %s...",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("Error");

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  printf ("OK\n");

  if (trace_file != NULL)
    fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first 
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      // Read chars from buffer and store them in AOML_number inside
      // structure. If no chars where read, set AOML_number to 0.
      //
      // Note: * means the field does not need to be included in list of
      // variable names to write to
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) == 0)
        meta_struct->AOML_number = 0;

      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION REPETITION RATE") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
        meta_struct->ARGOS_rep_rate = 0;
      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) == 0)
        x = 0;

      meta_struct->start_cycle_time += (float) (x/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"DOWN TIME") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
        x = 0;

      meta_struct->cycle_time += x;
      //added
      meta_struct->start_of_cycles += x;   
      
      continue;
    }
  //added
    if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
      x1 = 0;
      meta_struct->start_of_cycles += (float) (x1/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"UP TIME") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
        x = 0;

      meta_struct->cycle_time += (float) (x/24.0);
      meta_struct->start_cycle_time -= (float) (x/24.0);
      continue;
    }

  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
    if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
      meta_struct->start_time += (float) dfr97(syear,smonth,sday,shour,smin,0);
    else
      meta_struct->start_time = 0;
    meta_struct->start_cycle_time = (float) (meta_struct->start_time + .25);
    //added 
    meta_struct->start_of_cycles += meta_struct->start_time;    

    continue;
  }

  if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
  {
    if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
      meta_struct->wmo_instrument = 0;

    continue;
  }

  if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
  {
    if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
      meta_struct->wmo_recorder = 0;

    continue;
  }

  if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
  {
    if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
      meta_struct->exp_no = 0;

    continue;
  }

  if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
    if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
    {
      sprintf (meta_struct->float_type,"\0");
      continue;
    }

    if (strstr(strupr(buffer),"PI     ") != NULL)
    {
      if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
      {
        sprintf (meta_struct->PI,"\0");
        continue;
      }

      if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
        sprintf (meta_struct->PI,"%s",ptr);

      if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
        sprintf (ptr,"\0");

      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      // Get pointer to 1st occurence of cal_keys[i] in buffer
      if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof(ptr+j); // Read value after = sign ino c[i]

      i++;
    }
  }

  return 0;
}

/******************************************************************************
  'main' function
  
  Calls: get_meta_file_name, read_type1_meta_file, get_type1_data 
******************************************************************************/
int main (int argc, char *argv[])
{
  char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt, 
           *ptr_path_log, *ptr_path_list,
           pathname_input_filename[161]="", 
           pathname_trace[161]="", pathname_error[161]="";

  char     input_filename[80], output_directory[256], msg[11]="", title[257]="";
  enum     {LAST=0,ALL} get_which_profile;

  struct   type1_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
  unsigned long t;
  int ret_val = 0;

// Program Name
// ------------
 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature and pressure counts for ");
 strcat(title, "PALACE_T1 Floats");
 printf("%s\n", title);

  if (argc < 7 )
  {
    printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
           PRGNAME);
    printf("\n");
    printf("hex_infile:    hexadecimal input file name\n");
    printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
    printf("cnt_outdir:    output directory for generated count data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

  printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
  sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0=get last profile or 1=get all profiles
// -------------------------------------------------------------------------
  if (atoi(argv[2]) == 1)
    get_which_profile = ALL;
  else
    get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  printf ("%s: Output Directory - %s/", PRGNAME, argv[3]);
  sprintf (output_directory,"%s/",argv[3]);

// TRACE File
// ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  printf ("%s: Opening trace file %s\n", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[4]);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> Trace file opened\n", PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// ERROR File 
// ----------
  printf ("%s: Opening error file %s\n", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[5]);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);

    t = time(NULL); 
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  printf("%s: OK -> Error file opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Opening list file %s\n", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[6]);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val =1;
    return ret_val;
  }

  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);

// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): HEXFILEDIR <-- no such environment variable\n", PRGNAME);
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files 
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);
  
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }
   
  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);   

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {                                           // Fail to open Hex file
    printf ("\nERROR (%s): Opening input file %s", PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Input file %s not opened\n", 
               PRGNAME, pathname_input_filename);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): input file %s not opened\n", 
               PRGNAME, pathname_input_filename);
    ret_val = 1;
  }
  else                                        // Success: Hex file opened 
  {
    printf ("\n%s: Input file %s successfully opened",
            PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: Input file %s successfully opened\n", 
               PRGNAME, pathname_input_filename);

// META File
// ---------
    if (get_meta_file_name (input_filename) != 0)
    {                                              // Failure: couldn't get meta filename 
      printf ("\nERROR (%s): Finding meta file name", PRGNAME);
      if (error_file != NULL)
        fprintf (error_file, 
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file, 
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      ret_val = 1;
    }
    else 
    {                                              // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);

        if (trace_file != NULL)
          fprintf (trace_file,
                   "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                   PRGNAME);
        if (error_file != NULL)
          fprintf (error_file,
                   "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                   PRGNAME);

        if (trace_file != NULL)                  // Close Log files 
          fclose (trace_file);

        if (error_file != NULL)
          fclose (error_file);
  
        t = time(NULL);
        printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
        ret_val = 1;
        return(ret_val);
      } // endif: ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_type1_meta_file (pathname_input_filename, &s) != 0)
      {                                         // Failure: couldn't read meta_file
        printf ("\nERROR (%s): Reading meta file", PRGNAME);
        if (trace_file != NULL)
          fprintf (trace_file, "ERROR (%s): Reading meta file %s\n", 
                   PRGNAME, pathname_input_filename);
        if (error_file != NULL)
          fprintf (error_file,"ERROR (%s): Reading meta file %s\n", 
                   PRGNAME, pathname_input_filename);
        ret_val = 1;
      }
      else                                      // Success: read meta_file
      {
        if (trace_file != NULL)
          fprintf (trace_file,"%s: Meta file %s successfully read\n", 
                   PRGNAME, pathname_input_filename);
   printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename);

// Make profile(s) selection
// -------------------------
        if (get_type1_data(get_which_profile,&s,output_directory) != 0)
            ret_val = 1;
      } // endif: read_type1_meta_file
    } // endif: get_meta_file_name 
  } // endif: input_file (open hex file)

//added new
  printf("HEX filename=%s\n",argv[1]);
  printf("frame1 missing problem counts=%d\n",frame1_problem_count); 
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
    fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close data files
// ----------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
   fclose (list_file);

  printf("%s -> RET_VAL from get_type3_data: %d\n", PRGNAME, ret_val);
  return(ret_val);
}
