/******************************************************************************
   Revision Date: 11-Jun-12 (CS  - renamed round to round_argo to avoid conflict
                                   in newer versions of c)
                  14-Apr-11 (ER: - Unnecessary abs function was detecting false 
                                   ghosts
                  08-FEB-11 (ER  - fix 256 profile taking 255 profile data.)
                  12-DEC-06 (XXD - new profile number correction algorithm)
                  21-JUN-04 (XXD&CS - do not allow negative profile numbers
                            (bug fix); do not try to write a data file if
                            no lines are available (j=n_scans=0), in
                            get_type2_data)
                  07-OCT-03 (XX-correct profile_no if it is not
                            continuous,follow ts3 practice)
                  15-AUG-03 (XX&CS-add exit status 2 when frame1 missing problem happens 
                             in some but not all profile cycles)
                  14-JUL-03 (XX&CS-change frames_per_message to 256)
                  20-NOV-01 (CS - write abs(s->WMO_id) not s->WMO_id)
                  31-May-01 (JZ) - Changed 'BATTERY CPU' to 'BATTERY'
                  18-DEC-00 (CS - Make sure to use messages received only once - Test!)
                  20-Nov-00 (DB - change to uppercase in HEADER &
                             RS - fix return from get_type2_data) 
                  14-Nov-00 (CS - Bitterman updates: surface p/T pairs now correct)
                  09-Nov-00 (RS - Bitterman updates( diff1109.lis)
                  31-Aug-00 (JZ) 
                  10-Aug-00 (rename ARGOLIST_DIR)
                  27-Jul-00 (RS - eliminate redundant stm & add program name
                                  to stdout and trace/error output)
                  05-Jul-00 (RS - update get_raw_data)
                  29-Jun-00 (RS - replaced juldat with ymd2julday)
                  03-Feb-00 (RS - fixed union and added "strupr" function)
				  07-Sep-00 (DB - moved surface temperatures and pressures to header)
				  16-Sep-00 (DB - reversed surface p & t values if less than 3 values found)
				  01-Nov-00 (DB - removed redundant STRUPR when reading meta file)
				  01-Nov-00 (DB - modified GET_TYPE2_DATA to ignore scans before start time)

   hex2cnt_t2.c 
   ============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   PALACE floats APEX_T2 manufactured in 1999 by Webb Research Cooperation
*******************************************************************************/

#include <stdio.h>
/*#include <conio.h>*/
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans        2000   //Maximum scans for each profile
#define buff_size         256   //Maximum length of each scan read from *.str
#define max_positions     100   //Maximum number of positions per profile
#define min_frame_no        1   //Minimun frame number
#define frames_per_message 256   //Number of frames per message,it is the maximum allowed number.
#define good_bad_position  43   //Location of status byte in *.str records
#define start_of_hex_data  63   //Start position of data in *.str records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0

int frame1_problem_count=0,total_cycles_count=0;//added new.
int actual_frames_per_message;

char *strupr(char* ptr_to_string_to_be_upper_cased); 

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char   PRGNAME[] ="hex2cnt_t2";

double cal[13];
int    aoml_id;
char   buffer[512];
enum   flg {NO,YES} cal_flg;
//enum   data_flag {NONE,BAD,GOOD};
enum   {F,T} cd_input = F;

struct aoml3_cals { double TSLOPE;
                    double TOFF;
                    double PSLOPE;
                    double POFF;
                    double VSLOPE;
                    double VOFF;
                    double VACSLOPE;
                    double VACOFF;
                  };

union u_cals { struct aoml3_cals s;
               double c[sizeof(struct aoml3_cals)/sizeof(double)];
             };

struct type2_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
						   int exp_no;
						   unsigned char float_type[40];
                           unsigned char PI[60];
                           float ARGOS_rep_rate;
                           float cycle_time;
                           float start_cycle_time;
                           float start_time;
			   float start_of_cycles;//added Start_time+delay_of_start+down_time			   			   			   
                           union u_cals cals;
                         };

/*      Raw data structure used in decoding Temperature/Pressure (T/P)
 float data.  Part of RAW_DATA_RECORD described below */

struct test_fmt { unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char msg_type_flag;
                  unsigned char current_pressure[2];
                  unsigned char battery_volts;
                  unsigned char internal_vacuum;
                  unsigned char piston_position;
                  unsigned char float_status;
                  unsigned char hours_since_start;
                  unsigned char minutes_since_start;
                  unsigned char seconds_since_start;
                  unsigned char ARGOS_rep_counts;
                  unsigned char up_time_intervals;
                  unsigned char down_time_intervals[2];
                  unsigned char trip_interval_time;
                  unsigned char target_pressure[2];
                  unsigned char target_piston_position;
                  unsigned char depth_correction;
                  unsigned char ballast_piston_position;
                  unsigned char full_extension;
                  unsigned char ok_vacuum;
                  unsigned char init_piston_position;
                  unsigned char software_rev_month;
                  unsigned char software_rev_day;
                  unsigned char software_rev_year;
                };

struct data_fmt { unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char profile_length;
                  unsigned char termination_flag;
                  unsigned char piston_position;
                  unsigned char bottom_temperature[2];
                  unsigned char bottom_pressure[2];
                  unsigned char battery_volts;
                  unsigned char surface_pressure[2];
                  unsigned char internal_vacuum;
                  unsigned char bottom_piston_position;
                  unsigned char temperature[15*frames_per_message][2];
                };

/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { unsigned char data_bytes[frames_per_message][30];
                  unsigned char just_bytes[frames_per_message*30];
                  struct data_fmt d_fmt;
                  struct test_fmt t_fmt;
                };

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { int jday;
                         double jd[max_positions];
                         float latitude[max_positions];
                         float longitude[max_positions];
                         char date_time[max_positions][32];
                         int n_xmits[max_positions];
                         char sat_name[max_positions];
                         char loc_class[max_positions];
                         union data_size data;
                       };

/* Final structure containig T/P data and positions */

struct output_record { int    float_id;
                       int    profile_no;
                       int    jd;
                       int    status[frames_per_message];
                       int    msg_blk_no;
                       time_t first_blk_time;
                       int    controller_sn;
                       int    profile_length;
                       int    termination_flag;
                       int    piston_position;
                       int    bottom_temperature;
                       int    bottom_pressure;
                       int    battery_volts;
                       int    surface_pressure;
                       int    internal_vacuum;
                       int    bottom_piston_position;
                       int    msg_type_flag;
                       int    current_pressure;
                       int    battery_voltage;
                       int    float_status;
                       int    hours_since_start;
                       int    minutes_since_start;
                       int    seconds_since_start;
                       int    ARGOS_rep_counts;
                       int    up_time_intervals;
                       int    down_time_intervals;
                       int    trip_interval_time;
                       int    target_pressure;
                       int    target_piston_position;
                       int    depth_correction;
                       int    ballast_piston_position;
                       int    full_extension;
                       int    ok_vacuum;
                       int    init_piston_position;
                       int    software_rev_month;
                       int    software_rev_day;
                       int    software_rev_year;
                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       int    press_temp[15*frames_per_message][3]; //[0]=p,[1]=t,[2]=bin
					   int    surface_tp[4][2];
                     };

long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);


char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = passed_jd - jd_970101;
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}

int data_flag2(int bin_no, int *status_word)
{
 int i;
 int ranges[frames_per_message][2] = {0,7,8,22,23,37,38,52,53,67,68,82,83,97,98,112};
// int ranges[frames_per_message][2] =   {0,6,7,21,22,36,37,51,52,66,67,81,82,96,97,111};

 for (i=0; i<frames_per_message; i++)
 {
  if ((bin_no >= ranges[i][0]) && (bin_no <= ranges[i][1]))
	  return status_word[i];
 }
 return BAD;
}
// round_argo a floating number to its nearest integer
// this function needed to be renamed from round due to a conflict
// in newer versions of c
int round_argo(float x) {
      if (x >= 0) return (int) (x+0.5);
      return (int) (x-0.5);
}

//compute correct profile number
int correct_profilenumber(float jd_firstline_prev_old,float jd_firstline_current_old,int profSeqCounter,int profile_no_prev,float start_of_cycles,float
cycle_time,int n_prof,int has_prof0){
  int correct_profile_no;
  //int adjustment=round_argo((jd_firstline_current_old-jd_firstline_prev_old)/cycle_time)-1;
  //if(has_prof0>0) adjustment--;
  //correct_profile_no=profSeqCounter+adjustment;
  
  int adjustment=round_argo((jd_firstline_current_old-jd_firstline_prev_old)/cycle_time);
  correct_profile_no=profile_no_prev+adjustment;
  
  printf("profSeqCounter=%d,profile_no_prev=%d,adjustment=%d,cycle_time=%f,jd_current_old=%f,jd_prev_old=%f,jd_diff=%f\n",profSeqCounter,profile_no_prev,adjustment,cycle_time,jd_firstline_current_old,jd_firstline_prev_old,jd_firstline_current_old-jd_firstline_prev_old);
  
  if(n_prof!=correct_profile_no){
    printf("WARNING, n_prof NOT EQUAL TO correct_profile_no,n_prof=%d,correct_profile_no=%d,profSeqCounter=%d,profile_no_prev=%d,adjustment=%d\n",n_prof,correct_profile_no,profSeqCounter,profile_no_prev,adjustment);
  }
  return correct_profile_no;
}

int write_to_disk (char *dir_name, struct type2_meta_struct *s, struct output_record *x)
{

/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no 
 temperature or pressure data. */

 int  i,j,id;
 char *ptr_cnt_filename;
 struct tm *time_str;

 id = x->float_id;
 if ((x->status[0]) != GOOD)    //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file,"First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file,"First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 printf ("Opening output file %s...",buffer);
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  printf ("error");
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened",buffer);
 printf ("OK\n", PRGNAME);

// Write Output Profile Count File Name to List File
// -------------------------------------------------  
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  printf ("\nCan't get cnt file basename %s", buffer);
  if (trace_file != NULL)
   fprintf (trace_file,
            "\n%s : Can't get cnt file basename %s", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "\n%s: Can't get cnt file basename %s", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 
 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %d\n"
     "TRANSMISSION ID NUMBER                  %d\n"
     "PROFILE NUMBER                          %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n",s->float_type);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PROFILE TERMINATION FLAG                %d\n"
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
     x->termination_flag,s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 x->first_blk_time -= (long) (x->msg_blk_no * s->ARGOS_rep_rate);

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if (x->profile_no > 0)
 {
  sprintf (buffer,
     "DRIFT TEMPERATURE (COUNTS)              %d\n"
     "DRIFT PRESSURE (COUNTS)                 %d\n"
     "START SURFACE PRESSURE (COUNTS)         %d\n",
      x->bottom_temperature,x->bottom_pressure,x->surface_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  if (x->surface_tp[0][0] > -999)
  {
   sprintf (buffer,"END SURFACE PRESSURE (COUNTS)           %-5d",
    x->surface_tp[0][0]);
   fwrite (strupr(buffer),1,strlen(buffer),out_file);
   for (i=1; i<4; i++)
    if (x->surface_tp[i][0] > -999)
     fprintf (out_file," %-5d",x->surface_tp[i][0]);
	else
     break;
   fprintf (out_file,"\n");
   sprintf (buffer,"END SURFACE TEMPERATURE (COUNTS)        %-5d",
    x->surface_tp[0][1]);
   fwrite (strupr(buffer),1,strlen(buffer),out_file);
   for (j=1; j<i; j++)
    if (x->surface_tp[j][1] > -999)
     fprintf (out_file," %-5d",x->surface_tp[j][1]);
   fprintf (out_file,"\n");
  }

  sprintf (buffer,
     "BATTERY (COUNTS)                        %-5d\n",
      x->battery_voltage);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,"piston position                         %-3d\n"
     "INTERNAL VACUUM                         %-3d\n"
     "BOTTOM PISTON POSITION                  %-3d\n",
     x->piston_position,x->internal_vacuum,
     x->bottom_piston_position);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
 sprintf (buffer,
     "NUMBER OF COLUMNS                       3\n"
     "1. COLUMN                               PRESSURE (COUNTS)\n"
     "2. COLUMN                               TEMPERATURE (COUNTS)\n"
     "3. COLUMN                               CHECKSUM FLAG\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
 else
 {
  sprintf (buffer,
     "CURRENT PRESSURE (COUNTS)               %-5d\n"
     "BATTERY (COUNTS)                        %-3d\n"
     "PISTON POSITION                         %-3d\n",
     x->current_pressure,x->battery_voltage,
     x->piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
	    "FLOAT STATUS                            %-3d\n"
     "TIME SINCE START (HR/MIN/SEC)           %02d:%02d:%02d\n"
     "TRANSMISSION REPETITION RATE (SEC)      %-3d\n",
     x->float_status,x->hours_since_start,x->minutes_since_start,
     x->seconds_since_start,x->ARGOS_rep_counts);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
	    "UP TIME INTERVALS (HRS)                 %-3d\n"
     "DOWN TIME INTERVALS (HRS)               %-5d\n"
     "TRIP INTERVAL TIME                      %-3d\n",
     x->up_time_intervals,x->down_time_intervals,
     x->trip_interval_time);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "TARGET PRESSURE (CBAR)                  %-3d\n"
     "TARGET PISTON POSITION                  %-3d\n"
     "DEPTH CORRECTION FACTOR                 %-3d\n",
     x->target_pressure,x->target_piston_position,
     x->depth_correction);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "BALLAST PISTON POSITION                 %-3d\n"
     "FULL EXTENSION                          %-3d\n"
     "OK VACUUM COUNT                         %-3d\n",
     x->ballast_piston_position,x->full_extension,
     x->ok_vacuum);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
	    "INITIAL PISTON POSITION                 %-3d\n"
     "SOFTWARE REVISION DATE (MON/DAY/YR)     %02d/%02d/%02d\n",
     x->init_piston_position,x->software_rev_month,
     x->software_rev_day,x->software_rev_year);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
    "====================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
//x->status,);

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i++;
 }
 fprintf (out_file,"==========================================================\n");

/* Write depth, temperature, and salinity to disk */
   
 if (x->profile_no > 0)
 {
  i = 0;
  while (x->press_temp[i][0] > -999)
  {
   if ((j = data_flag2(x->press_temp[i][2],x->status)) == NEVER_RECEIVED)
    sprintf (buffer," %-5d  %-5d  %d\n",0,0,0);
   else
    sprintf (buffer," %-5d  %-5d  %d\n",
	                      x->press_temp[i][0],x->press_temp[i][1],j);
   fwrite (buffer,1,strlen(buffer),out_file);
   if (++i > 15*frames_per_message)
	   break;
  }
 }

   if (out_file != NULL)
    fclose (out_file);   /* Close output file */

  return 0;
}

int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad)
{
 int i,j,k,l,matches = 0;
 int num_matches,most_likely = -1;
 char tmp_buff[2][256];

//  Finds the most likely scan for the APEX floats
//  Returns number of most common scan
//  Returns -1 if error
//  good_bad=1 -> use checksum; good_bad=0->ignore checksum
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if ((tmp_buff[0][good_bad_position] != '0') && good_bad)
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
  if (k != frame_num)
   continue;
  if (k == 1)
  {
   tmp_buff[0][4] = '0';
   tmp_buff[0][5] = '0'; //set message blk number = 0 because it's not constant
  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
   if (l == 1)
   {
    tmp_buff[1][4] = '0';
    tmp_buff[1][5] = '0'; //set message blk number = 0 because it's not constant
   }
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches) //Is this the most matches?
  {
   matches = num_matches; //Yes - save match count
   most_likely = i;       //Save record number
  }
// inserted by Claudia. Does it work???
   if (num_matches==0 & matches==0) // Make sure to use messages only received once
   {
     most_likely=i;
   }

 }
 return most_likely;
}

int get_raw_data (struct output_record *y, char ** buffer, int n_scans, int m_type, int profile_no_prev)
{

  struct raw_data_record x;
  struct tm *time_str;
  int i,j,k,most_common[frames_per_message],n_frames=frames_per_message;
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0,n_surfs=4,n_meas;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32];
        int p_table[] = {12000,11500,11000,10500,10000,9800,9600,9400,9200,9000,
                        8800,8600,8400,8200,8000,7800,7600,7400,7200,7000,
                        6800,6600,6400,6200,6000,5800,5600,5400,5200,5000,
                        4800,4600,4400,4200,4000,3800,3600,3400,3200,3000,
                        2950,2900,2850,2800,2750,2700,2650,2600,2550,2500,
                        2450,2400,2350,2300,2250,2200,2150,2100,2050,2000,
                        1950,1900,1850,1800,1750,1700,1650,1600,1550,1500,
                        1450,1400,1350,1300,1250,1200,1150,1100,1050,1000,
                        950,900,850,800,750,700,650,600,550,500,450,400,350,
                        300,250,200,150,100,40,0,0,0,0,0};
//09334 1997 08 24 15 24 14 +00.000 +000.000 0 041 J 0 069516551 F506FB11C711B11191115C11541153115211501145113010DF10B2108E103E0E

  //added new
  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
   y->status[i] = NEVER_RECEIVED;
   for (j=0; j<30; j++)
    x.data.data_bytes[i][j] = 0;
   if ((j = most_likely_frame (buffer, n_scans, i+1, 1)) >= 0) //find most likely - good CRC only
    most_common[i] = j;
   else if ((most_likely_frame (buffer, n_scans, i+1, 0)) >= 0) //find most likely - any CRC
    most_common[i] = j;
   else 
    most_common[i] = -1;
  }

  if (most_common[0] < 0)
  {
   //added new
   frame1_problem_count++;

   if (trace_file != NULL)
    fprintf (trace_file," Can't decode message without frame 1\n");
   if (error_file != NULL)
    fprintf (error_file," Can't decode message without frame 1\n");
//   exit(9);	//Can't decode message without frame 1 - error return
   return(-1);	//Can't decode message without frame 1 - error return
  }

  for (j=1; j<max_positions; j++)
   x.jd[j] = -1.0;

  for (i=0; i<n_scans; i++)
  {
   if (sscanf(buffer[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
    continue;

    // Read third and fourth characters from hex_data, convert from hex to decimal and
    // skip if > frames_per_message.
    frames=0;
    for (l=0; l<2 ; l++)
    {
       ch=*(hex_data+l+2);
       switch (ch)
       {
         case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
         case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
         case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
         case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
         case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
         case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
         default : frms[0]=ch; frms[1]='\0'; break;
       }
       frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
    }
    if (frames > frames_per_message)
    {
      continue;
    }

   x.latitude[n_pos] = lat;
   x.longitude[n_pos] = lon;
   x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

   sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
   x.loc_class[n_pos] = l_c;
   x.sat_name[n_pos] = sat;
   x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
   if ((lat != (float) 0.0) || (lon != (float) 0.0))
   {
    n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
    continue;
   }
   x.latitude[n_pos] = (float) 99.999;
   x.longitude[n_pos] = (float) 999.999;
   x.n_xmits[n_pos] = 0;
   n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

   for (j=0; j<32; j++)
    sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

  if (z[1]>frames_per_message)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"z[1] exceeds frames_per_message.\n");
    exit (1);
  }


   if ( (z[1] - min_frame_no < 0) ||
        (z[1] - min_frame_no > frames_per_message - 1 + min_frame_no))
    continue;

   if (i != most_common[z[1] - min_frame_no] )
    continue;

// CS - not used
   if(z[1] == 1)
   {
    n_meas = z[6];
//    if (z[6] < 98)        //z[6]=number of data bytes
//     n_frames = 7;
//    if (z[6] < 83)
//     n_frames = 6;
//    if (z[6] < 68)
//     n_frames = 5;
//    if (z[6] < 53)
//     n_frames = 4;
//    if (z[6] < 38)
//     n_frames = 3;
//    if (z[6] < 23)
//     n_frames = 2;
//    if (z[6] < 8)         //includes profile length byte!!
//     n_frames = 1;
   }
   y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
   if (z[1] == 1)
   {
    y->msg_blk_no = z[2]-1; //count starts at 1
    y->first_blk_time = ((ymd2julday(iy,imo,ida)*24+ih)*60+im)*60+is;
   }
   for (k=0; k<30; k++)
    x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
  }

  x.jday = (int) floor(x.jd[0]);

  printf ("\nT/P Float: profile number %d status = ",(int)x.data.d_fmt.profile_no);
  for (j=(frames_per_message-1); j>=0; j--)
	 printf ("%1d",y->status[j]);
  printf (" (0=NOT RCVD,1=OK,4=BAD)\n");

  y->float_id = id;
  y->jd = x.jday;
//  if (x.data.t_fmt.profile_no > 0)
  if(m_type > 0)
  {
   y->controller_sn = 256*x.data.d_fmt.controller_sn[0] +
    x.data.d_fmt.controller_sn[1];

   y->profile_no = x.data.d_fmt.profile_no;
   y->profile_length = x.data.d_fmt.profile_length;
   y->termination_flag = x.data.d_fmt.termination_flag;
   y->piston_position = x.data.d_fmt.piston_position;
   y->bottom_temperature = (256*x.data.d_fmt.bottom_temperature[0] + 
    x.data.d_fmt.bottom_temperature[1]);
   y->bottom_pressure = (256*x.data.d_fmt.bottom_pressure[0] + 
    x.data.d_fmt.bottom_pressure[1]);
   y->battery_voltage = x.data.d_fmt.battery_volts;
   y->surface_pressure = (256*x.data.d_fmt.surface_pressure[0] + 
    x.data.d_fmt.surface_pressure[1]);
   y->internal_vacuum = x.data.d_fmt.internal_vacuum;
   y->bottom_piston_position = x.data.d_fmt.bottom_piston_position;
  }
//  else if (x.data.t_fmt.profile_no == 0)
  else
  {
   y->termination_flag = x.data.d_fmt.termination_flag;
   y->controller_sn = 256*x.data.t_fmt.controller_sn[0] +
    x.data.t_fmt.controller_sn[1];
   y->profile_no = x.data.t_fmt.profile_no;
   y->msg_type_flag = x.data.t_fmt.msg_type_flag;
   y->current_pressure = 256*x.data.t_fmt.current_pressure[0] + x.data.t_fmt.current_pressure[1];
   y->battery_voltage = x.data.t_fmt.battery_volts;
   y->internal_vacuum = x.data.t_fmt.internal_vacuum;
   y->piston_position = x.data.t_fmt.piston_position;
   y->float_status = x.data.t_fmt.float_status;
   y->hours_since_start = (x.data.t_fmt.hours_since_start/16)*10 + x.data.t_fmt.hours_since_start%16;
   y->minutes_since_start = (x.data.t_fmt.minutes_since_start/16)*10 + x.data.t_fmt.minutes_since_start%16;
   y->seconds_since_start = (x.data.t_fmt.seconds_since_start/16)*10 + x.data.t_fmt.seconds_since_start%16;
   y->ARGOS_rep_counts = x.data.t_fmt.ARGOS_rep_counts;
   y->up_time_intervals = x.data.t_fmt.up_time_intervals;
   y->down_time_intervals = 256*x.data.t_fmt.down_time_intervals[0] + x.data.t_fmt.down_time_intervals[1];
   y->trip_interval_time = x.data.t_fmt.trip_interval_time;
   y->target_pressure = 256*x.data.t_fmt.target_pressure[0] + x.data.t_fmt.target_pressure[1];
   y->target_piston_position = x.data.t_fmt.target_piston_position;
   y->depth_correction = x.data.t_fmt.depth_correction;
   y->ballast_piston_position = x.data.t_fmt.ballast_piston_position;
   y->full_extension = x.data.t_fmt.full_extension;
   y->ok_vacuum = x.data.t_fmt.ok_vacuum;
   y->init_piston_position = x.data.t_fmt.init_piston_position;
   y->software_rev_month = (x.data.t_fmt.software_rev_month/16)*10 + x.data.t_fmt.software_rev_month%16;
   y->software_rev_day = (x.data.t_fmt.software_rev_day/16)*10 + x.data.t_fmt.software_rev_day%16;
   y->software_rev_year = (x.data.t_fmt.software_rev_year/16)*10 + x.data.t_fmt.software_rev_year%16;
  }
  for (j=0; j<max_positions; j++)
  {
   y->latitude[j] = x.latitude[j];
   y->longitude[j] = x.longitude[j];
   y->position_jd[j] = x.jd[j];
   y->n_xmits[j] = x.n_xmits[j];
   y->sat_name[j] = x.sat_name[j];
   y->loc_class[j] = x.loc_class[j];
   strncpy(y->date_time[j],x.date_time[j],24);
  }

  actual_frames_per_message=ceil((( ((float)y->profile_length-8)*2)/30))+1;
  if (actual_frames_per_message > frames_per_message)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }

  actual_frames_per_message=ceil((( ((float)y->profile_length-8)*2)/30))+1;

  if ( (y->profile_no>0) || (profile_no_prev>0) )
  {
   y->press_temp[0][0] = y->bottom_pressure;
   y->press_temp[0][1] = y->bottom_temperature;
   y->press_temp[0][2] = 0;
   j = 0;
   k = 0;
   while (k < (n_meas -4))  //Three surface and one bottom value
   {
    if (p_table[j] < y->bottom_pressure)
    {
     y->press_temp[k+1][0] = p_table[j];
     y->press_temp[k+1][1] = (256*x.data.d_fmt.temperature[k][0] +
      x.data.d_fmt.temperature[k][1]);
     y->press_temp[k+1][2] = k+1;
	 k++;
	 if ((k+1) >= 15*frames_per_message)
	 {
      if (trace_file != NULL)
       fprintf (trace_file," Number of bins (%2d) exceeds allowable array size\n",n_meas);
      if (error_file != NULL)
       fprintf (error_file," Number of bins (%2d) exceeds allowable array size\n",n_meas);
      return(-1);
	 }
     y->press_temp[k+1][0] = -999;
    }
	j++;
   }
   j = k;
   for (n_surfs=0; n_surfs<3; n_surfs++)
   {
    if (k >= y->profile_length)
    {
        if (trace_file != NULL)
            fprintf (trace_file," GET_RAW_DATA: profile length exceeded (%d)\n",
                y->profile_length);
        if (error_file != NULL)
            fprintf (error_file," GET_RAW_DATA: profile length exceeded (%d)\n",
                y->profile_length);
        break;
    }
//    y->press_temp[k+1][1] = (256*x.data.d_fmt.temperature[j][0] +
//     x.data.d_fmt.temperature[j][1]);
//    y->press_temp[k+1][2] = j;
//    j++;
//    y->press_temp[k+1][0] = (256*x.data.d_fmt.temperature[j][0] +
//     x.data.d_fmt.temperature[j][1]);
//    j++;
//    k++;
//    if ((k+1) >= 15*frames_per_message)
//     return (y->profile_no);
//    y->press_temp[k+1][0] = -999;
    y->surface_tp[n_surfs][1] = (256*x.data.d_fmt.temperature[j][0] +
     x.data.d_fmt.temperature[j][1]);
    j++;
    y->surface_tp[n_surfs][0] = (256*x.data.d_fmt.temperature[j][0] +
     x.data.d_fmt.temperature[j][1]);
    j++;
	k++;
	if (n_surfs < 3)
     y->surface_tp[n_surfs+1][0] = -999;
   }
   y->msg_blk_no *= actual_frames_per_message;
  }
// Reverse surface temperatures and pressures if less than 3 surface values
// If values make sense ( p < 100) and (t > 100) don't reverse them
/*  if (n_surfs < 4)
  {
   k = 0;
   for (j=0; j<n_surfs; j++)
    if ((y->surface_tp[j][0] < 100) && (y->surface_tp[j][1] > 100))
     k = 1;
   if (k == 0)
    for (j=0; j<n_surfs; j++)
	{
	 k = y->surface_tp[j][0];
	 y->surface_tp[j][0] = y->surface_tp[j][1];
	 y->surface_tp[j][1] = k;
	}
  }*/
  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(y->profile_no);
}

int prescan_data (char **buffer, int n_scans)
{

 int i,j,k,l,matches[frames_per_message] = {0,0,0,0,0,0,0,0};
 int num_matches,most_likely[frames_per_message] = {-1,-1,-1,-1,-1,-1,-1,-1};
// unsigned char tmp_buff[2][256];
 char tmp_buff[2][256];

//  Finds the most likely message for the T/P floats
//  Returns with all position scan status bytes = '0'
//  Returns with most likely scan status byte for each frame = '0'
//  Returns with all other scan status bytes = '1'
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if (tmp_buff[0][good_bad_position] != '0')
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
  if (k<0)
   continue;   //Ignore if frame number out of range
  if (k > frames_per_message)
  {
   tmp_buff[0][good_bad_position] = '1';
   continue;   //Set status = bad if frame number out of range
  }
  if (k == 0)
  {
   tmp_buff[0][good_bad_position] = '2';
   continue;   //Frame number = 0 => position scan so mark with '2'
  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches[k-1]) //Is this the most matches?
  {
   matches[k-1] = num_matches; //Yes - save match count
   most_likely[k-1] = i;       //Save record number
  }

 }
 for (i=0; i<n_scans; i++)   //Reset position frame status to zero, all others to '1'
  tmp_buff[0][good_bad_position] = (tmp_buff[0][good_bad_position] == '2') ? (char) '0' : (char) '1';
 for (i=0; i<frames_per_message; i++)    //Set most likely scan status to '0'
  if (most_likely[i] >= 0)
   buffer[most_likely[i]][good_bad_position] = '0';
 return 0;
}

// int get_type2_data (int np, struct type2_meta_struct *s, unsigned char *out_dir)
int get_type2_data (int np, struct type2_meta_struct *s, char *out_dir)
{
  struct output_record y;
  char **buff;
  int i,j;
  int iy,imo,ida,ih,im,is,ndoy;
  int n_prof=0;
  float d_jd;
  int write_to_disk_error;

  //xxd new modification
  float jd_old=0,jd;
  int profSeqCounter=0,times=0,has_prof0=0,profile_no_prev=0,profile_no_current=0;
  float jd_firstline_prev_old=0,jd_firstline_current_old=0;
  float jd_firstline_prev_new=0,jd_firstline_current_new=0;
  char* onelinehex;  


/* Allocate needed memory */

  if ((buff = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)
  {
   if (trace_file != NULL)
        fprintf (trace_file," GET_type2_DATA: Error allocating buffer pointers\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type2_DATA: Error allocating buffer pointers\n");
   return 1;
  }
  for (i=0; i<max_scans; i++)
   if ((buff[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)
   {
   if (trace_file != NULL)
        fprintf (trace_file," GET_type2_DATA: Error allocating data buffer arrays\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type2_DATA: Error allocating data buffer arrays\n");
    return 1;
   }

   if((onelinehex=(char *) malloc(buff_size*(sizeof(char))))==NULL){
     if (trace_file != NULL)
        fprintf (trace_file," GET_type2_DATA: Error allocating data buffer arrays\n");
     if (error_file != NULL)
        fprintf (error_file," GET_type2_DATA: Error allocating data buffer arrays\n");
     return 1;
   }

  if (trace_file != NULL)
   fprintf (trace_file," GET_type2_DATA: Arrays successfully allocated\n");
/* Get the data */

  j=0;
  while (fgets(onelinehex,buff_size,input_file) != NULL)
  {
   if(sscanf(onelinehex,"%d %d %d %d %d %d %d", &i,&iy,&imo,&ida,&ih,&im,&is) != 7)    continue;
   if (s->start_time > (d_jd = (float) dfr97(iy,imo,ida,ih,im,0,&ndoy)))    continue;
   jd = (int) d_jd;times++;if(times==1) {jd_old=jd;jd_firstline_prev_new=jd;}
      
   if(jd-jd_old<2) strcpy(buff[j],onelinehex);
  
  //skip those lines whose jd difference is less than normal cycle, these are ghost messages.(XXD)  
   if(jd-jd_old>2 && (jd-jd_firstline_prev_new)<0.8*s->cycle_time) {printf("bypassing--%s\n",onelinehex);continue;}
   
   if((abs(jd - jd_old) > 2) && (j > 0)){
    profSeqCounter++;jd_firstline_current_new=jd;
    
    if(profSeqCounter==1){
	    if((jd_old - (int) s->start_time) < 2){
          y.profile_no=0;
          has_prof0=1;
        } 
        else {y.profile_no=1;}
	    if(np>0) n_prof = get_raw_data (&y,buff,j,y.profile_no,profile_no_prev);
    }else{
	    if(np>0) n_prof = get_raw_data (&y,buff,j,1,profile_no_prev);	   
            y.profile_no=correct_profilenumber(jd_firstline_prev_old,jd_firstline_current_old,profSeqCounter,profile_no_prev,s->start_of_cycles,s->cycle_time,n_prof,has_prof0);
    }
    
    //sprintf (buff[0],"%s",buff[j]);
    strcpy(buff[0],onelinehex);
    j = 0;times=0;
    
    
    profile_no_current=y.profile_no;
    printf("profSeqCounter=%d,profile_no_current=%d\n",profSeqCounter,profile_no_current);
    
    if(np>0 && y.profile_no>= 0 && n_prof>=0) write_to_disk (out_dir,s,&y);
    
    printf("profile_no finally is %d\n",y.profile_no);
     
    jd_firstline_prev_old=jd_firstline_prev_new;
    jd_firstline_current_old=jd_firstline_current_new;
    profile_no_prev=profile_no_current;
       
   }
   jd_old = jd;
   if (++j >= max_scans)
   {
    if (trace_file != NULL) fprintf (trace_file," Maximum scans for profile exceeded\n");
    if (error_file != NULL) fprintf (error_file," Maximum scans for profile exceeded\n");
    exit (1);
   }  
  }//end of while loop

if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
{  

  profSeqCounter++;    

  if(profSeqCounter==1){
  	    if((jd_old - (int) s->start_time) < 2){y.profile_no=0;has_prof0=1;} else {y.profile_no=1;}
  	    n_prof = get_raw_data (&y,buff,j,y.profile_no,profile_no_prev);
  }else{
  	    n_prof = get_raw_data (&y,buff,j,1,profile_no_prev);
  	    y.profile_no=correct_profilenumber(jd_firstline_prev_old,jd_firstline_current_old,profSeqCounter,profile_no_prev,s->start_of_cycles,s->cycle_time,n_prof,has_prof0);
  }
  
  profile_no_current=y.profile_no;
  printf("profSeqCounter=%d,profile_no_current=%d\n",profSeqCounter,profile_no_current);  
  if (y.profile_no >= 0 && n_prof>=0){write_to_disk_error=write_to_disk (out_dir,s,&y);} else{write_to_disk_error=1;}
  printf("profile_no finally is %d\n",y.profile_no); 
  
}else{
  write_to_disk_error =1; // no data to store -> something may be wrong 
}//end of if(j>0)

  
  free (buff);
  return (write_to_disk_error);  
}

// int get_meta_file_name (unsigned char *str_name)
int get_meta_file_name (char *str_name)
{
// unsigned char *ptr;
 char *ptr;

  
 if ((ptr = strstr(str_name,".hex"))!= NULL)
  sprintf (ptr,".meta");
 else if ((ptr = strstr(str_name,".HEX"))!= NULL)
  sprintf (ptr,".META");
 else
  strcat(str_name,".meta");
 return 0;
}

// int read_type2_meta_file (unsigned char *meta_name, struct type2_meta_struct *meta_struct)
int read_type2_meta_file (char *meta_name, struct type2_meta_struct *meta_struct)
{
//  FILE *meta_file;
 int i,j,syear,smonth,sday,shour,smin,x;
// unsigned char *ptr;
 char *ptr;
 float x1;//added   
// unsigned char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};
 char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};

 for (i=0; i<(sizeof(struct aoml3_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 //added
 meta_struct->start_of_cycles = (float) 0.0; 
 
 printf("\nOpening meta file %s...",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  printf ("error");
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return 1;
 }
 else
  printf ("OK");
 if (trace_file != NULL)
 fprintf (trace_file," OK");

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
    meta_struct->AOML_number = 0;
   continue;
  }
  if (strstr(buffer,"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
     meta_struct->WMO_id = 0;
   continue;
  }
  if (strstr(buffer,"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
     meta_struct->ARGOS_id = 0;
   continue;
  }
  if (strstr(buffer,"TRANSMISSION REPETITION RATE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
     meta_struct->ARGOS_rep_rate = 0;
   continue;
  }
  if (strstr(buffer,"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) != 1)
// CS ????? !!!!!!!!! this has to be changed quit if no value !!!!

    x = 0;
   meta_struct->start_cycle_time += (float) (x/24.0);
   continue;
  }
  if (strstr(buffer,"WMO INSTRUMENT TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
    meta_struct->wmo_instrument = 0;
   continue;
  }
  if (strstr(buffer,"WMO RECORDER TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
    meta_struct->wmo_recorder = 0;
   continue;
  }
  if (strstr(buffer,"PROGRAM NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
    meta_struct->exp_no = 0;
   continue;
  }
  if (strstr(buffer,"DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += x;
   //added
   meta_struct->start_of_cycles += x;   
   
   continue;
  }
  //added
    if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
      x1 = 0;
      meta_struct->start_of_cycles += (float) (x1/24.0);
      continue;
    }
  
  if ((strstr(buffer,"INSTRUMENT TYPE") != NULL) && (strstr(buffer,"WMO") == NULL))
	if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
	{
		sprintf (meta_struct->float_type,"\0");
	    continue;
	}
  if (strstr(buffer,"PI     ") != NULL)
  {
	if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
	{
		sprintf (meta_struct->PI,"\0");
	    continue;
	}
   if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
		sprintf (meta_struct->PI,"%s",ptr);
   if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
	   sprintf (ptr,"\0");
   continue;
  }
  if (strstr(buffer,"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += (float) (x/24.0);
   meta_struct->start_cycle_time -= (float) (x/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
    meta_struct->start_time += (float) dfr97(syear,smonth,sday,shour,smin,0,&x);
   else
    meta_struct->start_time = 0;
   meta_struct->start_cycle_time = (float) (meta_struct->start_time +.025);
   //added 
   meta_struct->start_of_cycles += meta_struct->start_time;    

   continue;
  }
  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
    meta_struct->cals.c[i] = atof(ptr+j);
   i++;
  }
 }
// for (i=0; i<(sizeof(struct aoml3_cals)/sizeof(double)); i++)
//  printf ("\nCal %0d %12.5lf",i,meta_struct->c[i]);
 return 0;
}

int main (int argc, char *argv[])
{
 char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt, 
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80],output_directory[256], msg[11]="", title[257]="";
 enum     {LAST=0,ALL} get_which_profile;
 struct   type2_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
 unsigned long t;
 int      ret_val = 0;

 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature and pressure counts for ");
 strcat(title, "APEX_T2 Floats");
 printf("%s\n", title);

 if ( argc < 7 )
 {
  printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
          PRGNAME);
  printf("\n");
  printf("hex_infile:    hexadecimal input file name\n");
  printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
  printf("cnt_outdir:    output directory for generated count data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Hex File name
// -------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
// -------------------------------------------------------------------------
 if (atoi(argv[2]) >= 1)
  get_which_profile = ALL;
 else
  get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
         PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 printf ("%s: Output Directory - %s\n", PRGNAME, argv[3]);
 sprintf (output_directory,"%s/",argv[3]);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("ERROR (%s): opening trace file %s\n", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> Trace file opened\n", PRGNAME);

 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[5]);
 printf ("%s: Opening error file %s\n", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
   if (trace_file != NULL)
    fprintf (trace_file,"ERROR (%s): Opening file %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,"ERROR (%s): Opening file %s\n", PRGNAME, buffer);

   t = time(NULL);
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   ret_val = 1;
   return ret_val;
 }
 printf("%s: OK -> Error file opened\n", PRGNAME);

 if (trace_file != NULL)
  fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

  sprintf (buffer,"%s",argv[6]);
  printf ("%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val =1;
   return ret_val;
  }
  printf("%s: OK -> List file opened\n", PRGNAME);
  if (trace_file != NULL)
   fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);

// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("\nERROR (%s): HEXFILES_DIR <-- no such environment variable", PRGNAME);

   if (trace_file != NULL)
    fprintf (trace_file,
             "\nERROR(%s): HEXFILES_DIR <-- no such environment variable\n",
             PRGNAME);

   if (error_file != NULL)
    fprintf (error_file,
             "\nERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
             PRGNAME);

   if (trace_file != NULL)                  // Close Log files
     fclose (trace_file);

   if (error_file != NULL)
     fclose (error_file);
 
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }
  
 sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

 if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
 {
   printf ("\nERROR (%s): Opening input file %s\n", PRGNAME, pathname_input_filename);
   if (trace_file != NULL)
    fprintf (trace_file,"\nERROR (%s): Input file %s not opened", 
             PRGNAME, pathname_input_filename);
   if (error_file != NULL)
    fprintf (error_file,"\nERROR (%s): Input file %s not opened", 
             PRGNAME, pathname_input_filename);
   ret_val = 1;
 }
 else
 {
  printf ("%s: Input file %s successfully opened\n", 
          PRGNAME, pathname_input_filename);
  if (trace_file != NULL)
   fprintf (trace_file,"\n%s: Input file %s successfully opened", 
            PRGNAME, pathname_input_filename);

// META File
// ---------
  if (get_meta_file_name (input_filename) != 0)
  {                                             // Failure: did not get meta filename
    printf ("ERROR (%s): Finding meta file name\n", PRGNAME);
    if (error_file != NULL)
     fprintf (error_file,
              "\nERROR (%s): Finding meta file name - returned name %s",
              PRGNAME, input_filename);

    if (trace_file != NULL)
     fprintf (trace_file,
              "\nERROR (%s): Finding meta file name - returned name %s",
              PRGNAME, input_filename);
    ret_val = 1;
   }
   else
  {                                            // Success: got meta filename
   if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
   {
    printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
     fprintf (trace_file,
              "\nERROR (%s): METAFILES_DIR <-- no such environment variable",
              PRGNAME);

    if (error_file != NULL)
     fprintf (error_file,
              "\nERROR (%s): METAFILES_DIR <-- no such environment variable",
              PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);

    if (error_file != NULL)
     fclose (error_file);

    ret_val = 1;
   } // endif  ptr_path_meta 

   sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

   if (read_type2_meta_file (pathname_input_filename, &s) != 0)
   {                                         // Failure: couldn't read meta_file
    printf ("\nERROR (%s): Reading meta file\n", PRGNAME);
    if (error_file != NULL)
     fprintf (error_file,
             "\nERROR (%s): Reading meta file %s", 
             PRGNAME, pathname_input_filename);
    if (error_file != NULL)
     fprintf (error_file,
              "\nERROR (%s): Reading meta file %s", 
              PRGNAME, pathname_input_filename);
    ret_val = 1;
   }
   else                                      // Success: read meta_file
   {
    if (trace_file != NULL)
      fprintf (trace_file,
               "\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename); 
    printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename); 
 
// Make profile(s) selection
// -------------------------
    if (get_type2_data(get_which_profile,&s,output_directory) !=0 )
      ret_val = 1;
   } // endif: read_type2_meta_file 
  } // endif: get_meta_file_name
 } //endif: input_file

//added new
  printf("HEX filename=%s\n",argv[1]);
  printf("frame1 missing problem counts=%d\n",frame1_problem_count); 
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }


// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  if (trace_file != NULL)
   fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));
  if (error_file != NULL)
   fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files 
// ---------------
 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 printf("%s -> RET_VAL from get_type2_data: %d\n", PRGNAME, ret_val);
 return ret_val;
}
