/******************************************************************************
   Revision Date: 10-MAR-05 (XXD -working version)
                  24-NOV-04 (XXD-adapted from hex2cnt_t2.c)

   hex2cnt_t3.c 
   ============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   PALACE floats APEX_T3 manufactured in 1999 by Webb Research Cooperation
*******************************************************************************/

#include <stdio.h>
/*#include <conio.h>*/
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans        2000   //Maximum scans for each profile
#define buff_size         256   //Maximum length of each scan read from *.str
#define max_positions     100   //Maximum number of positions per profile
#define min_frame_no        0   //Minimun frame number
#define frames_per_message 256   //Number of frames per message,it is the maximum allowed number.
#define good_bad_position  43   //Location of status byte in *.str records
#define start_of_hex_data  63   //Start position of data in *.str records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0
#define max_samples_at_depth 256 //maximum samples at depth


int debug=0;//1 debug, 0 no debug
int frame1_problem_count=0,total_cycles_count=0;//added new.
int actual_frames_per_message;

char *strupr(char* ptr_to_string_to_be_upper_cased); 

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char   PRGNAME[] ="hex2cnt_t3";

double cal[13];
int    aoml_id;
char   buffer[512];
enum   flg {NO,YES} cal_flg;
//enum   data_flag {NONE,BAD,GOOD};
enum   {F,T} cd_input = F;

struct aoml3_cals { double TSLOPE;
                    double TOFF;
                    double PSLOPE;
                    double POFF;
                    double VSLOPE;
                    double VOFF;
                    double VACSLOPE;
                    double VACOFF;
                  };

union u_cals { struct aoml3_cals s;
               double c[sizeof(struct aoml3_cals)/sizeof(double)];
             };

struct type2_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
			   int exp_no;
			   unsigned char float_type[40];
                           unsigned char PI[60];
                           float ARGOS_rep_rate;
                           float cycle_time;
                           float start_cycle_time;
                           float start_time;
			   float start_of_cycles;//added Start_time+delay_of_start+down_time			   			   			   
                           union u_cals cals;
                         };

/*      Raw data structure used in decoding Temperature/Pressure (T/P)
 float data.  Part of RAW_DATA_RECORD described below */

struct test_fmt { 
                  unsigned char controller_sn[2];
                  unsigned char time_from_startup[2];
                  unsigned char flag2;
                  unsigned char current_pressure[2];
                  unsigned char battery_volts;
		  unsigned char current_bladder_pressure;
		  unsigned char flag1;
		  unsigned char up_time_intervals;
		  unsigned char down_time_intervals[2];
		  unsigned char park_pressure[2];
		  unsigned char park_piston_position;
		  unsigned char depth_correction;
		  unsigned char ballast_piston_position;
		  unsigned char full_extension;
		  unsigned char ok_vacuum;
		  unsigned char ascent_time;
		  unsigned char target_bladder_pressure;
		  unsigned char profile_pressure[2];
		  unsigned char profile_piston_position;
		  unsigned char deep_cycle_counts;		  
                  unsigned char software_rev_month;
                  unsigned char software_rev_day;
                  unsigned char software_rev_year;
                };

struct data_fmt1{ unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char profile_length;
                  unsigned char termination_flag;
                  unsigned char piston_position;
		  unsigned char format_number;
		  unsigned char depth_table_number;
		  unsigned char pump_motor_time[2];
		  unsigned char init_battery_volts;//?
		  unsigned char init_battery_current;//?
		  unsigned char surface_piston_position;
		  unsigned char air_bladder_pressure;
		  unsigned char bottom_sample_length;
                };//part 1

		//only for illustration purpose,not used in the program,the program directly
		//parse the hex data and put to output struct.

struct data_fmt2{
                  int sample_at_depth_T[max_samples_at_depth];
                  int sample_at_depth_P[max_samples_at_depth];	
                };//part 2
		  
		//only for illustration purpose,not used in the program,the program directly
		//parse the hex data and put to output struct.

struct data_fmt3{ 
                  unsigned char park_temperature[2];
                  unsigned char park_pressure[2];
                  unsigned char battery_volts;
		  unsigned char battery_current;
		  //air_pump_current is not used, if used, then all P/T values are wrong,the documentation is wrong(XXD)		  
		  unsigned char surface_pressure[2];
		  unsigned char internal_vacuum;
		  unsigned char park_piston_position;
                  unsigned char temperature[15*frames_per_message];
                };//part 3,
		
	

/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { unsigned char data_bytes[frames_per_message][29];                  
                  struct data_fmt1 d_fmt1;
                  struct test_fmt t_fmt;
                };

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { int jday;
                         double jd[max_positions];
                         float latitude[max_positions];
                         float longitude[max_positions];
                         char date_time[max_positions][32];
                         int n_xmits[max_positions];
                         char sat_name[max_positions];
                         char loc_class[max_positions];
                         union data_size data;
                       };

/* Final structure containig T/P data and positions */

struct output_record { int    float_id;
                       int    controller_sn;
                       int    profile_no;
                       int    jd;
                       int    status[frames_per_message];
                       int    msg_blk_no;
                       time_t first_blk_time;
                       int    time_from_startup;
                       int    flag2;
                       int    current_pressure;
		       int    current_bladder_pressure;
		       int    flag1;
		       int    up_time_intervals;
		       int    down_time_intervals;
		       int    depth_correction;
		       int    ballast_piston_position;
		       int    full_extension;
		       int    ok_vacuum;
		       int    ascent_time;
		       int    target_bladder_pressure;
		       int    profile_pressure;
		       int    profile_piston_position;
		       int    deep_cycle_counts;		  
                       int    software_rev_month;
                       int    software_rev_day;
                       int    software_rev_year;
                       int    profile_length;
                       int    termination_flag;
                       int    piston_position;
		       int    format_number;
		       int    depth_table_number;
		       int    pump_motor_time;
		       int    init_battery_volts;//?
		       int    init_battery_current;//?
		       int    surface_piston_position;
		       int    air_bladder_pressure;
		       int    bottom_sample_length;
                       //part2,dynamic part
                       int sample_at_depth_T[max_samples_at_depth];
                       int sample_at_depth_P[max_samples_at_depth];	
                       //part3  
                       int    park_temperature;
                       int    park_pressure;
                       int    battery_volts;
		       int    battery_current;		       
		       int    surface_pressure;
		       int    internal_vacuum;
		       int    park_piston_position;

                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       int    press_temp[15*frames_per_message][3]; //[0]=p,[1]=t,[2]=bin
                     };

long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);


char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = passed_jd - jd_970101;
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}

int data_flag2(int bin_no, int *status_word,int actual_sample_at_depth_length)
{ 
 return status_word[(bin_no*4+actual_sample_at_depth_length*4+24)/29];
}

int write_to_disk (char *dir_name, struct type2_meta_struct *s, struct output_record *x)
{

/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no 
 temperature or pressure data. */

 int  i,j,id;
 char *ptr_cnt_filename;
 struct tm *time_str;
 int actual_sample_at_depth_length,max_rows_sample_at_depth,row,max_samples_per_row=10;//print 10 samples per row
 
 id = x->float_id;
/*
  does not work here!!!
 if (((x->status[0]) == NEVER_RECEIVED) && (x->profile_no > 0))   //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file,"First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file,"First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 */
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 printf ("Opening output file %s...",buffer);
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  printf ("error");
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened",buffer);
 printf ("OK\n", PRGNAME);

// Write Output Profile Count File Name to List File
// -------------------------------------------------  
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  printf ("\nCan't get cnt file basename %s", buffer);
  if (trace_file != NULL)
   fprintf (trace_file,
            "\n%s : Can't get cnt file basename %s", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "\n%s: Can't get cnt file basename %s", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 
 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %d\n"
     "TRANSMISSION ID NUMBER                  %d\n"
     "PROFILE NUMBER                          %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
     s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n"
     "CONTROLLER SN                           %d\n",
     s->float_type,x->controller_sn);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 
 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 x->first_blk_time -= (long) (x->msg_blk_no * s->ARGOS_rep_rate);

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((double)x->first_blk_time/24./3600.)));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if (x->profile_no > 0)
 {
 sprintf (buffer,
     "PROFILE LENGTH (BINS)                   %-5d\n"
     "BATTERY VOLTAGE (COUNTS)                %-5d\n"
     "BATTERY CURRENT (COUNTS)                %-5d\n"
     "AIR BLADDER PRESSURE (COUNTS)           %-5d\n",
      x->profile_length,x->battery_volts,x->battery_current,x->air_bladder_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "AIR PUMP MOTOR TIME                     %-5d\n"
     "BATTERY VOLTAGE INITIAL PUMP (COUNTS)   %-5d\n"
     "BATTERY CURRENT INITIAL PUMP (COUNTS)   %-5d\n",
      x->pump_motor_time,x->init_battery_volts,
                               x->init_battery_current);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "FORMAT NUMBER                           %-5d\n"
     "DEPTH TABLE NUMBER                      %-5d\n"          
     "SURFACE PRESSURE (COUNTS)               %-5d\n",
      x->format_number,
      x->depth_table_number,
      x->surface_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INTERNAL VACUUM                         %-5d\n"
     "PISTON POSITION                         %-5d\n"
     "PARK PISTON POSITION                    %-5d\n"
     "SURFACE PISTON POSITION                 %-5d\n",     
      x->internal_vacuum,x->piston_position,x->park_piston_position,
      x->surface_piston_position);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
 sprintf (buffer,
     "BOTTOM SAMPLE LENGTH                    %-5d\n"
     "PARK PRESSURE (COUNTS)                  %-5d\n"
     "PARK TEMPERATURE (COUNTS)               %-5d\n"
     "PROFILE TERMINATION FLAG (HEX)          %-2X\n",
      x->bottom_sample_length,x->park_pressure,x->park_temperature,x->termination_flag);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 //get actual_sample_at_depth_length
 for(j=0;j<max_samples_at_depth;j++){
    if(x->sample_at_depth_P[j]==-999){
       break;
    } 
 }
 actual_sample_at_depth_length=j;
 if(debug>0) printf("actual sample at depth length=%d\n",actual_sample_at_depth_length);
 max_rows_sample_at_depth=(int) ceil((float)actual_sample_at_depth_length/max_samples_per_row);
 if(debug>0) printf("maximum rows of sample at depth length=%d\n",max_rows_sample_at_depth);
 
 //print sample_at_depth samples P 
 for(row=0;row<max_rows_sample_at_depth;row++){
    sprintf (buffer, "SAMPLE AT DEPTH PRESSURE %d (COUNTS)     ",row+1);
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
    for(j=0;j<max_samples_per_row;j++){
       if(row*max_samples_per_row+j>=actual_sample_at_depth_length) break;
       sprintf(buffer,"%-6d ",x->sample_at_depth_P[row*max_samples_per_row+j]);       
       fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
    sprintf(buffer,"\n");  
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }

 //print sample_at_depth samples T 
 for(row=0;row<max_rows_sample_at_depth;row++){
    sprintf (buffer, "SAMPLE AT DEPTH TEMPERATURE %d (COUNTS)  ",row+1);
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
    for(j=0;j<max_samples_per_row;j++){
       if(row*max_samples_per_row+j>=actual_sample_at_depth_length) break;
       sprintf(buffer,"%-6d ",x->sample_at_depth_T[row*max_samples_per_row+j]);       
       fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
    sprintf(buffer,"\n");  
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
          
 sprintf (buffer,
     "NUMBER OF COLUMNS                       3\n"
     "1. COLUMN                               PRESSURE (COUNTS)\n"
     "2. COLUMN                               TEMPERATURE (COUNTS)\n"
     "3. COLUMN                               CHECKSUM FLAG\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
 else
 {
  sprintf (buffer,
     "TIME FROM STARTUP                       %-5d\n"
     "CURRENT PRESSURE (COUNTS)               %-5d\n"
     "BATTERY VOLTAGE(COUNTS)                 %-3d\n"
     "CURRENT BLADDER PRESSURE (COUNTS)       %-5d\n"
     "PROFILE PISTON POSITION                 %-3d\n",
     x->time_from_startup,x->current_pressure,x->battery_volts,
     x->current_bladder_pressure,x->profile_piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "PARK PRESSURE (COUNTS)                  %-5d\n"
     "PARK PISTON POSITION                    %-5d\n"
     "PROFILE PRESSURE (COUNTS)               %-5d\n"
     "TARGET BLADDER PRESSURE (COUNTS)        %-5d\n",
     x->park_pressure,x->park_piston_position,
     x->profile_pressure,x->target_bladder_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "UP TIME INTERVALS (HRS)                 %-3d\n"
     "DOWN TIME INTERVALS (HRS)               %-5d\n"
     "ASCENT TIME                             %-3d\n",
     x->up_time_intervals,x->down_time_intervals,
     x->ascent_time);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "DEEP CYCLE COUNTS                       %-3d\n"
     "DEPTH CORRECTION FACTOR                 %-3d\n",
     x->deep_cycle_counts,x->depth_correction);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "BALLAST PISTON POSITION                 %-3d\n"
     "FULL EXTENSION                          %-3d\n"
     "OK VACUUM COUNT                         %-3d\n",
     x->ballast_piston_position,x->full_extension,
     x->ok_vacuum);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "SOFTWARE REVISION DATE (MON/DAY/YR)     %02d/%02d/%02d\n",
     x->software_rev_month,x->software_rev_day,x->software_rev_year);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
    "==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
//x->status,);

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i++;
 }
 fprintf (out_file,"==========================================================\n");

/* Write depth, temperature, and salinity to disk */
   
 if (x->profile_no > 0)
 {
  //if park_pressure is the largest,then put it into profile data part,otherwise, do not put there
  if(x->park_pressure>=x->press_temp[1][0]){
    i = 0;
  }else{
    i = 1;
  }
  while (x->press_temp[i][0] > -999)
  {
   if ((j = data_flag2(x->press_temp[i][2],x->status,actual_sample_at_depth_length)) == NEVER_RECEIVED)
    sprintf (buffer," %-5d  %-5d  %d\n",0,0,0);
   else
    sprintf (buffer," %-5d  %-5d  %d\n",
	                      x->press_temp[i][0],x->press_temp[i][1],j);
   fwrite (buffer,1,strlen(buffer),out_file);
   if (++i > 15*frames_per_message)
	   break;
  }
 }

   if (out_file != NULL)
    fclose (out_file);   /* Close output file */

  return 0;
}

int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad)
{
 int i,j,k,l,matches = 0;
 int num_matches,most_likely = -1;
 char tmp_buff[2][256];

//  Finds the most likely scan for the APEX floats
//  Returns number of most common scan except for test message
//  Search for test message if frame_num = 0
//  Test message detected if block number > frames per message and flag2 bits 6 or 7 set
//  Returns -1 if error
//  good_bad=1 -> use checksum; good_bad=0->ignore checksum
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if ((tmp_buff[0][good_bad_position] != '0') && good_bad)
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
/*  if ((k > frames_per_message) && (frame_num == 0))
  {
   sscanf(&tmp_buff[0][start_of_hex_data+14],"%2X",&j); //j = flag2 byte
   if (((j & 0X60) == 0X20) || ((j & 0X60) == 0X40))
 most_likely = i; //returns last test message with block number > frames_per_message
   continue;
  }*/
  if ((k-1) != frame_num) //Frame = 1 to 15 => most_common[0-14]
   continue;
//  if (k == 1)
//  {
//   tmp_buff[0][start_of_hex_data+4] = '0';
//   tmp_buff[0][start_of_hex_data+5] = '0'; //set message blk number = 0 because it's not constant
//  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
//   if (l == 1)
//   {
//    tmp_buff[1][start_of_hex_data+4] = '0';
//    tmp_buff[1][start_of_hex_data+5] = '0'; //set message blk number = 0 because it's not constant
//   }
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches) //Is this the most matches?
  {
   matches = num_matches; //Yes - save match count
   most_likely = i;       //Save record number
  }
// TEST CS
  if ((num_matches==0) && (matches==0)) // Make sure to use messages only received once
  {
    most_likely=i;
  }

 }
 return most_likely;
}



int get_raw_data (struct output_record *y, char ** buffer, int n_scans,int m_type)
{

  struct raw_data_record x;
  struct tm *time_str;
  int i,j,k,most_common[frames_per_message],n_frames=frames_per_message;
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0,n_surfs=4,n_meas;
  int iy1,imo1,ida1,ih1,im1,is1;
  
  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";
  
  int tt,pp;
  int begin_byte_part3;//the position of the beginning of part3 in just_bytes
  unsigned char just_bytes[frames_per_message*29];
      
  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[31],roll_over=0,old_count=0;
  int p_table[] = {
  17000,16500,16000,15500,15000,14500,14000,13500,13000,12500,12000,11500,11000,10500,10000,9500,9000,8500,8000,7500,7000,6500,6000,5500,5000,
  4800,4600,4400,4200,4000,3800,3600,3400,3200,3000,2800,2600,
  2500,2400,2300,2200,2100,2000,1900,1800,1700,1600,1500,1400,1300,1200,1100,1000,900,800,700,600,500,400,300,200,100,40,0};


			
//09334 1997 08 24 15 24 14 +00.000 +000.000 0 041 J 0 069516551 F506FB11C711B11191115C11541153115211501145113010DF10B2108E103E0E
  //reset sample_at_depth_T and sample_at_depth_P for each profile
  for(i=0;i<max_samples_at_depth;i++){
    y->sample_at_depth_T[i]=-999;
    y->sample_at_depth_P[i]=-999;
  }
  //added new
  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
   y->status[i] = NEVER_RECEIVED;
   for (j=0; j<29; j++)
    x.data.data_bytes[i][j] = 0;
   if ((j = most_likely_frame (buffer, n_scans, i, 1)) >= 0) //find most likely - good CRC only
    most_common[i] = j;
   else if ((most_likely_frame (buffer, n_scans, i, 0)) >= 0) //find most likely - any CRC
    most_common[i] = j;
   else 
    most_common[i] = -1;
  }

  if ((m_type == 1) && most_common[0] < 0)
  {
   //added new
   frame1_problem_count++;

   if (trace_file != NULL)
    fprintf (trace_file," Can't decode message without frame 1\n");
   if (error_file != NULL)
    fprintf (error_file," Can't decode message without frame 1\n");
//   exit(9);	//Can't decode message without frame 1 - error return
   return(-1);	//Can't decode message without frame 1 - error return
  }

  for (j=1; j<max_positions; j++)
   x.jd[j] = -1.0;

  for (i=0; i<n_scans; i++)
  {
   if (sscanf(buffer[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
    continue;

    // Read third and fourth characters from hex_data, convert from hex to decimal and
    // skip if > frames_per_message.
    frames=0;
    for (l=0; l<2 ; l++)
    {
       ch=*(hex_data+l+2);
       switch (ch)
       {
         case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
         case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
         case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
         case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
         case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
         case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
         default : frms[0]=ch; frms[1]='\0'; break;
       }
       frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
    }
        
    if (frames > frames_per_message)
    {
      continue;
    }

   x.latitude[n_pos] = lat;
   x.longitude[n_pos] = lon;
   x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

   sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
   x.loc_class[n_pos] = l_c;
   x.sat_name[n_pos] = sat;
   x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
   if ((lat != (float) 0.0) || (lon != (float) 0.0))
   {
    n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
    continue;
   }
   x.latitude[n_pos] = (float) 99.999;
   x.longitude[n_pos] = (float) 999.999;
   x.n_xmits[n_pos] = 0;
   n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

   for (j=0; j<31; j++)
    sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);
   
   if ((z[1] - min_frame_no) < 1)
    continue;
   
     
   if (m_type == 1)
   {
    if (z[1]>frames_per_message)
    {
    if (trace_file != NULL)
      fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"z[1] exceeds frames_per_message.\n");
    exit (1);
    }
     
    if (i != (most_common[z[1] - 1]))
     continue;

    if(debug>0) {
      printf("m_type=%d,z[1]=%d\n",m_type,z[1]);
      printf("hex=%s\n",hex_data);       
    }

    if (z[1] == 1)
    {
     if (z[2] < old_count)
        roll_over++;
     old_count = z[2];
     y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
    }
    y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
    for (k=0; k<29; k++)
     x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
   }
   else if ((m_type == 0) && (g == 0))
   {
     if (z[1] < old_count)
     roll_over++;
     old_count = z[1];
     y->msg_blk_no = z[1]-1 + roll_over*256; //count starts at 1
     z[1] = 1; //Z[1] now has frame number = 1 for test msg
     for (k=0; k<29; k++)
      x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
   }
//Speed up - move time calculations outside loop
   if (z[1] == 1)
   {
    is1 = is;
    im1 = im;
    ih1 = ih;
    ida1 = ida;
    imo1 = imo;
    iy1 = iy;
   }
  }

    y->first_blk_time = ((ymd2julday(iy1,imo1,ida1)*24+ih1)*60+im1)*60+is1;

  x.jday = (int) floor(x.jd[0]);

  if ((trace_file != NULL) && (m_type == 1))
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",(int)x.data.d_fmt1.profile_no);
    for (j=(frames_per_message-1); j>=0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }

  y->float_id = id;
  y->jd = x.jday;

  if(debug>0) printf("message type=%d ==================\n",m_type);
  k=0;
  for(i=0;i<frames_per_message;i++){
    for(j=0;j<29;j++){      
      just_bytes[k]=x.data.data_bytes[i][j];
      //if(debug>0) printf("just_bytes k=%d,value=%d\n",k,just_bytes[k]);
      k++;
    }
  }
  k=0;
  if (m_type == 1) //Data message
  {
  //data message part 1
  y->controller_sn=256*just_bytes[1]+just_bytes[2];
  y->profile_no=just_bytes[3];
  y->profile_length=just_bytes[4];
  y->termination_flag =just_bytes[5]; 
  y->piston_position = just_bytes[6];
  y->format_number=just_bytes[7];
  y->depth_table_number=just_bytes[8];
  y->pump_motor_time=256*just_bytes[9]+just_bytes[10];;
  y->init_battery_volts=just_bytes[11];
  y->init_battery_current=just_bytes[12];
  y->surface_piston_position=just_bytes[13];
  y->air_bladder_pressure=just_bytes[14];
  y->bottom_sample_length=just_bytes[15];
  
  if(debug>0){
    printf("profileno=%d bottom_sample_depth=%d\n",y->profile_no,y->bottom_sample_length);  
  }
  
  //data message part 2
  for(i=0;i<y->bottom_sample_length;i++){
   //just_bytes from byte 16 on are sample at depth T/P pairs(totally there are bottom_sample_length pairs)
     tt=256*just_bytes[i*4+16]+just_bytes[i*4+16+1];
     pp=256*just_bytes[i*4+16+2]+just_bytes[i*4+16+3];
     y->sample_at_depth_T[i]=tt;
     y->sample_at_depth_P[i]=pp;     
  }
  //data message part 3
  begin_byte_part3=y->bottom_sample_length*4+2+16;//4 bytes of T/P,DDDD,16 bytes of part1  
  y->park_temperature=256*just_bytes[begin_byte_part3]+just_bytes[begin_byte_part3+1];
  y->park_pressure=256*just_bytes[begin_byte_part3+2]+just_bytes[begin_byte_part3+3];
  y->battery_volts=just_bytes[begin_byte_part3+4];
  y->battery_current=just_bytes[begin_byte_part3+5];
  y->surface_pressure=256*just_bytes[begin_byte_part3+6]+just_bytes[begin_byte_part3+7];
  y->internal_vacuum=just_bytes[begin_byte_part3+8];
  y->park_piston_position=just_bytes[begin_byte_part3+9];
  }
  else //Test message
  {
   y->controller_sn = 256*x.data.t_fmt.controller_sn[0] + x.data.t_fmt.controller_sn[1];
   y->time_from_startup=256*x.data.t_fmt.time_from_startup[0] + x.data.t_fmt.time_from_startup[1];   
   y->flag2=x.data.t_fmt.flag2;
   y->current_pressure = 256*x.data.t_fmt.current_pressure[0] + x.data.t_fmt.current_pressure[1];
   y->battery_volts = x.data.t_fmt.battery_volts;
   y->current_bladder_pressure=x.data.t_fmt.current_bladder_pressure;
   y->flag1=x.data.t_fmt.flag1;   
   y->up_time_intervals = x.data.t_fmt.up_time_intervals;
   y->down_time_intervals = 256*x.data.t_fmt.down_time_intervals[0] + x.data.t_fmt.down_time_intervals[1];
   y->park_pressure = 256*x.data.t_fmt.park_pressure[0] + x.data.t_fmt.park_pressure[1];   
   y->park_piston_position = x.data.t_fmt.park_piston_position;   
   y->depth_correction = x.data.t_fmt.depth_correction;
   y->ballast_piston_position=x.data.t_fmt.ballast_piston_position;
   y->full_extension = x.data.t_fmt.full_extension;   
   y->ok_vacuum = x.data.t_fmt.ok_vacuum;      
   y->ascent_time=x.data.t_fmt.ascent_time;
   y->target_bladder_pressure=x.data.t_fmt.target_bladder_pressure;
   y->profile_pressure=256*x.data.t_fmt.profile_pressure[0] + x.data.t_fmt.profile_pressure[1];
   y->profile_piston_position = x.data.t_fmt.profile_piston_position;
   y->deep_cycle_counts=x.data.t_fmt.deep_cycle_counts;   
   y->software_rev_month = (x.data.t_fmt.software_rev_month/16)*10 + x.data.t_fmt.software_rev_month%16;
   y->software_rev_day = (x.data.t_fmt.software_rev_day/16)*10 + x.data.t_fmt.software_rev_day%16;
   y->software_rev_year = (x.data.t_fmt.software_rev_year/16)*10 + x.data.t_fmt.software_rev_year%16;
  }
  for (j=0; j<max_positions; j++)
  {
   y->latitude[j] = x.latitude[j];
   y->longitude[j] = x.longitude[j];
   y->position_jd[j] = x.jd[j];
   y->n_xmits[j] = x.n_xmits[j];
   y->sat_name[j] = x.sat_name[j];
   y->loc_class[j] = x.loc_class[j];
   strncpy(y->date_time[j],x.date_time[j],24);
  }
  
  //bottom_sample has 4 bytes(T/P) each, 31-19+1=13 bytes are the bytes from byte 19th to 31th to be excluded
  //2+10 is the DDDD and the engineering data bytes after bottom samples.Basically, count from message number 2 to find exactly 
  //how many bytes are in the whole profile,then divided by 29 bytes, plus the message number 1 to get the final
  //actual_frames_per_message
  actual_frames_per_message=(int)ceil((float)(y->bottom_sample_length*4-(31-19+1)+2+10+y->profile_length*4)/29)+1;
  if (actual_frames_per_message > frames_per_message)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }

  if (y->profile_no > 0)
  {
   y->press_temp[0][0] = y->park_pressure;
   y->press_temp[0][1] = y->park_temperature;
   y->press_temp[0][2] = 0;
   j = 0;
   k = 0;
   while (k < (y->profile_length))  
   {
    //if (p_table[j] < y->park_pressure){
     //use depth table case 
     //y->press_temp[k+1][0] = p_table[j];
     //y->press_temp[k+1][1] = (256*just_bytes[k*2+begin_byte_part3+10] +just_bytes[k*2+1+begin_byte_part3+10]);
     
     //do not use depth table
     y->press_temp[k+1][1] = (256*just_bytes[k*4+begin_byte_part3+10] +just_bytes[k*4+1+begin_byte_part3+10]);
     //just_bytes beginning from byte begin_byte_part3+10 on,they are temperature values,each 2 bytes
     y->press_temp[k+1][0] = (256*just_bytes[k*4+2+begin_byte_part3+10] +just_bytes[k*4+3+begin_byte_part3+10]);
     y->press_temp[k+1][2] = k+1;
	 k++;
     if ((k+1) >= 15*frames_per_message)
     {
      if (trace_file != NULL)
       fprintf (trace_file," Number of bins (%2d) exceeds allowable array size\n",n_meas);
      if (error_file != NULL)
       fprintf (error_file," Number of bins (%2d) exceeds allowable array size\n",n_meas);
      return(-1);
     }
     y->press_temp[k+1][0] = -999;
    //}//end of if(p_table[j]
    
	j++;
   }
   j = k;
   y->msg_blk_no *= actual_frames_per_message;
  }

  if(debug>0){
    for(i=0;i<k*4+3+begin_byte_part3+11;i++){
      printf("i=%d,just_bytes=%d\n",i,just_bytes[i]);
    }       
  }
  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);
   
  if(debug>0) printf("===========profile no=%d==============\n",y->profile_no);
  
  return(y->profile_no);
}

int prescan_data (char **buffer, int n_scans)
{

 int i,j,k,l,matches[frames_per_message] = {0,0,0,0,0,0,0,0};
 int num_matches,most_likely[frames_per_message] = {-1,-1,-1,-1,-1,-1,-1,-1};
// unsigned char tmp_buff[2][256];
 char tmp_buff[2][256];

//  Finds the most likely message for the T/P floats
//  Returns with all position scan status bytes = '0'
//  Returns with most likely scan status byte for each frame = '0'
//  Returns with all other scan status bytes = '1'
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if (tmp_buff[0][good_bad_position] != '0')
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
  if (k<0)
   continue;   //Ignore if frame number out of range
  if (k > frames_per_message)
  {
   tmp_buff[0][good_bad_position] = '1';
   continue;   //Set status = bad if frame number out of range
  }
  if (k == 0)
  {
   tmp_buff[0][good_bad_position] = '2';
   continue;   //Frame number = 0 => position scan so mark with '2'
  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches[k-1]) //Is this the most matches?
  {
   matches[k-1] = num_matches; //Yes - save match count
   most_likely[k-1] = i;       //Save record number
  }

 }
 for (i=0; i<n_scans; i++)   //Reset position frame status to zero, all others to '1'
  tmp_buff[0][good_bad_position] = (tmp_buff[0][good_bad_position] == '2') ? (char) '0' : (char) '1';
 for (i=0; i<frames_per_message; i++)    //Set most likely scan status to '0'
  if (most_likely[i] >= 0)
   buffer[most_likely[i]][good_bad_position] = '0';
 return 0;
}

// int get_type2_data (int np, struct type2_meta_struct *s, unsigned char *out_dir)
int get_type2_data (int np, struct type2_meta_struct *s, char *out_dir)
{
  struct output_record y;
  char **buff;
  int i,j, n_prof, ret_val;
  int iy,imo,ida,ih,im,is,ndoy;
  int jd_old=0,jd;
  float jd_flt;
  int write_to_disk_error;
  int corrected_prof_no;
   
/* Allocate needed memory */

  if ((buff = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)
  {
   printf ("\n\nError allocating buffer pointers\n");
   return 1;
  }
  for (i=0; i<max_scans; i++)
   if ((buff[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)
   {
    printf ("\n\nError allocating data buffer arrays\n");
   if (trace_file != NULL)
        fprintf (trace_file," GET_type2_DATA: Error allocating data buffer arrays\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type2_DATA: Error allocating data buffer arrays\n");
    return 1;
   }

  if (trace_file != NULL)
   fprintf (trace_file," GET_type2_DATA: Arrays successfully allocated\n");
/* Get the data */

  j=0;
  while (fgets(buff[j],buff_size,input_file) != NULL)
  {
   if(sscanf(buff[j],"%d %d %d %d %d %d %d",
                &i,&iy,&imo,&ida,&ih,&im,&is) != 7)
    continue;

//   if ((jd_flt = (float) dfr97(iy,imo,ida,0,0,0,&ndoy)) < s->start_time)
   if ((jd_flt = (float) dfr97(iy,imo,ida,ih,im,is,&ndoy)) < s->start_time)
	   continue;
   jd = (int) jd_flt;
   if ((abs(jd - jd_old) > 2) && (j > 0))
   {
    if (np > 0)
    {
//     prescan_data (buff, j);
        if ((jd_old - (int) s->start_time) < 2)
        n_prof=get_raw_data (&y,buff,j,0);
	else
	n_prof=get_raw_data (&y,buff,j,1);
	
        //added profile_no correction
        if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
        {
         //changed here
         corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
         if(corrected_prof_no>0) y.profile_no=corrected_prof_no;
	
         fprintf (trace_file," GET_TYPE2_DATA: transmitted profile no. = %d",y.profile_no);
         fprintf (trace_file," computed = %d - using computed value\n",
         y.profile_no);
        }
     
     if (y.profile_no >= 0 && n_prof!=-1)
      write_to_disk (out_dir,s,&y);
    }
    sprintf (buff[0],"%s",buff[j]);
    j = 0;
   }
   jd_old = jd;
   if (++j >= max_scans)
   {
    if (trace_file != NULL)
     fprintf (trace_file," Maximum scans for profile exceeded\n");
    if (error_file != NULL)
     fprintf (error_file," Maximum scans for profile exceeded\n");
    exit (1);
   }  
  }
//  prescan_data (buff, j);

if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
{
  if ((jd_old - (int) s->start_time) < 2)
  n_prof=get_raw_data (&y,buff,j,0);
  else
  n_prof=get_raw_data (&y,buff,j,1);
  
  //added profile_no correction
  if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
  {
         //changed here
         corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
         if(corrected_prof_no>0) y.profile_no=corrected_prof_no;
  
         fprintf (trace_file," GET_TYPE2_DATA: transmitted profile no. = %d",y.profile_no);
         fprintf (trace_file," computed = %d - using computed value\n",
         y.profile_no);
  }
  
  if (y.profile_no>=0 && n_prof!=-1)
  {
    if (trace_file != NULL)
      printf ("GET_type2_DATA completed successfully\n");
      fprintf (trace_file," GET_type2_DATA completed successfully\n");
    write_to_disk_error=write_to_disk (out_dir,s,&y);
    ret_val = write_to_disk_error;
  }
  else
  {
    printf ("GET_type2_DATA n_prof < 0\n");
    if (trace_file != NULL)
      fprintf (trace_file," GET_type2_DATA n_prof < 0 OR Frame 1 Problem  %d \n",n_prof);
    ret_val = 1;
  }
}else
{
  write_to_disk_error =1; // no data to store -> something may be wrong
}//end of if(j>0)
  
  free (buff);
  return ret_val;
}

// int get_meta_file_name (unsigned char *str_name)
int get_meta_file_name (char *str_name)
{
// unsigned char *ptr;
 char *ptr;

  
 if ((ptr = strstr(str_name,".hex"))!= NULL)
  sprintf (ptr,".meta");
 else if ((ptr = strstr(str_name,".HEX"))!= NULL)
  sprintf (ptr,".META");
 else
  strcat(str_name,".meta");
 return 0;
}

// int read_type2_meta_file (unsigned char *meta_name, struct type2_meta_struct *meta_struct)
int read_type2_meta_file (char *meta_name, struct type2_meta_struct *meta_struct)
{
//  FILE *meta_file;
 int i,j,syear,smonth,sday,shour,smin,x;
// unsigned char *ptr;
 char *ptr;
 float x1;//added   
// unsigned char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};
 char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};

 for (i=0; i<(sizeof(struct aoml3_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 //added
 meta_struct->start_of_cycles = (float) 0.0; 
 
 printf("\nOpening meta file %s...",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  printf ("error");
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return 1;
 }
 else
  printf ("OK");
 if (trace_file != NULL)
 fprintf (trace_file," OK");

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
    meta_struct->AOML_number = 0;
   continue;
  }
  if (strstr(buffer,"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
     meta_struct->WMO_id = 0;
   continue;
  }
  if (strstr(buffer,"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
     meta_struct->ARGOS_id = 0;
   continue;
  }
  if (strstr(buffer,"TRANSMISSION REPETITION RATE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
     meta_struct->ARGOS_rep_rate = 0;
   continue;
  }
  if (strstr(buffer,"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) != 1)
// CS ????? !!!!!!!!! this has to be changed quit if no value !!!!

    x = 0;
   meta_struct->start_cycle_time += (float) (x/24.0);
   continue;
  }
  if (strstr(buffer,"WMO INSTRUMENT TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
    meta_struct->wmo_instrument = 0;
   continue;
  }
  if (strstr(buffer,"WMO RECORDER TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
    meta_struct->wmo_recorder = 0;
   continue;
  }
  if (strstr(buffer,"PROGRAM NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
    meta_struct->exp_no = 0;
   continue;
  }
  if (strstr(buffer,"DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += x;
   //added
   meta_struct->start_of_cycles += x;   
   
   continue;
  }
  //added
    if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
      x1 = 0;
      meta_struct->start_of_cycles += (float) (x1/24.0);
      continue;
    }
  
  if ((strstr(buffer,"INSTRUMENT TYPE") != NULL) && (strstr(buffer,"WMO") == NULL))
	if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
	{
		sprintf (meta_struct->float_type,"\0");
	    continue;
	}
  if (strstr(buffer,"PI     ") != NULL)
  {
	if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
	{
		sprintf (meta_struct->PI,"\0");
	    continue;
	}
   if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
		sprintf (meta_struct->PI,"%s",ptr);
   if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
	   sprintf (ptr,"\0");
   continue;
  }
  if (strstr(buffer,"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += (float) (x/24.0);
   meta_struct->start_cycle_time -= (float) (x/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
    meta_struct->start_time += (float) dfr97(syear,smonth,sday,shour,smin,0,&x);
   else
    meta_struct->start_time = 0;
   meta_struct->start_cycle_time = (float) (meta_struct->start_time +.025);
   //added 
   meta_struct->start_of_cycles += meta_struct->start_time;    

   continue;
  }
  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
    meta_struct->cals.c[i] = atof(ptr+j);
   i++;
  }
 }
// for (i=0; i<(sizeof(struct aoml3_cals)/sizeof(double)); i++)
//  printf ("\nCal %0d %12.5lf",i,meta_struct->c[i]);
 return 0;
}

int main (int argc, char *argv[])
{
 char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt, 
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80],output_directory[256], msg[11]="", title[257]="";
 enum     {LAST=0,ALL} get_which_profile;
 struct   type2_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
 unsigned long t;
 int      ret_val = 0;

 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature and pressure counts for ");
 strcat(title, "APEX_T3 Floats");
 printf("%s\n", title);

 if ( argc < 7 )
 {
  printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
          PRGNAME);
  printf("\n");
  printf("hex_infile:    hexadecimal input file name\n");
  printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
  printf("cnt_outdir:    output directory for generated count data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Hex File name
// -------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
// -------------------------------------------------------------------------
 if (atoi(argv[2]) >= 1)
  get_which_profile = ALL;
 else
  get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",
         PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 printf ("%s: Output Directory - %s\n", PRGNAME, argv[3]);
 sprintf (output_directory,"%s/",argv[3]);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
  printf ("ERROR (%s): opening trace file %s\n", PRGNAME, buffer);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }
 printf("%s: OK -> Trace file opened\n", PRGNAME);

 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[5]);
 printf ("%s: Opening error file %s\n", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
   if (trace_file != NULL)
    fprintf (trace_file,"ERROR (%s): Opening file %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,"ERROR (%s): Opening file %s\n", PRGNAME, buffer);

   t = time(NULL);
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   ret_val = 1;
   return ret_val;
 }
 printf("%s: OK -> Error file opened\n", PRGNAME);

 if (trace_file != NULL)
  fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

  sprintf (buffer,"%s",argv[6]);
  printf ("%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
     fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
     fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val =1;
   return ret_val;
  }
  printf("%s: OK -> List file opened\n", PRGNAME);
  if (trace_file != NULL)
   fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);

// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("\nERROR (%s): HEXFILES_DIR <-- no such environment variable", PRGNAME);

   if (trace_file != NULL)
    fprintf (trace_file,
             "\nERROR(%s): HEXFILES_DIR <-- no such environment variable\n",
             PRGNAME);

   if (error_file != NULL)
    fprintf (error_file,
             "\nERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
             PRGNAME);

   if (trace_file != NULL)                  // Close Log files
     fclose (trace_file);

   if (error_file != NULL)
     fclose (error_file);
 
   t = time(NULL);
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }
  
 sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

 if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
 {
   printf ("\nERROR (%s): Opening input file %s\n", PRGNAME, pathname_input_filename);
   if (trace_file != NULL)
    fprintf (trace_file,"\nERROR (%s): Input file %s not opened", 
             PRGNAME, pathname_input_filename);
   if (error_file != NULL)
    fprintf (error_file,"\nERROR (%s): Input file %s not opened", 
             PRGNAME, pathname_input_filename);
   ret_val = 1;
 }
 else
 {
  printf ("%s: Input file %s successfully opened\n", 
          PRGNAME, pathname_input_filename);
  if (trace_file != NULL)
   fprintf (trace_file,"\n%s: Input file %s successfully opened", 
            PRGNAME, pathname_input_filename);

// META File
// ---------
  if (get_meta_file_name (input_filename) != 0)
  {                                             // Failure: did not get meta filename
    printf ("ERROR (%s): Finding meta file name\n", PRGNAME);
    if (error_file != NULL)
     fprintf (error_file,
              "\nERROR (%s): Finding meta file name - returned name %s",
              PRGNAME, input_filename);

    if (trace_file != NULL)
     fprintf (trace_file,
              "\nERROR (%s): Finding meta file name - returned name %s",
              PRGNAME, input_filename);
    ret_val = 1;
   }
   else
  {                                            // Success: got meta filename
   if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
   {
    printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
     fprintf (trace_file,
              "\nERROR (%s): METAFILES_DIR <-- no such environment variable",
              PRGNAME);

    if (error_file != NULL)
     fprintf (error_file,
              "\nERROR (%s): METAFILES_DIR <-- no such environment variable",
              PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);

    if (error_file != NULL)
     fclose (error_file);

    ret_val = 1;
   } // endif  ptr_path_meta 

   sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

   if (read_type2_meta_file (pathname_input_filename, &s) != 0)
   {                                         // Failure: couldn't read meta_file
    printf ("\nERROR (%s): Reading meta file\n", PRGNAME);
    if (error_file != NULL)
     fprintf (error_file,
             "\nERROR (%s): Reading meta file %s", 
             PRGNAME, pathname_input_filename);
    if (error_file != NULL)
     fprintf (error_file,
              "\nERROR (%s): Reading meta file %s", 
              PRGNAME, pathname_input_filename);
    ret_val = 1;
   }
   else                                      // Success: read meta_file
   {
    if (trace_file != NULL)
      fprintf (trace_file,
               "\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename); 
    printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename); 
 
// Make profile(s) selection
// -------------------------
    if (get_type2_data(get_which_profile,&s,output_directory) !=0 )
      ret_val = 1;
   } // endif: read_type2_meta_file 
  } // endif: get_meta_file_name
 } //endif: input_file

//added new
  printf("HEX filename=%s\n",argv[1]);
  printf("frame1 missing problem counts=%d\n",frame1_problem_count); 
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }


// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  if (trace_file != NULL)
   fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));
  if (error_file != NULL)
   fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files 
// ---------------
 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 printf("%s -> RET_VAL from get_type2_data: %d\n", PRGNAME, ret_val);
 return ret_val;
}
