/******************************************************************************
   16-Dec-2011 ER - Replace PRESSURE SENSOR OFFSET (COUNTS) with
                    SURFACE PRES OFFSET APPLIED 1CBAR(CNTS)
   14-Sep-2007 ER - DRIFT PRESSURE n (COUNTS) lines to be printed as
                    DRIFT PRESSURE (COUNTS)  
               RS - turn off the debugg (set to 0)

   hex2cnt_t4.cpp
   ==================
   This is a PROVO float(metocean provor ver.1.7,April 2004)
         
   To compile: g++ -o hex2cnt_t4.x hex2cnt_t4.cpp

   Usage: hex2cnt_t4.sh

Author: 
        Xiangdong Xia,  30-Mar-2005 
	                Dec. 2004  
        Adapted from hex2cnt_ts11.cpp and hex2cnt_ts23.cpp               
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <vector>
#include <time.h>
#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <set>
#include <algorithm>
#include "sortPT.h"

#define MAXLINES  1000
#define HEXRECLEN 100
#define BINRECLEN 400
#define DECRECLEN 100
#define max(A,B) ( (A) > (B) ? (A):(B))
#define min(A,B) ( (A) > (B) ? (B):(A))
using namespace std;

FILE *input_file, *out_file, *error_file, *trace_file, *list_file;
char PRGNAME[]="hex2cnt_t4";
int debug=0;//1 for debug, 0 for operation

struct aoml4_cals {
  double TGAIN;
  double TOFF_A;
  double TOFF_R;
  double PGAIN;
  double POFF;
};

union u_cals { struct aoml4_cals s;
               double c[sizeof(struct aoml4_cals)/sizeof(double)];
             };

//t4 float meta data struct
struct t4_meta_struct{
  int AOML_number;
  int WMO_id;
  int ARGOS_id;
  int wmo_instrument;
  int wmo_recorder;
  float cycle_time;
  int exp_no;
  char float_type[40];
  char PI[60];
  float launch_time;
  float start_cycle_time;
  char hex_id[20];   //newly added hexadecimal ARGOS ID
  int year;
  int month;
  int day;
  int hour;
  int minute;
  float lat;
  float latm;
  float lon;
  float lonm;
union u_cals cals;
};


//T4 float engineering technical message struct
struct t4_technical_struct{			   
  int descent_start_time;                         // 3       
  int descent_start_time_hh;                      // 3       
  int descent_start_time_mm;                      // 3       			   
  int number_of_valve_actions_at_surface;         // 4
  int float_stable_time;                          // 5
  int float_stable_time_hh;                       // 5
  int float_stable_time_mm;                       // 5
  int float_stable_pressure;                      // 6
  int number_of_valve_actions_descent;            // 7
  int number_of_pump_actions_descent;             // 8
  int end_of_descent_time;                        // 9
  int end_of_descent_time_hh;                     // 9
  int end_of_descent_time_mm;                     // 9			   
  int number_of_repositions;                      //10
  int time_at_end_ascent;                         //11
  int time_at_end_ascent_hh;                      //11
  int time_at_end_ascent_mm;                      //11			   
  int number_of_pump_actions_ascent;              //12
  int duration_of_pump_actions_surface;           //13   
  int number_of_descent_pt_messages;              //14
  int number_of_drift_pt_messages;                //15  
  int number_of_ascent_pt_messages;               //16  
  int shallow_deep_boundary_isobar_descent;       //17
  int number_of_descent_slices_shallow_zone ;     //18
  int number_of_descent_slices_deep_zone;         //19
  int shallow_deep_boundary_isobar_ascent;        //20
  int number_of_ascent_slices_shallow_zone;       //21
  int number_of_ascent_slices_deep_zone;          //22
  int number_of_PT_measurements_in_drift;         //23
  int floats_time_hh;                             //24
  int floats_time_mm;                             //24 
  int floats_time_ss;                             //24  
  int pressure_sensor_offset;                     //25
  int internal_pressure;                          //26
  int surface_temperature;                        //27
  int profile_ascent_start_time;                  //28
  int profile_ascent_start_time_hh;               //28
  int profile_ascent_start_time_mm;               //28
  int number_of_entrance_in_target_range;         //29
  int minimum_pressure_in_drift;                  //30
  int maximum_pressure_in_drift;                  //31			  
  int grounding;                                  //32
  int time_from_satellite_hh;//time from satellite hour
  int time_from_satellite_mm;//time from satellite minute
  int time_from_satellite_ss;//time from satellite second	     
};


// position and satellite information struct, 
// used for output file between ==== and =====
struct position_info_struct {
  vector<float> lat_vector;   //latitude vector
  vector<float> lon_vector;   //longitude vector
  vector<int>   year_vector;  //year vector
  vector<int>   month_vector; //month vector
  vector<int>   day_vector;   //day vector
  vector<int>   hour_vector;  //hour vector
  vector<int>   min_vector;   //minute vector
  vector<int>   sec_vector;   //second vector
  vector<int>   xmits_vector; //transmits vector
  vector<char>  sat_vector;   //satellite type vector
  vector<char>  class_vector; //class vector
};

//drift pressure,temperature and salinity values from technical message type 5, used for count output header part
struct driftPT_struct{
  vector<int>   driftP0_vector;//original drift pressure vector
  vector<int>   driftT0_vector;//original drift temperature vector
  vector<int>   driftP_vector;//adjusted drift pressure vector
  vector<int>   driftT_vector;//adjusted drift temperature vector
};


double dfr97 (const int yp, const int mp, const int dp,const int hr, const int minutes, const int secs);
long ymd2julday (int, int, int);
char  *strupr(char* s);
int binchrstr2dec(char *binvalue);
void *getsubstr(int begin, int end, char *s, char *ss);
char chr_xor(char x, char y);
int twoscompl2dec(char *binvalue);
int get_meta_file_name (char *str_name);
int crc_test(char *binstr);

//added functions
const char* string2char(string str);
string chararray2string(char a[],int begin,int length);
int bin2int_unsigned(string  bin);
string bitInv(string input);
string hex2bin(string hex);
string int2bin(int input);
string int2bin12bits(int input);

//these functions are specific to T4
void getUniqueHexData(vector<int> start_row,vector<int> end_row,int fileno,vector<int> messtype,int message_type,vector<string> hexdata,vector<string>& uniqueHexdata,vector<int>& uniqueHexdataIndex);
int hasDriftData(vector<int> start_row,vector<int> end_row,int fileno,vector<int> messtype,int message_type_to_process,
                 vector<string> hexdata,vector<string> uniqueHexdata,vector<int> uniqueHexdataIndex);
int read_t4_meta_file(char *meta_name, struct t4_meta_struct &meta_struct);
int write_to_disk(char* argv[],struct t4_technical_struct ts,struct t4_meta_struct ms,
                  struct position_info_struct ps,struct driftPT_struct ds,
		  vector<int>& P_Vector,vector<int>& T_Vector,
		  vector<int>& CRC_Vector,int prof_no);
void decodeT4(char *argv[],char* pathname_input_filename, struct t4_meta_struct& ms);
int getCRC(char *data, int count);
int getMedian(vector<int> input,int window);
void correctOverflowT(vector<int>& dt,int thresh);
void correctOverflowC(vector<int>& dc,int thresh);
int most_likely_frame(vector<int> messtype,vector<string> hexdata,
                      vector<int> start_row,vector<int> end_row,
                      int fileno,vector<int> crc_status,int message_type,int message_num,int good_bad);
void decodeMessageType0(struct t4_technical_struct& ts,vector<int> messtype,
                        vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,
			vector<int> crc_status,struct t4_meta_struct ms,
			vector<int> year_ps,vector<int> month_ps,
                        vector<int> day_ps,vector<int> hour_ps,vector<int> minute_ps,vector<int> second_ps);
void decodeMessageType13(int message_type,vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& CRC_Vector,
                        struct t4_technical_struct ts,struct t4_meta_struct ms,vector<int> messtype,
			vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,vector<int> crc_status);
void  decodeMessageType2(struct driftPT_struct& ds,struct t4_technical_struct ts,struct t4_meta_struct ms,
                         vector<int> messtype,
                         vector<string> hexdata,vector<int> start_row,vector<int> end_row,
			 int fileno,vector<int> crc_status);       
void  resetVectors(vector<int>& PA_Vector,vector<int>& TA_Vector,vector<int>& CRCA_Vector,
                   vector<int>& PD_Vector,vector<int>& TD_Vector,vector<int>& CRCD_Vector,
                   struct position_info_struct& ps,struct driftPT_struct& ds);
void read_position(struct position_info_struct& ps,vector<int> year_ps,vector<int> month_ps,
                   vector<int> day_ps,vector<int> hour_ps,vector<int> minute_ps,vector<int> second_ps, 
		   vector<float> lat_ps,vector<float> lon_ps,vector<float> nl_ps,vector<char> sattype_ps,
		   vector<char> lc_ps,int fileno,vector<int> start_row,vector<int> end_row,struct t4_meta_struct ms);
void  cleanStartRowEndRow(vector<int>& start_row,vector<int>& end_row);			
void adjustStartRowEndRow(vector<int>& start_row,vector<int> end_row,vector<float> jdayarray,struct t4_meta_struct ms);
int getLastNonzeroCharIndex(string hexString);
void printMarker(int i);
 
int main(int argc, char *argv[])
{

  char *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
       *ptr_path_log, *ptr_path_list, buffer[512],
       pathname_input_filename[161]="",
       pathname_input_meta_filename[161]="";

  char input_filename[80];
  char output_directory[256];
  enum {LAST=0,ALL} get_which_profile;

  long t;

  int      ret_val = 0;
  struct   t4_meta_struct ms;


  // Program Name
  if (argc < 7)
  {
    printf("Program generates temperature, conductivity and pressure counts for SOLO T4 Floats\n");
    printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
           PRGNAME);
    printf("\n");
    printf("hex_infile:    hexadecimal input file name\n");
    printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
    printf("cnt_outdir:    output directory for generated count data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

  sprintf (input_filename,"%s",argv[1]);

  // Number of profiles to process: 0, get last profile or 1, get all profiles
  if (atoi(argv[2]) == 1)
    get_which_profile = ALL;
  else
    get_which_profile = LAST;

  // Output directory path for count file(s)
  // ---------------------------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  sprintf (output_directory,"%s/",argv[3]);

  // TRACE File
  // ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  sprintf (buffer,"%s",argv[4]);
  printf ("%s:Opening trace file %s\n",PRGNAME,buffer);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nError opening trace file %s",buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("%s: Trace file opened succesfully\n",PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

  // ERROR File
  // ----------
  sprintf (buffer,"%s",argv[5]);

  printf ("%s:Opening error file %s",PRGNAME,buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("Error(%s): opening error file %s\n",PRGNAME,buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("\nOK . error file opened\n");

  if (trace_file != NULL)
    fprintf (trace_file,"%s: Error file %s successfully opened\n",PRGNAME,buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }



  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[6]);

  printf ("\nOpening list file %s",buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nError opening list file %s\n",buffer);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Error opening list file %s\n",
               PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Error opening list file %s\n",
              PRGNAME, buffer);

    ret_val =1;
    return ret_val;
  }

  printf("\nOK . list file opened\n");

  if (trace_file != NULL)
    fprintf (trace_file," list file %s successfully opened\n", buffer);

  // INPUT Hex File
  // --------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): HEXFILES_DIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: HEXFILES_DIR <-- no such environment variable\n");

    if (error_file != NULL)
      fprintf (error_file," ERROR: HEXFILES_DIR <-- no such environment variable\n");

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);

    if (error_file != NULL)
      fclose (error_file);

    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {                                           // Fail to open Hex file
    printf ("\nError opening input file %s", pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: input file %s not opened\n", pathname_input_filename);

    if (error_file != NULL)
      fprintf (error_file," ERROR: input file %s not opened\n", pathname_input_filename);

    ret_val = 1;
  }
  else                                        // Success: Hex file opened
  {
    printf ("\nInput file %s successfully opened", pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file," Input file %s successfully opened\n", pathname_input_filename);

    // META File
    // ---------
    if (get_meta_file_name (input_filename) != 0)
    {                                              // Failure: couldn't get meta filename
      printf ("\nError finding meta file name");

      if (error_file != NULL)
        fprintf (error_file, " ERROR: meta file name construction - returned name %s\n",
                 input_filename);

      if (trace_file != NULL)
        fprintf (trace_file, " ERROR: meta file name construction - returned name %s\n",
                 input_filename);

      ret_val = 1;
    }
    else
    {                                              // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("METAFILES_DIR <-- no such environment variable\n");

        if (trace_file != NULL)
          fprintf (trace_file," ERROR: METAFILES_DIR <-- no such environment variable\n");

        if (error_file != NULL)
          fprintf (error_file," ERROR: METAFILES_DIR <-- no such environment variable\n");

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);

        if (error_file != NULL)
          fclose (error_file);

        ret_val = 1;
        return(ret_val);
      } // endif: ptr_path_meta

      sprintf(pathname_input_meta_filename, "%s/%s", ptr_path_meta, input_filename);
      
      
      if (read_t4_meta_file (pathname_input_meta_filename, ms) != 0)
      {                                         // Failure: couldn't read meta_file
        printf ("\nError reading meta file");

        if (error_file != NULL)
          fprintf (error_file," ERROR reading meta file %s\n", pathname_input_meta_filename);

        if (error_file != NULL)
          fprintf (error_file," ERROR reading meta file %s\n", pathname_input_meta_filename);

        ret_val = 1;
      }
      else                                      // Success: read meta_file
      {
        if (trace_file != NULL)
          fprintf (trace_file," Meta file %s successfully read\n", pathname_input_meta_filename);
                     
	// Make profile(s) selection
	decodeT4(argv,pathname_input_filename,ms);

      } // endif: read_type14_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file (open hex file)

  // Write finish message to trace & error files
  // -------------------------------------------
  t = time(NULL);
  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: finished %s\n\n", PRGNAME, ctime(&t));

// Close data files
// ----------------

  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
   fclose (list_file);

  return(ret_val);

}






/**********************************************************************
                          Read meta data file
***************************************************************************/

int read_t4_meta_file (char *meta_name, struct t4_meta_struct &meta_struct)
{
  FILE *meta_file;
  int i,j,syear,smonth,sday,shour,smin;
  float x,x1,lat,latm,lon,lonm;
  int lyear,lmonth,lday,lhour,lmin;
  char *ptr;
  char buffer[512];
  
  char manufacturer[20];
  
  char *cal_keys[] = {"TGAIN=", "TOFF_A=","TOFF_R=","PGAIN=", "POFF=","\0"};

  meta_struct.cycle_time = (float) 0.0;
  meta_struct.start_cycle_time = (float) 0.0;

  printf("\nOpening meta file!!!!!! %s\n",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("Error opening meta file!!!!!! %s\n",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (trace_file != NULL)
    fprintf (meta_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      // Read chars from buffer and store them in AOML_number inside
      // structure.
      //
      // Note: * means the field does not need to be included in vector of
      // variable names to write to
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.AOML_number) != 1)
        meta_struct.AOML_number = 0;
      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.WMO_id) != 1)
        meta_struct.WMO_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.ARGOS_id) != 1)
        meta_struct.ARGOS_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x);
      meta_struct.start_cycle_time += (float) (x/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct.wmo_instrument) != 1)
      meta_struct.wmo_instrument = 0;
     continue;
    }

   //newly added sensor manufacturer 
    if (strstr(strupr(buffer),"TEMPERATURE SENSOR MANUFACTURER") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %s",&manufacturer) != 1)
      sprintf (manufacturer,"\0");
     continue;
    }
    

    if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct.wmo_recorder) != 1)
      meta_struct.wmo_recorder = 0;

	 continue;
    }


// DOWN TIME was changed to CYCLE TIME in meta files (CS)
    if (strstr(strupr(buffer),"CYCLE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x1);
      meta_struct.cycle_time = x1;
      continue;
    }

// UP TIME was changed to SURFACE TIME in meta files (CS)
    if (strstr(strupr(buffer),"SURFACE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f %*s",&x);
      meta_struct.start_cycle_time -= (float) (x/24.0);
      continue;
    }

    // start_cycle_time = FIRST DOWN TIME - SURFACE TIME + START TIME
    //            ^                 ^
    //            6 hours           1 day     dfr97 days
    if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &sday,&smonth,&syear,&shour,&smin);

      meta_struct.start_cycle_time += (float) (dfr97(syear,smonth,sday,shour,smin,0));
      continue;
    }

    if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.exp_no) != 1)
        meta_struct.exp_no = 0;

      continue;
    }
    //newly added hexadecimal ARGOS ID field 
    if (strstr(strupr(buffer),"HEXADECIMAL ARGOS ID") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %s",&meta_struct.hex_id) != 1)
      sprintf (meta_struct.hex_id,"\0");
     continue;
    }
     

    if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
    {
      if (sscanf(buffer,"%*s %*s %s",meta_struct.float_type) == 0)
      {
        sprintf (meta_struct.float_type,"\0");
        continue;
      }
    }

    if (strstr(strupr(buffer),"PI     ") != NULL)
    {
      if (sscanf(buffer,"%*s %s",meta_struct.PI) == 0)
      {
        sprintf (meta_struct.PI,"\0");
        continue;
      }

      if ((ptr = strstr(&buffer[3],meta_struct.PI)) != NULL)
        sprintf (meta_struct.PI,"%s",ptr);

      if ((ptr = strstr(meta_struct.PI,"\n")) != NULL)
        sprintf (ptr,"\0");

      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
           &lday,&lmonth,&lyear,&lhour,&lmin);
      meta_struct.launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0));

      meta_struct.year=lyear;
      meta_struct.month=lmonth;
      meta_struct.day=lday;
      meta_struct.hour=lhour;
      meta_struct.minute=lmin;
      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH POSITION") != NULL) && (strstr(strupr(buffer),"LONM") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f %f %f %f",&lat,&latm,&lon,&lonm);
      meta_struct.lat=lat;
      meta_struct.latm=latm;
      meta_struct.lon=lon;
      meta_struct.lonm=lonm;
      continue;
    }


    i = 0;
    while ((j = strlen(cal_keys[i])) > 1)
    { 
      
      // Get pointer to 1st occurence of cal_keys[i] in buffer
      if ((ptr = (char *)strstr(buffer,cal_keys[i])) != NULL){
        meta_struct.cals.c[i] = atof((const char *)(ptr+j));	
	//if(debug==1) cout<<"meta coefficents="<<meta_struct.cals.c[i]<<endl;
      }
      i++;
    }

  }
  
  if(debug==1){    
     for(i=0;i<4;i++){
       cout<<"meta coefficents="<<meta_struct.cals.c[i]<<endl;       
     }  
  }
  
  if(debug==1) cout<<"Read Meta data file finished"<<endl;
  
  return 0;
}




/******************************************************************************************
 Decode T4 float to counts, counts are stored in PU,TU,CU vectors with duplicated removed
 Considered both Up and Down profile cases(Key part of Decoding)
*******************************************************************************************/

void decodeT4(char *argv[], char* pathname_input_filename, struct t4_meta_struct& ms){
  FILE  *fpprofile;
  vector<int> PA_Vector,TA_Vector,CRCA_Vector; //ascending
  vector<int> PD_Vector,TD_Vector,CRCD_Vector; //descending  
  struct t4_technical_struct ts;
  struct position_info_struct ps;
  struct driftPT_struct ds;
  
  /* The following arrays store the data related to the profile, 
     later they are filtered to only get positional data  
  */
  vector<int> year_ps,month_ps,day_ps,hour_ps,minute_ps,second_ps;
  vector<float> lat_ps,lon_ps,nl_ps;
  vector<char>  sattype_ps,lc_ps;
  vector<float>  jdayarray;
  vector<int> start_row,end_row;//the start_row and end_row of each profile are stored here
  vector<int> start_row_ps,end_row_ps;//the start_row and end_row of position information for each profile are stored here  
  vector<int> messtype;//message type for each hexdata line
  vector<int> crc_status;//1 means passed crc check, 4 means doesnot pass crc check,0 means does not apply  
  vector<string> hexdata;//hex data,used for comparision in most_likely_frame function      
  
  vector<string> uniqueHexdata;
  vector<int> uniqueHexdataIndex;
  
  char hexrecord[HEXRECLEN]="", binrecord[BINRECLEN]="";
  char hexrecord_tmp[HEXRECLEN]="",hex_partial[HEXRECLEN]="";  
  char ss[35]="",sat,l_c;
  char* out_dir;
  
  char twozeros[3]="00";//preappend two zero to hexdata if ARGOS ID>=32768  
  int  year, month, day, hour, minute, second,n_l;    
  int  i,j,k,cntr=0;
  int fileno=-1;
  int nprofile,index;
  int prof_no=0;//profile number(current cycle)
  int jd,old_jd;//julian day  
  int total_cycles;//total cycles  
  // message num and maximum message number, 
  // most common message index(from most_likely_frame function),
  // bin number of T,C for profile data per profile
  float latitude,longitude,jdaydiff=0;  
  int message_type_to_process;
    
  out_dir=argv[3];  
  //get which profile, 1--get all profile, 0--get last profile only
  if(atoi(argv[2])>=1) nprofile=1;else nprofile=0;

  // Open Profile File
  if(debug==1)printf("PROFILE WHOLE PATH=%s\n",pathname_input_filename);  
  if ((fpprofile = fopen(pathname_input_filename, "r")) == NULL)
  {
    printf (" >>> Profile Open Error <<<\n");
    //exit(1);
  }

  //
  //  Read the whole hex files into vectors
  //
  while (fscanf(fpprofile,"%*s %d %d %d %d %d %d %f %f %*s %d %c %c %*s %s",
         &year, &month, &day, &hour, &minute, &second,&latitude,&longitude,&n_l,&sat,&l_c, hexrecord_tmp) != EOF)
  {
    if (strlen(hexrecord_tmp) != 64) {
      if (debug==1) {
        printf(" not good enough=%s\n",hexrecord_tmp);
      }
      continue;
    }

    // if ARGOS ID >=32768, shift hex data string to the right by one byte.
    hexrecord[0]='\0';//every loop,reset hexrecord,otherwise, it will keep strcating.
    if(ms.ARGOS_id<32768){
      strcpy(hexrecord,hexrecord_tmp);      
    }else{
      strcat(hexrecord,twozeros);
      getsubstr(0,61,hexrecord_tmp,hex_partial);
      strcat(hexrecord,hex_partial);
    }

    strcpy(binrecord,string2char(hex2bin(chararray2string(hexrecord,0,64))));

    if(debug==1) {
      // printf(" cntr=%d binary=%s\n",cntr,binrecord);
      // printf(" cntr=%d hexdat=%s\n",cntr,hexrecord);
    }    

    // CRC test,refer to hex2cnt_ts16.cpp and ts11.cpp,need to verify it is correct!
    // only use good crc data
    if (crc_test(binrecord) != 0){//crc failed
      continue;
    }else{
      crc_status.push_back(1);
    }
/*
    if (crc_test(binrecord) != 0){//crc failed
      crc_status.push_back(4);
    }else{//crc succeeded
      crc_status.push_back(1);
    }
*/    
    //newly added for use in most_likely_frame function    
    hexdata.push_back(chararray2string(hexrecord,0,64));
    
    lat_ps.push_back(latitude);
    lon_ps.push_back(longitude);
    nl_ps.push_back(n_l);
    sattype_ps.push_back(sat);
    lc_ps.push_back(l_c);
    
    year_ps.push_back(year);
    month_ps.push_back(month);
    day_ps.push_back(day);
    hour_ps.push_back(hour);
    minute_ps.push_back(minute);
    second_ps.push_back(second);
    jdayarray.push_back((float)dfr97(year,month,day,hour,minute,second));

    //get message type for each hexline
    
    getsubstr(8,11,binrecord,ss);
    messtype.push_back(binchrstr2dec(ss));

    cntr++;        
  }

  if(debug==1){
   cout<<"length of vectors"<<endl;
   /*
   cout<<lat_ps.size()<<" "<<lon_ps.size()<<" "<<nl_ps.size()<<" "<<sattype_ps.size()<<" "<<lc_ps.size()
   <<" "<<year_ps.size()<<" "<<month_ps.size()<<" "<<day_ps.size()<<" "<<hour_ps.size()<<" "<<minute_ps.size()
   <<" "<<second_ps.size()<<" "<<jdayarray.size()<<" "<<messtype.size()<<" "
   <<" "<<hexdata.size()<<endl;
   */
   
   cout<<"non hex data"<<endl;
   /*
   for(i=0;i<lat_ps.size();i++){
     cout<<"i="<<i<<" messtype="<<messtype[i]<<" "<<year_ps[i]<<" "<<month_ps[i]
     <<" "<<day_ps[i]<<" "<<hour_ps[i]<<" "<<minute_ps[i]<<" "<<second_ps[i]<<" "<<lat_ps[i]<<" "<<lon_ps[i]<<endl;
   }
   */
  }

  //
  // get start_row and end_row number for each profile
  // a new profile happens when the is more than 2 days between two
  // lines
  //
  start_row.push_back(0);

  for(i=0;i<cntr;i++){
   if(i==0){jdaydiff=0;}
   else{
   jdaydiff=jdayarray[i]-jdayarray[i-1];
   }
   if(jdaydiff>2){
    start_row.push_back(i);
   }
  }

  for(index=1;index<start_row.size();index++){
     end_row.push_back(start_row[index]-1);
  }
  end_row.push_back(cntr-1);//last row number

  //display start_row and end_row to see if they are correct   
  if(debug==1){
    for(j=0;j<start_row.size();j++){
      cout<<"before adjustment,fileno= "<<j<<" start_row= "<<start_row[j]<<" "<<"end_row= "<<end_row[j]<<endl;
    }
  }
  //clean ghost messages/cycles, i.e. if a cycle only contain one line, skip it,
  //remove it from start_row,end_row vectors to make them clean
  cleanStartRowEndRow(start_row,end_row);
  
  //adjust start_row's line number based on launch_time, if in one cycle, the date is larger
  //than launch_time, then record the first row when this happens and reset this cycle's start_row
  //value to this recorded row number. This will make sure that we do not skip any profile with 
  //valid data in it.
  
  adjustStartRowEndRow(start_row,end_row,jdayarray,ms);
  //display start_row and end_row to see if they are correct   
  if(debug==1){
    for(j=0;j<start_row.size();j++){
    cout<<"after adjustment,fileno= "<<j<<" start_row= "<<start_row[j]<<" "<<"end_row= "<<end_row[j]<<endl;
    }
  }
  total_cycles=start_row.size();
  if(debug==1) {
   cout<<"total_cycles="<<total_cycles<<" cycle_time="<<ms.cycle_time<<endl;
  }    
/* ------------seperate profiles and generate counts, Up profile core processing --------------------------*/
  old_jd=(int)ms.start_cycle_time;
  
  message_type_to_process=2;
  fileno=0;
  prof_no=0;
/*  
  //decide if the hex data has profile 0 or not
  //by checking whetehr fileno 0 has drift data or not, if it has, then it is profile 1, otherwise, it is profile 0 to begin with

  if(hasDriftData(start_row,end_row,fileno,messtype,message_type_to_process,hexdata,uniqueHexdata,uniqueHexdataIndex)==0){		   
    prof_no=0;
  }else{
    prof_no=1;
  }
*/  
  for(k=0;k<total_cycles;k++){
    //for each cycle
   
    if(jdayarray[start_row[k]]>=ms.launch_time){
      fileno=k;//need to compute fileno.          
     
      jd=(int)dfr97(year_ps[start_row[k]],month_ps[start_row[k]],day_ps[start_row[k]],0,0,0);	              
      prof_no=prof_no+(int)((jd-old_jd)/ms.cycle_time+0.5);
      if(debug==1) {
        cout<<"k="<<fileno<<" prof_no="<<prof_no<<" jd="<<jd<<" start_cycle_time="<<ms.start_cycle_time<<endl;     
      }

      old_jd=jd;

      if(nprofile==1||(nprofile==0 && k==total_cycles-1)){//all of the profile
        //only output to file if the julian day is larger than deployment time 
        if(jdayarray[start_row[fileno]]>=ms.launch_time){

          //technical engineering data         
          decodeMessageType0(ts,messtype,hexdata,start_row,end_row,fileno,crc_status,ms,year_ps,month_ps,day_ps,
                             hour_ps,minute_ps,second_ps);      

          //message type 1: profile data descending
          message_type_to_process=1;	   
          decodeMessageType13(message_type_to_process,PD_Vector,TD_Vector,CRCD_Vector,ts,ms,messtype,hexdata,
                              start_row,end_row,fileno,crc_status);              

          //message type 3: profile data ascending
          message_type_to_process=3;
          decodeMessageType13(message_type_to_process,PA_Vector,TA_Vector,CRCA_Vector,ts,ms,messtype,hexdata,
                              start_row,end_row,fileno,crc_status);                 	   

          //message type 2: drift data
          decodeMessageType2(ds,ts,ms,messtype,hexdata,start_row,end_row,fileno,crc_status);

          if(debug==1) cout<<"Writing to output file no="<<prof_no<<endl; 
	   
          //if there are ascending profile data, print to output file
          if(PA_Vector.size()>0) {
            read_position(ps,year_ps,month_ps,day_ps,hour_ps,minute_ps,second_ps,
                          lat_ps,lon_ps,nl_ps,sattype_ps,lc_ps,fileno,start_row,end_row,ms);
	     
            //if it has drift data but prof_no=0,then prof_no should be 1, no profile 0		  
            //if(ds.driftT_vector.size()>0 && prof_no==0) prof_no=1;
            write_to_disk(argv,ts,ms,ps,ds,PA_Vector,TA_Vector,CRCA_Vector,prof_no);	   	    
            PA_Vector.clear();
            TA_Vector.clear();
            CRCA_Vector.clear();
            ps.year_vector.clear();
            ps.month_vector.clear();
            ps.day_vector.clear();
            ps.hour_vector.clear();
            ps.min_vector.clear();
            ps.sec_vector.clear();
            ps.xmits_vector.clear();
            ps.sat_vector.clear();
            ps.class_vector.clear();
            ps.lon_vector.clear();
            ps.lat_vector.clear();
          }

          //if there are descending profile data, print to output file
          if(debug==1) {
            cout<<"BEFORE 1001 file="<<prof_no<<PD_Vector.size()<<endl; 
          }
          if(PD_Vector.size()>0) {
            if(debug==1) {
              cout<<"1001 file="<<prof_no<<PD_Vector.size()<<endl; 
            }
            read_position(ps,year_ps,month_ps,day_ps,hour_ps,minute_ps,second_ps,
                          lat_ps,lon_ps,nl_ps,sattype_ps,lc_ps,fileno-1,start_row,end_row,ms);

            //if it has drift data but prof_no=0,then prof_no should be 1, no profile 0			  
            //if(ds.driftT_vector.size()>0 && prof_no==0) prof_no=1;
            write_to_disk(argv,ts,ms,ps,ds,PD_Vector,TD_Vector,CRCD_Vector,prof_no+1000);	   	    
            PD_Vector.clear();
            TD_Vector.clear();
            CRCD_Vector.clear();
            ps.year_vector.clear();
            ps.month_vector.clear();
            ps.day_vector.clear();
            ps.hour_vector.clear();
            ps.min_vector.clear();
            ps.sec_vector.clear();
            ps.xmits_vector.clear();
            ps.sat_vector.clear();
            ps.class_vector.clear();
            ps.lon_vector.clear();
            ps.lat_vector.clear();
          }
        }
      }//end of if(nprofile==1||)
     
    //reset or clear the vectors, to reuse them in next iteration step
    resetVectors(PA_Vector,TA_Vector,CRCA_Vector,PD_Vector,TD_Vector,CRCD_Vector,ps,ds);
 }//end of if >=ms.launch_time        
}//end of for(k=0)

}//end of void decodet4 function




/*******************************************************************************************
                          Write output result to file
********************************************************************************************/
int write_to_disk(char *argv[],struct t4_technical_struct ts,struct t4_meta_struct ms, struct position_info_struct ps,struct driftPT_struct ds,vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& CRC_Vector,int prof_no){

  char buffer[512];
  char* dir_name;
  char* ptr_cnt_filename;
  char date_time[32];
  int j;
  vector<int> PU_Vector,TU_Vector,CRCU_Vector;//unique P,T,CRC vector
  int actual_drift_samples,max_rows_drift_samples,row;   
  int max_samples_per_row=10;//drift data values allowed to print on one row
  
  dir_name=argv[3];
  
  sprintf (buffer,"%s/%04d_%05d_%03d.cnt",dir_name,ms.AOML_number,ms.ARGOS_id,prof_no);
  cout<<"BUFFER -> Printing result to outputfile: "<<prof_no<<"output filename="<<buffer<<endl;
  
 if (trace_file != NULL)
    fprintf (trace_file,"Opening output file %s...\n",buffer);


 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  //cout<<"output filename can not be created"<<endl;
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }



 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);



// Write Output Profile Count File Name to vector File
// -------------------------------------------------
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s : Canot get cnt file basename %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Canot get cnt file basename %s\n", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename

  
  
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);

 if (trace_file != NULL)
 fprintf(trace_file,"NAME TO LIST -> ptr_cnt_filename: %s\n", ptr_cnt_filename);
 
 
  // Write header info to buffer
  sprintf (buffer,"INTERNAL ID NUMBER                      %04d\n"
                  "WMO ID NUMBER                           %d\n"
                  "TRANSMISSION ID NUMBER                  %05d\n"
                  "PROFILE NUMBER                          %d\n"
                  "INSTRUMENT TYPE                         %s\n",
            ms.AOML_number,abs(ms.WMO_id),ms.ARGOS_id,prof_no,ms.float_type);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
                  "WMO RECORDER TYPE (TABLE 4770)          %d\n"
                  "ARGOS PROGRAM NUMBER                    %d\n"
                  "PI                                      %s\n",
           ms.wmo_instrument, ms.wmo_recorder, ms.exp_no, ms.PI);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  sprintf(buffer, "DESCENT START TIME (HH:MM)              %02d:%02d\n"
                  "NUMBER OF VALVE ACTIONS AT SURFACE      %d\n"
		  "FLOAT STABILISATION TIME (HH:MM)        %02d:%02d\n"
		  "FLOAT STABILISATION PRESSURE (COUNTS)   %d\n",		  
  ts.descent_start_time_hh,ts.descent_start_time_mm,ts.number_of_valve_actions_at_surface,
  ts.float_stable_time_hh,ts.float_stable_time_mm,ts.float_stable_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "NUMBER OF VALVE ACTIONS IN DESCENT      %d\n"
                  "NUMBER OF PUMP ACTIONS IN DESCENT       %d\n"
		  "END OF DESCENT TIME (HH:MM)             %02d:%02d\n"
		  "NUMBER OF REPOSITIONS                   %d\n",
  ts.number_of_valve_actions_descent,ts.number_of_pump_actions_descent,
  ts.end_of_descent_time_hh,ts.end_of_descent_time_mm,ts.number_of_repositions);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);


  sprintf(buffer, "TIME AT END OF ASCENT (HH:MM)           %02d:%02d\n"
                  "NUMBER OF PUMP ACTIONS IN ASCENT        %d\n"
		  "DURATION OF PUMP ACTIONS AT SURFACE     %d\n"
		  "NUMBER OF DESCENT PT MESSAGES           %d\n",
  ts.time_at_end_ascent_hh,ts.time_at_end_ascent_mm,ts.number_of_pump_actions_ascent,
  ts.duration_of_pump_actions_surface,ts.number_of_descent_pt_messages);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  

  sprintf(buffer, "NUMBER OF DRIFT PT MESSAGES             %d\n"
                  "NUMBER OF ASCENT PT MESSAGES            %d\n"
		  "SHALLOW/DEEP BOUNDARY ISOBAR FOR DESC.  %d\n"
		  "NUMBER OF DESC. SLICES IN SHALLOW ZONE  %d\n", 
  ts.number_of_drift_pt_messages,ts.number_of_ascent_pt_messages,
  ts.shallow_deep_boundary_isobar_descent,ts.number_of_descent_slices_shallow_zone);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);


  sprintf(buffer, "NUMBER OF DESCENT SLICES IN DEEP ZONE   %d\n"
                  "SHALLOW/DEEP BOUNDARY ISOBAR FOR ASCENT %d\n"
		  "NUMBER OF ASCENT SLICES IN SHALLOW ZONE %d\n"
		  "NUMBER OF ASCENT SLICES IN DEEP ZONE    %d\n",  
  ts.number_of_descent_slices_deep_zone,ts.shallow_deep_boundary_isobar_ascent,
  ts.number_of_ascent_slices_shallow_zone,ts.number_of_ascent_slices_deep_zone);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "NUMBER OF PT MEASUREMENTS IN DRIFT      %d\n"
                  "FLOAT'S TIME (HH:MM:SS)                 %02d:%02d:%02d\n"
		  "TIME FROM SATELLITE (HH:MM:SS)          %02d:%02d:%02d\n"
           "SURFACE PRES OFFSET APPLIED 1CBAR(CNTS) %d\n"
		  "INTERNAL PRESSURE (COUNTS)              %d\n",
  ts.number_of_PT_measurements_in_drift,ts.floats_time_hh,ts.floats_time_mm,ts.floats_time_ss,
  ts.time_from_satellite_hh,ts.time_from_satellite_mm,ts.time_from_satellite_ss,
  ts.pressure_sensor_offset,ts.internal_pressure);  
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "SURFACE TEMPERATURE (COUNTS)            %d\n"
                  "PROFILE ASCENT START TIME (HH:MM)       %02d:%02d\n"
		  "NUMBER OF ENTRANCE IN TARGET RANGE      %d\n"
		  "MINIMUM PRESSURE IN DRIFT (BAR)         %d\n",
  ts.surface_temperature,ts.profile_ascent_start_time_hh,ts.profile_ascent_start_time_mm,
  ts.number_of_entrance_in_target_range,ts.minimum_pressure_in_drift);  
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "MAXIMUM PRESSURE IN DRIFT (BAR)         %d\n"
                  "GROUNDING                               %d\n", 
  ts.maximum_pressure_in_drift,ts.grounding);  
  fwrite (strupr(buffer),1,strlen(buffer),out_file);


  //print the drift Pressure and Temperature
  actual_drift_samples=min(ds.driftT_vector.size(),ds.driftP_vector.size());
  if(debug==1) printf("actual drift samples length=%d\n",actual_drift_samples);
  max_rows_drift_samples=(int) ceil((float)actual_drift_samples/max_samples_per_row);
  if(debug==1) printf("maximum rows of drift samples=%d\n",max_rows_drift_samples);
 
 if(ds.driftP_vector.size()>0){ 
  for(row=0;row<max_rows_drift_samples;row++){
    sprintf (buffer, "DRIFT PRESSURE (COUNTS)                 ");
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
    for(j=0;j<max_samples_per_row;j++){
       if(row*max_samples_per_row+j>=actual_drift_samples) break;
       sprintf(buffer,"%-6d ",ds.driftP_vector[row*max_samples_per_row+j]);       
       fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
    sprintf(buffer,"\n");  
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
 }
 
 if(ds.driftT_vector.size()>0){  
  for(row=0;row<max_rows_drift_samples;row++){
    sprintf (buffer, "DRIFT TEMPERATURE %d (COUNTS)            ",row+1);
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
    for(j=0;j<max_samples_per_row;j++){
       if(row*max_samples_per_row+j>=actual_drift_samples) break;
       sprintf(buffer,"%-6d ",ds.driftT_vector[row*max_samples_per_row+j]);       
       fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
    sprintf(buffer,"\n");  
    fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
 }
 
  sprintf (buffer,"NUMBER OF COLUMNS                       3\n"
                  "1. COLUMN                               PRESSURE (COUNTS)\n"
                  "2. COLUMN                               TEMPERATURE (COUNTS)\n"
                  "3. COLUMN                               CHECKSUM FLAG\n");
 
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  int i = 0;
  sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  for(i=0;i<ps.sat_vector.size();i++)
  {
    //           "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
    sprintf(date_time,"%4d/%02d/%02d%3d:%02d:%02d",ps.year_vector[i],ps.month_vector[i],ps.day_vector[i],ps.hour_vector[i],ps.min_vector[i],ps.sec_vector[i]);

    sprintf (buffer,"    %+7.3f   %+8.3f %s   %03d   %1c     %1c\n",
             ps.lat_vector[i], ps.lon_vector[i],date_time,
             ps.xmits_vector[i], ps.sat_vector[i],ps.class_vector[i]);

    fwrite (strupr(buffer),1,strlen(buffer),out_file);

  }


  fprintf (out_file,"==========================================================");

  // Write depth, temperature, and conductivity or salinity to disk
/*
   //remove duplicate data
    if(P_Vector.size()>0){
         PU_Vector.push_back(P_Vector[0]);
         TU_Vector.push_back(T_Vector[0]);
         CRCU_Vector.push_back(CRC_Vector[0]);
         
         for(i=0;i<P_Vector.size()-1;i++){
            if(P_Vector[i+1]!=P_Vector[i]){
               PU_Vector.push_back(P_Vector[i+1]);
               TU_Vector.push_back(T_Vector[i+1]);
               CRCU_Vector.push_back(CRC_Vector[i+1]);            
            }
         }
    }//end of if(P_Vector.size()>0)    
*/
   if(debug==1) {
     cout<<"P_Vector Size in write routine "<<P_Vector.size()<<endl;
     cout<<"T_Vector Size in write routine "<<T_Vector.size()<<endl;
   }
    int first_cycle_with_data=0;
    for (i=(int)T_Vector.size()-1; i>=0; i--)
    { 
      if(T_Vector[i]!=0 && P_Vector[i]!=0) first_cycle_with_data=1;
      if(first_cycle_with_data==1){
        sprintf (buffer,"\n %5d %5d  %d",P_Vector[i],T_Vector[i],CRC_Vector[i]);
        fwrite (strupr(buffer),1,strlen(buffer),out_file);
      }
    }
    fprintf(out_file,"\n");

// Close output file

    if (trace_file != NULL)
      fprintf (trace_file," WRITE_SIO_TO_DISK successful\n");
  if(out_file!=NULL)
  fclose(out_file);
/*
  PU_Vector.clear();
  TU_Vector.clear();
  CRCU_Vector.clear();
*/
  return(0);
}





/******************************************************************************
  Computes days from 1997
******************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
  int    yr, mo, dy, day_from_97;
  long   passed_jd, jd_xx0101, jd_970101;
  double d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
   yr += 2000;
  else if (yr < 1900)
   yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
   return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year)
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
  day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs,
                                                // mins, secs)
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
  return (d_jd);                                // and adds this to d_jd
}

/***************************************************
Convert year,month,day to julian day
*****************************************************/

long ymd2julday(int year, int month, int day)
{
   int  ic, iy;
   long julday;

   if ( month > 2 )
     month = month - 3;
   else if ( month < 3 )
   {
     month = month + 9;
     year = year - 1;
   }

   ic = (int) floor((double)(year/100) );
   iy = year - 100*ic;

// printf("ic: %d iy: %d\n", ic, iy);

   julday =   (long) floor((double)(146097*ic)/4)
             + (long) floor((double)(1461*iy)/4)
             + (long) floor((double)(153*month+2)/5)
             + day + 1721119;
   julday = julday - 2440000;
   return julday;
}



/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char  *strupr(char* s)
{
  char* p = s;
  while (*p) {
    if (islower((int)*p)) *p = toupper((int)*p);
    ++p;
  }
  return s;
}


/****************************************************************************
convert binary string to decimal values
***************************************************************************/

int binchrstr2dec(char *binvalue)
{
  int len, i, j,
      iplace, accum=0, pwof2;
  char tmpchr[2]="";

  len=strlen(binvalue);

  for (i=len; i>0; i--)
  {
    tmpchr[0]=binvalue[i-1];
    tmpchr[1]='\0';

    iplace=atoi(tmpchr);

    pwof2=1;

    for (j=0; j<len-i; j++)
      pwof2*=2;

    accum+=iplace*pwof2;
  }

  return(accum);
}


/**********************************************************************************
get substring of a string s,from index begin to end, result is stored in string ss
***********************************************************************************/
void *getsubstr(int begin, int end, char *s, char *ss)
{
  int j;

  for (j=begin; j<=end; j++)
    ss[j-begin]=s[j];

  ss[end-begin+1]='\0';
}




/************************************************************************************
character xor operation z=x (xor) y
*************************************************************************************/
char chr_xor(char x, char y)
{
  char z;
  z=(x==y)?'0':'1';
  return z;
}






/**************************************************************************************
convert two's complement value of a string binvalue to decimal values
***************************************************************************************/

int twoscompl2dec(char *binstr)
{
  int decvalue, len, i;
  char binvalue[256]="";
  
  strcpy(binvalue,binstr);
  len=strlen(binvalue);

  if (binvalue[0]=='1')
  {
    for (i=0; i<len; i++)
      binvalue[i]=chr_xor(binvalue[i],'1');

    decvalue=binchrstr2dec(binvalue);
    decvalue++;
    decvalue*=-1;
  }
  else if (binvalue[0]=='0')
    decvalue=binchrstr2dec(binvalue);

  return(decvalue);
}




/******************************************************************************
  Assumes str_name = *.hex, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
 char *ptr;

 // Check if pointer to 1st occurrence of .hex in str_name is NULL
  if ((ptr = strstr(str_name,".hex"))!= NULL)
    sprintf (ptr,".meta");
  else
    strcat(str_name,".meta");

  return 0;
}


const char* string2char(string str){
  const char* array;
  array=str.c_str();
  return array;
}

string chararray2string(char a[],int begin,int length){
  string temp="";

  for(int i=begin;i<begin+length;i++){
   temp=temp+a[i];
  }

  return temp;
}


//convert binary string(unsigned) to int values
int bin2int_unsigned(string  bin)
{
int sum = 0;

for (int i=0;i <bin.length();i++)
{
       if (bin[i] == '0'){
         sum *= 2;
         continue;
	   } else
       if (bin[i] == '1'){
		   sum = (sum*2) + 1;
           continue;
	   }else{
        break;
	   }
}

return sum;

}

//inverse the bit from 1 to 0, from 0 to 1, C++ version
string bitInv(string input){

int i;

   for(i=0;i<input.size();i++){
     if(input[i]=='0')
     input[i]='1';
     else
     input[i]='0';
   }
return input;
}


//convert hex string to binary string

string hex2bin(string hex){
   string temp="",result="";
   int i;
   for(i=0;i<hex.size();i++){
	   if(hex.at(i)=='0'){temp="0000";}
	   if(hex.at(i)=='1'){temp="0001";}
	   if(hex.at(i)=='2'){temp="0010";}
	   if(hex.at(i)=='3'){temp="0011";}
	   if(hex.at(i)=='4'){temp="0100";}
	   if(hex.at(i)=='5'){temp="0101";}
	   if(hex.at(i)=='6'){temp="0110";}
	   if(hex.at(i)=='7'){temp="0111";}
           if(hex.at(i)=='8'){temp="1000";}
	   if(hex.at(i)=='9'){temp="1001";}
	   if((hex.at(i)=='A')||(hex.at(i)=='a')){temp="1010";}
	   if((hex.at(i)=='B')||(hex.at(i)=='b')){temp="1011";}
	   if((hex.at(i)=='C')||(hex.at(i)=='c')){temp="1100";}
	   if((hex.at(i)=='D')||(hex.at(i)=='d')){temp="1101";}
	   if((hex.at(i)=='E')||(hex.at(i)=='e')){temp="1110";}
	   if((hex.at(i)=='F')||(hex.at(i)=='f')){temp="1111";}
       result=result+temp;
   }
   return result;
}

//convert integer to 2 bytes hex data,then to binary string
string int2bin(int input){
  char buffer[10];
  string a,result;
  sprintf(buffer,"%04x",input);
  a=chararray2string(buffer,0,4);
  result=hex2bin(a);
  return result;
}

//convert integer to binary string, only 12 bits of binary string will be extracted
string int2bin12bits(int input){
  string result;
  string a=int2bin(input);  
  result=a.substr(4,12);
  return result;
}



/* The following function does CCITT CRC16 x16 + x12 + x5 + 1 generator polynomial 
*  The initial crc must be 0x0000, not 0xFFFF for T4 float to work.
*
*  If the data hex string last four characters(hex) contain the CRC from float, then
*  the function will return 0 if it matches the float original CRC, otherwise, it will not match
*
*  If the data hex string does not include CRC from float,then it will not do comparision, just
*  return the CRC caculated by the algorithm. In the main program, you should compare it with original 
*  CRC.
*  
*  char* data -----contains hex string
*  int count  -----contains the number of bytes
*/
int getCRC(char *data, int count)
{
unsigned int idx,i,k;
unsigned short const g=0x1021;   /* x16 + x12 + x5 + 1 generator polynomial */
unsigned short d, fcs, t;
unsigned int dd;
unsigned table[256];

 for (i=0; i<256; i++)
  {
  fcs=0x0000;
  d = i << 8;
  for (k=0; k<8; k++)
   {
    if ((fcs ^ d) & 0x8000)
      fcs = (fcs << 1) ^ g;
    else
      fcs = (fcs << 1);
    d <<= 1;
   }
   table[i]=fcs;
   
  }

// fcs=0xFFFF;    /* initial FCS value this does not work for T4 */
 fcs=0x0000;    /* initial FCS value */
 for (i=0; i<count; i++)
{
   t=fcs;
   
   sscanf(&data[i*2],"%2x",&dd);
    
   d=dd;

   idx=(t>>8) ^ d; //cout<<"i= "<<i<<"  d= "<<d<<"  t="<<t<<"  t>>8 ="<<(t>>8)<<"  index="<<idx;
   
   fcs=table[idx];
   fcs=fcs ^ (t<<8);
}
//printf "result:\n";
//  cout<<"crc="<<fcs<<endl;
return(fcs);
}


// decode message type13: 
// profile data for one cycle for t4
void decodeMessageType13(
  int message_type_to_process,
  vector<int>& P_Vector, vector<int>& T_Vector, vector<int>& CRC_Vector,
  struct t4_technical_struct ts, struct t4_meta_struct ms, vector<int> messtype,
  vector<string> hexdata, vector<int> start_row, vector<int> end_row, int fileno,
  vector<int> crc_status){     

  int max_message_num=0,pos1,pos2,i,kkk;
  int T0,P0; // Reference temperature and pressure
  int T_prev, T_current, P_prev, P_current;  // previous and current P/T counts
  char binrecord[BINRECLEN]="";   // binary string for one hex string
  char ss[35]="";                 // substring     
  int date_firstPT=0;             //date of first PT measurement
  vector<string> uniqueHexdata;   //unique hex data for message type 1 or 3
  vector<int> uniqueHexdataIndex; //indexes of the selected hexdata lines     
  int lastNonzeroCharIndex;       //the last nonzero binary character index for a binary string
  int message_type;               //the message_type decoded really from the hex message
     
  //find the maximum message number according to the engineering message     
  if(ms.wmo_recorder==61) max_message_num=ts.number_of_descent_pt_messages;
  if(ms.wmo_recorder==60) max_message_num=ts.number_of_ascent_pt_messages;
  if(max_message_num==0){
    if (trace_file != NULL) fprintf (trace_file,"max_message_num=0 when dealing with message type 13\n");
    if (error_file != NULL) fprintf (error_file,"max_message_num=0 when dealing with message type 13\n");
  }

  //reset P,T,CRC vector
  P_Vector.clear();
  T_Vector.clear();
  CRC_Vector.clear();
              
  if(debug==1) {
    if(message_type_to_process==1){
      cout<<"descending profile data part------------------------------"<<endl;
    }
    if(message_type_to_process==3){
      cout<<"ascending profile data part-------------------------------"<<endl;
    }
    cout<<"message type to process="<<message_type_to_process<<" max_message_num="<<max_message_num<<endl;
    cout<<"fileno="<<fileno<<" start_row="<<start_row[fileno]<<" end_row="<<end_row[fileno]<<endl;
  }    

  // find unique hex data lines,put into unique Hexdata vector
  if(message_type_to_process==1){//descending
    getUniqueHexData(start_row,end_row,fileno,messtype,message_type_to_process,hexdata,uniqueHexdata,uniqueHexdataIndex);         
  }
  if(message_type_to_process==3){//ascending
    getUniqueHexData(start_row,end_row,fileno,messtype,message_type_to_process,hexdata,uniqueHexdata,uniqueHexdataIndex);         
  }

  for(i=0;i<uniqueHexdata.size();i++){
    pos1=63;     
    //decode profile data here
    strcpy(binrecord,string2char(hex2bin(uniqueHexdata[i])));       

    getsubstr(8,11,binrecord,ss); message_type=binchrstr2dec(ss);//message type
    getsubstr(28,36,binrecord,ss);date_firstPT=binchrstr2dec(ss);//date of first PT measurement
    getsubstr(37,47,binrecord,ss);P0=binchrstr2dec(ss);//first pressure         
    getsubstr(48,62,binrecord,ss);T0=binchrstr2dec(ss);//first temperature
    P_Vector.push_back(P0);       
    T_Vector.push_back(T0);
    //CRC_Vector.push_back(crc_status[i]);
    CRC_Vector.push_back(1);
                    
    P_prev=P0;
    T_prev=T0;	        
    
    /*       
       find the last index of nonzero character in bin string(exclude trailing 0s)
       this is to make sure we do not use the fillvalues in the binary string for decoding,
       these fillvalues(0s) are there just to fill the hex string to make it 32 bytes
       because P/T can have absolute/relative positioning, there may not be enough room for another full P/T pair,
       so zeros are used as fillvalues, we need not use these fillvalues for decoding.
       This index found will be used to replace 255 usually used in TS11.
    */     
    lastNonzeroCharIndex=getLastNonzeroCharIndex(uniqueHexdata[i]);
    if(debug==1){          
      cout<<"i="<<i<<" message type="<<message_type<<" hexdata="<<uniqueHexdata[i]<<" uniqueHexdataIndex="<<uniqueHexdataIndex[i]<<" lastNonzeroCharIndex="<<lastNonzeroCharIndex<<" date_firstPT="<<date_firstPT<<" P0="<<P0<<" T0="<<T0<<endl;	  	  
      cout<<"i="<<i<<" binary= "<<binrecord<<endl;
      printMarker(i);
    }       
  
    //get the rest of the P/T pairs
    for(kkk=1;kkk<=15;kkk++){
      // PT measurements.
      if (pos1>lastNonzeroCharIndex) break;
      if (binrecord[pos1] == '0')//absolute P
      {
        pos1++;
        pos2=pos1+10;
        if (pos2>lastNonzeroCharIndex) break;
        getsubstr(pos1,pos2,binrecord,ss);
        P_current=binchrstr2dec(ss);
        if(debug==1) printf("A kkk=%d,message_type=%d,ss=%s,ss_count=%d,P_prev=%d,P_current=%d\n",kkk,message_type,ss,binchrstr2dec(ss),P_prev,P_current);
      }
      else if (binrecord[pos1] == '1')//relative P
      {
        pos1++;
        pos2=pos1+5;
        if (pos2>lastNonzeroCharIndex) break;
        getsubstr(pos1,pos2,binrecord,ss);
        if (message_type==3) { //ascending
          P_current=P_prev-binchrstr2dec(ss);
        }
        else if (message_type==1) { //descending
          P_current=P_prev+binchrstr2dec(ss);
        }
        if(debug==1) printf("R kkk=%d,message_type=%d,ss=%s,ss_count=%d,P_prev=%d,P_current=%d\n",kkk,message_type,ss,binchrstr2dec(ss),P_prev,P_current);  
      }
	
      pos1=++pos2;
      if (pos1>255) break;
      if (binrecord[pos1] == '0')//absolute T
      {
        pos1++;
        pos2=pos1+14;
        if (pos2>255) break;
        getsubstr(pos1,pos2,binrecord,ss);
        T_current=binchrstr2dec(ss);
        if(debug==1) printf("A kkk=%d,message_type=%d,ss=%s,ss_count=%d,T_prev=%d,T_current=%d\n",kkk,message_type,ss,binchrstr2dec(ss),T_prev,T_current);  	  
      }
      else if (binrecord[pos1] == '1')//relative T
      {
        pos1++;
        pos2=pos1+9;
        if (pos2>255) break;
        getsubstr(pos1,pos2,binrecord,ss);
        //maybe have problem here!!!!!!
        if (message_type==3) { //ascending
          T_current=T_prev+binchrstr2dec(ss)-(int)(ms.cals.s.TOFF_R/ms.cals.s.TGAIN); 
        }
        else if (message_type==1) { //descending
          T_current=T_prev-binchrstr2dec(ss)+(int)(ms.cals.s.TOFF_R/ms.cals.s.TGAIN);
        }	
        if(debug==1) printf("R kkk=%d,message_type=%d,ss=%s,ss_count=%d,T_prev=%d,T_current=%d\n",kkk,message_type,ss,binchrstr2dec(ss),T_prev,T_current);  	  	    
      }
      //only at this point, we must have both P_current and T_current and we can write to the vector
      //if for example, only P_current exist, T_current does not, then at T part, it will break the loop
      //and will not come to this point,by doing this, we can make sure we have matching number
      //of P and T
      P_Vector.push_back(P_current);
      T_Vector.push_back(T_current);	
      //CRC_Vector.push_back(crc_status[i]);  	    
      CRC_Vector.push_back(1);
        
      P_prev=P_current;
      T_prev=T_current;
	
      pos1=++pos2;
    }//end of for(kkk=1,kkk<=15) loop       	 
  }//end of for(int i=) loop
    
  //before sorting,print P,T
  if(debug==1){ 
    cout<<"-------------------------------------------------------------"<<endl;
    cout<<"Before sorting pressure, message_type_to_process="<<message_type_to_process<<" the P,T values are as follows:"<<endl;
    for(i=0;i<T_Vector.size();i++){
      cout<<"i="<<i<<" P="<<P_Vector[i]<<" T="<<T_Vector[i]<<endl;
    }
    cout<<"-------------------------------------------------------------"<<endl;
  }
  //Do quicksort to sort pressure, its corresponding temperature are ordered according to pressure
  Quicksort(P_Vector,T_Vector,0,T_Vector.size()-1);
  uniqueHexdata.clear();         
  uniqueHexdataIndex.clear();             
}//end of decodeMessageType13 function    


//decode message type0,technical engineering message for one cycle for t4,used in header part

void decodeMessageType0(struct t4_technical_struct& ts,vector<int> messtype,
                        vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,
			vector<int> crc_status,struct t4_meta_struct ms,vector<int> year_ps,vector<int> month_ps,
                        vector<int> day_ps,vector<int> hour_ps,vector<int> minute_ps,vector<int> second_ps){       
     int j;
     char binrecord[BINRECLEN]="",ss[35]="";
     vector<string> uniqueHexdata;
     vector<int> uniqueHexdataIndex;
     
     int message_type_to_process=0;
     int message_type;
     
     if(debug==1){
       cout<<"technical engineering message data follows--------------------"<<endl;
     } 
     //find unique hex data lines for the specific message type          
     getUniqueHexData(start_row,end_row,fileno,messtype,message_type_to_process,hexdata,uniqueHexdata,uniqueHexdataIndex);         

     for(j=0;j<uniqueHexdata.size();j++){          
       strcpy(binrecord,string2char(hex2bin(uniqueHexdata[j])));
       getsubstr(8,11,binrecord,ss); message_type=binchrstr2dec(ss);//message type
       
       if(debug==1){          
          cout<<"i="<<j<<" message type="<<message_type<<" hexdata="<<uniqueHexdata[j]<<endl;
	  cout<<"i="<<j<<" uniqueHexdataIndex="<<uniqueHexdataIndex[j]<<" hour="<<hour_ps[uniqueHexdataIndex[j]];
	  cout<<" minute="<<minute_ps[uniqueHexdataIndex[j]]<<" second="<<second_ps[uniqueHexdataIndex[j]]<<endl;
       }
       //get the time from satellite in the hh:mm:ss format,it is the timestamp when uniqueHexdata is transmitted
       ts.time_from_satellite_hh=hour_ps[uniqueHexdataIndex[j]];
       ts.time_from_satellite_mm=minute_ps[uniqueHexdataIndex[j]];
       ts.time_from_satellite_ss=second_ps[uniqueHexdataIndex[j]];
       
       //get engineering data from most likely frame,put into technical message struct
       getsubstr(28,35,binrecord,ss);  
       ts.descent_start_time=binchrstr2dec(ss);
       //convert descent_start_time in tenths of an hour to hh:mm format,tenths of an hour=6 minutes,so count*6/60=count/10=hours
       ts.descent_start_time_hh=(int)(floor((double)ts.descent_start_time/10));
       ts.descent_start_time_mm=(int)((((double)ts.descent_start_time/10)-ts.descent_start_time_hh)*60);
       getsubstr(36,42,binrecord,ss);  ts.number_of_valve_actions_at_surface=binchrstr2dec(ss);
       getsubstr(43,50,binrecord,ss);  ts.float_stable_time=binchrstr2dec(ss);
       ts.float_stable_time_hh=(int)(floor((double)ts.float_stable_time/10));
       ts.float_stable_time_mm=(int)((((double)ts.float_stable_time/10)-ts.float_stable_time_hh)*60);       
       getsubstr(51,58,binrecord,ss);  ts.float_stable_pressure=binchrstr2dec(ss);
       getsubstr(59,62,binrecord,ss);  ts.number_of_valve_actions_descent=binchrstr2dec(ss);
       getsubstr(63,66,binrecord,ss);  ts.number_of_pump_actions_descent=binchrstr2dec(ss);
       getsubstr(67,74,binrecord,ss);  ts.end_of_descent_time=binchrstr2dec(ss);
       ts.end_of_descent_time_hh=(int)(floor((double)ts.end_of_descent_time/10));
       ts.end_of_descent_time_mm=(int)((((double)ts.end_of_descent_time/10)-ts.end_of_descent_time_hh)*60);       
       getsubstr(75,78,binrecord,ss);  ts.number_of_repositions=binchrstr2dec(ss);
       getsubstr(79,86,binrecord,ss);  ts.time_at_end_ascent=binchrstr2dec(ss);	                  
       ts.time_at_end_ascent_hh=(int)(floor((double)ts.time_at_end_ascent/10));
       ts.time_at_end_ascent_mm=(int)((((double)ts.time_at_end_ascent/10)-ts.time_at_end_ascent_hh)*60);              
       getsubstr(87,91,binrecord,ss);  ts.number_of_pump_actions_ascent=binchrstr2dec(ss);
       getsubstr(92,96,binrecord,ss);  ts.duration_of_pump_actions_surface=binchrstr2dec(ss);
       getsubstr(97,101,binrecord,ss); ts.number_of_descent_pt_messages=binchrstr2dec(ss);	           
       getsubstr(102,106,binrecord,ss);ts.number_of_drift_pt_messages=binchrstr2dec(ss);
       getsubstr(107,111,binrecord,ss);ts.number_of_ascent_pt_messages=binchrstr2dec(ss);
       getsubstr(112,122,binrecord,ss);ts.shallow_deep_boundary_isobar_descent=binchrstr2dec(ss);
       getsubstr(123,128,binrecord,ss);ts.number_of_descent_slices_shallow_zone=binchrstr2dec(ss);       
       getsubstr(129,136,binrecord,ss);ts.number_of_descent_slices_deep_zone=binchrstr2dec(ss);       
       getsubstr(137,147,binrecord,ss);ts.shallow_deep_boundary_isobar_ascent=binchrstr2dec(ss);
       getsubstr(148,153,binrecord,ss);ts.number_of_ascent_slices_shallow_zone=binchrstr2dec(ss);
       getsubstr(154,161,binrecord,ss);ts.number_of_ascent_slices_deep_zone=binchrstr2dec(ss);
       getsubstr(162,169,binrecord,ss);ts.number_of_PT_measurements_in_drift=binchrstr2dec(ss);
       getsubstr(170,174,binrecord,ss);ts.floats_time_hh=binchrstr2dec(ss);
       getsubstr(175,180,binrecord,ss);ts.floats_time_mm=binchrstr2dec(ss);
       getsubstr(181,186,binrecord,ss);ts.floats_time_ss=binchrstr2dec(ss);
       getsubstr(187,192,binrecord,ss);ts.pressure_sensor_offset=binchrstr2dec(ss);
       getsubstr(193,195,binrecord,ss);ts.internal_pressure=binchrstr2dec(ss);
       getsubstr(196,211,binrecord,ss);ts.surface_temperature=binchrstr2dec(ss);       
       getsubstr(212,219,binrecord,ss);ts.profile_ascent_start_time=binchrstr2dec(ss);
       ts.profile_ascent_start_time_hh=(int)(floor((double)ts.profile_ascent_start_time/10));
       ts.profile_ascent_start_time_mm=(int)((((double)ts.profile_ascent_start_time/10)-ts.profile_ascent_start_time_hh)*60);                            
       getsubstr(220,222,binrecord,ss);ts.number_of_entrance_in_target_range=binchrstr2dec(ss);
       getsubstr(223,230,binrecord,ss);ts.minimum_pressure_in_drift=binchrstr2dec(ss);
       getsubstr(231,238,binrecord,ss);ts.maximum_pressure_in_drift=binchrstr2dec(ss);
       getsubstr(239,239,binrecord,ss);ts.grounding=binchrstr2dec(ss);
     }//end of for loop
     uniqueHexdata.clear();
     uniqueHexdataIndex.clear();     
}//end of decodeMessageType0 function

/* Test to see if a profile has drift data or not
*/

int hasDriftData(vector<int> start_row,vector<int> end_row,int fileno,vector<int> messtype,int message_type_to_process,
                 vector<string> hexdata,vector<string> uniqueHexdata,vector<int> uniqueHexdataIndex){

   getUniqueHexData(start_row,end_row,fileno,messtype,message_type_to_process,hexdata,uniqueHexdata,uniqueHexdataIndex);         
   if(uniqueHexdata.size()==0) {
     return 0;
   }else{
     return 1;
   }     
}

//decode message type2,drift data for one cycle for t4,used in header part
void  decodeMessageType2(struct driftPT_struct& ds,struct t4_technical_struct ts,struct t4_meta_struct ms,
                         vector<int> messtype,
                         vector<string> hexdata,vector<int> start_row,vector<int> end_row,
			 int fileno,vector<int> crc_status){       
			  
    int n,i,kkk,pos1,pos2;
    char binrecord[BINRECLEN]="",ss[35]="";        
    int T0,P0;//T0,P0 are reference temperature and pressure
    int T_prev,T_current,P_prev,P_current;//contain previous P/T and current P/T counts
    int date_firstPT=0;//date of first PT measurement
    int time_firstPT=0;//time of first PT measurement
    vector<string> uniqueHexdata;
    vector<int> uniqueHexdataIndex;
    int lastNonzeroCharIndex;//the last nonzero binary character index for a binary string
    int message_type_to_process=2;
    int message_type;
         
    if(ts.number_of_PT_measurements_in_drift==0){
       if (trace_file != NULL) fprintf (trace_file,"number of PT measurements in drift=0 when dealing with message type 13\n");
       if (error_file != NULL) fprintf (error_file,"number of PT measurements in drift=0 when dealing with message type 13\n");
    }
     
    if(debug==1) printf("drift data follows------------------------------------------\n");
    
    getUniqueHexData(start_row,end_row,fileno,messtype,message_type_to_process,hexdata,uniqueHexdata,uniqueHexdataIndex);         
     
    for(i=0;i<uniqueHexdata.size();i++){
     pos1=65;     
     
       //decode drift data here
       strcpy(binrecord,string2char(hex2bin(uniqueHexdata[i])));
       getsubstr(8,11,binrecord,ss); message_type=binchrstr2dec(ss);//message type
       getsubstr(28,33,binrecord,ss);date_firstPT=binchrstr2dec(ss);//date of first PT measurement
       getsubstr(34,38,binrecord,ss);time_firstPT=binchrstr2dec(ss);//time of first PT measurement
       getsubstr(39,49,binrecord,ss);P0=binchrstr2dec(ss);//first pressure         
       getsubstr(50,64,binrecord,ss);T0=binchrstr2dec(ss);//first temperature
       ds.driftP0_vector.push_back(P0);
       ds.driftT0_vector.push_back(T0);
           
       P_prev=P0;
       T_prev=T0;	        
/*       
       find the last index of nonzero character in binary string(exclude trailing 0s)
       this is to make sure we do not use the fillvalues in the hex string for decoding,
       these fillvalues(0s) are there just to fill the hex string to make it 32 bytes
       because P/T can have absolute/relative positioning, there may not be enough room for another full P/T pair,
       so zeros are used as fillvalues, we need not use these fillvalues for decoding.
       This index found will be used to replace 255 usually used in TS11.
*/     
       lastNonzeroCharIndex=getLastNonzeroCharIndex(uniqueHexdata[i]);

       if(debug==1){          
          cout<<"i="<<i<<" message type="<<message_type<<" hexdata="<<uniqueHexdata[i]<<" lastNonzeroCharIndex="<<lastNonzeroCharIndex<<" date_firstPT="<<date_firstPT<<" time_firstPT="<<time_firstPT<<" P0="<<P0<<" T0="<<T0<<endl;
	  cout<<"i="<<i<<" binary= "<<binrecord<<endl;
	  printMarker(i);
       }
              
       //get the rest of the P/T pairs
       for(kkk=1;kkk<=15;kkk++){
        // PT measurements.
        if (pos1>lastNonzeroCharIndex) break;
        if (binrecord[pos1] == '0')//absolute P
        {
          pos1++;
          pos2=pos1+10;
          if (pos2>lastNonzeroCharIndex) break;
          getsubstr(pos1,pos2,binrecord,ss);
	  P_current=binchrstr2dec(ss);
	  if(debug==1) printf("A message_type=%d,ss=%s,ss_count=%d,P_prev=%d,P_current=%d\n",message_type,ss,binchrstr2dec(ss),P_prev,P_current);  	  	  
        }
        else if (binrecord[pos1] == '1')//relative P
        {
          pos1++;
          pos2=pos1+5;
          if (pos2>lastNonzeroCharIndex) break;
          getsubstr(pos1,pos2,binrecord,ss);
          P_current=P_prev+twoscompl2dec(ss); 	  
	  if(debug==1) printf("R message_type=%d,ss=%s,ss_count=%d,P_prev=%d,P_current=%d\n",message_type,ss,twoscompl2dec(ss),P_prev,P_current);  	  	  
        }
	

        pos1=++pos2;
        if (pos1>255) break;
        if (binrecord[pos1] == '0')//absolute T
        {
          pos1++;
          pos2=pos1+14;
          if (pos2>255) break;
          getsubstr(pos1,pos2,binrecord,ss);
          T_current=binchrstr2dec(ss);	
	  if(debug==1) printf("A message_type=%d,ss=%s,ss_count=%d,T_prev=%d,T_current=%d\n",message_type,ss,binchrstr2dec(ss),T_prev,T_current);  	  	    
        }
        else if (binrecord[pos1] == '1')//relative T
        {
          pos1++;
          pos2=pos1+9;
          if (pos2>255) break;
          getsubstr(pos1,pos2,binrecord,ss);
	  //maybe have problem here!!!!!!
          T_current=T_prev+twoscompl2dec(ss); 
	  if(debug==1) printf("R message_type=%d,ss=%s,ss_count=%d,T_prev=%d,T_current=%d\n",message_type,ss,twoscompl2dec(ss),T_prev,T_current);  	  	  
        }
	//only at this point, we must have both P_current and T_current and we can write to the vector
	//if for example, only P_current exist, T_current does not, then at T part, it will break the loop
	//and will not come to this point,by doing this, we can make sure we have matching number
	//of P and T	
	ds.driftP0_vector.push_back(P_current);
	ds.driftT0_vector.push_back(T_current);
	P_prev=P_current;
	T_prev=T_current;
	
	pos1=++pos2;
       }//end of for(kkk=1,kkk<=15) loop       
	 
    }//end of for(int i=) loop

    //use original drift data(debug)
    for(n=0;n<ds.driftT0_vector.size();n++){            	    	    	    	    	 	    
	    ds.driftP_vector.push_back(ds.driftP0_vector[n]);
            ds.driftT_vector.push_back(ds.driftT0_vector[n]);	   
    }//end of for    

    uniqueHexdata.clear();
    uniqueHexdataIndex.clear();    
}//end of decodeMessageType2 function


//reset vectors after each cycle's decoding to prepare for next cycle
void  resetVectors(vector<int>& PA_Vector,vector<int>& TA_Vector,vector<int>& CRCA_Vector,
                   vector<int>& PD_Vector,vector<int>& TD_Vector,vector<int>& CRCD_Vector,
                   struct position_info_struct& ps,struct driftPT_struct& ds){   
    //ascending		   
    PA_Vector.clear();
    TA_Vector.clear();
    CRCA_Vector.clear();
    //descending
    PD_Vector.clear();
    TD_Vector.clear();
    CRCD_Vector.clear();
    
    ps.year_vector.clear();
    ps.month_vector.clear();
    ps.day_vector.clear();
    ps.hour_vector.clear();
    ps.min_vector.clear();
    ps.sec_vector.clear();
    ps.xmits_vector.clear();
    ps.sat_vector.clear();
    ps.class_vector.clear();
    ps.lon_vector.clear();
    ps.lat_vector.clear();

    ds.driftP0_vector.clear();
    ds.driftT0_vector.clear();
    ds.driftP_vector.clear();
    ds.driftT_vector.clear();
}//end of resetVectors function


/*******************************************************************************************************
  read position information for each profile,and return them by reference
*******************************************************************************************************/
void read_position(struct position_info_struct& ps,vector<int> year_ps,vector<int> month_ps,
vector<int> day_ps,vector<int> hour_ps,vector<int> minute_ps,vector<int> second_ps, vector<float> lat_ps,
vector<float> lon_ps,vector<float> nl_ps,vector<char> sattype_ps,vector<char> lc_ps,int fileno,
vector<int> start_row,vector<int> end_row,struct t4_meta_struct ms)
{

           int i;
           char date_time[32];
	   
    	  //read position information
	  //this if block only applys for floats which can have both descending and ascending profiles
	  if(fileno<0){//if fileno<0, then it must be the first descending profile after launch time,
	   //in this case, use the information from meta data file
	        //first 999 line
	        ps.lat_vector.push_back(99.999);
	        ps.lon_vector.push_back(999.999);
	        ps.xmits_vector.push_back(0);
	        ps.sat_vector.push_back('0');
	        ps.class_vector.push_back('0');                
	        ps.year_vector.push_back(ms.year);
	        ps.month_vector.push_back(ms.month);
	        ps.day_vector.push_back(ms.day);
	        ps.hour_vector.push_back(ms.hour);
	        ps.min_vector.push_back(ms.minute);
	        ps.sec_vector.push_back(0);
		//launch position and time line
	        //first 999 line
	        ps.lat_vector.push_back(ms.lat);
	        ps.lon_vector.push_back(ms.lon);
	        ps.xmits_vector.push_back(0);
	        ps.sat_vector.push_back('0');
	        ps.class_vector.push_back('1');                
	        ps.year_vector.push_back(ms.year);
	        ps.month_vector.push_back(ms.month);
	        ps.day_vector.push_back(ms.day);
	        ps.hour_vector.push_back(ms.hour);
	        ps.min_vector.push_back(ms.minute);
	        ps.sec_vector.push_back(0);		
	        //last 999 line
	        ps.lat_vector.push_back(99.999);
	        ps.lon_vector.push_back(999.999);
	        ps.xmits_vector.push_back(0);
	        ps.sat_vector.push_back('0');
	        ps.class_vector.push_back('0');                
	        ps.year_vector.push_back(ms.year);
	        ps.month_vector.push_back(ms.month);
	        ps.day_vector.push_back(ms.day);
	        ps.hour_vector.push_back(ms.hour);
	        ps.min_vector.push_back(ms.minute);
	        ps.sec_vector.push_back(0);	  
	  }
	  //this if block applys for any floats
	  if(fileno>=0 && fileno<start_row.size()){

	    for(i=start_row[fileno];i<=end_row[fileno];i++){
	      //get latitude and longitude,filtering non positional data out
              if((lat_ps[i]!=(float)0.0)|| (lon_ps[i]!=(float)0.0)){
	        ps.lat_vector.push_back(lat_ps[i]);
	        ps.lon_vector.push_back(lon_ps[i]);
	        ps.xmits_vector.push_back((int)floor((double)(nl_ps[i]-1)/8));
	        ps.sat_vector.push_back(sattype_ps[i]);
	        ps.class_vector.push_back(lc_ps[i]);                
	        ps.year_vector.push_back(year_ps[i]);
	        ps.month_vector.push_back(month_ps[i]);
	        ps.day_vector.push_back(day_ps[i]);
	        ps.hour_vector.push_back(hour_ps[i]);
	        ps.min_vector.push_back(minute_ps[i]);
	        ps.sec_vector.push_back(second_ps[i]);
  	      }//end of if(lat[i])

	    }//end of for(i=start_row[k])

	    //put 999 delimeter to the top and bottom of position_info struct

	      //append at the end 999 delimeter
	      ps.lat_vector.push_back(99.999);
	      ps.lon_vector.push_back(999.999);
	      ps.xmits_vector.push_back((int)floor((double)(nl_ps[end_row[fileno]]-1)/8));
	      ps.sat_vector.push_back(sattype_ps[end_row[fileno]]);
	      ps.class_vector.push_back(lc_ps[end_row[fileno]]);

	      ps.year_vector.push_back(year_ps[end_row[fileno]]);
	      ps.month_vector.push_back(month_ps[end_row[fileno]]);
	      ps.day_vector.push_back(day_ps[end_row[fileno]]);
	      ps.hour_vector.push_back(hour_ps[end_row[fileno]]);
	      ps.min_vector.push_back(minute_ps[end_row[fileno]]);
	      ps.sec_vector.push_back(second_ps[end_row[fileno]]);
              //cout<<"HERE------position-3"<<endl;
	     //insert in the front 999 delimeter
	      ps.lat_vector.insert(ps.lat_vector.begin(),99.999);
	      ps.lon_vector.insert(ps.lon_vector.begin(),999.999);
              ps.xmits_vector.insert(ps.xmits_vector.begin(),(int)floor((double)(nl_ps[start_row[fileno]]-1)/8));
	      ps.sat_vector.insert(ps.sat_vector.begin(),sattype_ps[start_row[fileno]]);
	      ps.class_vector.insert(ps.class_vector.begin(),lc_ps[start_row[fileno]]);

	      ps.year_vector.insert(ps.year_vector.begin(),year_ps[start_row[fileno]]);
	      ps.month_vector.insert(ps.month_vector.begin(),month_ps[start_row[fileno]]);
	      ps.day_vector.insert(ps.day_vector.begin(),day_ps[start_row[fileno]]);
	      ps.hour_vector.insert(ps.hour_vector.begin(),hour_ps[start_row[fileno]]);
	      ps.min_vector.insert(ps.min_vector.begin(),minute_ps[start_row[fileno]]);
	      ps.sec_vector.insert(ps.sec_vector.begin(),second_ps[start_row[fileno]]);
	      
         if(debug==1){
	        printf("inside read_position function:\n");
		  for(i=0;i<ps.sat_vector.size();i++){

                     sprintf(date_time,"%4d/%02d/%02d%3d:%02d:%02d",ps.year_vector[i],ps.month_vector[i],ps.day_vector[i],ps.hour_vector[i],ps.min_vector[i],ps.sec_vector[i]);
                     printf ("    %+7.3f   %+8.3f %s   %03d   %1c     %1c\n",ps.lat_vector[i], ps.lon_vector[i],date_time,
                              ps.xmits_vector[i], ps.sat_vector[i],ps.class_vector[i]);                     
                  }		
	      }
	    }//end of if(fileno<start_row)

};//end of function read_tech_position_drift

/*
   input vector contain the numbers,its size is window
   will first sort input vector, then return the middle value
   from it.
*/

int getMedian(vector<int> input,int window){

	int middle=(int)(window/2);//for example window=5,middle=2;because index begin from 0

	sort(input.begin(),input.end());
/*
    for(int i=0;i<input.size();i++){
	   cout<<"input array after "<<input[i]<<endl;
    }
*/
	return input[middle];
}//end of getMedian function


/*  correct overflow problem for temperature counts
*/
void correctOverflowT(vector<int>& dt,int thresh){
	int i=0,md=0;
	vector<int> tempT;//contain the values which need to get the median value
	
	
        for(i=0;i<dt.size();i++){
	  cout<<"before overflow correction dt="<<dt[i]<<endl;
	}
	
//matlab    	k=find(diff(dt)<-4000);, m=find(dt(k)>1000);, k=k(m);

	for(i=1;i<dt.size();i++){		
	  if(dt[i]-dt[i-1]<-4000){
	    if(dt[i-1]>1000){
	      dt[i-1]=dt[i-1]-4096;  
	    }	    
	  }	  
	}

/*	
	for(i=1;i<dt.size();i++){		
	  if(dt[i]-dt[i-1]<-4000){
	    dt[i-1]=dt[i-1]-4096;
	  }	  
	}
*/	
/*
    //apply median filter to dt vector
    The median filter  does not work! 
    (apply 3-point median filter to the second point,
    next to last point,for other points except first and last point, apply 5-point median filter, 
    get the median value thresh is a constant value, when (dt-median)>thresh, 
    then apply overflow correction)
*/
    
	for(i=1;i<dt.size();i++){
		
	  tempT.clear();
		
	  if((i==1)||(i==(int)dt.size()-2)){//3-point median filter
              tempT.push_back(dt[i-1]);tempT.push_back(dt[i]);tempT.push_back(dt[i+1]);
	      md=getMedian(tempT,3);
	      //if the difference of T is greater than median value by thresh value,then apply overflow correction  
              if(((abs(dt[i]-md)>thresh) && (dt[i]>0)))
	      {dt[i]=dt[i]-8192/2;}  
	  }
		
	  if((i>1) && (i<(int)dt.size()-2)){//5-point median filter
              tempT.push_back(dt[i-2]);tempT.push_back(dt[i-1]);tempT.push_back(dt[i]);  
              tempT.push_back(dt[i+1]);tempT.push_back(dt[i+2]);
	      md=getMedian(tempT,5);
              if(((abs(dt[i]-md)>thresh) && (dt[i]>0)))
	      {dt[i]=dt[i]-8192/2;}
	  }
        
	 
	}//end of for loop
	
       if(debug==1){
        for(i=0;i<dt.size();i++){
	  cout<<"after overflow correction dt="<<dt[i]<<endl;
	}
	}
	
}//end of correctOverflowT function

//clean start_row and end_row, if there are ghost messages/cycles, i.e, 
//if there is only one line in a cycle, then skip it,remove it from start_row,end_row vectors.
void  cleanStartRowEndRow(vector<int>& start_row,vector<int>& end_row){
  vector<int> start_row1,end_row1;

  for(int i=0;i<start_row.size();i++){
    if(start_row[i]!=end_row[i]){
      start_row1.push_back(start_row[i]);
      end_row1.push_back(end_row[i]);
    }
  }
  start_row=start_row1;
  end_row=end_row1;
}

//adjust start_row's line number based on launch_time, if in one cycle, the date is larger
//than launch_time, then record the first row when this happens and reset this cycle's start_row
//value to this recorded row number. This will make sure that we do not skip any profile with 
//valid data in it.
void adjustStartRowEndRow(vector<int>& start_row,vector<int> end_row,vector<float> jdayarray,struct t4_meta_struct ms){
  vector<int> start_row1;
  int k;
  int flag=0;
  
  for(int i=0;i<start_row.size();i++){
    k=start_row[i];  
    flag=0;
    for(int j=start_row[i];j<=end_row[i];j++){
       if(jdayarray[j]>=ms.launch_time){         
       //find the first time when jdayarray is >=launch_time, record its row number  
	    k=j;
	    flag=1;
	    break;	 
       }  
    }//end of for(int j=0)
    start_row1.push_back(k);
  }//end of for(int i=0)
  
  start_row=start_row1;
}
/**************************************************************************************
  Function used to find the CRC value for a given
  message using the 16 bit CRC-CCITT standard polynomial.
  The Modulo-2 Binary Division is implemented and the
  remainder of the division will be the CRC. This 
  implementation is rather slow and can be speeded up
  using the well known technique of generating first a
  look-table. (Will do when I get the time!).
 
  Generator polynomial used:  CRC-CCITT 16 bit
 
  X^16 + X^12 + X^5 + 1
 
  Polynomial is truncated. The MSB 1 is removed
  as we have the information in the uppermost bit
  and we don't need it for the XOR, thus polynomial
  can be stored in a 16 bit result.
  
  INPUT : hexadecimal message (64 character array)
 
  OUTPUT: CRC hexadecimal value 
 
**************************************************************************************/

int crc_test(char *binstr)
{
  char crc_prov[17];
  char acc[]="0000000000000000";
  char newbinstr[257]=""; 
  int i,a,b,c;
  //reorganize the newbinstr to comform to the CRC algorithm specified for T4 from the doc.  
  for(i=0;i<4;i++) {newbinstr[i]=binstr[i+8];} //message type,4 bits
  for(i=4;i<20;i++) {newbinstr[i]='0'; crc_prov[i-4]=binstr[i+8];} //reset crc field to 0 and get original crc field
  for(i=20;i<248;i++) {newbinstr[i]=binstr[i+8];}
  for(i=248;i<256;i++) {newbinstr[i]=binstr[i-248];}//append the ARGOS ID complemnts to the end of newbinstr
  
  crc_prov[17]='\0';
  newbinstr[257]='\0';
  
  for (i=0; i<256; i++)
  {
    a=chr_xor(newbinstr[i],acc[0]);
    b=chr_xor(a,acc[4]);
    c=chr_xor(a,acc[11]);

    acc[0]=acc[1];
    acc[1]=acc[2];
    acc[2]=acc[3];
    acc[3]=b;
    acc[4]=acc[5];
    acc[5]=acc[6];
    acc[6]=acc[7];
    acc[7]=acc[8];
    acc[8]=acc[9];
    acc[9]=acc[10];
    acc[10]=c;
    acc[11]=acc[12];
    acc[12]=acc[13];
    acc[13]=acc[14];
    acc[14]=acc[15];
    acc[15]=a;
  }

  if (strcmp(crc_prov, acc) == 0)
    return(0);
  else
    return(1);
}

/*
  remove duplicate hex data line
  (For technical message, exclude argos ID complement,message type,crc,float's time and surface temperature fields,use binary string to compare;
   For other message types,exclude argos ID complement and message type,crc,totally 7 characters of hex data,
       only compare hex data from character 8 to 64)
  return unique hexdata with argos ID complement,message type,crc,and the remaining hex data.
 
  vector<int> uniqueHexdataIndex;//contain the indexes of the selected uniqueHexdata lines in the hexdata vector
   
  The function return the unique hex data lines for one message type for one profile
*/

void getUniqueHexData(vector<int> start_row,vector<int> end_row,int fileno,vector<int> messtype,int message_type,vector<string> hexdata,vector<string>& uniqueHexdata,vector<int>& uniqueHexdataIndex){
 int i;
 set<string> uniqueHexAfterCRC;//unique hex string after crc field,from character 8 to 64
 set<string> uniqueBinCleaned;//unique binary string after crc,float's time and surface temperature are excluded
 char binrecord[BINRECLEN]="",newbinrecord[BINRECLEN]="",ss[BINRECLEN]="";
 
//64-7=57 
 for(i=start_row[fileno];i<=end_row[fileno];i++){
    //skip the position line
    if(hexdata[i]=="0000000000000000000000000000000000000000000000000000000000000000") continue;
    
    if(messtype[i]==message_type){
      if(message_type!=0){//if it is not technical message
        if(uniqueHexAfterCRC.insert(hexdata[i].substr(7,57)).second){
         uniqueHexdata.push_back(hexdata[i]);
	 uniqueHexdataIndex.push_back(i);
         //cout<<"unique hex data="<<hexdata[i]<<endl;
        }//end of if
      }else{//if it is a technical message, treat it specially,exclude crc,float's time,and surface temperature fields
            //then compare the binary data to see it there are duplicates 
       strcpy(binrecord,string2char(hex2bin(hexdata[i]))); 
       newbinrecord[0]='\0';       
       getsubstr( 28,169,binrecord,ss);strcpy(newbinrecord,ss);//142 bits
       getsubstr(187,195,binrecord,ss);strcat(newbinrecord,ss);//9 bits
       getsubstr(212,255,binrecord,ss);strcat(newbinrecord,ss);//44 bits
       //totally there are 142+9+44=196 bits        
       if(uniqueBinCleaned.insert(chararray2string(newbinrecord,0,196)).second){
         uniqueHexdata.push_back(hexdata[i]);
	 uniqueHexdataIndex.push_back(i);
         //cout<<"unique hex data="<<hexdata[i]<<endl;
       }//end of if       
      }//end of if(message_type!=1)
    }//end of if(messtype[i]==message_type)
    
 }//end of for loop

}//end of function

/*       
       find the last index of nonzero character in binary string(exclude trailing 0s)
       this is to make sure we do not use the fillvalues in the binary string for decoding,
       these fillvalues(0s) are there just to fill the hex string to make it 32 bytes
       because P/T can have absolute/relative positioning, there may not be enough room for another full P/T pair,
       so zeros are used as fillvalues, we need not use these fillvalues for decoding.
       This index found will be used to replace 255 usually used in TS11.
*/     
 int getLastNonzeroCharIndex(string hexString){
    int lastNonzeroCharIndex=255;//by default,assume no zeros at the end of hex string,256-1=255 since index starts from 0
    int i,length,zeroCounts=0;
    char binString[BINRECLEN]="";
    
    //hexString="0037E540019A76E93E9A54DFF5371D445C1A9A1AA6C0A9EEAA70DA9B12A66E00";
    strcpy(binString,string2char(hex2bin(hexString)));
    length=strlen(binString);
        
    if(binString[length-1]=='0'){
      for(i=length-1;i>0;i--){    
       if(binString[i]=='0') {
          zeroCounts++;
       }else{
         break;
       } 	  
      }
    }//end of if(binString[length-1]
    lastNonzeroCharIndex=length-zeroCounts-1;   
    return lastNonzeroCharIndex;
 }


/* print the marker for binary string
0000000000001101001000000100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000110010000000000000000000111110100000000000000000000000010010010000100001000000000010110001100101000000000000000000000000000000000000000000000
123456789 123456789 123456789
         1         2 
*/
void printMarker(int i){
   int j,times;
   printf("i=%d marker= ",i);
   times=(int)ceil((float)256/10);
   for(j=0;j<times;j++){     
     printf("123456789 ");
   }
   printf("\n");
   
   printf("i=%d marker= ",i);
   for(j=0;j<times;j++){     
     printf("         %d",j+1);
   }
   printf("\n");
   
}








