/******************************************************************************
   Revision Date: 10-Apr=13 (RS - commented  line: 
                                  printf("prof_no=%d\n",prof_no);)
                  01-Nov-11 (ER - PTS1 triplet (diagnostic) moved from end of 
                                  drift to start of ascent in 2004
                  28-Oct-11 (ER - minimum depth bin with valid data = zmin+1
                                  profile 0
                  25-Oct-11 (ER - Lines "AIR PRESSURE END OF LAST SURFACE" and
                                  "AIR PRESSURE START OF LAST SURFACE" eliminated
                                  for being deemed redundant.
                  21-Oct-11 (ER - 'TOTAL TIME FOR PROFILE (SECONDS)' 
                                  replaces P,T,S +TWO SECONDS (COUNTS)
                  20-Oct-11 (ER - Variables VACb, VACa, LIMsw (in and out) were
                                  incorrectly being set 0 always.
                  19-Oct-11 (ER - minimum depth bin with valid data = imin+1
                  04-ABR-11 (ER - fix 256 profile taking 255 profile data.)
                  26-AUG-10 (ER -Replace parameter name 'START SURFACE PRESSURE (DBAR)'
                                 with 'SURF PRES BEF RESET(PREV CYC)(COUNTS)' 
                                -Replace parameter name 'START SURFACE PRES AFTER RESET (COUNTS)'
                                 with 'SURF PRES AFTER RESET(PREV CYC)(COUNTS) ' 
                  14-Sep-06 (XXD - position julian day should be larger than launchtime's)
                  01-Jul-05 (XX -if imin is too big,do not output the profile data part)
                  13-Jun-05 (XX -fix the imin problem for the depth bin number)
                  26-AUG-04 (CS&XX- correction to the change of 15-AUG-03, exit with 2 only 
                                    if all profiles are processed(nprofile=1),otherwise 
				    exit with 1(nprofile=0)) 
                  20-JUL-04 (XX-if profile number is 0, it should have C diagnostic message, 
                             if profile number is not 0, it should have E diagnostic message
                  07-JUL-04 (XX-if number of data lines<=1, then return status 2 also)
                  01-JUL-04 (XX-if argos_id <=32767,use 20 bit id,do not use "DB"
                             as indicator)
                  25-Jun-04 (XX-change DS if DS>127 for Savg2)
                  08-Jan-04 (XX&CS-changed jd,jd_old,jday from int to float,
                             do not use abs to allow short down times of
                             >1day, < 10 days)
                  15-AUG-03 (XX&CS-add exit status 2 when frame missing
                             problem happens in some but not all profile
                             cycles)
                  27-Nov-02 (DB/CS - fixed problem for floats with ARGOS ID<32768
                             if all is OK they have DB in the first byte.
                             If something is wrong the first byte could not be
                             removed, i.e. the first half byte was erroneously
                             identified as the message (frame) 'number'. If it
                             happened to be a C the contents of the technical
                             message E were ignored and those of the C message
                             were decoded as if it was the technical message C
                             Now the most frequent technical message (E or C)
                             is used)
                  27-Sep-02 (CS - adjusted to revised documentation of 9/27
                             removed IN_curr (does not exist); corrected
                             scanning and decoding of diagnostic message C;
                             replaced j = 1; with j = 0; I think this needs
                             to be done for all hex2cnt)
                  05-Aug-02 (DB - started with HEX2CNT_TS9

   hex2cnt_ts12.c
   ==============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   SOLO floats SOLO_TS12 manufactured and deployed in 2001,
   Scripps Institution of Oceanorgaphy, California
   
Exit code
  nprofile=0(only process last profile) case:
        0--means success(one data scan profile will be ignored and return 0)
	1--means failure(e.g. frame missing problem)
  nprofile=1(process all profiles)      case:
        0--means success(all profiles are processed succefully,one data scan profile will be ignored)
	2--means not all of the profiles are successfully generated(one data scan profile will be ignored)
	
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256  //Maximum length of each scan read from *.hex
#define max_positions 100 //Maximum number of positions per profile
#define frames_per_message 8
#define good_bad_position 43 //Location of status byte in *.hex records
#define start_of_hex_data 63 //Start position of data in *.hex records
#define NO  0
#define YES 1

int debug=0;//1 debug, 0 no debug
int no_output_cycle_count=0;//added new
int frame_problem_count=0,total_cycles_count=0;

char* strupr(char* ptr_to_string_to_be_upper_cased); /* function prototype */

FILE *input_file, *out_file, *error_file, *trace_file, *list_file;

enum {E_diagnostic_message,C_diagnostic_message} diagnostic_message;

char PRGNAME[] ="hex2cnt_ts12";
char buffer[2048];

// Calibration values for Scripps style salinity float
struct sio_cals { double pgain;
                  double poff;
                  double tgain;
                  double toff;
                  double sgain;
                  double soff;
                  double pmp_gain;
                  double pmp_off;
                  double cpugain;
                  double cpuoff;
                  double blok;
                  double av1;
                  double av2;
                  double pb1;
                  double pb2;
                };


union u_cals  { struct sio_cals s;
                double c[sizeof(struct sio_cals)/sizeof(double)];};

struct solo_ts_meta_struct { int AOML_number;
                             int WMO_id;
                             int ARGOS_id;
                             int wmo_instrument;
                             int wmo_recorder;
                             float cycle_time;
                             int exp_no;
                             unsigned char float_type[40];
                             unsigned char PI[60];
                             float launch_time;
                             float start_cycle_time;
                             float start_time;
                             int before_2004;
                             union u_cals cals;};

// First structure used processing SIO salinity floats.
// Contains position info, linear array of data bytes.
struct sio_ctd_out_array { float jday;//changed from int to float
                           double position_jd[max_positions];
                           float latitude[max_positions];
                           float longitude[max_positions];
                           char date_time[max_positions][32];
                           int n_xmits[max_positions];
                           char sat_name[max_positions];
                           char loc_class[max_positions];
                         };

// The following two sub structures used in processing SIO 
// salinity floats.  These sub structures are part of the
// SIO_CTD_RAW structure described below
struct sio_ctd_data_frame { unsigned int ktmin;
                            unsigned int ktmax;
                            unsigned int ksmin;
                            unsigned int ksmax;
                            unsigned int tmsb2;
                            unsigned int tmsb1;
                            unsigned int tlsb[14];
                            unsigned int smsb1;
                            unsigned int slsb[14];
                          };

// E. Ramos  31-oct-11
// P1, T1, S1:
//    refer to the end of drift from oct 2002 to sept 2003
//    refer to the start of ascent starting on oct 2004
struct sio_ctd_diagnostics { unsigned int BST;  //Diagnostic messsage 'E'
                             unsigned int P1;
                             unsigned int T1;
                             unsigned int S1;
                             unsigned int SBnscan;
                             unsigned int SBntry;
                             unsigned int SBstat;
//                           unsigned int P2;
//                           unsigned int T2;
//                           unsigned int S2;
                             unsigned int VACb;
                             unsigned int VACa;
                             unsigned int in_lim;
                             unsigned int out_lim;
                             unsigned int PMPc;
                             unsigned int Vcpu;
                             unsigned int Vpump;
                             unsigned int SAVG1;
                             unsigned int DS;
                             unsigned int num_bad;
                             unsigned int ATE;
                             unsigned int ATS;
                             unsigned int PFS;
                             unsigned int PFE;
                             unsigned int PRE;
                             unsigned int TSK;
                             unsigned int PSK;
                             unsigned int TIP;
                             unsigned int lim_sw;   //Diagnostic message 'C'
                             unsigned int SPRXL;
                             unsigned int diag_P0;
                             unsigned int diag_T0;
                             unsigned int diag_S0;
                             unsigned int diag_P1;
                             unsigned int diag_T1;
                             unsigned int diag_S1;
                             unsigned int BT_vac;
                             unsigned int VAC_b;
                             unsigned int VAC_a;
                             unsigned int BTP_curr;
                             unsigned int OUT_curr;
                             unsigned int BTP_secs;
                             unsigned int IN_secs;
                             unsigned int OUT_secs;
                             unsigned int BT_pb;
                             unsigned int Vple;
                             unsigned int BT_cb;
                             unsigned int V_cpu;
                             unsigned int ST_secs;
                             unsigned int DUR_secs;
                           };

// Equivalence of above structure to array of integers to 
// simplify printing diagnostic values
union diags { struct sio_ctd_diagnostics sio;
              unsigned int ints[sizeof(struct sio_ctd_diagnostics)/sizeof(int)];};

// Second structure used in decoding SIO salinity floats
// All parameters encoded in linear array x in SIO_CTD_OUT_ARRAY 
// structure described above are decoded and put into the 
// following structure
struct sio_ctd_raw { unsigned int pavg1;
                     unsigned int tavg2;
                     unsigned int tavg1;
                     unsigned int pavg2;
                     unsigned int sprx;
                     unsigned int fallt;
                     unsigned int err;
                     unsigned int imin;
                     unsigned int bmax;
                     struct sio_ctd_data_frame data[4];
                     union diags diagnostics;
                   };

// Final array for decoding SIO salinity floats
struct temp_sal_counts { int depth[104];
                         float temp[104];
                         float salinity[104];
                         int iflag[104];
                       };


long ymd2julday (int, int, int);

/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char  *strupr(char* s)
{
  char* p = s;
  while (*p) {
    if (islower((int)*p)) *p = toupper((int)*p);
    ++p;
  }
  return s;
}

/******************************************************************************
  Computes days from 1997
******************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
  int    yr, mo, dy, day_from_97;
  long   passed_jd, jd_xx0101, jd_970101;
  double d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
   yr += 2000;
  else if (yr < 1900)
   yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
   return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year)
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
  day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs,
                                                // mins, secs)
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
  return (d_jd);                                // and adds this to d_jd
}

/******************************************************************************
  Computes final values for SIO salinity floats and fills temp_sal_counts 
  structure 
******************************************************************************/
int compute_sio_ctd_counts (struct solo_ts_meta_struct *s, struct sio_ctd_raw *x, 
                            struct temp_sal_counts *ts)
{
  int i,j;
  float lookup[16] = {-4.0,-2.5,-1.5,-1.0,-0.75,-0.5,-0.25,0.0,
                      0.25,0.5,0.75,1.0,1.5,2.5,4.0,6.25};
  float gain,offset;

  // Compute temperature counts - pretty complicated - see manual for details
  for (i=0; i<4; i++)
  {
    gain = (lookup[x->data[i].ktmax] - lookup[x->data[i].ktmin]); // Compute gain
    offset = (float) (lookup[x->data[i].ktmin] * 256.0); // Compute offset
    ts->temp[14*i] = (float) (x->data[i].tmsb1 * 256 + x->data[i].tlsb[0]); // Compute counts for    
    if(debug>0) printf("i=%d,Tgain=%6.3f,Toffset=%6.3f,tmsb1=%d,tlsb0=%d\n",i,gain,offset,x->data[i].tmsb1,x->data[i].tlsb[0]);
    if(debug>0) printf("T0=%6.3f\n",ts->temp[14*i]);      
    for (j=1; j<14; j++){
      ts->temp[14*i + j] = ts->temp[14*i + j - 1] + x->data[i].tlsb[j] * gain + offset;
      if(debug>0) printf("T=%6.3f\n",ts->temp[14*i+j]);
    }
  }

  // Compute conductivity counts - see manual for details
  for (i=0; i<4; i++)
  {
    gain = lookup[x->data[i].ksmax] - lookup[x->data[i].ksmin]; // Compute gain
    offset = (float) (lookup[x->data[i].ksmin] * 64.0); // Compute offset 
    ts->salinity[14*i] = (float) (x->data[i].smsb1 * 64 + x->data[i].slsb[0]); // Compute first bin counts
    if(debug>0) printf("i=%d,Sgain=%6.3f,Soffset=%6.3f,smsb1=%d,slsb0=%d\n",i,gain,offset,x->data[i].smsb1,x->data[i].slsb[0]);
    if(debug>0) printf("S0=%6.3f\n",ts->salinity[14*i]);
    
    for (j=1; j<14; j++) {//nint
      ts->salinity[14*i + j] = ts->salinity[14*i + j - 1] + x->data[i].slsb[j] * gain + offset; // Compute counts for
      if(debug>0) printf("S=%6.3f\n",ts->salinity[14*i+j]);
    }
  }
  
  if(debug>0) printf("in compute_sio function imin=%d bmax=%d\n",(int)x->imin,x->bmax);
  // Depth - see manual
  for (i=0; i<56; i++)
  {
    ts->depth[i] = x->imin+i+1;
    if (i < (int) x->imin)
       ts->iflag[i] = 4;
    else if (i >= (int) x->bmax)
       ts->iflag[i] = 4;
    else
       ts->iflag[i] = 1;
  }
  if (trace_file != NULL)
    fprintf(trace_file," COMPUTE_SIO_CTD_COUNTS successful\n");

  return(0);
}

/******************************************************************************
  Decodes raw data for SIO salinity floats and fills intermediate
  structure sio_ctd_raw 
******************************************************************************/
int decode_sio_ctd_data (unsigned char *x, struct sio_ctd_raw *y)
{
  struct VAC_struct {unsigned int byte1;
                     unsigned int byte2;
                     unsigned int byte3;
                     unsigned int byte4; } VAC;
  
//  unsigned char *ptr;
  unsigned int tmp[7];
  int i,j,k;

  if(debug>0) printf("hex string=%s\n",x);

//  ptr = &x[1];
  sscanf ( (const char *) &x[1],"%3x%2x",&y->pavg1,&y->tavg2); // Copies from  
                                                             // location ptr 
                                                             // into structure
//  ptr += 64;
  sscanf ( (const char *) &x[65],"%3x%2x",&y->tavg1,&y->pavg2);
//  ptr += 64;
  sscanf ( (const char *) &x[129],"%3x%2x",&y->sprx,&y->fallt);
//  ptr += 64;
  sscanf ( (const char *) &x[193],"%1x%2x%2x",&y->err,&y->imin,&y->bmax);

  y->tavg2 += (y->tavg1 & 0x0F00);
  y->pavg2 += (y->pavg1 & 0x0F00);

  if (y->pavg2 > y->pavg1 + 128)
     y->pavg2=y->pavg2-256;
  else if (y->pavg1 > y->pavg2 + 128)
     y->pavg2=y->pavg2+256;

  if (y->tavg2 > y->tavg1 + 128)
     y->tavg2=y->tavg2-256;
  else if (y->tavg1 > y->tavg2 + 128)
     y->tavg2=y->tavg2+256;

  for (k=0; k<4; k++)
  {
//    ptr = x + 64*k + 6;

    sscanf ((const char *) &x[64*k+6],"%1x%1x%1x%1x%1x%1x",
            &y->data[k].ktmin,&y->data[k].ktmax,
            &y->data[k].ksmin,&y->data[k].ksmax,
            &y->data[k].tmsb2,&y->data[k].tmsb1);

//    ptr += 6;

    sscanf ((const char *) &x[64*k+12],"%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x%2x",
            &y->data[k].tlsb[0],&y->data[k].tlsb[1],
            &y->data[k].tlsb[2],&y->data[k].tlsb[3],
            &y->data[k].tlsb[4],&y->data[k].tlsb[5],
            &y->data[k].tlsb[6],&y->data[k].tlsb[7],
            &y->data[k].tlsb[8],&y->data[k].tlsb[9],
            &y->data[k].tlsb[10],&y->data[k].tlsb[11],
            &y->data[k].tlsb[12],&y->data[k].tlsb[13]);
    if(debug>0){
      
        printf("======from decode_sio_ctd_data function=======================\n");
	printf("message_id=%d\n",k);
	printf("tlsb=%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d\n",
	     y->data[k].tlsb[0], y->data[k].tlsb[1],
             y->data[k].tlsb[2], y->data[k].tlsb[3],
             y->data[k].tlsb[4], y->data[k].tlsb[5],
             y->data[k].tlsb[6], y->data[k].tlsb[7],
             y->data[k].tlsb[8], y->data[k].tlsb[9],
             y->data[k].tlsb[10],y->data[k].tlsb[11],
             y->data[k].tlsb[12],y->data[k].tlsb[13]);
      
    }
//    ptr += 28;

    sscanf ((const char *) &x[64*k+40],"%1x%3x%3x%3x%3x%3x%3x%3x",&y->data[k].smsb1,
            &tmp[0],&tmp[1],&tmp[2],&tmp[3],&tmp[4],
            &tmp[5],&tmp[6]);

    j = 0;

    for (i=0; i<7; i++)
    {
      y->data[k].slsb[j++] = (tmp[i] & 0x0FC0) >> 6;
      y->data[k].slsb[j++] = tmp[i] & 0x003F;
    }
    
    if(debug>0){       
        printf("slsb=%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d\n",
	     y->data[k].slsb[0], y->data[k].slsb[1],
             y->data[k].slsb[2], y->data[k].slsb[3],
             y->data[k].slsb[4], y->data[k].slsb[5],
             y->data[k].slsb[6], y->data[k].slsb[7],
             y->data[k].slsb[8], y->data[k].slsb[9],
             y->data[k].slsb[10],y->data[k].slsb[11],
             y->data[k].slsb[12],y->data[k].slsb[13]);
	
    }
    
    
  }

// Decode 'E' diagnostic message - important values so include with other data

  sscanf(&x[257],"%1x%4x%4x%4x%4x%1x%1x%1x%1x%1x%1x%2x%2x%2x%4x%2x%2x%3x%3x%3x%3x%3x%3x%3x%3x",
   &y->diagnostics.sio.BST,
   &y->diagnostics.sio.P1, &y->diagnostics.sio.T1, &y->diagnostics.sio.S1,
   &y->diagnostics.sio.SBnscan, &y->diagnostics.sio.SBntry, &y->diagnostics.sio.SBstat,
// &y->diagnostics.sio.P2, &y->diagnostics.sio.T2, &y->diagnostics.sio.S2,
   &VAC.byte1, &VAC.byte2, &VAC.byte3, &VAC.byte4, 
   &y->diagnostics.sio.PMPc,&y->diagnostics.sio.Vcpu,&y->diagnostics.sio.Vpump,
   &y->diagnostics.sio.SAVG1,&y->diagnostics.sio.DS,&y->diagnostics.sio.num_bad,
   &y->diagnostics.sio.ATE,&y->diagnostics.sio.ATS,&y->diagnostics.sio.PFS,
   &y->diagnostics.sio.PFE,&y->diagnostics.sio.PRE,&y->diagnostics.sio.TSK,
   &y->diagnostics.sio.PSK,&y->diagnostics.sio.TIP);
  //adjust DS,added new(XXD)
   if(y->diagnostics.sio.DS>127){
     y->diagnostics.sio.DS=y->diagnostics.sio.DS-256;
   }

   if (debug>0){       
      printf("VAC.byte1=%d  VAC.byte2=%d  VAC.byte3=%d  VAC.byte4=%d\n", VAC.byte1, VAC.byte2, VAC.byte3, VAC.byte4);
   }

  y->diagnostics.sio.VACb = (VAC.byte1*8) +  ((VAC.byte2 & 0xE)/2) ;  // bits 1-7
  y->diagnostics.sio.VACa = (VAC.byte2 & 0x1)*64 + (VAC.byte3*4) + ((VAC.byte4 & 0xC)>>2) ; // bits 8-14
  y->diagnostics.sio.in_lim = (VAC.byte4 & 0x2);   // bit 15
  y->diagnostics.sio.out_lim = (VAC.byte4 & 0x1);  // bit 16

   if (debug>0){       
      printf("VACb=%d  VACa=%d  in_lim=%d  out_lim4=%d\n", y->diagnostics.sio.VACb, y->diagnostics.sio.VACa, 
              y->diagnostics.sio.in_lim, y->diagnostics.sio.out_lim);
   }


  if (y->diagnostics.sio.PSK > 2047)
     y->diagnostics.sio.PSK=y->diagnostics.sio.PSK - 4096;

//   y->diagnostics.sio.P2 += (y->diagnostics.sio.P1 & 0XFF00);
//   y->diagnostics.sio.T2 += (y->diagnostics.sio.T1 & 0XFF00);
//   y->diagnostics.sio.S2 += (y->diagnostics.sio.S1 & 0XFF00);
  
// Decode 'C' diagnostic message

  sscanf(&x[321],"%2x%3x%3x%4x%4x%3x%4x%4x%2x%2x%2x%2x%2x%3x%3x%3x%2x%2x%2x%2x%4x%3x",
   &y->diagnostics.sio.lim_sw,&y->diagnostics.sio.SPRXL,
   &y->diagnostics.sio.diag_P0,&y->diagnostics.sio.diag_T0,
   &y->diagnostics.sio.diag_S0,&y->diagnostics.sio.diag_P1,
   &y->diagnostics.sio.diag_T1,&y->diagnostics.sio.diag_S1,
   &y->diagnostics.sio.BT_vac,&y->diagnostics.sio.VAC_b,
   &y->diagnostics.sio.VAC_a,&y->diagnostics.sio.BTP_curr,
   &y->diagnostics.sio.OUT_curr,
   &y->diagnostics.sio.BTP_secs,
   &y->diagnostics.sio.IN_secs,
   &y->diagnostics.sio.OUT_secs,&y->diagnostics.sio.BT_pb,
   &y->diagnostics.sio.Vple,&y->diagnostics.sio.BT_cb,
   &y->diagnostics.sio.V_cpu,&y->diagnostics.sio.ST_secs,
   &y->diagnostics.sio.DUR_secs);

   if (trace_file != NULL)
    fprintf(trace_file," DECODE_SIO_CTD_DATA successful %d\n",j);

  return(0);
}

/******************************************************************************
  SIO salinity floats have no check sum, so must determine the most
  likely scan.  Each message consists of four scans with first hex
  character giving scan number (0 - 3).  There is also a less frequent
  diagnostic scan which has the first hex character 'E'.  This routine
  compares the hex strings of each of the scan types and finds the most
  common scan of each type.  It then composes a linear array of these
  most likely scans ond returns them in array x.  Returns 0 if successful,
  1 if error
******************************************************************************/
int compose_array (unsigned char *x, unsigned char **hex_data, int n_scans)
{
  int i=0,j,k,l;
  int most_likely[6]={-1,-1,-1,-1,-1,-1},matches[6]={0,0,0,0,0,0};
  
  
  if(debug>0) printf("Entering compose_array function--------------\n");
   
  total_cycles_count++;//added new
    
  
  diagnostic_message = E_diagnostic_message;
  for (j=0; j<6*64; j++)   // Zero returned array
      x[j] = '0';   //Zero array

  if (n_scans < 5)   // Need at least 4 scans to form a message,Note: n_scan is alway larger than the actual number
                     // of scans by 1 in DB's code.
  {
    x[0] = '\0';   //Error - return 0 length array

    if (error_file != NULL)
      fprintf(error_file," ERROR: COMPOSE_ARRAY to few scans to form complete message, n_scans = %d\n",n_scans);

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: COMPOSE_ARRAY to few scans to form complete message, n_scans = %d\n",n_scans);
    
    frame_problem_count++; //added new
    
    return(1);
  }       // end if (n_scans < 5)

// If first character is 'E' then make it '4' -> diagnostic message
// If first character is 'C' then make it '5' -> first cast diagnostic

  for (i=0; i<n_scans; i++)
  {
    // Do not include scans that have position data - hex string is all '0'
    if (strstr( (const char *) hex_data[i],
        "0000000000000000000000000000000000000000000000000000000000000000") != NULL)
       continue;
          
    if (hex_data[i][0] == 'E')
        hex_data[i][0] = '4';
    if (hex_data[i][0] == 'C')
    {
//        diagnostic_message = C_diagnostic_message;
        hex_data[i][0] = '5';
    }

    // Check if first hex character is in range 0-5 */
    if (((k = hex_data[i][0] & 0X07) > 5) || (k < 0))
       continue;

    l = 0;

    for (j=0; j<n_scans; j++)
      if (strcmp( (const char *) hex_data[i], (const char *) hex_data[j]) == 0) // Compare scans
        l++;   //Increment count if scans match

    if (l >= matches[k]) //Does this scan have the most matches?
    {
      most_likely[k] = i; //If yes, store this scan number
      matches[k] = l;
    }
    if ((l==0) && (matches[k]==0)) // Make sure to use messages received only once
    {
      matches[k]=-1;
    }
  }   // end for (i=0; i<n_scans-1; i++)

  diagnostic_message = (matches[4] >matches[5]) ? E_diagnostic_message : C_diagnostic_message;
  if(debug>0){
    printf("matches[4]=%d,matches[5]=%d\n",matches[4],matches[5]);
    if (diagnostic_message == E_diagnostic_message){      
      printf("diagnostic message==E\n");
    }else{
      printf("diagnostic message==C\n");
    }
  }//end of if(debug>0)

  for (j=0; j<=5; j++)
  {
    // If any data type scans missing, return error.  If diagnostic
    // scan missing, no problem
    if ((matches[j] == 0) && (j < 4))
    {
      x[0] = '\0';
      
      if (error_file != NULL)
        fprintf(error_file," ERROR: COMPOSE_ARRAY frame missing, scan %d\n",j);

      if (trace_file != NULL)
        fprintf(trace_file," ERROR: COMPOSE_ARRAY frame missing, scan %d\n",j);
      
      frame_problem_count++; //added new
      
      return 1; //Error return
    }   // end if (matches[j] == 0)

    // Put most likely scans in array in structure
    l = 64*j;
    if (most_likely[j] >= 0)
        for (i=0; i<64; i++)
          x[l+i] = hex_data[most_likely[j]][i];

//    x[l+i] = '\0'; // Append NULL character
  }   // end for (j=0; j<5; j++)
  
  if (trace_file != NULL)
    fprintf(trace_file," COMPOSE_ARRAY successful\n");

  return(0);
}

/******************************************************************************
  Writes output data to disk for floats.
******************************************************************************/
int write_sio_to_disk(char *dir_name, struct solo_ts_meta_struct *s, 
                      struct sio_ctd_out_array *out_array, 
                      unsigned char **hex_data, int j, int write_flg)
{
  struct sio_ctd_raw z;
  struct temp_sal_counts ts;
  static int prof_no=0;
  static float old_day=0;//changed
  int i,n_bins;
  int compose_array_error;
  unsigned char data_array[2048];
  char *ptr_cnt_filename;

  //Reject if only one data scan received
  if (j <= 1)
  {
    if (error_file != NULL)
      fprintf(error_file," ERROR: WRITE_SIO_TO_DISK, single data scan - no output file written\n");

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: WRITE_SIO_TO_DISK, single data scan - no output file written\n");
      
    if(debug>0) printf("single data scan!\n");
    
    return(2);
  }

  // Decode profile
  compose_array_error = compose_array (data_array,hex_data,j); // Write hex_data to
                                                               // data_array
							       
  if(debug>0) printf("compose_array_error=%d\n",compose_array_error);
  
 if (compose_array_error != 0)
   return 1;
  // Raw data is now stored in structures DATA_ARRAY[].
  // Now compute output values from DATA_ARRAY and write to disk
  //
  // WARNING:
  // If the numerator increases by < cycle_time (11 days), the new prof_no will
  // be the same as, and hence overwrite, the old one. So the new cnt file will
  // be written over the old one. This happens occasionally due to a clock drift
  // error in the float. For this reason the number is rounded (by adding .5 and
  // then truncating to an integer). This takes care of clock drift in start_cycle_time.
  //
  // Note: jday is a float (not int,changed) and start_cycle_time and cycle_time are float
  //

  if ((out_array->jday - s->start_cycle_time) < 0)
    return (1);
  if (old_day == 0)
  {
   prof_no = (int) ((out_array->jday - s->start_cycle_time)/s->cycle_time + .5);
   old_day = out_array->jday;
  }
  else if ((out_array->jday - old_day) < 1)
   return 1;
  else
  {
   prof_no = prof_no + (int) ((out_array->jday - old_day)/s->cycle_time + .5);
   old_day = out_array->jday;
  }
  
  //printf("prof_no=%d\n",prof_no);
  
  //if profile number is 0, it should be C diagnostic message, 
  //if profile number is not 0, it should be E diagnostic message
  if(prof_no==0) diagnostic_message = C_diagnostic_message;
  if(prof_no!=0) diagnostic_message = E_diagnostic_message;
  
  if (!write_flg)
	  return 0;
	  
  sprintf (buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,prof_no);
  printf ("Opening output file %s \n",buffer);

  if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
  {
    if (error_file != NULL)
      fprintf(error_file," ERROR: WRITE_SIO_TO_DISK, error opening output file %s\n",buffer);

    if (trace_file != NULL)
      fprintf(trace_file," ERROR: WRITE_SIO_TO_DISK, error opening output file %s\n",buffer);

    return(1);
  }

  if (trace_file != NULL)
    fprintf(trace_file," WRITE_SIO_TO_DISK: output file %s successfully opened\n",buffer);

  // Write Output Profile Count File Name to List File

  // Check if last occurence of '/' in buffer is NULL
  if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
  {
    printf ("\nCan't get cnt file basename %s", buffer);

    if (trace_file != NULL)
      fprintf (trace_file, "\n%s : Can't get cnt file basename %s", PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file, "\n%s: Can't get cnt file basename %s", PRGNAME, buffer);

    return 1;
  } // endif: ptr_cnt_filename

  ptr_cnt_filename++;
  fprintf (list_file,"%s\n", ptr_cnt_filename); // Count File Name

  decode_sio_ctd_data (data_array,&z); // Decode data from data_array and write into z
  compute_sio_ctd_counts(s,&z,&ts); // Compute counts from z and write into ts

  // Write header info to buffer 
  sprintf (buffer,"INTERNAL ID NUMBER                      %04d\n"
                  "WMO ID NUMBER                           %d\n"
                  "TRANSMISSION ID NUMBER                  %d\n"
                  "PROFILE NUMBER                          %d\n"
                  "INSTRUMENT TYPE                         %s\n",
            s->AOML_number,abs(s->WMO_id),s->ARGOS_id,prof_no, s->float_type);
  // Write buffer to out_file
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,"WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
                  "WMO RECORDER TYPE (TABLE 4770)          %d\n"
                  "ARGOS PROGRAM NUMBER                    %d\n"
                  "PI                                      %s\n",
           s->wmo_instrument, s->wmo_recorder, s->exp_no, s->PI);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  if (diagnostic_message == E_diagnostic_message)
  {
      sprintf (buffer,
                    "DRIFT PRESSURE (COUNTS)                 %-6d %-6d\n"
                    "DRIFT TEMPERATURE (COUNTS)              %-6d %-6d\n"
                    "DRIFT SALINITY (COUNTS)                 %-6d %-6d\n"
                    "FALL TIME TO 50 DECIBARS (COUNTS)       %d\n"
                    "SURF PRES BEF RESET(PREV CYC)(COUNTS)   %d\n"
                    "ERROR CODE                              %d\n"
                    "MINIMUM DEPTH BIN WITH GOOD DATA        %d\n"
                    "MAXIMUM DEPTH BIN WITH VALID DATA       %d\n",
               z.pavg1,z.pavg2,z.tavg1,z.tavg2,
               z.diagnostics.sio.SAVG1, z.diagnostics.sio.SAVG1+z.diagnostics.sio.DS,
               z.fallt,z.sprx,z.err,z.imin+1,z.bmax);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      sprintf (buffer,"MISCELLANEOUS OPERATIONS STATUS         %d\n",
               z.diagnostics.sio.BST );
      fwrite (strupr(buffer),1,strlen(buffer),out_file); 

      if (s->before_2004 == YES) {
        sprintf (buffer, "P,T,S END OF DRIFT (COUNTS)             %-6d %-6d %-6d\n",
               z.diagnostics.sio.P1,  z.diagnostics.sio.T1,  z.diagnostics.sio.S1 );
      }
      else {
        sprintf (buffer, "P,T,S BEFORE ASCENT (COUNTS)            %-6d %-6d %-6d\n",
               z.diagnostics.sio.P1,  z.diagnostics.sio.T1,  z.diagnostics.sio.S1 );
      }
      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      sprintf (buffer,"TOTAL TIME FOR PROFILE (SECONDS)        %d\n"
                      "VACUUM BEFORE/AFTER AIR BLADDER FILL    %-6d %-6d\n"
                      "LIMIT SWITCHES IN/OUT                   %-6d %-6d\n"
                      "NUMBER OF BINS WITH BAD DATA            %d\n"
                      "MOTOR CURRENT AT END OF ASCENT (COUNTS) %d\n"
                      "BATTERY CPU (COUNTS)                    %d\n"
                      "BATTERY PUMP (COUNTS)                   %d\n",
               z.diagnostics.sio.SBnscan, 
               z.diagnostics.sio.VACb,z.diagnostics.sio.VACa,
               z.diagnostics.sio.in_lim,z.diagnostics.sio.out_lim,
               z.diagnostics.sio.num_bad,z.diagnostics.sio.PMPc,
               z.diagnostics.sio.Vcpu,z.diagnostics.sio.Vpump);

      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      sprintf (buffer,
//                "AIR PRESSURE END OF LAST SURFACE        %d\n"
//                "AIR PRESSURE START OF LAST SURFACE      %d\n"
                  "SURF PRES AFTER RESET(PREV CYC)(COUNTS) %d\n"
                  "PRESSURE AT END OF DESCENT (COUNTS)     %d\n"
                  "PRESSURE AT END OF ASCENT (COUNTS)      %d\n"
                  "TIME PISTON RAN FIRST SEEK (COUNTS)     %d\n"
                  "CHANGE IN PRESSURE FIRST SEEK (COUNTS)  %d\n"
                  "TIME PISTON UP TO SEEK DEPTH (COUNTS)   %d\n",
//               z.diagnostics.sio.ATE,z.diagnostics.sio.ATS,
               z.diagnostics.sio.PFS,
               z.diagnostics.sio.PFE,z.diagnostics.sio.PRE,z.diagnostics.sio.TSK,
               z.diagnostics.sio.PSK,z.diagnostics.sio.TIP);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  if (diagnostic_message == C_diagnostic_message)
  {
      sprintf (buffer,
                   "MINIMUM DEPTH BIN WITH GOOD DATA        %d\n"
                   "MAXIMUM DEPTH BIN WITH VALID DATA       %d\n",
               z.imin+1,z.bmax);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
      sprintf (buffer,"LIMIT SWITCH STATUS (HEX)               %02x\n"
                    "BIT PRESSURE AFTER RESET OFFSET (CNTS)  %d\n"
                    "P,T,S WHEN IN WATER SENSED (COUNTS)     %-6d %-6d %-6d\n"
                    "P,T,S SHALLOWEST IN PROFILE (COUNTS)    %-6d %-6d %-6d\n",
               z.diagnostics.sio.lim_sw,
               z.diagnostics.sio.SPRXL,
               z.diagnostics.sio.diag_P0,z.diagnostics.sio.diag_T0,z.diagnostics.sio.diag_S0,
               z.diagnostics.sio.diag_P1,z.diagnostics.sio.diag_T1,z.diagnostics.sio.diag_S1);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      sprintf (buffer,"BIT VACUUM (COUNTS)                     %d\n"
                      "VACUUM BEFORE/AFTER FILL(COUNTS)        %-6d %-6d\n"
                      "BIT MOTOR CURRENT OUT (COUNTS)          %d\n"
                      "MOTOR CURRENT OUT TO ASCEND (COUNTS)    %d\n",
               z.diagnostics.sio.BT_vac,
               z.diagnostics.sio.VAC_b,z.diagnostics.sio.VAC_a,
               z.diagnostics.sio.BTP_curr,
               z.diagnostics.sio.OUT_curr);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      sprintf (buffer,"BIT MOTOR TIME OUT (COUNTS)             %d\n"
                      "MOTOR TIME IN TO SINK (COUNTS)          %d\n"
                      "MOTOR TIME OUT TO ASCEND (COUNTS)       %d\n"
                      "BIT PUMP BATTERY (COUNTS)               %d\n"
                      "PUMP BATTERY AT END (COUNTS)            %d\n"
                      "BIT CPU BATTERY (COUNTS)                %d\n",
               z.diagnostics.sio.BTP_secs,
               z.diagnostics.sio.IN_secs,
               z.diagnostics.sio.OUT_secs,
               z.diagnostics.sio.BT_pb,
               z.diagnostics.sio.Vple,
               z.diagnostics.sio.BT_cb);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);

      sprintf (buffer,"BATTERY CPU (COUNTS)                    %d\n"
                      "TIME FROM BLADDERS EMPTY TO START       %d\n"
                      "TIME FROM START TO END FIRST PROFILE    %d\n",
               z.diagnostics.sio.V_cpu,
               z.diagnostics.sio.ST_secs,
               z.diagnostics.sio.DUR_secs);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

//check if the profile data part has data or not
  int has_profile_data=0;//0 no profile data
  
  if (compose_array_error == 0)
  {
    n_bins = (z.bmax > 56) ? 55 : z.bmax-1;
    
    //find if the t=s=0 or not, if it is,then do not print profile data part
    for (i=n_bins; i>=0; i--)
    {
      if(ts.temp[i]!=0.0 | ts.salinity[i]!=0.0){
        has_profile_data=1;
      }      
    }
  }
  if(z.imin<0||z.bmax<0||z.imin>z.bmax){ has_profile_data=0;}


// TS12 has profile data in profile zero -> always write this part ...
//  if (prof_no > 0)
//  {
    if(has_profile_data==1){
    sprintf (buffer,"NUMBER OF COLUMNS                       4\n"
                    "1. COLUMN                               PRESSURE (COUNTS)\n"
                    "2. COLUMN                               TEMPERATURE (COUNTS)\n"
                    "3. COLUMN                               SALINITY (COUNTS)\n"
                    "4. COLUMN                               CHECKSUM FLAG\n");

    fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
//  }
  sprintf (buffer,"==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i = 0;
  sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

 while (out_array->position_jd[i] > 0.0)
 {
 //added new checking to make sure the position line's julian day is larger than the launch time's. by xia
  int iy,imo,ida,ih,im,is,ndoy;
  float pos_jday,launchtime_jday;
  sscanf(out_array->date_time[i],"%4d/%02d/%02d%3d:%02d:%02d",&iy,&imo,&ida,&ih,&im,&is);
  pos_jday=(float) dfr97(iy,imo,ida,ih,im,is);
  launchtime_jday=s->launch_time;
  //printf("%4d/%02d/%02d%3d:%02d:%02d\n",iy,imo,ida,ih,im,is);
  if(pos_jday>launchtime_jday){  
    
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    out_array->latitude[i],out_array->longitude[i],out_array->date_time[i],out_array->n_xmits[i],out_array->sat_name[i],
    out_array->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
  
  i++;
 }


  fprintf (out_file,"==========================================================\n");

  //if something goes wrong with the pressure bin, skip the profile data part printing.
  sprintf (buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,prof_no);
  printf("cnt filename=%s,imin=%d,bmax=%d\n",buffer,z.imin,z.bmax);  
  
  // Write depth, temperature, and salinity to disk
  if (compose_array_error == 0)
  {
    n_bins = (z.bmax > 56) ? 55 : z.bmax-1;
    
    if(has_profile_data==1){
      for (i=n_bins; i>=0; i--)
      {
        sprintf (buffer,"%5d %9.3f %9.3f 0\n",ts.depth[i],ts.temp[i],ts.salinity[i]);
        fwrite (buffer,1,strlen(buffer),out_file);
      }
    }  
  }
  else
  {
    if (trace_file != NULL)
      fprintf (trace_file," ERROR: missing scans - no T,C,D counts written\n");

    if (error_file != NULL)
      fprintf (error_file," ERROR: missing scans - no T,C,D counts written\n");
  }

  fclose (out_file);                        // Close output file 

  if (trace_file != NULL)
    fprintf (trace_file," WRITE_SIO_TO_DISK successful\n");

  return(0);

}

/******************************************************************************
  Main routine for decoding SIO salinity float data.  Reads input
  file and gets data scans for float number = id and profile on
  julian day = jday.  If jday=0, return all profiles.  Returns 0
  if successful, 1 if error
******************************************************************************/
int get_sio_ctd_data (int flg, struct solo_ts_meta_struct *s, char *out_dir)
{
  struct sio_ctd_out_array out_array;
  struct sio_ctd_raw;
  struct temp_sal;

  unsigned char **hex_data,l_c,sat,tmp_buff[buff_size];
  int n_pos=0,n_prof=0,n_l;
  int j=0,i,iy,imo,ida,ih,im,is,g;
  long int dummy;
  float lat,lon;
  int write_sio_to_disk_error;
  //change from int to float
  float jd_old=0,jd;
  
  // Allocate needed memory 
  if ((hex_data = (void *) malloc (max_scans*sizeof(void *))) == NULL)
  {
    printf ("\nError allocating raw out_array");
    return(1);
  }

  for (i=0; i<max_scans; i++)
    if ((hex_data[i] = (unsigned char *) malloc (buff_size)) == NULL)
    {
      printf ("\nError allocating raw out_array");
      return(1);
    }

  for (i=1; i<max_positions; i++)
    out_array.position_jd[i] = -1.0;

  // Read in the data
  j = 0;

  // Reads a line from input_file into buffer and check if NULL
  // (Data accumulates in buffer until a new profile is read)

  while (fgets(buffer,sizeof(buffer),input_file) != NULL)
  {
    // Read chars from buffer, store them in variables and check if
    // exactly 14 were read
    if(sscanf(buffer,"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %64s",
    &i,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,tmp_buff) == 15)
    {
        
// Check if data collected after launch time
// if (dfr97(iy,imo,ida,ih,im,is) < s->launch_time)
      if (dfr97(iy,imo,ida,ih,im,is) < s->start_time)
        continue;
       
      //if (tmp_buff[0] == 'D' && tmp_buff[1] == 'B')
      if(s->ARGOS_id<=32767)       
      {//use 20 bit id,strip first two characters,append "00",copy data to hex_data buffer(XXD)
        strcat (tmp_buff,"00");
        strcpy(hex_data[j],&tmp_buff[2]);
      }
      else
      { //use 28 bit id,copy data to hex_data buffer directly,no striping,appending needed(XXD) 
        strcpy(hex_data[j],tmp_buff);
      }
      // Check if new profile (since data collected every 10 days) and
      // not 1st profile
      jd=(float) dfr97 (iy,imo,ida,ih,im,is);      
      
      //printf("j=%5d, jd_old=%10.6f, jd=%10.6f, diff=%10.6f\n",j,jd_old,jd,jd - jd_old);
      //if (((abs(jd_old - jd)) > 1) && (j>0))
      
      if (((jd - jd_old) > 2.0) && (j>0))
      {
        // Check if you want to include all profiles or just the last one. If so,
        // then read/write profile each time. Otherwise, do not read/write profile.
        // The last profile will be written when the loop exits.
//      if (flg > 0)
        write_sio_to_disk_error=write_sio_to_disk(out_dir,s,&out_array,hex_data,j,flg);
	//printf("j=%d\n",j);
	
	//if processing all profiles, and for some reason it can not write output file,count it
	if(flg==1 && write_sio_to_disk_error==1) no_output_cycle_count++;//added new 
          
     
        strcpy((char *) hex_data[0],(const char *) hex_data[j]); // write  hex_data[j] to
                                                                 // hex_data[0], since 
                                                                 // new profile
// seems to be wrong CS:        j = 1; // Reset to 1 in preparation for next profile (>1 day later)
        j = 0; // Reset to 0 in preparation for next profile (>1 day later)

        for (i=1; i<max_positions; i++)
          out_array.position_jd[i] = -1.0;

        n_pos = 0;
      }

      jd_old = jd; // set to days currently read

      if (n_pos == 0)
        out_array.jday = jd;

      out_array.sat_name[n_pos] = sat;
      out_array.loc_class[n_pos] = l_c;
      out_array.position_jd[n_pos] = dfr97 (iy,imo,ida,ih,im,is);
      out_array.latitude[n_pos] = lat;
      out_array.longitude[n_pos] = lon;

      sprintf(out_array.date_time[n_pos],
              "%4d/%02d/%02d %2d:%02d:%02d",iy,imo,ida,ih,im,is);

      out_array.n_xmits[n_pos] = (int) floor( (float)(n_l-1)/8.);

      if ((lat == (float) 0.0) && (lon == (float) 0.0))
      {
        out_array.latitude[n_pos] = (float) 99.999;
        out_array.longitude[n_pos] = (float) 999.999;
        out_array.n_xmits[n_pos] = 0;
/*
        // Increment j and check if maximum scans exceeded
        if (++j >= max_scans) // hex_data[0],...,hex_data[max_scans-1]
          {if(debug>0) printf("number of lines is greater than max_scans\n");
	   exit (1);
	  }
*/
        n_pos =  (n_pos == 0) ? 1 : n_pos;
      }
      else
      {
        if (++n_pos >= max_positions){
	  printf("number of positions is greater than max_positions\n");
          exit(1);
	} 
      }
      
        // Increment j and check if maximum scans exceeded
      if (++j >= max_scans) // hex_data[0],...,hex_data[max_scans-1]
      { if(debug>0) printf("number of lines is greater than max_scans\n");
	   exit (1);
      }
            
    } // if
  } // while

  // Read/write last profile
  write_sio_to_disk_error=write_sio_to_disk(out_dir,s,&out_array,hex_data,j,1);
  //if processing all profiles, and for some reason it can not write output file,count it
  if(flg==1 && write_sio_to_disk_error==1) no_output_cycle_count++;//added new 
  
  free (hex_data);
  
  //if processing last profile only and it can not output,return 1
  if(flg==0 && write_sio_to_disk_error==1)   return(1);

  //if processing all profiles, not all profiles are generated to output directory,then return  2
  if(flg==1 && no_output_cycle_count>0 && no_output_cycle_count<total_cycles_count) return(2);

  if (trace_file != NULL)
    fprintf (trace_file," GET_SIO_CTD_DATA successful\n");

  return(0);
}

/******************************************************************************
  Assumes str_name = *.hex, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
 char *ptr;

 // Check if pointer to 1st occurrence of .hex in str_name is NULL
  if ((ptr = strstr(str_name,".hex"))!= NULL)
    sprintf (ptr,".meta");
  else
    strcat(str_name,".meta");

  return 0;
}

/******************************************************************************
  Reads meta_file
******************************************************************************/
int read_solo_ts_meta_file (char *meta_name, struct solo_ts_meta_struct *meta_struct)
{
  FILE *meta_file;
  int i,j,syear,smonth,sday,shour,smin;
  int lyear,lmonth,lday,lhour,lmin;
  float x;
  unsigned char *ptr;

  char *cal_keys[] = {"PGAIN=", "POFF=","TGAIN=", "TOFF=","SGAIN=", "SOFF=", 
                      "PMP_GAIN=", "PMP_OFF=", "CPUGAIN=", "CPUOFF=",  
                      "BLOK=", "AV1=", "AV2=", "PB1=", "PB2=", "CDIV=", "\0"};

  meta_struct->cycle_time = (float) 0.0;
  meta_struct->start_time = (float) 0.0;
  meta_struct->start_cycle_time = (float) 0.0;

  printf("\nOpening meta file %s\n",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("Error opening meta file %s\n",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (meta_file != NULL)
    fprintf (meta_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      // Read chars from buffer and store them in AOML_number inside
      // structure.
      //
      // Note: * means the field does not need to be included in list of
      // variable names to write to
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
        meta_struct->AOML_number = 0;
      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
        meta_struct->WMO_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
        meta_struct->ARGOS_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x);
      meta_struct->start_cycle_time += (float) (x/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
      meta_struct->wmo_instrument = 0;
     continue;
    }

    if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
      meta_struct->wmo_recorder = 0;
     continue;
    }

    if (strstr(strupr(buffer),"CYCLE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
      meta_struct->cycle_time = x;
      continue;
    }

    if (strstr(strupr(buffer),"SURFACE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
      meta_struct->start_cycle_time -= (float) (x/24.0);
      continue;
    }

    // start_cycle_time = FIRST DOWN TIME - UP TIME + START TIME
    //                    ^                 ^
    //                    6 hours           1 day     dfr97 days  
    if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) { 
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &sday,&smonth,&syear,&shour,&smin);

      meta_struct->start_time = (float) (dfr97(syear,smonth,sday,shour,smin,0));
      meta_struct->start_cycle_time += meta_struct->start_time;
      continue;
    }

    if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
        meta_struct->exp_no = 0;

      continue;
    }

    if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
    {
      if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
      {
        sprintf (meta_struct->float_type,"\0");
        continue;
      }
    }

    if (strstr(strupr(buffer),"PI     ") != NULL)
    {
      if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
      {
        sprintf (meta_struct->PI,"\0");
        continue;
      }

      if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
        sprintf (meta_struct->PI,"%s",ptr);

      if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
        sprintf (ptr,"\0");

      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &lday,&lmonth,&lyear,&lhour,&lmin);

      meta_struct->launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0));
      if (lyear<2004) {
        meta_struct->before_2004 = YES;
      }
      else {
        meta_struct->before_2004 = NO;
      }
      continue;
    }

    i = 0;

    while ((j = strlen(cal_keys[i])) > 1)
    {
      // Get pointer to 1st occurence of cal_keys[i] in buffer
      if ((ptr = (unsigned char *)strstr(buffer,cal_keys[i])) != NULL)
        meta_struct->cals.c[i] = atof((const char *)(ptr+j));

      i++;
    }
  }

  return 0;
}

/******************************************************************************
  'main' function
******************************************************************************/
int main (int argc, char *argv[])
{
  char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
           *ptr_path_log, *ptr_path_list,
           pathname_input_filename[161]="",
           pathname_trace[161]="", pathname_error[161]="";

  char     input_filename[80], output_directory[256], msg[11]="", title[257]="";
  enum     {LAST=0,ALL} get_which_profile;
  struct   solo_ts_meta_struct s ={0,0,0,0,0,(float)0.0,0,"","",(float)0.0,(float)0.0,(float)0.0};
  unsigned long t;
  int      ret_val = 0;

// Program Name 
// ------------
  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program generates temperature, salinity and pressure counts\n");
  strcat(title, "for SOLO_TS12 Floats");
  printf("%s\n", title);

// Input Paramenters
// -----------------
  if (argc < 7)
  {
    printf("\n");
    printf("hex_infile:    hexadecimal input file name\n");
    printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
    printf("cnt_outdir:    output directory for generated count data file(s)\n");
    printf("trace_outile:  file name for generated trace file\n");
    printf("error_outfile: file name for generated error file\n");
    printf("list_outfile:  file name for generated list file\n");
    return 1;
  }

// Input Hex File name
// -------------------
   printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
   sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
  if (atoi(argv[2]) == 1)
    get_which_profile = ALL;
  else
    get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Output Directory - %s\n", PRGNAME, argv[3]);
  sprintf (output_directory,"%s/",argv[3]);

// TRACE File
// ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  printf ("%s: Opening trace file %s\n", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[4]);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> Trace file opened\n", PRGNAME);
  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// ERROR File
// ----------
  printf ("%s: Opening error file %s\n", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[5]);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf("%s: OK -> Error file opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

// LIST File
// ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    t = time(NULL);
    printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  printf ("%s: Opening list file %s\n", PRGNAME, buffer);
  sprintf (buffer,"%s",argv[6]);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val =1;
    return ret_val;
  }

  printf("%s: OK -> list file opened\n", PRGNAME);
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);


// INPUT Hex File
// --------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): HEXFILEDIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {                                           // Fail to open Hex file
    printf ("\nERROR (%s): Opening input file %s ",
            PRGNAME, pathname_input_filename);
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Input file %s not opened\n",
               PRGNAME, pathname_input_filename);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): input file %s not opened\n",
               PRGNAME, pathname_input_filename);
    ret_val = 1;
  }
  else                                        // Success: Hex file opened
  {
    printf ("\n%s: Input file %s successfully opened",
            PRGNAME, pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file,"%s: Input file %s successfully opened\n",
               PRGNAME, pathname_input_filename);

// META File
// ---------
    if (get_meta_file_name (input_filename) != 0)
    {                                              // Failure: couldn't get meta filename
      printf ("\nERROR (%s): Finding meta file name", PRGNAME);
      if (error_file != NULL)
        fprintf (error_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
  
      ret_val = 1;
    }
    else
    {                                              // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);

        if (trace_file != NULL)
          fprintf (trace_file,
                   "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                      PRGNAME);
        if (error_file != NULL)
            fprintf (error_file,
                     "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                     PRGNAME);

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);
        if (error_file != NULL)
          fclose (error_file);
 
        t = time(NULL);
        printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
        ret_val = 1;
        return(ret_val);
      } // endif: ptr_path_meta

      sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_solo_ts_meta_file (pathname_input_filename, &s) != 0)
      {                                         // Failure: couldn't read meta_file
        printf ("\nERROR (%s): Reading meta file", PRGNAME);
        if (trace_file != NULL)
          fprintf (trace_file, "ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        if (error_file != NULL)
          fprintf (error_file,"ERROR (%s): Reading meta file %s\n",
                   PRGNAME, pathname_input_filename);
        ret_val = 1;
      }
      else                                      // Success: read meta_file
      {
        if (trace_file != NULL)
          fprintf (trace_file,"%s: Meta file %s successfully read\n",
                   PRGNAME, pathname_input_filename);
        printf ("%s: Meta file %s successfully read\n",
                   PRGNAME, pathname_input_filename);

// Make profile(s) selection
// -------------------------
      ret_val=get_sio_ctd_data(get_which_profile,&s,output_directory);
/*      
        if (get_sio_ctd_data(get_which_profile,&s,output_directory) != 0)
          ret_val = 1;
*/
      } // endif: read_solo_ts_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file (open hex file)

  printf("HEX filename=%s\n",argv[1]);
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  
  if(get_which_profile==1){
    printf("frame missing problem counts=%d\n",frame_problem_count); 
    printf("number of profiles which failed to produce cnt file=%d\n",no_output_cycle_count);
  }
  printf("return status=%d\n",ret_val);

/*  
  if((frame_problem_count>0) && (frame_problem_count<total_cycles_count)){
    ret_val=2;
  }
*/  

// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
    fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close data files
// ----------------
  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
   fclose (list_file);

  return(ret_val);
}
