/******************************************************************************

   hex2cnt_ts14.cpp, 
   revision:  13-08-2004 (XXD) add adjustStartRowEndRow function
              02-06-2004 (XXD) debug version,changed the profile number computation part
   ==================

   Converts the 'hex' data portion (last part) of each record to counts.   
   
  To compile: g++ -o hex2cnt_ts14.x hex2cnt_ts14.cpp

  Usage: hex2cnt_ts14.sh

Author: Xiangdong Xia, Aug 13 2004  
                       May 30 2003
                        
  Note: TS14 use direct difference for profile data, TS16 use forward difference instead.
        
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <vector>
#include <time.h>
#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <algorithm>


#define MAXLINES  60000
#define HEXRECLEN 100
#define BINRECLEN 400
#define DECRECLEN 100
using namespace std;

FILE *input_file, *out_file, *error_file, *trace_file, *list_file;
char PRGNAME[]="hex2cnt_ts14";
int debug=0;//1 for debug, 0 for operation

// Calibration values for Scripps style salinity float
struct sio_cals{ 
                  double pgain;
                  double poff;
                  double tgain;
                  double toff;
                  double cgain;		  		  
                  double coff;
                  double p1;
                  double p2;
                  double p3;
                  double dp1;
                  double dp2;
		  double dp3;
		  double cpugain;
		  double cpuoff;
		  double auxgain;
		  double auxoff;
                };


union u_cals{ struct sio_cals s;
              double c[sizeof(struct sio_cals)/sizeof(double)];};

//ts14 float meta data struct
struct ts14_meta_struct{
	      int AOML_number;
              int WMO_id;
              int ARGOS_id;
              int wmo_instrument;
              int wmo_recorder;
              float cycle_time;
              int exp_no;
              char float_type[40];
              char PI[60];
              float launch_time;
              float start_cycle_time;
              char hex_id[20];//newly added hexadecimal ARGOS ID
	      
	      int year;
    	      int month;
   	      int day;
 	      int hour;
	      int minute;
	      float lat;
	      float latm;
	      float lon;
	      float lonm;
	      union u_cals cals;
};


//TS14 float engineering technical message struct
struct ts14_technical_struct{			   
  	     int Pinit; //init pressure
	     int Tinit; //init temperature	   
             int Cinit; //init conductivity
  	     int R0;    //reference resistance measurement
             int R50;	//reference resistance measurement		   
	     int R100;  //reference resistance measurement
	     int Npts;  //number of bins for the profile
  	     int Pd1;   //first half of drift pressure
	     int Td1;   //first half of drift temperature
	     int Cd1;   //first half of drift conductivity
	     int Pd2;   //second half of drift pressure
	     int Td2;   //second half of drift temperature			   
	     int Cd2;   //second half of drift conductivity
             int Vbaux; //voltage of aux
 	     int Vbcpu; //voltage of cpu
	     int Vac;   //voltage of ac
	     int SysFlg;//System Flag
};


//position and sateliate information struct,used for output file between==== and =====
struct position_info_struct {
              vector<float> lat_vector;  //latitude vector
              vector<float> lon_vector;  //longitude vector
	      vector<int>   year_vector; //year vector
   	      vector<int>   month_vector;//month vector
	      vector<int>   day_vector;  //day vector
       	      vector<int>   hour_vector; //hour vector
              vector<int>   min_vector;  //minute vector
              vector<int>   sec_vector;  //second vector
              vector<int>   xmits_vector;//transmits vector
	      vector<char>  sat_vector;  //sateliate type vector
              vector<char>  class_vector;//class vector
};

//drift pressure,temperature and salinity values from technical message type 5, used for count output header part
struct driftPTC_struct{
              vector<int>   driftP0_vector;//original drift pressure vector
              vector<int>   driftT0_vector;//original drift temperature vector
              vector<int>   driftC0_vector;//original drift conductivity vector for FSI or salinity for SBE
	       
              vector<int>   driftP_vector;//adjusted drift pressure vector
              vector<int>   driftT_vector;//adjusted drift temperature vector
              vector<int>   driftC_vector;//adjusted drift conductivity vector  for FSI or salinity for SBE	       	       
              vector<int>   driftPump_vector;//drift pump vector
};


double dfr97 (const int yp, const int mp, const int dp,const int hr, const int minutes, const int secs);
long ymd2julday (int, int, int);
char  *strupr(char* s);
int binchrstr2dec(char *binvalue);
void *getsubstr(int begin, int end, char *s, char *ss);
char chr_xor(char x, char y);
int twoscompl2dec(char *binvalue);
int get_meta_file_name (char *str_name);
//added functions
const char* string2char(string str);
string chararray2string(char a[],int begin,int length);
int bin2int_unsigned(string  bin);
string bitInv(string input);
string hex2bin(string hex);
string int2bin(int input);
string int2bin12bits(int input);

//these functions are specific to TS14
int read_ts14_meta_file(char *meta_name, struct ts14_meta_struct &meta_struct);
int write_to_disk(char* argv[],struct ts14_technical_struct ts,struct ts14_meta_struct ms,
                  struct position_info_struct ps,struct driftPTC_struct ds,
		  vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& C_Vector,
		  vector<int>& CRC_Vector,int prof_no);
void decodeTS14(char *argv[],char* pathname_input_filename, struct ts14_meta_struct& ms);
int getCRC(char *data, int count);
int most_likely_frame(vector<int> messtype, vector<int> messnum,vector<string> hexdata,
                      vector<int> start_row,vector<int> end_row,
                      int fileno,vector<int> crc_status,int message_type,int message_num,int good_bad);
void decodeMessageType1(struct ts14_technical_struct& ts,vector<int> messtype,vector<int> messnum,
                        vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,
			vector<int> crc_status,struct ts14_meta_struct ms);
void decodeMessageType0(vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& C_Vector,vector<int>& CRC_Vector,
                        struct ts14_technical_struct ts,struct ts14_meta_struct ms,vector<int> messtype,vector<int> messnum,
			vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,vector<int> crc_status);
void  decodeMessageType2(struct driftPTC_struct& ds,struct ts14_technical_struct ts,
                         vector<int> messtype,vector<int> messnum,
                         vector<string> hexdata,vector<int> start_row,vector<int> end_row,
			 int fileno,vector<int> crc_status);       
void  resetVectors(vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& C_Vector,vector<int>& CRC_Vector,
                   struct position_info_struct& ps,struct driftPTC_struct& ds);
void read_position(struct position_info_struct& ps,vector<int> year_ps,vector<int> month_ps,
                   vector<int> day_ps,vector<int> hour_ps,vector<int> minute_ps,vector<int> second_ps, 
		   vector<float> lat_ps,vector<float> lon_ps,vector<float> nl_ps,vector<char> sattype_ps,
		   vector<char> lc_ps,int fileno,vector<int> start_row,vector<int> end_row);
void  cleanStartRowEndRow(vector<int>& start_row,vector<int>& end_row);			
void adjustStartRowEndRow(vector<int>& start_row,vector<int> end_row,vector<float> jdayarray,struct ts14_meta_struct ms);

int main(int argc, char *argv[])
{

  char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
           *ptr_path_log, *ptr_path_list,buffer[512],
           pathname_input_filename[161]="",
	   pathname_input_meta_filename[161]="",
	   hex_filename[161]="",
           pathname_trace[161]="", pathname_error[161]="";

  char     input_filename[80];
  char     output_directory[256];
  enum     {LAST=0,ALL} get_which_profile;

  long t;

  int      ret_val = 0;
  struct   ts14_meta_struct ms;


  // Program Name
  if (argc < 7)
  {
    printf("Program generates temperature, conductivity and pressure counts for SOLO TS14 Floats\n");
    printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
           PRGNAME);
    printf("\n");
    printf("hex_infile:    hexadecimal input file name\n");
    printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
    printf("cnt_outdir:    output directory for generated count data file(s)\n");
    printf("trace_outile:  output directory for generated trace file\n");
    printf("error_outfile: output directory for generated error file\n");
    printf("list_outfile:  output directory for generated list file\n");
    return 1;
  }

  sprintf (input_filename,"%s",argv[1]);

  // Number of profiles to process: 0, get last profile or 1, get all profiles
  if (atoi(argv[2]) == 1)
    get_which_profile = ALL;
  else
    get_which_profile = LAST;

  // Output directory path for count file(s)
  // ---------------------------------------
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  sprintf (output_directory,"%s/",argv[3]);

  // TRACE File
  // ----------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    ret_val = 1;
    return(ret_val);
  }

  sprintf (buffer,"%s",argv[4]);
  printf ("%s:Opening trace file %s\n",PRGNAME,buffer);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nError opening trace file %s",buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("%s: Trace file opened succesfully\n",PRGNAME);

  if (trace_file != NULL)
  {
    t = time(NULL);
    fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }

  // ERROR File
  // ----------
  sprintf (buffer,"%s",argv[5]);

  printf ("%s:Opening error file %s",PRGNAME,buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("Error(%s): opening error file %s\n",PRGNAME,buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("\nOK . error file opened\n");

  if (trace_file != NULL)
    fprintf (trace_file,"%s: Error file %s successfully opened\n",PRGNAME,buffer);

  if (error_file != NULL)
  {
    t = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
  }



  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
           PRGNAME);
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[6]);

  printf ("\nOpening list file %s",buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    printf ("\nError opening list file %s\n",buffer);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Error opening list file %s\n",
               PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Error opening list file %s\n",
              PRGNAME, buffer);

    ret_val =1;
    return ret_val;
  }

  printf("\nOK . list file opened\n");

  if (trace_file != NULL)
    fprintf (trace_file," list file %s successfully opened\n", buffer);

  // INPUT Hex File
  // --------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): HEXFILES_DIR <-- no such environment variable\n", PRGNAME);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: HEXFILES_DIR <-- no such environment variable\n");

    if (error_file != NULL)
      fprintf (error_file," ERROR: HEXFILES_DIR <-- no such environment variable\n");

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);

    if (error_file != NULL)
      fclose (error_file);

    ret_val = 1;
    return(ret_val);
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {                                           // Fail to open Hex file
    printf ("\nError opening input file %s", pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: input file %s not opened\n", pathname_input_filename);

    if (error_file != NULL)
      fprintf (error_file," ERROR: input file %s not opened\n", pathname_input_filename);

    ret_val = 1;
  }
  else                                        // Success: Hex file opened
  {
    printf ("\nInput file %s successfully opened", pathname_input_filename);

    if (trace_file != NULL)
      fprintf (trace_file," Input file %s successfully opened\n", pathname_input_filename);

    // META File
    // ---------
    if (get_meta_file_name (input_filename) != 0)
    {                                              // Failure: couldn't get meta filename
      printf ("\nError finding meta file name");

      if (error_file != NULL)
        fprintf (error_file, " ERROR: meta file name construction - returned name %s\n",
                 input_filename);

      if (trace_file != NULL)
        fprintf (trace_file, " ERROR: meta file name construction - returned name %s\n",
                 input_filename);

      ret_val = 1;
    }
    else
    {                                              // Success: got meta filename
      if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
      {
        printf("METAFILES_DIR <-- no such environment variable\n");

        if (trace_file != NULL)
          fprintf (trace_file," ERROR: METAFILES_DIR <-- no such environment variable\n");

        if (error_file != NULL)
          fprintf (error_file," ERROR: METAFILES_DIR <-- no such environment variable\n");

        if (trace_file != NULL)                  // Close Log files
          fclose (trace_file);

        if (error_file != NULL)
          fclose (error_file);

        ret_val = 1;
        return(ret_val);
      } // endif: ptr_path_meta

      sprintf(pathname_input_meta_filename, "%s/%s", ptr_path_meta, input_filename);

      if (read_ts14_meta_file (pathname_input_meta_filename, ms) != 0)
      {                                         // Failure: couldn't read meta_file
        printf ("\nError reading meta file");

        if (error_file != NULL)
          fprintf (error_file," ERROR reading meta file %s\n", pathname_input_meta_filename);

        if (error_file != NULL)
          fprintf (error_file," ERROR reading meta file %s\n", pathname_input_meta_filename);

        ret_val = 1;
      }
      else                                      // Success: read meta_file
      {
        if (trace_file != NULL)
          fprintf (trace_file," Meta file %s successfully read\n", pathname_input_meta_filename);

        // Make profile(s) selection
        // ------------------------
//	decodeTS14(P_Vector,T_Vector,C_Vector,argv,tech,ms,ps,ds);
	decodeTS14(argv,pathname_input_filename,ms);

      } // endif: read_type14_meta_file
    } // endif: get_meta_file_name
  } // endif: input_file (open hex file)

  // Write finish message to trace & error files
  // -------------------------------------------
  t = time(NULL);
  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: finished %s\n\n", PRGNAME, ctime(&t));

// Close data files
// ----------------

  if (input_file != NULL)
    fclose (input_file);

  if (trace_file != NULL)
    fclose (trace_file);

  if (error_file != NULL)
    fclose (error_file);

  if (list_file != NULL)
   fclose (list_file);

  return(ret_val);

}






/**********************************************************************
                          Read meta data file
***************************************************************************/

int read_ts14_meta_file (char *meta_name, struct ts14_meta_struct &meta_struct)
{
  FILE *meta_file;
  int i,j,syear,smonth,sday,shour,smin;
  float x,lat,latm,lon,lonm;
  int lyear,lmonth,lday,lhour,lmin;
  char *ptr;
  char buffer[512];
  
  char manufacturer[20];
  char fsi[]="FSI";
  char sbe[]="SBE";
  
  char *cal_keys[] = {"PGAIN=", "POFF=","TGAIN=", "TOFF=","CGAIN=", "COFF=","SGAIN=", "SOFF=",
   "P1=", "P2=","P3=","DP1=", "DP2=","DP3=","CPUGAIN=","CPUOFF=","AUXGAIN=","AUXOFF=","\0"};

  meta_struct.cycle_time = (float) 0.0;
  meta_struct.start_cycle_time = (float) 0.0;

  printf("\nOpening meta file!!!!!! %s\n",meta_name);

  if ((meta_file = fopen(meta_name,"r")) == NULL)
  {
    printf ("Error opening meta file!!!!!! %s\n",meta_name);

    if (error_file != NULL)
      fprintf (error_file," ERROR opening meta file %s\n",meta_name);

    return(1);
  }

  if (trace_file != NULL)
    fprintf (meta_file," Meta file %s successfully opened\n",meta_name);

  // Reads a line from meta_file into buffer and checks if NULL
  while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
  {
    // Convert buffer to upper case, return a pointer to the first
    // occurrence of INTERNAL ID NUMBER in buffer, and check if NULL
    if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
    {
      // Read chars from buffer and store them in AOML_number inside
      // structure.
      //
      // Note: * means the field does not need to be included in vector of
      // variable names to write to
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.AOML_number) != 1)
        meta_struct.AOML_number = 0;
      continue;
    }

    if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.WMO_id) != 1)
        meta_struct.WMO_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.ARGOS_id) != 1)
        meta_struct.ARGOS_id = 0;
      continue;
    }

    if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x);
      meta_struct.start_cycle_time += (float) (x/24.0);
      continue;
    }

    if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct.wmo_instrument) != 1)
      meta_struct.wmo_instrument = 0;
     continue;
    }

   //newly added sensor manufacturer 
    if (strstr(strupr(buffer),"TEMPERATURE SENSOR MANUFACTURER") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %s",&manufacturer) != 1)
      sprintf (manufacturer,"\0");
     continue;
    }
    

    if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct.wmo_recorder) != 1)
      meta_struct.wmo_recorder = 0;

	 continue;
    }

// DOWN TIME was changed to CYCLE TIME in meta files (CS)
    if (strstr(strupr(buffer),"CYCLE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
      meta_struct.cycle_time = x;
      continue;
    }

// UP TIME was changed to SURFACE TIME in meta files (CS)
    if (strstr(strupr(buffer),"SURFACE TIME") != NULL)
    {
      sscanf(buffer,"%*s %*s %*s %f",&x);
// (CS) now obsolete     meta_struct.cycle_time += (float) (x/24.0);
      meta_struct.start_cycle_time -= (float) (x/24.0);
      continue;
    }

    // start_cycle_time = FIRST DOWN TIME - SURFACE TIME + START TIME
    //            ^                 ^
    //            6 hours           1 day     dfr97 days
    if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &sday,&smonth,&syear,&shour,&smin);

      meta_struct.start_cycle_time += (float) (dfr97(syear,smonth,sday,shour,smin,0));
      continue;
    }

    if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
    {
      if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct.exp_no) != 1)
        meta_struct.exp_no = 0;

      continue;
    }
    //newly added hexadecimal ARGOS ID field 
    if (strstr(strupr(buffer),"HEXADECIMAL ARGOS ID") != NULL)
    {
     if (sscanf(buffer,"%*s %*s %*s %s",&meta_struct.hex_id) != 1)
      sprintf (meta_struct.hex_id,"\0");
     continue;
    }
     

    if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
    {
      if (sscanf(buffer,"%*s %*s %s",meta_struct.float_type) == 0)
      {
        sprintf (meta_struct.float_type,"\0");
        continue;
      }
    }

    if (strstr(strupr(buffer),"PI     ") != NULL)
    {
      if (sscanf(buffer,"%*s %s",meta_struct.PI) == 0)
      {
        sprintf (meta_struct.PI,"\0");
        continue;
      }

      if ((ptr = strstr(&buffer[3],meta_struct.PI)) != NULL)
        sprintf (meta_struct.PI,"%s",ptr);

      if ((ptr = strstr(meta_struct.PI,"\n")) != NULL)
        sprintf (ptr,"\0");

      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
           &lday,&lmonth,&lyear,&lhour,&lmin);
      meta_struct.launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0));

      meta_struct.year=lyear;
      meta_struct.month=lmonth;
      meta_struct.day=lday;
      meta_struct.hour=lhour;
      meta_struct.minute=lmin;
      continue;
    }

    if ((strstr(strupr(buffer),"LAUNCH POSITION") != NULL) && (strstr(strupr(buffer),"LONM") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d %d %d %d",&lat,&latm,&lon,&lonm);
      meta_struct.lat=lat;
      meta_struct.latm=latm;
      meta_struct.lon=lon;
      meta_struct.lonm=lonm;
      continue;
    }

//need to be careful here since it will accomodate both FSI and SBE meta data calibration coefficients
    i = 0;
    while ((j = strlen(cal_keys[i])) > 1)
    {
      // Get pointer to 1st occurence of cal_keys[i] in buffer
      if ((ptr = (char *)strstr(buffer,cal_keys[i])) != NULL){
        meta_struct.cals.c[i] = atof((const char *)(ptr+j));
      }else{//changed here
        meta_struct.cals.c[i] = 0.0;
      }
      i++;
    }

  }
  //cout<<"Read Meta data file finished"<<endl;
  
  //check if the wmo_instrument type is consistent with the manufacturer, if not, exit with status 1
  if((meta_struct.wmo_instrument==852) && (strcmp(manufacturer,fsi)!=0)) {
     printf ("Error in meta file %s, manufacturer is not consistent with WMO instrument type\n",meta_name);

     if (error_file != NULL)
     fprintf (error_file,"Error in meta file %s, manufacturer is not consistent with WMO instrument type\n",meta_name);

     return(1);
  }
  
  return 0;
}




/******************************************************************************************
 Decode TS14 float to counts, counts are stored in PU,TU,CU vectors with duplicated removed
 Considered both Up and Down profile cases(Key part of Decoding)
*******************************************************************************************/

void decodeTS14(char *argv[], char* pathname_input_filename, struct ts14_meta_struct& ms){
  FILE  *fpprofile;
/*
  for simplicity, C_Vector also can represent salinity,when output,
  it is decided by WMO_Instrument Type
*/  
  vector<int> P_Vector,T_Vector,C_Vector,CRC_Vector;
  struct ts14_technical_struct ts;
  struct position_info_struct ps;
  struct driftPTC_struct ds;
  
  
/* The following arrays store the data related to the profile, 
   later they are filtered to only get positional data  
*/
  vector<int> year_ps,month_ps,day_ps,hour_ps,minute_ps,second_ps;
  vector<float> lat_ps,lon_ps,nl_ps;
  vector<char>  sattype_ps,lc_ps;
  vector<float>  jdayarray;
  vector<int> start_row,end_row;//the start_row and end_row of each profile are stored here
  vector<int> start_row_ps,end_row_ps;//the start_row and end_row of position information for each profile are stored here  
  vector<int> messtype,messnum;//message type and number for each hexdata line
  vector<int> crc_status;//1 means passed crc check, 4 means doesnot pass crc check,0 means does not apply  
  vector<string> hexdata;//hex data,used for comparision in most_likely_frame function      
  
  char hexrecord[HEXRECLEN]="", binrecord[BINRECLEN]="";
  char hexrecord_tmp[HEXRECLEN]="",hex_partial[HEXRECLEN]="";  
  char ss[35]="",sat,l_c;
  char* out_dir;
  char temp[6]="";//newly added temp array to contain the original crc 
  char crcinput[100]="";//assembled getCRC function input, it is 28 bit ARGOS ID plus 29 bytes data*/  
  char zero[2]="0"; //preappend a '0' to ARGOS ID.
  char twozeros[3]="00";//preappend two zero to hexdata if ARGOS ID>=32768  
  int  year, month, day, hour, minute, second,n_l;    
  int  i,j,k,cntr=0;
  int fileno=-1;
  int nprofile,index;
  int num=-1;//used to get beginning and ending line number of a profile  
  int prof_no=0,prof_no_prev=-1;//profile number(current and previous cycle)//added new.change prev=-1
  int jd,old_jd;//julian day  
  int total_cycles;//total cycles  
  //message num and maximum message number,most common message index(from most_likely_frame function),bin number of T,C for profile data per profile
  float latitude,longitude,jdaydiff=0;  
  


    
  out_dir=argv[3];  
  //get which profile, 1--get all profile, 0--get last profile only
  if(atoi(argv[2])>=1) nprofile=1;else nprofile=0;

  // Open Profile File
  printf("PROFILE WHOLE PATH=%s\n",pathname_input_filename);  
  if ((fpprofile = fopen(pathname_input_filename, "r")) == NULL)
  {
    printf (" >>> Profile Open Error <<<\n");
    //exit(1);
  }

  while (fscanf(fpprofile,"%*s %d %d %d %d %d %d %f %f %*s %d %c %c %*s %s",
         &year, &month, &day, &hour, &minute, &second,&latitude,&longitude,&n_l,&sat,&l_c, hexrecord_tmp) != EOF)
  {
    if (strlen(hexrecord_tmp) != 64)
      continue;

    /* if ARGOS ID >=32768, shift hex data string to the right by one byte.
    */
    hexrecord[0]='\0';//every loop,reset hexrecord,otherwise, it will keep strcating.
    if(ms.ARGOS_id<32768){
      strcpy(hexrecord,hexrecord_tmp);      
    }else{
      strcat(hexrecord,twozeros);
      getsubstr(0,61,hexrecord_tmp,hex_partial);
      strcat(hexrecord,hex_partial);
    }
    
// CRC test
    strcat(crcinput,zero);         //preappend a '0' to ARGOS ID 	
    if(strlen(ms.hex_id)==7){//normally hex_id should be 8 digits,if it is 7 digits,preappend 0 in the front
     strcat(crcinput,ms.hex_id);     //28 bits ARGOS ID
    }
    if(strlen(ms.hex_id)==8){//normally hex_id should be 8 digits,if it is 7 digits,preappend 0 in the front
     strcat(crcinput,&ms.hex_id[1]);     //28 bits ARGOS ID,discard the leading character
    }    
     strcat(crcinput,&hexrecord[6]); //29 bytes hex data
     //extract crc from original hex data
    for(i=2;i<=5;i++){
       temp[i-2]=hexrecord[i];
    }    
    strcat(crcinput,temp);          //append extracted crc from original hex data    
    if (getCRC(crcinput,35) != 0){//crc failed
      crc_status.push_back(4);
    }else{//crc succeeded
      crc_status.push_back(1);
    }
    crcinput[0]='\0';//reset crcinput for next loop count
    
    //newly added for use in most_likely_frame function    
    hexdata.push_back(chararray2string(hexrecord,0,64));
    
    lat_ps.push_back(latitude);
    lon_ps.push_back(longitude);
    nl_ps.push_back(n_l);
    sattype_ps.push_back(sat);
    lc_ps.push_back(l_c);
    
    year_ps.push_back(year);
    month_ps.push_back(month);
    day_ps.push_back(day);
    hour_ps.push_back(hour);
    minute_ps.push_back(minute);
    second_ps.push_back(second);
    jdayarray.push_back((float)dfr97(year,month,day,hour,minute,second));

    //get message type for each hexline
    strcpy(binrecord,string2char(hex2bin(chararray2string(hexrecord,0,64))));
    if(debug==1) cout<<" cntr="<<cntr<<" binary="<<binrecord<<endl;
    
    getsubstr(24,26,binrecord,ss);
    messtype.push_back(binchrstr2dec(ss));
    
    //get message number for each hexline
    getsubstr(27,31,binrecord,ss);
    messnum.push_back(binchrstr2dec(ss));
    
    cntr++;        
  }//end of while loop
  if(debug==1){  
   cout<<lat_ps.size()<<" "<<lon_ps.size()<<" "<<nl_ps.size()<<" "<<sattype_ps.size()<<" "<<lc_ps.size()
   <<" "<<year_ps.size()<<" "<<month_ps.size()<<" "<<day_ps.size()<<" "<<hour_ps.size()<<" "<<minute_ps.size()
   <<" "<<second_ps.size()<<" "<<jdayarray.size()<<" "<<messtype.size()<<" "<<messnum.size()
   <<" "<<hexdata.size()<<endl;
   
   for(i=0;i<lat_ps.size();i++){
     cout<<"i="<<i<<" messtype="<<messtype[i]<<" messnum="<<messnum[i]<<" "<<year_ps[i]<<" "<<month_ps[i]
     <<" "<<day_ps[i]<<" "<<hour_ps[i]<<" "<<minute_ps[i]<<" "<<second_ps[i]<<" "<<lat_ps[i]<<" "<<lon_ps[i]<<endl;
   }
  }//end of if(debug==1)
  
  //get start_row and end_row number for each profile
  start_row.push_back(0);

  for(i=0;i<cntr;i++){
   if(i==0){jdaydiff=0;}
   else{
   jdaydiff=jdayarray[i]-jdayarray[i-1];
   }
   if(jdaydiff>2){
    start_row.push_back(i);
   }
  }//end of for loop

  for(index=1;index<start_row.size();index++){
     end_row.push_back(start_row[index]-1);
  }
  end_row.push_back(cntr-1);//last row number

  //display start_row and end_row to see if they are correct   
  if(debug==1){   
    for(j=0;j<start_row.size();j++){
     cout<<"before cleaning ghost messages/cycles,fileno= "<<j<<" start_row= "<<start_row[j]<<" "<<"end_row= "<<end_row[j]<<endl;
    }
  }//end of if(debug==1)
  //clean ghost messages/cycles, i.e. if a cycle only contain one line, skip it,
  //remove it from start_row,end_row vectors to make them clean
  cleanStartRowEndRow(start_row,end_row);

  //display start_row and end_row to see if they are correct   
  if(debug==1){   
    for(j=0;j<start_row.size();j++){
      cout<<"after cleaning ghost messages/cycles,fileno= "<<j<<" start_row= "<<start_row[j]<<" "<<"end_row= "<<end_row[j]<<endl;
    }
  }//end of if(debug==1)

  //adjust start_row's line number based on launch_time, if in one cycle, the date is larger
  //than launch_time, then record the first row when this happens and reset this cycle's start_row
  //value to this recorded row number. This will make sure that we do not skip any profile with 
  //valid data in it.
  
  adjustStartRowEndRow(start_row,end_row,jdayarray,ms);
  //display start_row and end_row to see if they are correct   
  if(debug==1){
    for(j=0;j<start_row.size();j++){
    cout<<"after adjustment,fileno= "<<j<<" start_row= "<<start_row[j]<<" "<<"end_row= "<<end_row[j]<<endl;
    }
  }//end of if(debug==1)
  total_cycles=start_row.size();
  if(debug==1) cout<<"total_cycles="<<total_cycles<<endl;
     
/* ------------seperate profiles and generate counts, Up profile core processing --------------------------*/
  //added new
  int times=0;//times when jdayarray[start_row[k]]>=ms.launch_time
  
  for(k=0;k<total_cycles;k++){
     //for each cycle
   //added new
   if(jdayarray[start_row[k]]>=ms.launch_time){
     fileno=k;//need to compute fileno.     
     if(debug==1) cout<<"k=======  "<<fileno<<"  ================================================================="<<endl;
     times++;
     if(times==1){
       jd=(int)dfr97(year_ps[start_row[k]],month_ps[start_row[k]],day_ps[start_row[k]],0,0,0);
       prof_no=(int)floor((double)((jd-ms.start_cycle_time)/ms.cycle_time+0.5));	  
       //prof_no_prev=prof_no;//added new,comment out this line
       if(debug==1) cout<<"k="<<k<<" prof_no computed="<<prof_no<<endl;
       old_jd=jd;
     }//end of if(times==1)
     
     if(nprofile==0){//only last profile
       if(k!=(int)start_row.size()-1){
       //if it is not the last profile, accumate prof_no,skip the computation       
        jd=(int)dfr97(year_ps[start_row[k]],month_ps[start_row[k]],day_ps[start_row[k]],0,0,0);	       
        prof_no=prof_no+(int)((jd-old_jd)/ms.cycle_time+0.5);
        if(debug==1) cout<<"xxx k="<<k<<" prof_no computed="<<prof_no<<endl;
        old_jd=jd;
        continue;//if it is not the last profile, skip,jump to last profile part
       }
       else {//it is the last profile, read its position part
        jd=(int)dfr97(year_ps[start_row[k]],month_ps[start_row[k]],day_ps[start_row[k]],0,0,0);	       
        prof_no=prof_no+(int)((jd-old_jd)/ms.cycle_time+0.5);            
        if(debug==1) cout<<"last profile="<<k<<" prof_no="<<prof_no<<endl;
	read_position(ps,year_ps,month_ps,day_ps,hour_ps,minute_ps,second_ps,
	lat_ps,lon_ps,nl_ps,sattype_ps,lc_ps,fileno,start_row,end_row);	                 
       }
     }//end of if(nprofile==0)
          
     if(nprofile==1){//all of the profile

	//if for all of the profiles, accumate prof_no,do the computation.   
	if(times>1){//added new
            jd=(int)dfr97(year_ps[start_row[k]],month_ps[start_row[k]],day_ps[start_row[k]],0,0,0);
	    //if(debug==1) cout<<" nprofile=all,jd="<<jd<<" old_jd="<<old_jd<<" prof_no before adding="<<prof_no<<endl;	       
            prof_no=prof_no+(int)((jd-old_jd)/ms.cycle_time+0.5);
            //if(debug==1) cout<<" nprofile=all,k="<<k<<" adding="<<(int)((jd-old_jd)/ms.cycle_time+0.5)<<" prof_no computed="<<prof_no<<endl;
            old_jd=jd;
        }
	//only output to file if the julian day is larger than deployment time 
	if(jdayarray[start_row[fileno]]>=ms.launch_time){
	    read_position(ps,year_ps,month_ps,day_ps,hour_ps,minute_ps,second_ps,
	    lat_ps,lon_ps,nl_ps,sattype_ps,lc_ps,fileno,start_row,end_row);	        
        }
      }//end of if(nprofile==1)
               

/************************************************************************************           
     //find most likely frame for message type 1-----technical engineering data
************************************************************************************/
     decodeMessageType1(ts,messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,ms);
/*****************************************************************************            
     //find most likely frame for message type 0-----profile data
******************************************************************************/ 
     decodeMessageType0(P_Vector,T_Vector,C_Vector,CRC_Vector,ts,ms,messtype,messnum,hexdata,start_row,end_row,fileno,crc_status);    
/*************************************************************************************    
     //find most likely frame for message type 2-----drift data
*************************************************************************************/    
     decodeMessageType2(ds,ts,messtype,messnum,hexdata,start_row,end_row,fileno,crc_status);
    
    //create output filename
    if(nprofile==1){//all of the profile
    //only output to file if the julian day is larger than deployment time and prof_no is different from
    //prvious one.
       //if((jdayarray[start_row[fileno]]>=ms.launch_time)&&((k==0)||(k!=0 && prof_no!=prof_no_prev))){	    
       //added new
       if((jdayarray[start_row[fileno]]>=ms.launch_time)&&(prof_no!=prof_no_prev)){	    
	    cout<<"Writing to output file no="<<prof_no<<endl; 
            write_to_disk(argv,ts,ms,ps,ds,P_Vector,T_Vector,C_Vector,CRC_Vector,prof_no);
        }
     }else{//last profile
       if(fileno==(int)start_row.size()-1){
            cout<<"Writing to output file no="<<prof_no<<endl; 
	    write_to_disk(argv,ts,ms,ps,ds,P_Vector,T_Vector,C_Vector,CRC_Vector,prof_no);		
       }

    }//end of if(nprofile==1)

   
    //reset or clear the vectors, to reuse them in next iteration step
    resetVectors(P_Vector,T_Vector,C_Vector,CRC_Vector,ps,ds);

    prof_no_prev=prof_no;
 }//end of if >=ms.launch_time    
}//end of for(k=0)

}//end of void decodets14 function




/*******************************************************************************************
                          Write output result to file
********************************************************************************************/
int write_to_disk(char *argv[],struct ts14_technical_struct ts,struct ts14_meta_struct ms, struct position_info_struct ps,struct driftPTC_struct ds,vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& C_Vector,vector<int>& CRC_Vector,int prof_no){

  char buffer[512];
  char* dir_name;
  char* ptr_cnt_filename;
  char date_time[32];
  int j;
  
  dir_name=argv[3];
  
  sprintf (buffer,"%s/%04d_%05d_%03d.cnt",dir_name,ms.AOML_number,ms.ARGOS_id,prof_no);
  cout<<"BUFFER -> Printing result to outputfile: "<<prof_no<<"output filename="<<buffer<<endl;
  
 if (trace_file != NULL)
    fprintf (trace_file,"Opening output file %s...\n",buffer);


 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  //cout<<"output filename can not be created"<<endl;
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }



 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);



// Write Output Profile Count File Name to vector File
// -------------------------------------------------
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s : Canot get cnt file basename %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Canot get cnt file basename %s\n", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename

  
  
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);

 printf("NAME TO LIST -> ptr_cnt_filename: %s\n", ptr_cnt_filename);
 
 
  // Write header info to buffer
  sprintf (buffer,"INTERNAL ID NUMBER                      %04d\n"
                  "WMO ID NUMBER                           %d\n"
                  "TRANSMISSION ID NUMBER                  %05d\n"
                  "PROFILE NUMBER                          %d\n"
                  "INSTRUMENT TYPE                         %s\n",
            ms.AOML_number,abs(ms.WMO_id),ms.ARGOS_id,prof_no,ms.float_type);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);


  sprintf (buffer,"WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
                  "WMO RECORDER TYPE (TABLE 4770)          %d\n"
                  "ARGOS PROGRAM NUMBER                    %d\n"
                  "PI                                      %s\n",
           ms.wmo_instrument, ms.wmo_recorder, ms.exp_no, ms.PI);

  fwrite (strupr(buffer),1,strlen(buffer),out_file);


  sprintf(buffer, "INITIAL PRESSURE (BAR)                  %d\n"
                  "INITIAL TEMPERATURE (COUNTS)            %d\n"
		  "INITIAL CONDUCTIVITY (COUNTS)           %d\n"
		  "REFERENCE RESISTANCE R0                 %d\n",
               ts.Pinit,ts.Tinit,ts.Cinit,ts.R0);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "REFERENCE RESISTANCE R50                %d\n"
                  "REFERENCE RESISTANCE R100               %d\n"
		  "NUMBER OF POINTS                        %d\n"
		  "START OF DRIFT PRESSURE (COUNTS)        %d\n",
               ts.R50,ts.R100,ts.Npts,ts.Pd1);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "START OF DRIFT TEMPERATURE (COUNTS)     %d\n"
                  "START OF DRIFT CONDUCTIVITY (COUNTS)    %d\n"
		  "END OF DRIFT PRESSURE (COUNTS)          %d\n"
		  "END OF DRIFT TEMPERATURE (COUNTS)       %d\n",
               ts.Td1,ts.Cd1,ts.Pd2,ts.Td2);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf(buffer, "END OF DRIFT CONDUCTIVITY (COUNTS)      %d\n"
                  "VOLTAGE OF CPU (COUNTS)                 %d\n"
		  "VOLTAGE OF AUX (COUNTS)                 %d\n"
		  "INTERNAL VACUUM (COUNTS)                %d\n"
		  "SYSTEM FLAG                             %d\n",
               ts.Cd2,ts.Vbcpu,ts.Vbaux,ts.Vac,ts.SysFlg);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

//drift pressure vector values
  sprintf (buffer, "DRIFT PRESSURE (COUNTS)                 ");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

   //cout<<"from write function ds.driftP_vector.size()"<<ds.driftP_vector.size()<<endl;
    
  for(j=0;j<ds.driftP_vector.size();j++){
     sprintf(buffer,"%-6d ",ds.driftP_vector[j]);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }  
  
  if(ds.driftP_vector.size()==0){
     sprintf(buffer,"%-6d ",99999);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
  
  sprintf(buffer,"\n");
  
  fwrite (strupr(buffer),1,strlen(buffer),out_file);



//drift temperature vector values
  sprintf (buffer,"DRIFT TEMPERATURE (COUNTS)              ");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  for(j=0;j<ds.driftT_vector.size();j++){
     sprintf(buffer,"%-6d ",ds.driftT_vector[j]);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  if(ds.driftT_vector.size()==0){
     sprintf(buffer,"%-6d ",99999);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  sprintf(buffer,"\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  if(ms.wmo_instrument==852){
  //drift conductivity vector values
  sprintf (buffer, "DRIFT CONDUCTIVITY (COUNTS)             ");
  }
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  for(j=0;j<ds.driftC_vector.size();j++){
     sprintf(buffer,"%-6d ",ds.driftC_vector[j]);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
  
  if(ds.driftC_vector.size()==0){
     sprintf(buffer,"%-6d ",99999);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  sprintf(buffer,"\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

//drift pump vector values
  sprintf (buffer, "DRIFT PUMP CORRECTION                   ");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  for(j=0;j<ds.driftPump_vector.size();j++){
     sprintf(buffer,"%-6d ",ds.driftPump_vector[j]);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }
  
  if(ds.driftPump_vector.size()==0){
     sprintf(buffer,"%-6d ",99999);
     fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  sprintf(buffer,"\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);



  if(ms.wmo_instrument==852){
    sprintf (buffer,  "NUMBER OF COLUMNS                       4\n"
                  "1. COLUMN                               PRESSURE (COUNTS)\n"
                  "2. COLUMN                               TEMPERATURE (COUNTS)\n"
                  "3. COLUMN                               CONDUCTIVITY (COUNTS)\n"
                  "4. COLUMN                               CHECKSUM FLAG\n");
  }

  fwrite (strupr(buffer),1,strlen(buffer),out_file);




  sprintf (buffer,"==========================================================\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  int i = 0;
  sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  for(i=0;i<ps.sat_vector.size();i++)
  {
    //           "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
    sprintf(date_time,"%4d/%02d/%02d%3d:%02d:%02d",ps.year_vector[i],ps.month_vector[i],ps.day_vector[i],ps.hour_vector[i],ps.min_vector[i],ps.sec_vector[i]);

    sprintf (buffer,"    %+7.3f   %+8.3f %s   %03d   %1c     %1c\n",
             ps.lat_vector[i], ps.lon_vector[i],date_time,
             ps.xmits_vector[i], ps.sat_vector[i],ps.class_vector[i]);

    fwrite (strupr(buffer),1,strlen(buffer),out_file);

  }


  fprintf (out_file,"==========================================================");

  // Write depth, temperature, and conductivity or salinity to disk

   //cout<<"P_Vector Size in write routine "<<P_Vector.size()<<endl;
   //deal with the last line 0 0 0 problem, if last point is multiple of 9 and t=c=0, do not print,else print
    int last=(int)T_Vector.size()-1; cout<<"T_Vector.size="<<T_Vector.size()<<" last="<<last<<endl;
    if(!((last%9==0) && T_Vector[last]==0 && C_Vector[last]==0)){
      sprintf (buffer,"\n %5d %5d %5d  %d",P_Vector[last],T_Vector[last],C_Vector[last],CRC_Vector[last]);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);    
    }
    
    for (i=(int)T_Vector.size()-2; i>=0; i--)
    {
      sprintf (buffer,"\n %5d %5d %5d  %d",P_Vector[i],T_Vector[i],C_Vector[i],CRC_Vector[i]);
      fwrite (strupr(buffer),1,strlen(buffer),out_file);
    }
    fprintf(out_file,"\n");

// Close output file

    if (trace_file != NULL)
      fprintf (trace_file," WRITE_SIO_TO_DISK successful\n");
  if(out_file!=NULL)
  fclose(out_file);

  return(0);
}





/******************************************************************************
  Computes days from 1997
******************************************************************************/
double dfr97 (const int yp, const int mp, const int dp,
              const int hr, const int minutes, const int secs)
{
  int    yr, mo, dy, day_from_97;
  long   passed_jd, jd_xx0101, jd_970101;
  double d_jd;

  yr = yp;
  mo = mp;
  dy = dp;

  if (yr <= 15)
   yr += 2000;
  else if (yr < 1900)
   yr += 1900;

  if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
   return (0.0);

  passed_jd = ymd2julday( yr, mo, dy ) + 1;     // (current day/current month/current
                                                //  year)
  dy = 1;
  mo = 1;
  jd_xx0101 = ymd2julday( yr, mo, dy );         // (1/1/current year)
  yr = 1997;
  jd_970101 = ymd2julday( yr, mo, dy );         // (1/1/1997)
  day_from_97 = (int) (passed_jd - jd_970101);  // Days from 1/1/1997 (ignoring hrs,
                                                // mins, secs)
  d_jd = day_from_97;
  d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0; // Converts hrs, mins, secs to days
  return (d_jd);                                // and adds this to d_jd
}

/***************************************************
Convert year,month,day to julian day
*****************************************************/

long ymd2julday(int year, int month, int day)
{
   int  ic, iy;
   long julday;

   if ( month > 2 )
     month = month - 3;
   else if ( month < 3 )
   {
     month = month + 9;
     year = year - 1;
   }

   ic = (int) floor((double)(year/100) );
   iy = year - 100*ic;

// printf("ic: %d iy: %d\n", ic, iy);

   julday =   (long) floor((double)(146097*ic)/4)
             + (long) floor((double)(1461*iy)/4)
             + (long) floor((double)(153*month+2)/5)
             + day + 1721119;
   julday = julday - 2440000;
   return julday;
}



/******************************************************************************
  Converts an ASCII string to uppercase
******************************************************************************/
char  *strupr(char* s)
{
  char* p = s;
  while (*p) {
    if (islower((int)*p)) *p = toupper((int)*p);
    ++p;
  }
  return s;
}


/****************************************************************************
convert binary string to decimal values
***************************************************************************/

int binchrstr2dec(char *binvalue)
{
  int decvalue, len, i, j,
      iplace, accum=0, pwof2;
  char tmpchr[2]="";

  len=strlen(binvalue);

  for (i=len; i>0; i--)
  {
    tmpchr[0]=binvalue[i-1];
    tmpchr[1]='\0';

    iplace=atoi(tmpchr);

    pwof2=1;

    for (j=0; j<len-i; j++)
      pwof2*=2;

    accum+=iplace*pwof2;
  }

  return(accum);
}


/**********************************************************************************
get substring of a string s,from index begin to end, result is stored in string ss
***********************************************************************************/
void *getsubstr(int begin, int end, char *s, char *ss)
{
  int j;

  for (j=begin; j<=end; j++)
    ss[j-begin]=s[j];

  ss[end-begin+1]='\0';
}




/************************************************************************************
character xor operation z=x (xor) y
*************************************************************************************/
char chr_xor(char x, char y)
{
  char z;
  z=(x==y)?'0':'1';
  return z;
}






/**************************************************************************************
convert two's complement value of a string binvalue to decimal values
***************************************************************************************/

int twoscompl2dec(char *binstr)
{
  int decvalue, len, i, j,
      iplace, accum=0, pwof2;
  char tmpchr;
  char binvalue[256]="";
  
  strcpy(binvalue,binstr);
  len=strlen(binvalue);

  if (binvalue[0]=='1')
  {
    for (i=0; i<len; i++)
      binvalue[i]=chr_xor(binvalue[i],'1');

    decvalue=binchrstr2dec(binvalue);
    decvalue++;
    decvalue*=-1;
  }
  else if (binvalue[0]=='0')
    decvalue=binchrstr2dec(binvalue);

  return(decvalue);
}




/******************************************************************************
  Assumes str_name = *.hex, returns *.meta in str_name
******************************************************************************/
int get_meta_file_name (char *str_name)
{
 char *ptr;

 // Check if pointer to 1st occurrence of .hex in str_name is NULL
  if ((ptr = strstr(str_name,".hex"))!= NULL)
    sprintf (ptr,".meta");
  else
    strcat(str_name,".meta");

  return 0;
}


const char* string2char(string str){
  const char* array;
  array=str.c_str();
  return array;
}

string chararray2string(char a[],int begin,int length){
  string temp="";

  for(int i=begin;i<begin+length;i++){
   temp=temp+a[i];
  }

  return temp;
}


//convert binary string(unsigned) to int values
int bin2int_unsigned(string  bin)
{
int sum = 0;

for (int i=0;i <bin.length();i++)
{
       if (bin[i] == '0'){
         sum *= 2;
         continue;
	   } else
       if (bin[i] == '1'){
		   sum = (sum*2) + 1;
           continue;
	   }else{
        break;
	   }
}

return sum;

}

//inverse the bit from 1 to 0, from 0 to 1, C++ version
string bitInv(string input){

int i;

   for(i=0;i<input.size();i++){
     if(input[i]=='0')
     input[i]='1';
     else
     input[i]='0';
   }
return input;
}


//convert hex string to binary string

string hex2bin(string hex){
   string temp="",result="";
   int i;
   for(i=0;i<hex.size();i++){
	   if(hex.at(i)=='0'){temp="0000";}
	   if(hex.at(i)=='1'){temp="0001";}
	   if(hex.at(i)=='2'){temp="0010";}
	   if(hex.at(i)=='3'){temp="0011";}
	   if(hex.at(i)=='4'){temp="0100";}
	   if(hex.at(i)=='5'){temp="0101";}
	   if(hex.at(i)=='6'){temp="0110";}
	   if(hex.at(i)=='7'){temp="0111";}
           if(hex.at(i)=='8'){temp="1000";}
	   if(hex.at(i)=='9'){temp="1001";}
	   if((hex.at(i)=='A')||(hex.at(i)=='a')){temp="1010";}
	   if((hex.at(i)=='B')||(hex.at(i)=='b')){temp="1011";}
	   if((hex.at(i)=='C')||(hex.at(i)=='c')){temp="1100";}
	   if((hex.at(i)=='D')||(hex.at(i)=='d')){temp="1101";}
	   if((hex.at(i)=='E')||(hex.at(i)=='e')){temp="1110";}
	   if((hex.at(i)=='F')||(hex.at(i)=='f')){temp="1111";}
       result=result+temp;
   }
   return result;
}

//convert integer to 2 bytes hex data,then to binary string
string int2bin(int input){
  char buffer[10];
  string a,result;
  sprintf(buffer,"%04x",input);
  a=chararray2string(buffer,0,4);
  result=hex2bin(a);
  return result;
}

//convert integer to binary string, only 12 bits of binary string will be extracted
string int2bin12bits(int input){
  string result;
  string a=int2bin(input);  
  result=a.substr(4,12);
  return result;
}



/* The following function does CCITT CRC16 x16 + x12 + x5 + 1 generator polynomial 
*  The initial crc must be 0x0000, not 0xFFFF for TS14 float to work.
*
*  If the data hex string last four characters(hex) contain the CRC from float, then
*  the function will return 0 if it matches the float original CRC, otherwise, it will not match
*
*  If the data hex string does not include CRC from float,then it will not do comparision, just
*  return the CRC caculated by the algorithm. In the main program, you should compare it with original 
*  CRC.
*  
*  char* data -----contains hex string
*  int count  -----contains the number of bytes
*/
int getCRC(char *data, int count)
{
unsigned int idx,i,k;
unsigned short const g=0x1021;   /* x16 + x12 + x5 + 1 generator polynomial */
unsigned short d, fcs, t;
unsigned int dd;
unsigned table[256];

 for (i=0; i<256; i++)
  {
  fcs=0x0000;
  d = i << 8;
  for (k=0; k<8; k++)
   {
    if ((fcs ^ d) & 0x8000)
      fcs = (fcs << 1) ^ g;
    else
      fcs = (fcs << 1);
    d <<= 1;
   }
   table[i]=fcs;
   
  }

// fcs=0xFFFF;    /* initial FCS value this does not work for TS14 */
 fcs=0x0000;    /* initial FCS value */
 for (i=0; i<count; i++)
{
   t=fcs;
   
   sscanf(&data[i*2],"%2x",&dd);
    
   d=dd;

   idx=(t>>8) ^ d; //cout<<"i= "<<i<<"  d= "<<d<<"  t="<<t<<"  t>>8 ="<<(t>>8)<<"  index="<<idx;
   
   fcs=table[idx];
   fcs=fcs ^ (t<<8);
}
//printf "result:\n";
//  cout<<"crc="<<fcs<<endl;
return(fcs);
}




/* get the index of most likely frame, if message_num is 1 or 2, ignore the comparison with message_num
  
*/


int most_likely_frame(vector<int> messtype, vector<int> messnum,vector<string> hexdata,
                      vector<int> start_row,vector<int> end_row,
                      int fileno,vector<int> crc_status,int message_type,int message_num,int good_bad){
  int i,j,matches=0;
  int num_matches,most_likely=-1;
  
  for(i=start_row[fileno];i<=end_row[fileno];i++){
    
    if((crc_status[i]==4) && (good_bad)) continue;

    if((messtype[i]!=message_type) || (messnum[i]!=message_num)) continue;
    //cout<<"fileno="<<fileno<<" i="<<i<<" messtype="<<messtype[i]<< " message_type="<<message_type<<" here"<<endl;              
    num_matches=0;

    for(j=start_row[fileno];j<=end_row[fileno];j++){              
      if((messtype[j]==message_type) && (messnum[j]==message_num)){
       if(hexdata[i]==hexdata[j]) num_matches++;       
      }
    }//end of for(j=)
    
    if(num_matches>matches){ //is this the most matches?
      matches=num_matches; //Yes, save match count
      most_likely=i;       //save record number
    }
    if((num_matches==0) && (matches==0)){//make sure to use messages only received once
      most_likely=i;
    }
    
  }//end of for(i=)
  return most_likely;
}

//decode message type0,profile data for one cycle for ts14
void decodeMessageType0(vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& C_Vector,vector<int>& CRC_Vector,
                        struct ts14_technical_struct ts,struct ts14_meta_struct ms,vector<int> messtype,vector<int> messnum,
			vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,vector<int> crc_status){     
     int max_message_num,message_num,ww,bin_num,pos1,pos2,j,kkk;
     int T0,C0;//T0,C0 are reference temperature and conductivity/salinity
     int tempT,tempC;//they store the final accumulative counts for each bin_num
     int most_common=-1;//most likely frame index
     char binrecord[BINRECLEN]="",ss[35]="";//binrecord is binary string for one hex string,ss is a substring     
     vector<int> dt,dc;//they store forward difference of tempterature and conductivity/salinity counts
          
     //find the maximum message number according to Npts from engineering message     
     max_message_num=(int)ceil((double)ts.Npts/9.0);
          
     bin_num=-1;//bin number for one profile, bin number corresponds to pressure
     if(debug==1) cout<<" for type 0 profile data,max_message_num="<<max_message_num<<" Npts="<<ts.Npts<<endl;
    for(message_num=1;message_num<=max_message_num;message_num++){
     pos1=63;
     if((j = most_likely_frame(messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,0,message_num,1)) >= 0) //find most likely - good CRC only
      {most_common = j;}
     else 
     if((j = most_likely_frame(messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,0,message_num,0)) >= 0) //find most likely - any CRC
      {most_common = j;}
     else{ 
       most_common = -1;
       if (trace_file != NULL) fprintf (trace_file,"most_common return -1 when dealing with message type 0\n");
       if (error_file != NULL) fprintf (error_file,"most_common return -1 when dealing with message type 0\n");
     }
     
     if(debug==1) cout<<"most common for type 0="<<most_common<<endl;
     if(most_common==-1){//missing message_num,which means the messages are not complete,fill 0 into P,T,C,crc
      //if(debug==1) cout<<"XXXXXXXXXXXXXXXXXXXXXXXXXX  profile data part,most_common=-1,messagenum not continuous"<<endl;
      for(kkk=0;kkk<9;kkk++){
         T_Vector.push_back(0);
	 C_Vector.push_back(0);
	 P_Vector.push_back(++bin_num);
         CRC_Vector.push_back(0);	
	 if((bin_num+1)>=ts.Npts) break; 	
      }                   
     }
     
     
     if(most_common!=-1){
       //decode profile data here
       strcpy(binrecord,string2char(hex2bin(hexdata[most_common])));
       getsubstr(32,47,binrecord,ss); T0=binchrstr2dec(ss);//reference temperature         
       getsubstr(48,63,binrecord,ss); C0=binchrstr2dec(ss);//reference conductivity/salinity         
       //get dt,dc,put into vector                  

       if(dt.size()>0) dt.clear();
       if(dc.size()>0) dc.clear();

       for(kkk=1;kkk<=8;kkk++){
          if (pos1>255) break;
        //get dt
          pos1++; pos2=pos1+12; if (pos2>255) break;
          getsubstr(pos1,pos2,binrecord,ss); 
	  //cout<<"before ss="<<ss<<" dt="<<twoscompl2dec(ss)<<endl;
	  dt.push_back(twoscompl2dec(ss));
	  //cout<<"after  ss="<<ss<<" dt="<<twoscompl2dec(ss)<<endl;
	  //cout<<hexdata[most_common]<<endl;
          pos1=++pos2;  if (pos1>255) break;
	  
        //get ds
          pos2=pos1+10; if (pos2>255) break;
	  getsubstr(pos1,pos2,binrecord,ss);  dc.push_back(twoscompl2dec(ss));
		    
  	  pos1=pos2;    if (pos1>255) break;
       }//end of for(kkk=1,kkk<=8) loop       

       for(kkk=0;kkk<dt.size();kkk++){
          //cout<<"check just after assigned dt="<<dt[kkk]<<endl;
       }
       
       //assemble  T_Vector,C_Vector,P_Vector,CRC_Vector here,first put initial values
         T_Vector.push_back(T0);
	 C_Vector.push_back(C0);
	 P_Vector.push_back(++bin_num);
         CRC_Vector.push_back(crc_status[most_common]);	 
	 tempT=T0;  	 
	 tempC=C0;

       for(kkk=0;kkk<8;kkk++){//one message has 9 pair of T,C values         
/*  use first order difference       
         tempT=tempT+dt[kkk]; 
	 tempC=tempC+dc[kkk]; 
*/	 
/*  use direct difference
*/
         tempT=T0+dt[kkk];
	 tempC=C0+dc[kkk];
         T_Vector.push_back(tempT);
         C_Vector.push_back(tempC);       
         P_Vector.push_back(++bin_num);
         CRC_Vector.push_back(crc_status[most_common]);
         if((bin_num+1)>=ts.Npts) break;
       }//end of for(kkk=0) loop
                     
//       if(dt.size()>0) dt.clear();
//       if(dc.size()>0) dc.clear();
     }//end of if(most_common!=-1)
    }//end of for(message_num) loop

    

//debug testing    
    for(j=0;j<(int)T_Vector.size()-1;j++){
      if(abs(T_Vector[j+1]-T_Vector[j])>4096 && C_Vector[j+1]!=0 && CRC_Vector[j+1]!=0){
        T_Vector[j+1]=T_Vector[j+1]-4096*2;
      }
      //cout<<"after adjustment T="<<T_Vector[j]<<endl;
    }
}//end of decodeMessageType0 function    


//decode message type1,technical engineering message for one cycle for ts14,used in header part

void decodeMessageType1(struct ts14_technical_struct& ts,vector<int> messtype,vector<int> messnum,
                        vector<string> hexdata,vector<int> start_row,vector<int> end_row,int fileno,
			vector<int> crc_status,struct ts14_meta_struct ms){       
     int j,most_common=-1;
     char binrecord[BINRECLEN]="",ss[35]="";
     
     if ((j = most_likely_frame(messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,1,0,1)) >= 0) //find most likely - good CRC only
       most_common = j;
     else 
     if ((j = most_likely_frame(messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,1,0,0)) >= 0) //find most likely - any CRC
       most_common = j;
     else{ 
       most_common = -1;
       if (trace_file != NULL) fprintf (trace_file,"most_common return -1 when dealing with message type 1\n");
       if (error_file != NULL) fprintf (error_file,"most_common return -1 when dealing with message type 1\n");
     }          
          
     if(most_common!=-1){
       
       strcpy(binrecord,string2char(hex2bin(hexdata[most_common])));
       //get engineering data from most likely frame,put into technical message struct
       getsubstr( 32, 39,binrecord,ss); ts.Pinit=binchrstr2dec(ss);//initial pressure
       getsubstr( 40, 55,binrecord,ss); ts.Tinit=binchrstr2dec(ss);//initial temperature
       getsubstr( 56, 71,binrecord,ss); ts.Cinit =binchrstr2dec(ss);//initial conductivity       
       
       getsubstr( 72, 87,binrecord,ss); ts.R0    =binchrstr2dec(ss);//R0     
       getsubstr( 88,103,binrecord,ss); ts.R50   =binchrstr2dec(ss);//R50
       getsubstr(104,119,binrecord,ss); ts.R100  =binchrstr2dec(ss);//R100
       getsubstr(120,127,binrecord,ss); ts.Npts  =binchrstr2dec(ss);//number of points
       getsubstr(128,143,binrecord,ss); ts.Pd1   =binchrstr2dec(ss);//first half of drift pressure
       getsubstr(144,159,binrecord,ss); ts.Td1   =binchrstr2dec(ss);//first half of drift temperature
       getsubstr(160,175,binrecord,ss); ts.Cd1   =binchrstr2dec(ss);//first half of drift conductivity(FSI)
       getsubstr(176,191,binrecord,ss); ts.Pd2   =binchrstr2dec(ss);//second half of drift pressure
       getsubstr(192,207,binrecord,ss); ts.Td2   =binchrstr2dec(ss);//second half of drift temperature
       getsubstr(208,223,binrecord,ss); ts.Cd2   =binchrstr2dec(ss);//second half of drift conductivity(FSI)
       getsubstr(224,231,binrecord,ss); ts.Vbaux =binchrstr2dec(ss);//voltage of aux
       getsubstr(232,239,binrecord,ss); ts.Vbcpu =binchrstr2dec(ss);//voltage of cpu
       getsubstr(240,247,binrecord,ss); ts.Vac   =binchrstr2dec(ss);//internal vacuum
       getsubstr(248,255,binrecord,ss); ts.SysFlg=binchrstr2dec(ss);//system flag
     }
}//end of decodeMessageType1 function

//decode message type2,drift data for one cycle for ts14,used in header part
void  decodeMessageType2(struct driftPTC_struct& ds,struct ts14_technical_struct ts,
                         vector<int> messtype,vector<int> messnum,
                         vector<string> hexdata,vector<int> start_row,vector<int> end_row,
			 int fileno,vector<int> crc_status){       
			  
    int n,j,pos3,pos4,kkk,drift_pair_num,most_common=-1;
    char binrecord[BINRECLEN]="",ss[35]="";    
    list<int> message_num_drift_list;
    vector<int> message_num_drift_vector;
    int max_message_num=(int)ceil((double)ts.Npts/9.0);      
    
    //first get the message_num_drift_list 
    for(n=start_row[fileno];n<=end_row[fileno];n++){
      if(messtype[n]==2){//if message_type=2         
         message_num_drift_list.push_back(messnum[n]);
      }      
    }//end of for n=
    message_num_drift_list.sort();
    message_num_drift_list.unique();
    

    
    //copy to vector
    for(list<int>::iterator it=message_num_drift_list.begin();it!=message_num_drift_list.end();it++){
     message_num_drift_vector.push_back(*it);     
    }
    
    
    //delete those message number which is less than max_message_num from message_num_drift_vector
    for(n=(int)message_num_drift_vector.size()-1;n>=0;n--){
     if(message_num_drift_vector[n]<=max_message_num){
       message_num_drift_vector.erase(message_num_drift_vector.begin(),message_num_drift_vector.begin()+n+1);     
       break;
     }
    }
/*
    //check to see the message number after deletion(mentioned above)
    for(n=0;n<message_num_drift_vector.size();n++){
     cout<<"after deletion,message number="<<message_num_drift_vector[n]<<endl;
    }
*/    

    //make it continuous sequence,no jump message_num,for example,1,2,3,5,7,8 will become 1,2,3
    for(n=0;n<(int)message_num_drift_vector.size()-1;n++){
     if(message_num_drift_vector[n+1]-message_num_drift_vector[n]>1){
       message_num_drift_vector.erase(message_num_drift_vector.begin()+n+1,message_num_drift_vector.end());     
     }
    }
         
    for(n=0;n<message_num_drift_vector.size();n++){ 
     pos3=31;
     if ((j = most_likely_frame(messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,2,message_num_drift_vector[n],1)) >= 0) //find most likely - good CRC only
       most_common = j;
     else 
     if ((j = most_likely_frame(messtype,messnum,hexdata,start_row,end_row,fileno,crc_status,2,message_num_drift_vector[n],0)) >= 0) //find most likely - any CRC
       most_common = j;
     else{ 
      most_common = -1;
      if (trace_file != NULL) fprintf (trace_file,"most_common return -1 when dealing with message type 2\n");
      if (error_file != NULL) fprintf (error_file,"most_common return -1 when dealing with message type 2\n");
     }
          
     //if(debug==1) cout<<"most common for type 2="<<most_common<<" message_num="<<message_num_drift_vector[n]<<endl;
          
     if(most_common!=-1){
       strcpy(binrecord,string2char(hex2bin(hexdata[most_common])));
       //printf("for drift,binrecord length=%d, string=%s,most common=%d\n",strlen(binrecord),binrecord,most_common);
         //decode drift data
	  pos3++;
	 for (kkk=1; kkk<=16; kkk++){
	  pos4=pos3+11; if(pos4>255) break;
          getsubstr(pos3,pos4,binrecord,ss); 
 	    if(kkk%4==1){
	      ds.driftP0_vector.push_back(binchrstr2dec(ss));
	      //cout<<"drift P="<<binchrstr2dec(ss)<<endl;
	    }else
	    if(kkk%4==2){
	      ds.driftT0_vector.push_back(binchrstr2dec(ss));
	      //cout<<"drift T="<<binchrstr2dec(ss)<<endl;
	    }else
            if(kkk%4==3){
	      ds.driftC0_vector.push_back(binchrstr2dec(ss));
	      //cout<<"drift C="<<binchrstr2dec(ss)<<endl;
	    }else
	    if(kkk%4==0){
	      if(binchrstr2dec(ss)>2047){ ds.driftPump_vector.push_back(binchrstr2dec(ss)-4096);
	      }else
	      {ds.driftPump_vector.push_back(binchrstr2dec(ss));}
	    }	  
          pos3=++pos4; if(pos3>255) break;
         }//end of for(k=1;k<=8) loop

     
       drift_pair_num=(int)(ds.driftP0_vector.size()/4);  
       //cout<<"internal fileno="<<fileno<<" drift_pair_num="<<drift_pair_num<<endl;
     
    }//end of if(most_common!=-1)
    }//end of for(n=0) loop

    //adjust drift P,T,C vector based on 4bits of Pd1,Cd1,Td1,Pd2,Cd2,Td2 and 12 bits of P,T,C values
    //cout<<"fileno="<<fileno<<" CONDUCTIVITY LEADING 4 BITS ARE="<<int2bin(ts.Cd1).substr(0,4)<<endl;
    if(int2bin(ts.Cd1).substr(0,4)=="0000"){// if this condition is met, most likely the float is not in water
           cout<<" the float is not in water!"<<endl;
	   
           for(n=0;n<ds.driftP0_vector.size();n++){   
	    if(ds.driftC0_vector[n]<=ts.Cd1+5){//out of water
	     ds.driftP_vector.push_back(bin2int_unsigned(int2bin(ts.Pd1).substr(0,4)+int2bin12bits(ds.driftP0_vector[n])));
             ds.driftT_vector.push_back(bin2int_unsigned(int2bin(ts.Td1).substr(0,4)+int2bin12bits(ds.driftT0_vector[n])));	   
             ds.driftC_vector.push_back(bin2int_unsigned(int2bin(ts.Cd1).substr(0,4)+int2bin12bits(ds.driftC0_vector[n])));	                            
	    }else{//under water
	     ds.driftP_vector.push_back(bin2int_unsigned(int2bin(ts.Pd2).substr(0,4)+int2bin12bits(ds.driftP0_vector[n])));
             ds.driftT_vector.push_back(bin2int_unsigned(int2bin(ts.Td2).substr(0,4)+int2bin12bits(ds.driftT0_vector[n])));	   
	     ds.driftC_vector.push_back(bin2int_unsigned(int2bin(ts.Cd2).substr(0,4)+int2bin12bits(ds.driftC0_vector[n])));	   	     
	    }//end of if(ds.driftC0)        	    	    	    	    	 	    
	   }//end of for
    }else{ //print warning message
           if((int2bin(ts.Pd1).substr(0,4)!=int2bin(ts.Pd2).substr(0,4))||(int2bin(ts.Td1).substr(0,4)!=int2bin(ts.Td2).substr(0,4))||(int2bin(ts.Cd1).substr(0,4)!=int2bin(ts.Cd2).substr(0,4))){
	     //cout<<"Leading 4 bits of P start="<<int2bin(ts.Pd1).substr(0,4)<<" Leading 4 bits of P end="<<int2bin(ts.Pd2).substr(0,4)<<endl;
             //cout<<"Leading 4 bits of T start="<<int2bin(ts.Td1).substr(0,4)<<" Leading 4 bits of T end="<<int2bin(ts.Td2).substr(0,4)<<endl;	     
	     //cout<<"Leading 4 bits of C start="<<int2bin(ts.Cd1).substr(0,4)<<" Leading 4 bits of C end="<<int2bin(ts.Cd2).substr(0,4)<<endl;
	     if (trace_file != NULL) fprintf (trace_file,"The leading 4 bits of drift P,T and/or C start and end differ!\n");
             if (error_file != NULL) fprintf (error_file,"The leading 4 bits of drift P,T and/or C start and end differ!\n");
	   } 
	   
	   for(n=0;n<ds.driftP0_vector.size();n++){            	    	    	    	    	 	    
	    ds.driftP_vector.push_back(bin2int_unsigned(int2bin(ts.Pd2).substr(0,4)+int2bin12bits(ds.driftP0_vector[n])));
            ds.driftT_vector.push_back(bin2int_unsigned(int2bin(ts.Td2).substr(0,4)+int2bin12bits(ds.driftT0_vector[n])));	   
	    ds.driftC_vector.push_back(bin2int_unsigned(int2bin(ts.Cd2).substr(0,4)+int2bin12bits(ds.driftC0_vector[n])));	   
	   }//end of for    
    }//end of if(=="0000")
    
    message_num_drift_list.clear();
    message_num_drift_vector.clear();

}//end of decodeMessageType2 function


//reset vectors after each cycle's decoding to prepare for next cycle
void  resetVectors(vector<int>& P_Vector,vector<int>& T_Vector,vector<int>& C_Vector,vector<int>& CRC_Vector,
                   struct position_info_struct& ps,struct driftPTC_struct& ds){   
    P_Vector.clear();
    T_Vector.clear();
    C_Vector.clear();
    CRC_Vector.clear();
    
    ps.year_vector.clear();
    ps.month_vector.clear();
    ps.day_vector.clear();
    ps.hour_vector.clear();
    ps.min_vector.clear();
    ps.sec_vector.clear();
    ps.xmits_vector.clear();
    ps.sat_vector.clear();
    ps.class_vector.clear();
    ps.lon_vector.clear();
    ps.lat_vector.clear();

    ds.driftP0_vector.clear();
    ds.driftT0_vector.clear();
    ds.driftC0_vector.clear();
    ds.driftP_vector.clear();
    ds.driftT_vector.clear();
    ds.driftC_vector.clear();
    ds.driftPump_vector.clear();
}//end of resetVectors function


/*******************************************************************************************************
  read position information for each profile,and return them by reference
*******************************************************************************************************/
void read_position(struct position_info_struct& ps,vector<int> year_ps,vector<int> month_ps,
vector<int> day_ps,vector<int> hour_ps,vector<int> minute_ps,vector<int> second_ps, vector<float> lat_ps,
vector<float> lon_ps,vector<float> nl_ps,vector<char> sattype_ps,vector<char> lc_ps,int fileno,
vector<int> start_row,vector<int> end_row)
{

           int i;

    	  //read position information
	  if(fileno<start_row.size()){

	    for(i=start_row[fileno];i<=end_row[fileno];i++){
	      //get latitude and longitude,filtering non positional data out
              if((lat_ps[i]!=(float)0.0)|| (lon_ps[i]!=(float)0.0)){
	        ps.lat_vector.push_back(lat_ps[i]);
	        ps.lon_vector.push_back(lon_ps[i]);
	        ps.xmits_vector.push_back((int)floor((double)(nl_ps[i]-1)/8));
	        ps.sat_vector.push_back(sattype_ps[i]);
	        ps.class_vector.push_back(lc_ps[i]);
                //cout<<"ps_cnt="<<ps_cnt<<" i="<<i<<"lat="<<lat_ps[i]<<"lon="<<lon_ps[i]<<" HERE------position-1"<<endl;
	        ps.year_vector.push_back(year_ps[i]);
	        ps.month_vector.push_back(month_ps[i]);
	        ps.day_vector.push_back(day_ps[i]);
	        ps.hour_vector.push_back(hour_ps[i]);
	        ps.min_vector.push_back(minute_ps[i]);
	        ps.sec_vector.push_back(second_ps[i]);
  	      }//end of if(lat[i])

	    }//end of for(i=start_row[k])

	    //put 999 delimeter to the top and bottom of position_info struct

	      //append at the end 999 delimeter
	      ps.lat_vector.push_back(99.999);
	      ps.lon_vector.push_back(999.999);
	      ps.xmits_vector.push_back((int)floor((double)(nl_ps[end_row[fileno]]-1)/8));
	      ps.sat_vector.push_back(sattype_ps[end_row[fileno]]);
	      ps.class_vector.push_back(lc_ps[end_row[fileno]]);
              //cout<<"HERE------position-2"<<endl;
	      ps.year_vector.push_back(year_ps[end_row[fileno]]);
	      ps.month_vector.push_back(month_ps[end_row[fileno]]);
	      ps.day_vector.push_back(day_ps[end_row[fileno]]);
	      ps.hour_vector.push_back(hour_ps[end_row[fileno]]);
	      ps.min_vector.push_back(minute_ps[end_row[fileno]]);
	      ps.sec_vector.push_back(second_ps[end_row[fileno]]);
              //cout<<"HERE------position-3"<<endl;
	     //insert in the front 999 delimeter
	      ps.lat_vector.insert(ps.lat_vector.begin(),99.999);
	      ps.lon_vector.insert(ps.lon_vector.begin(),999.999);
              ps.xmits_vector.insert(ps.xmits_vector.begin(),(int)floor((double)(nl_ps[start_row[fileno]]-1)/8));
	      ps.sat_vector.insert(ps.sat_vector.begin(),sattype_ps[start_row[fileno]]);
	      ps.class_vector.insert(ps.class_vector.begin(),lc_ps[start_row[fileno]]);
              //cout<<"HERE------position-4"<<endl;
	      ps.year_vector.insert(ps.year_vector.begin(),year_ps[start_row[fileno]]);
	      ps.month_vector.insert(ps.month_vector.begin(),month_ps[start_row[fileno]]);
	      ps.day_vector.insert(ps.day_vector.begin(),day_ps[start_row[fileno]]);
	      ps.hour_vector.insert(ps.hour_vector.begin(),hour_ps[start_row[fileno]]);
	      ps.min_vector.insert(ps.min_vector.begin(),minute_ps[start_row[fileno]]);
	      ps.sec_vector.insert(ps.sec_vector.begin(),second_ps[start_row[fileno]]);
	    }//end of if(fileno<start_row)

};//end of function read_tech_position_drift

//clean start_row and end_row, if there are ghost messages/cycles, i.e, 
//if there is only one line in a cycle, then skip it,remove it from start_row,end_row vectors.
void  cleanStartRowEndRow(vector<int>& start_row,vector<int>& end_row){
  vector<int> start_row1,end_row1;

  for(int i=0;i<start_row.size();i++){
    if(start_row[i]!=end_row[i]){
      start_row1.push_back(start_row[i]);
      end_row1.push_back(end_row[i]);
    }
  }
  start_row=start_row1;
  end_row=end_row1;
}//end of cleanStartRowEndRow function

//adjust start_row's line number based on launch_time, if in one cycle, the date is larger
//than launch_time, then record the first row when this happens and reset this cycle's start_row
//value to this recorded row number. This will make sure that we do not skip any profile with 
//valid data in it.
void adjustStartRowEndRow(vector<int>& start_row,vector<int> end_row,vector<float> jdayarray,struct ts14_meta_struct ms){
  vector<int> start_row1;
  int k;
  int flag=0;
  
  for(int i=0;i<start_row.size();i++){
    k=start_row[i];  
    flag=0;
    for(int j=start_row[i];j<=end_row[i];j++){
       if(jdayarray[j]>=ms.launch_time){         
       //find the first time when jdayarray is >=launch_time, record its row number  
	    k=j;
	    flag=1;
	    break;	 
       }  
    }//end of for(int j=0)
    start_row1.push_back(k);
  }//end of for(int i=0)
  
  start_row=start_row1;
}
