/******************************************************************************
   Revision Date: 21-JUN-04 (XXD&CS - do not allow negative profile numbers
                            (bug fix); do not try to write a data file if
                            no lines are available (j=n_scans=0), in
                            get_type5_data)
                  15-AUG-03 (XX&CS-add exit status 2 when frame1 missing problem happens 
                             in some but not all profile cycles)
                  14-JUL-03 (XX&CS-change frames_per_message to 256)
                  20-NOV-01 (CS - write abs(s->WMO_id) not s->WMO_id)
                  31-May-01 (JZ) - changed 'BATTERY VOLTAGE' to 'BATTERY'
                  13-JAN-01 (DB) - original adapted from HEX2CNT_TS3.C
 

   hex2cnt_ts5.c 
   =============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   APEX floats APEX_TS5 manufactured in 2000 by Webb Research Cooperation
   and deployed by the University of Washington
*******************************************************************************/

#include <stdio.h>
/*#include <conio.h>*/
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans        2000    //Maximum scans for each profile
#define buff_size         256    //Maximum length of each scan read from *.str
#define max_positions     100    //Maximum number of positions per profile
#define min_frame_no        0    //Minimun frame number
#define frames_per_message 256   //Number of frames per message,it is the maximum allowed number
#define good_bad_position  43    //Location of status byte in *.str records
#define start_of_hex_data  63    //Start position of data in *.str records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0

int frame1_problem_count=0,total_cycles_count=0;//added new.

int actual_frames_per_message;

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;
char   PRGNAME[] ="hex2cnt_ts5";
char   buffer[512];

char *strupr(char* ptr_to_string_to_be_upper_cased); 

long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);

struct aoml5_cals { double TSLOPE;
                    double TOFF;
                    double PSLOPE;
                    double POFF;
                    double SSLOPE;
                    double SOFF;
                    double VSLOPE;
                    double VOFF;
                    double VACSLOPE;
                    double VACOFF;
                  };

union u_cals { struct aoml5_cals s;
               double c[sizeof(struct aoml5_cals)/sizeof(double)];
             };

struct type5_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
						   int exp_no;
						   unsigned char float_type[40];
                           unsigned char PI[60];
                           float ARGOS_rep_rate;
                           float cycle_time;
                           float start_time;
						   float start_of_cycles;	//Start_time+delay_of_start+down_time
                           union u_cals cals;
                         };

/*      Raw data structure used in decoding Temperature/Pressure (T/P)
 float data.  Part of RAW_DATA_RECORD described below */

struct test_fmt { unsigned char controller_sn[2];
                  unsigned char hours_since_start;
                  unsigned char minutes_since_start;
                  unsigned char seconds_since_start;
                  unsigned char flag2;
                  unsigned char current_pressure[2];
                  unsigned char battery_volts;
                  unsigned char current_bladder_pressure;
                  unsigned char flag1;
                  unsigned char up_time_intervals;
                  unsigned char down_time_intervals[2];
                  unsigned char trip_interval_time;
                  unsigned char target_pressure[2];
                  unsigned char target_piston_position;
                  unsigned char depth_correction;
                  unsigned char ballast_piston_position;
                  unsigned char fully_extended_piston_position;
                  unsigned char ok_vacuum;
                  unsigned char ascend_time_intervals;
                  unsigned char target_bladder_pressure;
                  unsigned char park_pressure[2];
                  unsigned char park_piston_position;
                  unsigned char software_rev_month;
                  unsigned char software_rev_day;
                  unsigned char software_rev_year;
                };

struct data_fmt { unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char profile_length;
                  unsigned char termination_flag;
                  unsigned char piston_position;
                  unsigned char bottom_temperature[2];
                  unsigned char bottom_salinity[2];
                  unsigned char bottom_pressure[2];
                  unsigned char battery_volts;
                  unsigned char surface_pressure[2];
                  unsigned char internal_vacuum;
                  unsigned char target_piston_position;
                  unsigned char current_piston_position;
                  unsigned char bladder_pressure;
                  unsigned char T_S_P[5*frames_per_message][6];
                };

/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { unsigned char data_bytes[frames_per_message][30];
                  struct data_fmt d_fmt;
                  struct test_fmt t_fmt;
                };

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { int jday;
                         double jd[max_positions];
                         float  latitude[max_positions];
                         float  longitude[max_positions];
                         char   date_time[max_positions][32];
                         int    n_xmits[max_positions];
                         char   sat_name[max_positions];
                         char   loc_class[max_positions];
                         union  data_size data;
                       };

/* Final structure containig T/P data and positions */

struct output_record { int    float_id;
                       int    profile_no;
                       int    jd;
                       int    status[frames_per_message];
                       int    msg_blk_no;
                       time_t first_blk_time;
                       int    controller_sn;
                       int    battery_volts;
                       int    hours_since_start;	//Test stuff follows
                       int    minutes_since_start;
                       int    seconds_since_start;
                       int    flag2;
                       int    current_pressure;
                       int    current_bladder_pressure;
                       int    flag1;
                       int    up_time_intervals;
                       int    down_time_intervals;
                       int    trip_interval_time;
                       int    target_pressure;
                       int    target_piston_position;
                       int    depth_correction;
                       int    ballast_piston_position;
                       int    fully_extended_piston_position;
                       int    ok_vacuum;
                       int    ascend_time_intervals;
                       int    target_bladder_pressure;
                       int    park_pressure;
                       int    park_piston_position;
                       int    software_rev_month;
                       int    software_rev_day;
                       int    software_rev_year;
                       int    profile_length;	//Data stuff follows
                       int    termination_flag;
                       int    piston_position;
                       int    bottom_temperature;
                       int    bottom_salinity;
                       int    bottom_pressure;
                       int    surface_pressure;
                       int    internal_vacuum;
                       int    current_piston_position;
                       int    bladder_pressure;

                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       int    press_sal_temp[15*frames_per_message][4]; //[0]=p,[1]=t,[2]=bin
                     };

char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = passed_jd - jd_970101;
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}

int data_flag2(int bin_no, int *status_word)
{
 int i,ts_stat=BAD,p_stat=BAD;
 int p_ranges[frames_per_message][2] =  {0,1,2,6,7,11,12,16,17,21,22,26,27,31,32,36,37,41,42,46,47,51,52,56,57,61,62,66,67,71};
 int ts_ranges[frames_per_message][2] = {0,2,3,7,8,12,13,17,18,22,23,27,28,32,33,37,38,42,43,47,48,52,53,57,58,62,63,67,68,72};

// if (bin_no >= p_ranges[frames_per_message-1][2])
//	 return BAD;
 for (i=0; i<frames_per_message; i++)
 {
  if ((bin_no >= ts_ranges[i][0]) && (bin_no <= ts_ranges[i][1]))
	  ts_stat = status_word[i];
  if ((bin_no >= p_ranges[i][0]) && (bin_no <= p_ranges[i][1]))
	  p_stat = status_word[i];
 }
 if ((p_stat == BAD) || (ts_stat == BAD))
	 return BAD;
 else if ((p_stat == NEVER_RECEIVED) || (ts_stat == NEVER_RECEIVED))
	 return NEVER_RECEIVED;
 return GOOD;
}

int write_to_disk (char *dir_name, struct type5_meta_struct *s, struct output_record *x)
{
 int dp,dpp;

/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no 
 temperature or pressure data. */

 int  i,j,id;
 char *ptr_cnt_filename;
 struct tm *time_str;

 id = x->float_id;
 if (((x->status[0]) == NEVER_RECEIVED) && (x->profile_no > 0))   //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file,"First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file,"First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 if (trace_file != NULL)
    fprintf (trace_file,"Opening output file %s...\n",buffer);
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened",buffer);

// Write Output Profile Count File Name to List File
// -------------------------------------------------  
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Can't get cnt file basename %s\n", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 
 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %-5d\n"
     "TRANSMISSION ID NUMBER                  %-5d\n"
     "PROFILE NUMBER                          %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n",
	    s->float_type);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
		s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
	 "CONTROLLER SN                           %d\n",
	    x->controller_sn);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 x->first_blk_time -= (long) (x->msg_blk_no * s->ARGOS_rep_rate);

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if (x->profile_no > 0)
 {
  sprintf (buffer,
     "PROFILE LENGTH (BINS)                   %-5d\n"
     "BATTERY (COUNTS)                        %-5d\n"
     "BLADDER PRESSURE (COUNTS)               %-5d\n",
      x->profile_length,x->battery_volts,x->bladder_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DRIFT TEMPERATURE (COUNTS)              %-5d\n"
     "DRIFT PRESSURE (COUNTS)                 %-5d\n"
     "DRIFT SALINITY (COUNTS)                 %-5d\n"
     "SURFACE PRESSURE (COUNTS)               %-5d\n",
      x->bottom_temperature,x->bottom_pressure,x->bottom_salinity,
      x->surface_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INTERNAL VACUUM                         %-5d\n"
     "PROFILE PISTON POSITION                 %-5d\n"
     "CURRENT PISTON POSITION                 %-5d\n",
     x->internal_vacuum,x->piston_position,x->current_piston_position);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
 sprintf (buffer,
     "PROFILE TERMINATION FLAG                %-2d\n",
		x->termination_flag);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 sprintf (buffer,
     "NUMBER OF COLUMNS                       4\n"
     "1. COLUMN                               PRESSURE (COUNTS)\n"
     "2. COLUMN                               TEMPERATURE (COUNTS)\n"
     "3. COLUMN                               SALINITY (COUNTS)\n"
     "4. COLUMN                               CHECKSUM FLAG\n"
	 "==========================================================\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
 else
 {
  sprintf (buffer,
     "CURRENT PRESSURE (COUNTS)               %d\n"
     "BATTERY (COUNTS)                        %d\n"
     "TARGET PISTON POSITION                  %d\n",
     x->current_pressure, x->battery_volts, x->target_piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "CURRENT BLADDER PRESSURE                %d\n"
     "TARGET BLADDER PRESSURE                 %d\n",
	 x->current_bladder_pressure,x->target_bladder_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "FLAG1 (HEX)                             %02X\n"
     "FLAG2 (HEX)                             %02X\n"
     "TIME SINCE START (HR/MIN/SEC)           %02d:%02d:%02d\n",
     x->flag1,x->flag2,x->hours_since_start,x->minutes_since_start,
	 x->seconds_since_start);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "ASCEND TIME INTERVALS (HRS)             %d\n"
     "UP TIME INTERVALS (HRS)                 %d\n"
     "DOWN TIME INTERVALS (HRS)               %d\n"
     "TRIP INTERVAL TIME                      %d\n",
     x->ascend_time_intervals,x->up_time_intervals,x->down_time_intervals,
     x->trip_interval_time);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "TARGET PRESSURE (CBAR)                  %d\n"
     "DEPTH CORRECTION FACTOR                 %d\n"
     "BALLAST PISTON POSITION                 %d\n"
     "FULL EXTENSION PISTON POSITION          %d\n",
     x->target_pressure,x->depth_correction,x->ballast_piston_position,
     x->fully_extended_piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  dp=x->park_pressure;
  if (dp==65535)
  {
    dp=-9990;
  }

  dpp=x->park_piston_position;
  if (dpp==255)
  {
    dpp=-999;
  }

  sprintf (buffer,
     "DRIFT PRESSURE (COUNTS)                 %d\n"
     "DRIFT PISTON POSITION                   %d\n"
     "TARGET PISTON POSITION                  %d\n"
     "OK VACUUM COUNT                         %d\n",
          dp,dpp,x->target_piston_position,x->ok_vacuum);
/*         
  sprintf (buffer,
     "DRIFT PRESSURE (COUNTS)                 %d\n"
     "DRIFT PISTON POSITION                   %d\n"
     "TARGET PISTON POSITION                  %d\n"
     "OK VACUUM COUNT                         %d\n",
          x->park_pressure,x->park_piston_position,
		  x->target_piston_position,x->ok_vacuum);
 */
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
         
  sprintf (buffer,
     "SOFTWARE REVISION DATE (MON/DAY/YR)     %02d/%02d/%02d\n"
	 "==========================================================\n",
     x->software_rev_month,
     x->software_rev_day,x->software_rev_year);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i++;
 }
 fprintf (out_file,"==========================================================\n");

/* Write depth, temperature, and salinity to disk */
   
 if (x->profile_no > 0)
 {
  i = 1;
  while (x->press_sal_temp[i][0] > -999)
  {
   if ((j = data_flag2(x->press_sal_temp[i][3],x->status)) == NEVER_RECEIVED)
    sprintf (buffer," %5d  %5d  %5d  %d\n",0,0,0,0);
   else
    sprintf (buffer," %5d  %5d  %5d  %d\n",
	                      x->press_sal_temp[i][0],x->press_sal_temp[i][1],
		              x->press_sal_temp[i][2],j);
   fwrite (strupr(buffer),1,strlen(buffer),out_file);
   if (++i > 5*frames_per_message)
	   break;
  }
 }

   if (out_file != NULL)
    fclose (out_file);   /* Close output file */

  return 0;
}

int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad)
{
 int i,j,k,l,matches = 0;
 int num_matches,most_likely = -1;
 char tmp_buff[2][256];

//  Finds the most likely scan for the APEX floats
//  Returns number of most common scan except for test message
//  Search for test message if frame_num = 0
//  Test message detected if block number > frames per message and flag2 bits 6 or 7 set
//  Returns -1 if error
//  good_bad=1 -> use checksum; good_bad=0->ignore checksum
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if ((tmp_buff[0][good_bad_position] != '0') && good_bad)
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
/*  if ((k > frames_per_message) && (frame_num == 0))
  {
   sscanf(&tmp_buff[0][start_of_hex_data+14],"%2X",&j); //j = flag2 byte
   if (((j & 0X60) == 0X20) || ((j & 0X60) == 0X40))
	most_likely = i;	//returns last test message with block number > frames_per_message
   continue;
  }*/
  if ((k-1) != frame_num)	//Frame = 1 to 15 => most_common[0-14]
   continue;
//  if (k == 1)
//  {
//   tmp_buff[0][start_of_hex_data+4] = '0';
//   tmp_buff[0][start_of_hex_data+5] = '0'; //set message blk number = 0 because it's not constant
//  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
//   if (l == 1)
//   {
//    tmp_buff[1][start_of_hex_data+4] = '0';
//    tmp_buff[1][start_of_hex_data+5] = '0'; //set message blk number = 0 because it's not constant
//   }
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches) //Is this the most matches?
  {
   matches = num_matches; //Yes - save match count
   most_likely = i;       //Save record number
  }
// TEST CS
  if ((num_matches==0) && (matches==0)) // Make sure to use messages only received once
  {
    most_likely=i;
  }

 }
 return most_likely;
}
 
int get_raw_data (struct output_record *y, char ** buff, int n_scans, int m_type)
{
  struct raw_data_record x;
  struct tm *time_str;
  int i,j,k,most_common[frames_per_message+1]; //frames_per_mesage+1 for test mesage
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32],roll_over=0,old_count=0;

  //added new
  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
   y->status[i] = NEVER_RECEIVED;
   for (j=0; j<30; j++)
    x.data.data_bytes[i][j] = 0;
   if (m_type == 0)
	   break;
   if ((j = most_likely_frame (buff, n_scans, i, 1)) >= 0) //find most likely - good CRC only
    most_common[i] = j;
   else if ((j = most_likely_frame (buff, n_scans, i, 0)) >= 0) //find most likely - any CRC
    most_common[i] = j;
   else 
    most_common[i] = -1;
  }

  if ((m_type == 1) && (most_common[0] < 0)) //frame =1 => most_common[0]
  {
      //added new
   frame1_problem_count++;

   if (trace_file != NULL)
    fprintf (trace_file," Can't decode message without frame 1\n");
   if (error_file != NULL)
    fprintf (error_file," Can't decode message without frame 1\n");
   return -1;	//Can't decode message without frame 1 - error return
  }

  for (j=1; j<max_positions; j++)
   x.jd[j] = -1.0;

  for (i=0; i<n_scans; i++)
  {
   if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
   {
    continue;
   }

  if (m_type == 1)
  {
    // Read third and fourth characters from hex_data, convert from hex to decimal and
    // skip if > frames_per_message. 
    frames=0;
    for (l=0; l<2 ; l++)
    { 
       ch=*(hex_data+l+2);
       switch (ch)
       {
         case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
         case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
         case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
         case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
         case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
         case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
         default : frms[0]=ch; frms[1]='\0'; break;
       }
       frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
    }
    if (frames > frames_per_message)
    {
      continue;
    }
  }
   x.latitude[n_pos] = lat;
   x.longitude[n_pos] = lon;
   x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

   sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
   x.loc_class[n_pos] = l_c;
   x.sat_name[n_pos] = sat;
   x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
   if ((lat != (float) 0.0) || (lon != (float) 0.0))
   {
    n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
    continue;
   }
   x.latitude[n_pos] = (float) 99.999;
   x.longitude[n_pos] = (float) 999.999;
   x.n_xmits[n_pos] = 0;
   n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

//16920 2000 12 13 06 21 50 +00.000 +000.000 0  25 L 0 126882763 497101C401595248000F8F5A0C1000F0014E20230C18F8450957FFFFFF072800
   for (j=0; j<32; j++)
    sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

   if ((z[1] - min_frame_no) < 1)
    continue;

   if (m_type == 1)
   {
    if (z[1]>frames_per_message)
    {
    if (trace_file != NULL)
      fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"z[1] exceeds frames_per_message.\n");
    exit (1);
    }    
   
    if (i != (most_common[z[1] - 1]))
     continue;

    if (z[1] == 1)
    {
     if (z[2] < old_count)
		roll_over++;
     old_count = z[2];
     y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
    }
    y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
    for (k=0; k<30; k++)
     x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
   }
   else if ((m_type == 0) && (g == 0))
   {
	if (z[1] < old_count)
		roll_over++;
	old_count = z[1];
    y->msg_blk_no = z[1]-1 + roll_over*256; //count starts at 1
     z[1] = 1;	//Z[1] now has frame number = 1 for test msg
     for (k=0; k<30; k++)
      x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
   }

   if (z[1] == 1)
   {
    y->first_blk_time = ((ymd2julday(iy,imo,ida)*24+ih)*60+im)*60+is;
   }
  }
  x.jday = (int) floor(x.jd[0]);

  if ((trace_file != NULL) && (m_type == 1))
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",(int)x.data.d_fmt.profile_no);
    for (j=(frames_per_message-1); j>0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }

  y->float_id = id;
  y->jd = x.jday;
  if (m_type == 1)
  {
   y->controller_sn = 256*x.data.d_fmt.controller_sn[0] +
    x.data.d_fmt.controller_sn[1];
   y->profile_no = x.data.d_fmt.profile_no;
   y->profile_length = x.data.d_fmt.profile_length;
   y->termination_flag = x.data.d_fmt.termination_flag;
   y->piston_position = x.data.d_fmt.piston_position;
   y->bottom_temperature = (256*x.data.d_fmt.bottom_temperature[0] + 
    x.data.d_fmt.bottom_temperature[1]);
   y->bottom_salinity = (256*x.data.d_fmt.bottom_salinity[0] + 
    x.data.d_fmt.bottom_salinity[1]);
   y->bottom_pressure = (256*x.data.d_fmt.bottom_pressure[0] + 
    x.data.d_fmt.bottom_pressure[1]);
   y->battery_volts = x.data.d_fmt.battery_volts;
   y->surface_pressure = (256*x.data.d_fmt.surface_pressure[0] + 
    x.data.d_fmt.surface_pressure[1]);
   y->internal_vacuum = x.data.d_fmt.internal_vacuum;
   y->target_piston_position = x.data.d_fmt.target_piston_position;
   y->current_piston_position = x.data.d_fmt.current_piston_position;
   y->bladder_pressure = x.data.d_fmt.bladder_pressure;
  }
  else
  {
   y->controller_sn = 256*x.data.t_fmt.controller_sn[0] +
    x.data.t_fmt.controller_sn[1];
   y->hours_since_start = (x.data.t_fmt.hours_since_start/16)*10 + x.data.t_fmt.hours_since_start%16;
   y->minutes_since_start = (x.data.t_fmt.minutes_since_start/16)*10 + x.data.t_fmt.minutes_since_start%16;
   y->seconds_since_start = (x.data.t_fmt.seconds_since_start/16)*10 + x.data.t_fmt.seconds_since_start%16;
   y->profile_no = 0;
   y->flag2 = x.data.t_fmt.flag2;
   y->current_pressure = 256*x.data.t_fmt.current_pressure[0] + x.data.t_fmt.current_pressure[1];
   y->battery_volts = x.data.t_fmt.battery_volts;
   y->current_bladder_pressure = x.data.t_fmt.current_bladder_pressure;
   y->flag1 = x.data.t_fmt.flag1;
   y->up_time_intervals = x.data.t_fmt.up_time_intervals;
   y->down_time_intervals = 256*x.data.t_fmt.down_time_intervals[0] + x.data.t_fmt.down_time_intervals[1];
   y->trip_interval_time = x.data.t_fmt.trip_interval_time;
   y->target_pressure = 256*x.data.t_fmt.target_pressure[0] + x.data.t_fmt.target_pressure[1];
   y->target_piston_position = x.data.t_fmt.target_piston_position;
   y->depth_correction = x.data.t_fmt.depth_correction;
   y->ballast_piston_position = x.data.t_fmt.ballast_piston_position;
   y->fully_extended_piston_position = x.data.t_fmt.fully_extended_piston_position;
   y->ok_vacuum = x.data.t_fmt.ok_vacuum;
   y->ascend_time_intervals = x.data.t_fmt.ascend_time_intervals;
   y->target_bladder_pressure = x.data.t_fmt.target_bladder_pressure;
   y->park_pressure = 256*x.data.t_fmt.park_pressure[0] + x.data.t_fmt.park_pressure[1];
   y->park_piston_position = x.data.t_fmt.park_piston_position;
   y->software_rev_month = (x.data.t_fmt.software_rev_month/16)*10 + x.data.t_fmt.software_rev_month%16;
   y->software_rev_day = (x.data.t_fmt.software_rev_day/16)*10 + x.data.t_fmt.software_rev_day%16;
   y->software_rev_year = (x.data.t_fmt.software_rev_year/16)*10 + x.data.t_fmt.software_rev_year%16;
  }
  for (j=0; j<max_positions; j++)
  {
   y->latitude[j] = x.latitude[j];
   y->longitude[j] = x.longitude[j];
   y->position_jd[j] = x.jd[j];
   y->n_xmits[j] = x.n_xmits[j];
   y->sat_name[j] = x.sat_name[j];
   y->loc_class[j] = x.loc_class[j];
   strncpy(y->date_time[j],x.date_time[j],24);
  }

  actual_frames_per_message=ceil((( ((float)y->profile_length-1.6)*6)/30))+1;
  if (actual_frames_per_message > frames_per_message)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }

  if (y->profile_no > 0)
  {
   y->press_sal_temp[0][0] = y->bottom_pressure;
   y->press_sal_temp[0][1] = y->bottom_temperature;
   y->press_sal_temp[0][2] = y->bottom_salinity;
   y->press_sal_temp[0][3] = 0;
   for (j=1; j<=(y->profile_length); j++)	//length-1 because bottom values included
   {
     y->press_sal_temp[j][1] = (256*x.data.d_fmt.T_S_P[j-1][0] + //Temperature
      x.data.d_fmt.T_S_P[j-1][1]);
     y->press_sal_temp[j][2] = (256*x.data.d_fmt.T_S_P[j-1][2] + //Salinity
      x.data.d_fmt.T_S_P[j-1][3]);
     y->press_sal_temp[j][0] = (256*x.data.d_fmt.T_S_P[j-1][4] + //Pressure
      x.data.d_fmt.T_S_P[j-1][5]);
     y->press_sal_temp[j][3] = j;	//Bin number
   }
   y->press_sal_temp[j][0] = -999;
   y->msg_blk_no *= actual_frames_per_message;
  }
  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(y->profile_no);
}

int prescan_data (char **buffer, int n_scans)
{

 int i,j,k,l,matches[frames_per_message] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 int num_matches,most_likely[frames_per_message] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
 char tmp_buff[2][256];

//  Finds the most likely message for the T/P floats
//  Returns with all position scan status bytes = '0'
//  Returns with most likely scan status byte for each frame = '0'
//  Returns with all other scan status bytes = '1'
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  if (buffer[i][good_bad_position] != '0')
   continue;       //Ignore if status byte = bad
  sscanf(&buffer[i][start_of_hex_data+2],"%2X",&k);
  if (k<0)
   continue;   //Ignore if frame number out of range
  strcpy(tmp_buff[0],buffer[i]);
  if (k > frames_per_message)
  {
   tmp_buff[0][good_bad_position] = '1';
   continue;   //Set status = bad if frame number out of range
  }
  if (k == 0)
  {
   tmp_buff[0][good_bad_position] = '2';
   continue;   //Frame number = 0 => position scan so mark with '2'
  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches[k-1]) //Is this the most matches?
  {
   matches[k-1] = num_matches; //Yes - save match count
   most_likely[k-1] = i;       //Save record number
  }

 }
 for (i=0; i<n_scans; i++)   //Reset position frame status to zero, all others to '1'
  tmp_buff[0][good_bad_position] = (tmp_buff[0][good_bad_position] == '2') ? (char) '0' : (char) '1';
 for (i=0; i<frames_per_message; i++)    //Set most likely scan status to '0'
  if (most_likely[i] >= 0)
   buffer[most_likely[i]][good_bad_position] = '0';
 return 0;
}
 
// int get_type5_data (int np, struct type5_meta_struct *s, unsigned char *out_dir)
int get_type5_data (int np, struct type5_meta_struct *s, char *out_dir)
{
  struct output_record y;
  char **buff;
  int i,j;
  int iy,imo,ida,ih,im,is,ndoy;
  int jd_old=0,jd;
  float d_jd;
  int write_to_disk_error;
  int corrected_prof_no;

/* Allocate needed memory */

  if ((buff = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)
  {
   if (trace_file != NULL)
        fprintf (trace_file," GET_type5_DATA: Error allocating buffer pointers\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type5_DATA: Error allocating buffer pointers\n");
   return 1;
  }
  for (i=0; i<max_scans; i++)
   if ((buff[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)
   {
   if (trace_file != NULL)
        fprintf (trace_file," GET_type5_DATA: Error allocating data buffer arrays\n");
   if (error_file != NULL)
        fprintf (error_file," GET_type5_DATA: Error allocating data buffer arrays\n");
    return 1;
   }

  if (trace_file != NULL)
   fprintf (trace_file," GET_type5_DATA: Arrays successfully allocated\n");
/* Get the data */

  j=0;
  while (fgets(buff[j],buff_size,input_file) != NULL)
  {
   if(sscanf(buff[j],"%d %d %d %d %d %d %d",
       &i,&iy,&imo,&ida,&ih,&im,&is) != 7)
    continue;

   if (s->start_time > (d_jd = (float) dfr97(iy,imo,ida,ih,im,0,&ndoy)))
	   continue;
   jd = (int) d_jd;
//   jd = (int) dfr97(iy,imo,ida,0,0,0,&ndoy);
   if ((abs(jd - jd_old) > 2) && (j > 0))
   {
    if (np > 0)
    {
//     prescan_data (buff, j);
     if ((jd_old - (int) s->start_time) < 2)
      i=get_raw_data (&y,buff,j,0);
	 else
      i=get_raw_data (&y,buff,j,1);
     if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
	 {
                 //changed here
                 corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
                 if(corrected_prof_no>0) y.profile_no=corrected_prof_no;

		 fprintf (trace_file," GET_TYPE5_DATA: transmitted profile no. = %d",y.profile_no);
		 fprintf (trace_file," computed = %d - using computed value\n",
	         y.profile_no);
	 }
     if (y.profile_no >= 0 && i!=-1)
      write_to_disk (out_dir,s,&y);
    }
    sprintf (buff[0],"%s",buff[j]);
    j = 0;
   }
   jd_old = jd;
   if (++j >= max_scans)
   {
    if (trace_file != NULL)
     fprintf (trace_file," Maximum scans for profile exceeded\n");
    if (error_file != NULL)
     fprintf (error_file," Maximum scans for profile exceeded\n");
    exit (1);
   }  
  }//end of while loop
  
if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
{  
//  prescan_data (buff, j);
  if ((jd_old - (int) s->start_time) < 2)
   i=get_raw_data (&y,buff,j,0);
  else
   i=get_raw_data (&y,buff,j,1);
  if (abs (y.jd - (int)(s->start_of_cycles + (y.profile_no - 1) * s->cycle_time)) >= 2)
  {
   //changed here
   corrected_prof_no=(int) (((float) y.jd - s->start_of_cycles)/s->cycle_time + 1.5);
   if(corrected_prof_no>0) y.profile_no=corrected_prof_no;

   fprintf (trace_file," GET_TYPE5_DATA: transmitted profile no. = %d",y.profile_no);
   fprintf (trace_file," computed = %d - using computed value\n",
   y.profile_no);
  }

  if (y.profile_no >= 0 && i!=-1)
  {
    write_to_disk_error=write_to_disk (out_dir,s,&y);
  }
  else
  {
    write_to_disk_error=1;
  }
}else
{
  write_to_disk_error =1; // no data to store -> something may be wrong
}//end of if(j>0)

  free (buff);
  if (trace_file != NULL)
      fprintf (trace_file," GET_type5_DATA completed successfully\n");
  return (write_to_disk_error);
}

// int get_meta_file_name (unsigned char *str_name)
int get_meta_file_name (char *str_name)
{
// unsigned char *ptr;
 char *ptr;

 if ((ptr = strstr(str_name,".hex"))!= NULL)
  sprintf (ptr,".meta");
 else if ((ptr = strstr(str_name,".HEX"))!= NULL)
  sprintf (ptr,".META");
 else
  strcat(str_name,".meta");
 return 0;
}

// int read_type5_meta_file (unsigned char *meta_name, struct type5_meta_struct *meta_struct)
int read_type5_meta_file (char *meta_name, struct type5_meta_struct *meta_struct)
{
//  FILE *meta_file;
 int i,j,syear,smonth,sday,shour,smin,x;
// unsigned char *ptr;
 char *ptr;
 float x1;
// unsigned char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};
 char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};

 for (i=0; i<(sizeof(struct aoml5_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 meta_struct->start_of_cycles = (float) 0.0;
 if (trace_file != NULL)
  fprintf(trace_file,"Opening meta file %s...\n",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return 1;
 }
 else
 if (trace_file != NULL)
 fprintf (trace_file," Opened meta-file succesfully\n");

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
    meta_struct->AOML_number = 0;
   continue;
  }
  if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
     meta_struct->WMO_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
    meta_struct->ARGOS_id = 0;
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION REPETITION RATE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
    meta_struct->ARGOS_rep_rate = 0;
   continue;
  }

  if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x1) != 1)
    x1 = 0;
   meta_struct->start_of_cycles += (float) (x1/24.0);
   continue;
  }
  if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
    meta_struct->wmo_instrument = 0;
   continue;
  }
  if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
    meta_struct->wmo_recorder = 0;
   continue;
  }
  if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
    meta_struct->exp_no = 0;
   continue;
  }
  if (strstr(strupr(buffer),"DOWN TIME [DAYS]") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += x;
   meta_struct->start_of_cycles += x;
   continue;
  }
  if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
  {
	if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
	{
		sprintf (meta_struct->float_type,"\0");
	    continue;
	}
	if ((ptr = strstr(&buffer[strlen("FLOAT TYPE")],meta_struct->float_type)) != NULL)
		sprintf (meta_struct->float_type,"%s",ptr);
    if ((ptr = strstr(meta_struct->float_type,"\n")) != NULL)
	   sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"PI     ") != NULL)
  {
	if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
	{
		sprintf (meta_struct->PI,"\0");
	    continue;
	}
   if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
		sprintf (meta_struct->PI,"%s",ptr);
   if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
	   sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&x) != 1)
    x = 0;
   meta_struct->cycle_time += (float) (x/24.0);
//   meta_struct->start_time -= (float) (x/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
    meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x));
   else
    meta_struct->start_time = (float) 0.0;
   meta_struct->start_of_cycles += meta_struct->start_time;
   continue;
  }
  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
    meta_struct->cals.c[i] = atof(ptr+j);
   i++;
  }
 }
// for (i=0; i<(sizeof(struct aoml5_cals)/sizeof(double)); i++)
//  printf ("\nCal %0d %12.5lf",i,meta_struct->c[i]);
 return 0;
}

int main (int argc, char *argv[])
{
 char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt, 
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80], output_directory[256], msg[11]="", title[257]="";
 enum     {LAST=0,ALL} get_which_profile;
 struct   type5_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
 unsigned long t;
 int      ret_val = 0;

// Program Name
// ------------
 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature, salinity and pressure counts\n");
 strcat(title, "for APEX_TS3 Floats");
 printf("%s\n", title);


// Input Paramenters 
// -----------------
 if ( argc < 7 )
 {
  printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
          PRGNAME);
  printf("\n");
  printf("hex_infile:    hexadecimal input file name\n");
  printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
  printf("cnt_outdir:    output directory for generated count data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Hex File name
// -------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
// -------------------------------------------------------------------------
 if (atoi(argv[2]) >= 1)
  get_which_profile = ALL;
 else
  get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (output_directory,"%s/",argv[3]);
 printf ("%s: Output Directory - %s\n", PRGNAME, output_directory);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
   t = time(NULL);
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   ret_val = 1;
   return ret_val;
 }

 printf("%s: Trace file opened succesfully\n", PRGNAME);
 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[5]);
 printf ("%s: Opening error file %s\n", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
   if (trace_file != NULL)
     fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
 }

 if (trace_file != NULL)
   fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   if (trace_file != NULL)
     fprintf(trace_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   if (error_file != NULL)
     fprintf(error_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

// LIST File
// ---------
  sprintf (buffer,"%s",argv[6]);
  if (trace_file != NULL)
     fprintf (trace_file,"%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    t = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    if (error_file != NULL)
       fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val =1;
    return ret_val;
  }
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);

// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);
 
    t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }
  
  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
     if (trace_file != NULL)
       fprintf (trace_file,"ERROR (%s): Input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     if (error_file != NULL)
       fprintf (error_file,"ERROR (%s): input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     ret_val = 1;
  }
  else
  {
     if (trace_file != NULL)
       fprintf (trace_file,"%s: Input file %s successfully opened\n",
                PRGNAME, pathname_input_filename);

// META File
// ---------
     if (get_meta_file_name (input_filename) != 0)
     {                                             // Failure: did not get meta filename
      if (error_file != NULL)
        fprintf (error_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
        ret_val = 1;
     }
     else
     {                                            // Success: got meta filename
        if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
        {
           if (trace_file != NULL)
             fprintf (trace_file,
                      "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                      PRGNAME);
           if (error_file != NULL)
                fprintf (error_file,
                         "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                          PRGNAME);

           if (trace_file != NULL)                  // Close Log files
             fclose (trace_file);
           if (error_file != NULL)
             fclose (error_file);
 
           t = time(NULL);
           if (trace_file != NULL)
             fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           if (error_file != NULL)
             fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           ret_val = 1;
           return(ret_val);
        } // endif  ptr_path_meta 

        sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

        if (read_type5_meta_file (pathname_input_filename, &s) != 0)
        {                                         // Failure: couldn't read meta_file
           if (trace_file != NULL)
             fprintf (trace_file, "ERROR (%s): Reading meta file %s\n", 
                      PRGNAME, pathname_input_filename);
           if (error_file != NULL)
             fprintf (error_file,"ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
           ret_val = 1;
        }
        else                                      // Success: read meta_file
        {
           if (trace_file != NULL)
             fprintf (trace_file,"%s: Meta file %s successfully read\n",
                     PRGNAME, pathname_input_filename);

// Make profile(s) selection
// -------------------------
           if (get_which_profile == ALL)
           {
              if (get_type5_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           }
           else
           {
              if (get_type5_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           } // endif: get_which_profile

        } // endif: read_type5_meta_file 
     } // endif: get_meta_file_name
  } //endif: input_file

//added new
  printf("HEX filename=%s\n",argv[1]);
  printf("frame1 missing problem counts=%d\n",frame1_problem_count); 
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }


// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
    fprintf (error_file,"=> %s: %s %s", PRGNAME, msg, ctime(&t));

// Close all files 
// ---------------
 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
