/******************************************************************************
   Revision Date: 11-Jun-12 (CS  - renamed round to round_argo to avoid conflict
                                   in newer versions of c)
                  06-May-11 (ER - corrected bug causing first profile always being
                                  corrected to profile number=1
                  13-Aug-07 (CS - bug fix in line (2247 and 2249 didn't like it ...)
                                  if ((abs(jd - jd_old) >= 2) && (j > 0))
                                  changed >2 to >=2
                  27-Sep-06 (XXD - fix the profile number problem overflow with
                            256,still use float transmitted prof_no)
                  22-May- 06 (XXD-deal with missing data for bounce profile part
                                  to get correct profile length,modified split function
                                  julian day computation)
                  23-June-05 (XXD)


   hex2cnt_tsb1.c

   This is a bounce float

   =============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/pressure
   APEX floats APEX_TSB1 manufactured in 2002 by Webb Research Cooperation
   and deployed by the University of Hawaii
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans        2000    //Maximum scans for each profile
#define buff_size         256    //Maximum length of each scan read from *.str
#define max_positions     100    //Maximum number of positions per profile
#define min_frame_no        0    //Minimun frame number
#define frames_per_message 256   //Number of frames per message,it is the maximum allowed number
#define good_bad_position  43    //Location of status byte in *.str records
#define start_of_hex_data  63    //Start position of data in *.str records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0

int debug=0;//1 debug, 0 no debug
int frame1_problem_count=0,total_cycles_count=0;//.

int actual_frames_per_message;

char *strupr(char* ptr_to_string_to_be_upper_cased);

long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char   PRGNAME[] ="hex2cnt_tsb1";

double cal[13];
int    aoml_id;
char   buffer[512];
enum   flg {NO,YES} cal_flg;
//enum   data_flag {NONE,BAD,GOOD};
enum   {F,T} cd_input = F;
int   msg_type;

struct aoml4_cals { double TSLOPE;
                    double TOFF;
                    double PSLOPE;
                    double POFF;
                    double SSLOPE;
                    double SOFF;
                    double VSLOPE;
                    double VOFF;
                    double VACSLOPE;
                    double VACOFF;
                  };

union u_cals { struct aoml4_cals s;
               double c[sizeof(struct aoml4_cals)/sizeof(double)];
             };

struct typeTSB1_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
                           int exp_no;
                           unsigned char float_type[40];
                           unsigned char PI[60];
                           float ARGOS_rep_rate;
                           float cycle_time;
                           float start_time;
                           float bounce_time;
                           float start_of_cycles;//added Start_time+delay_of_start+down_time
                           union u_cals cals;
                         };


/*      Raw data structure used in decoding Temperature/Pressure (T/P)
 float data.  Part of RAW_DATA_RECORD described below */
struct test_fmt { unsigned char controller_sn[2];
                  unsigned char time_since_startup[2];
                  unsigned char flag2;
                  unsigned char current_pressure[2];
                  unsigned char battery_volts;
                  unsigned char current_bladder_pressure;
                  unsigned char flag1;
                  unsigned char up_time_intervals;
                  unsigned char down_time_intervals[2];
                  unsigned char park_pressure[2];
                  unsigned char park_piston_position;
                  unsigned char depth_correction_factor;
                  unsigned char storage_piston_position;
                  unsigned char fully_extended_piston_position;
                  unsigned char ok_vacuum;
                  unsigned char ascend_time_intervals;
                  unsigned char target_bladder_pressure;
                  unsigned char profile_pressure[2];
                  unsigned char profile_piston_position;
                  unsigned char deep_cycle_counts;
                  unsigned char software_rev_month;
                  unsigned char software_rev_day;
                  unsigned char software_rev_year;
                };

struct normal_data_fmt {
                  unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char profile_length;
                  unsigned char termination_flag;
                  unsigned char piston_position;
                  unsigned char format_number;
                  unsigned char depth_table_number;
                  unsigned char air_pump_on_time[2];
                  unsigned char battery_volts;
                  unsigned char battery_current;
                  unsigned char bounce_bottom_piston_position;
                  unsigned char air_bladder_pressure;
                  unsigned char park_temperature[2];
                  unsigned char park_salinity[2];
                  unsigned char park_pressure[2];
                  unsigned char park_battery_volts;
                  unsigned char park_battery_current;
                  unsigned char surface_pressure[2];
                  unsigned char internal_vacuum;
                  unsigned char park_piston_position;
                  unsigned char battery_volts_sbe_pump_on;
                  unsigned char battery_current_sbe_pump_on;
                  unsigned char T_S_P[5*frames_per_message][6];
                };

//only for illustration purpose
struct bounce_data_fmt {
                  unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char no7_bounce_profile_length;
                  unsigned char termination_flag;
                  unsigned char piston_position;
                  unsigned char format_number;
                  unsigned char depth_table_number;
                  unsigned char air_pump_on_time[2];
                  unsigned char battery_volts;
                  unsigned char battery_current;
                  unsigned char bounce_bottom_piston_position;
                  unsigned char air_bladder_pressure;
                  unsigned char no1_bounce_profile_length;
                  unsigned char T_P[5*frames_per_message][4];
                };


/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { unsigned char data_bytes[frames_per_message][29];
                  struct normal_data_fmt d_fmt;
                  struct test_fmt t_fmt;
                  //bounce profile data handles specially,not here
                };

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { int jday;
                         double jd[max_positions];
                         float  latitude[max_positions];
                         float  longitude[max_positions];
                         char   date_time[max_positions][32];
                         int    n_xmits[max_positions];
                         char   sat_name[max_positions];
                         char   loc_class[max_positions];
                         union  data_size data;
                       };

//normal profile output struct
struct output_record_np { int float_id;
                       int profile_no;
                       int jd;
                       int status[frames_per_message];
                       int msg_blk_no;
                       time_t first_blk_time;
                       int controller_sn;
                       int time_since_startup; //Test stuff follows
                       int flag2;
                       int current_pressure;
                       int current_bladder_pressure;
                       int flag1;
                       int up_time_intervals;
                       int down_time_intervals;
                       int depth_correction_factor;
                       int storage_piston_position;
                       int fully_extended_piston_position;
                       int ok_vacuum;
                       int ascend_time_intervals;
                       int target_bladder_pressure;
                       int profile_pressure;
                       int profile_piston_position;
                       int deep_cycle_counts;
                       int software_rev_month;
                       int software_rev_day;
                       int software_rev_year;

                  //normal data stuff follows
                       int profile_length;
                       int termination_flag;
                       int piston_position;
                       int format_number;
                       int depth_table_number;
                       int air_pump_on_time;
                       int battery_volts;
                       int battery_current;
                       int bounce_bottom_piston_position;
                       int air_bladder_pressure;
                       int park_temperature;
                       int park_salinity;
                       int park_pressure;
                       int park_battery_volts;
                       int park_battery_current;
                       int surface_pressure;
                       int internal_vacuum;
                       int park_piston_position;
                       int battery_volts_sbe_pump_on;
                       int battery_current_sbe_pump_on;

                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       int    press_sal_temp[15*frames_per_message][4]; //[0]=p,[1]=t,[2]=s,[3]=bin
                     };
//bounce profile output struct
struct output_record_bp { int float_id;
                       int profile_no;
                       int jd;
                       int status[frames_per_message];
                       int msg_blk_no;
                       time_t first_blk_time;
                       int controller_sn;
                       int time_since_startup; //Test stuff follows
                       int flag2;
                       int current_pressure;

                       int current_bladder_pressure;
                       int flag1;
                       int up_time_intervals;
                       int down_time_intervals;
                       int park_pressure;
                       int park_piston_position;
                       int depth_correction_factor;
                       int storage_piston_position;
                       int fully_extended_piston_position;
                       int ok_vacuum;
                       int ascend_time_intervals;
                       int target_bladder_pressure;
                       int profile_pressure;
                       int profile_piston_position;
                       int deep_cycle_counts;
                       int software_rev_month;
                       int software_rev_day;
                       int software_rev_year;

                  //bounce data stuff follows
                       int termination_flag;
                       int piston_position;
                       int format_number;
                       int depth_table_number;
                       int air_pump_on_time;
                       int battery_volts;
                       int battery_current;
                       int bounce_bottom_piston_position;
                       int air_bladder_pressure;
                       //7 bounce profiles' profile length
                       int bounce_profile_length[7];

                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       //bounce profile only has P/T values, no salinity
                       int    press_temp[7][15*frames_per_message][3]; //[0]=p,[1]=t,[2]=bin
                     };
char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = passed_jd - jd_970101;
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}
//bin_no starts from 1
int data_flag_normal(int bin_no, int *status_word){
   return status_word[(int)(floor((bin_no+23)/29.0))];
}

int data_flag_bounce(int bin_no, int *status_word,struct output_record_bp *x)
{
  int start_byte=24;
  int msg_num=1;
  int byte_num[frames_per_message],msg[frames_per_message];
  int i,total_bins=frames_per_message;

  int prof1_length=x->bounce_profile_length[0],prof2_length=x->bounce_profile_length[1];
  int prof3_length=x->bounce_profile_length[2],prof4_length=x->bounce_profile_length[3];
  int prof5_length=x->bounce_profile_length[4],prof6_length=x->bounce_profile_length[5];
  int prof7_length=x->bounce_profile_length[6];

  int a1,a2,a3,a4;

   for(i=0;i<total_bins;i++){
    if(i==prof1_length||i==prof1_length+prof2_length||
       i==prof1_length+prof2_length+prof3_length||
       i==prof1_length+prof2_length+prof3_length+prof4_length||
       i==prof1_length+prof2_length+prof3_length+prof4_length+prof5_length||
       i==prof1_length+prof2_length+prof3_length+prof4_length+prof5_length+prof6_length||
       i==prof1_length+prof2_length+prof3_length+prof4_length+prof5_length+prof6_length+prof7_length){
       start_byte=start_byte+2;
     }
    a1=++start_byte;if(a1==31) start_byte=2;
    a2=++start_byte;if(a2==31) start_byte=2;
    a3=++start_byte;if(a3==31) start_byte=2;
    a4=++start_byte;if(a4==31) start_byte=2;


    byte_num[i]=a1;
    byte_num[i+1]=a2;
    byte_num[i+2]=a3;
    byte_num[i+3]=a4;


    if(a1==31) {
      msg_num++;msg[i]=msg_num;
    }else if(a2==31||a3==31||a4==31) {
      msg[i]=msg_num++;}
    else{
      msg[i]=msg_num;
    }

    //if(debug>0) printf("byte_num=%02d  %02d  %02d  %02d bin=%02d,msg_num=%d\n",a1,a2,a3,a4,i+1,msg[i]);

  }
  return status_word[msg[bin_no-1]-1]; //msg number starts from 1 but status_word index starts from 0,so -1
}

// round a floating number to its nearest integer
// this function needed to be renamed from round due to a conflict
// in newer versions of c
int round_argo(float x) {
      if (x >= 0) return (int) (x+0.5);
      return (int) (x-0.5);
}

//compute correct profile number
int correct_profilenumber(float jd, float jd_firstline_prev_old, float jd_firstline_current_old, int profSeqCounter, 
                          int profile_no_prev, float start_of_cycles, float cycle_time, int n_prof, int has_prof0)
{
  int correct_profile_no;

  int adjustment=0;

  
  if (jd_firstline_current_old==0 && jd_firstline_prev_old==0) {
    // first profile.
	adjustment=round_argo((jd-start_of_cycles)/cycle_time)*2-1;
  }
  else {
    if( jd_firstline_current_old-jd_firstline_prev_old < 180 )
    {
      adjustment=round_argo((jd_firstline_current_old-jd_firstline_prev_old)/cycle_time)*2-1;
      if( n_prof < 0 && profile_no_prev > 1)
      {
        adjustment += -2;
      }
      if (adjustment < 0 )
      {
        if( n_prof == 0 )
        {
          adjustment=0;
        }
        else
          adjustment=1;
      }
      if( n_prof == 1 )
      {
        profile_no_prev = 0;
      }

    }
    else
      adjustment=1;  //profSeqCounter-profile_no_prev; //1
  }

  correct_profile_no=profile_no_prev+adjustment;

//  printf("profSeqCounter=%d,profile_no_prev=%d,adjustment=%d,cycle_time=%f,jd_current_old=%f,jd_prev_old=%f,jd_diff=%f\n",profSeqCounter,profile_no_prev,adjustment,cycle_time,jd_firstline_current_old,jd_firstline_prev_old,jd_firstline_current_old-jd_firstline_prev_old);

  if(n_prof!=correct_profile_no){
    printf("WARNING, n_prof NOT EQUAL TO correct_profile_no\n");
    printf("n_prof=%d,correct_profile_no=%d,profSeqCounter=%d,profile_no_prev=%d,adjustment=%d,total_cycles_count=%d\n",n_prof,correct_profile_no,profSeqCounter,profile_no_prev,adjustment,total_cycles_count);
  }
  return correct_profile_no;
}

int write_to_disk_normal (char *dir_name, struct typeTSB1_meta_struct *s, struct output_record_np *x)
{
/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no
 temperature or pressure data. */

 int  i,j,id;
 char *ptr_cnt_filename;
 struct tm *time_str;

 id = x->float_id;
 if (((x->status[0]) == NEVER_RECEIVED) && (x->profile_no > 0))   //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file," First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file," First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 if (trace_file != NULL)
    fprintf (trace_file,"Opening output file %s...\n",buffer);
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);

  if(debug) printf("write to output file %s\n",buffer);

// Write Output Profile Count File Name to List File
// -------------------------------------------------
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Can't get cnt file basename %s\n", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 printf ("%s\n", ptr_cnt_filename);

 //PROFILE NUMBER below is the natural profile number only for normal profile, increase continuously
 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %-5d\n"
     "TRANSMISSION ID NUMBER                  %-5d\n"
     "PROFILE NUMBER                          %d\n"
     "FILE NUMBER                             %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,(x->profile_no+1)/2,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
  s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n"
     "CONTROLLER SN                           %d\n",
     s->float_type,x->controller_sn);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if(debug>0) printf("=================================================\n\n");
 if(debug>0) printf("msg_blk_no should be:%d\n",x->msg_blk_no);
 //if(debug>0) printf("first_blk_time should be=%f\n",x->msg_blk_no * s->ARGOS_rep_rate);
 //if(debug>0) printf("before *,first_blk_time=%d\n",x->first_blk_time);

 x->first_blk_time -= (long) (x->msg_blk_no * s->ARGOS_rep_rate);
 if(debug>0) printf("first_blk_time long should be=%d\n",x->first_blk_time);

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));

 if(debug>0) printf("msg_blk_no=%d, first_blk_time=%d\n",x->msg_blk_no,x->first_blk_time);

 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if (x->profile_no > 0)
 {
  sprintf (buffer,
     "PROFILE LENGTH (BINS)                   %-5d\n"
     "BATTERY VOLTAGE INITIAL PUMP (COUNTS)   %-5d\n"
     "BATTERY CURRENT INITIAL PUMP (COUNTS)   %-5d\n"
     "AIR BLADDER PRESSURE                    %-5d\n",
      x->profile_length,x->battery_volts,x->battery_current,
                                      x->air_bladder_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "PUMP MOTOR TIME                         %-5d\n",
      x->air_pump_on_time);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DRIFT BATTERY VOLTAGE (COUNTS)          %-5d\n"
     "DRIFT BATTERY CURRENT (COUNTS)          %-5d\n"
     "DRIFT TEMPERATURE (COUNTS)              %-5d\n"
     "DRIFT PRESSURE (COUNTS)                 %-5d\n"
     "DRIFT SALINITY (COUNTS)                 %-5d\n"
     "SURFACE PRESSURE (COUNTS)               %-5d\n",
   x->park_battery_volts,x->park_battery_current,
      x->park_temperature,x->park_pressure,x->park_salinity,
      x->surface_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INTERNAL VACUUM                         %-5d\n"
     "PISTON POSITION                         %-5d\n"
     "BOUNCE BOTTOM PISTON POSITION           %-5d\n"
     "DRIFT PISTON POSITION                   %-5d\n",
     x->internal_vacuum,x->piston_position,
  x->bounce_bottom_piston_position,x->park_piston_position);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 sprintf (buffer,
     "FORMAT NUMBER                           %-5d\n"
     "DEPTH TABLE NUMBER                      %-5d\n"
     "BATTERY VOLTAGE SBE PUMP ON (COUNTS)    %-5d\n"
     "BATTERY CURRENT SBE PUMP ON (COUNTS)    %-5d\n"
     "PROFILE TERMINATION FLAG (HEX)          %-2X\n",
     x->format_number,x->depth_table_number,
     x->battery_volts_sbe_pump_on,x->battery_current_sbe_pump_on,
          x->termination_flag);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 sprintf (buffer,
     "NUMBER OF COLUMNS                       4\n"
     "1. COLUMN                               PRESSURE (COUNTS)\n"
     "2. COLUMN                               TEMPERATURE (COUNTS)\n"
     "3. COLUMN                               SALINITY (COUNTS)\n"
     "4. COLUMN                               CHECKSUM FLAG\n"
  "==========================================================\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }
 else
 {
  sprintf (buffer,
     "CURRENT PRESSURE (COUNTS)               %d\n"
     "BATTERY VOLTAGE INITIAL PUMP (COUNTS)   %d\n"
     "CURRENT BLADDER PRESSURE                %d\n"
     "TARGET BLADDER PRESSURE                 %d\n",
     x->current_pressure, x->battery_volts,
  x->current_bladder_pressure,x->target_bladder_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "FLAG1 (HEX)                             %02X\n"
     "FLAG2 (HEX)                             %02X\n"
     "TIME SINCE START (2 SEC COUNTS)         %d\n",
     x->flag1,x->flag2,x->time_since_startup);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "ASCEND TIME INTERVALS (HRS)             %d\n"
     "UP TIME INTERVALS (HRS)                 %d\n"
     "DOWN TIME INTERVALS (HRS)               %d\n",
     x->ascend_time_intervals,x->up_time_intervals,x->down_time_intervals);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "PROFILE PRESSURE (COUNTS)               %d\n"
     "PROFILE PISTON POSITION                 %d\n"
     "DEEP CYCLE COUNT                        %d\n",
     x->profile_pressure,x->profile_piston_position,x->deep_cycle_counts);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DEPTH CORRECTION FACTOR                 %d\n"
     "STORAGE PISTON POSITION                 %d\n"
     "FULLY EXTEDED PISTON POSITION           %d\n",
     x->depth_correction_factor,x->storage_piston_position,x->fully_extended_piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DRIFT PRESSURE (COUNTS)                 %d\n"
     "DRIFT PISTON POSITION                   %d\n"
     "OK VACUUM COUNT                         %d\n",
          x->park_pressure,x->park_piston_position,x->ok_vacuum);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "SOFTWARE REVISION DATE (MON/DAY/YR)     %02d/%02d/%02d\n"
  "==========================================================\n",
     x->software_rev_month,
     x->software_rev_day,x->software_rev_year);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i++;
 }
 fprintf (out_file,"==========================================================\n");

/* Write depth, temperature, and salinity to disk */

 if (x->profile_no > 0)
 {
  i = 1;
  while (x->press_sal_temp[i][0] > -999)
  {

   if ((j = data_flag_normal(x->press_sal_temp[i][3],x->status)) == NEVER_RECEIVED)
    sprintf (buffer," %5d  %5d  %5d  %d\n",0,0,0,0);
   else
   if(x->press_sal_temp[i][0]==0 && x->press_sal_temp[i][1]==0 && x->press_sal_temp[i][2]==0){
    sprintf (buffer," %5d  %5d  %5d  %d\n",
                       x->press_sal_temp[i][0],x->press_sal_temp[i][1],
                x->press_sal_temp[i][2],0);//if p=t=s=0,set checksum=0
   }else{
    sprintf (buffer," %5d  %5d  %5d  %d\n",
                              x->press_sal_temp[i][0],x->press_sal_temp[i][1],
         x->press_sal_temp[i][2],j);
   }
   fwrite (strupr(buffer),1,strlen(buffer),out_file);
   if (++i > 5*frames_per_message)
    break;
  }
 }

   if (out_file != NULL)
    fclose (out_file);   /* Close output file */

  return 0;
}

int write_to_disk_bounce (char *dir_name, struct typeTSB1_meta_struct *s, struct output_record_bp *x)
{
/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no
 temperature or pressure data. */

 int  i,j,k,id,bin_no=0;
 char *ptr_cnt_filename;
 struct tm *time_str;

 id = x->float_id;
 if (((x->status[0]) == NEVER_RECEIVED) && (x->profile_no > 0))   //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file," First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file," First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 if (trace_file != NULL)
    fprintf (trace_file,"Opening output file %s...\n",buffer);
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  if (trace_file != NULL)
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 1;
 }
 if (trace_file != NULL)
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);

if(debug) printf("write to output file %s\n",buffer);

// Write Output Profile Count File Name to List File
// -------------------------------------------------
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {
  if (trace_file != NULL)
   fprintf (trace_file,
            "%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Can't get cnt file basename %s\n", PRGNAME, buffer);
   return 1;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 printf ("%s\n", ptr_cnt_filename);

 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %-5d\n"
     "TRANSMISSION ID NUMBER                  %-5d\n"
     "PROFILE NUMBER                          1-7\n"
     "FILE NUMBER                             %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
  s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n"
     "CONTROLLER SN                           %d\n",
     s->float_type,x->controller_sn);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if(debug>0) printf("msg_blk_no should be:%d\n",x->msg_blk_no);
 if(debug>0) printf("first_blk_time should be=%f\n",x->msg_blk_no * s->ARGOS_rep_rate);
 if(debug>0) printf("before *,first_blk_time=%d\n",x->first_blk_time);

 x->first_blk_time -= (long) (x->msg_blk_no * s->ARGOS_rep_rate);
 if(debug>0) printf("first_blk_time long should be=%d\n",x->first_blk_time);

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));

 if(debug>0) printf("msg_blk_no=%d, first_blk_time=%d\n",x->msg_blk_no,x->first_blk_time);

 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if (x->profile_no > 0)
 {
  sprintf (buffer,
     "PUMP MOTOR TIME                         %-5d\n"
     "BATTERY VOLTAGE INITIAL PUMP (COUNTS)   %-5d\n"
     "BATTERY CURRENT INITIAL PUMP (COUNTS)   %-5d\n"
     "AIR BLADDER PRESSURE                    %-5d\n",
      x->air_pump_on_time,x->battery_volts,x->battery_current,
                                      x->air_bladder_pressure);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "1ST BOUNCE PROFILE LENGTH (BINS)        %-5d\n"
     "2ND BOUNCE PROFILE LENGTH (BINS)        %-5d\n"
     "3RD BOUNCE PROFILE LENGTH (BINS)        %-5d\n"
     "4TH BOUNCE PROFILE LENGTH (BINS)        %-5d\n"
     "5TH BOUNCE PROFILE LENGTH (BINS)        %-5d\n"
     "6TH BOUNCE PROFILE LENGTH (BINS)        %-5d\n"
     "7TH BOUNCE PROFILE LENGTH (BINS)        %-5d\n",
      x->bounce_profile_length[0],x->bounce_profile_length[1],x->bounce_profile_length[2],
      x->bounce_profile_length[3],x->bounce_profile_length[4],x->bounce_profile_length[5],
      x->bounce_profile_length[6]);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PISTON POSITION                         %-5d\n"
     "BOUNCE BOTTOM PISTON POSITION           %-5d\n",
      x->piston_position,x->bounce_bottom_piston_position);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 sprintf (buffer,
     "FORMAT NUMBER                           %-5d\n"
     "DEPTH TABLE NUMBER                      %-5d\n"
     "PROFILE TERMINATION FLAG (HEX)          %-2X\n",
     x->format_number,x->depth_table_number,x->termination_flag);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);


 sprintf (buffer,
     "NUMBER OF COLUMNS                       4\n"
     "1. COLUMN                               BOUNCE PROFILE NUMBER\n"
     "2. COLUMN                               PRESSURE (COUNTS)\n"
     "3. COLUMN                               TEMPERATURE (COUNTS)\n"
     "4. COLUMN                               CHECKSUM FLAG\n"
  "==========================================================\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 }

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  i++;
 }
 fprintf (out_file,"==========================================================\n");

/* Write the bounce profile depth, temperature to disk */

if(x->profile_no>0){
  for(k=0;k<7;k++){
       for(i=0;i<x->bounce_profile_length[k];i++){
            bin_no++;
            j = data_flag_bounce(bin_no,x->status,x);

            //if(debug>0) printf("bin=%02d  flag=%d\n",bin_no,j);

            if(x->press_temp[k][i][0]==0 && x->press_temp[k][i][1]==0 && x->press_temp[k][i][2]==0){
              sprintf (buffer,"%d  %5d  %5d  %d\n",k+1,x->press_temp[k][i][0],x->press_temp[k][i][1],0);
            }else{
              sprintf (buffer,"%d  %5d  %5d  %d\n",k+1,x->press_temp[k][i][0],x->press_temp[k][i][1],j);
            }

            fwrite (strupr(buffer),1,strlen(buffer),out_file);
       }//end of for(i) loop
  }//end of for(k) loop
}//end of if(x->profile_no>0)

  if (out_file != NULL) fclose (out_file);   /* Close output file */
  return 0;
}

int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad)
{
 int i,j,k,l,matches = 0;
 int num_matches,most_likely = -1;
 char tmp_buff[2][256];

//  Finds the most likely scan for the APEX floats
//  Returns number of most common scan except for test message
//  Returns -1 if error
//  good_bad=1 -> use checksum; good_bad=0->ignore checksum

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if ((tmp_buff[0][good_bad_position] != '0') && good_bad)
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
  if ((k-1) != frame_num) //Frame = 1 to 15 => most_common[0-14]
   continue;
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches) //Is this the most matches?
  {
   matches = num_matches; //Yes - save match count
   most_likely = i;       //Save record number
  }
  if ((num_matches==0) && (matches==0)) // Make sure to use messages only received once
  {
    most_likely=i;
  }

 }
 return most_likely;
}

/*
  decode normal profile,m_type here can be 0(test message) or 1(normal profile)
*/

int decode_normal_profile(struct output_record_np *y, char ** buff, int n_scans, int m_type)
{
  struct raw_data_record x;
  struct tm *time_str;
  int i,j,k,most_common[frames_per_message+1]; //frames_per_mesage+1 for test mesage
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0;
  int iy1,imo1,ida1,ih1,im1,is1;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[31],roll_over=0,old_count=0;

  if(debug>0) printf("m_type=%d n_scans=%d\n",m_type,n_scans);
  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
     y->status[i] = NEVER_RECEIVED;
     for (j=0; j<29; j++) x.data.data_bytes[i][j] = 0;
     if (m_type == 0)  break;
     if ((j = most_likely_frame (buff, n_scans, i, 1)) >= 0) //find most likely - good CRC only
        most_common[i] = j;
     else if ((j = most_likely_frame (buff, n_scans, i, 0)) >= 0) //find most likely - any CRC
        most_common[i] = j;
     else
        most_common[i] = -1;
  }

// CS - FRAME 1 PROBLEM
  if ((m_type == 1) && (most_common[0] < 0)) //frame =1 => most_common[0]
  {
     frame1_problem_count++;

     if (trace_file != NULL) fprintf (trace_file," Can't decode message without frame 1\n");
     if (error_file != NULL) fprintf (error_file," Can't decode message without frame 1\n");
     return -1; //Can't decode message without frame -1 - error return
  }

  for (j=1; j<max_positions; j++)
     x.jd[j] = -1.0;

  for (i=0; i<n_scans; i++)
  {
     if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
         &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
        continue;

     if (m_type == 1)
     {
     // Read third and fourth characters from hex_data, convert from hex to decimal and
     // skip if > frames_per_message.
        frames=0;
        for (l=0; l<2 ; l++)
        {
           ch=*(hex_data+l+2);
           switch (ch)
           {
             case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
             case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
             case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
             case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
             case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
             case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
             default : frms[0]=ch; frms[1]='\0'; break;
           }
           frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
        }
        if (frames > frames_per_message)
        {
           continue;
        }
     } // if (m_type == 1)

     x.latitude[n_pos] = lat;
     x.longitude[n_pos] = lon;
     x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

     sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
     x.loc_class[n_pos] = l_c;
     x.sat_name[n_pos] = sat;
     x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
     if ((lat != (float) 0.0) || (lon != (float) 0.0))
     {
        n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
        continue;
     }
     x.latitude[n_pos] = (float) 99.999;
     x.longitude[n_pos] = (float) 999.999;
     x.n_xmits[n_pos] = 0;
     n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

//16920 2000 12 13 06 21 50 +00.000 +000.000 0  25 L 0 126882763 497101C401595248000F8F5A0C1000F0014E20230C18F8450957FFFFFF072800
     for (j=0; j<31; j++)
        sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

     if(debug>0) printf("m_type=%d,z[1]=%d\n",m_type,z[1]);

     //if(debug>0) printf("i=%d,buff=%s\n",i,buff[i]);

     if ((z[1] - min_frame_no) < 1)
        continue;


     if (m_type == 1)
     {
        if (z[1]>frames_per_message)
        {
        if (trace_file != NULL)
          fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
        if (error_file != NULL)
          fprintf (error_file,"z[1] exceeds frames_per_message.\n");
        exit (1);
        }
        if (i != (most_common[z[1] - 1]))
         continue;

        if (z[1] == 1)
        {
         if (z[2] < old_count)
         roll_over++;
         old_count = z[2];
         y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
        }
        y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
        for (k=0; k<29; k++)
           x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
     }
     else if ((m_type == 0) && (g == 0))
     {
        if (z[1] < old_count)
        roll_over++;
        old_count = z[1];
        y->msg_blk_no = z[1]-1 + roll_over*256; //count starts at 1
        z[1] = 1; //Z[1] now has frame number = 1 for test msg
        for (k=0; k<29; k++)
           x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];

        //if(debug>0) printf("get_raw_data, msg_blk_no=%d\n",y->msg_blk_no);
     } // if (m_type == 1)
//Speed up - move time calculations outside loop
     if (z[1] == 1)
     {
        is1 = is;
        im1 = im;
        ih1 = ih;
        ida1 = ida;
        imo1 = imo;
        iy1 = iy;
     }
  } // for (i=0; i<n_scans; i++)

  y->first_blk_time = ((ymd2julday(iy1,imo1,ida1)*24+ih1)*60+im1)*60+is1;

  x.jday = (int) floor(x.jd[0]);

  if ((trace_file != NULL) && (m_type == 1))
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",(int)x.data.d_fmt.profile_no);
    for (j=(frames_per_message-1); j>0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }

  y->float_id = id;
  y->jd = x.jday;
  if (m_type == 1) //normal profile data message
  {
     y->profile_no = x.data.d_fmt.profile_no;
     y->controller_sn = 256*x.data.d_fmt.controller_sn[0] + x.data.d_fmt.controller_sn[1];
     y->profile_length = x.data.d_fmt.profile_length;
     y->termination_flag = x.data.d_fmt.termination_flag;
     y->piston_position = x.data.d_fmt.piston_position;
     y->format_number = x.data.d_fmt.format_number;
     y->depth_table_number = x.data.d_fmt.depth_table_number;
     y->battery_volts = x.data.d_fmt.battery_volts;
     y->battery_current = x.data.d_fmt.battery_current;
     y->bounce_bottom_piston_position = x.data.d_fmt.bounce_bottom_piston_position;
     y->air_pump_on_time = 256*x.data.d_fmt.air_pump_on_time[0] + x.data.d_fmt.air_pump_on_time[1];
     y->air_bladder_pressure = x.data.d_fmt.air_bladder_pressure;
     y->park_battery_volts = x.data.d_fmt.park_battery_volts;
     y->park_battery_current = x.data.d_fmt.park_battery_current;
     y->park_temperature = (256*x.data.d_fmt.park_temperature[0] + x.data.d_fmt.park_temperature[1]);
     y->park_salinity = (256*x.data.d_fmt.park_salinity[0] + x.data.d_fmt.park_salinity[1]);
     y->park_pressure = (256*x.data.d_fmt.park_pressure[0] + x.data.d_fmt.park_pressure[1]);
     y->surface_pressure = (256*x.data.d_fmt.surface_pressure[0] + x.data.d_fmt.surface_pressure[1]);
     y->internal_vacuum = x.data.d_fmt.internal_vacuum;
     y->park_piston_position = x.data.d_fmt.park_piston_position;
     y->battery_volts_sbe_pump_on = x.data.d_fmt.battery_volts_sbe_pump_on;
     y->battery_current_sbe_pump_on = x.data.d_fmt.battery_current_sbe_pump_on;
  }
  else if(m_type == 0)             //Test message
  {
     y->profile_no = 0;
     y->controller_sn = 256*x.data.t_fmt.controller_sn[0] + x.data.t_fmt.controller_sn[1];
     y->time_since_startup = 256*x.data.t_fmt.time_since_startup[0] + x.data.t_fmt.time_since_startup[1];
     y->flag2 = x.data.t_fmt.flag2;
     y->current_pressure = 256*x.data.t_fmt.current_pressure[0] + x.data.t_fmt.current_pressure[1];
     y->battery_volts = x.data.t_fmt.battery_volts;
     y->current_bladder_pressure = x.data.t_fmt.current_bladder_pressure;
     y->flag1 = x.data.t_fmt.flag1;
     y->up_time_intervals = x.data.t_fmt.up_time_intervals;
     y->down_time_intervals = 256*x.data.t_fmt.down_time_intervals[0] + x.data.t_fmt.down_time_intervals[1];

     y->depth_correction_factor = x.data.t_fmt.depth_correction_factor;
     y->storage_piston_position = x.data.t_fmt.storage_piston_position;
     y->fully_extended_piston_position = x.data.t_fmt.fully_extended_piston_position;
     y->ok_vacuum = x.data.t_fmt.ok_vacuum;
     y->ascend_time_intervals = x.data.t_fmt.ascend_time_intervals;
     y->target_bladder_pressure = x.data.t_fmt.target_bladder_pressure;
     y->profile_pressure = 256*x.data.t_fmt.park_pressure[0] + x.data.t_fmt.park_pressure[1];
     y->profile_piston_position = x.data.t_fmt.profile_piston_position;
     y->park_pressure = 256*x.data.t_fmt.park_pressure[0] + x.data.t_fmt.park_pressure[1];
     y->park_piston_position = x.data.t_fmt.park_piston_position;
     y->deep_cycle_counts = x.data.t_fmt.deep_cycle_counts;

     y->software_rev_month = (x.data.t_fmt.software_rev_month/16)*10 + x.data.t_fmt.software_rev_month%16;
     y->software_rev_day = (x.data.t_fmt.software_rev_day/16)*10 + x.data.t_fmt.software_rev_day%16;
     y->software_rev_year = (x.data.t_fmt.software_rev_year/16)*10 + x.data.t_fmt.software_rev_year%16;
  }
  for (j=0; j<max_positions; j++)
  {
     y->latitude[j] = x.latitude[j];
     y->longitude[j] = x.longitude[j];
     y->position_jd[j] = x.jd[j];
     y->n_xmits[j] = x.n_xmits[j];
     y->sat_name[j] = x.sat_name[j];
     y->loc_class[j] = x.loc_class[j];
     strncpy(y->date_time[j],x.date_time[j],24);
  }

  actual_frames_per_message=(int)ceil((( (float)y->profile_length*6)/29))+1;

  if (actual_frames_per_message > frames_per_message)
  {
    if (trace_file != NULL) fprintf (trace_file,"normal actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL) fprintf (error_file,"normal actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }

  if (y->profile_no > 0)
  {
   y->press_sal_temp[0][0] = y->park_pressure;
   y->press_sal_temp[0][1] = y->park_temperature;
   y->press_sal_temp[0][2] = y->park_salinity;
   y->press_sal_temp[0][3] = 0;
   for (j=1; j<=(y->profile_length); j++) //length-1 because bottom values included
   {
     y->press_sal_temp[j][1] = (256*x.data.d_fmt.T_S_P[j-1][0] + //Temperature
      x.data.d_fmt.T_S_P[j-1][1]);
     y->press_sal_temp[j][2] = (256*x.data.d_fmt.T_S_P[j-1][2] + //Salinity
      x.data.d_fmt.T_S_P[j-1][3]);
     y->press_sal_temp[j][0] = (256*x.data.d_fmt.T_S_P[j-1][4] + //Pressure
      x.data.d_fmt.T_S_P[j-1][5]);
     y->press_sal_temp[j][3] = j; //Bin number
   }
   y->press_sal_temp[j][0] = -999;
   y->msg_blk_no *= actual_frames_per_message;
  }
  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(y->profile_no);
}

/*
  decode bounce profile,m_type here can only be 2,need to work on this function
*/

int decode_bounce_profile(struct output_record_bp *y, char ** buff, int n_scans, int m_type)
{
  struct raw_data_record x;
  unsigned char data_bytes[frames_per_message][29];
  unsigned char just_bytes[frames_per_message*29];

  struct tm *time_str;
  int i,j,k,most_common[frames_per_message+1]; //frames_per_mesage+1 for test mesage
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0;
  int iy1,imo1,ida1,ih1,im1,is1;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  int tt,pp,last_byte_position=0,begin_byte_position=0,total_bounce_length=0;

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[31],roll_over=0,old_count=0;

  if(debug>0) printf("m_type=%d,n_scans=%d\n",m_type,n_scans);
  if(m_type==0||m_type==1) return -1;

  //total_cycles_count++;

  //initialize the bounce_profile_length array
  for(k=0;k<7;k++){
    y->bounce_profile_length[k]=0;
  }

  for (i=0; i<frames_per_message; i++)
  {
     y->status[i] = NEVER_RECEIVED;
     for (j=0; j<29; j++) data_bytes[i][j] = 0;
     if ((j = most_likely_frame (buff, n_scans, i, 1)) >= 0) //find most likely - good CRC only
        most_common[i] = j;
     else if ((j = most_likely_frame (buff, n_scans, i, 0)) >= 0) //find most likely - any CRC
        most_common[i] = j;
     else
        most_common[i] = -1;
  }

// CS - FRAME 1 PROBLEM
  if ((m_type == 2) && (most_common[0] < 0)) //frame =1 => most_common[0]
  {
     //frame1_problem_count++;

     if (trace_file != NULL) fprintf (trace_file," Can't decode message without frame 1\n");
     if (error_file != NULL) fprintf (error_file," Can't decode message without frame 1\n");
     return -1; //Can't decode message without frame -1 - error return
  }

  for (j=1; j<max_positions; j++)
     x.jd[j] = -1.0;

  for (i=0; i<n_scans; i++)
  {
     if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
         &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
        continue;

     if (m_type == 2)
     {
     // Read third and fourth characters from hex_data, convert from hex to decimal and
     // skip if > frames_per_message.
        frames=0;
        for (l=0; l<2 ; l++)
        {
           ch=*(hex_data+l+2);
           switch (ch)
           {
             case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
             case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
             case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
             case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
             case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
             case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
             default : frms[0]=ch; frms[1]='\0'; break;
           }
           frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
        }
        if (frames > frames_per_message)
        {
           continue;
        }
     } // if (m_type == 2)

     x.latitude[n_pos] = lat;
     x.longitude[n_pos] = lon;
     x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

     sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
     x.loc_class[n_pos] = l_c;
     x.sat_name[n_pos] = sat;
     x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
     if ((lat != (float) 0.0) || (lon != (float) 0.0))
     {
        n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
        continue;
     }
     x.latitude[n_pos] = (float) 99.999;
     x.longitude[n_pos] = (float) 999.999;
     x.n_xmits[n_pos] = 0;
     n_pos = (n_pos == 0) ? n_pos+1 : n_pos;

//16920 2000 12 13 06 21 50 +00.000 +000.000 0  25 L 0 126882763 497101C401595248000F8F5A0C1000F0014E20230C18F8450957FFFFFF072800
     for (j=0; j<31; j++)
        sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

     if(debug>0) printf("m_type=%d,z[1]=%d\n",m_type,z[1]);

     //if(debug>0) printf("i=%d,buff=%s\n",i,buff[i]);

     if ((z[1] - min_frame_no) < 1)
        continue;

     if (m_type == 2)
     {
        if (z[1]>frames_per_message)
        {
        if (trace_file != NULL)
          fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
        if (error_file != NULL)
          fprintf (error_file,"z[1] exceeds frames_per_message.\n");
        exit (1);
        }
        if (i != (most_common[z[1] - 1]))
         continue;

        if (z[1] == 1)
        {
         if (z[2] < old_count)
         roll_over++;
         old_count = z[2];
         y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
        }
        y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
        for (k=0; k<29; k++)
           data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
     }

//Speed up - move time calculations outside loop
     if (z[1] == 1)
     {
        is1 = is;
        im1 = im;
        ih1 = ih;
        ida1 = ida;
        imo1 = imo;
        iy1 = iy;
     }
  } // for (i=0; i<n_scans; i++)

  if(debug>0) {
    for(i=0;i<frames_per_message;i++){
      printf("msg_num=%02d, bounce_status_flag=%d\n",i,y->status[i]);
    }
  }

  y->first_blk_time = ((ymd2julday(iy1,imo1,ida1)*24+ih1)*60+im1)*60+is1;

  x.jday = (int) floor(x.jd[0]);

  y->float_id = id;
  y->jd = x.jday;

  k=0;
  if(debug>0) printf("just_bytes=\n");
  for(i=0;i<frames_per_message;i++){
    for(j=0;j<29;j++){
      just_bytes[k]=data_bytes[i][j];
      //if(debug>0) printf("just_bytes k=%d,value=%d\n",k,just_bytes[k]);
      if(debug>0) printf("%02x",data_bytes[i][j]);
      k++;
    }
  }
  if(debug>0) printf("\n");

  k=0;

//bounce profile data message

     y->controller_sn =256*just_bytes[1]+just_bytes[2];
     y->profile_no = just_bytes[3]; if(debug>0) printf("from raw data, prof_no=%d\n",y->profile_no);
     y->bounce_profile_length[6]=just_bytes[4]; if(debug>0) printf("from raw data, no7 profile length=%d\n",just_bytes[4]);
     y->termination_flag = just_bytes[5];
     y->piston_position = just_bytes[6];
     y->format_number = just_bytes[7];
     y->depth_table_number =just_bytes[8];
     y->air_pump_on_time=256*just_bytes[9]+just_bytes[10];
     y->battery_volts = just_bytes[11];
     y->battery_current = just_bytes[12];
     y->bounce_bottom_piston_position=just_bytes[13];
     y->air_bladder_pressure = just_bytes[14];

     //get the bounce data part for 7 subprofiles
     for(k=0;k<7;k++){
       y->bounce_profile_length[k] =just_bytes[15+k];
     }

     begin_byte_position=22;
     for(k=0;k<7;k++){
       i=0;
       while(1){
         tt=256*just_bytes[i*4+begin_byte_position]+just_bytes[i*4+begin_byte_position+1];
         pp=256*just_bytes[i*4+begin_byte_position+2]+just_bytes[i*4+begin_byte_position+3];
         //deal with missing data problem, DDDD may be missing. So if we find DDDD and it is still less than the prescribed subprofile
         //length, trust DDDD. However, if we find the number of data points reaches subprofile length first, then  trust it instead of DDDD
         if((tt==56797 && i<y->bounce_profile_length[k])||(i==y->bounce_profile_length[k])) break;

         y->press_temp[k][i][0]=pp;
         y->press_temp[k][i][1]=tt;
         y->press_temp[k][i][2]=i+1;
         if(debug>0) printf("pp=%d tt=%d\n",pp,tt);
         i++;
       };

       last_byte_position=(i-1)*4+begin_byte_position+3;
       begin_byte_position=last_byte_position+2+1;
       //if(debug>0) printf("i=%d,last_byte_position=%d,begin_byte_position=%d\n",i,last_byte_position,begin_byte_position);
       y->bounce_profile_length[k]=i;
       if(debug>0) printf("bounce profile %d length=%d \n",k+1,y->bounce_profile_length[k]);
     }

if(debug>0){
  for(k=0;k<7;k++){
       for(i=0;i<y->bounce_profile_length[k];i++){
         printf("%d  %5d  %5d  %d\n",k+1,y->press_temp[k][i][0],y->press_temp[k][i][1],y->press_temp[k][i][2]);
       }//end of for(i) loop
  }//end of for(k) loop
}

  for (j=0; j<max_positions; j++)
  {
     y->latitude[j] = x.latitude[j];
     y->longitude[j] = x.longitude[j];
     y->position_jd[j] = x.jd[j];
     y->n_xmits[j] = x.n_xmits[j];
     y->sat_name[j] = x.sat_name[j];
     y->loc_class[j] = x.loc_class[j];
     strncpy(y->date_time[j],x.date_time[j],24);
  }
  for(k=0;k<7;k++){
    total_bounce_length=total_bounce_length+y->bounce_profile_length[k];
  }
  actual_frames_per_message=(int)ceil((float)(18-2+total_bounce_length*4+2*7)/29);

  if (actual_frames_per_message > frames_per_message)
  {
    if(debug>0) printf("actual_frames_per_message=%d, frames_per_message=%d\n",actual_frames_per_message,frames_per_message);
    if (trace_file != NULL) fprintf (trace_file,"bounce actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL) fprintf (error_file,"bounce actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }
  y->msg_blk_no *= actual_frames_per_message;

  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(y->profile_no);
}
/* split the normal profile and bounce profile raw hex messages
*/

void split_normal_bounce(char **buff,int nscans,char **buff_normal,int* nlines,char **buff_bounce,int* blines){
   int i,j=0,k;
   int iy,imo,ida,ih,im,is,ndoy;
   float jd[nscans],jd_diff_max;
   int hasDDDD=0;//0 no DDDD, 1 has DDDD marker

  //find if buff contains any DDDD in it
   for(i=0;i<nscans;i++){
     if(strstr(buff[i],"DDDD")!=NULL){
      hasDDDD=1;
     }
   }

   //get julian day for each hex data line
   for(i=0;i<nscans;i++){
     if(sscanf(buff[i],"%d %d %d %d %d %d %d", &k,&iy,&imo,&ida,&ih,&im,&is) != 7) continue;
     jd[i]=(float) dfr97(iy,imo,ida,ih,im,is,&ndoy);
     //if(debug>0) printf("jd[i]=%f\n",jd[i]);
   }
   //find the index where jd difference is the largest
   jd_diff_max=14/24.0;
   for(i=0;i<nscans-1;i++){
     if(jd[i+1]-jd[i]>jd_diff_max) {
       jd_diff_max=jd[i+1]-jd[i];
       j=i;
     }
   }

   if(j==0) j=-1;

   if(hasDDDD==1){
     *nlines=j+1;
     *blines=nscans-*nlines;
     for(i=*nlines;i<nscans;i++){ strcpy(buff_bounce[i-*nlines],buff[i]);}
   }else{
     *nlines=nscans;
     *blines=0;
   }

   for(i=0;i<*nlines;i++){
      strcpy(buff_normal[i],buff[i]);
   }
   if(debug>0) printf("in split function, nscans=%d,nlines=%d,blines=%d\n",nscans,*nlines,*blines);
}


// int get_typeTSB1_data (int np, struct typeTSB1_meta_struct *s, unsigned char *out_dir)
int get_typeTSB1_data (int np, struct typeTSB1_meta_struct *s, char *out_dir)
{
  struct output_record_np normal_profile;
  struct output_record_bp bounce_profile;
  char **buff;
  char **buff_normal;
  char **buff_bounce;
  int nlines=0;//normal profile number of lines
  int blines=0;//bounce profile number of lines
  float first_line_jd=0;//julian day of the first line from buff

  int i,j;
  int iy,imo,ida,ih,im,is,ndoy;
  int n_prof=0,b_prof=0;//,jd_old=0,jd;
  float d_jd;
  int write_to_disk_error;
  int corrected_prof_no;

  //xxd new modification
  float jd_old=0,jd;
  int profSeqCounter=0,times=0,has_prof0=0,profile_no_prev=0,profile_no_current=0;
  float jd_firstline_prev_old=0,jd_firstline_current_old=0;
  float jd_firstline_prev_new=0,jd_firstline_current_new=0;
  char* onelinehex;

/* Allocate needed memory for buff,buff_normal,buff_bounce*/
  if (((buff_normal = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)||
      ((buff_bounce = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)||
      ((buff        = (void *) malloc(max_scans*(sizeof(void *)))) == NULL))
  {
   if (trace_file != NULL)
        fprintf (trace_file," GET_typeTSB1_DATA: Error allocating buffer pointers\n");
   if (error_file != NULL)
        fprintf (error_file," GET_typeTSB1_DATA: Error allocating buffer pointers\n");
   return 1;
  }

  for (i=0; i<max_scans; i++)
   if (((buff[i]        = (char *) malloc(buff_size*(sizeof(char)))) == NULL)||
       ((buff_normal[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)||
       ((buff_bounce[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL))
   {
   if (trace_file != NULL)
        fprintf (trace_file," GET_typeTSB1_DATA: Error allocating data buffer arrays\n");
   if (error_file != NULL)
        fprintf (error_file," GET_typeTSB1_DATA: Error allocating data buffer arrays\n");
    return 1;
   }

  if (trace_file != NULL) fprintf (trace_file," GET_typeTSB1_DATA: Arrays successfully allocated\n");

/* Get the data */

  j=0;
  while (fgets(buff[j],buff_size,input_file) != NULL)
  {
     if(sscanf(buff[j],"%d %d %d %d %d %d %d", &i,&iy,&imo,&ida,&ih,&im,&is) != 7) continue;

     if(debug>0) printf("buff[j] %s\n",buff[j]);
     if(debug>0) printf("start_time=%f,dfr97=%f\n",(float) s->start_time,(float) dfr97(iy,imo,ida,ih,im,0,&ndoy));

     if (s->start_time > (d_jd = (float) dfr97(iy,imo,ida,ih,im,0,&ndoy))) continue;

     jd = (int) d_jd;
     times++;if(times==1) {jd_old=jd;jd_firstline_prev_new=jd;}

     if (((jd - jd_old) >= 2) && (j > 0)) // changed >2 to >=2
     {
          profSeqCounter++;jd_firstline_current_new=jd;

           if(debug>0) printf("jd_old %d, (int) s->start_time %d, jd_old - (int) s->start_time %d\n",jd_old,(int) s->start_time,jd_old - (int) s->start_time);
           if ((jd_old - (int) s->start_time) < 2)
           {
              n_prof = decode_normal_profile (&normal_profile,buff,j,0);
              if(debug>0) printf("new profile with number %d\n",n_prof);
           }else{
              if(debug>0) printf("old profile with number %d\n",n_prof);
              split_normal_bounce(buff,j,buff_normal,&nlines,buff_bounce,&blines);
			  n_prof=decode_normal_profile(&normal_profile,buff_normal,nlines,1);
              b_prof=decode_bounce_profile(&bounce_profile,buff_bounce,blines,2);
              if(debug>0) printf("before correction,nlines=%d,blines=%d,n_prof=%d,b_prof=%d\n",nlines,blines,n_prof,b_prof);
 			  profSeqCounter++;
           }

  	       if(profile_no_prev >= 250 && n_prof>0) //LL: prof # > 256
  	       {
			    if( n_prof < 250 )
				n_prof = n_prof+256;
		   }

           normal_profile.profile_no = correct_profilenumber(d_jd, jd_firstline_prev_old,jd_firstline_current_old,profSeqCounter,profile_no_prev,s->start_of_cycles,s->cycle_time,n_prof,has_prof0);
           if(n_prof == 1 && normal_profile.profile_no < 1)
           {
           		normal_profile.profile_no = 1;
		   }
           bounce_profile.profile_no = normal_profile.profile_no+1;

		   if(n_prof<0 && np > 0)
  	       {
				printf("there is no profile %d!\n",normal_profile.profile_no);
				if (trace_file != NULL)
					fprintf (trace_file," GET_typeB1_DATA: WARNING, there is no profile %d!\n",normal_profile.profile_no);
				if (error_file != NULL)
		       	 	fprintf (error_file," GET_typeB1_DATA: WARNING, there is no profile %d!\n",normal_profile.profile_no);
		   }

           fprintf(trace_file,"corrected_prof_no=%d,n_prof=%d\n",normal_profile.profile_no,n_prof);


//           if(n_prof==255) b_prof=256;
//           if(corrected_prof_no>=255){
//             while(corrected_prof_no-n_prof>250){
//                n_prof=n_prof+256;
//             }
//             b_prof=n_prof+1;
//           }

//           fprintf(trace_file,"after correction,n_prof=%d\n",n_prof);

           profile_no_current=bounce_profile.profile_no;
//         printf("profSeqCounter=%d,profile_no_current=%d\n",profSeqCounter,profile_no_current);

           if (n_prof>= 0){
//             normal_profile.profile_no=n_prof;
             if(debug>0) printf("normal_profile.profile_no=%d\n",normal_profile.profile_no);
             if(nlines!=0 && np>0) write_to_disk_error=write_to_disk_normal(out_dir,s,&normal_profile);
           }
           if(b_prof>=0){
//             bounce_profile.profile_no=b_prof;
             if(debug>0) printf("bounce_profile.profile_no=%d\n",bounce_profile.profile_no);
             if(blines!=0 && np>0) write_to_disk_error=write_to_disk_bounce(out_dir,s,&bounce_profile);
           }


        sprintf (buff[0],"%s",buff[j]);
        j = 0; times=0;

        jd_firstline_prev_old=jd_firstline_prev_new;
    	jd_firstline_current_old=jd_firstline_current_new;
   		profile_no_prev=profile_no_current;

   } //if
     jd_old = jd;
     if (++j >= max_scans)
     {
        if (trace_file != NULL) fprintf (trace_file," Maximum scans for profile exceeded\n");
        if (error_file != NULL) fprintf (error_file," Maximum scans for profile exceeded\n");
        exit (1);
     }
  } // while (fgets(buff[j],buff_size,input_file) != NULL)

  if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
  {
     if ((jd_old - (int) s->start_time) < 2)
     {
        n_prof = decode_normal_profile(&normal_profile,buff,j,0);
        printf("HERE n_prof=%d,jd_old=%d,start_time=%d\n",n_prof,jd_old,(int) s->start_time);
     }
     else
     {
        split_normal_bounce(buff,j,buff_normal,&nlines,buff_bounce,&blines);
//        if(nlines!=0) n_prof=decode_normal_profile(&normal_profile,buff_normal,nlines,1);
//        if(blines!=0) b_prof=decode_bounce_profile(&bounce_profile,buff_bounce,blines,2);
        n_prof=decode_normal_profile(&normal_profile,buff_normal,nlines,1);
        b_prof=decode_bounce_profile(&bounce_profile,buff_bounce,blines,2);
     }


      //added profile_no correction
//      if(debug>0) printf("Test start_of_cycles=%f,cycle_time=%f\n",s->start_of_cycles,s->cycle_time);
//      sscanf(buff[0],"%d %d %d %d %d %d %d",&i,&iy,&imo,&ida,&ih,&im,&is);
//      first_line_jd=(float) dfr97(iy,imo,ida,ih,im,is,&ndoy);
//      printf("DEBUG: first line=%s\n",buff[0]);
//      printf("julday of first line=%d,start_of_cycles=%f,cycle_time=%f",first_line_jd,s->start_of_cycles,s->cycle_time);
//     corrected_prof_no=((int) (((float)first_line_jd - s->start_of_cycles)/s->cycle_time + 1.5))*2-1;
//     fprintf(trace_file,"corrected_prof_no=%d,n_prof=%d\n",corrected_prof_no,n_prof);
/*
     if (n_prof>=0 && (first_line_jd - (s->start_of_cycles + (n_prof-1)/2 * s->cycle_time)) >= s->cycle_time/2)
     {        fprintf (trace_file," GET_TYPE21_DATA: first_line_jd=%f,start_cycles=%f,cycle_time=%f\n",first_line_jd,s->start_of_cycles,s->cycle_time);
              fprintf (trace_file," GET_TYPE21_DATA: transmitted profile no. = %d",n_prof);
            //changed here
              //corrected_prof_no=((int) (((float)first_line_jd - s->start_of_cycles)/s->cycle_time + 1.5))*2-1;
              if(corrected_prof_no>0){
                n_prof=corrected_prof_no;
                b_prof=n_prof+1;
              }

              fprintf (trace_file," computed = %d - using computed value\n",corrected_prof_no);
     }
*/


  	       if(profile_no_prev >= 250 && n_prof>0) //LL: prof # > 256
  	       {
			    if( n_prof < 250 )
				n_prof = n_prof+256;
		   }

           normal_profile.profile_no = correct_profilenumber(d_jd, jd_firstline_prev_old,jd_firstline_current_old,profSeqCounter,profile_no_prev,s->start_of_cycles,s->cycle_time,n_prof,has_prof0);
           if(n_prof == 1 && normal_profile.profile_no < 1)
           {
           		normal_profile.profile_no = 1;
		   }
           bounce_profile.profile_no = normal_profile.profile_no+1;

		   if(n_prof<0 && np > 0)
  	       {
				printf("there is no profile %d!\n",normal_profile.profile_no);
				if (trace_file != NULL)
					fprintf (trace_file," GET_typeB1_DATA: WARNING, there is no profile %d!\n",normal_profile.profile_no);
				if (error_file != NULL)
		       	 	fprintf (error_file," GET_typeB1_DATA: WARNING, there is no profile %d!\n",normal_profile.profile_no);
		   }

     fprintf(trace_file,"after correction,n_prof=%d\n",n_prof);

     profile_no_current=bounce_profile.profile_no;

     if (n_prof>= 0 )
     {
//		 normal_profile.profile_no=n_prof;
        if(nlines!=0) write_to_disk_error=write_to_disk_normal(out_dir,s,&normal_profile);
     }
     if(b_prof>=0){
//        bounce_profile.profile_no=b_prof;
        if(blines!=0) write_to_disk_error=write_to_disk_bounce(out_dir,s,&bounce_profile);
     }
  }
  else
  {
    write_to_disk_error =1; // no data to store -> something may be wrong
  } // if (j > 0)

  free (buff);
  free (buff_normal);
  free (buff_bounce);

  if (trace_file != NULL)
      fprintf (trace_file," GET_typeTSB1_DATA completed successfully\n");
  return (write_to_disk_error);
}



// int get_meta_file_name (unsigned char *str_name)
int get_meta_file_name (char *str_name)
{
 char *ptr;

 if ((ptr = strstr(str_name,".hex"))!= NULL)
  sprintf (ptr,".meta");
 else if ((ptr = strstr(str_name,".HEX"))!= NULL)
  sprintf (ptr,".META");
 else
  strcat(str_name,".meta");
 return 0;
}

// int read_typeTSB1_meta_file (unsigned char *meta_name, struct typeTSB1_meta_struct *meta_struct)
int read_typeTSB1_meta_file (char *meta_name, struct typeTSB1_meta_struct *meta_struct)
{
//  FILE *meta_file;
 int i,j,syear,smonth,sday,shour,smin,x;
 float x1;//added
 char *ptr;
 char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};

 for (i=0; i<(sizeof(struct aoml4_cals)/sizeof(double)); i++)
  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 //added
 meta_struct->start_of_cycles = (float) 0.0;

 if (trace_file != NULL)
  fprintf(trace_file,"Opening meta file %s...\n",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return 1;
 }
 else
 if (trace_file != NULL)
 fprintf (trace_file," Opened meta-file succesfully\n");

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
    meta_struct->AOML_number = 0;

   continue;
  }
  if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
     meta_struct->WMO_id = 0;
   if(debug>0) printf("WMO_ID=%d\n",meta_struct->WMO_id);
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
    meta_struct->ARGOS_id = 0;
    if(debug>0) printf("ARGOS_ID=%d\n",meta_struct->ARGOS_id);
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION REPETITION RATE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
    meta_struct->ARGOS_rep_rate = 0;
    if(debug>0) printf("ARGOS_REP_RATE=%f\n",meta_struct->ARGOS_rep_rate);
   continue;
  }
/*
  if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) != 1)
     x = 0;
// CS !!!!!! the above has to be changed quit if no value !!!!

//   meta_struct->start_time += (float) (x/24.0);
   continue;
  }
*/
  if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
    meta_struct->wmo_instrument = 0;
    if(debug>0) printf("WMO_INSTRUMENT=%d\n",meta_struct->wmo_instrument);
   continue;
  }
  if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
    meta_struct->wmo_recorder = 0;
    if(debug>0) printf("wmo_recorder=%d\n",meta_struct->wmo_recorder);
   continue;
  }
  if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
    meta_struct->exp_no = 0;
    if(debug>0) printf("exp_no=%d\n",meta_struct->exp_no);
   continue;
  }
  if (strstr(strupr(buffer),"DOWN TIME [DAYS]") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
    x1 = 0;
   if(debug>0) printf("down time=%f\n",x1);
   meta_struct->cycle_time += (float)x1;
   //added
   meta_struct->start_of_cycles += (float)x1;
   continue;
  }

  if (strstr(strupr(buffer),"BOUNCE TIME [HOURS]") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
    x1 = 0;
   if(debug>0) printf("bounce time=%f\n",x1);
   meta_struct->cycle_time += (float)x1/24.0;
   continue;
  }

  //added,changed
  if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x1) != 1)
    x1 = 0;
    if(debug>0) printf("first down time=%f\n",x1);
   meta_struct->start_of_cycles += (float) (x1/24.0);
   continue;
  }

  if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
  {
 if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
 {
  sprintf (meta_struct->float_type,"\0");
     continue;
 }
 if ((ptr = strstr(&buffer[strlen("FLOAT TYPE")],meta_struct->float_type)) != NULL)
  sprintf (meta_struct->float_type,"%s",ptr);
    if ((ptr = strstr(meta_struct->float_type,"\n")) != NULL)
    sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"PI     ") != NULL)
  {
 if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
 {
  sprintf (meta_struct->PI,"\0");
  if(debug>0) printf("PI=%s\n",meta_struct->PI);
     continue;
 }
   if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
  sprintf (meta_struct->PI,"%s",ptr);
   if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
    sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
    x1 = 0;
    if(debug>0) printf("UP TIME=%f\n",x1);
   meta_struct->cycle_time += (float) (x1/24.0)*2;//note 2 times up_time is added to cycle_time
//   meta_struct->start_time -= (float) (x1/24.0);
   continue;
  }
  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL))
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
   {
    if(debug>0) printf("year=%d month=%d day=%d hour=%d min=%d,dfr97=%f\n",syear,smonth,sday,shour,smin,
    (float) (dfr97(syear,smonth,sday,shour,smin,0,&x)));

    meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x));
    if(debug>0) printf("start_time=%f\n",meta_struct->start_time);

    if (meta_struct->start_time <= (float) 0.0)
    {
     if (error_file != NULL)
      fprintf (error_file,"Error reading float start time\n");
     if (error_file != NULL)
      fprintf (trace_file,"Error reading float start time\n");
     printf ("Error reading float start time\n");
     return (1);
    }
   }
   else
    meta_struct->start_time = (float) 0.0;

   //added
   meta_struct->start_of_cycles += meta_struct->start_time;

   continue;
  }
  i = 0;
  while ((j = strlen(cal_keys[i])) > 1)
  {
   if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
    meta_struct->cals.c[i] = atof(ptr+j);
   i++;
  }
 }
// for (i=0; i<(sizeof(struct aoml4_cals)/sizeof(double)); i++)
//  printf ("\nCal %0d %12.5lf",i,meta_struct->c[i]);

if(debug>0) printf("start_of_cycles=%f,cycle_time=%f\n",meta_struct->start_of_cycles,meta_struct->cycle_time);
 return 0;
}

int main (int argc, char *argv[])
{
 char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80], output_directory[256], msg[11]="", title[257]="";
 enum     {LAST=0,ALL} get_which_profile;
 struct   typeTSB1_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
 unsigned long t;
 int      ret_val = 0;

// Program Name
// ------------
 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature, salinity and pressure counts\n");
 strcat(title, "for bouncing Floats");
 printf("%s\n", title);


// Input Paramenters
// -----------------
 if ( argc < 7 )
 {
  printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
          PRGNAME);
  printf("\n");
  printf("hex_infile:    hexadecimal input file name\n");
  printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
  printf("cnt_outdir:    output directory for generated count data file(s)\n");
  printf("trace_outile:  output directory for generated trace file\n");
  printf("error_outfile: output directory for generated error file\n");
  printf("list_outfile:  output directory for generated list file\n");
  return 1;
 }

// Input Hex File name
// -------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
// -------------------------------------------------------------------------
 if (atoi(argv[2]) >= 1)
  get_which_profile = ALL;
 else
  get_which_profile = LAST;

 if(debug>0) printf("get_which_profile=%d\n",get_which_profile);

// Output directory path for count file(s)
// ---------------------------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (output_directory,"%s/",argv[3]);
 printf ("%s: Output Directory - %s\n", PRGNAME, output_directory);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 1;
  return ret_val;
 }

 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
   t = time(NULL);
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   ret_val = 1;
   return ret_val;
 }

 printf("%s: Trace file opened succesfully\n", PRGNAME);
 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[5]);
 printf ("%s: Opening error file %s\n", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
   if (trace_file != NULL)
     fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
 }

 if (trace_file != NULL)
   fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   if (trace_file != NULL)
     fprintf(trace_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   if (error_file != NULL)
     fprintf(error_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 1;
   return ret_val;
  }

// LIST File
// ---------
  sprintf (buffer,"%s",argv[6]);
  if (trace_file != NULL)
     fprintf (trace_file,"%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    t = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    if (error_file != NULL)
       fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val =1;
    return ret_val;
  }
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);

// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 1;
    return ret_val;
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
     if (trace_file != NULL)
       fprintf (trace_file,"ERROR (%s): Input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     if (error_file != NULL)
       fprintf (error_file,"ERROR (%s): input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     ret_val = 1;
  }
  else
  {
     if (trace_file != NULL)
       fprintf (trace_file,"%s: Input file %s successfully opened\n",
                PRGNAME, pathname_input_filename);

// META File
// ---------
     if (get_meta_file_name (input_filename) != 0)
     {                                             // Failure: did not get meta filename
      if (error_file != NULL)
        fprintf (error_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
        ret_val = 1;
     }
     else
     {                                            // Success: got meta filename
        if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
        {
           if (trace_file != NULL)
             fprintf (trace_file,
                      "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                      PRGNAME);
           if (error_file != NULL)
                fprintf (error_file,
                         "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                          PRGNAME);

           if (trace_file != NULL)                  // Close Log files
             fclose (trace_file);
           if (error_file != NULL)
             fclose (error_file);

           t = time(NULL);
           if (trace_file != NULL)
             fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           if (error_file != NULL)
             fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           ret_val = 1;
           return(ret_val);
        } // endif  ptr_path_meta

        sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

        if (read_typeTSB1_meta_file (pathname_input_filename, &s) != 0)
        {                                         // Failure: couldn't read meta_file
           if (trace_file != NULL)
             fprintf (trace_file, "ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
           if (error_file != NULL)
             fprintf (error_file,"ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
           ret_val = 1;
        }
        else                                      // Success: read meta_file
        {
           if (trace_file != NULL)
             fprintf (trace_file,"%s: Meta file %s successfully read\n",
                     PRGNAME, pathname_input_filename);

// Make profile(s) selection
// -------------------------
           if (get_which_profile == ALL)
           {
              if (get_typeTSB1_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           }
           else
           {
              if (get_typeTSB1_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           } // endif: get_which_profile

        } // endif: read_typeTSB1_meta_file
     } // endif: get_meta_file_name
  } //endif: input_file

//
  printf("HEX filename=%s\n",argv[1]);
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }


// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (trace_file != NULL)
    fprintf (trace_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

  if (error_file != NULL)
    fprintf (error_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------
 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);

 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
