/******************************************************************************
   Revision Date: 11-Jun-12 (CS  - renamed round to round_argo to avoid conflict
                                   in newer versions of c)
                  14-Apr-11 (ER: - Unnecessary abs function was detecting false 
                                   ghosts
                  08-FEB-11 (ER  - fix 256 profile taking 255 profile data.)
                  02-Feb-11 (ER  - corrected wrong assigment of park pressure to profile pressure
                  26-Oct-09 (LL  - add profile number > 265 case)
   				    09-Oct-08 (LL  - revise 23-Aug-06 (XXD) part)
   				    07-Oct-08 (LL  - revison of split_test_normal function)
   				    01-Oct-08 (LL  - profile number correction)
   				    25-Sep-08 (LL  - change (jd-jd_old)>2 to >=2)
    			    09-Apr-07 (XXD - fix the profile overwrite problem)
                  23-Aug-06 (XXD - position julian day should be larger than launchtime's)
                  07-Mar-06 (XXD - adapted from hex2cnt_tso3)

   hex2cnt_tso7.c
   =============
   Decoding of the hexadecimal ARGOS data for temperature/salinity/DO/pressure
   APEX floats APEX_TSO7 manufactured in 2002 by Webb Research Cooperation
  Notice: the number of samples always larger than the profile length(from message number 1)
   by 1 for this float

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans        2000    //Maximum scans for each profile
#define buff_size         256    //Maximum length of each scan read from *.str
#define max_positions     100    //Maximum number of positions per profile
#define min_frame_no        0    //Minimun frame number
#define frames_per_message 256   //Number of frames per message,it is the maximum allowed number
#define good_bad_position  43    //Location of status byte in *.str records
#define start_of_hex_data  63    //Start position of data in *.str records
#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0

int debug=0;//1 debug, 0 no debug
int frame1_problem_count=0,total_cycles_count=0;// .

int actual_frames_per_message;

char *strupr(char* ptr_to_string_to_be_upper_cased);

long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

char   PRGNAME[] ="HEX2CNT_TSO7";

int    aoml_id;
char   buffer[1024];
enum   flg {NO,YES} cal_flg;
//enum   data_flag {NONE,BAD,GOOD};
enum   {F,T} cd_input = F;
int   msg_type;

struct aoml_tso7_cals {double TSLOPE;
                       double TOFF;
                       double PSLOPE;
                       double POFF;
                       double SSLOPE;
                       double SOFF;
                       double SOC;
                       double FOFFSET;
                       double TCOR;
                       double PCOR;
                       double VSLOPE;
                       double VOFF;
                       double VACSLOPE;
                       double VACOFF;
                      };

union u_cals { struct aoml_tso7_cals s;
               double c[sizeof(struct aoml_tso7_cals)/sizeof(double)];
             };

struct type_O7_meta_struct { int AOML_number;
                           int WMO_id;
                           int ARGOS_id;
                           int wmo_instrument;
                           int wmo_recorder;
                           int exp_no;
                           unsigned char float_type[40];
                           unsigned char PI[60];
			   float launch_time;
                           float ARGOS_rep_rate;
                           float cycle_time;
                           float start_time;
			   float delay_first_down_time;
			   float up_time;
			   float start_of_cycles;//added Start_time+delay_of_start+down_time
                           union u_cals cals;
                         };

/*      Raw data structure used in decoding Temperature/Pressure/Oxygen (T/P/O)
 float data.  Part of RAW_DATA_RECORD described below */

struct test_fmt { unsigned char msg_blk_no;
                  unsigned char software_rev_month;
                  unsigned char software_rev_day;
                  unsigned char software_rev_year;
                  unsigned char controller_sn[2]; //serial number
                  unsigned char time_since_startup[2];
		  unsigned char status_word[2];
                  unsigned char current_pressure[2];
		  unsigned char vacuum;
                  unsigned char current_bladder_pressure;
                  unsigned char battery_volts;
                  unsigned char up_time_intervals;
                  unsigned char down_time_intervals[2];
                  unsigned char park_pressure[2];
                  unsigned char park_piston_position;
                  unsigned char depth_correction;
                  unsigned char ok_vacuum;
                  unsigned char ascend_time_intervals;
                  unsigned char target_bladder_pressure;
                  unsigned char profile_pressure[2];
                  unsigned char profile_piston_position;
                  unsigned char deep_cycle_count;
		  unsigned char dummy;
	          //test message #2
		  unsigned char piston_count_full_extention;
		  unsigned char piston_count_full_retraction;
		  unsigned char initial_buoyancy_nudge;
		  unsigned char deep_profile_descent_period;
		  unsigned char park_descent_period;
		  unsigned char mission_prelude_period;
		  unsigned char argos_repetition_period;
                };

struct data_fmt { unsigned char msg_blk_no;
                  unsigned char controller_sn[2];
                  unsigned char profile_no;
                  unsigned char profile_length;
                  unsigned char termination_flag[2];//status word
                  unsigned char surface_pressure[2];
                  unsigned char internal_vacuum;
                  unsigned char air_bladder_pressure;
                  unsigned char surface_piston_position;
                  unsigned char park_piston_position;
                  unsigned char profile_piston_position;
		  unsigned char ctd_status[2];
                  unsigned char air_pump_on_time[2];
                  unsigned char park_battery_volts;
		  unsigned char park_battery_current;
		  unsigned char battery_volts_sbe_pump_on;
                  unsigned char battery_current_sbe_pump_on;
                  unsigned char battery_volts;
                  unsigned char battery_current;
                  unsigned char battery_volts_air_pump_on;
                  unsigned char battery_current_air_pump_on;
		  //the following 3 fields are dummy values, for byte 28,29,30,ignore them in output file
                  //they corresponds to BT field in the specification
		  unsigned char bt1;
                  unsigned char bt2;
                  unsigned char bt3;

                  unsigned char nadj;
                  //next comes the 11 statistics
                  unsigned char num_samples[2];
		  unsigned char mean_temperature[2];
		  unsigned char mean_pressure[2];
		  unsigned char sd_temperature[2];//standard deviation of temperature
		  unsigned char sd_pressure[2];
		  unsigned char min_temperature[2];//minimum temperature
		  unsigned char pressure_when_min_temperature[2];//pressure associated with minimum temperature
		  unsigned char max_temperature[2];
		  unsigned char pressure_when_max_temperature[2];//pressure associated with maximum temperature
		  unsigned char min_pressure[2];
		  unsigned char	max_pressure[2];

                  unsigned char T_S_P[15*frames_per_message][9];
		  //p(2bytes),t(2bytes),s(2bytes),bt1(1 byte),bt2(1 byte),bt3(1 byte)
		  //note:bt1,bt2,bt3 are dummy values for BPhase and T2, totally 9 bytes
                };

/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { unsigned char data_bytes[frames_per_message][30];
                  struct data_fmt d_fmt;
                  struct test_fmt t_fmt;
                };

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { int jday;
                         double jd[max_positions];
                         float  latitude[max_positions];
                         float  longitude[max_positions];
                         char   date_time[max_positions][32];
                         int    n_xmits[max_positions];
                         char   sat_name[max_positions];
                         char   loc_class[max_positions];
                         union  data_size data;
                       };


/* Final structure containig T/P data and positions */

struct test_record   { int float_id;
                       int profile_no;
                       int jd;
                       int status[frames_per_message];
                       int msg_blk_no;
                       time_t first_blk_time;
                       int controller_sn;
                       int time_since_startup;	//Test stuff follows
		       int status_word;
                       int current_pressure;
		       int vacuum;
                       int current_bladder_pressure;
                       int battery_volts;
                       int up_time_intervals;
                       int down_time_intervals;
                       int park_pressure;
		       int park_piston_position;
                       int depth_correction;
                       int ok_vacuum;
                       int ascend_time_intervals;
                       int target_bladder_pressure;
                       int profile_pressure;
		       int profile_piston_position;
                       int deep_cycle_count;
                       int software_rev_month;
                       int software_rev_day;
                       int software_rev_year;
		       //from test message #2
		       int piston_count_full_extention;
		       int piston_count_full_retraction;
		       int initial_buoyancy_nudge;
		       int deep_profile_descent_period;
		       int park_descent_period;
		       int mission_prelude_period;
		       int argos_repetition_period;

                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                     };

/* Final structure containig T/P data and positions */

struct output_record { int float_id;
                       int profile_no;
                       int jd;
                       int status[frames_per_message];
                       int msg_blk_no;
                       time_t first_blk_time;
                       int controller_sn;
                       int time_since_startup;
                       int profile_length;	//data stuff follows
                       int termination_flag;//status word
                       int surface_pressure;
                       int internal_vacuum;
                       int air_bladder_pressure;
                       int surface_piston_position;
                       int park_piston_position;
                       int profile_piston_position;
		       int ctd_status;
                       int air_pump_on_time;
                       int park_battery_volts;
		       int park_battery_current;
		       int battery_volts_sbe_pump_on;
                       int battery_current_sbe_pump_on;
		       int battery_volts;
                       int battery_current;
                       int battery_volts_air_pump_on;
                       int battery_current_air_pump_on;
                       int nadj;

                       int num_samples;
		       int mean_temperature;
		       int mean_pressure;
		       int sd_temperature;//standard deviation of temperature
		       int sd_pressure;
		       int min_temperature;//minimum temperature
		       int pressure_when_min_temperature;//pressure associated with minimum temperature
		       int max_temperature;
		       int pressure_when_max_temperature;//pressure associated with maximum temperature
		       int min_pressure;
		       int max_pressure;

                       float  latitude[max_positions];
                       float  longitude[max_positions];
                       char   date_time[max_positions][32];
                       int    n_xmits[max_positions];
                       char   sat_name[max_positions];
                       char   loc_class[max_positions];
                       double position_jd[max_positions];
                       int    press_sal_temp[15*frames_per_message][6];
		       //[0]=p,[1]=t,[2]=s,[3]=bphase,[4]=t2,[5]=bin
                     };
char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

double dfr97 (const int yp, const int mp, const int dp,
     const int hr, const int minutes, const int secs, int *doy)
{
 int     yr, mo, dy, day_from_97;
 long    passed_jd, jd_xx0101, jd_970101;
 double  d_jd;

 yr = yp;
 mo = mp;
 dy = dp;
 if (yr <= 15)
  yr += 2000;
 else if (yr < 1900)
  yr += 1900;
 if ((yr<1997) || (yr>2015) || (mo<1) || (mo>12) || (dy<1) || (dy>31))
  return (0.0);
 passed_jd = ymd2julday( yr, mo, dy ) + 1;
 dy = 1;
 mo = 1;
 jd_xx0101 = ymd2julday( yr, mo, dy );
 yr = 1997;
 jd_970101 = ymd2julday( yr, mo, dy );
 day_from_97 = passed_jd - jd_970101;
 *doy = (int) (passed_jd - jd_xx0101);
 d_jd = day_from_97;
 d_jd += (hr+(minutes + secs/60.0)/60.0)/24.0;
 return (d_jd);
}

//need to consider carefully
int data_flag2(int bin_no, int *status_word)
{//9 because we have 9 bytes.the last 2 is added to fine tune it.
 return status_word[((bin_no-1)*9+30+5+2)/30];  //Add 30 since data starts at second frame,5.5+2 to make it upper bound
// return status_word[(bin_no+4)/5];
}

/* from position part, the first time line is parsed to get year,month,day,hour,minute,second,
   then compute the julian day,the date_time is in this format,for example
   2004/02/08 15:54:04
*/
long get_tim1(char* date_time){
   long tim1=0;
   static char tokensep[] = " /:";
   char *token;
   int ntokens=0;
   char temp[32];
   int year,month,day,hour,minute,second;
   token = (char *)strtok(date_time, tokensep);

   while(token != NULL)
   {
    ntokens++;
    switch (ntokens)
    {
      case 1:
        strcpy(temp, token);year=atoi(temp);
        break;
      case 2:
        strcpy(temp, token);month=atoi(temp);
        break;
      case 3:
        strcpy(temp, token);day=atoi(temp);
        break;
      case 4:
        strcpy(temp, token);hour=atoi(temp);
        break;
      case 5:
        strcpy(temp, token);minute=atoi(temp);
        break;
      case 6:
        strcpy(temp, token);second=atoi(temp);
        break;
      default: break;
    }
    token = (char *)strtok(NULL, tokensep);
   }
   tim1=((ymd2julday(year,month,day)*24+hour)*60+minute)*60+second;
   if(debug>0) printf("year=%d month=%d day=%d hour=%d minute=%d second=%d julday=%d tim1=%d\n",year,month,day,hour,minute,second,ymd2julday(year,month,day),tim1);
   return tim1;
}

//compute x_>first_blk_time,
//make sure the start of transmission time is less  than the first time in position block
 void getFirstBLKTime(struct type_O7_meta_struct *s, struct output_record *x){
  int kk;
  long tim1;
  char temp[32];

   kk=0;
   //find tim1
  for(kk=0;kk<max_positions;kk++){
   if(x->position_jd[kk] > 0.0){
    break;
   }
  }
  if(debug>0) printf("After kk loop! kk=%d,date=%s\n",kk,x->date_time[0]);
  strcpy(temp,x->date_time[kk]);
  tim1=get_tim1(temp);

  if(debug>0) printf("tim1=%d msg_blk_no=%d rep_rate=%f\n",tim1,x->msg_blk_no,s->ARGOS_rep_rate);

  if(x->msg_blk_no>0){
    while(x->first_blk_time>=tim1){
      x->first_blk_time -= (long) (x->msg_blk_no* s->ARGOS_rep_rate);
      //printf("in while loop,first_blk_time=%d\n",x->first_blk_time);
    }
  }

  if(debug>0){
    printf("normal profile,prof_no=%d start of transmission=%f msg_blk_no=%d rep_rate=%f\n",x->profile_no,(float)x->first_blk_time/24./3600.,x->msg_blk_no,s->ARGOS_rep_rate);
  }
}

//compute t->first_blk_time in test message
//make sure the start of transmission time is less  than the first time in position block
 void getFirstBLKTimeTest(struct type_O7_meta_struct *s, struct test_record *x){
  int kk;
  long tim1;
  char temp[32];

   kk=0;
   //find tim1
  for(kk=0;kk<max_positions;kk++){
   if(x->position_jd[kk] > 0.0){
    break;
   }
  }
  if(debug>0) printf("After kk loop! kk=%d,date=%s\n",kk,x->date_time[0]);
  strcpy(temp,x->date_time[kk]);
  tim1=get_tim1(temp);

  if(debug>0) printf("tim1=%d msg_blk_no=%d rep_rate=%f\n",tim1,x->msg_blk_no,s->ARGOS_rep_rate);

  if(x->msg_blk_no>0){
    while(x->first_blk_time>=tim1){
      x->first_blk_time -= (long) (x->msg_blk_no* s->ARGOS_rep_rate);
      //printf("in while loop,first_blk_time=%d\n",x->first_blk_time);
    }
  }

  if(debug>0){
    printf("test record,prof_no=%d start of transmission=%f msg_blk_no=%d rep_rate=%f\n",x->profile_no,(float)x->first_blk_time/24./3600.,x->msg_blk_no,s->ARGOS_rep_rate);
  }
}

// round a floating number to its nearest integer
// this function needed to be renamed from round due to a conflict
// in newer versions of c
int round_argo(float x) {
      if (x >= 0) return (int) (x+0.5);
      return (int) (x-0.5);
}

//compute correct profile number
int correct_profilenumber(float jd_firstline_prev_old,float jd_firstline_current_old,int profSeqCounter,int profile_no_prev,float start_of_cycles,float
cycle_time,int n_prof,int has_prof0){
  int correct_profile_no;
  //int adjustment=round_argo((jd_firstline_current_old-jd_firstline_prev_old)/cycle_time)-1;
  //if(has_prof0>0) adjustment--;
  //correct_profile_no=profSeqCounter+adjustment;

  int adjustment=round_argo((jd_firstline_current_old-jd_firstline_prev_old)/cycle_time);
  correct_profile_no=profile_no_prev+adjustment;

//  printf("profSeqCounter=%d,profile_no_prev=%d,adjustment=%d,cycle_time=%f,jd_current_old=%f,jd_prev_old=%f,jd_diff=%f\n",profSeqCounter,profile_no_prev,adjustment,cycle_time,jd_firstline_current_old,jd_firstline_prev_old,jd_firstline_current_old-jd_firstline_prev_old);

  if(n_prof!=correct_profile_no){
    printf("WARNING, n_prof NOT EQUAL TO correct_profile_no,n_prof=%d,correct_profile_no=%d,profSeqCounter=%d,profile_no_prev=%d,adjustment=%d\n",n_prof,correct_profile_no,profSeqCounter,profile_no_prev,adjustment);
  }
  return correct_profile_no;
}

int write_to_disk_test (char *dir_name, struct type_O7_meta_struct *s, struct test_record *x)
{
/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no
 temperature or pressure data. */

 int  i,j=0,id;
 char *ptr_cnt_filename;
// struct tm *time_str;

 id = x->float_id;
 if (((x->status[0]) == NEVER_RECEIVED) && (x->profile_no > 0))   //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file," First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file," First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 printf("%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);

 if (trace_file != NULL){
    fprintf (trace_file,"Opening output file %s...\n",buffer);
    printf("Opening output file %s...\n",buffer);
    }
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  if (trace_file != NULL){
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
   printf(" WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
   }
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 9;
 }
 if (trace_file != NULL){
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);
  printf(" WRITE_TO_DISK: output file %s successfully opened\n",buffer);
  }
// Write Output Profile Count File Name to List File
// -------------------------------------------------
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {printf("HEHE\n");
  if (trace_file != NULL){
   fprintf (trace_file,"%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
    printf("%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
   }
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Can't get cnt file basename %s\n", PRGNAME, buffer);
   return 9;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 printf("%s\n", ptr_cnt_filename);

 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %-5d\n"
     "TRANSMISSION ID NUMBER                  %-5d\n"
     "PROFILE NUMBER                          %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
		s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n"
     "CONTROLLER SN                           %d\n",
	    s->float_type,x->controller_sn);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 getFirstBLKTimeTest(s,x);
 if(debug>0) {
    printf("after function,first_blk_time=%d\n",x->first_blk_time);
    printf("(float)t->first_blk_time/24./3600.=%10.4f\n",(double)x->first_blk_time/24./3600.);
    printf("(long)=%d\n",(long)((double)x->first_blk_time/24./3600.));
    printf("julday2ymd=%s\n",julday2ymd((long)((double)x->first_blk_time/24./3600.)));
 }

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));

 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 //pure test message header goes here
  sprintf (buffer,
     "CURRENT PRESSURE (COUNTS)               %d\n"
     "BATTERY VOLTAGE (COUNTS)                %d\n"
     "CURRENT BLADDER PRESSURE                %d\n"
     "TARGET BLADDER PRESSURE                 %d\n",
     x->current_pressure, x->battery_volts,
	 x->current_bladder_pressure,x->target_bladder_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "STATUS WORD (HEX)                       %02X\n"
     "VACUUM                                  %d\n"
     "TIME SINCE START (2 SEC COUNTS)         %d\n",
     x->status_word,x->vacuum,x->time_since_startup);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "ASCEND TIME INTERVALS (HRS)             %d\n"
     "UP TIME INTERVALS (HRS)                 %d\n"
     "DOWN TIME INTERVALS (HRS)               %d\n",
     x->ascend_time_intervals,x->up_time_intervals,x->down_time_intervals);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "PROFILE PRESSURE (COUNTS)               %d\n"
     "PROFILE PISTON POSITION                 %d\n"
     "DEEP CYCLE COUNT                        %d\n",
     x->profile_pressure,x->profile_piston_position,x->deep_cycle_count);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DEPTH CORRECTION FACTOR                 %d\n",
     x->depth_correction);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DRIFT PRESSURE (COUNTS)                 %d\n"
     "DRIFT PISTON POSITION                   %d\n"
     "OK VACUUM COUNT                         %d\n",
          x->park_pressure,x->park_piston_position,x->ok_vacuum);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  //from test message #2
  sprintf (buffer,
     "PISTON CNT AT FULL EXTENSION (COUNTS)   %d\n"
     "PISTON CNT AT FULL RETRACTION (COUNTS)  %d\n"
     "INITIAL BUOYANCY NUDGE (COUNTS)         %d\n"
     "DEEP PROFILE DESCENT PERIOD (HOURS)     %d\n",
     x->piston_count_full_extention,x->piston_count_full_retraction,x->initial_buoyancy_nudge,x->deep_profile_descent_period);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "PARK DESCENT PERIOD (HOURS)             %d\n"
     "MISSION PRELUDE PERIOD (HOURS)          %d\n"
     "ARGOS REPETITION PERIOD (SECONDS)       %d\n",
     x->park_descent_period,x->mission_prelude_period,x->argos_repetition_period);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "SOFTWARE REVISION DATE (MON/DAY/YR)     %02d/%02d/%02d\n"
	 "==========================================================\n",
     x->software_rev_month,
     x->software_rev_day,x->software_rev_year);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);



 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 //printf ("   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");

 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 while (x->position_jd[i] > 0.0)
 {
 //added new checking to make sure the position line's julian day is larger than the launch time's. by xia
  int iy,imo,ida,ih,im,is,ndoy;
  float pos_jday,launchtime_jday;
  sscanf(x->date_time[i],"%4d/%02d/%02d%3d:%02d:%02d",&iy,&imo,&ida,&ih,&im,&is);
  pos_jday=(float) dfr97(iy,imo,ida,ih,im,is,&ndoy);
  //launchtime_jday=s->launch_time;
  //LL add this part
  if( s->launch_time-s->start_time < s->up_time && s->launch_time>s->start_time)
  	launchtime_jday=s->launch_time-s->up_time;
  else
    launchtime_jday=s->launch_time;
//  printf("%d=%4d/%02d/%02d%3d:%02d:%02d\n",i,iy,imo,ida,ih,im,is);
  if(pos_jday>launchtime_jday){

//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  i++;
 }

 fprintf (out_file,"==========================================================\n");

 if (out_file != NULL)
 fclose (out_file);   /* Close output file */

  return 0;
}

int write_to_disk (char *dir_name, struct type_O7_meta_struct *s, struct output_record *x)
{
/*  Writes output data to disk for T/P floats.  Profile 0 is
 transmitted when float is first turned on.  It has no
 temperature or pressure data. */

 int  i,j=0,id;
 char *ptr_cnt_filename;
// struct tm *time_str;

 id = x->float_id;
 if (((x->status[0]) == NEVER_RECEIVED) && (x->profile_no > 0))   //Must have received first scan
 {
  if (trace_file != NULL)
     fprintf (trace_file," First scan not received from float %05d - no data written\n",x->float_id);
  if (error_file != NULL)
     fprintf (error_file," First scan not received from float %05d - no data written\n",x->float_id);
  return(1);
 }
 sprintf(buffer,"%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);
 printf("%s%04d_%05d_%03d.cnt",dir_name,s->AOML_number,s->ARGOS_id,x->profile_no);

 if (trace_file != NULL){
    fprintf (trace_file,"Opening output file %s...\n",buffer);
    printf("Opening output file %s...\n",buffer);
    }
 if ((out_file = fopen(buffer,"w+")) == NULL) // used to be fopen(buffer,"a+")
 {
  if (trace_file != NULL){
   fprintf (trace_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
   printf(" WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
   }
  if (error_file != NULL)
   fprintf (error_file," WRITE_TO_DISK: ERROR opening output file %s\n",buffer);
  return 9;
 }
 if (trace_file != NULL){
  fprintf (trace_file," WRITE_TO_DISK: output file %s successfully opened\n",buffer);
  printf(" WRITE_TO_DISK: output file %s successfully opened\n",buffer);
  }
// Write Output Profile Count File Name to List File
// -------------------------------------------------
 if ( (ptr_cnt_filename = strrchr(buffer, '/')) == NULL)
 {printf("HEHE\n");
  if (trace_file != NULL){
   fprintf (trace_file,"%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
    printf("%s : Can't get cnt file basename %s\n", PRGNAME, buffer);
   }
   if (error_file != NULL)
    fprintf (error_file,
             "%s: Can't get cnt file basename %s\n", PRGNAME, buffer);
   return 9;
 } // endif: ptr_cnt_filename
 ptr_cnt_filename++;
 fprintf (list_file,"%s\n", ptr_cnt_filename);
 printf("%s\n", ptr_cnt_filename);

 sprintf (buffer,
     "INTERNAL ID NUMBER                      %04d\n"
     "WMO ID NUMBER                           %-5d\n"
     "TRANSMISSION ID NUMBER                  %-5d\n"
     "PROFILE NUMBER                          %d\n",
      s->AOML_number,abs(s->WMO_id),s->ARGOS_id,x->profile_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "WMO INSTRUMENT TYPE (TABLE 1770)        %d\n"
     "WMO RECORDER TYPE (TABLE 4770)          %d\n"
     "ARGOS PROGRAM NUMBER                    %d\n",
		s->wmo_instrument,s->wmo_recorder,s->exp_no);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "INSTRUMENT TYPE                         %s\n"
     "CONTROLLER SN                           %d\n",
	    s->float_type,x->controller_sn);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 sprintf (buffer,
     "PI                                      %s\n",s->PI);
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 getFirstBLKTime(s,x);
 if(debug>0) {
    printf("after function,first_blk_time=%d\n",x->first_blk_time);
    printf("(float)x->first_blk_time/24./3600.=%10.4f\n",(double)x->first_blk_time/24./3600.);
    printf("(long)=%d\n",(long)((double)x->first_blk_time/24./3600.));
    printf("julday2ymd=%s\n",julday2ymd((long)((double)x->first_blk_time/24./3600.)));
 }

 sprintf (buffer, "START OF TRANSMISSION                   %s",
          julday2ymd((long)((float)x->first_blk_time/24./3600.)));

 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 sprintf(buffer, " %s\n", timefrac2hms((double)x->first_blk_time/24./3600.));
 fwrite (strupr(buffer),1,strlen(buffer),out_file);

 if (x->profile_no > 0)
 {
  sprintf (buffer,
     "PROFILE LENGTH (BINS)                   %-5d\n"
     "AIR BLADDER PRESSURE                    %-5d\n"
     "AIR PUMP ON TIME                        %-5d\n",
      x->profile_length,x->air_bladder_pressure,x->air_pump_on_time);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "CTD STATUS                              %-5d\n"
     "NADJ                                    %-5d\n"
     "INTERNAL VACUUM                         %-5d\n",
      x->ctd_status,x->nadj,x->internal_vacuum);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "PROFILE PISTON POSITION                 %-5d\n"
     "PROFILE TERMINATION FLAG (HEX)          %-2X\n"
     "SURFACE PISTON POSITION                 %-5d\n",
      x->profile_piston_position,x->termination_flag,x->surface_piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "BATTERY VOLTAGE (COUNTS)                %-5d\n"
     "BATTERY CURRENT (COUNTS)                %-5d\n"
     "BATTERY VOLTAGE AIR PUMP ON (COUNTS)    %-5d\n"
     "BATTERY CURRENT AIR PUMP ON (COUNTS)    %-5d\n",
     x->battery_volts,x->battery_current,
     x->battery_volts_air_pump_on,x->battery_current_air_pump_on);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);


  sprintf (buffer,
     "BATTERY VOLTAGE SBE PUMP ON (COUNTS)    %-5d\n"
     "BATTERY CURRENT SBE PUMP ON (COUNTS)    %-5d\n",
      x->battery_volts_sbe_pump_on,x->battery_current_sbe_pump_on);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "SURFACE PRESSURE (COUNTS)               %-5d\n",
  x->surface_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DRIFT BATTERY VOLTAGE (COUNTS)          %-5d\n"
     "DRIFT BATTERY CURRENT (COUNTS)          %-5d\n"
     "DRIFT PISTON POSITION                   %-5d\n",
     x->park_battery_volts,x->park_battery_current,
     x->park_piston_position);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "DRIFT PRESSURE (COUNTS)                 %-5d\n"
     "DRIFT TEMPERATURE (COUNTS)              %-5d\n"
     "DRIFT SALINITY (COUNTS)                 %-5d\n"
     "DRIFT OXYGEN (COUNTS)                   %-5d %-5d\n",
     x->press_sal_temp[1][0],x->press_sal_temp[1][1],
     x->press_sal_temp[1][2],x->press_sal_temp[1][3],x->press_sal_temp[1][4]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

//11 statistics
  sprintf (buffer,
     "NUMBER OF SAMPLES (COUNTS)              %-5d\n"
     "MEAN TEMPERATURE (COUNTS)               %-5d\n"
     "MEAN PRESSURE (COUNTS)                  %-5d\n",
      x->num_samples,x->mean_temperature,x->mean_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "STANDARD DEVIATION OF TEMP (COUNTS)     %-5d\n"
     "STANDARD DEVIATION OF PRESSURE (COUNTS) %-5d\n"
     "MINIMUM TEMPERATURE (COUNTS)            %-5d\n",
      x->sd_temperature,x->sd_pressure,x->min_temperature);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "PRESSURE WHEN MINIMUM TEMP (COUNTS)     %-5d\n"
     "MAXIMUM TEMPERATURE (COUNTS)            %-5d\n"
     "PRESSURE WHEN MAXIMUM TEMP (COUNTS)     %-5d\n",
      x->pressure_when_min_temperature,x->max_temperature,x->pressure_when_max_temperature);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "MINIMUM PRESSURE (COUNTS)               %-5d\n"
     "MAXIMUM PRESSURE (COUNTS)               %-5d\n",
      x->min_pressure,x->max_pressure);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);

  sprintf (buffer,
     "NUMBER OF COLUMNS                       6\n"
     "1. COLUMN                               PRESSURE (COUNTS)\n"
     "2. COLUMN                               TEMPERATURE (COUNTS)\n"
     "3. COLUMN                               SALINITY (COUNTS)\n"
     "4. COLUMN                               BPHASE (COUNTS)\n"
     "5. COLUMN                               T2 (COUNTS)\n"
     "6. COLUMN                               CHECKSUM FLAG\n"
	 "==========================================================\n");

  fwrite (strupr(buffer),1,strlen(buffer),out_file);

 }

 i = 0;
 sprintf (buffer,"   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");
 //printf ("   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT CLASS\n");

 fwrite (strupr(buffer),1,strlen(buffer),out_file);
 while (x->position_jd[i] > 0.0)
 {
 //added new checking to make sure the position line's julian day is larger than the launch time's. by xia
  int iy,imo,ida,ih,im,is,ndoy;
  float pos_jday,launchtime_jday;
  sscanf(x->date_time[i],"%4d/%02d/%02d%3d:%02d:%02d",&iy,&imo,&ida,&ih,&im,&is);
  pos_jday=(float) dfr97(iy,imo,ida,ih,im,is,&ndoy);
  launchtime_jday=s->launch_time;
  //printf("%4d/%02d/%02d%3d:%02d:%02d\n",iy,imo,ida,ih,im,is);
  if(pos_jday>launchtime_jday){

//                  "----LLL.LLL---LLLL.LLL-YYYY/MM/DD-HH:MM:SS---XXX---S-----C"
  sprintf (buffer,"    %+7.3f   %+8.3f %-19s   %03d   %1c     %1c\n",
    x->latitude[i],x->longitude[i],x->date_time[i],x->n_xmits[i],x->sat_name[i],
    x->loc_class[i]);
  fwrite (strupr(buffer),1,strlen(buffer),out_file);
  }

  i++;
 }

 fprintf (out_file,"==========================================================\n");

/* Write depth, temperature, and salinity, o2 data to disk */

 if (x->profile_no > 0)
 {
  i = 2;//the i=1 line contains drift data, pay attention here. it is different from tso1 here.
  while (x->press_sal_temp[i][0] > -999)
  {


   if ((j = data_flag2(x->press_sal_temp[i][5],x->status)) == NEVER_RECEIVED)
    sprintf (buffer," %5d  %5d  %5d  %5d  %5d  %d\n",0,0,0,0,0,0);
   else

    sprintf (buffer," %5d  %5d  %5d  %5d  %5d  %d\n",
	                      x->press_sal_temp[i][0],x->press_sal_temp[i][1],
		              x->press_sal_temp[i][2],x->press_sal_temp[i][3],x->press_sal_temp[i][4],
			      j);
   fwrite (strupr(buffer),1,strlen(buffer),out_file);

   if (++i > 9*frames_per_message)	   break;
  }
 }

   if (out_file != NULL)
    fclose (out_file);   /* Close output file */

  return 0;
}

int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad)
{
 int i,j,k,l,matches = 0;
 int num_matches,most_likely = -1;
 char tmp_buff[2][256];

//  Finds the most likely scan for the APEX floats
//  Returns number of most common scan except for test message
//  Search for test message if frame_num = 0
//  Test message detected if block number > frames per message and flag2 bits 6 or 7 set
//  Returns -1 if error
//  good_bad=1 -> use checksum; good_bad=0->ignore checksum
/*
09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF
*/

 for (i=0; i<n_scans; i++)
 {
  strcpy(tmp_buff[0],buffer[i]);
  if ((tmp_buff[0][good_bad_position] != '0') && good_bad)
   continue;       //Ignore if status byte = bad
  sscanf(&tmp_buff[0][start_of_hex_data+2],"%2X",&k);
/*  if ((k > frames_per_message) && (frame_num == 0))
  {
   sscanf(&tmp_buff[0][start_of_hex_data+14],"%2X",&j); //j = flag2 byte
   if (((j & 0X60) == 0X20) || ((j & 0X60) == 0X40))
	most_likely = i;	//returns last test message with block number > frames_per_message
   continue;
  }*/
  if ((k-1) != frame_num)	//Frame = 1 to 15 => most_common[0-14]
   continue;
//  if (k == 1)
//  {
//   tmp_buff[0][start_of_hex_data+4] = '0';
//   tmp_buff[0][start_of_hex_data+5] = '0'; //set message blk number = 0 because it's not constant
//  }
  num_matches = 0;
  for (j=0; j<n_scans; j++)
  {
   strcpy(tmp_buff[1],buffer[j]);
   sscanf(&tmp_buff[1][start_of_hex_data+2],"%2X",&l);
   if (k != l)
    continue;   //If frames are different then skip
//   if (l == 1)
//   {
//    tmp_buff[1][start_of_hex_data+4] = '0';
//    tmp_buff[1][start_of_hex_data+5] = '0'; //set message blk number = 0 because it's not constant
//   }
   if (strcmp(&tmp_buff[0][start_of_hex_data+6],&tmp_buff[1][start_of_hex_data+6]) == 0)
    num_matches++;  //Hex data the same => increment match count
  }
  if (num_matches > matches) //Is this the most matches?
  {
   matches = num_matches; //Yes - save match count
   most_likely = i;       //Save record number
  }
// TEST CS
  if ((num_matches==0) && (matches==0)) // Make sure to use messages only received once
  {
    most_likely=i;
  }

 }

 return most_likely;

}

//decode pure test messages,only header and position part.
int get_raw_data_test (struct test_record *y, char ** buff, int n_scans, int m_type)
{
  struct raw_data_record x;
  struct tm *time_str;
  int i,j,k,most_common[frames_per_message+1]; //frames_per_mesage+1 for test mesage
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0;
  int iy1,imo1,ida1,ih1,im1,is1;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32],roll_over=0,old_count=0;


  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
   y->status[i] = NEVER_RECEIVED;
   for (j=0; j<30; j++)
    x.data.data_bytes[i][j] = 0;
   if (m_type == 1)
	   break;
   if ((j = most_likely_frame (buff, n_scans, i, 1)) >= 0) //find most likely - good CRC only
    most_common[i] = j;
   else if ((j = most_likely_frame (buff, n_scans, i, 0)) >= 0) //find most likely - any CRC
    most_common[i] = j;
   else
    most_common[i] = -1;
  }

  for (j=1; j<max_positions; j++)
   x.jd[j] = -1.0;


  for (i=0; i<n_scans; i++)
  {
   if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
    continue;
    x.latitude[n_pos] = lat;
    x.longitude[n_pos] = lon;
    x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

    sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
    x.loc_class[n_pos] = l_c;
    x.sat_name[n_pos] = sat;
    x.n_xmits[n_pos] = (int) floor((n_l-1)/8);

   if(n_pos==0){//reset to first 99.999 999.999 line
     x.latitude[n_pos] = (float) 99.999;
     x.longitude[n_pos] = (float) 999.999;
     x.n_xmits[n_pos] = 0;
     n_pos++;
   }

   if((lat != (float) 0.0) || (lon != (float) 0.0))
   {
    x.latitude[n_pos] = lat;
    x.longitude[n_pos] = lon;
    x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

    sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
    x.loc_class[n_pos] = l_c;
    x.sat_name[n_pos] = sat;
    x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
    n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
    continue;
   }

//16920 2000 12 13 06 21 50 +00.000 +000.000 0  25 L 0 126882763 497101C401595248000F8F5A0C1000F0014E20230C18F8450957FFFFFF072800
   for (j=0; j<32; j++)
    sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

   if ((z[1] - min_frame_no) < 1)
    continue;

   if ((m_type == 0) && (g == 0))
   {
	if (z[2] < old_count)
		roll_over++;
	old_count = z[2];
    y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
     for (k=0; k<30; k++){//special treatment for test message with two message numbers
        //for test message #1
        if(z[1]==1)    {x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];}
     }
     for(k=6;k<13;k++){//for test message #2,FEXT,FRET,IBN,DPDP,PDP,PRE,REP
	if(z[1]==2)  {  x.data.data_bytes[z[1]-1][k-6] = (unsigned char) z[k];}//from FEXT on
     }

   }
//Speed up - move time calculations outside loop
   if (z[1] == 1)
   {
    is1 = is;
    im1 = im;
    ih1 = ih;
    ida1 = ida;
    imo1 = imo;
    iy1 = iy;
   }
  }//end of for n_scans loop

//add the last 999.999 line
     x.latitude[n_pos] = (float) 99.999;
     x.longitude[n_pos] = (float) 999.999;
     x.n_xmits[n_pos] = 0;

    y->first_blk_time = ((ymd2julday(iy1,imo1,ida1)*24+ih1)*60+im1)*60+is1;

  x.jday = (int) floor(x.jd[0]);

  if ((trace_file != NULL) && (m_type == 1))
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",(int)x.data.d_fmt.profile_no);
    for (j=(frames_per_message-1); j>0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }

  y->float_id = id;
  y->jd = x.jday;


  if (m_type == 0)   //Test message
  {
   y->profile_no = 0;
   y->controller_sn = 256*x.data.t_fmt.controller_sn[0] + x.data.t_fmt.controller_sn[1];
   y->time_since_startup = 256*x.data.t_fmt.time_since_startup[0] + x.data.t_fmt.time_since_startup[1];
   y->status_word = 256*x.data.t_fmt.status_word[0]+x.data.t_fmt.status_word[1];
   y->current_pressure = 256*x.data.t_fmt.current_pressure[0] + x.data.t_fmt.current_pressure[1];
     if(y->current_pressure>32767){
       y->current_pressure=y->current_pressure-65535;
     }
   y->battery_volts = x.data.t_fmt.battery_volts;
   y->current_bladder_pressure = x.data.t_fmt.current_bladder_pressure;
     if(y->current_bladder_pressure>32767){
       y->current_bladder_pressure=y->current_bladder_pressure-65535;
     }
   y->vacuum = x.data.t_fmt.vacuum;
   y->up_time_intervals = x.data.t_fmt.up_time_intervals;
   y->down_time_intervals = 256*x.data.t_fmt.down_time_intervals[0] + x.data.t_fmt.down_time_intervals[1];
   y->depth_correction = x.data.t_fmt.depth_correction;
   y->ok_vacuum = x.data.t_fmt.ok_vacuum;
   y->ascend_time_intervals = x.data.t_fmt.ascend_time_intervals;
   y->target_bladder_pressure = x.data.t_fmt.target_bladder_pressure;
     if(y->target_bladder_pressure>32767){
       y->target_bladder_pressure=y->target_bladder_pressure-65535;
     }
// ER 02-Feb-11  corrected wrong assigment of park pressure to profile pressure
//   y->profile_pressure = 256*x.data.t_fmt.park_pressure[0] + x.data.t_fmt.park_pressure[1];
   y->profile_pressure = 256*x.data.t_fmt.profile_pressure[0] + x.data.t_fmt.profile_pressure[1];
     if(y->profile_pressure>32767){
       y->profile_pressure=y->profile_pressure-65535;
     }
   y->profile_piston_position = x.data.t_fmt.profile_piston_position;
   y->park_pressure = 256*x.data.t_fmt.park_pressure[0] + x.data.t_fmt.park_pressure[1];
     if(y->park_pressure>32767){
       y->park_pressure=y->park_pressure-65535;
     }
   y->park_piston_position = x.data.t_fmt.park_piston_position;
   y->deep_cycle_count = x.data.t_fmt.deep_cycle_count;
   y->software_rev_month = (x.data.t_fmt.software_rev_month/16)*10 + x.data.t_fmt.software_rev_month%16;
   y->software_rev_day = (x.data.t_fmt.software_rev_day/16)*10 + x.data.t_fmt.software_rev_day%16;
   y->software_rev_year = (x.data.t_fmt.software_rev_year/16)*10 + x.data.t_fmt.software_rev_year%16;
   //test message #2
   y->piston_count_full_extention=x.data.t_fmt.piston_count_full_extention;
   y->piston_count_full_retraction=x.data.t_fmt.piston_count_full_retraction;
   y->initial_buoyancy_nudge=x.data.t_fmt.initial_buoyancy_nudge;
   y->deep_profile_descent_period=x.data.t_fmt.deep_profile_descent_period;
   y->park_descent_period=x.data.t_fmt.park_descent_period;
   y->mission_prelude_period=x.data.t_fmt.mission_prelude_period;
   y->argos_repetition_period=x.data.t_fmt.argos_repetition_period;

  }
  for (j=0; j<max_positions; j++)
  {
   y->latitude[j] = x.latitude[j];
   y->longitude[j] = x.longitude[j];
   y->position_jd[j] = x.jd[j];
   y->n_xmits[j] = x.n_xmits[j];
   y->sat_name[j] = x.sat_name[j];
   y->loc_class[j] = x.loc_class[j];
   strncpy(y->date_time[j],x.date_time[j],24);
  }

  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(0);
}

int get_raw_data (struct output_record *y, char ** buff, int n_scans, int m_type)
{
  struct raw_data_record x;
//  struct tm *time_str;
  int i,j,k,most_common[frames_per_message+1]; //frames_per_mesage+1 for test mesage
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0,n_pos=0;
  int iy1,imo1,ida1,ih1,im1,is1;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32],roll_over=0,old_count=0;


  total_cycles_count++;

  for (i=0; i<frames_per_message; i++)
  {
   y->status[i] = NEVER_RECEIVED;
   for (j=0; j<30; j++)
    x.data.data_bytes[i][j] = 0;
   if (m_type == 0)
	   break;
   if ((j = most_likely_frame (buff, n_scans, i, 1)) >= 0) //find most likely - good CRC only
    most_common[i] = j;
   else if ((j = most_likely_frame (buff, n_scans, i, 0)) >= 0) //find most likely - any CRC
    most_common[i] = j;
   else
    most_common[i] = -1;
  }

// CS - FRAME 1 PROBLEM
  if ((m_type == 1) && (most_common[0] < 0)) //frame =1 => most_common[0]
  {
   //
   frame1_problem_count++;

   if (trace_file != NULL)
//    fprintf (trace_file," frame 1 missing\n");
    fprintf (trace_file," Can't decode message without frame 1\n");
   if (error_file != NULL)
    fprintf (error_file," Can't decode message without frame 1\n");
   return -1;	//Can't decode message without frame -1 - error return
  }

  for (j=1; j<max_positions; j++)
   x.jd[j] = -1.0;


  for (i=0; i<n_scans; i++)
  {
   if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
    continue;


  if (m_type == 1)
  {
     // Read third and fourth characters from hex_data, convert from hex to decimal and
     // skip if > frames_per_message.
     frames=0;
     for (l=0; l<2 ; l++)
     {
        ch=*(hex_data+l+2);
        switch (ch)
        {
          case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
          case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
          case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
          case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
          case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
          case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
          default : frms[0]=ch; frms[1]='\0'; break;
        }
        frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
     }
     if (frames > frames_per_message)
     {
       continue;
     }
  }
//first read each line
    x.latitude[n_pos] = lat;
    x.longitude[n_pos] = lon;
    x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

    sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
    x.loc_class[n_pos] = l_c;
    x.sat_name[n_pos] = sat;
    x.n_xmits[n_pos] = (int) floor((n_l-1)/8);

   if(n_pos==0){//reset to first 99.999 999.999 line
     x.latitude[n_pos] = (float) 99.999;
     x.longitude[n_pos] = (float) 999.999;
     x.n_xmits[n_pos] = 0;
     n_pos++;
   }

   if((lat != (float) 0.0) || (lon != (float) 0.0))
   {
    x.latitude[n_pos] = lat;
    x.longitude[n_pos] = lon;
    x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

    sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
    x.loc_class[n_pos] = l_c;
    x.sat_name[n_pos] = sat;
    x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
    n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
    continue;
   }

//16920 2000 12 13 06 21 50 +00.000 +000.000 0  25 L 0 126882763 497101C401595248000F8F5A0C1000F0014E20230C18F8450957FFFFFF072800
   for (j=0; j<32; j++)
    sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

   if ((z[1] - min_frame_no) < 1)
    continue;

   if (m_type == 1)
   {
    if (z[1]>frames_per_message)
    {
    if (trace_file != NULL)
      fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"z[1] exceeds frames_per_message.\n");
    exit (1);
    }
    if (i != (most_common[z[1] - 1]))
     continue;

//print the most likely message,important debug statement!(xxd)
  if(debug>0) printf("Most likely message: msg num=%d,line index=%d,hex data=%s\n",z[1]-1,i,hex_data);

    if (z[1] == 1)
    {
     if (z[2] < old_count)
		roll_over++;
     old_count = z[2];
     y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
    }
    y->status[z[1]-1] = (g == 0) ? GOOD : BAD;
    for (k=0; k<30; k++)
     x.data.data_bytes[z[1]-1][k] = (unsigned char) z[k+2];
   }

//Speed up - move time calculations outside loop
   if (z[1] == 1)
   {
    is1 = is;
    im1 = im;
    ih1 = ih;
    ida1 = ida;
    imo1 = imo;
    iy1 = iy;
   }
  }//end of for n_scans loop

//add the last 999.999 line
     x.latitude[n_pos] = (float) 99.999;
     x.longitude[n_pos] = (float) 999.999;
     x.n_xmits[n_pos] = 0;

    y->first_blk_time = ((ymd2julday(iy1,imo1,ida1)*24+ih1)*60+im1)*60+is1;

  x.jday = (int) floor(x.jd[0]);

  if ((trace_file != NULL) && (m_type == 1))
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",(int)x.data.d_fmt.profile_no);
    for (j=(frames_per_message-1); j>0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }

  y->float_id = id;
  y->jd = x.jday;


  if (m_type == 1) //Data message
  {
   y->profile_no = x.data.d_fmt.profile_no;
   y->controller_sn = 256*x.data.d_fmt.controller_sn[0] + x.data.d_fmt.controller_sn[1];
   y->profile_length = x.data.d_fmt.profile_length;
   y->termination_flag = 256*x.data.d_fmt.termination_flag[0]+x.data.d_fmt.termination_flag[1];
   y->battery_volts = x.data.d_fmt.battery_volts;
   y->battery_current = x.data.d_fmt.battery_current;
   y->profile_piston_position = x.data.d_fmt.profile_piston_position;
   y->surface_piston_position = x.data.d_fmt.surface_piston_position;
   y->air_pump_on_time = 256*x.data.d_fmt.air_pump_on_time[0] + x.data.d_fmt.air_pump_on_time[1];
   y->air_bladder_pressure = x.data.d_fmt.air_bladder_pressure;
    if(y->air_bladder_pressure>32767){//correct overflow
      y->air_bladder_pressure=y->air_bladder_pressure-65535;
    }
   y->battery_volts_air_pump_on = x.data.d_fmt.battery_volts_air_pump_on;
   y->battery_current_air_pump_on = x.data.d_fmt.battery_current_air_pump_on;
   y->park_battery_volts = x.data.d_fmt.park_battery_volts;
   y->park_battery_current = x.data.d_fmt.park_battery_current;
   y->nadj = x.data.d_fmt.nadj;
   y->surface_pressure = (256*x.data.d_fmt.surface_pressure[0] +
    x.data.d_fmt.surface_pressure[1]);
    if(y->surface_pressure>32767){//correct overflow
      y->surface_pressure=y->surface_pressure-65535;
    }
   y->ctd_status = 256*x.data.d_fmt.ctd_status[0]+x.data.d_fmt.ctd_status[1];
   y->internal_vacuum = x.data.d_fmt.internal_vacuum;
   y->park_piston_position = x.data.d_fmt.park_piston_position;
   y->battery_volts_sbe_pump_on = x.data.d_fmt.battery_volts_sbe_pump_on;
   y->battery_current_sbe_pump_on = x.data.d_fmt.battery_current_sbe_pump_on;
   //11 statistics
   y->num_samples=256*x.data.d_fmt.num_samples[0] + x.data.d_fmt.num_samples[1];
   y->mean_temperature=256*x.data.d_fmt.mean_temperature[0] + x.data.d_fmt.mean_temperature[1];
   y->mean_pressure=256*x.data.d_fmt.mean_pressure[0] + x.data.d_fmt.mean_pressure[1];
   y->sd_temperature=256*x.data.d_fmt.sd_temperature[0] + x.data.d_fmt.sd_temperature[1];
   y->sd_pressure=256*x.data.d_fmt.sd_pressure[0] + x.data.d_fmt.sd_pressure[1];
   y->min_temperature=256*x.data.d_fmt.min_temperature[0] + x.data.d_fmt.min_temperature[1];
   y->pressure_when_min_temperature=256*x.data.d_fmt.pressure_when_min_temperature[0] + x.data.d_fmt.pressure_when_min_temperature[1];
   y->max_temperature=256*x.data.d_fmt.max_temperature[0] + x.data.d_fmt.max_temperature[1];
   y->pressure_when_max_temperature=256*x.data.d_fmt.pressure_when_max_temperature[0] + x.data.d_fmt.pressure_when_max_temperature[1];
   y->min_pressure=256*x.data.d_fmt.min_pressure[0] + x.data.d_fmt.min_pressure[1];
   y->max_pressure=256*x.data.d_fmt.max_pressure[0] + x.data.d_fmt.max_pressure[1];
  }
  for (j=0; j<max_positions; j++)
  {
   y->latitude[j] = x.latitude[j];
   y->longitude[j] = x.longitude[j];
   y->position_jd[j] = x.jd[j];
   y->n_xmits[j] = x.n_xmits[j];
   y->sat_name[j] = x.sat_name[j];
   y->loc_class[j] = x.loc_class[j];
   strncpy(y->date_time[j],x.date_time[j],24);
  }

  actual_frames_per_message = (int) ceil((( (float)y->profile_length*9)/30))+1;
// CS - FRAME 1 PROBLEM
//  if ((m_type == 1) && (most_common[0] < 0)) //frame =1 => most_common[0]
//  {
//     actual_frames_per_message=frames_per_message;
//     if ((prof_no = (int) (((out_array->jday - s->start_cycle_time)/(s->cycle_time)) + .5)) < 0)
//     {
//          printf("write_sio_disk: profile number problem %d\n",prof_no);
//          printf("%d %f %f\n",out_array->jday,s->start_cycle_time,s->cycle_time);
//          return 1;
//     }
//  }
// CS - FRAME 1 PROBLEM

  if (actual_frames_per_message > frames_per_message)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"actual_frames_per_message exceeds frames_per_message.\n");
    if (error_file != NULL)
      fprintf (error_file,"actual_frames_per_message exceeds frames_per_message.\n");
    exit (1);
  }


//  if (y->profile_no > 0)
//  {//ATTENTION,CHANGED FROM TSO1
   y->press_sal_temp[0][0] = 0;
   y->press_sal_temp[0][1] = 0;
   y->press_sal_temp[0][2] = 0;
   y->press_sal_temp[0][3] = 0;
   y->press_sal_temp[0][4] = 0;
   y->press_sal_temp[0][5] = 0;
   //Notice: the number of samples always larger than the profile length(from message number 1) by 1 for this
   //float
   int bphase,t2;
   unsigned char bt1,bt2,bt3;

   for (j=1; j<=(y->profile_length+1); j++)	//length-1 because bottom values included
   {
     y->press_sal_temp[j][1] = (256*x.data.d_fmt.T_S_P[j-1][0] +x.data.d_fmt.T_S_P[j-1][1]); //Temperature
     y->press_sal_temp[j][2] = (256*x.data.d_fmt.T_S_P[j-1][2] +x.data.d_fmt.T_S_P[j-1][3]); //Salinity
     y->press_sal_temp[j][0] = (256*x.data.d_fmt.T_S_P[j-1][4] +x.data.d_fmt.T_S_P[j-1][5]); //Pressure
     bt1=x.data.d_fmt.T_S_P[j-1][6];bt2=x.data.d_fmt.T_S_P[j-1][7];bt3=x.data.d_fmt.T_S_P[j-1][8];
     bphase=bt1;bphase<<=4;bphase|=((bt2>>4)&0xf);
     t2=bt2&0xf;t2<<=8;t2|=bt3;
     y->press_sal_temp[j][3] = bphase;
     y->press_sal_temp[j][4] = t2 ;

     y->press_sal_temp[j][5] = j;	                                                     //Bin number
   }
   y->press_sal_temp[j][0] = -999;
   y->msg_blk_no *= actual_frames_per_message;
// }

  if (trace_file != NULL)
   fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  return(y->profile_no);
}

/*
   split the whole test message into real test message and profile data part based
   on the julian day, because in the test message, there are profile data(including header)
   we will merge the test data and profile0 data into profile cycle 0 cnt and phy file.
*/

void split_test_normal(char **buff,int nscans,char **buff_test,int* tlines,char **buff_normal,int* nlines){
   int i,j=0,k;
   int iy,imo,ida,ih,im,is,ndoy;
   float jd[nscans],jd_diff_max;

   //get julian day for each hex data line
   for(i=0;i<nscans;i++){
     if(sscanf(buff[i],"%d %d %d %d %d %d %d", &k,&iy,&imo,&ida,&ih,&im,&is) != 7) continue;
     jd[i]=(float) dfr97(iy,imo,ida,ih,im,0,&ndoy);
     if(debug>0) printf("jd[%d]=%f, buff=%s\n",i,jd[i],buff[i]);
   }
   //find the index where jd difference is the largest
   // jd_diff_max=0.0/24.0; //LL: diff>6 hours
   jd_diff_max=6.0/24.0;
   for(i=0;i<nscans-1;i++){
     if(jd[i+1]-jd[i]>jd_diff_max) {
       jd_diff_max=jd[i+1]-jd[i];
   //    printf("jd_diff_max=%f\n",jd_diff_max);
       j=i;
     }
   }

   // if(j==0) j=-1; //LL: make sure there is 000 file
   if(j==0) j=i;
   *tlines=j+1;
   *nlines=nscans-*tlines;

   for(i=*tlines;i<nscans;i++){ strcpy(buff_normal[i-*tlines],buff[i]);}

   for(i=0;i<*tlines;i++){
      strcpy(buff_test[i],buff[i]);
   }
   if(debug>0) printf("in split function, nscans=%d,test_lines=%d,normal_lines=%d\n",nscans,*tlines,*nlines);
}


int get_type_O7_data(int np, struct type_O7_meta_struct *s, char *out_dir)
{
  struct output_record y;
  struct test_record test;
  char **buff;
  char **buff_test;
  char **buff_normal;
  int nlines=0;//in test message part, the number of lines which contain profile0 data
  int tlines=0;//in test message part, the number of lines for traditional test messages
  int i,j;
  int iy,imo,ida,ih,im,is,ndoy;
  int n_prof=0; //,jd_old=0,jd;
  float d_jd;
  int write_to_disk_error;
  int kk,find=0;//added
//  int corrected_prof_no;

  //new modification
  float jd_old=0,jd;
  int profSeqCounter=0,times=0,has_prof0=0,profile_no_prev=0,profile_no_current=0;
  float jd_firstline_prev_old=0,jd_firstline_current_old=0;
  float jd_firstline_prev_new=0,jd_firstline_current_new=0;
  char* onelinehex;

/* Allocate needed memory */
/* Allocate needed memory for buff,buff_normal,buff_bounce*/
  if (((buff_normal = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)||
      ((buff_test   = (void *) malloc(max_scans*(sizeof(void *)))) == NULL)||
      ((buff        = (void *) malloc(max_scans*(sizeof(void *)))) == NULL))
  {
   if (trace_file != NULL)
        fprintf (trace_file," GET_typeTSO7_DATA: Error allocating buffer pointers\n");
   if (error_file != NULL)
        fprintf (error_file," GET_typeTSO7_DATA: Error allocating buffer pointers\n");
   return 9;
  }

  for (i=0; i<max_scans; i++)
   if (((buff[i]        = (char *) malloc(buff_size*(sizeof(char)))) == NULL)||
       ((buff_normal[i] = (char *) malloc(buff_size*(sizeof(char)))) == NULL)||
       ((buff_test[i]   = (char *) malloc(buff_size*(sizeof(char)))) == NULL))
   {
   if (trace_file != NULL)
        fprintf (trace_file," GET_typeTSO7_DATA: Error allocating data buffer arrays\n");
   if (error_file != NULL)
        fprintf (error_file," GET_typeTSO7_DATA: Error allocating data buffer arrays\n");
    return 9;
   }
   if((onelinehex=(char *) malloc(buff_size*(sizeof(char))))==NULL){
     if (trace_file != NULL)
        fprintf (trace_file," GET_typeO7_DATA: Error allocating data buffer arrays\n");
     if (error_file != NULL)
        fprintf (error_file," GET_typeO7_DATA: Error allocating data buffer arrays\n");
     return 9;
   }
  if (trace_file != NULL)
   fprintf (trace_file," GET_typeO7_DATA: Arrays successfully allocated\n");

/* Get the data */

  j=0;
//  while (fgets(buff[j],buff_size,input_file) != NULL)
//  {
//   if(sscanf(buff[j],"%d %d %d %d %d %d %d",&i,&iy,&imo,&ida,&ih,&im,&is) != 7)   continue;

  while (fgets(onelinehex,buff_size,input_file) != NULL)
  {
     if(sscanf(onelinehex,"%d %d %d %d %d %d %d",&i,&iy,&imo,&ida,&ih,&im,&is) != 7)  continue;
     /*
     usually float launched within delay of first down time since start time, if not,then do the following
     if to process all profiles,and there are dates which are less than launch time,print warning
     not done if cycle 0 is genereated */

     if (np==1){
//	 if ((s->launch_time - s->start_time > s->delay_first_down_time) && np==1){
       if(debug>0) printf("launch time=%f, start_time=%f,delay_first_down_time=%f\n",s->launch_time,s->start_time,s->delay_first_down_time);
      if (dfr97(iy,imo,ida,ih,im,0,&ndoy) < s->launch_time-s->up_time){
       if (trace_file != NULL)
        fprintf (trace_file," GET_typeO7_DATA: WARNING, data before launch time\n");
       if (error_file != NULL)
        fprintf (error_file," GET_typeO7_DATA: WARNING, data before launch time\n");
	continue;
      }
     }

     if (s->start_time > (d_jd = (float) dfr97(iy,imo,ida,ih,im,0,&ndoy))) continue;

     jd = (int) d_jd;
     times++;if(times==1) {jd_old=jd;jd_firstline_prev_new=jd;}

     if(jd-jd_old<2) strcpy(buff[j],onelinehex);

	 //skip those lines whose jd difference is less than normal cycle, these are ghost messages.(XXD)
	 if(jd-jd_old>=2 && debug>0) printf("jd=%f,jd_firstline_prev_old=%f,jd_firstline_prev_new=%f,jd-jd_firstline_prev_old=%f,0.8*cycle=%f\n",jd,jd_firstline_prev_old,jd_firstline_prev_new,jd-jd_firstline_prev_old,0.8*s->cycle_time);
	 if(jd-jd_old>=2 && (jd-jd_firstline_prev_new)<0.8*s->cycle_time) {printf("bypassing--%s\n",onelinehex);continue;}

     if ((abs(jd - jd_old) >= 2) && (j > 0))
     {  if(debug>0) printf("jd_old=%d,start_time=%d,jd_old-start_time=%d\n",jd_old,(int)s->start_time,jd_old-(int)s->start_time);
        //if (np > 0)
        //{  if(debug>0) printf("jd_old=%d,start_time=%d,jd_old-start_time=%d\n",jd_old,(int)s->start_time,jd_old-(int)s->start_time);

		   profSeqCounter++;jd_firstline_current_new=jd;

           if ((jd_old - (int) s->launch_time) < 3){
	       		split_test_normal(buff,j,buff_test,&tlines,buff_normal,&nlines);
	       		if(debug>0) printf("tlines=%d nlines=%d\n",tlines,nlines);
	       		n_prof = get_raw_data_test(&test,buff_test,tlines,0);
	       		if(n_prof>=0) write_to_disk_error=write_to_disk_test(out_dir,s,&test);
	       		if(write_to_disk_error==1) printf("write profile 0 failed!");
                n_prof = get_raw_data (&y,buff_normal,nlines,1);
//	       		if(n_prof>=0) write_to_disk_error=write_to_disk(out_dir,s,&y);
	       }
           else{
                n_prof = get_raw_data (&y,buff,j,1);
//		        if(n_prof>=0) write_to_disk_error=write_to_disk (out_dir,s,&y);
	       }

	       if(profile_no_prev >= 250 && n_prof>=0) //LL: prof # > 256
		   {
		   		if(n_prof<250)
		   			n_prof = n_prof+256;
		   }

		   if(debug>0) printf("profile_no is %d, n_prof=%d, profSeqCounter=%d, ",y.profile_no,n_prof,profSeqCounter);
		   if(profSeqCounter>1)
		        y.profile_no=correct_profilenumber(jd_firstline_prev_old,jd_firstline_current_old,profSeqCounter,profile_no_prev,s->start_of_cycles,s->cycle_time,n_prof,has_prof0);

	       //sprintf (buff[0],"%s",buff[j]); j = 0;
		   strcpy(buff[0],onelinehex);
     	   j = 0;times=0;

	       profile_no_current=y.profile_no;
		   if(debug>0) printf("profile_no finally is %d\n",y.profile_no);

    	   if(n_prof>=0 && np>0) write_to_disk (out_dir,s,&y);

     	   jd_firstline_prev_old=jd_firstline_prev_new;
     	   jd_firstline_current_old=jd_firstline_current_new;
     	   profile_no_prev=profile_no_current;
//        }
     }
     jd_old = jd;
     if (++j >= max_scans)
     {
        if (trace_file != NULL)
           fprintf (trace_file," Maximum scans for profile exceeded\n");
        if (error_file != NULL)
           fprintf (error_file," Maximum scans for profile exceeded\n");
           exit (1);
     }
  } // while (fgets(buff[j],buff_size,input_file) != NULL)


if(debug>0) printf("before added block, j=%d\n",j);

//add this part to check last profile's date against launch time
   //usually float launched within delay of first down time since start time, if not,then do the following
   //if to process only last profile,and there are dates which are less than launch time,print warning

     if (np==0){
//     if ((s->launch_time - s->start_time > s->delay_first_down_time) && np==0){
     for(kk=0;kk<j;kk++){
       if(sscanf(buff[kk],"%d %d %d %d %d %d %d",&i,&iy,&imo,&ida,&ih,&im,&is) != 7) continue;
       if (dfr97(iy,imo,ida,ih,im,0,&ndoy) < s->launch_time-s->up_time){
         if (trace_file != NULL)
          fprintf (trace_file," GET_typeO7_DATA: WARNING, data before launch time\n");
         if (error_file != NULL)
          fprintf (error_file," GET_typeO7_DATA: WARNING, data before launch time\n");
	 continue;
       }else{
         find=1;//find data whose data is larger than launch time-up time
	 break;
       }
     }//end of for(int kk=0) loop for checking last profile's date against launch time
     if(find==0) j=0;//if all data are before launch time-up time, then set j=0, no data to process
     if(find==1) {//if there are some data whose date is larger than launch time-up time,then process data
       //reset buff to begin from kk, skip data before kk since their date are less than launch time-up time
       sprintf (buff[0],"%s",buff[kk]);
       j=j-kk;//reset buff length j
     }
   }//end of if ((s->launch_time - s->start_time > s->delay_first_down_time) && np==0)

  if(debug>0) printf("after added block, j=%d\n",j);

  if (j > 0) // are there any data lines to save to a file? (j is n_scans in get_raw_data)
  {  if(debug>0) printf("jd_old=%d,start_time=%d,jd_old-start_time=%d\n",jd_old,(int)s->start_time,jd_old-(int)s->start_time);
      profSeqCounter++;

      if ((jd_old - (int) s->launch_time) < 3)
      {
 		 split_test_normal(buff,j,buff_test,&tlines,buff_normal,&nlines);
 		 n_prof = get_raw_data_test(&test,buff_test,tlines,0);
 		 if(n_prof>=0) write_to_disk_error=write_to_disk_test(out_dir,s,&test);
          n_prof = get_raw_data (&y,buff_normal,nlines,1);
 //	     if(n_prof>=0) write_to_disk_error=write_to_disk(out_dir,s,&y);
 //        printf("HERE n_prof=%d,jd_old=%d,start_time=%d\n",n_prof,jd_old,(int) s->start_time);
      }
      else{
 			n_prof = get_raw_data (&y,buff,j,1);
 //			if(n_prof>=0) write_to_disk_error=write_to_disk (out_dir,s,&y);
      }

	  if(profile_no_prev >= 250 && n_prof>=0) //LL: prof # > 256
	  {
	 	 if(n_prof<250)
	 		 n_prof = n_prof+256;
	  }

      if(profSeqCounter>1)
 	   	 y.profile_no=correct_profilenumber(jd_firstline_prev_old,jd_firstline_current_old,profSeqCounter,profile_no_prev,s->start_of_cycles,s->cycle_time,n_prof,has_prof0);

 	 profile_no_current=y.profile_no;
 //	 printf("profSeqCounter=%d,profile_no_current=%d\n",profSeqCounter,profile_no_current);

 	 if (y.profile_no >= 0 && n_prof>=0){write_to_disk_error=write_to_disk (out_dir,s,&y);}
 	 else{write_to_disk_error=1;}
     //printf("profile_no finally is %d\n",y.profile_no);
  }
  else
  {
    write_to_disk_error =1; // no data to store -> something may be wrong
  } // if (j > 0)
  free (buff);
  if (trace_file != NULL)
      fprintf (trace_file," GET_TYPEO7_DATA completed successfully\n");
  return (write_to_disk_error);

}


int get_meta_file_name (char *str_name)
{
// unsigned char *ptr;
 char *ptr;

 if ((ptr = strstr(str_name,".hex"))!= NULL)
  sprintf (ptr,".meta");
 else if ((ptr = strstr(str_name,".HEX"))!= NULL)
  sprintf (ptr,".META");
 else
  strcat(str_name,".meta");
 return 0;
}

int read_type_O7_meta_file (char *meta_name, struct type_O7_meta_struct *meta_struct)
{
 int lyear,lmonth,lday,lhour,lmin,ndoy;
 int i,j,syear,smonth,sday,shour,smin,x;
// unsigned char *ptr;
 float x1;//added
 char *ptr;
// char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=","OSLOPE=","OOFF=","VSLOPE=","VOFF=","VACSLOPE=","VACOFF=","\0"};

// for (i=0; i<(sizeof(struct aoml_O7_cals)/sizeof(double)); i++)
//  meta_struct->cals.c[i] = 0.0;
 meta_struct->cycle_time = (float) 0.0;
 meta_struct->start_time = (float) 0.0;
 //added
 meta_struct->start_of_cycles = (float) 0.0;
 meta_struct->delay_first_down_time=(float) 0.0;
 meta_struct->up_time=(float) 0.0;

 if (trace_file != NULL)
  fprintf(trace_file,"Opening meta file %s...\n",meta_name);
 if ((meta_file = fopen(meta_name,"r")) == NULL)
 {
  if (error_file != NULL)
   fprintf (error_file," ERROR opening meta file %s\n",meta_name);
  if (trace_file != NULL)
   fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
  return 8;
 }
 else
 if (trace_file != NULL)
 fprintf (trace_file," Opened meta-file succesfully\n");

 while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
 {
  if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
  {
   if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) != 1)
    meta_struct->AOML_number = 0;

   continue;
  }
  if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) != 1)
     meta_struct->WMO_id = 0;
   if(debug>0) printf("WMO_ID=%d\n",meta_struct->WMO_id);
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) != 1)
    meta_struct->ARGOS_id = 0;
    if(debug>0) printf("ARGOS_ID=%d\n",meta_struct->ARGOS_id);
   continue;
  }
  if (strstr(strupr(buffer),"TRANSMISSION REPETITION RATE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %f",&meta_struct->ARGOS_rep_rate) != 1)
    meta_struct->ARGOS_rep_rate = 0;
    if(debug>0) printf("ARGOS_REP_RATE=%f\n",meta_struct->ARGOS_rep_rate);
   continue;
  }

  if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) != 1)
     x = 0;
// CS !!!!!! the above has to be changed quit if no value !!!!
//   meta_struct->start_time += (float) (x/24.0);
     meta_struct->delay_first_down_time=(float)(x/24.0);
   continue;
  }

  if (strstr(strupr(buffer),"WMO INSTRUMENT TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_instrument) != 1)
    meta_struct->wmo_instrument = 0;
    if(debug>0) printf("WMO_INSTRUMENT=%d\n",meta_struct->wmo_instrument);
   continue;
  }
  if (strstr(strupr(buffer),"WMO RECORDER TYPE") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %d",&meta_struct->wmo_recorder) != 1)
    meta_struct->wmo_recorder = 0;
    if(debug>0) printf("wmo_recorder=%d\n",meta_struct->wmo_recorder);
   continue;
  }
  if (strstr(strupr(buffer),"PROGRAM NUMBER") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->exp_no) != 1)
    meta_struct->exp_no = 0;
    if(debug>0) printf("exp_no=%d\n",meta_struct->exp_no);
   continue;
  }
  if (strstr(strupr(buffer),"DOWN TIME [DAYS]") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
    x1 = 0;
   if(debug>0) printf("down time=%f\n",x1);
   meta_struct->cycle_time += (float)x1;
   //added
   meta_struct->start_of_cycles += (float)x1;
   continue;
  }
  //added,changed
  if (strstr(strupr(buffer),"FIRST DOWN TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %f",&x1) != 1)
    x1 = 0;
    if(debug>0) printf("first down time=%f\n",x1);
   meta_struct->start_of_cycles += (float) (x1/24.0);
   continue;
  }

  if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
  {
 if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
 {
  sprintf (meta_struct->float_type,"\0");
     continue;
 }
 if ((ptr = strstr(&buffer[strlen("FLOAT TYPE")],meta_struct->float_type)) != NULL)
  sprintf (meta_struct->float_type,"%s",ptr);
    if ((ptr = strstr(meta_struct->float_type,"\n")) != NULL)
    sprintf (ptr,"\0");
   continue;
  }
  if (strstr(strupr(buffer),"PI     ") != NULL)
  {
   if (sscanf(buffer,"%*s %s",meta_struct->PI) == 0)
   {
    sprintf (meta_struct->PI,"\0");
    if(debug>0) printf("PI=%s\n",meta_struct->PI);
     continue;
   }
   if ((ptr = strstr(&buffer[3],meta_struct->PI)) != NULL)
    sprintf (meta_struct->PI,"%s",ptr);
   if ((ptr = strstr(meta_struct->PI,"\n")) != NULL)
    sprintf (ptr,"\0");
   continue;
  }

  if ((strstr(strupr(buffer),"LAUNCH TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL)) {
      sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",
             &lday,&lmonth,&lyear,&lhour,&lmin);

      meta_struct->launch_time = (float) (dfr97(lyear,lmonth,lday,lhour,lmin,0,&ndoy));
      continue;
  }

  if (strstr(strupr(buffer),"UP TIME") != NULL)
  {
   if (sscanf(buffer,"%*s %*s %*s %f",&x1) != 1)
    x1 = 0;
   meta_struct->cycle_time += (float) (x1/24.0);
   meta_struct->up_time = (float) (x1/24.0);
   continue;
  }

  if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL))
  {
   if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
   {
    if(debug>0) printf("year=%d month=%d day=%d hour=%d min=%d,dfr97=%f\n",syear,smonth,sday,shour,smin,
    (float) (dfr97(syear,smonth,sday,shour,smin,0,&x)));

    meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x));
    if(debug>0) printf("start_time=%f\n",meta_struct->start_time);

    if (meta_struct->start_time <= (float) 0.0)
    {
     if (error_file != NULL)
      fprintf (error_file,"Error reading float start time\n");
     if (error_file != NULL)
      fprintf (trace_file,"Error reading float start time\n");
     printf ("Error reading float start time\n");
     return (1);
    }
   }
   else
    meta_struct->start_time = (float) 0.0;

   //added
   meta_struct->start_of_cycles += meta_struct->start_time;

   continue;
  }
}
 if(debug>0) printf("launchtime-uptime=%f\n",meta_struct->launch_time-meta_struct->up_time);

 return 0;
}

int main (int argc, char *argv[])
{
 char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
          *ptr_path_log, *ptr_path_list,
          pathname_input_filename[161]="",
          pathname_trace[161]="", pathname_error[161]="";

 char     input_filename[80], output_directory[256], msg[11]="", title[257]="";
 enum     {LAST=0,ALL} get_which_profile;
 struct   type_O7_meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
 unsigned long t;
 int      ret_val = 0;

// Program Name
// ------------
 printf("Entering Program: %s\n", PRGNAME);
 strcpy(title, "Program generates temperature, salinity, pressure and dissolved oxygen counts\n");
 strcat(title, "for APEX_TSO7 Floats");
 printf("%s\n", title);


// Input Paramenters
// -----------------
 if ( argc < 7 )
 {
  printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
          PRGNAME);
  printf("\n");
  printf("hex_infile:    hexadecimal input file name\n");
  printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
  printf("cnt_outdir:    output directory for generated count data file(s)\n");
  printf("trace_outile:  output filename for generated trace file\n");
  printf("error_outfile: output filename for generated error file\n");
  printf("list_outfile:  output filename for generated list file\n");
  return 9;
 }

// Input Hex File name
// -------------------
 printf ("%s: Input Hex File Name - %s\n", PRGNAME, argv[1]);
 sprintf (input_filename,"%s",argv[1]);

// Number of profiles to process: 0, get last profile or 1, get all profiles
// -------------------------------------------------------------------------
 if (atoi(argv[2]) >= 1)
  get_which_profile = ALL;
 else
  get_which_profile = LAST;

// Output directory path for count file(s)
// ---------------------------------------
 if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
 {
  printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 9;
  return ret_val;
 }

 sprintf (output_directory,"%s/",argv[3]);
 printf ("%s: Output Directory - %s\n", PRGNAME, output_directory);

// TRACE File
// ----------
 if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
 {
  printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
  t = time(NULL);
  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  ret_val = 9;
  return ret_val;
 }

 sprintf (buffer,"%s",argv[4]);
 printf ("%s: Opening trace file %s\n", PRGNAME, buffer);

 if ((trace_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
   t = time(NULL);
   printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
   ret_val = 9;
   return ret_val;
 }

 printf("%s: Trace file opened succesfully\n", PRGNAME);
 if (trace_file != NULL)
 {
  t = time(NULL);
  fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&t));
 }

// ERROR File
// ----------
 sprintf (buffer,"%s",argv[5]);
 printf ("%s: Opening error file %s\n", PRGNAME, buffer);

 if ((error_file = fopen(buffer,"a+")) == NULL)
 {
   printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
   if (trace_file != NULL)
     fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   printf ("%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 9;
   return ret_val;
 }

 if (trace_file != NULL)
   fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

 if (error_file != NULL)
 {
   t = time(NULL);
   fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&t));
 }

  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   if (trace_file != NULL)
     fprintf(trace_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   if (error_file != NULL)
     fprintf(error_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   ret_val = 9;
   return ret_val;
  }

// LIST File
// ---------
  sprintf (buffer,"%s",argv[6]);
  if (trace_file != NULL)
     fprintf (trace_file,"%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    t = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    if (error_file != NULL)
       fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val =9;
    return ret_val;
  }
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);


// Open Input Hex File
// -------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);

    if (trace_file != NULL)                  // Close Log files
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    t = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
    ret_val = 9;
    return ret_val;
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
     if (trace_file != NULL)
       fprintf (trace_file,"ERROR (%s): Input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     if (error_file != NULL)
       fprintf (error_file,"ERROR (%s): input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     ret_val = 8;
  }
  else
  {
     if (trace_file != NULL)
       fprintf (trace_file,"%s: Input file %s successfully opened\n",
                PRGNAME, pathname_input_filename);

// META File
// ---------
     if (get_meta_file_name (input_filename) != 0)
     {                                             // Failure: did not get meta filename
      if (error_file != NULL)
        fprintf (error_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,
                 "ERROR (%s): Finding meta file name - returned name %s\n",
                 PRGNAME, input_filename);
        ret_val = 8;
     }
     else
     {
                                              // Success: got meta filename
        if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
        {
           if (trace_file != NULL)
             fprintf (trace_file,
                      "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                      PRGNAME);
           if (error_file != NULL)
                fprintf (error_file,
                         "ERROR (%s): METAFILES_DIR <-- no such environment variable\n",
                          PRGNAME);

           if (trace_file != NULL)                  // Close Log files
             fclose (trace_file);
           if (error_file != NULL)
             fclose (error_file);

           t = time(NULL);
           if (trace_file != NULL)
             fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           if (error_file != NULL)
             fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&t));
           ret_val = 9;
           return(ret_val);
        } // endif  ptr_path_meta

        sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

        if (read_type_O7_meta_file (pathname_input_filename, &s) != 0)
        {                                         // Failure: couldn't read meta_file
           if (trace_file != NULL)
             fprintf (trace_file, "ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
           if (error_file != NULL)
             fprintf (error_file,"ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
           ret_val = 6;
        }
        else                                      // Success: read meta_file
        {
           if (trace_file != NULL)
             fprintf (trace_file,"%s: Meta file %s successfully read\n",
                     PRGNAME, pathname_input_filename);

// Make profile(s) selection
// -------------------------
           if (get_which_profile == ALL)
           {
              if (get_type_O7_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           }
           else
           {
              if (get_type_O7_data(get_which_profile,&s,output_directory) !=0 )
              ret_val = 1;
           } // endif: get_which_profile

        } // endif: read_type_O7_meta_file
     } // endif: get_meta_file_name
  } //endif: input_file

  printf("HEX filename=%s\n",argv[1]);
  printf("frame1 missing problem counts=%d\n",frame1_problem_count);
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }


// Write finish message to trace & error files
// -------------------------------------------
  t = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&t));
  if (trace_file != NULL) fprintf (trace_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));
  if (error_file != NULL) fprintf (error_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&t));

// Close all files
// ---------------

 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);


 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
