#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>


#define max_scans         2000    //Maximum scans for each profile
#define buff_size          256    //Maximum length of each scan read from *.str
#define max_positions      100    //Maximum number of positions per profile
#define min_frame_no         0    //Minimun frame number
#define frames_per_message 256    //Number of frames per message,it is the maximum allowed number

#define good_bad_position  43    //Location of status byte in *.str records
#define start_of_hex_data  63    //Start position of data in *.str records

#define GOOD 1
#define BAD 4
#define NEVER_RECEIVED 0
#define OFF 0
#define ON 1
#define FALSE 0
#define TRUE 1

#define buffer_size 256
#define candidate_size 500 

// calibration coefficient for p, t, psal, volt, curr, etc.
struct aoml_tso_cals {
  double TSLOPE;
  double TOFF;
  double PSLOPE;
  double POFF;
  double SSLOPE;
  double SOFF;
  double SOC;
  double FOFFSET;
  double TCOR;
  double PCOR;
  double VSLOPE;
  double VOFF;
  double VACSLOPE;
  double VACOFF;
};

union u_cals { 
  struct aoml_tso_cals s;
  double c[sizeof(struct aoml_tso_cals)/sizeof(double)];
};


// meta data.
struct meta_struct { 
  int AOML_number;
  int WMO_id;
  int ARGOS_id;
  int wmo_instrument;
  int wmo_recorder;
  int exp_no;
  unsigned char float_type[40];
  unsigned char PI[60];
  float launch_time;
  float ARGOS_rep_rate;
  float cycle_time;
  float start_time;
  float delay_first_down_time;
  float up_time;
  float start_of_cycles;  // <== start_time + delay_of_start + down_time
  union u_cals cals;
};


/* Final structure containig T/P data and positions */
struct test_record   { 
  int float_id;
  int profile_no;
  int jd;
  int status[frames_per_message];
  int msg_blk_no;
  time_t first_blk_time;
  int first_blk_time_via_st_of_trans;
  int controller_sn;
  int time_since_startup;	//Test stuff follows
  int status_word;
  int current_pressure;
  int vacuum;
  int current_bladder_pressure;
  int battery_volts;
  int up_time_intervals;
  int down_time_intervals;
  int park_pressure;
  int park_piston_position;
  int depth_correction;
  int ok_vacuum;
  int ascend_time_intervals;
  int target_bladder_pressure;
  int profile_pressure;
  int profile_piston_position;
  int deep_cycle_count;
  int software_rev_month;
  int software_rev_day;
  int software_rev_year;
  //from test message #2
  int piston_count_full_extention;
  int piston_count_full_retraction;
  int initial_buoyancy_nudge;
  int deep_profile_descent_period;
  int park_descent_period;
  int mission_prelude_period;
  int argos_repetition_period;
  int sbe_serial_number;
  int sbe_firmware_revision;
  int optode_serial_number;
  time_t current_epoch;
  int mins_past_midnight;
  int debug_verbosity_for_log;

  float  latitude[max_positions];
  float  longitude[max_positions];
  char   date_time[max_positions][32];
  int    n_xmits[max_positions];
  char   sat_name[max_positions];
  char   loc_class[max_positions];
  double position_jd[max_positions];

  char   hex_test_print[100][100];
  char   hex_test_time[100][100];

};



/* Final structure containig T/P data and positions */
struct output_record { 
  int float_id;
  int profile_no;
  int jd;
  int status[frames_per_message];
  int msg_blk_no;
  time_t first_blk_time;
  int first_blk_time_via_st_of_trans;
  int controller_sn;
  int time_since_startup;
  int profile_length;	//data stuff follows
  int termination_flag;//status word
  int surface_pressure;
  int internal_vacuum;
  int air_bladder_pressure;
  int surface_piston_position;
  int park_piston_position;
  int profile_piston_position;
  int ctd_status;
  int air_pump_on_time;
  int park_battery_volts;
  int park_battery_current;
  int battery_volts_sbe_pump_on;
  int battery_current_sbe_pump_on;
  int battery_volts;
  int battery_current;
  int battery_volts_air_pump_on;
  int battery_current_air_pump_on;
  time_t downtime_epoch;
  int time_initiat_epoch;
  int nadj;

  int num_samples;
  int mean_temperature;
  int mean_pressure;
  int sd_temperature;//standard deviation of temperature
  int sd_pressure;
  int min_temperature;//minimum temperature
  int pressure_when_min_temperature;//pressure associated with minimum temperature
  int max_temperature;
  int pressure_when_max_temperature;//pressure associated with maximum temperature
  int min_pressure;
  int max_pressure;

  float  latitude[max_positions];
  float  longitude[max_positions];
  char   date_time[max_positions][32];
  int    n_xmits[max_positions];
  char   sat_name[max_positions];
  char   loc_class[max_positions];
  double position_jd[max_positions];
  int    press_sal_temp[15*frames_per_message][6];

  char   hex_data_print[100][100];
  char   hex_data_time[100][100];
                   
};

/* Raw data structure used in decoding Temperature/Pressure/Oxygen (T/P/O)
   float data.  Part of RAW_DATA_RECORD described below */

struct test_fmt { 
  // test message 1
  unsigned char msg_blk_no;
  unsigned char software_rev_month;
  unsigned char software_rev_day;
  unsigned char software_rev_year;
  unsigned char controller_sn[2]; //serial number
  unsigned char time_since_startup[2];
  unsigned char status_word[2];
  unsigned char current_pressure[2];
  unsigned char vacuum;
  unsigned char current_bladder_pressure;
  unsigned char battery_volts;
  unsigned char up_time_intervals;
  unsigned char down_time_intervals[2];
  unsigned char park_pressure[2];
  unsigned char park_piston_position;
  unsigned char depth_correction;
  unsigned char ok_vacuum;
  unsigned char ascend_time_intervals;
  unsigned char target_bladder_pressure;
  unsigned char profile_pressure[2];
  unsigned char profile_piston_position;
  unsigned char deep_cycle_count;
  //test message 2
  unsigned char piston_count_full_extention;
  unsigned char piston_count_full_retraction;
  unsigned char initial_buoyancy_nudge;
  unsigned char compensator_hyper_retraction;
  unsigned char deep_profile_descent_period;
  unsigned char park_descent_period;
  unsigned char mission_prelude_period;
  unsigned char argos_repetition_period;
  unsigned char sbe_serial_number[2];
  unsigned char sbe_firmware_revision[2];
  unsigned char optode_serial_number[2];
  unsigned char current_epoch[4];
  unsigned char mins_past_midnight[2];
  unsigned char debug_verbosity_for_log[2];
};

struct data_fmt { 
  // message 1
  unsigned char msg_blk_no;
  unsigned char controller_sn[2];
  unsigned char profile_no;
  unsigned char profile_length;
  unsigned char termination_flag[2];//status word
  unsigned char surface_pressure[2];
  unsigned char internal_vacuum;
  unsigned char air_bladder_pressure;
  unsigned char surface_piston_position;
  unsigned char park_piston_position;
  unsigned char profile_piston_position;
  unsigned char ctd_status[2];
  unsigned char air_pump_on_time[2];
  unsigned char park_battery_volts;
  unsigned char park_battery_current;
  unsigned char battery_volts_sbe_pump_on;
  unsigned char battery_current_sbe_pump_on;
  unsigned char battery_volts;
  unsigned char battery_current;
  unsigned char battery_volts_air_pump_on;
  unsigned char battery_current_air_pump_on;
  unsigned char pulse_air_pump; 
  unsigned char volume_air_pump[2]; 

  // message 2 
  unsigned char current_epoch[4];
  unsigned char time_of_downtime_expire[2];
  unsigned char nadj;
  unsigned char num_samples[2];
  unsigned char mean_temperature[2];
  unsigned char mean_pressure[2];
  unsigned char sd_temperature[2];//standard deviation of temperature
  unsigned char sd_pressure[2];
  unsigned char min_temperature[2];//minimum temperature
  unsigned char pressure_when_min_temperature[2];//pressure associated with minimum temperature
  unsigned char max_temperature[2];
  unsigned char pressure_when_max_temperature[2];//pressure associated with maximum temperature
  unsigned char min_pressure[2];
  unsigned char max_pressure[2];

  //new guys
  unsigned char surf_press_compl_surf_optode_data[2];
  unsigned char surf_optode_bphase_n_temp_data[3];


  unsigned char T_S_P[15*frames_per_message][9];
  //p(2bytes),t(2bytes),s(2bytes),bt1(1 byte),bt2(1 byte),bt3(1 byte)
  //note:bt1,bt2,bt3 are dummy values for BPhase and T2, totally 9 bytes
};

/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size { 
  unsigned char data_bytes[frames_per_message][29];
  struct data_fmt d_fmt;
  struct test_fmt t_fmt;
};

/* Intermediate structure containing T/P raw data and positions */

struct raw_data_record { 
  int jday;
  double jd[max_positions];
  float  latitude[max_positions];
  float  longitude[max_positions];
  char   date_time[max_positions][32];
  int    n_xmits[max_positions];
  char   sat_name[max_positions];
  char   loc_class[max_positions];
  union  data_size data;
};

extern int frame1_problem_count;
extern int total_cycles_count;

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file;

extern char buffer[1024];   
extern char PRGNAME[10];

long st_trans_blk_time;
