/*  Revision Date: 04-Jun-14   CA - The Program will exit with 6 if the old file has fill value in
				    the position block
  		   22-Jul-11   ER - Corrected bug converting dates to milliseconds since 1/1/70
                   17-jun-11   ER - Complete redevelopment of program.
                   03-Dec-08   CC - fix stderror produced when the new 'cnt' file and the
                                    one in 'done' differs in length of the position records.
                   01-Jul-03   XX - Initial release
   
   Check the current (old) cnt data file and new cnt data file 
   data file "file_new.cnt" which is generated from ARGO_MANUAL.SH process, the purpose is to 
   check whether there are any difference in the position part of these two files, if yes
   it will exit with different status code,printed on the screen(stdout).
   
   Usage: java DiffCnt old_cnt_filename new_cnt_filename log_filename result_table_filename
   
   Command line arguments:
   old_cnt_filename      -----current(old) cnt filename
   new_cnt_filename      -----cnt file generated from argo_manual.sh
   log_filename          -----contain detailed information about difference between two cnt files
   result_table_filename -----contain two columns, new_cnt_filename and exit status 6-bit code
 
   exit code:
   1--cnt files different, without other problems from below,including 3,4,5,6
   2--float could be a surface drifter (more than 1.5 days between first and last transmissions) 
   3--program usage error
   4--Date Time parsing error and other I/O errors
   5--Screw-up with profile number
*/

import java.io.IOException;
import java.util.*;
import java.io.*;
import java.text.*;
import java.math.*;

public class DiffCnt{
 
  static String old_cnt_filename; //old(current ) cnt filename from real time processing
  static String new_cnt_filename; //new(original) cnt filename from argo_manual.sh processing
  static String log_filename;     //log filename
  static String result_table_filename;//exit status 6-bit code result table filename, it will keep appending.
   
  PositionEntry first_old_position;
  PositionEntry last_old_position;
  PositionEntry first_new_position;
  PositionEntry last_new_position;

  FileWriter  fw_log;//log file FileWriter, can append to log file
  PrintWriter pw_log;//log file PrintWriter
  FileWriter  fw_result;//result table file FileWriter, can append to file
  PrintWriter pw_result;//result table file PrintWriter
   
  static int exitCode=1;//by default,finally System.exit code is 1

  public void openFiles() {

    try {
      fw_log=new FileWriter(log_filename, true);
      pw_log=new PrintWriter(fw_log);
      fw_result=new FileWriter(result_table_filename, true);
      pw_result=new PrintWriter(fw_result);
    }
    catch(Exception ioe) {
      ioe.printStackTrace();
      System.exit(4);    	
    }   	
  }   


  public void closeFiles() {

    try {
      pw_log.close();
      pw_result.close();
    }
    catch(Exception ioe) {
      ioe.printStackTrace();
      System.exit(4);    	
    }   	
  }   


  public void getFirstAndLastPositions(String old_cnt_filename, String new_cnt_filename){
   	
    String line;
    int i;
    boolean inPositionBlock;  // is the program looping thru the position block now?

    // Initializations
    inPositionBlock=false;
    first_old_position = null;
    last_old_position = null;
    first_new_position = null;
    last_new_position = null;
   	
    try{ 
      // looping throu old cnt file
      FileReader fr_old=new FileReader(old_cnt_filename);
      BufferedReader in_old= new BufferedReader(fr_old);
         
      while ((line=in_old.readLine()) != null) {

        if (inPositionBlock) {
          if (inPositionBlock && line.startsWith("=====")) {
            inPositionBlock=false;
          }
          else {
            PositionEntry entry=new PositionEntry(line);
            if ( first_old_position == null ) {
              first_old_position = entry;
            }
            last_old_position = entry;
          }
        }
        else if( line.indexOf("LATITUDE")> 0 && line.indexOf("LONGITUDE YEAR")>(line.indexOf("LATITUDE")+8) )
        {
          inPositionBlock=true;
        }
      }
         
      //get all data from new cnt file
      FileReader fr_new=new FileReader(new_cnt_filename);
      BufferedReader in_new= new BufferedReader(fr_new);

      inPositionBlock=false;
      while ((line= in_new.readLine()) != null){

        if (inPositionBlock) {
          if (inPositionBlock && line.startsWith("=====")) {
            inPositionBlock=false;
          }
          else {
            PositionEntry entry=new PositionEntry(line);
            if ( first_new_position == null ) {
              first_new_position = entry;
            }
            last_new_position = entry;
          }
        }
        else if( line.indexOf("LATITUDE")> 0 && line.indexOf("LONGITUDE YEAR")>(line.indexOf("LATITUDE")+8) )
        {
          inPositionBlock=true;
        }
      }   


      pw_log.println("old filename:  "+old_cnt_filename);
      pw_log.println("new filename:  "+new_cnt_filename);          
      if (first_old_position != null) {
        pw_log.println( "first old position data: " + first_old_position.getWholeLine());
      }
      if (last_old_position != null) {
        pw_log.println( "last old position data:  " + last_old_position.getWholeLine());
      }
      if (first_new_position != null) {
        pw_log.println( "first new position data: " + first_new_position.getWholeLine());
      }
      if (last_new_position != null) {
        pw_log.println( "last new position data:  " + last_new_position.getWholeLine());
      }

      in_old.close();
      fr_old.close();
      in_new.close();
      fr_new.close();
    }
    catch(IOException ioe) {
      ioe.printStackTrace();
      System.exit(4);
    }	
  }

  public int checkDifference(){
    int i=0,j=0;

    String ymd_old, hms_old; // year, month, day, hour, minute, second for old cnt file
    String ymd_new, hms_new; // year, month, day, hour, minute, second for new cnt file
    String ymd1, hms1, ymd2, hms2;
    String old_str="", new_str=""; //one line of string from old and new cnt file
    String Lat_old,Lon_old,YMD_old,HMS_old,Transmits_old,SatType_old,LocationClass_old;
    String Lat_new,Lon_new,YMD_new,HMS_new,Transmits_new,SatType_new,LocationClass_new;   	 
    String str;
     
    try{

      File cntfile=new File(new_cnt_filename);  // will be used later for logging purposes
      // Case 1) Old Msg file Does not have a position instead a fill value.
      //+99.999   +999.999 9999/99/99 99:99:99   000   0     0
      // Exit Code : 6
      System.out.println(" first_old_position.getYMD and first_old_position.getHMS = "+first_old_position.getYMD()+" "+first_old_position.getHMS());
      if(first_old_position.getYMD().equals("9999/99/99") || first_old_position.getHMS().equals("99:99"))
      {
        
        pw_log.println( "Old File has a Fill Value for YMD and Time         (" + 
                        first_old_position.getYMD()+" "+first_old_position.getHMS()+")");
        pw_log.println( "program exit code=6" );
	pw_result.println(cntfile.getName()+"       6" );
	System.out.println("program exit code=6");
	return 6;


      }


      // Case 2) Possible profile number screw-up.
      //         (There is more than 1 day of difference between first transmissions 
      //          of old and new files)
      //         Exit Code: 5
      if (((timeDifference(first_old_position.getYMD(), first_old_position.getHMS(),
                           first_new_position.getYMD(), first_new_position.getHMS(),
                           cntfile.getName() ))/1000.0/3600.0/24.0)>1.0){
        pw_log.println( "More than 1 day between first position in old file (" + 
                        first_old_position.getYMD()+" "+first_old_position.getHMS()+") and first position in new file ("+
                        first_new_position.getYMD()+" "+first_new_position.getHMS() + ")" );
        pw_log.println( "program exit code=5" );
        pw_result.println(cntfile.getName()+"       5" );   	        
        System.out.println("program exit code=5");   	        
        return 5;       	
      }
   	 
      // Case 3) Possible surface drifter 
      //         (There is more than 1.5 days of difference between first and last transmissions of new file)
      //          Exit code: 2 
      if (((timeDifference(first_new_position.getYMD(), first_new_position.getHMS(),
                           last_new_position.getYMD(),  last_new_position.getHMS(),
                           cntfile.getName() ))/1000.0/3600.0/24.0)>1.5){
        pw_log.println( "Possible drifter: more than 1.5 days between first position in new file (" + 
                        first_new_position.getYMD()+" "+first_new_position.getHMS()+") and last position in new file ("+
                        last_new_position.getYMD()+" "+last_new_position.getHMS() + ")" );
        pw_log.println( "program exit code=2" );
        pw_result.println(cntfile.getName()+"       2" );   	        
        System.out.println("program exit code=2");   	        
        return 2;       	
      }
      pw_log.println( "program exit code=1" );
      System.out.println("program exit code=1");
      pw_result.println(cntfile.getName()+"       1");
      return 1;
      
    }
    catch (Exception e) {
      e.printStackTrace();
      return 4;
    }
  }//end of function checkDifference
   

  // Returns time difference between ymd_old+hms_old with ymd_new+hms_new
  // in milliseconds unit
  public double timeDifference(String ymd_old,String hms_old,String ymd_new,String hms_new, String fileName ){

    //compare date difference
    double timeDiffMilliSeconds=0.0;
    SimpleDateFormat ymdhms_formatter=new SimpleDateFormat("yyyy/MM/ddHH:mm:ss"); 
    try
    {
      Date ymdhms_date_old=ymdhms_formatter.parse(ymd_old+hms_old);
      long milliSeconds_old=ymdhms_date_old.getTime();	
      Date ymdhms_date_new=ymdhms_formatter.parse(ymd_new+hms_new);
      long milliSeconds_new=ymdhms_date_new.getTime();	
      timeDiffMilliSeconds=(double)(Math.abs(milliSeconds_old-milliSeconds_new));
    }
    catch(ParseException e){

      pw_log.println( "Date time unparseable. " );
      pw_log.println( "program exit code=4" );
      pw_result.println(fileName+"       4" );   	        
      System.out.println("Date time unparseable.");
      System.out.println("program exit code=4");   	        
      pw_log.close();
      pw_result.close();
      System.exit(4);
    }   	 
   	 
    return timeDiffMilliSeconds;
  } 


  public static void main(String[] args) {
   	
    if(args.length!=4){
      System.out.println("Usage:java DiffCnt old_cnt_filename new_cnt_filename log_filename result_table_filename");
      System.exit(3);
    }
    old_cnt_filename=args[0];
    new_cnt_filename=args[1];
    log_filename=args[2];
    result_table_filename=args[3];
   	      	
    DiffCnt dm=new DiffCnt();

    dm.openFiles();
    dm.getFirstAndLastPositions(old_cnt_filename, new_cnt_filename);
    int result=dm.checkDifference();
    dm.closeFiles();

    System.exit(result); 
  }


}
