/*********************************************************************
  Revision Date: 28-May-14 (CA - prv2hex now gets the correct CRC for the hex messages whose count > 1
                 01-Apr-11 (ER - Discard a prv entry when its hex string length > 64.  This
                                 makes the 128-character rule unnecessary. 
                 03-Mar-11 (ER - adapted the 128-character rule to floats with
                                 6-digit ids.
                 24-Feb-11 (ER - when a line in the prv file is too long
                                 it gets truncated to 128 characters in the hex file.
                 22-Nov-10 (ER - 'max_entries' decreased to 60000)
                 19-Oct-10 (RS - 'max_entries' increased to 70000)
                 05-Mar-10 (LL - revise for 6 digits float number)
                 09-Nov-09 (LL - not stop processing the next float for compress function)
  				 28-Aug-08 (LL - add compress_index and loop for generating hex data)
  				 13-Jan-06 (XXD - excluding position lines if all transmitted messages associated
                            with a header line are <=8 bytes)
                 01-Mar-05 (DB&XX - skip data from different program number)
                 03-Jun-04 (XX&CS - modified read_prv_file to deal with 31 bytes case)
                 18-Aug-03 (XX&CS - modified sort function and compress function to remove single line case
                            ignore number_of_lines field when doing comparision in compress function)
                 30-Oct-02 (DB - modified READ_PRV_FILE to read data values until all data fields
                                 have been read rather than ending when hex_string length equals
                                 bytes_per_frame.  If hex_string length is less than bytes_per_frame
                                 it is padded with 0's at end)
                 23-Oct-02 (CS - can now properly read hex data with 8 data blocks
                                 for ids >= 32768 (last data block has only 6
                                 characters -> have to add two 0's at the end)
                 20-Aug-01 (DB - modified formatting of LINES to fill leading spaces with 0)
                 20-Aug-01 (DB - only accept hex data that equals n_bytes in length)
                 01-Aug-01 (DB - added new GET_FILE_TIME routine)
                 30-Jul-01 (DB - added test for number of data fields and
                                 correction if short)
                 30-Jul-01 (DB - added hexdigit test to data)
                 09-Jul-01 (JZ - add prv filename to
                                 crossref_noentry_file)
                 07-Jul-01 (RS - correct print format for argos_id)
                 08-Jan-01 (RS - create list for ARGOS ID not found in
                            cross reference file)
                 09-Nov-00 (RS - max_entries exceeded return)
                 10-AUg-00 (RS - rename ARGOLIST_DIR)
                 29-Jun-00 (RS - replace juldat with ymd2julday)
                 20-Jun-00 (RS - change LOG file name)
                 09-Jun-99
  prv2hex
  =======
  Converts prv ARGOS float data to hexadecimal

  Input: PRV file
  Output: HEX file(s)
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>

#define max_entries 60000
#define max_floats 1000
#define buff_size 256

FILE *input_file, *output_file, *cross_ref_file, *trace_file, *error_file,
     *list_file, *crossref_noentry_file, *output_file_none;
char buffer[256],buffer1[256],**data_buffers;

int debug=0;

char PRGNAME[]="prv2hex.c";
long *flt_dates;
int  *seq;
long file_time;
char file_date[30];

char *strupr(char *input)          // changes characters to upper case
{
   int  i;

   for (i=0; input[i] != '\0'; i++)
     input[i] = toupper(input[i]);
   return input;
}

long ymd2julday(int, int, int);

/**********************************************************************************
get substring of a string s,from index begin to end, result is stored in string ss
***********************************************************************************/
void *getsubstr(int begin, int end, char *s, char *ss)
{
  int j;

  for (j=begin; j<=end; j++)
    ss[j-begin]=s[j];

  ss[end-begin+1]='\0';
  return s;				//Microsoft compiler gives warning in no return value
}


/*************************************************************************
  Returns creation date of file since 1/1/97 00:00:00 in seconds
**************************************************************************/
// Requires a filename in the form PRVp..p.yyyy.mm.dd.hh.mm
// p..p = program number; yyyy = year; mm = month; dd = day; hh = hour; mm = minutes
long get_file_time (char *file_name)
{
  struct tm t;
  long base_time;
  int year,mon,day,hr,minute;
  char *ptr,tmp[256];

  sprintf(tmp,"%s",file_name);
  if ((ptr = strstr(strupr(tmp),"PRV")) == NULL)
   return (long) 0;
  if ((ptr = strstr(ptr,".")) == NULL)
   return (long) 0;
  ptr++;
  if (sscanf (ptr,"%4d.%2d.%2d.%2d.%2d",&year,&mon,&day,&hr,&minute) != 5)
   return (long) 0;

  t.tm_isdst = 0;
  t.tm_year  = 97;
  t.tm_mon   = 0;
  t.tm_mday  = 1;
  t.tm_hour  = 0;
  t.tm_min   = 0;
  t.tm_sec  = 0;
  base_time = mktime(&t); // Converts a tm structure to a calendar time

  t.tm_isdst = 0;         //No daylight savings time
  t.tm_year  = year-1900;
  t.tm_mon   = mon-1;     //Month = 0-11
  t.tm_mday  = day;
  t.tm_hour  = hr;
  t.tm_min   = minute;
  t.tm_sec  = 0;
  return (mktime(&t) - base_time);
}


/*************************************************************************
  Check sum routine for each byte of input string
*************************************************************************/
unsigned int Hasard(unsigned int ByteN)
{
  int x = 0;

  if (ByteN == 0)
    return(127);

  x = (ByteN & 1) ? x + 1 : x;
  x = (ByteN & 4) ? x + 1 : x;
  x = (ByteN & 8) ? x + 1 : x;
  x = (ByteN & 16) ? x + 1 : x;
  ByteN = (x & 1) ? (int)floor(ByteN/2.0) + 128 : (int)floor(ByteN/2.0);

  return(ByteN);
}


/*************************************************************************
  Computes checksum for array x
  The first integer x[0] contains the checksum as generated by the float
  This routine computes the checksum for integers x[1] - x[array_size-1]

  Calls: Hasard
**************************************************************************/
unsigned int compute_CRC (unsigned int *x, int array_size)
{
  unsigned int ByteN;
  int i=1;

  ByteN = x[1];

  while (++i < array_size){
 
    ByteN = Hasard(ByteN) ^ x[i];
 
 }

  return(Hasard(ByteN));
}

/*************************************************************************
  Converts hex ascii string y to 8-bit integer array x
  Computes checksum over integer array
  Compares checksum to x[0] which contains checksum generated by float
  Returns 0 if checksums agree, 1 if not

  Calls: compute_CRC
*************************************************************************/
int check_CRC (char *y)
{
  int i,j;
  unsigned int x[50];

  j = 0;

  for (i=0; i<(int)strlen(y); i+=2){
    
    sscanf(&y[i],"%2x",&x[j++]);
 }

  if (x[0] == compute_CRC(x,j)){
    return(0);
}
else

  return(1);
}



/*************************************************************************
  Sorts by date
**************************************************************************/
int sort_by_date (int n)
{
  int  i,j,k,m;
  long l;

  if (n < 2)
    return(0);

  do
  {
    k = 0;
    for (i=0; i<n-1; i++)
    {
      for (j=i+1; j<n; j++)
      {
        if (flt_dates[i] <= flt_dates[j])
          continue;

         m = seq[i];
         seq[i] = seq[j];
         seq[j] = m;

         l = flt_dates[i];
         flt_dates[i] = flt_dates[j];
         flt_dates[j] = l;

         k++;
      }
    }
  } while (k > 0);

  //added new
  //if there is only one line in the middle of two cycle,set seq array to negative values,
  //later skip it.

  for(i=1;i<n-1;i++){
    if(abs(flt_dates[i]-flt_dates[i+1])>100000 && abs(flt_dates[i]-flt_dates[i-1])>100000){
      seq[i]=-10;
    }
  }

  return(0);
}



/*************************************************************************
  Find redundant positions and mark
  If id,date,time,positions the same => keep earliest file date
  If id,date,time the same but positions differ => keep later file date
  ID YEAR MON DAY HR MIN SEC LAT LON GOOD/BAD NNN S C FILEDATE 0000000000...
  ....00000 (more new) 09333 1997 08 23 06 43 25 +00.000 +000.000 0 025 D 0
  069516551 2808FD09FD09FE0A170A3BFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF

  Calls: ymd2julday, sort_by_date
**************************************************************************/
int compress (int n_scans, int float_number)
{
  int debug=0;
  int i=0,j,iyr,imon,ida,ihr,imin,isec;
  long fd,fd1;
  float lat,lat1,lon,lon1;
  //added new
  char satype[max_entries][4],satype_loc[max_entries][10];
  char prv_time_str[max_entries][20],hexdata[max_entries][80];
  char ss0[]="0000000000000000000000000000000000000000000000000000000000000000";
  long prv_time[max_entries];

  int file_date_pos = 53;
  int fd_len = 9;
  int num_of_line_start_pos = 45;
  int num_of_line_len = 3;
  int lat_pos = 26;
  int end_pos = 126;

  if( float_number > 99999 ) // floats with 6-character ids.
  {
	  file_date_pos = 54;
	  num_of_line_start_pos = 46;
	  lat_pos = 27;
	  end_pos = 127;
  }

  /*
     Read in data from input file
     extract substrings into arrays for comparsion later in this function
     Put float ID, sequence number and Julian day in array
     (Julian day is multiplied by 100000 to make into integer)
  */
  for (j=0; j<n_scans; j++)
  {
     //extract substring from position 48-50 to get satellite type(index start from 0)
     getsubstr(num_of_line_start_pos+num_of_line_len,num_of_line_start_pos+num_of_line_len+2,data_buffers[j],satype[j]);
     //printf("satype[%d]=%s\n",j,satype[j]);

     //extract substring from position 48-52 to get satellite type and loc_class (index start from 0)
     getsubstr(num_of_line_start_pos+num_of_line_len,file_date_pos-1,data_buffers[j],satype_loc[j]);
     //printf("satype_loc[%d]=%s\n",j,satype_loc[j]);

     //extract substring from position 53-61 to get prv_time (index start from 0)
     getsubstr(file_date_pos,file_date_pos+fd_len-1,data_buffers[j],prv_time_str[j]);
     prv_time[j]=atoi(prv_time_str[j]);
     //printf("prv_time_str[%d]=%s\n,prv_time=%d\n",j,prv_time_str[j],prv_time[j]);

     //extract substring from position 63-126 to get hexdata string (index start from 0)
     //getsubstr(63,126,data_buffers[j],hexdata[j]); //LL: 6 digits
     getsubstr(file_date_pos+fd_len+1,end_pos,data_buffers[j],hexdata[j]);
     //printf("hexdata[%d]=%s\n",j,hexdata[j]);

    if (sscanf(data_buffers[j],"%*d %d %d %d %d %d %d",&iyr,&imon,&ida,&ihr,&imin,&isec) == 6)
    {
       seq[j] = j;
       flt_dates[j] = ymd2julday(iyr, imon, ida) + 1;  // External C routine
       flt_dates[j] = (long int) (100000.0*(flt_dates[j] + ((float) ihr + ((float) imin + (float) isec/60)/60.0)/24.0));
    }
    else if (strlen(data_buffers[j]) <= 2) //lin: blank lines
    {
       seq[j]=j;
       flt_dates[j]=99999;
       if (trace_file != NULL)
       {
		   if( j<n_scans-1 )
	       fprintf (trace_file,"=> compress (func): remove blank line %d in hex file\n",j);
	   }
    }
    else
    {
       printf ("\n=> compress (func): can not read date information from data_buffers array");
       printf("\ndata_buffers[%d]=%s",j,data_buffers[j]);
       if (trace_file != NULL)
       {
           fprintf (trace_file,"\n=> compress (func): can not read date information from data_buffers array");
           fprintf (trace_file,"\ndata_buffers[%d]=%s",j,data_buffers[j]);
	   }
       if (error_file != NULL)
       {
           fprintf (error_file,"\n=> compress (func): can not read date information from data_buffers array\n");
	   }

//       fclose (output_file);
       return -9;
    }
  }



  for (j=0; j<n_scans-1; j++)
  {
    if (data_buffers[j][0] == '\0')
      continue;

    for (i=j+1; i<n_scans; i++)
    {
      if (data_buffers[i][0] == '\0')
        continue;

      if (strncmp(data_buffers[j],data_buffers[i],lat_pos) != 0)
        continue; //Id,date,or time differ => nothing to do

      if ((sscanf(data_buffers[j],"%*d %*d %*d %*d %*d %*d %*d %f %f %*c %*d %*c %*c %ld",
          &lat,&lon,&fd) != 3) ||
            ((lat == (float) 0.0) && (lon == (float) 0.0)) || (fd == 0))
        break; //Data_scans[j] has error or not a position record => skip

      if ((sscanf(data_buffers[i],"%*d %*d %*d %*d %*d %*d %*d %f %f %*c %*d %*c %*c %ld",
          &lat1,&lon1,&fd1) != 3) ||
            ((lat1 == (float) 0.0) && (lon1 == (float) 0.0)) || (fd1 == 0))
        continue; //data_buffers[i] has error or not a position record => skip

      if ((lat == lat1) && (lon == lon1))
      {
        if (fd >= fd1)//Positions same, fd later
        {
          data_buffers[j][0] = '\0';//Remove fd
          break;
        }

        data_buffers[i][0] = '\0';//Positions same, fd earlier => remove fd1
      }
      else
      {
        if (fd < fd1)//Positions different, fd earlier => remove fd
        {
          data_buffers[j][0] = '\0';
          break;
        }

        data_buffers[i][0] = '\0';//Positions different, fd later => remove fd
      } // endif: lat == lat1
    } // endfor: inner loop (i=j+1)
  } // endfor: outter loop (j=0)


  // Compare scans except for file date - mark redundant entries
  for (i=0; i<(n_scans-1); i++)
  {
    if (strlen(data_buffers[i]) == 0)
      continue;

    for (j=i+1; j<n_scans; j++)
    {
      if (strlen(data_buffers[j]) == 0)
        continue;

// remove duplications of hex data (originating from prv files collected on different days)

     if((strncmp(data_buffers[i],data_buffers[j],num_of_line_start_pos) == 0) &&
        (strcmp(satype_loc[i],satype_loc[j]) ==0) &&
	(strcmp(&data_buffers[i][file_date_pos+fd_len],&data_buffers[j][file_date_pos+fd_len]) == 0)
	)

      {

	 if(prv_time[i]>=prv_time[j]){
          data_buffers[j][0] = '\0'; // Redundant - mark it with null char
         }else{
	  data_buffers[i][0] = '\0'; // Redundant - mark it with null char
	 }

        break;
      }
// remove near duplications of positions (originating from prv files collected on different days)
// same date and hours, hex data all 0...

/*
if
abs(flt_dates[i]-flt_dates[j])<10minutes (700 in the units of flt_dates)
& hexdata==0
& satellite the same
take the line with the larger prvfile-time
example 1:                                          prvfile-time
11874 2002 08 10 04 44 32 +39.062 -047.123 0  89 L 3 178761540 0000000000000000000000000000000000000000000000000000000000000000
11874 2002 08 10 04 45 32 +39.062 -047.127 0  89 L 3 176887800 0000000000000000000000000000000000000000000000000000000000000000
take first line only (second line data_buffers[j][0] = '\0');
example 2:
11874 2002 08 10 04 45 32 +39.062 -047.127 0  89 L 3 176887800 0000000000000000000000000000000000000000000000000000000000000000
11874 2002 08 10 04 44 32 +39.062 -047.123 0  89 L 3 178761540 0000000000000000000000000000000000000000000000000000000000000000
take second line only data_buffers[i][0] = '\0';
*/


     if(abs(flt_dates[i]-flt_dates[j])<700 &&
        (strcmp(satype[i],satype[j]) == 0) &&
	(strcmp(hexdata[i],ss0) == 0) && (strcmp(hexdata[j],ss0) == 0)
	)

      {
	 if(prv_time[i]>=prv_time[j]){
          data_buffers[j][0] = '\0'; // Redundant - mark it with null char
         }else{
	  data_buffers[i][0] = '\0'; // Redundant - mark it with null char
	 }
        break;
      }

    } // endfor-inner
  } // endfor-outter

  // Now remove marked scans,reorder data_buffers,seq and flt_dates array
  j = 0;

  for (i=0; i<n_scans; i++)
  {
    if (strlen(data_buffers[i]) == 0)
      continue;
    else
    {
      if (i != j){
        strcpy(data_buffers[j],data_buffers[i]);
	flt_dates[j]=flt_dates[i];
	}
        j++;
    }
  }

  n_scans = j;
  printf ("\nRemove redundant scans - %1d remaining",n_scans);

  // Now sort by Julian day
  // put scan number in array[i][0] and julian day in array[i][1]

  sort_by_date (j);

  return j;
}

/*************************************************************************
  Gets file name
**************************************************************************/
int get_file_name(int flt_num, char *dir_name, char *flt_file_name, char *dir_name_none, char*flt_file_name_none, char *prv_file_name)
{
  int aoml_no,argos_id;

  rewind(cross_ref_file);

  while (fgets(buffer,sizeof(buffer),cross_ref_file) != NULL)
  {
    if (sscanf(buffer,"%d %d",&aoml_no,&argos_id) == 2)
    {
      if (argos_id == flt_num)
      {
//Lin: 6 digits
		 if( flt_num > 99999 )
		 {
			 sprintf(flt_file_name,"%s%04d_%06d.hex",dir_name,aoml_no,flt_num);
		 }
		 else
		 {
			 sprintf(flt_file_name,"%s%04d_%05d.hex",dir_name,aoml_no,flt_num);
		 }

         return 0;
      }
    }
  }

// Argos float number identification missing in cross reference file
// -----------------------------------------------------------------
  fprintf(crossref_noentry_file, "%07d   %s\n", flt_num, prv_file_name);
//Lin: 6 digits
  if( flt_num > 99999 )
  {
	  sprintf(flt_file_name_none,"%snone_%06d.hex",dir_name_none,flt_num);
  }
  else
  {
	  sprintf(flt_file_name_none,"%snone_%05d.hex",dir_name_none,flt_num);
  }

  flt_file_name = NULL;

  if (error_file != NULL)
    fprintf(error_file,
      "\n ERROR: GET_FILE_NAME - Argos ID %07d not found in cross reference file",flt_num);

  if (trace_file != NULL)
    fprintf(trace_file,
      "\n ERROR: GET_FILE_NAME - Argos ID %07d not found in cross reference file",flt_num);

  return 1;
}

/*************************************************************************
  Reads PRV file as received from SERVICEARGOS
  Writes output file named [float number].str (e.g., 09332.str)

  Calls: check_CRC, get_file_name, compress
**************************************************************************/
int read_prv_file (int exp_no, int bytes_per_frame, char *out_dir, char *out_dir_none, char *prv_file_name)
{
   int   i,j=0,iargosnum=0,k,n=1,data_fields=0,data_field_count;
   enum  {NO,YES} reading_data_block=NO,valid_float_id = NO,all_hex=YES;
   float latitude,longitude;
   int   iyr,imo,ida,ihr,imin,isec,scans_added,present_float_no,*used_float_nos;
   char  s[10][30], hex_string[256],hex_string2[256],sat[5],lines[5],loc_class[5];
   char  output_file_name[256],output_file_name_none[256];
   char  *ptr_hex_filename;
   unsigned int chars_per_frame,m;
   int tmp_data_fields;//added to deal with 31 bytes case
   int char_pos;
   char header1_string[256];//contain new header1 line
   char header1_prev_string[256];//contain previous header1 line

   int compress_index=0;
   int flag_compress=0;

   header1_string[0]='\0';
   header1_prev_string[0]='\0';

   int kkk;

   // Allocate array and initialize entries to zero
   if ((used_float_nos = (int *) calloc (max_floats,sizeof(int))) == NULL)
   {
     printf("ERROR (read_prv_file): cannot allocate array used_float_nos\n");
     if (error_file != NULL)
       fprintf(error_file,"ERROR (read_prv_file): cannot allocate array used_float_nos\n");

     if (trace_file != NULL)
       fprintf(trace_file,"ERROR (read_prv_file): cannot allocate array used_float_nos\n");
     return 1;
   }
   chars_per_frame = 2*bytes_per_frame;
   do // Repeat for each float number (argos id)
   {
      present_float_no = 0;
      valid_float_id = NO;
      scans_added = 0;

     // Read line from input file into buffer
     while (fgets(buffer,sizeof(buffer),input_file) != NULL)
     {

       // Read chars from buffer into variables and check if more than 3 fields
       // have been successfully read.
       // i = number of space-delimeted tokens
       if ((i=sscanf (buffer,"%s%n %s %s %s %s %s %s %s %s %s",
         	 s[0],&char_pos,s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9])) > 0)
       {
         // If s[0] is first five chars in line and does not equal exp_no then skip until
         // next exp_no
         if ((i > 3 ) && (char_pos < 6) && (atoi(s[0]) != exp_no))
         {
           valid_float_id = NO;
           continue;
         }

         // Check if s[0] contains program #. If so then a header 1 line
         // s[0] must be first five chars of line
         if ((i > 3 ) && (char_pos < 6) && (atoi(s[0]) == exp_no))
         {
           valid_float_id = NO;
           reading_data_block = NO;
           iargosnum = atoi(s[1]); // Read float # (argos id)
           // Check if no present float #
           if (present_float_no == 0)
           {
             for (k=0; k<max_floats; k++)
             {
               // Check if no float # at kth position of array of processed floats
               // If none, then set to iargosnum and flag this float as ok to process
               if (used_float_nos[k] == 0)
               {
                 present_float_no = iargosnum;
                 valid_float_id = YES; // OK to process this float
                 used_float_nos[k] = iargosnum;
                 if( iargosnum > 99999 )
                 {
                   if (trace_file != NULL)
                     fprintf (trace_file,"Float %06d found\n",iargosnum);
                 }
                 else
                 {
                   if (trace_file != NULL)
                     fprintf (trace_file,"Float %05d found\n",iargosnum);
                 }
                 printf ("\n\nReading float %1d data\n",present_float_no);
                 break; // Leave for loop and continue with: if (valid_float_id == NO)
               }
               // If already set to iargosnum, then leave alone
               else if (iargosnum == used_float_nos[k])
                 break; // leave for loop and continue with: if (valid_float_id == NO)
             } // loop if used_float_nos[k] contains a float #, but it is not iargosnum
           } // present_float_no
           // If already set to iargosnum, then flag this float as ok to process
           else if (iargosnum == present_float_no)
           {
             valid_float_id = YES;
           } // endif: present_float_no == 0
           if (valid_float_id == NO)
              continue; // continue with next iteration of while loop

           // Check if there is a 5th entry
           if (i < 5)
              sprintf (sat,"0 "); // No satellite name, so make it zero
           else
              sprintf (sat,"%1s ",s[4]); // Otherwise, satellite name

           sprintf (lines,"%3d ",atoi(s[2])); // DS lines (# of lines to next header)
           data_fields = atoi(s[3]); // Number of data fields per transmission
           // Check if lat field exists
           if (i >= 10)
           {
             sprintf (loc_class,"%1s ",s[5]); // Position quality flag
             //Force date, time, latitude, and longitude to have proper format - otherwise skip
             if ((strlen(s[6]) != 10)  || (strlen(s[7]) != 8) || (strlen(s[8]) < 5) || (strlen(s[9]) < 5))
               continue;
             //Make sure all are converted properly - otherwise skip
             if ((sscanf(s[6],"%d-%d-%d",&iyr,&imo,&ida) != 3) ||
                (sscanf(s[7],"%d:%d:%d",&ihr,&imin,&isec) != 3) ||
                (sscanf(s[8],"%f",&latitude) != 1) ||
                (sscanf(s[9],"%f",&longitude) != 1))
               continue;
             longitude = (longitude > (float) 180.0) ? longitude - (float) 360.0 : longitude;
             //LL: 6 digits
             if (debug>0) printf("iargosnum:%d",iargosnum);
             if( iargosnum > 99999 )
             {
               sprintf (header1_string,"%06d %04d %02d %02d %02d %02d %02d %+07.3f %+08.3f 0 ",
               present_float_no,iyr,imo,ida,ihr,imin,isec,latitude,longitude);
             }
             else
             {
               sprintf (header1_string,"%05d %04d %02d %02d %02d %02d %02d %+07.3f %+08.3f 0 ",
               present_float_no,iyr,imo,ida,ihr,imin,isec,latitude,longitude);
             }
             strcat(header1_string,lines);      //Append DS lines
             strcat(header1_string,sat);        //Append satellite name
             strcat(header1_string,loc_class);  //Append fix class
             strcat(header1_string,file_date); //Append file date
             // Since a header 1 line, then set entire data frame to zero
             for (j=0; j<bytes_per_frame; j++)
               strcat(header1_string,"00");
             strcat(header1_string,"\n");
     		  //if (trace_file != NULL) fprintf (trace_file," header1: %s\n",header1_string);
             //scans_added++;
           } // endif: i >=10
           continue; // continue with next iteration of while loop
         } // s[0] == exp_no
         // Check if should continue reading the float
         if (valid_float_id == YES)
         {
           // Check if a header 2 (date/time) and 1st line of data block combination line
           if ((i>=4) && (strstr(s[0],"-") != NULL))
           {
             if (sscanf(s[0],"%4d-%2d-%2d",&iyr,&imo,&ida) != 3)
               continue; // if not complete, continue with next iteration of while loop
             if (sscanf(s[1],"%2d:%2d:%2d",&ihr,&imin,&isec) != 3)
               continue; // if not complete, continue with next iteration of while loop
             if (sscanf(s[2],"%d",&compress_index) != 1)
               continue; // if not complete, continue with next iteration of while loop

             hex_string[0] = '\0';
             data_field_count = 0;
             for (j=3; j<i; j++)
             {
               // Check if data is all hexadecimal - if not discard
               all_hex = YES;
               for (m=0; m<strlen(s[j]); m++)
                 if (!isxdigit(s[j][m])) 
                   all_hex = NO;
               if (all_hex == NO)
                 break;
               // If number of data blocks = 8 and iargosnum < 32768
               //    => all data is 8 characters long so pad with leading 0's if necessary
               // If number of data blocks = 8 and iargosnum >= 32768
               //    => all data is 8 characters long so pad with leading 0's if necessary
               //       Only the last data block is special. It has 6 characters. Therefore
               //       add two 0's to the end before padding with leading 0's
               // If number of data blocks = 32 => all data is two bytes and nothing to do
               // If not 8 or 32 => first data group is 8 bytes and rest are 2 bytes
               if (((data_fields == 8) || ((data_fields == 1) && (j == 3)))
                     && (strlen(s[j]) != 8))
               {
                 sscanf(s[j],"%X",&m);
                 sprintf(s[j],"%08X",m);
               }
               strcat (hex_string,s[j]);
               data_field_count++;
             }
             reading_data_block = YES;
           } // endif: i >= 7
           // Check if strictly a data block line
           else if (reading_data_block == YES)
           {
             all_hex = YES;
             for (j=0; j<i; j++)
             {
               for (m=0; m<strlen(s[j]); m++)
                 if (!isxdigit(s[j][m]))
                   all_hex = NO;
               if (all_hex == NO)
                 break;
               if ((data_fields == 8) && (strlen(s[j]) != 8)) //If data blocks = 8 => all data 8 bytes
               {
                 if ((j==3) && (iargosnum>=32768) && (strlen(s[j]) <= 6))
                 {
                   // have to add two 0's to the end in data block 8 (data block 4
                   // in line after header #2)
                   strcat(s[j],"00");
                 }
                 sscanf(s[j],"%X",&m);
                 sprintf (s[j],"%08X",m);
               }
               strcat (hex_string,s[j]);
               data_field_count++;
             } // endif: j += 4

             //deal with 31 bytes case here to make the if (data_field_count >= data_fields) condiction more lienient
             if(data_fields==32)
             {
               tmp_data_fields=31;
             }
             else{
               tmp_data_fields=data_fields;
             }

             if (data_field_count >= tmp_data_fields)
             {
               if (strlen(hex_string)>64) {
                 printf( "hex_string before: %s\n" , hex_string );
                 hex_string[0]='\0';
                 printf( "hex_string after: %s\n" , hex_string );
               }
               // add header1 line here, only if it is totally new header1 line,add it.
               if (strcmp(header1_string,header1_prev_string)!=0)
               {
                 strcpy(data_buffers[scans_added],header1_string);
                 strcpy(header1_prev_string,header1_string);
                 if (trace_file != NULL)
                   fprintf (trace_file," header1 strcpy: %s",data_buffers[scans_added]);
                 scans_added++;
               }
               reading_data_block = NO;
               // If the record contains NO position information, then it is either a
               // header 2 line with first line of data block or a strictly a data block
               // line
               // Format:
               // ID YEAR MON DAY HR MIN SEC GOOD/BAD [64 Byte Hex ASCII Character String]
               sprintf (data_buffers[scans_added],"%3s",s[2]);
               sprintf (s[2],"%s",data_buffers[scans_added]);  //Put back in s[2]
               int p=0;
               while( p<compress_index )
               {
                 if( iargosnum > 99999 )
                 {
                   sprintf (data_buffers[scans_added],"%06d %04d %02d %02d %02d %02d %02d %+07.3f %+08.3f %1d ",
                            present_float_no,iyr,imo,ida,ihr,imin,isec+p,(float)0.0,(float)0.0,check_CRC(hex_string));
                 }
                 else
                 {
                   sprintf (data_buffers[scans_added],"%05d %04d %02d %02d %02d %02d %02d %+07.3f %+08.3f %1d ",
                            present_float_no,iyr,imo,ida,ihr,imin,isec+p,(float)0.0,(float)0.0,check_CRC(hex_string));
                 }
                 strcat(data_buffers[scans_added],lines);  //Append DS lines
                 strcat(data_buffers[scans_added],sat);    //Append satellite name
                 strcat(data_buffers[scans_added],"0 ");   //Append fix class = 0
                 strcat (data_buffers[scans_added],file_date);  //Append file date
                 strcpy(hex_string2,hex_string); 
                 while (strlen(hex_string2) < chars_per_frame)
                   strcat (hex_string2,"0");
                 strcat(data_buffers[scans_added],hex_string2);
                 strcat(data_buffers[scans_added],"\n");
                 scans_added++;
                 p++;
               }
             } // endif: data_field_count >= data_fields
           } // endif: i == 4 && reading_data_block == YES
         } // endif: valid_float_id == YES
       } // endif: sscanf
      } // endwhile: fgets
      if (trace_file != NULL)
         fprintf (trace_file," New scans found: %1d\n",scans_added);
      
      if (scans_added > 0)
      {
        int flag = get_file_name(present_float_no,out_dir,output_file_name,out_dir_none,output_file_name_none,prv_file_name);

        if ( flag == 0)
        {
          if ((output_file = fopen(output_file_name,"r")) != NULL)
          {
            while (fgets(data_buffers[scans_added++],buff_size,output_file) != NULL)
            {
              
              if (scans_added >= max_entries)
              {
                printf ("\n=> read_prv_file (func): scans_added=%d > max number of scans (%d)\n",scans_added+1, max_entries);
                if (trace_file != NULL)
                  fprintf (trace_file," => read_prv_file (func): scans_added=%d > max number of scans (%d)\n",scans_added+1, max_entries);
                fclose (output_file);
                return 9;
              } // endif scans_added >= max_entries
            } // end  while
            fclose (output_file);
          } // endif output_file

          if (trace_file != NULL)
            fprintf (trace_file," Total existing plus new scans: %1d\n",scans_added);
          printf ("\nTotal existing plus new scans = %1d",scans_added);

          j = compress (scans_added, present_float_no);
	    		if(j<0)
	    		{
					if (trace_file != NULL)
               		{
                  		fprintf (trace_file,"\n%s : compress function return -9 for %s", PRGNAME,output_file_name);
                  		fprintf (trace_file,"\n ERROR: data in hex file cannot be decoded\n");
					}
               		if (error_file != NULL)
               		{
						fprintf (error_file,"\n%s : compress function return -9 for %s", PRGNAME,output_file_name);
						fprintf (error_file,"\n ERROR: data in hex file cannot be decoded\n");
					}
	       	   		flag_compress = 1;
	    		}
	    		else
	    		{
            	if (trace_file != NULL)
               		fprintf (trace_file," Total remaining after compression: %1d\n",scans_added);
            	printf ("\nOpening for write, file: %s",output_file_name);

            	if ((output_file = fopen(output_file_name,"w")) == NULL)
            	{
               		if (trace_file != NULL)
                  		fprintf (trace_file,"\n%s : Can't open output file %s", PRGNAME, output_file_name);
               		if (error_file != NULL)
                  		fprintf (error_file,"\n%s: Can't open output file %s", PRGNAME, output_file_name);
               		printf ("\nCan't open for read file %s",output_file_name);
               		free (used_float_nos);
               		return(1);
            	} // endif: output_file

            	if (trace_file != NULL)
               		fprintf (trace_file," Writing to output file %s\n",output_file_name);
            	printf ("\nWriting hex output file and file name to list file ...");

            	for (i=0; i< j; i++)
            	{ 
                //added new
	       		  if (seq[i]<0) continue;
                  //Final check for float number
                  if (atoi(data_buffers[seq[i]]) == present_float_no) { 
//                    if (present_float_no<100000) {   // float number has 5 digits
//                      if (strlen(data_buffers[seq[i]])>128) {
//                        getsubstr(0, 126, data_buffers[seq[i]], data_buffers[seq[i]]);
//                        strcat( data_buffers[seq[i]], "\n" );
//                     }
//                   }
//                    else {
//                      if (strlen(data_buffers[seq[i]])>129) {
//                        getsubstr(0, 127, data_buffers[seq[i]], data_buffers[seq[i]]);
//                       strcat( data_buffers[seq[i]], "\n" );
//                      }
//                   }
                    fwrite(data_buffers[seq[i]],1,strlen(data_buffers[seq[i]]),output_file);
                  }
            	}

            	printf ("%1d records - OK",j);
            	if ( (ptr_hex_filename = strrchr(output_file_name, '/')) == NULL)
            	{
               		printf ("\nCan't get hex file basename %s",output_file_name);
               		if (trace_file != NULL)
                 		fprintf (trace_file,"\n%s : Can't get hex file basename %s",PRGNAME, output_file_name);
               		if (error_file != NULL)
                 		fprintf (error_file,"\n%s: Can't get hex file basename %s",PRGNAME, output_file_name);
               		return 1;
            	} // endif: ptr_hex_filename

           		ptr_hex_filename++;
            	fprintf (list_file,"%s\n", ptr_hex_filename);
				}
            	fclose(output_file);
		 	} //flag = 0
		 	else if( flag == 1) //not found in cross reference file
			{
		     	if ((output_file_none = fopen(output_file_name_none,"r")) != NULL)
		     	{
		             while (fgets(data_buffers[scans_added++],buff_size,output_file_none) != NULL)
		             {
		                   if (scans_added >= max_entries)
		                   {
		                       printf ("\n=> read_prv_file (func): scans_added=%d > max number of scans (%d)\n",scans_added+1, max_entries);
		                       if (trace_file != NULL)
		                          fprintf (trace_file," => read_prv_file (func): scans_added=%d > max number of scans (%d)\n",scans_added+1, max_entries);
		                       fclose (output_file_none);
		                       return 9;
		                   } // endif scans_added >= max_entries
		              } // end  while
		              fclose (output_file_none);
		        } // endif output_file_none

		        if (trace_file != NULL)
		        	fprintf (trace_file," Total existing plus new scans: %1d\n",scans_added);
		        printf ("\nTotal existing plus new scans = %1d",scans_added);

		 	    j = compress (scans_added, present_float_no);
		 	    if(j<0)
		 	    {
				    if (trace_file != NULL)
               		{
                  		fprintf (trace_file,"\n%s : compress function return -9 for %s", PRGNAME,output_file_name);
                  		fprintf (trace_file,"\n ERROR: data in hex file cannot be decoded\n");
					}
               		if (error_file != NULL)
               		{
						fprintf (error_file,"\n%s : compress function return -9 for %s", PRGNAME,output_file_name);
						fprintf (error_file,"\n ERROR: data in hex file cannot be decoded\n");
					}
		 	       	flag_compress = 1;
		 	    }
				else
				{
		        if (trace_file != NULL)
		        	fprintf (trace_file," Total remaining after compression: %1d\n",scans_added);
		        printf ("\nOpening for write, file: %s",output_file_name_none);

		        if ((output_file_none = fopen(output_file_name_none,"w")) == NULL)
		        {
		            if (trace_file != NULL)
		            	fprintf (trace_file,"\n%s: Can't open output file %s", PRGNAME, output_file_name_none);
		            if (error_file != NULL)
		                fprintf (error_file,"\n%s: Can't open output file %s", PRGNAME, output_file_name_none);
		            printf ("\nCan't open for read file %s",output_file_name_none);
		            free (used_float_nos);
		            return(1);
		         } // endif: output_file_none

		         if (trace_file != NULL)
		         	fprintf (trace_file," Writing to output file %s\n",output_file_name_none);
		         printf ("\nWriting hex output file and file name to list file ...");

		         for (i=0; i< j; i++)
		         { //added new
		 	       	if (seq[i]<0) continue;
                    //Final check for float number
                    if (atoi(data_buffers[seq[i]]) == present_float_no) { 
//                      if (present_float_no<100000) {   // float number has 5 digits
//                        if (strlen(data_buffers[seq[i]])>128) {
//                          getsubstr(0, 126, data_buffers[seq[i]], data_buffers[seq[i]]);
//                          strcat( data_buffers[seq[i]], "\n" );
//                        }
//                      }
//                      else {
//                        if (strlen(data_buffers[seq[i]])>129) {
//                          getsubstr(0, 127, data_buffers[seq[i]], data_buffers[seq[i]]);
//                          strcat( data_buffers[seq[i]], "\n" );
//                        }
//                      }
                      fwrite(data_buffers[seq[i]],1,strlen(data_buffers[seq[i]]),output_file_none);
                    }
		         }

		         printf ("%1d records - OK",j);
				 }
            	 fclose(output_file_none);
		    }//end flag = 1

      } // end true block: scans_added
      else           // Failure: scans_added
      {
         if (trace_file != NULL)
            fprintf (trace_file," No new scans found\n" );
      } // endif: scans_added

      rewind (input_file); // Set reading position to beginning of file
   }
   while (present_float_no != 0 ); // Continue do loop

   free (used_float_nos);

   if( flag_compress > 0 )
   {
   		return -9;
   }
   else
   {
   		return 0;
   }
}

/*************************************************************************
  'main' function

  Calls: get_file_time, read_prv_file
**************************************************************************/
int main(int argc, char *argv[])
{
  char    *ptr_path_hex, *ptr_path_log, *ptr_path_list,
          pathname_trace[161]="", pathname_error[161]="",
          pathname_list[161]="";

  int    i=0,n_bytes=32,exp_no,n_ids=0;
  enum   {NO,YES};
  char   input_file_name[256],output_directory[256],output_directory_none[256];
  time_t now;
  int    ret_val = 0;

  printf("\nEnter Program (%s) to strip out PALACE data from ARGOS PRV files\n",
         PRGNAME);

  if (argc < 11 )                   // Allow one extra for progname
  {
    printf("Format: %s program prv_infile floatid_infile\n", PRGNAME);
    printf("                hex_outdir  trace_oufile error_outfile\n");
    printf("                list_outfile n_bytes list_crossref_noentry hex_outdir_noentry\n\n");

    printf("program: program number\n");
    printf("prv_infile: prv input file name\n");
    printf("floatid_infile: float identification input file\n");
    printf("hex_outdir: output directory for generated HEX file(s)\n");
    printf("trace_outile: pathname for the trace file\n");
    printf("error_outfile: pathname for the error file\n");
    printf("list_outfile: pathname for the HEX file(s) output list file\n");
    printf("n_bytes: number of bytes\n");
    printf("list_crossref_noentry: pathname ARGOS ID not in cross reference file\n");
    printf("hex_outdir_noentry: output directory for generated HEX file(s)\n");
    return 1;
  }

  // Program Number
  // --------------
  exp_no = atoi(argv[1]);
  printf ("\nProgram number = %d",exp_no);

  // PRV
  // ---
  sprintf (input_file_name,"%s",argv[2]);
  printf ("\nInput file name = %s",input_file_name);

  // FLOAT Identification  File (cross reference)
  // --------------------------------------------
  sprintf (buffer,"%s",argv[3]);

  printf ("\nInput cross reference file name = %s",buffer);
  if ((cross_ref_file = fopen(buffer,"r")) == NULL)
  {
    if (error_file != NULL)
      fprintf (error_file,"ERROR: %s (main) - opening cross reference file %s\n",PRGNAME, buffer);

    printf ("\nERROR: %s (main) - opening cross reference file %s\n",PRGNAME, buffer);

    return 1;
  }

  // Output directory path: defaults to HEXFILES_DIR
  // -----------------------------------------------
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    printf("ERROR: %s (main) - HEXFILES_DIR <-- no such environment variable\n",PRGNAME);
    ret_val = 2;
    return ret_val;
  }

  sprintf (output_directory,"%s/",argv[4]);

  // TRACE File: defaults to ARGOSLOG_DIR/prv2hex2.today_trace
  // ---------------------------------------------------------
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR: %s (main) ARGOSLOG_DIR <-- no such environment variable\n",PRGNAME);
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[5]);
  printf ("\n%s (main): Opening output trace file %s",  PRGNAME, buffer);

  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR: %s (main) -  opening trace file %s\n", PRGNAME, buffer);
    ret_val = 1;
    return ret_val;
  }

  printf("\n%s (main): OK -> trace file opened", PRGNAME);

  if (trace_file != NULL)
  {
    fprintf (trace_file,"%s (main): Trace file %s successfully opened\n",PRGNAME, buffer);

    now = time(NULL);
    fprintf (trace_file,"\n=> %s: start %s", PRGNAME, ctime(&now));
    fprintf (trace_file,"Command input: %s\n", argv[0]);

    for (i=1; i<argc; i++)
      fprintf(trace_file, "               %s\n",argv[i]);
  }

  // ERROR File
  // ----------
  sprintf (buffer,"%s",argv[6]);
  printf ("\n%s (main): Opening File %s", PRGNAME, buffer);

  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR: %s (main) - Opening Error File %s\n", PRGNAME, buffer);
    printf ("%s (main): Command input - %s\n", PRGNAME, argv[0]);

    for (i=1; i<argc; i++)
      printf("              %s\n",argv[i]);

    if (trace_file != NULL)
      fprintf (trace_file," ERROR: %s (main) - Opening Error File %s\n",PRGNAME, buffer);

    ret_val = 1;
    return ret_val;
  }

  printf("\n%s (main): OK -> Error File Opened", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s: Error File %s Successfully Opened\n",PRGNAME, buffer);

  if (error_file != NULL)
  {
    now = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&now));
  }

  // LIST File
  // ---------
  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
    printf("ERROR: %s (main) - ARGOLIST_DIR <-- no such environment variable\n",PRGNAME);
    ret_val = 1;
    return ret_val;
  }

  sprintf (buffer,"%s",argv[7]);
  printf ("\n%s: Opening List File %s", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR: %s (main) - Opening List File %s\n", PRGNAME, buffer);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR: %s (main) - Opening List File %s\n",PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file,"ERROR: %s (main) - Opening List File %s\n",PRGNAME, buffer);
    return 1;
  }

  printf("\n%s (main: OK -> List File Successfully Opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file," %s (main): List File %s Successfully Opened\n",PRGNAME, buffer);

  // Number of Bytes
  // ---------------
  n_bytes = atoi(argv[8]);

  // LIST Cross Reference No Entry Output File
  // -----------------------------------------
  sprintf (buffer,"%s",argv[9]);
  printf ("\n%s: Opening List CrossRef No Entry File %s", PRGNAME, buffer);

  sprintf (output_directory_none,"%s/",argv[10]);

  if ((crossref_noentry_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("\nERROR: %s (main) - Opening List CrossRef No Entry File %s\n",PRGNAME, buffer);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR: %s (main) - Opening List CrossRef No Entry File %s\n",PRGNAME, buffer);

    if (error_file != NULL)
      fprintf (error_file,"ERROR: %s (main) - Opening List CrossRef No Entry File %s\n",PRGNAME, buffer);
    return 1;
  }

  printf("\n%s (main: OK -> List File CrossRef No Entry Successfully Opened\n", PRGNAME);

  if (trace_file != NULL)
    fprintf (trace_file,"%s (main): List CrossRef No Entry File %s Successfully Opened\n",PRGNAME, buffer);

/* ---------------------------------------------------------------------
                      Start Process
   --------------------------------------------------------------------- */
  // Open PRV file (input_file_name)
  // -------------------------------
  if (trace_file != NULL)
    fprintf(trace_file,"%s (main): Opening Input PRV File %s...",PRGNAME, input_file_name);

  if ((input_file = fopen(input_file_name,"r")) == NULL)
  {
     printf ("ERROR: %s (main)- Opening Input PRV File %s\n",PRGNAME, input_file_name);

     if (trace_file != NULL)
       fprintf (trace_file,"ERROR: %s (main) - Opening Input PRV File %s\n",PRGNAME, input_file_name);

     if (error_file != NULL)
       fprintf (error_file,"ERROR: %s (main) - Opening Input PRV File %s\n",PRGNAME, input_file_name);

     return 1 ;
  }

  printf (" OK\n");
  if (trace_file != NULL)
    fprintf (trace_file," OK\n");

  // 999999999 = about 30 years - I'll be long gone (D.Bitterman,5/3/99)
  if (((file_time = get_file_time(input_file_name)) < 0) || (file_time > 999999999))
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR: %s (main) - Can't get file creation time - set to zero\n",PRGNAME);

      printf ("ERROR: %s (main) - Can't get file creation time - set to zero\n",PRGNAME);

    if (error_file != NULL)
      fprintf (error_file,"ERROR: %s (main) - Can't get file creation time - set to zero\n",PRGNAME);

    file_time = 0;
  }

  sprintf (file_date,"%09ld ",file_time);

  if (trace_file != NULL)
  {
    fprintf (trace_file,"Input file creation date: %s\n",file_date);
    fprintf(trace_file,"Allocating data buffer pointers...");
  }

  printf ("\nAllocating arrays...");
  if ((data_buffers = (char **) malloc(max_entries*sizeof(char *))) == NULL)
  {
    printf ("ERROR: %s (main) - allocating data buffer pointers\n", PRGNAME);

    if (trace_file != NULL)
      fprintf (trace_file,"ERROR: %s (main): - allocating data buffer pointers\n",PRGNAME);

    if (error_file != NULL)
      fprintf (error_file,"ERROR: %s (main): - allocating data buffer pointers\n",PRGNAME);

    return 1;
  }

  printf ("%s (main): Pointers OK...", PRGNAME);

  // Loop to Allocate Data Buffer
  // ----------------------------
  if (trace_file != NULL)
     fprintf(trace_file,"%s: allocating data buffers...", PRGNAME);

  for (i=0; i<max_entries; i++)
  {
    if ((data_buffers[i] = (char *) malloc(buff_size*sizeof(char))) == NULL)
    {
      printf ("ERROR: %s (main) - allocating data buffers\n", PRGNAME);

      if (trace_file != NULL)
        fprintf (trace_file, "ERROR: %s (main) - allocating data buffers (%1d)\n",PRGNAME, i);

      if (error_file != NULL)
        fprintf (error_file, "ERROR: %s (main) - allocating data buffers (%1d)\n",PRGNAME, i);

      return 1;
    }
  }

  printf (" OK\n");
  fprintf (trace_file, " OK\n");

  if (trace_file != NULL)
      fprintf (trace_file, "%s: allocating float date and seq arrays...", PRGNAME);

  if (((flt_dates = (long *) malloc (max_entries*sizeof(long))) == NULL) ||
       ((seq = (int *) malloc (max_entries*sizeof(int))) == NULL))
  {
      printf ("ERROR: %s (main) -  allocating data indices\n", PRGNAME);

      if (trace_file != NULL)
        fprintf (trace_file,"ERROR: %s - allocating data indices\n", PRGNAME);

      if (error_file != NULL)
        fprintf (error_file,"ERROR: %s - allocating data indices\n", PRGNAME);

      return 1;
  }

  if (trace_file != NULL)
  	  fprintf (trace_file,"%s (main): OK\nFrame size = %2d bytes\n",PRGNAME, n_bytes);

  // Read/Process PRV File
  // ---------------------
  printf ("\n%s (main): Frame size = %2d bytes\n", PRGNAME, n_bytes);
  ret_val = read_prv_file (exp_no, n_bytes,output_directory,output_directory_none,input_file_name);
  printf ("\n%s (main): read_prv_file Return Value = %d\n",PRGNAME, ret_val);

  now = time(NULL);
  printf ("\n%s (main): Finished reading input file\n", PRGNAME);
  if ( ret_val == 9 )
  {
      fprintf (trace_file, "\n%s (main): ERROR while reading input file\n",PRGNAME);

      if (trace_file != NULL)
    	  fprintf (trace_file,"\n=> %s (main): ERROR while reading input file\n",PRGNAME);
      if (error_file != NULL)
    	  fprintf (error_file,"\n=> %s (main): ERROR while reading input file\n",PRGNAME);
  }
  else if (ret_val==-9)
  {
      fprintf (trace_file, "\n%s (main): ERROR while reading hex file\n",PRGNAME);

	  if (trace_file != NULL)
	 	  fprintf (trace_file,"\n=> %s (main): ERROR while reading hex file\n",PRGNAME);
	  if (error_file != NULL)
     	  fprintf (error_file,"\n=> %s (main): ERROR while reading hex file\n",PRGNAME);
  }
  else
  {
      fprintf (trace_file, "\n=> %s (main): Finished reading prv and hex files%s\n",PRGNAME, ctime(&now));
      //fprintf (error_file, "\n=> %s (main): Finished reading prv and hex files%s\n",PRGNAME, ctime(&now));
  }

  if (input_file != NULL)
      fclose (input_file);

  // Write Trail Message in Log Files (trace & error)
  // ------------------------------------------------
  if (trace_file != NULL)
      fprintf (trace_file,"\n=> %s (main): finished %s", PRGNAME, ctime(&now));

  if (error_file != NULL)
      fprintf (error_file,"\n=> %s (main): finished %s", PRGNAME, ctime(&now));

  // Close Files
  // -----------
  if (cross_ref_file != NULL)
      fclose (cross_ref_file);

  if (trace_file != NULL)
      fclose (trace_file);

  if (error_file != NULL)
      fclose (error_file);

  if (list_file != NULL)
      fclose (list_file);

  if (crossref_noentry_file != NULL)
      fclose (list_file);

  return ret_val;
}
