/*
% function [theta] = ptem(p_bar,t,s,pr)
%
% COMPUTATION OF LOCAL POTENTIAL TEMPERATURE AT
% REFERENCE PRESSURE PR USING BRYDEN 1973
% POLYNOMIAL FOR ADIABATIC LAPSE RATE AND 
% RUNGE-KUTTA FOURTH ORDER INTEGRATION ALGORITHM
%
% p_bar  pressure in bar
% t      temeprature in deg. C
% s      salinity in psu
% pr     reference pressure in bar
%
% References:   using atg80 procedure for adiabadic lapse rate
%               Fofonoff,N.,Deep-Sea Res.,24,489-491
%
% translated from matlab of Schmid to C
% by Xiangdong Xia(Jan 2005)
*/


#include <stdio.h>
#include <math.h>
double atg(double p,double t,double s);
double ptem(double p_bar,double t,double s,double pr){
    double h1,h2,xk,theta,q,p;

    h1=pr-p_bar;                        //printf("h1=%f\n",h1);
    h2=atg(p_bar,t,s);                  //printf("h2=%f\n",h2);
    xk=h1*h2;                           //printf("xk=%f\n",xk);
    theta=t+0.5*xk;                     //printf("theta=%f\n",theta);
    q=xk;                               //printf("q=%f\n",q);
    p=p_bar+0.5*h1;                     //printf("p=%f\n",p);
    h2=atg(p,theta,s);                  //printf("h2=%f\n",h2);
    xk=h1*h2;                           //printf("xk=%f\n",xk);
    theta=theta+0.29289322*(xk-q);      //printf("theta=%f\n",theta);
    q=0.58578644*xk+0.121320344*q;      //printf("q=%f\n",q);
    h2=atg(p,theta,s);                  //printf("h2=%f\n",h2);
    xk=h1*h2;                           //printf("xk=%f\n",xk);
    theta=theta+1.707106781*(xk-q);     //printf("theta=%f\n",theta);
    q=3.414213562*xk-4.121320344*q;     //printf("q=%f\n",q);
    p=p+0.5*h1;                         //printf("p=%f\n",p);
    h2=atg(p,theta,s);                  //printf("h2=%f\n",h2);
    xk=h1*h2;                           //printf("xk=%f\n",xk); 
    theta=theta+(xk-2.0*q)/6.0;         //printf("theta=%f\n",theta);
    return theta;
}    
