#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>

// Revision Date: 02-May-02 
// Program to remove ghost float data from HEX files
// Calling format:
// REMOVE_GHOSTS hex_infile min_lat max_lat min_lon max_lon delta_time_minutes
//               hex_directory trace_filename error_filename
//
// longitude from 0 to 360
// latitude from -90 to 90
//
// program returns a 0 if all is OK and no data cycles were removed
// program returns a 1 if all is OK and data cycles were removed
// program returns a 4 if an error occurred

#define PRGNAME "REMOVE_GHOSTS"

FILE *input_file,*trace_file,*error_file;
unsigned char input_filename[256],new_filename[256];

long ymd2julday(int, int, int);

unsigned char *get_root_name(unsigned char *file_name, unsigned char *out_buff)
{
    unsigned char *ptr;

    sprintf (out_buff,"%s",file_name);
    ptr = strrchr (out_buff, '.');
    if (ptr != NULL)
        *ptr = '\0';
    return out_buff;
} // end get_root_name

    

int remove_ghosts(float min_lat, float max_lat, float min_lon, float max_lon,
                  unsigned char *input_filename, float delta_time)
{
   FILE *output_file;
   float min_lon1[2],max_lon1[2];
   enum flag {FALSE1=0,TRUE1} bad_position_found = FALSE1; 
   double julday,julday0,julday_sav,julday1;
   unsigned char buffer[256];
   int yr,mon,day,hr,minute,sec,yr1,mon1,day1,hr1,minute1,sec1,n_scans=0,r_scans=0;
   float lat,lon,lat1,lon1;
   unsigned char sat,sat1;

   if (min_lon > max_lon)
   {
      min_lon1[0] = min_lon;
      min_lon1[1] = -180.0;
      max_lon1[0] = +180.0;
      max_lon1[1] = max_lon;
   }
 else
   {
      min_lon1[0] = min_lon;
      min_lon1[1] = max_lon;
      max_lon1[0] = 999.0;
      max_lon1[1] = 999.0;
   }

//First find a bad position and get time and satellite identifier
		 julday_sav=0;
		 julday0=0;
		 julday=0;
   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
   {
      if (sscanf(buffer,"%*s %d %d %d %d %d %d %f %f %*s %*s %c",
         &yr,&mon,&day,&hr,&minute,&sec,&lat,&lon,&sat) == 9)
      {
								 julday = (double) ymd2julday(yr, mon, day);
         julday += (hr*3600.0 + minute*60.0 + sec)/(24.0*3600.0);
         if (julday > julday_sav+delta_time) // -> new satellite
         {
								    julday0 = julday;
								 }
					    julday_sav = julday;
								 if (((lat != (float) 0.0) && (lon != (float) 0.0)) &&
            ((lat <= max_lat) && (lat >= min_lat)) && 
            (((lon >= min_lon1[0]) && (lon <= max_lon1[0])) ||
            ((lon >= min_lon1[1]) && (lon <= max_lon1[1]))))
         {
            fprintf (trace_file,"%02d/%02d/%04d %02d:%02d:%02d %+6.3f %+7.3f %c\n",
             mon,day,yr,hr,minute,sec,lat,lon,sat);
            julday = (double) ymd2julday(yr, mon, day);
            julday += (hr*3600.0 + minute*60.0 + sec)/(24.0*3600.0);
            fprintf (trace_file,"julday = %9.4f lat = %+6.3f lon = %+7.3f sat = %c\n",julday,lat,lon,sat);
                     bad_position_found = TRUE1;
            break;
         }
      }
   }

//If no bad positions found return without doing anything

   if (!bad_position_found)
   {
       fprintf (trace_file,"%s - no bad positions found\n",PRGNAME);
       return 0;
   }

//Open a new file with _1 appended to filename

//   get_root_name(input_filename, buffer);
//   sprintf (new_filename,"%s",buffer);
//   strcat (new_filename,"_1.HEX");
   if ((output_file = fopen(new_filename,"w")) == NULL)
   {
      fprintf (trace_file,"%s - error opening output file %s\n",PRGNAME,new_filename);
      fprintf (error_file,"%s - error opening input file %s\n",PRGNAME,new_filename);
      return 4;
   }

//Now remove all scans within +/- abs(julday0-julday) from same satellite

   rewind(input_file);
   while (fgets(buffer,sizeof(buffer),input_file) != NULL)
   {
      n_scans++;
      if (sscanf(buffer,"%*s %d %d %d %d %d %d %f %f %*s %*s %c",
      &yr1,&mon1,&day1,&hr1,&minute1,&sec1,&lat1,&lon1,&sat1) == 9)
      {
         julday1 = (double) ymd2julday(yr1, mon1, day1);
         julday1 += (hr1*3600.0 + minute1*60.0 + sec1)/(24.0*3600.0);
//         if ((fabs(julday -julday1) <= fabs(julday0-julday)) && 
//								    (sat1 == sat) && julday1 >= julday0)
         if ((fabs(julday -julday1) <= fabs(julday0-julday)) && (sat1 == sat))
         {
            r_scans++;
            fprintf (trace_file,
											 "Remove scan (%9.4f): %9.4f  %+7.3f %+9.3f %c %4d %2d %2d %2d %2d %2d\n",
            julday,julday1,lat1,lon1,sat1,yr1,mon1,day1,hr1,minute1,sec1);
            continue;
         }
      }
      fprintf (output_file,buffer);
   }
   fclose (output_file);
   fprintf (trace_file,"\nTotal scans = %d, number removed = %d\n",n_scans,r_scans);
   return 1;
} // end remove_ghosts

int main(int n_args, char *args[])
{
   time_t t;
   int return_value = 0;
   float min_lat,max_lat,min_lon,max_lon,delta_time;
  
   printf ("\n   %s - program to remove ghost float data from HEX files\n",PRGNAME);
   if (n_args < 10)
   {
      printf ("\nCalling format: %s hex_infile",PRGNAME);  //arg 1
      printf ("\n                hex_oufile");             //arg 2
      printf ("\n                min_latitude");           //arg 3
      printf ("\n                max_latitude");           //arg 4
      printf ("\n                min_longitude");          //arg 5
      printf ("\n                max_longitude");          //arg 6
      printf ("\n                delta_time_minutes");     //arg 7
      printf ("\n                hex_directory");          //arg 8
      printf ("\n                trace_filename");         //arg 9
      printf ("\n                error_filename\n");       //arg 10
      return 4;
   }
   if ((error_file = fopen(args[10],"a")) == NULL)
   {
      printf ("ERROR opening file %s\n",error_file);
      return 4;
   }
   t = time(NULL);
   fprintf (error_file,"%s: start %s", PRGNAME, ctime(&t));
   if ((trace_file = fopen(args[9],"a")) == NULL)
   {
      fprintf (error_file,"%s: Error opening trace file %s\n",PRGNAME,args[8]);
      return_value = 4;
   }
   else
   {
      fprintf (trace_file,"%s: start %s", PRGNAME, ctime(&t));
      delta_time = (float) (atof (args[7])/60./24.0);        //Convert to fraction of a day
      sprintf (input_filename,"%s/%s",args[8],args[1]);      //Copy hex_directory and file_name to input_filename
      sprintf (new_filename,"%s/%s",args[8],args[2]);

      printf("   input_filename(hex_infile): %s\n", input_filename);
      printf("   new_filename(hex_outfile):  %s\n", new_filename);

      if ((input_file = fopen(input_filename,"r")) == NULL)
      {
         fprintf (error_file,"%s: Error opening input file %s\n",PRGNAME,args[1]);
         fprintf (trace_file,"%s: Error opening input file %s\n",PRGNAME,args[1]);
         return_value = 4;
      }
      else
      {
         fprintf (trace_file,"%s: Input file %s successfuly opened\n",PRGNAME,input_file);
         min_lat = (float) atof(args[3]);
         max_lat = (float) atof(args[4]);
         min_lon = (float) atof(args[5]);
         max_lon = (float) atof(args[6]);
         
         if ((min_lat <= max_lat) && (min_lat <= 90.0) && (min_lat >= -90.0) &&
            (max_lat <= 90.0) && (max_lat >= -90.0) && (return_value == 0) &&
            (min_lon <= 180.0) && (min_lon >= -180.0) && (max_lon <= 180.0) && (max_lon >= -180.0))
            return_value = remove_ghosts(min_lat,max_lat,min_lon,max_lon,input_filename,delta_time);
         else
         {
            fprintf (error_file,"%s: Error - latitude-longitude out of range\n",PRGNAME);
            fprintf (trace_file,"%s: Error - latitude-longitude out of range\n",PRGNAME);
            return_value = 4;
         }
      } // open input_file
   } // open trace_file
  
   t = time(NULL);
   fprintf (error_file,"%s: end %s\n", PRGNAME, ctime(&t));
   fprintf (trace_file,"%s: end %s\n", PRGNAME, ctime(&t));

   if (input_file != NULL)
      fclose(input_file);
   if (trace_file != NULL)
      fclose (trace_file);
   if (error_file != NULL)
      fclose (error_file);
   return return_value;
} // end main
