// sort.h
//* contains various sorting algorithms

#include <string>
#include <vector>
#include <algorithm>
using namespace std;


// Swap utility
//   Receive:   Type parameter ElementType
//              a and b of type ElementType
//   Pass back: a and b with values interchanged
//--------------------------------------------------
template <typename ElementType>
void Swap(ElementType & a, ElementType & b)
{
  ElementType temp = a;
  a = b;
  b = temp;
}


template <class ElementType>

void Split(vector<ElementType> & x,vector<ElementType> & y,int first, int last, int & pos)
{
  ElementType pivot_x = x[first]; // pivot element
  ElementType pivot_y = y[first]; // pivot element

      
  int left = first,             // index for left search
      right = last;             // index for right search

  while (left < right)
  {
    // Search from right for element <= pivot
    while (x[right] > pivot_x)
      right--;
    // Search from left for element > pivot
    while (left < right && x[left] <= pivot_x)
      left++;
    // Interchange elements if searches haven't met
    if (left < right){
      Swap(x[left], x[right]);Swap(y[left], y[right]);
    } 
  }
  // End of searches; place pivot in correct position
  pos = right;
  
  x[first] = x[pos];
  x[pos] = pivot_x;
  
  y[first] = y[pos];
  y[pos] = pivot_y;
  
}


// QUICKSORT

template <class ElementType>

void Quicksort(vector<ElementType> & x,vector<ElementType> & y, int first, int last)
{
  int pos;          // final position of pivot
  if (first < last) // list has more than one element
  {
    // Split into two sublists
    Split(x,y, first, last, pos);
    // Sort left sublist
    Quicksort(x,y, first, pos - 1);
    // Sort right sublist
    Quicksort(x,y, pos + 1, last);
  }
  // else list has 0 or 1 element and
  // requires no sorting
}


