/******************************************************************************
  Revision Date: 13-Aug-13 (CA -  "ARGOS FREQUENCY" is now being transmitted in the 
				  test message of APEX_TS41, upon occurance of the 
				  test data  "ARGOS FREQUENCY" 401.650 MHZ is added to 
				  the counts value.Verification of the occurance of
				  "ARGOS FREQUENCY" is hardcoded as ,
				  until now test data never has been computed unlike 
				  pressure, current, temperature, salinity values
		 12-Apr-12 (CS - fix p3 references fo 4ch hex for 2ch hex
		 08-Mar-10 (LL - add p3 for sea surface pressure for TS32)
		 13-Jan-10 (LL - revise pressure,tempreture, salinity functions)

  cnt2phy_tsall.c
  ============
  Converts the counts ARGOS data for temperature/salinity/pressure
  University of Washington PALACE floats manufactured in 2000 by Webb Research
  Cooperation and deployed by Steve Riser
 *******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#define max_scans 2000  //Maximum scans for each profile
#define buff_size 256   //Maximum length of each scan read from *.str

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file, *table_file;

char PRGNAME[] ="CNT2PHY_TS";
char buffer[1024];

struct aoml_cals
{
	double TSLOPE;
	double TOFF;
	double PSLOPE;
	double POFF;
	double SSLOPE;
	double SOFF;
	double VSLOPE;
	double VOFF;
	double VACSLOPE;
	double VACOFF;
};

union u_cals
{
	struct aoml_cals s;
	double c[sizeof(struct aoml_cals)/sizeof(double)];
};

struct meta_struct
{
	int   AOML_number;
	int   WMO_id;
	int   ARGOS_id;
	unsigned char float_type[40];
	float cycle_time;
	float start_time;
	float up_time;
	union u_cals cals;
};

struct table_struct
{

	char test_print[100][100];
	char test_unitcnt[100][100];
	char test_compute[100][5];
	char test_unitphy[100][100];
	int test_length;

	char data_print[200][100];
	char data_unitcnt[200][100];
	char data_compute[200][5];
	char data_unitphy[200][100];
	int data_length;

};

long ymd2julday(int, int, int);
char *strupr(char *);
char *strlor(char *);
double dfr97 (const int, const int, const int, const int, const int, const int, int *);
int get_meta_file_name_cnt2phy (char *);


//get table name
int get_table_name(char *str_name, struct meta_struct *s)
{
	strcpy(str_name,"table_");

	char float_type_str[30];
	strcpy(float_type_str,strlor(s->float_type));

	char *pch1;
	pch1 = strtok(float_type_str,"_");
	int n=0;
	while ( pch1 != NULL )
	{
		if (n==1)
		{
			strcat(str_name,pch1);
			//             printf("%s\n",str_name);
		}

		pch1 = strtok(NULL,"_");
		n++;
	}

	strcat(str_name,".txt");

	return 0;
}

//read table
int read_table_file (char *table_name, struct table_struct *table_struct)
{

	int m=0;
	int n=0;

	int i=0;


	if (trace_file != NULL)
		fprintf(trace_file,"Opening table file %s...\n",table_name);

	if ((table_file = fopen(table_name,"r")) == NULL)
	{
		if (error_file != NULL)
			fprintf (error_file," ERROR opening table file %s\n",table_name);
		if (trace_file != NULL)
			fprintf (trace_file," ERROR opening table file %s\n",table_name);
		return 1;
	}
	else
	{
		if (trace_file != NULL)
			fprintf (trace_file," Opened table-file succesfully\n");
	}

	while (fgets(buffer,sizeof(buffer),table_file) != NULL)
	{

		if (strncmp(buffer,"TEST",4)==0)
		{

			m = 0;
			while (fgets(buffer,sizeof(buffer),table_file) != NULL && strncmp(buffer,"DATA",4)!=0)
			{
				char *pch2;
				pch2 = strtok(buffer,",");

				n = 0;
				while ( pch2 != NULL )
				{

					if (n==2)
					{
						if(strcmp(pch2,"\n")==0) pch2 = "";
						strcpy(table_struct->test_print[m],pch2);
					}

					if (n==3)
					{
						if(strcmp(pch2,"\n")==0) pch2 = "";
						strcpy(table_struct->test_unitcnt[m],pch2);
					}

					if (n==4)
					{
						if(strcmp(pch2,"\n")==0) pch2 = "";
						strcpy(table_struct->test_compute[m],pch2);
					}

					if (n==5)
					{
						if(strcmp(pch2,"\n")==0) pch2 = "";
						strcpy(table_struct->test_unitphy[m],pch2);
					}

					pch2 = strtok(NULL,",");
					n++;

				}
				m++;
			}
			table_struct->test_length = m;
		}

		if (strncmp(buffer,"DATA",4)==0)
		{

			m = 0;
			while (fgets(buffer,sizeof(buffer),table_file) != NULL )
			{
				char *pch4;
				pch4 = strtok(buffer,",");

				n = 0;
				while ( pch4 != NULL )
				{

					if (n==2)
					{
						if(strcmp(pch4,"\n")==0) pch4 = "";
						strcpy(table_struct->data_print[m],pch4);
					}

					if (n==3)
					{
						if(strcmp(pch4,"\n")==0) pch4 = "";
						strcpy(table_struct->data_unitcnt[m],pch4);
					}

					if (n==4)
					{
						if(strcmp(pch4,"\n")==0) pch4 = "";
						strcpy(table_struct->data_compute[m],pch4);
					}

					if (n==5)
					{
						if(strcmp(pch4,"\n")==0) pch4 = "";
						strcpy(table_struct->data_unitphy[m],pch4);
					}

					pch4 = strtok(NULL,",");
					n++;

				}
				m++;
			}
			table_struct->data_length = m;
		}

	}
	return 0;
}

float compute_temperature1 (struct meta_struct *s, int t_counts)
{
	float temp;

	if (t_counts==99999)
	{
		return -9.00;
	}
	if(t_counts==0XF000) {temp=-9.00;}
	else
	{
		if( t_counts == 0XFFFF) {t_counts = 0XFFFE;}
		temp = (t_counts <= 0XEFFF) ? ((float) (s->cals.s.TSLOPE * t_counts + s->cals.s.TOFF)) :
			((float) (-s->cals.s.TSLOPE * (0XFFFF - t_counts + 1) + s->cals.s.TOFF));

		if(temp<=-4.096) {temp=-9.00;}
		else if(temp>=61.440) {temp=-9.00;}
	}

	return temp;
}

float compute_temperature2 (struct meta_struct *s, int t_counts)
{
	float temp;

	temp = (t_counts <= 0XF448) ? ((float) (s->cals.s.TSLOPE * t_counts + s->cals.s.TOFF)) :
		((float) (-s->cals.s.TSLOPE * (0XFFFF - t_counts + 1) + s->cals.s.TOFF));

	if(temp<-3.000) temp=-9.00;
	else if(temp>62.535) temp=-9.00;

	return temp;
}



float compute_pressure1 (struct meta_struct *s, int p_counts)
{
	float pres;

	if (p_counts==99999)
	{
		return -999.0;
	}

	if(p_counts == 0XFFFF) {p_counts = 0XFFFE;}
	pres = (p_counts <= 0X7FFF) ? ((float) (s->cals.s.PSLOPE * p_counts + s->cals.s.POFF)) :
		((float) (-s->cals.s.PSLOPE * (0XFFFF - p_counts + 1) + s->cals.s.POFF));

	if(pres<=-3276.7) pres=-999.0;
	else if(pres>=3276.7)  pres=-999.0;

	return pres;
}

float compute_pressure2 (struct meta_struct *s, int p_counts)
{
	float pres;

	pres = (float) (s->cals.s.PSLOPE * p_counts + s->cals.s.POFF);

	return pres;
}


float compute_ss_pressure (struct meta_struct *s, int p_counts)
{
	float pres;

	if(p_counts==0X00FE) {
		pres=-0.1;
	}
	else
	{
		pres = (p_counts <= 0X00E9) ? ((float) (s->cals.s.PSLOPE * p_counts + s->cals.s.POFF)) :
			((float) (-s->cals.s.PSLOPE * (0X00FF - p_counts + 1) + s->cals.s.POFF));
		if(pres<=-2.0) {
			pres=-999.0;
		}
		else if(pres>=23.4) {
			pres=-999.0;
		}
	}
	return pres;
}


float compute_salinity1 (struct meta_struct *s, int s_counts)
{
	float salt;

	if (s_counts==99999)
	{
		return -999.0;
	}


	if(salt == 0XFFFF) {salt=0XFFFE;}
	salt = (s_counts <= 0XEFFF) ? ((float) (s->cals.s.SSLOPE * s_counts + s->cals.s.SOFF)) :
		((float) (-s->cals.s.SSLOPE * (0XFFFF - s_counts + 1) + s->cals.s.SOFF));

	if(salt<=-4.095) {salt=-999.0;}
	else if(salt>=61.439) {salt=-999.0;}

	return salt;
}

float compute_salinity2 (struct meta_struct *s, int s_counts)
{
	float salt;

	salt = (float) (s->cals.s.SSLOPE * s_counts + s->cals.s.SOFF);

	return salt;
}

float compute_voltage1 (struct meta_struct *s, int vol_counts)
{
	float b;
	b=(float) (s->cals.s.VSLOPE * vol_counts + s->cals.s.VOFF);
	if (vol_counts==0)
		b=0.;
	return(b);
}

float compute_vacuum1 (struct meta_struct *s, int vac_counts)
{
	float b;
	b=(float) (s->cals.s.VACSLOPE * vac_counts + s->cals.s.VACOFF);
	if (vac_counts==0)
		b=0.;
	return(b);
}

float compute_current1 (struct meta_struct *s, int c_counts)
{
	float b;
	// 	b=(float) (s->cals.s.CSLOPE * vac_counts + s->cals.s.COFF);
	//	CSLOPE=4.017; COFF=-18.6;


	//printf("c_counts =%d\n",c_counts);

	b=(float) (4.017 * c_counts - 18.6);
	//uncomment the above line following -CA
	//b=(float) (3.9935 * c_counts - 0.5521);
	if (c_counts==0)
		b=0.;
	return(b);
}

float compute_current2 (struct meta_struct *s, int c_counts)
{
	float b;
	// 	b=(float) (s->cals.s.CSLOPE * vac_counts + s->cals.s.COFF);
	//	CSLOPE=12.99; COFF=-0.6;

	b=(float) (12.99 * c_counts - 0.6);
	if (c_counts==0)
		b=0.;
	return(b);
}


float compute_current3 (struct meta_struct *s, int c_counts)
{
	float b;
	// 	b=(float) (s->cals.s.CSLOPE * vac_counts + s->cals.s.COFF);
	//	CSLOPE=4.052; COFF=-3.606;

	b=(float) (4.052 * c_counts - 3.606);
	if (c_counts==0)
		b=0.;
	return(b);
}


int convert_data(struct meta_struct *s, struct table_struct *t, char *output_name)
{
	int i1=0,i2=0,i3=0,i4=0,i5=0,i6=0;
	int iflag=0, pflag=0, tflag=0, cflag=0;
	char *ptr_phy_filename;
	int dd=-1;

	if ((out_file = fopen(output_name,"w")) == NULL)
	{
		if (trace_file != NULL)
			fprintf (trace_file," CONVERT_DATA: error opening output file %s\n",output_name);
		if (error_file != NULL)
			fprintf (error_file," CONVERT_DATA: error opening output file %s\n",output_name);
		return 1;
	}

	if (trace_file != NULL)
		fprintf (trace_file," CONVERT_DATA: output file %s successfully opened\n",output_name);

	// Write Output Profile Physical Unit File Name to List File
	// ---------------------------------------------------------

	if ( (ptr_phy_filename = strrchr(output_name, '/')) == NULL)
	{
		printf ("\nCan't get phy file basename %s", output_name);
		if (trace_file != NULL)
			fprintf (trace_file,"\n%s : Can't get phy file basename %s", PRGNAME, output_name);
		if (error_file != NULL)
			fprintf (error_file,"\n%s: Can't get phy file basename %s", PRGNAME, output_name);
		return 1;
	} // endif: ptr_phy_filename

	ptr_phy_filename++;
	fprintf (list_file,"%s\n", ptr_phy_filename);

	int profile_number = 0;
	int k = 0;
	char *ptr;
	char tmp[3];

	ptr = strrchr(ptr_phy_filename,'_');    // get pointer to last '_'
	for(k=0;k<3;k++)
	{
		tmp[k] = ptr[k+1];
	}
	profile_number = atoi(tmp);
	//    printf("prof_num:%d\n",profile_number);


	while (fgets(buffer,sizeof(buffer),input_file) != NULL)
	{

		if (strstr(buffer,"++++++++++++++++++++++++++++++") != NULL)
		{
			iflag = 100;
		}

		if(profile_number == 0 && iflag==0)
		{
			int m=0;
			for(m=0;m<t->test_length;m++)
			{
				if ( strncmp( strupr(buffer),t->test_print[m],strlen(t->test_print[m]) ) == 0 )
				{
					char tmp_buff[10] = "";
					int j=0;
					//printf("\n%d,%d,%d,",strlen(buffer),strlen(t->test_print[m]),strlen(t->test_unitcnt[m]));
					if( strlen(t->test_print[m])+strlen(t->test_unitcnt[m]) > 0 )
					{
						while( j < ( strlen(buffer)-strlen(t->test_print[m])-strlen(t->test_unitcnt[m])) )
						{
							tmp_buff[j] = buffer[strlen(t->test_print[m])+strlen(t->test_unitcnt[m])+j];
							j++;
						}
					}
					if ( strcmp(tmp_buff,"") == 0 )
						dd = 0;
					else
						dd = atoi(tmp_buff);
					if ( strcmp(t->test_compute[m],"p1")==0 )
					{
						sprintf (buffer,"%s%s%-6.1f\n",t->test_print[m],t->test_unitphy[m],compute_pressure1(s,dd));
					}
					else if ( strcmp(t->test_compute[m],"p2")==0 )
					{
						sprintf (buffer,"%s%s%-6.1f\n",t->test_print[m],t->test_unitphy[m],compute_pressure2(s,dd));
					}
					else if ( strcmp(t->test_compute[m],"p3")==0 )
					{
						sprintf (buffer,"%s%s%-6.1f\n",t->test_print[m],t->test_unitphy[m],compute_ss_pressure(s,dd));
					}
					else if( strcmp(t->test_compute[m],"t1")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->test_print[m],t->test_unitphy[m],compute_temperature1(s,dd));
					}
					else if( strcmp(t->test_compute[m],"t2")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->test_print[m],t->test_unitphy[m],compute_temperature2(s,dd));
					}
					else if( strcmp(t->test_compute[m],"s1")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->test_print[m],t->test_unitphy[m],compute_salinity1(s,dd));
					}
					else if( strcmp(t->test_compute[m],"s2")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->test_print[m],t->test_unitphy[m],compute_salinity2(s,dd));
					}
					else if( strcmp(t->test_compute[m],"vo1")==0 )
					{
						sprintf (buffer,"%s%s%-5.2f\n",t->test_print[m],t->test_unitphy[m],compute_voltage1(s,dd));
					}
					else if( strcmp(t->test_compute[m],"va1")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->test_print[m],t->test_unitphy[m],compute_vacuum1(s,dd));
					}
					else if( strcmp(t->test_compute[m],"c1")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->test_print[m],t->test_unitphy[m],compute_current1(s,dd));
					}
					else if( strcmp(t->test_compute[m],"c2")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->test_print[m],t->test_unitphy[m],compute_current2(s,dd));
					}
					else if( strcmp(t->test_compute[m],"c3")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->test_print[m],t->test_unitphy[m],compute_current3(s,dd));
					}
					else if( ( strcmp(t->test_compute[m]," ")==0 )&& (strcmp(t->test_print[m],"ARGOS FREQUENCY")==0 ) )
					{
						sprintf (buffer,"%s%s%-4.3f\n",t->test_print[m],t->test_unitphy[m],(dd+401.650));
					}


					m=t->test_length;
				}
			}
		}
		else if(profile_number > 0 && iflag==0)
		{
			int n=0;
			for(n=0;n<t->data_length;n++)
			{
				if ( strncmp( strupr(buffer),t->data_print[n],strlen(t->data_print[n]) ) == 0 )
				{
					char tmp_buff[10] = "";
					int j=0;
					if( strlen(t->data_print[n])+strlen(t->data_unitcnt[n]) > 0 )
					{
						while( j < ( strlen(buffer)-strlen(t->data_print[n])-strlen(t->data_unitcnt[n])) )
						{
							tmp_buff[j] = buffer[strlen(t->data_print[n])+strlen(t->data_unitcnt[n])+j];
							j++;
						}
					}
					if ( strcmp(tmp_buff,"") == 0 )
						dd = 0;
					else
						dd = atoi(tmp_buff);


					if ( strcmp(t->data_compute[n],"p1")==0 )
					{
						sprintf (buffer, "%s%s%-6.1f\n",t->data_print[n],t->data_unitphy[n],compute_pressure1(s,dd));
						pflag = 1;
					}
					else if( strcmp(t->data_compute[n],"p2")==0 )
					{
						sprintf (buffer, "%s%s%-6.1f\n",t->data_print[n],t->data_unitphy[n],compute_pressure2(s,dd));
						pflag = 2;
					}
					else if( strcmp(t->data_compute[n],"p3")==0 )
					{
						sprintf (buffer, "%s%s%-6.1f\n",t->data_print[n],t->data_unitphy[n],compute_ss_pressure(s,dd));
					}
					else if( strcmp(t->data_compute[n],"t1")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->data_print[n],t->data_unitphy[n],compute_temperature1(s,dd));
						tflag = 1;
					}
					else if( strcmp(t->data_compute[n],"t2")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->data_print[n],t->data_unitphy[n],compute_temperature2(s,dd));
						tflag = 2;
					}
					else if( strcmp(t->data_compute[n],"s1")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->data_print[n],t->data_unitphy[n],compute_salinity1(s,dd));
					}
					else if( strcmp(t->data_compute[n],"s2")==0 )
					{
						sprintf (buffer,"%s%s%-6.3f\n",t->data_print[n],t->data_unitphy[n],compute_salinity2(s,dd));
					}
					else if( strcmp(t->data_compute[n],"vo1")==0 )
					{
						sprintf (buffer,"%s%s%-5.2f\n",t->data_print[n],t->data_unitphy[n],compute_voltage1(s,dd));
					}
					else if( strcmp(t->data_compute[n],"va1")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->data_print[n],t->data_unitphy[n],compute_vacuum1(s,dd));
					}
					else if( strcmp(t->data_compute[n],"c1")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->data_print[n],t->data_unitphy[n],compute_current1(s,dd));
						//printf("t->data_compute[n], t->data_print[n] , t->data_unitphy[n], s,dd,compute_current1(s,dd) = %s,%s,%s,%s,%d,%-4.1f \n",t->data_compute[n],t->data_print[n],t->data_unitphy[n],s,dd,compute_current1(s,dd));
						cflag = 1;
					}
					else if( strcmp(t->data_compute[n],"c2")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->data_print[n],t->data_unitphy[n],compute_current2(s,dd));
						cflag = 2;
					}
					else if(  strcmp(t->data_compute[n],"c3")==0 )
					{
						sprintf (buffer,"%s%s%-4.1f\n",t->data_print[n],t->data_unitphy[n],compute_current3(s,dd));
						cflag = 3;
					}
					n=t->data_length;
				}
			}
		}

		if (strstr(buffer,"++++++++++++++++++++++++++++++++") != NULL)
		{
			if( profile_number == 0 )
			{
				while (fgets(buffer,sizeof(buffer),input_file) != NULL)
				{
					if (strstr(buffer,"================================") != NULL)
						break;
				}
			}
			else
			{
				while (fgets(buffer,sizeof(buffer),input_file) != NULL)
				{
					if (strstr(buffer,"++++++++++++++++++++++++++++++++") != NULL)
					{
						fgets(buffer,sizeof(buffer),input_file);
						break;
					}
				}
			}
		}

		if ((strstr(buffer,"COLUMNS") != NULL) || (strstr(buffer,"COLUMNS")))
		{
			int col_flag = 0;

			while (fgets(buffer,sizeof(buffer),input_file) != NULL)
			{
				if (strstr(buffer,"================================") != NULL)
					break;
				else
					col_flag++;

			}

			if( col_flag > 4 )
			{
				fprintf (out_file,"NUMBER OF COLUMNS                       6\n");
				fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
				fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
				fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
				fprintf (out_file,"4. COLUMN                               PRESSURE2 (DBAR)\n");
				fprintf (out_file,"5. COLUMN                               TEMPERATURE2-90 (DEG C)\n");
				fprintf (out_file,"6. COLUMN                               CHECKSUM FLAG\n");

				iflag = 2;
			}
			else
			{
				fprintf (out_file,"NUMBER OF COLUMNS                       4\n");
				fprintf (out_file,"1. COLUMN                               PRESSURE (DBAR)\n");
				fprintf (out_file,"2. COLUMN                               TEMPERATURE-90 (DEG C)\n");
				fprintf (out_file,"3. COLUMN                               SALINITY (PSU)\n");
				fprintf (out_file,"4. COLUMN                               CHECKSUM FLAG\n");

				iflag = 1;
			}

			char buffer1[1024] = "";

			rewind(meta_file);
			while (fgets(buffer1,sizeof(buffer1),meta_file) != NULL)
			{
				if (strstr(buffer1,"================") != NULL)
					break;
			}
			while (fgets(buffer1,sizeof(buffer1),meta_file) != NULL)
			{
				if (strstr(buffer1,"================") != NULL)
				{
					break;
				}
				fwrite (strupr(buffer1),1,strlen(buffer1),out_file);
			}

			if(cflag == 1)
			{
				fprintf (out_file,"CALIB EQ 1 FOR CURRENT                  CSLOPE*CNTS+COFF\n");
				fprintf (out_file,"CALIB COEF FOR CURRENT                  CSLOPE=4.017; COFF=-18.6;\n");
			}
			else if(cflag == 2)
			{
				fprintf (out_file,"CALIB EQ 1 FOR CURRENT                  CSLOPE*CNTS+COFF\n");
				fprintf (out_file,"CALIB COEF FOR CURRENT                  CSLOPE=12.99; COFF=-0.6;\n");
			}
			else if(cflag == 3)
			{
				fprintf (out_file,"CALIB EQ 1 FOR CURRENT                  CSLOPE*CNTS+COFF\n");
				fprintf (out_file,"CALIB COEF FOR CURRENT                  CSLOPE=4.052; COFF=-3.606;\n");
			}

			fwrite (buffer,1,strlen(buffer),out_file);

			while (fgets(buffer,sizeof(buffer),input_file) != NULL)
			{
				if (strstr(buffer,"================================") != NULL)
				{
					fwrite (buffer,1,strlen(buffer),out_file);
					break;
				}
				else
					fwrite (buffer,1,strlen(buffer),out_file);
			}

			continue;
		}

		if (iflag == 1)
		{
			if (sscanf(buffer,"%d %d %d %d",&i1,&i2,&i3,&i4) == 4)
			{
				if (i1==0 & i2==0 & i3==0 & i4==0)  // no data
				{
					fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",(float)i1,(float)i2,(float)i3,i4);
				}
				else
				{
					if( pflag == 1 && tflag == 1)
					{
						/*if( tflag == 3 )
						  {
						  fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",
						  compute_pressure1(s,i1),compute_temperature3(s,i2),compute_salinity1(s,i3),i4);
						  }
						  else if( tflag == 1 )
						  {
						  fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",compute_pressure1(s,i1),
						  compute_temperature1(s,i2),compute_salinity1(s,i3),i4);
						  }*/
						fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",compute_pressure1(s,i1),
								compute_temperature1(s,i2),compute_salinity1(s,i3),i4);
					}
					else if( pflag == 2 && tflag == 2)
					{
						fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",compute_pressure2(s,i1),
								compute_temperature2(s,i2),compute_salinity2(s,i3),i4);
					}
					/*else if( pflag == 3 )
					  {
					  fprintf (out_file,"%6.1f   %6.3f   %6.3f   %1d\n",compute_pressure3(s,i1),
					  compute_temperature2(s,i2),compute_salinity2(s,i3),i4);
					  }*/
				}
			}
		}
		else if (iflag == 2)
		{
			if (sscanf(buffer,"%d %d %d %d %d %d",&i1,&i2,&i3,&i5,&i6,&i4) == 6)
			{
				if (i1==0 & i2==0 & i3==0 & i5==0 & i6==0 & i4==0)  // no data
				{
					fprintf (out_file,"%6.1f   %6.3f   %6.3f   %6.1f   %6.3f   %1d\n",
							(float)i1,(float)i2,(float)i3,(float)i5,(float)i6,i4);
				}
				else if (i5 == 99999 && i6 == 99999)
				{
					fprintf (out_file,"%6.1f   %6.3f   %6.3f   -999.0    -9.00   %1d\n",compute_pressure1(s,i1),
							compute_temperature1(s,i2),compute_salinity1(s,i3),i4);
				}
				else
				{
					fprintf (out_file,"%6.1f   %6.3f   %6.3f   %6.1f   %6.3f   %1d\n",compute_pressure1(s,i1),compute_temperature1(s,i2),
							compute_salinity1(s,i3),compute_ss_pressure(s,i5),compute_temperature1(s,i6),i4);
				}
			}
		}
		else
			fprintf (out_file,buffer);

	}// end of while

	if (trace_file != NULL)
		fprintf (trace_file," CONVERT_DATA:%s completed successfully\n",output_name);

	return 0;

}




int read_meta_file (char *meta_name, struct meta_struct *meta_struct)
{

	int i,j,syear,smonth,sday,shour,smin,x;
	char *ptr;
	char *cal_keys[] = {"TSLOPE=","TOFF=","PSLOPE=","POFF=","SSLOPE=","SOFF=",
		"VSLOPE=","VOFF=",
		"VACSLOPE=","VACOFF=","\0"};

	for (i=0; i<(sizeof(struct aoml_cals)/sizeof(double)); i++)
		meta_struct->cals.c[i] = 0.0;
	meta_struct->cycle_time = (float) 0.0;
	meta_struct->start_time = (float) 0.0;

	// 	printf("Opening meta file %s",meta_name);
	if ((meta_file = fopen(meta_name,"r")) == NULL)
	{
		printf ("\nError opening meta file %s",meta_name);
		if (error_file != NULL)
			fprintf (error_file," ERROR opening meta file %s\n",meta_name);
		if (trace_file != NULL)
			fprintf (trace_file," ERROR opening meta file %s\n",meta_name);
		return(1);
	}

	if (trace_file != NULL)
		fprintf (trace_file," Meta file %s successfully opened\n",meta_name);

	while (fgets(buffer,sizeof(buffer),meta_file) != NULL)
	{
		if (strstr(strupr(buffer),"INTERNAL ID NUMBER") != NULL)
		{
			if(sscanf(buffer,"%*s %*s %*s %d",&meta_struct->AOML_number) == 0)
				meta_struct->AOML_number = 0;
			continue;
		}

		if (strstr(strupr(buffer),"WMO ID NUMBER") != NULL)
		{
			if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->WMO_id) == 0)
				meta_struct->WMO_id = 0;
			continue;
		}

		if (strstr(strupr(buffer),"TRANSMISSION ID NUMBER") != NULL)
		{
			if (sscanf(buffer,"%*s %*s %*s %d",&meta_struct->ARGOS_id) == 0)
				meta_struct->ARGOS_id = 0;
			continue;
		}

		if ((strstr(strupr(buffer),"INSTRUMENT TYPE") != NULL) && (strstr(strupr(buffer),"WMO") == NULL))
		{
			if (sscanf(buffer,"%*s %*s %s",meta_struct->float_type) == 0)
			{
				sprintf (meta_struct->float_type,"\0");
				continue;
			}

			if ((ptr = strstr(&buffer[strlen("FLOAT TYPE")],meta_struct->float_type)) != NULL)
				sprintf (meta_struct->float_type,"%s",ptr);
			if ((ptr = strstr(meta_struct->float_type,"\n")) != NULL)
				sprintf (ptr,"\0");
			continue;
		}

		if (strstr(strupr(buffer),"DELAY OF FIRST DOWN TIME") != NULL)
		{
			if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %d",&x) == 0)
				x = 0;
			meta_struct->start_time += (float) (x/24.0);
			continue;
		}

		if (strstr(strupr(buffer),"DOWN TIME") != NULL)
		{
			if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
				x = 0;
			meta_struct->cycle_time += x;
			continue;
		}

		if (strstr(strupr(buffer),"UP TIME") != NULL)
		{
			if (sscanf(buffer,"%*s %*s %*s %d",&x) == 0)
				x = 0;
			meta_struct->cycle_time += (float) (x/24.0);
			meta_struct->start_time -= (float) (x/24.0);
			continue;
		}

		if ((strstr(strupr(buffer),"START TIME") != NULL) && (strstr(strupr(buffer),"YYYY") != NULL))
		{
			if (sscanf(buffer,"%*s %*s %*s %*s %*s %*s %*s %*s %d %d %d %d %d",&sday,&smonth,&syear,&shour,&smin) == 5)
				meta_struct->start_time += (float) (dfr97(syear,smonth,sday,shour,smin,0,&x) + 0.25);
			else
				meta_struct->start_time = 0;
			continue;
		}

		i = 0;
		while ((j = strlen(cal_keys[i])) > 1)
		{
			if ((ptr = strstr(buffer,cal_keys[i])) != NULL)
				meta_struct->cals.c[i] = atof(ptr+j);
			i++;
		}
	}
	return 0;
}

int main (int argc, char *argv[])
{

	char     *ptr_path_meta, *ptr_path_cnt, *ptr_path_phy,
		 *ptr_path_log, *ptr_path_list, *ptr_path_table,
		 pathname_input_filename[161]="",
		 pathname_trace[161]="", pathname_error[161]="";

	char     input_filename[80],table_filename[80],output_directory[256],
		 msg[11]="", title[11]="", *ptr;
	struct   meta_struct s;
	struct   table_struct t={};                                                   // long t;
	unsigned long tim;
	int      ret_val = 0;

	// Program Name
	// ------------
	//  printf("Entering Program: %s\n", PRGNAME);
	strcpy(title, "Program calibrates counts files for APEX float");
	//  printf("%s\n", title);

	// Input Arguments
	// ---------------
	if (argc < 6)
	{
		printf("Format: %s cnt_infile phy_outdir trace_outile error_outfile\n", PRGNAME);
		printf("\n");
		printf("cnt_infile:    counts input file name\n");
		printf("phy_outdir:    output directory for generated calibrated data file(s)\n");
		printf("trace_outile:  output directory for generated trace file\n");
		printf("error_outfile: output directory for generated error file\n");
		printf("list_outfile:  output directory for generated list file\n");
		return 1;
	}

	// Input Counts File Name
	// ----------------------
	// printf ("%s: Input CNT File Name - %s\n", PRGNAME, argv[1]);
	sprintf (input_filename,"%s",argv[1]);

	// Output directory path: defaults to PHYFILES_DIR
	// -----------------------------------------------
	if ( (ptr_path_phy = getenv("PHYFILES_DIR")) == NULL)
	{
		printf("ERROR (%s): PHYFILES_DIR <-- no such environment variable\n", PRGNAME);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}

	// printf ("%s: Output Directory - %s\n", PRGNAME, argv[2]);
	sprintf (output_directory,"%s/",argv[2]);

	// TRACE File
	// ----------
	if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
	{
		printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val ;
	}

	// printf ("%s: Opening trace file %s ", PRGNAME, buffer);
	sprintf (buffer,"%s",argv[3]);

	if ((trace_file = fopen(buffer,"a+")) == NULL)
	{
		printf ("\nERROR (%s): Opening trace file %s", PRGNAME, buffer);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}
	// printf("%s: OK -> trace file opened\n", PRGNAME);

	if (trace_file != NULL)
	{
		tim = time(NULL);
		fprintf (trace_file,"=> %s: start %s", PRGNAME, ctime(&tim));
	}

	// ERROR File
	// ----------
	sprintf (buffer,"%s",argv[4]);
	// printf ("%s: Opening error file %s ", PRGNAME, buffer);

	if ((error_file = fopen(buffer,"a+")) == NULL)
	{
		printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
		if (trace_file != NULL)
			fprintf (trace_file," ERROR (%s): Opening file %s\n", PRGNAME, buffer);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}
	// printf("%s: OK -> error file opened\n", PRGNAME);

	if (trace_file != NULL)
		fprintf (trace_file,"%s: File %s successfully opened\n", PRGNAME, buffer);
	if (error_file != NULL)
	{
		tim = time(NULL);
		fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&tim));
	}

	// LIST File
	// ---------
	if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
	{
		printf("\nERROR (%s): ARGOLIST_DIR <-- no such environment variable",
				PRGNAME);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}

	sprintf (buffer,"%s",argv[5]);
	//  printf ("%s: Opening list file %s ", PRGNAME, buffer);

	if ((list_file = fopen(buffer,"a+")) == NULL)
	{
		printf ("\nERROR (%s): Opening list file %s\n", PRGNAME, buffer);

		if (trace_file != NULL)
			fprintf (trace_file,"ERROR (%s): Opening list file %s\n",
					PRGNAME, buffer);
		if (error_file != NULL)
			fprintf (error_file,"ERROR (%s): Opening list file %s\n",
					PRGNAME, buffer);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}
	//  printf("%s: OK -> list file opened\n", PRGNAME);
	if (trace_file != NULL)
		fprintf (trace_file,"%s: List file %s successfully opened\n",PRGNAME, buffer);

	// Open Input Counts File
	// ----------------------
	if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
	{
		printf("ERROR (%s): CNTFILEDIR <-- no such environment variable\n", PRGNAME);

		if (trace_file != NULL)
			fprintf (trace_file,"ERROR (%s) : CNTFILES_DIR <-- no such environment variable\n",PRGNAME);
		if (error_file != NULL)
			fprintf (error_file,"ERROR (%s): CNTFILES_DIR <-- no such environment variable\n",PRGNAME);

		if (trace_file != NULL)                  // Close Log files
			fclose (trace_file);
		if (error_file != NULL)
			fclose (error_file);

		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}

	sprintf(pathname_input_filename, "%s/%s", ptr_path_cnt, input_filename);

	if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
	{                                                 // Failure: could not open cnt file
		printf ("\nERROR (%s): Opening input file %s",PRGNAME, pathname_input_filename);

		if (trace_file != NULL)
			fprintf (trace_file, "ERROR (%s): Opening input file %s\n",PRGNAME, pathname_input_filename);
		if (error_file != NULL)
			fprintf (error_file, "ERROR (%s): Opening input file %s\n",PRGNAME, pathname_input_filename);
		tim = time(NULL);
		printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
		ret_val = 1;
		return ret_val;
	}
	else                                              // Success: cnt file opened
	{
		//  		printf ("%s: Input file %s successfully opened\n",PRGNAME, pathname_input_filename);
		if (trace_file != NULL)
			fprintf (trace_file,"%s: Input file %s successfully opened\n",PRGNAME, pathname_input_filename);

		// META File
		// ---------
		if (get_meta_file_name_cnt2phy (input_filename) != 0)
		{                                             // Failure: did not get meta filename
			printf ("ERROR (%s): Finding meta file name", PRGNAME);
			if (error_file != NULL)
				fprintf (error_file,"\nERROR (%s): Finding meta file name - returned name %s\n",PRGNAME, input_filename);
			if (trace_file != NULL)
				fprintf (trace_file,"\nERROR (%s): Finding meta file name - returned name %s\n",PRGNAME, input_filename);
			ret_val = 1;
			return ret_val;
		}
		else
		{                                             // Success: got meta filename
			if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
			{
				printf("%s: METAFILES_DIR <-- no such environment variable\n", PRGNAME);
				if (trace_file != NULL)
					fprintf (trace_file,"ERROR (%s): METAFILES_DIR <-- no such environment variable\n",PRGNAME);
				if (error_file != NULL)
					fprintf (error_file,"ERROR (%s): METAFILES_DIR <-- no such environment variable\n",PRGNAME);

				if (trace_file != NULL)                  // Close Log files
					fclose (trace_file);
				if (error_file != NULL)
					fclose (error_file);

				ret_val = 1;
				return ret_val;
			} // endif  ptr_path_meta

			sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

			if (read_meta_file (pathname_input_filename, &s) != 0)
			{                                         // Failure: couln't read meta_file
				printf ("\nERROR (%s): Reading meta file", PRGNAME);
				if (trace_file != NULL)
					fprintf (trace_file,"ERROR (%s): Reading meta file %s\n",PRGNAME, pathname_input_filename);
				if (error_file != NULL)
					fprintf (error_file, "ERROR (%s): Reading meta file %s\n",PRGNAME, pathname_input_filename);
				ret_val = 1;
				return ret_val;
			}
			else
			{
				if (trace_file != NULL)
					fprintf (trace_file, "%s: Meta file %s successfully read\n",PRGNAME, pathname_input_filename);
				//    			printf ("\n%s: Meta file %s successfully read\n", PRGNAME, pathname_input_filename);

				// Table File
				// -------------------------
				if (get_table_name (table_filename, &s) != 0)
				{                                             // Failure: did not get table filename
					if (error_file != NULL)
						fprintf (error_file,"ERROR (%s): Finding table file name - returned name %s\n",PRGNAME, input_filename);
					if (trace_file != NULL)
						fprintf (trace_file,"ERROR (%s): Finding table file name - returned name %s\n",PRGNAME, input_filename);
					ret_val = 1;
					return ret_val;
				}
				else
				{
					// Success: got table filename
					if ( (ptr_path_table = getenv("TABLE_DECODERS_DIR")) == NULL)
					{
						if (trace_file != NULL)
							fprintf (trace_file,"ERROR (%s): TABLE_DECODERS_DIR <-- no such environment variable\n",PRGNAME);
						if (error_file != NULL)
							fprintf (error_file,"ERROR (%s): TABLE_DECODERS_DIR <-- no such environment variable\n",PRGNAME);

						if (trace_file != NULL)                  // Close Log files
							fclose (trace_file);
						if (error_file != NULL)
							fclose (error_file);

						tim = time(NULL);
						if (trace_file != NULL)
							fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
						if (error_file != NULL)
							fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
						ret_val = 1;
						return ret_val;
					} // endif  ptr_path_table

					sprintf(pathname_input_filename, "%s/%s", ptr_path_table, table_filename);


					if (read_table_file (pathname_input_filename, &t) != 0)
					{                                         // Failure: couldn't read table_file
						if (trace_file != NULL)
							fprintf (trace_file, "ERROR (%s): Reading table file %s\n",
									PRGNAME, pathname_input_filename);
						if (error_file != NULL)
							fprintf (error_file, "ERROR (%s): Reading table file %s\n",
									PRGNAME, pathname_input_filename);
						ret_val = 9;
						return ret_val;
					}
					else                                      // Success: read table_file
					{
						if (trace_file != NULL)
							fprintf (trace_file,"%s: Table file %s successfully read\n",PRGNAME, pathname_input_filename);
						//						printf ("\n%s: Table file %s successfully read\n", PRGNAME, pathname_input_filename);
					} // endif: read_table_file
				} // endif: get_table_name

				// Convert Counts to Physical Units
				// --------------------------------
				sprintf (input_filename,"%s%s", output_directory, argv[1]); // !! using input storage

				if ( (ptr = strrchr(input_filename,'.')) == NULL)
					strcat (input_filename,".phy");
				else
					sprintf(ptr,".phy");                             // ???

				//             	printf ("%s: Output Path & File Name - %s", PRGNAME,input_filename);

				if (convert_data(&s, &t, input_filename) == 0)
				{
					//      				printf ("\n%s: CONVERT_DATA  successful\n", PRGNAME);
					if (trace_file != NULL)
						fprintf (trace_file, "%s: CONVERT_DATA successful\n", PRGNAME);
				}
				else
				{
					printf ("\nERROR (%s): CONVERT_DATA\n");
					if (trace_file != NULL)
						fprintf (trace_file, "ERROR (%s): CONVERT_DATA\n",PRGNAME);
					if (error_file != NULL)
						fprintf (error_file, "ERROR (%s): CONVERT_DATA\n",PRGNAME);
					ret_val = 1;
					return ret_val;
				} // endif: convert_data
			} // endif: read_meta_file
		} // endif: get_meta_file_name_cnt2phy
	} // endif: input_file

	// Write finish message to trace & error files
	// -------------------------------------------
	tim = time(NULL);
	if ( ret_val == 0)
		strcpy(msg, "finished");
	else
		strcpy(msg, "terminated");

	// printf ("%s: %s %s\n", PRGNAME, msg, ctime(&tim));

	if (trace_file != NULL)
		fprintf (trace_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&tim));

	if (error_file != NULL)
		fprintf (error_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&tim));

	// Close all files
	// ---------------
	if (input_file != NULL)
		fclose (input_file);

	if (table_file != NULL)
		fclose (table_file);

	if (trace_file != NULL)
		fclose (trace_file);

	if (error_file != NULL)
		fclose (error_file);

	if (list_file != NULL)
		fclose (list_file);

	return ret_val;
}
