/***********************************************************************************
    Revision Date: 26-Nov-2010 (ER  Initial release )

    get_profile_number
    ==================
    Given the raw data corresponding to a profile, returns the profile number 

    Input Parameters
      t: info pieces extracted from table_ts*.txt file
      n_scans: number of lines of buff to be processed (size of the profile in the
               raw data in hex file)
      buff: array of array of chars.  holds a profile. each line is a raw line from
            the hex file.  
               
    Returns and integer with this posible values:
      -1: error
      n: profile number.
      
************************************************************************************/ 
#include "hex2cnt_tsall.h"

int get_profile_number(char ** buff, int n_scans, struct table_struct *t)
{
  
  int debug=OFF; 

  int prof_nbr_msg;      // in which message the profile number is.
  int prof_nbr_position; // in which position (byte) of the msg the prof number is
  int prof_nbr;          // profile number as it appears in the hex file.
  int hexline_msg_nbr;      

  int MAX_DIFF_PROF_NBR_VALUES=20;
  int prof_nbr_value[MAX_DIFF_PROF_NBR_VALUES];
  int prof_nbr_count[MAX_DIFF_PROF_NBR_VALUES];
  int diff_prof_nbr_values_cnt;
  int most_repeated_prof_nbr=-1;
  int most_repeated_prof_nbr_cnts=0;

  int found;
  int int_dummy;
  long long_dummy;
  float float_dummy;
  unsigned char char_dummy;
  unsigned char hex_data[80];

  if (debug==ON) {
    printf( "BEGIN get_profile_number\n");
    fflush(stdout);
  }

  int i;
  for (i=0; i<t->data_length; i++) {
    if( strncmp(t->data_name[i],"PRFN",4)==0 ) {
      prof_nbr_msg =  (t->data_count[i] / t->data_length) + 1;
      prof_nbr_position = t->data_count[i] % t->data_length;
      break;
    }
  }

  // Profile Number to be the most repeated profile number.
  // 
  //    Initializing variables 
  diff_prof_nbr_values_cnt = 0;
  for (i=1; i<MAX_DIFF_PROF_NBR_VALUES; i++) {
    prof_nbr_count[i]=0;
  }

  for (i=0; i<n_scans; i++) {
    if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       	 &int_dummy,&int_dummy,&int_dummy,&int_dummy,&int_dummy,&int_dummy,&int_dummy,
        &float_dummy,&float_dummy,&int_dummy,&int_dummy,&char_dummy,&char_dummy,
        &long_dummy,hex_data) == 15) {
      // Get msg number for the line.
      sscanf(&hex_data[2],"%2X",&hexline_msg_nbr);
      if (hexline_msg_nbr==prof_nbr_msg) {
        // msg number = msg number of profile number parameter
        sscanf(&hex_data[(prof_nbr_position+2)*2], "%2X", &prof_nbr );
        found=FALSE;
        int j;
        for (j=1; j<=diff_prof_nbr_values_cnt; j++) {
          if (prof_nbr_value[j]==prof_nbr) {
            found=TRUE;
            prof_nbr_count[j]++;
            break;
          }
        }
        if (found==FALSE) {
          diff_prof_nbr_values_cnt++;
          prof_nbr_value[diff_prof_nbr_values_cnt]=prof_nbr;
          prof_nbr_count[diff_prof_nbr_values_cnt]=1;
        }
      }
    }
  }

  most_repeated_prof_nbr=-1;
  most_repeated_prof_nbr_cnts=0;
  for (i=1; i<=diff_prof_nbr_values_cnt; i++ ) {
    if (prof_nbr_count[i]>most_repeated_prof_nbr_cnts) {
      most_repeated_prof_nbr=prof_nbr_value[i];
      most_repeated_prof_nbr_cnts=prof_nbr_count[i];
    }
  }

  if (debug==ON) {
    printf( "END get_profile_number\n");
  }

  return (most_repeated_prof_nbr);
}

