/***********************************************************************************
    Revision Date: 04-Dec-2012 (CA: Introduced a variable test_flag which is always zero
			            in get_raw_data . It is to differentiate between a test message
				    and a data message during the call of start_of_transmission)
    		   04-Dec-2012 (CA: Formula for calculating the actual_frames_per_message
    				    is changed due to the presence of SSTLEN in the table
				    using the variable sum_data_byte which is used to calculate
				    the total bytes in the table)
    		   04-Dec-2012 (CA: frames_per_message now has a different value coming in
		   	            It previously had 256 , now it has only 20 as only 20 messages
				    are printed in the cnts file. This Change is also important for
				    the MOST_LIKELY_FRAME_STATUS problem)

    		   07-Jul-2010 (ER: y->flag_blk_time is calculated once; therefore
                                    it was moved out of a loop.
                   02-Jul-2010 (ER: Changes in layout and added comments to make it
                                    more readable.
                   01-Jul-2010 (CS: Method gets to be alone in a file of its own.)

    get_raw_data
    ==================
    Given the raw data corresponding to a profile, returns the profile in a 
    data structure.

    Input Parameters
      t: info pieces extracted from table_ts*.txt file
      s: info pieces extracted from meta file
      buff: array of array of chars.  holds a profile. each line is a raw l    ine from
            the hex file.  
      n_scans: number of lines of buff to be processed (size of the profile in the
               raw data in hex file)
      m_type:  type of message 
               
    Output parameters
      y: profile data in data structure output (see hex2cnt_tsall.h)

    Returns and integer with this posible values:
      -1: error
      n: profile number.
      
************************************************************************************/ 
#include "hex2cnt_tsall.h"
int get_raw_data (struct output_record *y, struct table_struct *t, struct meta_struct *s, char ** buff, int n_scans, int m_type)
{
  int debug=OFF; 
//  int debug=ON; 
  int actual_frames_per_message;

  struct raw_data_record x;
  int i,j,k,most_common[frames_per_message+1]; //frames_per_mesage+1 for test mesage
  int id,iy,imo,ida,ih,im,is,g,ndoy,n_l,jd_old=0;
  int n_pos=0;
  int iy1,imo1,ida1,ih1,im1,is1;

  long int dummy;
  int l, frames;
  char ch;
  char frms[3]="";

  float lat,lon;
  unsigned char hex_data[80],sat,l_c;
  unsigned int z[32];  // hold hexadecimal part of a frame
  unsigned roll_over=0,old_count=0;

  int skip_byte[64];
  int m = 0;
  int n = 0;
  int kk = 0;
  int start = 0;
  int end = 0;
  int test_flag = 0;// is always zero for get_raw_data
 
  long blk_time;
  int sum_data_byte = 0; //Sum of all the data bytes in the table
  if (debug==ON) {
    printf( "IN GRD==============================================================================================\n");
    printf( "IN GRD BEGIN get_raw_data\n");
  }

  if(s->ARGOS_id > 32767 && t->data_cycle > 29)
  {
    if (trace_file != NULL)
      fprintf (trace_file," ARGOS id indicates 28-bit\n");
    if (error_file != NULL)
      fprintf (error_file," ARGOS id indicates 28-bit\n");
    return -1;
  }
  else if(s->ARGOS_id < 32768 && t->data_cycle < 30)
  {
    if (trace_file != NULL)
      fprintf (trace_file," ARGOS id indicates 20-bit\n");
    if (error_file != NULL)
      fprintf (error_file," ARGOS id indicates 20-bit\n");
    return -1;
  }

  y->flag_blk_time = start_of_transmission(buff, n_scans, &blk_time, 1,test_flag);
  y->flag_most_likely_frame = 0;

  // Initialize the 29 or 30 data bytes for each frame.
  // Initialize status of each frame as NEVER_RECEIVED(0)
  //   (later on the status will be changed to 1(GOOD) or 4(BAD)
  //    for those frames present)
  // Get the most likely frames
  // 
  for (i=0; i<frames_per_message; i++)
  {
    int flag_mlf = 0;

    y->status[i] = NEVER_RECEIVED;

    if(t->data_cycle == 29 )
    {
      for (j=0; j<29; j++)
        x.data.data_bytes_29[i][j] = 0;
    }
    else if(t->data_cycle == 30 )
    {
      for (j=0; j<30; j++)
        x.data.data_bytes_30[i][j] = 0;
    }

    m = 0;
//    printf( "data_count:%d\n", (int)ceil((double)(t->data_count[t->data_length-1])/(double)(t->data_cycle)) );
    if( i < (int)ceil((double)(t->data_count[t->data_length-1])/(double)(t->data_cycle)) )
    {
      for( n=0; n<t->data_length; n++ )
      {
        if( (i+1)*t->data_cycle > t->data_count[t->data_length-1] )
        {
          if( i == 0 ) start = 0;
          else start = end;
          end = t->data_length;
          n = t->data_length+1; //speed up
        }
        else if( (i+1)*t->data_cycle == t->data_count[n] )
        {
          if( i == 0 ) start = 0;
          else start = end;
          end = n;
          n = t->data_length+1; //spead up
        }

      }

      skip_byte[0] = 0;
      skip_byte[1] = 1;
      m = 2;

      for( n=start; n<end; n++ )
      {
        if( (strncmp(t->data_name[n],"BLK",3) == 0 && strlen(t->data_name[n]) == 3) ||
            (strncmp(t->data_name[n],"ABP",3) == 0 && strlen(t->data_name[n]) == 3) ||
            (strncmp(t->data_name[n],"SST",3) == 0 && strlen(t->data_name[n]) == 3) ||
            (strncmp(t->data_name[n],"SSP",3) == 0 && strlen(t->data_name[n]) == 3) ||
            (strncmp(t->data_name[n],"CP",2) == 0 && strlen(t->data_name[n]) == 2) )
        {
          for( kk=0; kk<2*t->data_byte[n]; kk++ )
          {
            skip_byte[m+kk] = 4+kk+2*(t->data_count[n]-t->data_count[start]);
          }
          m = m + 2*t->data_byte[n];
        }
      }
    }
		
    skip_byte[m] = 100;
    //last parameter is 0, for calls from get_raw_data where as it is 1,2 or 3 in get_raw_data_test
    if ((j = most_likely_frame (buff, n_scans, i, 1, skip_byte, &flag_mlf,0)) >= 0){ //find most likely - good CRC only
      most_common[i] = j;
      
      }
    else if ((j = most_likely_frame (buff, n_scans, i, 0, skip_byte, &flag_mlf,0)) >= 0) {//find most likely - any CRC
      most_common[i] = j;
      }
    else{
      most_common[i] = -1;
      }

    if( flag_mlf > 0 )
    {
      y->flag_most_likely_frame = 1;
      if (debug==ON) 
      printf("FLAG_MLF =%d I am greater than 0 for cycle number = %d\n",flag_mlf,total_cycles_count);
    }
  }

  // CS - FRAME 1 PROBLEM
  //
  if ((m_type == 1) && (most_common[0] < 0)) //frame =1 => most_common[0]
  {
    if ( total_cycles_count > 1 )  {
      frame1_problem_count++;

      if (trace_file != NULL)
        fprintf (trace_file," Can't decode message without frame 1\n");
      if (error_file != NULL)
        fprintf (error_file," Can't decode message without frame 1\n");
    }
    return -1;	//Can't decode message without frame -1 - error return
  }
  total_cycles_count++;

  // Initializing julian dates for all possible positions
  for (j=1; j<max_positions; j++)
    x.jd[j] = -1.0;

  // Initializing 
  for (i=0; i<20; i++)
  {
  	strcpy(y->hex_data_print[i],"");
  	strcpy(y->hex_data_time[i],"");
  }
  for (i=0; i<n_scans; i++)
  {
    if (sscanf(buff[i],"%d %d %d %d %d %d %d %f %f %d %d %c %c %ld %s",
       	 &id,&iy,&imo,&ida,&ih,&im,&is,&lat,&lon,&g,&n_l,&sat,&l_c,&dummy,hex_data) != 15)
      continue;

    if (m_type == 1)
    {
      // 3rd and 4th bytes from hex_data are the frame number
      // 
      frames=0;
      for (l=0; l<2 ; l++)
      {
        ch=*(hex_data+l+2);
        switch (ch)
        {
          case 'A': strcpy(frms,"10"); strcat(frms, "\0"); break;
          case 'B': strcpy(frms,"11"); strcat(frms, "\0"); break;
          case 'C': strcpy(frms,"12"); strcat(frms, "\0"); break;
          case 'D': strcpy(frms,"13"); strcat(frms, "\0"); break;
          case 'E': strcpy(frms,"14"); strcat(frms, "\0"); break;
          case 'F': strcpy(frms,"15"); strcat(frms, "\0"); break;
          default : frms[0]=ch; frms[1]='\0'; break;
        }
        frames+=((1-l)*16*atoi(frms)+l*atoi(frms));
      }
      // skip if > frames_per_message.
      //
      if (frames > frames_per_message)
        continue;
    }

    //first read each line
    //
    x.latitude[n_pos] = lat;
    x.longitude[n_pos] = lon;
    x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);
    sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);

    x.loc_class[n_pos] = l_c;
    x.sat_name[n_pos] = sat;
    x.n_xmits[n_pos] = (int) floor((n_l-1)/8);

    // first line; the 99.999 position line
    //
    if(n_pos==0)
    { 
      //reset to first 99.999 999.999 line
      x.latitude[n_pos] = (float) 99.999;
      x.longitude[n_pos] = (float) 999.999;
      x.n_xmits[n_pos] = 0;
      n_pos++;
    }

    // Line with  lat an long values; discard the hex part of the line
    //
    if((lat != (float) 0.0) || (lon != (float) 0.0))
    {
      x.latitude[n_pos] = lat;
      x.longitude[n_pos] = lon;
      x.jd[n_pos] = dfr97(iy,imo,ida,ih,im,is,&ndoy);

      sprintf(x.date_time[n_pos], "%4d/%02d/%02d%3d:%02d:%02d\0",iy,imo,ida,ih,im,is);
      x.loc_class[n_pos] = l_c;
      x.sat_name[n_pos] = sat;
      x.n_xmits[n_pos] = (int) floor((n_l-1)/8);
      n_pos = (n_pos < (max_positions-1)) ? n_pos + 1 : n_pos;
      continue;
    }


    // current frame's hexadecimal data assigned to array z
    //
   	for (j=0; j<32; j++)
      sscanf( (const char *) &hex_data[2*j],"%2x",&z[j]);

    // If frame number < minimum frame number, discard it
    if (z[1] < min_frame_no) 
      continue;

    if (m_type == 1)
    {
      if (z[1]>frames_per_message)
      {
        if (trace_file != NULL)
          fprintf (trace_file,"z[1] exceeds frames_per_message.\n");
        if (error_file != NULL)
          fprintf (error_file,"z[1] exceeds frames_per_message.\n");
        exit (1);
      }

      // if it is not the most common line ((most likely)
      // for that frame, discard it
      //
      if (i != (most_common[z[1] - 1]))
        continue;

      if (z[1] == 1)
      {
        if (z[2] < old_count)
          roll_over++;
        old_count = z[2];
        y->msg_blk_no = z[2]-1 + roll_over*256; //count starts at 1
       if (debug==ON) {
		 printf( "IN GRD 2: msg_blk_no is %d\n", y->msg_blk_no );
	       }

	      }

	      y->status[z[1]-1] = (g == 0) ? GOOD : BAD;

	      if( t->data_cycle == 29 )
	      {
		for (k=0; k<29; k++)
		  x.data.data_bytes_29[z[1]-1][k] = (unsigned char) z[k+2];
	      }
	      else if( t->data_cycle == 30 )
	      {
		for (k=0; k<30; k++)
		  x.data.data_bytes_30[z[1]-1][k] = (unsigned char) z[k+2];
	      }
	    }

	    //Speed up - move time calculations outside loop
	    //
	    if (z[1] == 1)
	    {
	      is1 = is;
	      im1 = im;
	      ih1 = ih;
	      ida1 = ida;
	      imo1 = imo;
	      iy1 = iy;
	    }

	    // printing the first 20 selected messages on the cnt file.  
	    //
	    if( z[1] < 20 ) 
	    {
	      sprintf(y->hex_data_print[z[1]-1], "%02d/%02d/%02d %02d:%02d:%02d  %d  %s", iy,imo,ida,ih,im,is,g,hex_data );
	      sprintf(y->hex_data_time[z[1]-1], "%04d %02d %02d %02d %02d %02d", iy,imo,ida,ih,im,is );
	    }

  }//end of for n_scans loop


  //add the last 999.999 line
  x.latitude[n_pos] = (float) 99.999;
  x.longitude[n_pos] = (float) 999.999;
  x.n_xmits[n_pos] = 0;

  if( y->flag_blk_time == 0 )
  {
    y->first_blk_time = ((ymd2julday(iy1,imo1,ida1)*24+ih1)*60+im1)*60+is1;
  }
  else
  {
    y->first_blk_time = blk_time;
      if (debug==ON) 
   printf("IN GRD RESULT FROM SOT 2: y->first_blk_time =%d \n", y->first_blk_time );
  }

  x.jday = (int) floor(x.jd[0]);

  if ((trace_file != NULL) && (m_type == 1))
  {
    fprintf (trace_file,"T/P Float: profile number %d status = ",y->profile_no);
    for (j=(frames_per_message-1); j>0; j--)
       fprintf (trace_file,"%1d",y->status[j]);
    fprintf (trace_file," (0=NOT RCVD,1=OK,4=BAD)\n");
  }

  y->float_id = id;
  y->jd = x.jday;


  if (m_type == 1) //Data message
  {

    int n = 0;
    int sstflag = 0;

    for(n=0;n<t->data_length;n++)
    {
      //printf("t->data_name[n] %s \n",t->data_name[n]);

      if( strncmp(t->data_name[n],"PRFN",4)==0 )
      {
        y->profile_no = x.data.d_fmt.data_message[t->data_count[n]];
	  //printf("IN GRD: y->profile_no =%d \n ",y->profile_no);
      }
      else if(t->data_byte[n] == 2)
      {
        y->data_message[t->data_count[n]] = 256*x.data.d_fmt.data_message[t->data_count[n]] + 
                                                x.data.d_fmt.data_message[t->data_count[n]+1];
      }
      else if(t->data_byte[n] == 4)
      {
        y->data_message[t->data_count[n]] = 256*256*256*x.data.d_fmt.data_message[t->data_count[n]+3] + 
                                                256*256*x.data.d_fmt.data_message[t->data_count[n]+2] + 
                                                    256*x.data.d_fmt.data_message[t->data_count[n]+1] + 
                                                        x.data.d_fmt.data_message[t->data_count[n]];
       // printf(" y->data_message[t->data_count[n]] in byte 4 %l \n", y->data_message[t->data_count[n]]);
      }
      else
      {
        y->data_message[t->data_count[n]] = x.data.d_fmt.data_message[t->data_count[n]];
        if( strncmp(t->data_name[n],"PRFL",4)==0 )
        {
          y->profile_len = x.data.d_fmt.data_message[t->data_count[n]];
	  //printf("IN GRD: y->profile_len = %d,y->profile_no =%d  \n",y->profile_len,y->profile_no);
	}
        //else if( strncmp(t->data_name[n],"SSTLEN",6)==0 )
        if( strncmp(t->data_name[n],"SSTLEN",6)==0 )
        {
          y->sst_len = x.data.d_fmt.data_message[t->data_count[n]];
          sstflag = 1;
	 // printf("IN GRD: y->sst_len = %d, y->profile_no =%d \n",y->sst_len,y->profile_no);
        }
      }
    }

    if(sstflag == 0)
      y->sst_len = 0;
      
      sum_data_byte = t->data_count[t->data_length-1]+ t->data_byte[t->data_length-1];

      actual_frames_per_message = (int) ceil(( ( ((float)y->profile_len*6) + ((float)y->sst_len*3) + (sum_data_byte))  /t->data_cycle));
      if (debug==ON) 
      printf("IN GRD : actual_frames_per_message NEW = %d,%04d_%05d_%03d.cnt \n",actual_frames_per_message,s->AOML_number,s->ARGOS_id,y->profile_no);

    if (actual_frames_per_message > frames_per_message)
    {
      if (trace_file != NULL)
        fprintf( trace_file,"actual_frames_per_message (%d) exceeds frames_per_message (%d).\n", 
                 actual_frames_per_message, frames_per_message  );
      if (error_file != NULL)
        fprintf( error_file,"actual_frames_per_message (%d) exceeds frames_per_message (%d).\n", 
                 actual_frames_per_message, frames_per_message  );
      exit (1);
    }


    int start = t->data_count[t->data_length-1]+t->data_byte[t->data_length-1];
    int frame_1st_char, frame_2nd_char; // frame in which the first and second
                                        // character of pres, temp or salinity
                                        // are. (they might not be the same)

//    if (y->profile_no > 0)
//    {
      for (j=0; j<y->profile_len; j++)	//length-1 because bottom values included
      {
        y->press_sal_temp[j][1] = (256*x.data.d_fmt.data_message[start+j*6+0] +
                                       x.data.d_fmt.data_message[start+j*6+1]); // Temperature
        y->press_sal_temp[j][2] = (256*x.data.d_fmt.data_message[start+j*6+2] +
                                       x.data.d_fmt.data_message[start+j*6+3]); // Salinity
        y->press_sal_temp[j][0] = (256*x.data.d_fmt.data_message[start+j*6+4] +
                                       x.data.d_fmt.data_message[start+j*6+5]); // Pressure
        y->press_sal_temp[j][3] = j;	//Bin number

        // pressure
        frame_1st_char = ( (start+j*6+4) / t->data_cycle ) + 1;
        frame_2nd_char = ( (start+j*6+5) / t->data_cycle ) + 1;
        y->press_sal_temp[j][4] = frame_1st_char;                               // Pres's frame
        if ( y->status[frame_1st_char-1]==NEVER_RECEIVED || 
             y->status[frame_2nd_char-1]==NEVER_RECEIVED ) {
          // if one the characters in a non-received frame, assign 0 to pressure 
          y->press_sal_temp[j][0] = 0;  
        }

        // temperature
        frame_1st_char = (  (start+j*6)  / t->data_cycle ) + 1;
        frame_2nd_char = ( (start+j*6+1) / t->data_cycle ) + 1;
        y->press_sal_temp[j][5] = frame_1st_char;                               // Temp's frame
        if ( y->status[frame_1st_char-1]==NEVER_RECEIVED || 
             y->status[frame_2nd_char-1]==NEVER_RECEIVED ) {
          // if one the characters in a non-received frame, assign 0 to temperature 
          y->press_sal_temp[j][1] = 0;  
        }

        // salinity
        frame_1st_char = ( (start+j*6+2) / t->data_cycle ) + 1;
        frame_2nd_char = ( (start+j*6+3) / t->data_cycle ) + 1;
        y->press_sal_temp[j][6] = frame_1st_char;                               // Sal's frame
        if ( y->status[frame_1st_char-1]==NEVER_RECEIVED || 
             y->status[frame_2nd_char-1]==NEVER_RECEIVED ) {
          // if one the characters in a non-received frame, assign 0 to salinity
          y->press_sal_temp[j][2] = 0;  
        }

      }
      // following statement is to mark
      // that there are no more pres, temp, salinity levels
      //
      y->press_sal_temp[j][0] = -999;

      y->msg_blk_no *= actual_frames_per_message;
       if (debug==ON) {
         printf( "IN GRD 3: msg_blk_no * actual_frames_per_message is %d\n", y->msg_blk_no );
       }


      int end = start+y->profile_len*6;

      for(j=0;j<y->sst_len;j++)
      {
	
        y->press_sst_temp[j][1] = (256*x.data.d_fmt.data_message[end+j*3+0]+x.data.d_fmt.data_message[end+j*3+1]); //Temperature

        y->press_sst_temp[j][0] = (x.data.d_fmt.data_message[end+j*3+2]); //Pressure
        frame_1st_char = ( (end+j*3+2) / t->data_cycle ) + 1;
        frame_2nd_char = ( (end+j*3+3) / t->data_cycle ) + 1;
        y->press_sst_temp[j][2] = frame_1st_char;                         // Pres's frame


        if ( y->status[frame_1st_char-1]==NEVER_RECEIVED || 
             y->status[frame_2nd_char-1]==NEVER_RECEIVED ) {
          // if one the characters in a non-received frame, assign 0 to pressure
          y->press_sst_temp[j][0] = 0;  
        }

        y->press_sst_temp[j][3] = (  (end+j*3)  / t->data_cycle ) + 1;    // Temp's frame
      }
      // following statement is to mark
      // that there are no more near-surface pres, temp pairs
      //
      y->press_sst_temp[j][0] = -999;
  	}
//  }

  for (j=0; j<max_positions; j++)
  {
    y->latitude[j] = x.latitude[j];
    y->longitude[j] = x.longitude[j];
    y->position_jd[j] = x.jd[j];
    y->n_xmits[j] = x.n_xmits[j];
    y->sat_name[j] = x.sat_name[j];
    y->loc_class[j] = x.loc_class[j];
    strncpy(y->date_time[j],x.date_time[j],24);
  }


  if (trace_file != NULL)
    fprintf(trace_file," GET_RAW_DATA: finished (profile number = %d,number of meas = %d)\n", y->profile_no,k);

  if (debug==ON) {
/*    printf("IN GRD RESULT FROM SOT : blk_time =%d,  %04d_%05d_%03d.cnt  \n", blk_time,s->AOML_number,s->ARGOS_id,y->profile_no );
    printf("IN GRD RESULT FROM SOT  Date Converted = %s, %04d_%05d_%03d.cnt \n ",julday2ymd((long)((float)blk_time/24./3600.)),s->AOML_number,s->ARGOS_id,y->profile_no);
    printf("IN GRD RESULT FROM SOT  hh min ss = %s, %04d_%05d_%03d.cnt \n",timefrac2hms((double)(blk_time/24./3600.)),s->AOML_number,s->ARGOS_id,y->profile_no);

    printf("IN GRD END GRD: %04d_%05d_%03d.cnt  \n",s->AOML_number,s->ARGOS_id,y->profile_no);
  */  printf("IN GRD FROM RTF: t->data_length =%d IN  %04d_%05d_%03d.cnt \n ",t->data_length,s->AOML_number,s->ARGOS_id,y->profile_no);
    printf("IN GRD FROM RTF: t->data_count[t->data_length-1] =%d IN %04d_%05d_%03d.cnt\n ",t->data_count[t->data_length-1],s->AOML_number,s->ARGOS_id,y->profile_no);
    printf("IN GRD FROM RTF: t->data_byte[t->data_length-1] =%d IN %04d_%05d_%03d.cnt\n ", t->data_byte[t->data_length-1],s->AOML_number,s->ARGOS_id,y->profile_no);
    printf("IN GRD: y->sst_len = %d, y->profile_no =%d \n",y->sst_len,y->profile_no);
    fflush(stdout);
  }

  return(y->profile_no);
}

