/******************************************************************************
   Revision Date: 01-Jul-10 (ER - No need to cast to int when doing: jd = d_jd
                                  in function get_data)
                  19-Apr-10 (LL - when data is not received add 0 for p2,t2)
                  05-Apr-10 (LL - convert time since start 2 sec count to sec)
                  29-Mar-10 (LL - no column information if no data provided)
   				    23-Dec-09 (LL - revise back frames_per_message to 256)
                  09-Oct-08 (LL - 08-Dec-06 split functions)

   hex2cnt_tsall.c
   ==================

   Notice: the number of samples is always larger than the profile 
           length(from message number 1) by 1 for this float

*******************************************************************************/
#include "hex2cnt_tsall.h"

char   PRGNAME[] ="HEX2CNT_TS";

int total_cycles_count=0;

int frame1_problem_count=0;

int get_meta_file_name_hex2cnt (char *);
int read_meta_file (char *meta_name, struct meta_struct *meta_struct);

char buffer[1024];

int    aoml_id;
enum   flg {NO,YES} cal_flg;
enum   {F,T} cd_input = F;
int   msg_type;


int main (int argc, char *argv[])
{

  int debug=0; 

  char     *ptr_path_meta, *ptr_path_hex, *ptr_path_cnt,
           *ptr_path_log, *ptr_path_list, *ptr_path_table,
           pathname_input_filename[161]="",
           pathname_trace[161]="", pathname_error[161]="";

  char     input_filename[80], table_filename[80], 
           output_directory[256], 
           msg[11]="", 
           title[257]="";

  enum     {LAST=0,ALL} get_which_profile;

  struct   meta_struct s = {0,0,0,0,0,0,"UNKNOWN","UNKNOWN",0.0,0.0,0.0};
  struct   table_struct t={};
  unsigned long tim;
  int      ret_val = 0;

  printf("Entering Program: %s\n", PRGNAME);
  strcpy(title, "Program generates temperature, salinity, pressure counts\n");
  strcat(title, "for APEX Floats");
  printf("%s\n", title);

  // Input Paramenters
  // 
  if ( argc < 7 )
  {
    printf("Format: %s hex_infile nprofile cnt_outdir trace_outile error_outfile\n",
           PRGNAME);
    printf("\n");
    printf("hex_infile:    hexadecimal input file name\n");
    printf("nprofile:      1=get all profiles (default); 0=get last profile\n");
    printf("cnt_outdir:    output directory for generated count data file(s)\n");
    printf("trace_outile:  output filename for generated trace file\n");
    printf("error_outfile: output filename for generated error file\n");
    printf("list_outfile:  output filename for generated list file\n");
    return 1;
  }

  // Hex file name
  printf ("%s: Input HEX File Name - %s\n", PRGNAME, argv[1]);
  sprintf (input_filename,"%s",argv[1]);


  // Which profiles to process?
  if (atoi(argv[2]) >= 1)
    get_which_profile = ALL;
  else
    get_which_profile = LAST;


  // Output directory path for count file(s)
  if ( (ptr_path_cnt = getenv("CNTFILES_DIR")) == NULL)
  {
    printf("ERROR (%s): CNTFILES_DIR <-- no such environment variable\n", PRGNAME);
    tim = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&tim));
    ret_val = 1;
    return ret_val;
  }
  sprintf (output_directory,"%s/",argv[3]);
  printf ("%s: Output Directory - %s\n", PRGNAME, output_directory);

  // Trace file
  if ( (ptr_path_log = getenv("ARGOSLOG_DIR")) == NULL)
  {
    printf("ERROR (%s): ARGOSLOG_DIR <-- no such environment variable\n", PRGNAME);
    tim = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&tim));
    ret_val = 1;
    return ret_val;
  }
  sprintf (buffer,"%s",argv[4]);
  printf ("%s: Opening trace file %s\n", PRGNAME, buffer);
  if ((trace_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening trace file %s\n", PRGNAME, buffer);
    tim = time(NULL);
    printf ("%s: %s %s\n", PRGNAME, msg, ctime(&tim));
    ret_val = 1;
    return ret_val;
  }
  printf("%s: Trace file opened succesfully\n", PRGNAME);
  if (trace_file != NULL)
  {
    tim = time(NULL);
    fprintf (trace_file, "=> %s: start %s", PRGNAME, ctime(&tim));
  }

  // Error file
  // 
  sprintf (buffer,"%s",argv[5]);
  printf ("%s: Opening error file %s\n", PRGNAME, buffer);
  if ((error_file = fopen(buffer,"a+")) == NULL)
  {
    printf ("ERROR (%s): Opening error file %s\n", PRGNAME, buffer);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR (%s): error opening file %s\n", PRGNAME, buffer);
    tim = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
    printf ("%s: terminated %s\n", PRGNAME, ctime(&tim));
    ret_val = 1;
    return ret_val;
  }

  if (trace_file != NULL)
    fprintf (trace_file,"%s: Error file %s successfully opened\n", PRGNAME, buffer);

  if (error_file != NULL)
  {
    tim = time(NULL);
    fprintf (error_file,"=> %s: start %s", PRGNAME, ctime(&tim));
  }

  if ( (ptr_path_list = getenv("ARGOLIST_DIR")) == NULL)
  {
   if (trace_file != NULL)
     fprintf(trace_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   if (error_file != NULL)
     fprintf(error_file,"ERROR (%s): ARGOLIST_DIR <-- no such environment variable",
          PRGNAME);
   tim = time(NULL);
   if (trace_file != NULL)
     fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
   if (error_file != NULL)
      fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
   ret_val = 1;
   return ret_val;
  }

  // List file
  // 
  sprintf (buffer,"%s",argv[6]);
  if (trace_file != NULL)
     fprintf (trace_file,"%s: Opening list file %s\n", PRGNAME, buffer);

  if ((list_file = fopen(buffer,"w")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): Opening list file %s\n", PRGNAME, buffer);

    tim = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
    if (error_file != NULL)
       fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
    ret_val =1;
    return ret_val;
  }
  if (trace_file != NULL)
    fprintf (trace_file,"%s: List file %s successfully opened\n", PRGNAME, buffer);


  // Open Input Hex File
  // 
  if ( (ptr_path_hex = getenv("HEXFILES_DIR")) == NULL)
  {
    if (trace_file != NULL)
      fprintf (trace_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    if (error_file != NULL)
      fprintf (error_file,"ERROR (%s): HEXFILES_DIR <-- no such environment variable\n",
               PRGNAME);
    // Close Log files
    if (trace_file != NULL)                  
      fclose (trace_file);
    if (error_file != NULL)
      fclose (error_file);

    tim = time(NULL);
    if (trace_file != NULL)
      fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
    if (error_file != NULL)
       fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
    ret_val = 1;
    return ret_val;
  }

  sprintf(pathname_input_filename, "%s/%s", ptr_path_hex, input_filename);

  if ((input_file = fopen(pathname_input_filename,"r")) == NULL)
  {
     if (trace_file != NULL)
       fprintf (trace_file,"ERROR (%s): Input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     if (error_file != NULL)
       fprintf (error_file,"ERROR (%s): input file %s not opened\n",
                PRGNAME, pathname_input_filename);
     ret_val = 1;
     return ret_val;
  }
  else
  {
     if (trace_file != NULL)
       fprintf (trace_file,"%s: Input file %s successfully opened\n",
                PRGNAME, pathname_input_filename);
     if (error_file != NULL)
	        fprintf (error_file,"%s: Input file %s \n",
                PRGNAME, pathname_input_filename);

     // Meta file
     if (get_meta_file_name_hex2cnt(input_filename) != 0)
     {                                             
      // Failure: did not get meta filename
      if (error_file != NULL)
        fprintf (error_file,"ERROR (%s): Finding meta file name - returned name %s\n",PRGNAME, input_filename);
      if (trace_file != NULL)
        fprintf (trace_file,"ERROR (%s): Finding meta file name - returned name %s\n",PRGNAME, input_filename);
        ret_val = 1;
        return ret_val;
     }
     else
     {
       // Test meta file exists
       if ( (ptr_path_meta = getenv("METAFILES_DIR")) == NULL)
       {
         if (trace_file != NULL)
           fprintf (trace_file,"ERROR (%s): METAFILES_DIR <-- no such environment variable\n",PRGNAME);
         if (error_file != NULL)
           fprintf (error_file,"ERROR (%s): METAFILES_DIR <-- no such environment variable\n",PRGNAME);

         if (trace_file != NULL)                  // Close Log files
           fclose (trace_file);
         if (error_file != NULL)
           fclose (error_file);

         tim = time(NULL);
         if (trace_file != NULL)
           fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
         if (error_file != NULL)
           fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
         ret_val = 1;
         return ret_val;
       } // endif  ptr_path_meta

       sprintf(pathname_input_filename, "%s/%s", ptr_path_meta, input_filename);

       if (read_meta_file (pathname_input_filename, &s) != 0)
       {  // Failure: couldn't read meta_file
         if (trace_file != NULL)
           fprintf (trace_file, "ERROR (%s): Reading meta file %s\n",
                    PRGNAME, pathname_input_filename);
         if (error_file != NULL)
           fprintf (error_file,"ERROR (%s): Reading meta file %s\n",
                      PRGNAME, pathname_input_filename);
         ret_val = 1;
         return ret_val;
       }
       else                                      // Success: read meta_file
       {
         if (trace_file != NULL)
           fprintf (trace_file,"%s: Meta file %s successfully read\n",PRGNAME, pathname_input_filename);


         // Table file
         // 
     	  if (get_table_name (table_filename, &s) != 0)
	      {                                             // Failure: did not get table filename
	      	 if (error_file != NULL)
	          fprintf (error_file,
	                  "ERROR (%s): Finding table file name - returned name %s\n",
	                  PRGNAME, input_filename);
	       	 if (trace_file != NULL)
	         	fprintf (trace_file,
	                  "ERROR (%s): Finding table file name - returned name %s\n",
	                  PRGNAME, input_filename);
	        ret_val = 1;
	        return ret_val;
	      }
	      else
	      {
	                                               // Success: got table filename
	         	if ( (ptr_path_table = getenv("TABLE_DECODERS_DIR")) == NULL)
	         	{
	            	if (trace_file != NULL)
	              	fprintf (trace_file,"ERROR (%s): TABLEFILES_DIR <-- no such environment variable\n",PRGNAME);
	            	if (error_file != NULL)
	                fprintf (error_file,"ERROR (%s): TABLEFILES_DIR <-- no such environment variable\n",PRGNAME);

	            	if (trace_file != NULL)                  // Close Log files
	              	fclose (trace_file);
	            	if (error_file != NULL)
	              	fclose (error_file);

	            	tim = time(NULL);
	            	if (trace_file != NULL)
	              		fprintf (trace_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
	            	if (error_file != NULL)
	              		fprintf (error_file,"%s: terminated %s\n", PRGNAME, ctime(&tim));
	            	ret_val = 1;
	            	return ret_val;
	         	} // endif  ptr_path_table

	         	sprintf(pathname_input_filename, "%s/%s", ptr_path_table, table_filename);


	         	if (read_table_file (pathname_input_filename, &t) != 0)
	         	{                                         // Failure: couldn't read table_file
	            	if (trace_file != NULL)
	              		fprintf (trace_file, "ERROR (%s): Reading table file %s\n",
	                       PRGNAME, pathname_input_filename);
	            	if (error_file != NULL)
	              		fprintf (error_file, "ERROR (%s): Reading table file %s\n",
	                       PRGNAME, pathname_input_filename);
	            	ret_val = 9;
	            	return ret_val;
	         	}
	         	else                                      // Success: read table_file
	         	{
	            	if (trace_file != NULL)
	              		fprintf (trace_file,"%s: Table file %s successfully read\n",PRGNAME, pathname_input_filename);
//					printf ("\n%s: Table file %s successfully read\n", PRGNAME, pathname_input_filename);
	         	} // endif: read_table_file
     		} // endif: get_table_name


           // Read Data
           // 
           if (get_data(get_which_profile,&s,&t,output_directory) !=0 ) {
             ret_val = 1;
           }

        } // endif: read_meta_file
     } // endif: get_meta_file_name_hex2cnt
  } //endif: input_file

  printf("HEX filename=%s\n",argv[1]);
  printf("frame1 missing problem counts=%d\n",frame1_problem_count);
  printf("total cycles for this hex file=%d\n",total_cycles_count);
  if((frame1_problem_count>0) && (frame1_problem_count<total_cycles_count)){
    ret_val=2;
  }


// Write finish message to trace & error files
// -------------------------------------------
  tim = time(NULL);

  if ( ret_val == 0)
    strcpy(msg, "finished");
  else
    strcpy(msg, "terminated");

  printf ("%s: %s %s\n", PRGNAME, msg, ctime(&tim));
  if (trace_file != NULL) fprintf (trace_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&tim));
  if (error_file != NULL) fprintf (error_file,"=> %s: %s %s\n", PRGNAME, msg, ctime(&tim));

// Close all files
// ---------------

 if (input_file != NULL)
  fclose (input_file);

 if (trace_file != NULL)
  fclose (trace_file);


 if (error_file != NULL)
  fclose (error_file);

 if (list_file != NULL)
  fclose (list_file);

 return ret_val;
}
