//######################################################################################################################
// Revision Date : 11-Dec-2013 CA: Changed max_scans from 2000 to 4000 as 2796_39523 float's transmission repetition
//######################################################################################################################

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>


#define max_scans        4000    //Maximum scans for each profile
#define buff_size         256    //Maximum length of each scan read from *.str
#define max_positions     100    //Maximum number of positions per profile
#define min_frame_no        0    //Minimun frame number
//#define frames_per_message 256  //Number of frames per message,it is the maximum allowed number
#define frames_per_message 20  //Number of frames per message,it is the maximum allowed number
#define frames_per_message_test 3  //Number of frames per message,it is the maximum allowed number
#define message_size     2000

// next 3 constants refer to the status of a frame
//
#define GOOD 1             
#define BAD 4              
#define NEVER_RECEIVED 0
#define OFF 0
#define ON  1
#define FALSE 0
#define TRUE  1


// this is the fill value that qc, netcdf expect for 
// pres, temp and salinity.
// 
#define FILL_VALUE (const char *)("-999.0")

extern int frame1_problem_count;
extern int total_cycles_count;

double dfr97 (const int, const int, const int, const int, const int, const int, int *);
int most_likely_frame (char **, int, int, int, int*, int*,int);
long ymd2julday(int, int, int);
char *julday2ymd(long);
char *timefrac2hms(double);
char *strupr(char *);
char *strlor(char *);
long get_tim1(char* );
int round1(float );
int data_flag2(int, int *);
int start_of_transmission (char **, int, long *, int, int );

struct aoml_cals
{
  double TSLOPE;
  double TOFF;
  double PSLOPE;
  double POFF;
  double SSLOPE;
  double SOFF;
  double VSLOPE;
  double VOFF;
  double VACSLOPE;
  double VACOFF;
};


union u_cals
{
  struct aoml_cals s;
  double c[sizeof(struct aoml_cals)/sizeof(double)];
};

struct meta_struct
{
  int AOML_number;
  int WMO_id;
  int ARGOS_id;
  int wmo_instrument;
  int wmo_recorder;
  int exp_no;
  unsigned char float_type[40];
  unsigned char PI[60];
  float launch_time;
  float ARGOS_rep_rate;
  float cycle_time;
  float start_time;
  float delay_first_down_time;
  float up_time;
  float start_of_cycles;//added Start_time+delay_of_start+down_time
  union u_cals cals;
};

struct table_struct
{
  int supports_pres_activation; // could be 0(FALSE) or 1(TRUE)
  float days_between_cycles_0_and_1; // if 0, then it is the regular cycle length
  int test_flag;                // could be 1 or 2
  int test_cycle;               // could be 29 or 30
  int test_count[100];          // sum of test_byte[1..i-1]
  int test_byte[100];           // size in bytes of test parameter[i] 
                                // (1st element of test parameter line)
  char test_name[100][20];      // short name of parameter[i]
                                // (2nd element of test parameter line)
  char test_print[100][100];    // long name of parameter[i]
                                // (3rd element of test parameter line)
  char test_unit[100][100];     // measurement unit of parameter[i]
                                // (4th element of test parameter line)
  int test_length;              // number of test parameters

  int data_flag;                // could be 1 or 2
  int data_cycle;               //could be 29 or 30
  int data_count[200];
  int data_byte[200];
  char data_name[200][20];
  char data_print[200][100];
  char data_unit[100][100];
  int data_length;

};


// Raw data structure used in decoding Temperature/Pressure/Salinity (T/P/S)
// float data.  Part of RAW_DATA_RECORD described below 

struct test_fmt
{
  unsigned char test_message[message_size];
};

struct data_fmt
{
  //data message #1
  unsigned char data_message[message_size];
};


/* Equivalence of structure with linear array.  Linear array
 contains raw data byte minus first two bytes (check sum and
 frame number) of each frame.  Read T/P data format description */

union data_size
{
  unsigned char data_bytes_29[frames_per_message][29];//32-3=29
  unsigned char data_bytes_30[frames_per_message][30];//32-2=30
  struct data_fmt d_fmt;
  struct test_fmt t_fmt;
};

/* Intermediate structure containing T/P raw data and positions */
struct raw_data_record
{
  int jday;
  double jd[max_positions];
  float  latitude[max_positions];
  float  longitude[max_positions];
  char   date_time[max_positions][32];
  int    n_xmits[max_positions];
  char   sat_name[max_positions];
  char   loc_class[max_positions];
  union  data_size data;
};

/* Final structure containig T/P data and positions */
struct test_record
{
  int float_id;
  int profile_no;
  int jd;
  int status[frames_per_message];
  int msg_blk_no;
  long first_blk_time;
  int reference_time[6];
  time_t gmttime;
  int flag_blk_time;
  int msg_blk_id;


  //from test message
  int test_message[message_size];

  float  latitude[max_positions];
  float  longitude[max_positions];
  char   date_time[max_positions][32];
  int    n_xmits[max_positions];
  char   sat_name[max_positions];
  char   loc_class[max_positions];
  double position_jd[max_positions];

  char   hex_test_print[100][100];
  char   hex_test_time[100][100];
  int    flag_most_likely_frame;
};

struct output_record
{
  int float_id;
  int profile_no;
  int profile_len;
  int jd;
  int status[frames_per_message];
  int msg_blk_no;
  long first_blk_time;  // start of transmission
  time_t gmttime;
  int sst_len;
  int flag_blk_time;      // was first_blk_time calculated throu start_of_transmission.c method?

  //from data message
  int data_message[message_size];

  float  latitude[max_positions];
  float  longitude[max_positions];
  char   date_time[max_positions][32];
  int    n_xmits[max_positions];
  char   sat_name[max_positions];
  char   loc_class[max_positions];
  double position_jd[max_positions];

  int    press_sal_temp[15*frames_per_message][7]; //[0]=p, [1]=t, [2]=s, [3]=bin
                                                   //[4]=status of frame of p, [5]=status of frame of t
                                                   //[6]=status of frame of s
  int    press_sst_temp[15*frames_per_message][4]; //[0]=p, [1]=t
                                                   //[2]=status of frame of p, [3]=status of frame of t

  char   hex_data_print[100][100];
  char   hex_data_time[100][100];
  int    flag_most_likely_frame;
};

FILE *input_file, *out_file, *error_file, *trace_file, *list_file, *meta_file, *table_file;

extern char buffer[1024];   
extern char PRGNAME[10];


