/************************************************************************
  Revision Date: 13-Aug-00 (JZ)

  julday2ymd
  ==========

  Converts RAFOS JULIAN day (rjul) to day, month, year (dmy(1:3))
  RAFOS JULIAN day = julian day (ref. to 24 nov 4713BC) - 2440000

  translated from dates.c from GSO RAFOS processing code
  which was taken from Collected Algorithms from ACM, #199
           "astronomically correct"
  and converted to RAFOS JULIAN day   20.03.91 kst

  Usage:
    date = julday2ymd(julday)           - Where julday is the Rafos Julian
                                          day as a long integer and date is
                                          a string "year month day"

 ***********************************************************************/
#include <stdio.h>
#include <math.h>

char *julday2ymd(long rjul)
{
  char datestring[11]="";
  long julian;
  int iyr, month, iday;

  julian = rjul - 1721119 + 2440000;
  iyr = floor(( 4*julian - 1 )/ 146097);
  julian = 4*julian - 1 - 146097*iyr;
  iday = floor(julian/4);
  julian = floor(( 4*iday + 3 )/ 1461);
  iday = 4*iday + 3 - 1461*julian;
  iday = floor(( iday + 4 )/ 4);
  month = floor(( 5*iday - 3 )/ 153);
  iday = 5*iday - 3 - 153*month;
  iday = floor(( iday + 5 )/ 5);
  iyr = 100*iyr + julian;

  if (month < 10)
  {
    month += 3;
  }
  else if (month > 9)
  {
    month -= 9;
    iyr += 1;
  }

  sprintf(datestring, "%04d %02d %02d", iyr, month, iday);

  return datestring;
}
