 /************************************************************************
  Revision Date: 04-Dec-2012: CA - most_likely_frame has been modularized
                 04-Dec-2012: CA - data_size has been changed from 64 to 65
		                   to accomodate proper hex data comparisions
				   in the array
		 04-Dec-2012: CA - blk1char and blk2char arrays are implemented 
				   to accomodate hex_data with no Message numbers

  most_likely_frame.c
  ====================
  Finds the most likely scan for the APEX floats
  Returns number of most common scan except for test message
  Search for test message if frame_num = 0
  Test message detected if block number > frames per message and flag2 bits 6 or 7 set
  Returns -1 if error
  good_bad=1 -> use checksum; good_bad=0->ignore checksum
  
  For every frame number, the program first enters flag==0 code initially to get hold of the skip_bytes
  and copies the hex_data to a candidate_data for future comparisions
  If there are more than one messages for a frame number, then it enters the loop for flag!=0
  The program compares the incoming hex_data(which is replaced by skip bytes) with the candidate_hex_data 
  mentioned  above. If a match is found it increments the candidate_hex_count. Candidate_hex_count is 
  later compared with matches, if the count is greater than the initial matches count, then candidate_hex_count
  is assigned to matches and candidate_hex_index is assigned to the most_likely
  
  ID YEAR MON DAY HR MIN SEC LAT LON GOOD/BAD NNN S C FILEDATE 0000000000
  09335 1997 07 19 06 04 59 +00.000 +000.000 1 033 D 0 069515951 CA012000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFFFFFFFDFFFFFFFFFDF

 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "hex2cnt_tsall.h"

#define data_size 65
#define buffer_size 256
#define candidate_size 300


int max_element( int array[], int length )
{
	int i;
	int max = 0;
	for(i=0; i<length; i++)
	{
		if( array[i] > max )
		{
			max = array[i];
		}
	}

	return max;
}
/* Function to Initialize the variables*/
void initialize(int num_bytes[data_size][16],int *max_col, char candidate_hex_data[candidate_size][data_size], int *candidate_hex_count,int *candidate_hex_index)
{
	int string = 0;
	int hexValue = 0;
	
  	for( string=0; string<data_size; string++ )
  	{
  		for( hexValue=0; hexValue<16; hexValue++ )
		{
  		  num_bytes[string][hexValue] = 0;
		}
		max_col[string] = 0;

	}
	

	for(string=0;string<candidate_size;string++ )
	{
		strcpy(candidate_hex_data[string],"");		
		candidate_hex_count[string] = 0;
 	   candidate_hex_index[string] = 0;

	}
}

/* Function to get the count of a character that appears in a column of the profile */
void numBytes(int num_bytes[data_size][16],char *hex_data)
{

	int string;
	int hexValue;

	for( string=0; string<(data_size-1); string++ )
	{
		switch (hex_data[string])
		{
			case 'A' : hexValue = 10; break;
			case 'B' : hexValue = 11; break;
			case 'C' : hexValue = 12; break;
			case 'D' : hexValue = 13; break;
			case 'E' : hexValue = 14; break;
			case 'F' : hexValue = 15; break;
			default  : hexValue = (int)(hex_data[string]-48); break;
        }
                num_bytes[string][hexValue]++;
	}

}
/* Function to separate the skip bytes separated from Hexadecimal Data*/
//void getSkipBytes(char candidate_hex_data[candidate_size][data_size],char hex_data[data_size],int *skip_byte,int *flag,int *size_skip_byte,char tmp_hex[data_size],int candidate_hex_count[candidate_size],int candidate_hex_index[candidate_size],int *nScanCounter)

void getSkipBytes(char candidate_hex_data[candidate_size][data_size],char hex_data[data_size],int *skip_byte,int flag,int *size_skip_byte,char tmp_hex[data_size])
{
 int n;
	
	strcpy(candidate_hex_data[flag],hex_data);
	for( n=0; n<data_size; n ++ )
	{
		int position = skip_byte[n];
		//printf("Skip Byte Position = %d \n",position);
		if( position < data_size )
		{
      	         tmp_hex[*size_skip_byte] = candidate_hex_data[flag][position];
	 	 *size_skip_byte = *size_skip_byte + 1;
		}
		else
		{
			n=data_size+1; //spree up
	 	}
	}
	
}
/* Function called when there is no data match found for a fram number in the profile */
void allDifferent(int *max_col,int num_bytes[data_size][16],int *flag_mlf,char *create_buff,int *length,int *most_likely,int *first_frame,char **buffer,int frame_num)
{
			printf("IN MLF: Entered allDifferent Function \n");
		int flag_byte = 0;
		int string,hexValue;

		for( string=0; string<(data_size); string++ )
  		{
			max_col[string] = max_element( num_bytes[string], 16 );
			if ( max_col[string] > 1 )
			{
				flag_byte++;
			}
		}
		//printf("flag_byte = %d , data_size = %d \n " , flag_byte,data_size);
 		if( flag_byte == data_size )
		{
			*flag_mlf = 0;			
		}
		else
		{
			*flag_mlf = 1;
		}

		for( string=0; string<(data_size-1); string++ )
		{
			for( hexValue=0; hexValue<16; hexValue++ )
			{
				if( num_bytes[string][hexValue] == max_col[string] )
				{
					char create_char[1];

					switch (hexValue)
					{
					    case 10 : create_char[0] = 'A'; break;
					    case 11 : create_char[0] = 'B'; break;
					    case 12 : create_char[0] = 'C'; break;
					    case 13 : create_char[0] = 'D'; break;
					    case 14 : create_char[0] = 'E'; break;
					    case 15 : create_char[0] = 'F'; break;
					    default : create_char[0] = (char)(hexValue+48); break;
        			}	
        						
					create_buff[*length+string] = create_char[0];

					hexValue = 17; //speed up
				}
			}
		}
		
		*most_likely = *first_frame;
//		 printf("MOST LIKELY FRAME INDEX =%d \n",*most_likely);
//                 printf("IN MLF: create_buff[0] =%s \n",create_buff);
		strcpy( buffer[*most_likely], create_buff);

}
/* Function to get the most likely frame */
int most_likely_frame (char **buffer, int n_scans, int frame_num, int good_bad, int *skip_byte, int *flag_mlf,int test_flag)
{

  int debug=OFF;
 	int i,j,k,l;
 	int matches = 0;
 	int most_likely = -2;
	int first_frame = 0;
 	char tmp_buff[buffer_size];
 	char create_buff[buffer_size];
 	char tmp_hex[data_size];

 	char candidate_hex_data[candidate_size][data_size];
 	int  candidate_hex_count[candidate_size];
 	int  candidate_hex_index[candidate_size];
 	int num_bytes[data_size][16];
 	int s,t,m;
 	int string;
 	int flag = 0;
 	int length = 0;
 	int hexValue=0;
 	int n=0;
 	int size_skip_byte=0;

 	int aoml_id = 0;
 	int year = 0;
 	int month = 0;
 	int day = 0;
 	int hour = 0;
 	int minute = 0;
 	int second = 0;
 	float latitude = 0.0;
 	float longtitude = 0.0;
 	int good_bad_position = 0;
 	int nnn = 0;
 	char satellite[1];
 	int loc_class = 0;
 	int file_data = 0;
 	char hex_data[data_size];
 	char blk1char[n_scans];// variable to store character 1 of  blk number to retain if no message number found (t->test_flag==1)
 	char blk2char[n_scans];// variable to store character 2 of blk number to retain if no message number found (t->test_flag==1)
 	int max_col[data_size];
 	int skip_count = 0;
	int nScanCounter;

	//Change 1
	
	initialize(num_bytes,max_col,candidate_hex_data,candidate_hex_count,candidate_hex_index);
        
 	for( nScanCounter=0; nScanCounter<n_scans; nScanCounter++ )
 	{
		strcpy(hex_data,"");
		strcpy(tmp_buff,"");

  		strcpy(tmp_buff,buffer[nScanCounter]);
  		sscanf(tmp_buff,"%6d %04d %02d %02d %02d %02d %02d +%06f +%07f %d %03d %c %d %d %s",&aoml_id,&year,&month,&day,
  		&hour,&minute,&second,&latitude,&longtitude,&good_bad_position,&nnn,&satellite,&loc_class,&file_data,&hex_data);
 
 		if( strcmp(hex_data,"") == 0 )
 		{
			strcpy(hex_data,"0000000000000000000000000000000000000000000000000000000000000000");
		}
		//printf("hex_data just inside the nScanCounter,n_scans,good_bad,good_bad_position= %s,%d,%d,%d,%d \n",hex_data,nScanCounter,n_scans,good_bad,good_bad_position);

  		if( (good_bad_position != 0) && good_bad )
  		{
   			//printf("Skipping %s because good_bad_position = %d and good_bad = %d \n",hex_data,good_bad_position,good_bad);
			continue;       //Ignore if status byte = bad
		}

  		sscanf(&hex_data[2],"%2X",&k);
		//test_flag 1,2,3  means most_likely_frame() is called from get_raw_data_test.
		if((test_flag == 1))
		{
		  blk1char[nScanCounter] = hex_data[2];//saves the 1st character of the blk number of the hex_data
		  blk2char[nScanCounter] = hex_data[3];//saves the 2nd character of the blk number of the hex_data
		  hex_data[2] = '0'; // BLK number in hex_data is replaced with Message number 01 in case of
		  hex_data[3] = '1'; // no message number found (case where t->test_flag = 1) 
		  k=1;
		}



	       //printf(" k-1 %d and frame num %d \n",k-1,frame_num); 	

  		if( (k-1) != frame_num )	//Frame = 0 to 15 => most_common[0-14]
  		{
   			continue;
   			//printf("Skipping %s because I need %d but I got %d  \n",hex_data,frame_num,k-1);
		}
		else
		{
			/*Enters the below loop for the first message that the program encounters for a particular frame number*/
			if( flag == 0 )
			{
				//printf(" I am in flag equal to Zero \n");
			 //Change 2
			 getSkipBytes(candidate_hex_data,hex_data,skip_byte,flag,&size_skip_byte,tmp_hex);
			 candidate_hex_count[flag]++;
			 candidate_hex_index[flag] = nScanCounter;
			 flag++; 
				
		
	matches = 1;
			//Change 3
	    		numBytes(num_bytes,hex_data);	    		
			//CHANGE IT BACK TO THE BELOW CODE
		    	length = strlen(tmp_buff)-strlen(hex_data)-1;
	               /* printf("IN MLF strlen(tmp_buff) calculated= %d \n",strlen(tmp_buff));
	                printf("IN MLF strlen(hex_data) calculated= %d \n",strlen(hex_data));
	                printf("IN MLF length calculated= %d \n",length);*/
		    	//length = strlen(tmp_buff)-strlen(hex_data);
   	 		strcpy(create_buff,buffer[nScanCounter]);  //save the first candidate
    			first_frame = nScanCounter;                //save the first candidate location
			}
			/*Enters the below loop after the first message that the program encounters for a particular frame number*/
			else //if( flag != 0 )
			{
			
				//printf(" I am in flag not equal to Zero for frame num =%d \n",frame_num);
				int flagDifferent = 0;
				for(j=0;j<flag;j++)
				{
					for( n=0; n<size_skip_byte; n++ )
					{
						hex_data[ skip_byte[n] ] = tmp_hex[n];
					}
					//printf("Comparing the below 2 HEX DATA\n");
					if( strcmp(candidate_hex_data[j],hex_data) != 0 )
					{
						flagDifferent++;
					}
					else
					{
					
						candidate_hex_count[j]++;
						j = flag+1; //speed up
					}
				}//end of j for loop


			if( flagDifferent == flag )
			{
				//printf("In FLAG ==FLAG DIFFERENT \n");
				strcpy(candidate_hex_data[flag],hex_data);
				//printf("candidate_hex_data[%d]  = %s inf FLAG == FLAGDIFF \n",flag,candidate_hex_data[flag]);
				candidate_hex_count[flag]++;
				candidate_hex_index[flag] = nScanCounter;
				j = flag*10;
			}
			//Change 4
			
	    		numBytes(num_bytes,hex_data);
			if( j == flag*10 )
			{
				flag++;
			}
			} //if( flag == 0 )
		} // end (k-1) == frame_num
 	} // end for n_scans
	// printf("out of nscans \n");
   /* Compares candidate_hex_count and matches to get the most_likely frame and the number of matches*/
 	for(j=0;j<flag;j++)
	{
		if( candidate_hex_count[j] > matches )
		{
			matches = candidate_hex_count[j];
			most_likely = candidate_hex_index[j];
		}
	}
   /* Enters the following loop when all the data is different */
  	if( matches == 1 ) //all different
  	{ 		
  		
		//Change 5
  		allDifferent(max_col,num_bytes,flag_mlf,create_buff,&length,&most_likely,&first_frame,buffer,frame_num);
                //printf(" buffer[most_likely] = %s \n", buffer[most_likely]);	 	
                //printf(" buffer[most_likely][length+2] = %c \n", buffer[most_likely][length+2]);	 	
                //printf(" buffer[most_likely][length+3] = %c \n", buffer[most_likely][length+3]);	 
	
		if(test_flag ==1){
		buffer[most_likely][length+2] = blk1char[most_likely];
		buffer[most_likely][length+3] = blk2char[most_likely];
		}

		//printf("%d  entered allDifferent function \n",frame_num); 
		//printf(" CRC = %d and good_bad = %d \n",good_bad_position,good_bad );

	
	}// end of matches = 1
	else
	{
		
		*flag_mlf = 0;
		//printf(" %d did not enter allDifferent function \n",frame_num);
	//	printf(" CRC = %d and good_bad = %d \n",good_bad_position,good_bad );
	}

//printf("EXITing Most Likely Frame \n");
	//printf("IN MLF most_likely = %d \n",most_likely); 
 	return most_likely;

}
