/***********************************************************************************
    Revision Date: 13-Jun-2011 (ER: Added parameters DAYS_BETWEEN_CYCLES_0_AND_1 
                                    and SUPPORTS_PRESSURE_ACTIVATION.)
                   26-Nov-2010 (ER: improve indenting and add some comments)
                   01-Jul-2010 (CS: Method gets to be alone in a file of its own.)

    read_table_file
    ==================
    Input Parameters
      table_name: complete path of table_tsxx.txt file where the test and data msg
                  are described for a given float type.  Ex: table_ts20.txt
       
    Output Parameters
      table_struct: data structure with decoding table data.

    Returns and integer with this posible values:
      0: success
      1: failure 
      
************************************************************************************/  
#include "hex2cnt_tsall.h"

int read_table_file (char *table_name, struct table_struct *table_struct)
{

  int m=0;
  int n=0;
  int i=0;

  if (trace_file != NULL)
    fprintf(trace_file,"Opening table file %s...\n",table_name);
  
  if ((table_file = fopen(table_name,"r")) == NULL)
  {
    if (error_file != NULL)
      fprintf (error_file," ERROR opening table file %s\n",table_name);
    if (trace_file != NULL)
      fprintf (trace_file," ERROR opening table file %s\n",table_name);
    return 1;
  }
  else
  {
    if (trace_file != NULL)
    fprintf (trace_file," Opened table-file succesfully\n");
  }

  // Initializing 
  table_struct->supports_pres_activation = FALSE;  // if parameter not found, instrument type 
                                                   // doesn't support pressure activation
  table_struct->days_between_cycles_0_and_1 = 0;   // if parameter not found, days between 
                                                   // cycles 0 and 1 = regular cycle length
  while (fgets(buffer,sizeof(buffer),table_file) != NULL)
  {                     
    char *pch;

    if (strncmp(buffer,"SUPPORTS_PRESSURE_ACTIVATION",28)==0)
    {
      pch = strtok(buffer,",");
      pch = strtok(NULL,",");
      if ( strcmp(pch,"Y")==0 ) {
        table_struct->supports_pres_activation = TRUE;
      }
    }
                        
    if (strncmp(buffer,"DAYS_BETWEEN_CYCLES_0_AND_1",27)==0)
    {
      pch = strtok(buffer,",");
      pch = strtok(NULL,",");
      table_struct->days_between_cycles_0_and_1 = atof(pch);
    }

    if (strncmp(buffer,"TEST",4)==0)
    {
      char *pch3;
      pch3 = strtok(buffer,",");

      i = 0;
      while ( pch3 != NULL )
      {
        if( i==1 )
        {
          table_struct->test_flag = atoi(pch3);
        }

        if( i==2 )
        {
          table_struct->test_cycle = atoi(pch3);
        }

        pch3 = strtok(NULL,",");
        i++;
      }

      m = 0;
      while (fgets(buffer,sizeof(buffer),table_file) != NULL && strncmp(buffer,"DATA",4)!=0)
      {
        char *pch2;
        pch2 = strtok(buffer,",");

        n = 0;
        while ( pch2 != NULL )
        {
          if (n==0)
          {
            table_struct->test_byte[m] = atoi(pch2);
	    //printf("table_struct->test_byte[m] = %d \n",table_struct->test_byte[m]);
            if(m == 0)
            {
              table_struct->test_count[m] = 0;
            }
            else
            {
              table_struct->test_count[m] = table_struct->test_count[m-1] + table_struct->test_byte[m-1];
	    //printf("table_struct->test_count[m] = %d \n",table_struct->test_count[m]);
            }
          }
          if (n==1)
          {
            strcpy(table_struct->test_name[m],pch2);
	    //printf("table_struct->test_name[m] = %s \n",table_struct->test_name[m]);
          }
          if (n==2)
          {
            if(strcmp(pch2,"\n")==0) pch2 = "";
            strcpy(table_struct->test_print[m],pch2);
	   // printf("table_struct->test_print[m] = %s \n",table_struct->test_print[m]);
          }
          if (n==3)
          {
            if(strcmp(pch2,"\n")==0) pch2 = "";
            strcpy(table_struct->test_unit[m],pch2);
	    //printf("table_struct->test_unit[m] = %s \n",table_struct->test_unit[m]);
          }
          pch2 = strtok(NULL,",");
          n++;
        }
        m++;
      }
      table_struct->test_length = m;
    }

    if (strncmp(buffer,"DATA",4)==0)
    {
      char *pch5;
      pch5 = strtok(buffer,",");

      i = 0;
      while ( pch5 != NULL )
      {
        if( i==1 )
        {
          table_struct->data_flag = atoi(pch5);
        }
        if( i==2 )
        {
          table_struct->data_cycle = atoi(pch5);
        }

        pch5 = strtok(NULL,",");
        i++;
      }

      m = 0;
      while (fgets(buffer,sizeof(buffer),table_file) != NULL )
      {
        char *pch4;
        pch4 = strtok(buffer,",");

        n = 0;
        while ( pch4 != NULL )
        {
          if (n==0)
          {
            table_struct->data_byte[m] = atoi(pch4);
            if(m == 0)
            {
              table_struct->data_count[m] = 0;
            }
            else
            {
              table_struct->data_count[m] = table_struct->data_count[m-1] + table_struct->data_byte[m-1];
            }
          }
          if (n==1)
          {
            strcpy(table_struct->data_name[m],pch4);
          }
          if (n==2)
          {
            if(strcmp(pch4,"\n")==0) pch4 = "";
            strcpy(table_struct->data_print[m],pch4);
          }
          if (n==3)
          {
            if(strcmp(pch4,"\n")==0) pch4 = "";
            strcpy(table_struct->data_unit[m],pch4);
          }
          pch4 = strtok(NULL,",");
          n++;
        }
        m++;
      }
      table_struct->data_length = m;
    }

  }

  return 0;
}
