/************************************************************************
   Revision Date: 02-Jul-2010 (ER: Changes in layout; commenting
                                    of code in order to make it more readable.
                   01-Jul-2010 (CS: Method gets to be alone in a file of its own.)

  split_test_normal.c
  ===================
  split the first message into test message and profile data part based on 
  the julian date.  the fall will be between lines l and l+1 for which:
  julian date(l+1)-julian_date(l) = max( l(i+1)-l(i)) for i =1,nscans
  on the julian day, because in the test message, there are profile data(including header)
  we will merge the test data and profile0 data into profile cycle 0 cnt and phy file.

  Input parameters:
     buff: array of array of chars.  holds a profile. each line is a raw line from
           the hex file.  
     n_scans: number of lines of buff to be processed (size of the profile in the
              raw data in hex file)
     buff_test:  first tlines of buff
     buff_normal: last nlines of buff
     tlines:  number of lines in test message
     nlines:  number of lines in 'normal' or profile message

 ***********************************************************************/
#include <stdio.h>
#include <string.h>

double dfr97 (const int, const int, const int, const int, const int, const int, int *);

void split_test_normal(char **buff,int nscans,char **buff_test,int* tlines,char **buff_normal,int* nlines)
{

   int debug=0; //1 debug,0 no debug

   int i,j=0,k;
   int iy,imo,ida,ih,im,is,ndoy;
   float jd[nscans],jd_diff_max;


   if (debug>0) {
     printf( "<< BEGIN split_test_normal >>\n" );
   }

   //get julian day for each hex data line
   //
   for(i=0;i<nscans;i++){
     if(sscanf(buff[i],"%d %d %d %d %d %d %d", &k,&iy,&imo,&ida,&ih,&im,&is) != 7) continue;
     jd[i]=(float) dfr97(iy,imo,ida,ih,im,0,&ndoy);
   }

   //find the index where jd difference is the largest
   // jd_diff_max minimum is 6 hrs
   //
   jd_diff_max=6.0/24.0;
   for(i=0;i<nscans-1;i++){
     if(jd[i+1]-jd[i]>jd_diff_max) {
       jd_diff_max=jd[i+1]-jd[i];
       j=i;
     }
   }

   // LL: make sure there is 000 file
   if(j==0) 
     j=i;
 
   *tlines=j+1;
   *nlines=nscans-*tlines;

   // lines after tlines: normal profile
   //
   for(i=*tlines;i<nscans;i++){ strcpy(buff_normal[i-*tlines],buff[i]);}

   // lines before tlines: test profile
   //
   for(i=0;i<*tlines;i++){
      strcpy(buff_test[i],buff[i]);
   }

   if (debug>0) {
     printf( "<< END split_test_normal >>\n" );
     fflush(stdout);
   }

}
