/************************************************************************
  Revision Date: 28-Jun-00 (RS) 

  ymd2julday
  ===========
  Returns RAFOS JULIAN day (julday) converted from date (year, month, day)

  RAFOS JULIAN day = julian day (ref. to 24 nov 4713BC) - 2440000

          translated from dates.c from GSO RAFOS processing code
          which was taken from Collected Algorithms from ACM, #199
                   "astronomically correct"
          and converted to RAFOS JULIAN day   20.03.91 kst
          This routine is valid forever ....

   NOTE:  this RAFOS JULIAN day = old ARGOS day - 6718 + 1

  Signature
  --------     
             long ymd2julday(int year, int month, int day)

  Usage
  -----
            julday = ymd2julday (year, month, day)

            year  = 4 digit year representation
            month = month 
            day   day
  
  Example:
           unix> ymd2julday 2000 1 1
           unix> 11545
  or
           unix-sh>juday=`ymd2julday 2000 1 1`

 ***********************************************************************/
#include <stdio.h>
#include <math.h>

long ymd2julday(int year, int month, int day)
{
   int  ic, iy;
   long julday;

   if ( month > 2 )
     month = month - 3;
   else if ( month < 3 )
   {
     month = month + 9;
     year = year - 1;
   }

   ic = (int) floor( (year/100) );
   iy = year - 100*ic;

// printf("ic: %d iy: %d\n", ic, iy);
 
   julday =   (long) floor((146097*ic)/4) 
             + (long) floor((1461*iy)/4)
             + (long) floor((153*month+2)/5) 
             + day + 1721119;
   julday = julday - 2440000;
   return julday;
}
