#!/bin/sh
#######################################################################
# Revision Date: 31-Aug-10 (RS)
#
# any_oldfiles_in_dirphy.sh
# =========================
# Checks for 'phy' files in DATA_PHY_DIR that are older than a given 
# days threshold (set to 2 days).
#
# Definitions
# -----------
# DATA_PHY_DIR: ARGO_DIR/data/phy
#######################################################################
. $HOME/.argorc_sh
#######################################################################
#----------------------------------------------------------------------
#                          SendEmail
#----------------------------------------------------------------------
SendEmail()
{
  type=$1
  title=$2
  body=$3

  echo "=> SendEmail Enter (type=$type)"

  case $type in
   'FAILURE') title="$up_root_script_name - $type, $title";;
   'FOUND')   title="$up_root_script_name - $type, $title";;
  esac

  echo "   [Email Begin]"
  echo "     To: $EMAIL_OPERATOR"
  echo "     Subj: $title"
  echo "     Text:"
  echo "$body" | awk '{print "   " $0}'
  printf "     Sending ... "
  echo "$body" | mailx  -s"$title" $EMAIL_OPERATOR
  echo "OK"
  echo "   [Email End]"
  
  echo "=> SendEmail Exit"
  return
}
#---------------------------- Main ------------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M.%S'`
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

if [ $OPERATIONAL_SITE = "ARGOS" ]
then
    site_operations=CLS
else
    site_operations=AOML
fi
olderthandays=1   # modified more than 48 hours ago (2 days, when prefixed with '+'))

DATA_PHY_DIR=$ARGO_DIR/data/phy
LOG_DIR=$ARGO_DIR/argos/log
TMP_DIR=$ARGO_DIR/argos/tmp

logfile=$LOG_DIR/$root_script_name"_"$TODAY.log
error=$LOG_DIR/$root_script_name"_"$TODAY.error
anyfile=$TMP_DIR/$root_script_name"_"$TODAY.anyfile
exec 1> $logfile
exec 2> $error

echo "<--------------------------------------   Job Begin   -------------------------------->"
echo "=> Directories"
echo "   ARGO_DIR: $ARGO_DIR"
echo "   DATA_PHY_DIR: $DATA_PHY_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   TMP_DIR: $TMP_DIR"

echo "=> Logfiles"
echo "   logfile: $logfile"
echo "   error: $error"
echo "   anyfile: $anyfile"
# --------------------------------------------------------------------
#             Find files modified more than 'olderthandays'
# --------------------------------------------------------------------
printf "=> Any files older than 'olderthandays' in $DATA_PHY_DIR? "
cd $DATA_PHY_DIR
find . ! -name . -prune -name "*.phy" -mtime +$olderthandays 2> /dev/null > $anyfile 
if  [ -s $anyfile ]
then
  echo "YES"
  type=FOUND   
  ref="Files older the $olderthandays days at $site_operations"
  msg="  Files older that $olderthandays days found in: $DATA_PHY_DIR\n"
  msg=$msg"  See logfile:\n   $logfile\n   $anyfile" 
  if [ -s $error ]
  then
    ref=$ref" and Standard-Error reported"
    msg=$msg"\n   $error"
  else
    rm -f $error
  fi
  SendEmail $type "$ref" "$msg"
else
  echo "NO"
  rm -f $anyfile
  if [ -s $error ]
  then
    type=FAILURE
    ref="No 'phy' files found but Standard-error Reported at $site_operations"
    msg="  No 'phy' files found in: $DATA_PHY_DIR\n"
    msg=$msg"  See logfiles:\n   $logfile\n   $error"
    SendEmail $type "$ref" "$msg"
  else
    rm -f $error
  fi
fi #endif - anyfiles?

echo "<------------------------------------- Job Finished ---------------------------------->"
exit
