#!/bin/sh
########################################################################
# Revision Date: 14-May-14 (RS - renamed to 'argos_failure_or_done.sh') 
#                06-May-13 (RS - modified for 'done/aomlid' directories.
#                                Run it directly from 'crontab' )
#                07-Mar-02 (RS)
#
# argos_failure_or_done.sh
# ========================
# If a phy file is in both directories (done and failure) then delete
# the the one in 'failure'.
#
########################################################################
. $HOME/.argorc_sh 
########################################################################
TODAY=`date +'%Y.%m.%d_%H.%M'`

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

ARGOS_DIR=$ARGO_DIR/argos
SCRIPTS_DIR=$ARGOS_DIR/scripts
LOG_DIR=$ARGOS_DIR/log

PHYFILES_DIR=$ARGO_DIR/data/phy           # physical file (*.phy) directory
DONE_PHYFILES_DIR=$PHYFILES_DIR/done      # QC succesfuly processed PHY file(s)
FAILED_PHYFILES_DIR=$PHYFILES_DIR/failure # QC unsuccesfuly processed PHY file(s)

logfile=$LOG_DIR/$root_script_name.$TODAY"_log"
error=$LOG_DIR/$root_script_name.$TODAY"_error"
exec 1> $logfile
exec 2> $error

echo "=> Directories"
echo "   ARGOS_DIR: $ARGOS_DIR"
echo "   SCRIPTS_DIR: $SCRIPTS_DIR"
echo "   LOG_DIR: $LOG_DIR"

echo "=> Logfiles"
echo "   logfile: $logfile"
echo "   error: $error"

echo "=> Data Directories"
echo "   PHYFILES_DIR: $PHYFILES_DIR"
echo "   DONE_PHYFILES_DIR: $DONE_PHYFILES_DIR"
echo "   FAILED_PHYFILES_DIR: $FAILED_PHYFILES_DIR"

echo "=> Loop for files in $FAILED_PHYFILES_DIR"
for pathname in `ls $FAILED_PHYFILES_DIR/*.phy 2> /dev/null`
do
  filename=`basename $pathname`
  aomlid=`echo $filename | awk -F'_' '{print $1}`
  printf "   Does $pathname exists in $DONE_PHYFILES_DIR/$aomlid? "
  if [ -f $DONE_PHYFILES_DIR/$aomlid/$filename ]
  then
    printf "Yes -> Remove - "
    rm -f $pathname
    echo "OK"
  else
    echo "No"
  fi
done

if [  -s $error ]
then
  echo "=> $error not Empy"
else
  rm -f $error
fi

echo "=> Finish"

exit
