#!/bin/sh
################################################################################################
# Revision Date: 22-May-13 (RS - 'aomlid' directories are not created under 'failure' directory) 
#                22-Oct-12( RS - code changes related to the creation of the 'aomlid' directory 
#                                for each float under:
#                                  /a1/ARGO/argos/data/cnt/done argos/data/cnt/done_old
#                                  /a1/ARGO/argos/data/cnt/failure
#                                  /a1/ARGO/data/phy/done data/phy/done_old
#                                  /a1/ARGO/data/phy/failure) 
#                22-Jun-11 (ER   Changes due to redesign of DiffCnt program)
#                13-Apr-09 (RS,LL - uses table driven decoder))
#                11-DEC-08 (RS - update DiffCnt exit-status definitions)
#                12-Sep-07 (RS - Trap WARNINGS messages in the profile 'trace' file and write 
#                                to 'warning.[timestamp]_log' for email reporting)
#                09-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                10-Feb-06 (XX - reorder the DIFF_JAVA_STATUS_TABLE before printing to log file)
#                02-Feb-06 (CS - added echo for $hexfiles2process)
#                06-Dec-05 (RS - reduce prv2hex processing time for sorting the float hex 
#                                files by using hex files in HEXFILES_RECENT_DIR)
#                01-Mar-05 (RS - introduce MANUAL_METAFILE_LIST, used to copy meta file
#                                names to ARGO_DIR/manualprocesses/ftp/tmp for ftp2argos.sh) 
#                10-Jan-05 (RS - report float with data before launch time)
#                20-Aug-04 (RS - set total to zero before hex loop)
#                29-Aug-03 (RS - corrections for the display or exit status (Success
#                                column) from hex2cnt_[float].c in the SUMMUARY_HEX2CNT
#                                table)
#                27-Aug-03 (RS - modifications to deal with not enough data
#                                to create a profile, Exit Status=2 from
#                                hex2cnt_[float].c)
#                26-Aug-03 (RS - fixed MANUAL_PHY_SUCCESS_LIST file extension)
#                15-Aug-03 (RS - save success lists for 'hex', 'cnt' and'phy' only
#                                for manual process. Lists will be in located in 
#                                ARGO_MANUALPROCESS_DIR with timestamp)
#                05-Aug-03 (RS - remove lines for debugging)
#                04-Aug-03 (RS - set global diff log files variables)
#                31-Jul-03 (XX - test new hex2cnt.sh file from manualprocess directory)
#                16-Jul-03 (RS - set global variables & streamline code & implement 
#                                all valid parameter options (all,argos,qc,prv,hex) )
#                10-Jun-03 (XX - modified it to process cdrom prv and hex files
#                                besides the original processes for prv or hex) 
#                15-May-03 (RS - improve mail subject)
#                30-Apr-03 (RS - 4 digit for profile number & 10 for trasmission id)
#                02-May-02 (RS - add remove_ghosts.sh)
#                02-Nov-01 (RS - export prv_input_file_names)
#                26-Jul-01 (RS - subtotals and totals for "Existence Status"
#                19-Jul-01 (RS - back to 5 digits)
#                10-Jul-01 (RS - argos id expansion)
#                26-Jun-02 (RS - correction to Exit Code 5)
#                01-May-01 (RS - change 'Float Type' to 'Instrument Type')
#                15-Dec-00 (RS - corrections to email)
#                20-Nov-00 (RS - fix for hex2cnt table)
#                08-Nov-00 (RS - prv list too long, use xarg)
#                20-Sep-00 (RS - new meta data)
#                27-Aug-00 (RS - typo correction)
#                25-Aug-00 (RS - reformat output summaries)
#                18-Aug-00 (RS - set exit status value)
#                04-Aug-00 (RS - updates for QC interface)
#                19-Jul-00 (RS - delete debugging echo line)
#                10-Jul-00 (RS - corrections to headings)
#                29-Jun-00 (RS - detect profile problem (today-first_date > down time)
#                16-Jun-00 (RS - created send_email.sh & 
#                                introduced NEW_OR_CHANGED_CNTPROFILE to
#                                exit script when set to 'no')
#                24-Feb-00 (RS)
#
# argos_process.sh
# ================
# Processes Argos data by executing the followinw scripts: 
#
#     prv2hex.sh  
#     hex2cnt.sh
#     cnt2phy.sh
#     report_metadata_stderror.sh: reports mestadata and stderror  
#     phy_complete.sh: PHY profiles completion report
#
# Calling Sequence Options:
# ------------------------
#        argos_process.sh [mode] {[prv_list_file][hex_list_file]}  
#
# where mode is: all, argos, qc, prv or hex
#                if not specified it is set to 'all'
#
# Examples:
#   unix> argo_process.sh all 
#   unix> argo_process.sh argos 
#   unix> argo_process.sh qc 
#   unix> argo_process.sh prv prv_list_file
#   unix> argo_process.sh hex hex_list_file
#
# Note: inside prv_list_file, the prv filenames should  contain fullpath, 
# while for hex_list_file, the hex filenames should not contain fullpath,
# for prv_list_file, argo_manual .sh will move prv files to $PRVFILES_DIR
# for hex_list_file, argos_process.sh assume hex files are already located
# in HEX_DIR and process it, no need to move files.
#
# Exit status values
# ------------------
# 0, success
# 1, no PRV file to process
# 2, no PRV profile process successfully by C-program (prv2hex.x)  
# 3, no HEX profile created by C-program (prv2hex.x)
# 4, no CNT/PHY profile created by C-program (hex2cnt_?.x/cnt2phy_?); ? -> float type    
# 5, no new or updated CNT profiles created    
################################################################################################
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  ExitCodeTable
#  =============
#  Creates a table with exit code, reason and action for each process
#
#  Usage: ExitCodeTable
# --------------------------------------------------------------------
ExitCodeTable()
{
  processexit_table=$ARGOSTMP_DIR/argos_process_processexit.$TODAY"_"table
  cat > $processexit_table << EOF
Code              Reason                                                        Action
 0  /Process finished successfully                                     /Continue with ARGO
 1  /No PRV file to process                                            /Exit ARGO, see log-files
 2  /No PRV profile was processed successfully by C-program: prv2hex.x /Exit ARGO, see log-files
 3  /No HEX profile created by C-program (prv2hex)                     /Exit ARGO, see log-files
 4  /No CNT or PHY profile created by C-program (hex2cnt or cnt2phy)   /Exit ARGO, see log-files
 5  /No new or updated CNT profile created                             /Exit ARGO, see log-files
EOF
 return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  DelExitCodeTable
#  =================
#  Deletes 'Exit Code Table'
#
#  Usage: DelExitCodeTable
# --------------------------------------------------------------------
DelExitCodeTable()
{
   rm -f $processexit_table
   return
}
# --------------------------------------------------------------------
#  Revision Date: 29-Nov-00 (RS)
#
#  MatchExitCode
#  =============
#  Looks up the exit code. If code is matched, it prints the reason
#  and action else prints a no match message.
#
#  Usage: MatchExitCode code
# --------------------------------------------------------------------
MatchExitCode()
{
  code=$1                                      
  grep "$code " $processexit_table > /dev/null     # 0, match; !- 0 otherwise
  if [ "$?" = "0" ]
  then
    reason=`grep "$code " $processexit_table | cut -d"/"  -f2`
    action=`grep "$code " $processexit_table | cut -d"/"  -f3`
    echo "\tExitCode: $code => $reason"
    echo "\tAction: $action"
  else
   echo "Exit Code not found in: $processexit_table"
  fi
  return
}
#-------------------------------- Main  --------------------------------
FAILURE_METADATA_DIR=$ARGOSTMP_DIR
TABLE_DECODERS_DIR=$ARGOSPRG_DIR/ts_all
export TABLE_DECODERS_DIR

mode=$1

case $mode in
  prv) manual=1;;
  hex) manual=1;;
    *) manual=0;;
esac

if [ $manual -eq 1 ]
then
  logdir=$ARGO_MANUALPROCESS_LOG_DIR
else
  logdir=$ARGOSLOG_DIR
fi

diffcnt_log=$logdir/diffcnt.$TODAY"_log"
DIFF_JAVA_STATUS_TABLE=$logdir/DiffStatus.$TODAY"_"table
export diffcnt_log DIFF_JAVA_STATUS_TABLE
touch $DIFF_JAVA_STATUS_TABLE
  
if [ "$mode" = "all" -o "$mode" = "argos"  -o  "$mode" = "prv" ]
then
  nprv_failures_file=$ARGOSTMP_DIR/nprv_failures.$TODAY
  echo 0 > $nprv_failures_file
  export nprv_failures_file 
fi

head_zone=`date +'%Z`
HEAD_DATE_TIME="\t\t\t +++++ $TODAY_DATE $TODAY_TIME $head_zone +++++"

export HEAD_DATE_TIME

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}`

head_zone=`date +'%Z`
HEAD_DATE_TIME="\t\t\t +++++ $TODAY_DATE $TODAY_TIME $head_zone +++++"

export HEAD_DATE_TIME

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

# For reports
# -----------
SUMMARY_PRV2HEX=$ARGOSLOG_DIR/prv2hex.$TODAY"_"summary
SUMMARY_HEX2CNT=$ARGOSLOG_DIR/hex2cnt.$TODAY"_"summary
SUMMARY_HEX2CNT_1=$ARGOSLOG_DIR/hex2cnt_1.$TODAY"_"summary
SUMMARY_HEX2CNT_2=$ARGOSLOG_DIR/hex2cnt_2.$TODAY"_"summary
SUMMARY_CNT2PHY=$ARGOSLOG_DIR/cnt2phy.$TODAY"_"summary
SUMMARY_PHY_COMPLETE=$ARGOSLOG_DIR/phy_complete.$TODAY"_"summary
SUMMARY_REPORT=$ARGOSLOG_DIR/$root_script_name.$TODAY"_"report  # global: summary report

export SUMMARY_REPORT SUMMARY_PRV2HEX SUMMARY_HEX2CNT_1 SUMMARY_HEX2CNT_2
export SUMMARY_CNT2PHY SUMMARY_PHY_COMPLETE

# For filtered email
# ------------------
SUMMARY_PRV2HEX_EMAIL=$ARGOSLOG_DIR/prv2hex_email.$TODAY"_"summary
SUMMARY_HEX2CNT_EMAIL=$ARGOSLOG_DIR/hex2cnt_email.$TODAY"_"summary
SUMMARY_HEX2CNT_1_EMAIL=$ARGOSLOG_DIR/hex2cnt_1_email.$TODAY"_"summary
SUMMARY_HEX2CNT_2_EMAIL=$ARGOSLOG_DIR/hex2cnt_2_email.$TODAY"_"summary
SUMMARY_CNT2PHY_EMAIL=$ARGOSLOG_DIR/cnt2phy_email.$TODAY"_"summary
SUMMARY_PHY_EMAIL=$ARGOSLOG_DIR/phy_email.$TODAY"_"summary
EMAIL_MESSAGE=$EMAILFILES_DIR/$root_script_name.$TODAY"_"email  # global: argos_process 

export EMAIL_MESSAGE SUMMARY_PRV2HEX_EMAIL SUMMARY_HEX2CNT_1_EMAIL SUMMARY_HEX2CNT_2_EMAIL
export SUMMARY_CNT2PHY_EMAIL SUMMARY_PHY_EMAIL

total_new_entry=0                # Cummulative sums for 'Existence Status Report'
total_differ_entry=0             # in reference to 'SUMMARY_HEX2CNT_2'
total_same_entry=0
total_problem_entry=0

success_cnt=0
failure_cnt=0

success_cnt_list=$ARGOSLIST_SUCCESS_DIR/cnt2phy.$TODAY"_"success
failure_cnt_list=$ARGOSLIST_FAILURE_DIR/cnt2phy.$TODAY"_"fail
phylist_outfile=$ARGOSLIST_SUCCESS_DIR/cnt2phy.$TODAY"_"list
totals_existance=$ARGOSLIST_SUCCESS_DIR/totals_existance.$TODAY
touch $totals_existance

export success_cnt failure_cnt success_cnt_list failure_cnt_list
export phylist_outfile totals_existance

floatfail_trace_error_list=$ARGOSLOG_DIR/float_failure.$TODAY"_"trace"_"error.list
argos_process_stderror=$ARGOSLOG_DIR/$root_script_name.$TODAY.stderror # Error Logfile
hex2cnt_stderror=$ARGOSLOG_DIR/hex2cnt.$TODAY"_"stderror
cnt2phy_stderror=$ARGOSLOG_DIR/cnt2phy.$TODAY"_"stderror
phy_complete_stderror=$ARGOSLOG_DIR/phy_complete.$TODAY"_"stderror

no_metadata_list=$FAILURE_METADATA_DIR/no_metadata.$TODAY"_"list
noinfo_in_metadata_list=$FAILURE_METADATA_DIR/noinfo_in_metadata.$TODAY"_"list
remove_ghosts_msg=$ARGOSLOG_DIR/remove_ghosts.$TODAY"_"emailmsg
withdata_before_launchtime=$ARGOSLOG_DIR/withdata_before_launchtime.$TODAY"_"emailmsg
warning_log=$ARGOSLOG_DIR/warning.$TODAY"_"log

touch $withdata_before_launchtime

export argos_process_stderror
export floatfail_trace_error_list
export hex2cnt_stderror cnt2phy_stderror phy_complete_stderror
export no_metadata_list noinfo_in_metadata_list
export remove_ghosts_msg withdata_before_launchtime warning_log

search_string_in_trace="WARNING, data before launch time"
export search_string_in_trace
#----------------------------- end local init ------------------------------

if [ -f $argos_process_stderror ]
then
  rm -f $argos_process_stderror
fi
exec 2>>$argos_process_stderror
  
now=`date +'%h %d, %Y %T %p'`
echo "<+++++++++++++++++++ START  $script_name: $now +++++++++++++++++>"
echo "=> mode=$mode"

ExitCodeTable                  # load Exit Code Table

# -------------------------------------------------------------------
#                   Any prv files (prv*) ?
#                   yes, continue execution
#                   no,  Email summary report and Exit
# -------------------------------------------------------------------
if [ "$mode" = "all" -o "$mode" = "argos"  -o  "$mode" = "prv" ]
then
  prv_input_file_names=$ARGOSTMP_DIR/prv_input_file_names.$TODAY  # use in 'prv2hex.sh'
  export prv_input_file_names

  ls $PRVFILES_DIR/ | grep prv | xargs -l echo $PRVFILES_DIR/ | sed 's/ //' \
              >  $prv_input_file_names 

  if [ ! -s "$prv_input_file_names"  ]
  then
    ret_val=1
    subj="ARGOS_PROCESS - PRV2HEX - No Files to Process"
    msg=" NO PRV Files to Process" 
    if [ -s $argos_process_stderror ]
    then
      echo "For more information see: $argos_process_stderror" >> $msg
    fi
    echo $msg
    echo $msg > $EMAIL_MESSAGE

    echo "[EMAIL Begin]"
    echo "  To: $EMAIL_OPERATOR"
    echo "  Subject: $subj"
    echo "  Text:"
    cat $EMAIL_MESSAGE | awk '{print "  " $0}'
    printf "\nSending Email ... "
    mailx -s "$subj" $EMAIL_OPERATOR < $EMAIL_MESSAGE
    echo "OK"
    echo "[EMAIL End]"

    rm -f $prv_input_file_names
    MatchExitCode $ret_val
    DelExitCodeTable

    echo "<+++++++++++++++++++ Terminated $script_name: $now +++++++++++++++++>" 
    exit $ret_val  # no PRV file to process 
  fi

# -------------------------------------------------------------------
#               Execute Shell Script to Process PRV Files  
#                              prv2hex.sh
# -------------------------------------------------------------------
  echo "\nPRV file(s) to be processed by PRV2HEX located in $PRVFILES_DIR are: "
  cat $prv_input_file_names

  $ARGOSSCRIPT_DIR/prv2hex.sh                             # Execute Script prv2hex.sh

  nprv_success="$?" 
  echo "$root_script_name: Number of PRV Files Processed Successfully: $nprv_success"

  if [ $nprv_success -eq 0 ]    # Are all failures? 
  then                          #   yes, print summary report
    ret_val=2
    subj="ARGOS_PROCESS - PRV2HEX - No PRV file proccessed successfully -> Output list is EMPTY"
    cat $SUMMARY_PRV2HEX >> $SUMMARY_REPORT
    rm -f $SUMMARY_PRV2HEX
    msg="No PRV profile proccessed successfully, see log-files:"
    echo "\t$SUMMARY_REPORT" >> $EMAIL_MESSAGE
    msg=$msg"\n\t$log_file"
    msg=$msg"\n\t$ARGOSLOG_DIR/prv2hex.$TODAY"_trace
    msg=$msg"\n\t$ARGOSLOG_DIR/prv2hex.$TODAY"_error
    msg=$msg"\n\t$SUMMARY_PRV2HEX"
    echo $msg                >  $EMAIL_MESSAGE

    echo "[EMAIL Begin]"
    echo "  To: $EMAIL_OPERATOR"
    echo "  Subject: $subj"
    echo "  Text:"
    cat $EMAIL_MESSAGE | awk '{print "  " $0}'
    printf "\nSending Email ... "
    mailx -s "$subj" $EMAIL_OPERATOR < $EMAIL_MESSAGE
    echo "OK"
    echo "[EMAIL End]"

    rm -f $prv_input_file_names
    MatchExitCode $ret_val
    DelExitCodeTable

    now=`date +'%h %d, %Y %T %p'`
    echo "<+++++++++++++++++++ Terminated $script_name: $now +++++++++++++++++>" 
    exit $ret_val   # no PRV profile process successfully by C-program (prv2hex.x)
  fi
  rm -f $prv_input_file_names

# -------------------------------------------------------------------
#             Any prv list files (prv2hex*.*_list) ?
#                   yes, continue execution
#                   no,  Email summary report and Exit
# -------------------------------------------------------------------
  filter=$ARGOSLIST_SUCCESS_DIR/prv2hex*.*_list
  prvlist_outfile=$ARGOSTMP_DIR/prvlist_outfile
  ls $filter > $prvlist_outfile

  if [ ! -s $prvlist_outfile ]                       # is it empty?
  then                                               #  yes
    ret_val=3
    echo $msg                 > $EMAIL_MESSAGE
    subj="ARGOS_PROCESS - No PRV2HEX List File to Process"
    cat $SUMMARY_PRV2HEX >> $SUMMARY_REPORT
    rm -f $SUMMARY_PRV2HEX
    msg="The prv2hex output list does not exists , see log-files:"
    echo "\t$SUMMARY_REPORT" >> $SUMMARY_REPORT 
    msg=$msg"\n\t$log_file"
    msg=$msg"\n\t$ARGOSLOG_DIR/prv2hex.$TODAY"_trace
    msg=$msg"\n\t$ARGOSLOG_DIR/prv2hex.$TODAY"_error
    echo $msg                 > $EMAIL_MESSAGE

    echo "[EMAIL Begin]"
    echo "To: $EMAIL_OPERATOR"
    echo "Subject: $subj"
    echo "Text:"
    cat $EMAIL_MESSAGE | awk '{print "  " $0"}'
    printf "\nSending Email ..."
    mailx -s "$subj" $EMAIL_OPERATOR < $EMAIL_MESSAGE
    echo "OK"
    echo "[EMAIL End]"

    rm -f $prvlist_outfile                           # remove temporary file 
    MatchExitCode $ret_val
    DelExitCodeTable
 
    now=`date +'%h %d, %Y %T %p'`
    echo "<+++++++++++++++++++ Terminated $script_name: $now +++++++++++++++++>" 
    exit $ret_val  # no HEX profile created by C-program (prv2hex.x)                   
  fi #endif - prvlist_outfile

  rm -f $prvlist_outfile                             # remove temporary file 

  hexfiles2process=$ARGOSTMP_DIR/hexfiles2process.$TODAY
  cat $filter > $hexfiles2process 
fi  #end of if [ "$mode" = "all" -o "mode" = "argos"  -o  "mode" = "prv" ]


if [ "$mode" = "hex" ]
then                       # $2 contains only the name of the hex files to be processed, 
  hexfiles2process=$2      
fi

echo "hex file list is: $hexfiles2process"

sort -u $hexfiles2process -o $hexfiles2process     # eliminate possible duplicate(s)

echo "HEX file(s) to be processed by HEX2CNT located in $HEXFILES_DIR are: "
cat $hexfiles2process 

# -------------------------------------------------------------------
#             Process Float hex files in prv2hex_expno.*_list
# -------------------------------------------------------------------
failure_hex=0
success_hex=0
success_hex_list=$ARGOSTMP_DIR/hex2cnt.$TODAY"_"success
failure_hex_list=$FAILURE_LIST_DIR/hex2cnt.$TODAY"_"fail

export success_hex_list

# Parent process environmental variable can't no be changed in child process 
# --------------------------------------------------------------------------
new_or_changed_cntprof_file=$ARGOSTMP_DIR/new_or_changed_cntprof_file.$TODAY
sum_success_cnt_file=$ARGOSTMP_DIR/sum_success_cnt.$TODAY
sum_failure_cnt_file=$ARGOSTMP_DIR/sum_failure_cnt.$TODAY

export new_or_changed_cntprof_file
export sum_success_cnt_file
export sum_failure_cnt_file

echo "0"  > $sum_success_cnt_file
echo "0"  > $sum_failure_cnt_file
echo "no" > $new_or_changed_cntprof_file

# Heading for HEX2CNT Summary Report
# ----------------------------------
head="\n\t+++++ "
head=$head"Summary Report of Argos HEX Files Processed by HEX2CNT Script"
head=$head" +++++"
head=$head"\n$HEAD_DATE_TIME"
head=$head"\n\n\tLocation: $HEXFILES_DIR\n"
echo $head >> $SUMMARY_HEX2CNT_1
printf '\tHEX File          Success   Failure   Inst. Type   Metafile\n' >> $SUMMARY_HEX2CNT_1
echo $head >> $SUMMARY_HEX2CNT_1_EMAIL
printf '\tHEX File          Success   Failure   Inst. Type   Metafile\n' >> $SUMMARY_HEX2CNT_1_EMAIL

head="\n\n\t                   Existence Status for Float Profiles\n"
printf "$head" >> $SUMMARY_HEX2CNT_2
printf "\tFloat File         New       Changed   Unchanged   Problem\n" >> $SUMMARY_HEX2CNT_2
printf "$head" >> $SUMMARY_HEX2CNT_2_EMAIL
printf "\tFloat File         New       Changed   Unchanged   Problem\n" >> $SUMMARY_HEX2CNT_2_EMAIL

# Heading for CNT2PHY Summary Report
# ----------------------------------
head="\n\t+++++ "
head=$head"Summary Report of Argos CNT Files Procesed by CNT2PHY Script"
head=$head" +++++"
head=$head"\n$HEAD_DATE_TIME"
head=$head"\n\n\tLocation: $CNTFILES_DIR"
echo $head >> $SUMMARY_CNT2PHY
printf '\n\n\tCNT File                Success   Failure   Inst. Type\n' >> $SUMMARY_CNT2PHY
echo $head >> $SUMMARY_CNT2PHY_EMAIL
printf '\n\n\tCNT File                Success   Failure   Inst. Type\n' >> $SUMMARY_CNT2PHY_EMAIL

# -------------------------------------------------------------------
#           Loop over HEX Files to Execute Shell Scripts:
#                           hex2cnt.sh 
#                           cnt2phy.sh
#
# Note - executes remove_ghosts.sh before hex2cnt.sh 
# -------------------------------------------------------------------
echo $total_new_entry $total_differ_entry \
           $total_same_entry $total_problem_entry > $totals_existance

for hex_file in `cat  $hexfiles2process` 
do 
  now=`date +'%h %d, %Y %T %p'`
  float_id=`echo $hex_file | awk -F"." '{print $1}'` 

  echo "<------------------- Begin HEX $float_id $now --------------------->"
  trace_outfile=$ARGOSLOG_DIR/$float_id.$TODAY"_"trace   # one per float 
  error_outfile=$ARGOSLOG_DIR/$float_id.$TODAY"_"error   # one per float 
  export trace_outfile
  export error_outfile

  meta_file=$METAFILES_DIR/$float_id.meta
    
  if [ -f $meta_file ]
  then                                              # Success: meta file exists
    if [ $manual -eq 1 ]                            #   Only for manual process (prv or hex) 
    then
      echo $float_id.meta >> $MANUAL_METAFILE_LIST
    fi

    up_float_type=`grep -i "instrument type   " $meta_file | \
                   awk '{print $3}' | awk -F"_" '{print $2}' `
    if [ "$?" = "0" ] 
    then                                            # Success: float_type found
      $ARGOSSCRIPT_DIR/remove_ghosts.sh $hex_file $remove_ghosts_msg 

      float_type=`echo $up_float_type | tr "[:upper:]" "[:lower:]"`
      table_name=table_$float_type.txt
      export table_name

      prgname=hex2cnt_$float_type 

      if [ -f $HEXFILES_HISTORICAL_DIR/$hex_file ]
      then
        printf "Merge 'HEX Historical & Recent files to HEXFILES_DIR - "
        cat $HEXFILES_HISTORICAL_DIR/$hex_file $HEXFILES_RECENT_DIR/$hex_file > $HEXFILES_DIR/$hex_file 
        status_cat=$?
      else
        printf "Copy 'HEX Recent file to HEXFILES_DIR - "
        cp -p $HEXFILES_RECENT_DIR/$hex_file $HEXFILES_DIR/$hex_file
        status_cat=$?
      fi

      if [ $status_cat -eq 0 ]
      then
        echo "OK"
        $ARGOSSCRIPT_DIR/hex2cnt.sh $prgname $hex_file $float_id  # Script: hex2cnt.sh
        status="$?"
        echo "=> hex2cnt.sh status: $status"
      else
        status=9
        echo "FAILURE, STATUS=$status_cat"
      fi

      if [ $status -eq 0 -o $status -eq 2 ]
      then
        success_hex=`expr $success_hex + 1`
        if [ $status -eq 0 ]
        then
          display_val=1
        else
          display_val=2
        fi  	
        echo "$float_id.hex" >> $success_hex_list
        printf '\t%-15s      %1d         0          %-4s        yes\n' \
               $float_id $display_val $float_type  >> $SUMMARY_HEX2CNT_1
        if [ $status -eq 2 ]
        then
          printf '\t%-15s      %1d         0          %-4s        yes\n' \
                 $float_id $display_val $float_type  >> $SUMMARY_HEX2CNT_1_EMAIL
        fi
        prgname=cnt2phy_$float_type
        $ARGOSSCRIPT_DIR/cnt2phy.sh $prgname $float_type $float_id  # Script: cnt2phy.sh 
        status="$?"
        echo "=> cnt2phy.sh status: $status"
        success_cnt=`cat $sum_success_cnt_file`  # reset env success counter
        failure_cnt=`cat $sum_failure_cnt_file`  # reset env failure counter
      else
        failure_hex=`expr $failure_hex + 1`
        echo $float_id.hex >> $failure_hex_list
        printf '\t%-15s      0         1          %-4s        yes\n' \
               $float_id  $float_type 
        printf '\t%-15s      0         1          %-4s        yes\n' \
               $float_id  $float_type  >> $SUMMARY_HEX2CNT_1
        printf '\t%-15s      0         1          %-4s        yes\n' \
               $float_id  $float_type  >> $SUMMARY_HEX2CNT_1_EMAIL
      fi

    else                                           # Failure: float_type not found
      failure_hex=`expr $failure_hex + 1`
      echo $float_id.hex >> $failure_hex_list
      printf '\t%-15s      0         1          %-4s        yes\n' \
             $float_id  'none'  >> SUMMARY_HEX2CNT_1
      printf '\t%-15s      0         1          %-4s        yes\n' \
             $float_id  'none'  >> SUMMARY_HEX2CNT_1_EMAIL
      echo $float_id.meta >> $noinfo_in_metadata_list
    fi # endif: float_type found ? 

  else                                             # Failure: meta file does not exists
      failure_hex=`expr $failure_hex + 1`
      echo $float_id.hex >> $failure_hex_list
      printf '\t%-15s      0         1          %-3s        no\n' \
             $float_id  'n/a'  >> $SUMMARY_HEX2CNT_1
      printf '\t%-15s      0         1          %-3s        no\n' \
             $float_id  'n/a'  >> $SUMMARY_HEX2CNT_1_EMAIL

      # Note: Email  PI (keyword in meta file). A table with PI & Email needed 
      #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      echo $float_id.meta >> $no_metadata_list
  fi # endif: does meta file exists? 

  if [ -f $HEXFILES_DIR/$hex_file ]
  then
    printf "Remove $HEXFILES_DIR/$hex_file - "
    rm -f $HEXFILES_DIR/$hex_file
    echo "OK, STATUS=$?" 
  fi
  now=`date +'%h %d, %Y %T %p'`
  echo "<------------------- End   HEX $float_id $now --------------------->"
done

# Remove  working files
# ---------------------
if [ ! "$mode" = "hex" ]
then
  rm -f $ARGOSLIST_SUCCESS_DIR/prv2hex*.*_list
fi

rm -f $hexfiles2process

# -------------------------------------------------------------------------- 
#               Any HEX file process successfully?
#                   yes, continue execution
#                   no,  Email summary report and Exit
# -------------------------------------------------------------------------- 

if [ $success_hex -eq 0 ]
then
  ret_val=4
  
  if [ ! "$mode" = "hex" ]
  then
    mv $SUMMARY_PRV2HEX  $SUMMARY_REPORT
  fi
  
  $ARGOSSCRIPT_DIR/report_metadata_stderror.sh $script_name

  subj="ARGOS_PROCESS - No CNT and/or PHY file Created"
  msg="No CNT and/or PHY files created by C-programs: hex2cnt and/or cnt2phy,\n"
  msg=$msg"see log-files:"
  msg=$msg"\n\t$SUMMARY_REPORT"
  msg=$msg"\n\t$log_file"
  msg=$msg"\n\t$ARGOSLOG_DIR/hex2cnt.$TODAY"_trace
  msg=$msg"\n\t$ARGOSLOG_DIR/hex2cnt.$TODAY"_error
  msg=$msg"\nand/or"
  msg=$msg"\n\t$ARGOSLOG_DIR/cnt2phy.$TODAY"_trace
  msg=$msg"\n\t$ARGOSLOG_DIR/cnt2phy.$TODAY"_error
  echo $msg                 > $EMAIL_MESSAGE

  echo "[EMAIL Begin]"
  echo "  To: $EMAIL_OPERATOR"
  echo "  Subject: $subj"
  echo "  Text:"
  cat $EMAIL_MESSAGE | awk '{print "  " $0}'
  printf "\n  Sending Email ... "
  mailx -s "$subj" $EMAIL_OPERATOR < $EMAIL_MESSAGE
  echo "OK"
  echo "[EMAIL End]"

  MatchExitCode $ret_val
  DelExitCodeTable

  now=`date +'%h %d, %Y %T %p'`
  echo "<+++++++++++++++++++ Terminated $script_name: $now +++++++++++++++++>" 
  exit $ret_val   # no CNT/PHY profile created by C-program (hex2cnt_?.x/cnt2phy_?);
           # ? -> float type
fi

# -------------------------------------------------------------------------
#                   Report for PRV2HEX Processing 
# -------------------------------------------------------------------------
#        SUMMARY_PRV2HEX  (successes, failures, subtotals and total)
#        prv2hex_stderror (stderror produced by script)
#        trace_error      (trace & error files produced by prv2hex)
# 
# -------------------------------------------------------------------------

if [ "$mode" = "all" -o "$mode" = "argos"  -o  "$mode" = "prv" ]
then
  mv  $SUMMARY_PRV2HEX           $SUMMARY_REPORT
  cat $SUMMARY_PRV2HEX_EMAIL >>  $EMAIL_MESSAGE
fi

# -------------------------------------------------------------------------
#                   Report for HEX2CNT Processing 
# -------------------------------------------------------------------------
#        SUMMARY_HEX2CNT_1    (successes, failures, subtotals and total) 
#        SUMMARY_HEX2CNT_2    (new, changed and unchanged profiles) 
#        success_hex_list     (hex files successfully processed)
#        failure_hex_list     (hex files unsuccessfully processed) 
#        hex2cnt_stderror     (stderror produced by script)
# -------------------------------------------------------------------------
n_hex=`expr $success_hex + $failure_hex`
printf '\n\t      Subtotals    %3s       %3s\n' $success_hex $failure_hex \
       >> $SUMMARY_HEX2CNT_1
printf '\t         Totals\t\t\t   %s\n' $n_hex >> $SUMMARY_HEX2CNT_1
printf '\n\t      Subtotals    %3s       %3s\n' $success_hex $failure_hex \
       >> $SUMMARY_HEX2CNT_1_EMAIL
printf '\t         Totals\t\t\t   %s\n' $n_hex >> $SUMMARY_HEX2CNT_1_EMAIL

# Total and subtotals for 'Existence Status' report
# -------------------------------------------------
total_cnt=0
totals=`head -n 1  $totals_existance`
total_new_entry=`echo $totals | awk '{print $1}'`
total_differ_entry=`echo $totals | awk '{print $2}'`
total_same_entry=`echo $totals | awk '{print $3}'`
total_problem_entry=`echo $totals | awk '{print $4}'`
rm -f $totals_existance

total_cnt=`expr $total_cnt + $total_new_entry`
total_cnt=`expr $total_cnt + $total_differ_entry`
total_cnt=`expr $total_cnt + $total_same_entry`
total_cnt=`expr $total_cnt + $total_problem_entry`

printf '\n\t      Subtotals     %-3s         %-3s        %-3s        %-3s \n' \
       $total_new_entry $total_differ_entry \
       $total_same_entry $total_problem_entry   >> $SUMMARY_HEX2CNT_2
printf '\t         Totals\t\t%s\n' $total_cnt   >> $SUMMARY_HEX2CNT_2

printf '\n\t      Subtotals     %-3s         %-3s        %-3s        %-3s \n' \
       $total_new_entry $total_differ_entry \
       $total_same_entry $total_problem_entry   >> $SUMMARY_HEX2CNT_2_EMAIL
printf '\t         Totals\t\t%s\n' $total_cnt   >> $SUMMARY_HEX2CNT_2_EMAIL

if [ -s $withdata_before_launchtime ]    # Any float reported data before the launch time ?
then                                     #   Yes
  text="HEX files with data before the launch time"
  echo "   $text"
  cat $withdata_before_launchtime | awk '{print "   " $0}'
  echo "$text"                      > $SUMMARY_HEX2CNT
  cat $withdata_before_launchtime  >> $SUMMARY_HEX2CNTL
  echo "$text"                      > $SUMMARY_HEX2CNT_EMAIL
  cat $withdata_before_launchtime  >> $SUMMARY_HEX2CNT_EMAIL
else                                     #   No
  touch $SUMMARY_HEX2CNT $SUMMARY_HEX2CNT_EMAIL
  rm -f $withdata_before_launchtime
fi
  
cat $SUMMARY_HEX2CNT_1 >> $SUMMARY_HEX2CNT
cat $SUMMARY_HEX2CNT_2 >> $SUMMARY_HEX2CNT
cat $SUMMARY_HEX2CNT_1_EMAIL >> $SUMMARY_HEX2CNT_EMAIL
cat $SUMMARY_HEX2CNT_2_EMAIL >> $SUMMARY_HEX2CNT_EMAIL

rm -f $SUMMARY_HEX2CNT_1 $SUMMARY_HEX2CNT_2
rm -f $SUMMARY_HEX2CNT_1_EMAIL $SUMMARY_HEX2CNT_2_EMAIL

if [ -f $hex2cnt_stderror ]
then
  if [ -s $hex2cnt_stderror ]
  then
    msg="\nFor details on script (hex2cnt.sh) execution look at the"
    msg=$msg"\nerror file:  $hex2cnt_stderror"
    echo $msg              >> $SUMMARY_HEX2CNT
    echo $hex2cnt_stderror >> $SUMMARY_HEX2CNT
    echo $msg              >> $SUMMARY_HEX2CNT_EMAIL
    echo $hex2cnt_stderror >> $SUMMARY_HEX2CNT_EMAIL
  else
    rm -f $hex2cnt_stderror
  fi
fi

cat $SUMMARY_HEX2CNT >> $SUMMARY_REPORT
cat $SUMMARY_HEX2CNT_EMAIL >> $EMAIL_MESSAGE
rm -f $SUMMARY_HEX2CNT 

if [ -s $DIFF_JAVA_STATUS_TABLE ]
then
  echo " " >> $SUMMARY_REPORT
  echo "\t +++++ Summary Report of Argos CNT Files Processed by DiffCnt Java Program  +++++" >> $SUMMARY_REPORT
  echo " " >> $SUMMARY_REPORT
  echo "\t =======================================================================" >> $SUMMARY_REPORT
  echo "\tExit Status Explanation" >> $SUMMARY_REPORT
  echo "\t1, files are different, no errors, old file replaced" >> $SUMMARY_REPORT
  echo "\t2, float is potentially a surface drifter, new file moved to failure" >> $SUMMARY_REPORT
  echo "\t3, error in passing parameters to DiffCnt, new file moved to failure" >> $SUMMARY_REPORT
  echo "\t4, error parsing Date/Time or I/O, new file moved to failure" >> $SUMMARY_REPORT
  echo "\t5, profile number problem, new file moved to failure" >> $SUMMARY_REPORT
  echo "\t=======================================================================\n" >> $SUMMARY_REPORT
  echo "\t File Name          Exit Status"   >> $SUMMARY_REPORT
  cat $DIFF_JAVA_STATUS_TABLE | sort -k2 | awk '{print "\t"$0}' >> $SUMMARY_REPORT

  echo " " >> $EMAIL_MESSAGE
  echo "\t +++++ Summary Report of Argos CNT Files Processed by DiffCnt Java Program  +++++" >> $EMAIL_MESSAGE
  echo " " >> $EMAIL_MESSAGE
  echo "\t =======================================================================" >> $EMAIL_MESSAGE
  echo "\tExit Status Explanation" >> $EMAIL_MESSAGE
  echo "\t1, files are different, no errors, old file replaced" >> $EMAIL_MESSAGE
  echo "\t2, float is potentially a surface drifter, new file moved to failure" >> $EMAIL_MESSAGE
  echo "\t3, error in passing parameters to DiffCnt, new file moved to failure" >> $EMAIL_MESSAGE
  echo "\t4, error parsing Date/Time or I/O, new file moved to failure" >> $EMAIL_MESSAGE
  echo "\t5, profile number problem, new file moved to failure" >> $EMAIL_MESSAGE
  echo "\t=======================================================================\n" >> $EMAIL_MESSAGE
  echo "\t File Name          Exit Status"   >> $EMAIL_MESSAGE
  cat $DIFF_JAVA_STATUS_TABLE | sort -k2 | awk '{print "\t"$0}' >> $EMAIL_MESSAGE

fi

# ----------------------------------------------------------------------
#        Are there any new or updated CNT profiles?
#          yes, continue processing
#          no,  send Email to OPERATOR and exit
# ----------------------------------------------------------------------
NEW_OR_CHANGED_CNTPROFILE=`cat $new_or_changed_cntprof_file`
echo "NEW_OR_CHANGED_CNTPROFILE: $NEW_OR_CHANGED_CNTPROFILE"

if [ "$NEW_OR_CHANGED_CNTPROFILE" = "no" ]
then
  ret_val=5
  if [ "$mode" = "all" -o "$mode" = "argos"  -o  "$mode" = "prv" ]
  then
    NPRV_FAILURES=`cat $nprv_failures_file`     
    $ARGOSSCRIPT_DIR/send_email.sh $up_script_name  $NPRV_FAILURES   # Send Email
  fi
  
  $ARGOSSCRIPT_DIR/report_metadata_stderror.sh $script_name
  if [ -f $SUMMARY_CNT2PHY ]
  then
    rm -f $SUMMARY_CNT2PHY
  fi

  if [ -f $SUMMARY_CNT2PHY_EMAIL ]
  then
    rm -f $SUMMARY_CNT2PHY_EMAIL
  fi

  rm -f $new_or_changed_cntprof_file     # remove working files in ARGOSTMP_DIR
  rm -f $sum_success_cnt_file
  rm -f $sum_failure_cnt_file
  
  if [ -f $success_hex_list ]
  then
    echo "=> Success Hex List ($success_hex_list):"
    cat $success_hex_list | awk '{print "   " $0}'
    rm -f $success_hex_list
  fi

  if [ -f $success_cnt_list ]
  then
    echo "=> Success Cnt List ($success_cnt_list):"
    cat $success_cnt_list | awk '{print "   " $0}'
    rm -f $success_cnt_list
  fi
 
  if [ "$mode" = "all" -o "$mode" = "argos"  -o  "$mode" = "prv" ]
  then
    rm -f $nprv_failures_list
  fi
  
  MatchExitCode $ret_val
  DelExitCodeTable

  now=`date +'%h %d, %Y %T %p'`
  echo "<+++++++++++++++++++ Finish $script_name: $now +++++++++++++++++>"
  exit $ret_val 
fi
 
# -------------------------------------------------------------------------
#                   Report for CNT2PHY Processing 
# -------------------------------------------------------------------------
#    SUMMARY_CNT2PHY   (successes, failures, subtotals and total) 
#    success_cnt_list  (cnt files successfully processed
#    failure_cnt_list  (cnt files unsuccessfully processed)
#    phylist_outfile   (list of phy files created) 
#    cnt2phy_stderror  (stderror produced by script)
#    hex2cnt_stderror  (stderror produced by script)
# -------------------------------------------------------------------------
n_cnt=`expr $success_cnt + $failure_cnt`
printf '\n\t         Subtotals       %3s       %3s\n' $success_cnt  $failure_cnt >> $SUMMARY_CNT2PHY
printf '\t             Total\t\t\t    %s\n' $n_cnt  >> $SUMMARY_CNT2PHY
printf '\n\t         Subtotals       %3s       %3s\n' $success_cnt  $failure_cnt >> $SUMMARY_CNT2PHY_EMAIL
printf '\t             Total\t\t\t    %s\n' $n_cnt  >> $SUMMARY_CNT2PHY_EMAIL

if [ -f $failure_cnt_list ]
then
  msg="\nArgo CNT Files Processed Unsuccessfully by cnt2phy C-program(s):"
  echo "$msg" >> $SUMMARY_CNT2PHY 
  cat $failure_cnt_list >> $SUMMARY_CNT2PHY 
  echo "$msg" >> $SUMMARY_CNT2PHY_EMAIL 
  cat $failure_cnt_list >> $SUMMARY_CNT2PHY_EMAIL 
fi

if [ -f $cnt2phy_stderror ]
then
  if [ -s $cnt2phy_stderror ]
  then
    msg="\nFor details on script (cnt2phy.sh) execution look at the"
    msg=$msg"\nerror file:  $cnt2phy_stderror"
    echo $msg   >> $SUMMARY_CNT2PHY
    echo $msg   >> $SUMMARY_CNT2PHY_EMAIL
  else
    rm -f $cnt2phy_stderror
  fi
fi

cat $SUMMARY_CNT2PHY >> $SUMMARY_REPORT
cat $SUMMARY_CNT2PHY_EMAIL >> $EMAIL_MESSAGE
rm $SUMMARY_CNT2PHY $SUMMARY_CNT2PHY_EMAIL

# ---------------------------------------------------------------------
# List float trace & error logfiles created when a failure was
# detected while executing hex2cnt and/or cnt2phy
# ---------------------------------------------------------------------
if [ -f $floatfail_trace_error_list ]
then
  if [ -s $floatfail_trace_error_list ]
  then
    msg="\nFor details on float hex2cnt and/or cnt2phy C-programs execution"
    msg=$msg" failure\n look at the following trace and error log files"
    msg=$msg" in $ARGOSLOG_DIR:"
    echo $msg
    cat $floatfail_trace_error_list
    echo $msg >> $SUMMARY_REPORT
    cat $floatfail_trace_error_list >> $SUMMARY_REPORT
    echo $msg >> $EMAIL_MESSAGE
    cat $floatfail_trace_error_list >> $EMAIL_MESSAGE
  else
    rm -f $floatfail_trace_error_list
  fi
fi
 
# -----------------------------------------------------------------
# Report possible metadata errors (missing file and/or float type),
# error messages produced by 'remove_ghosts.sh' and stderrror
# produce by this script.  
# -----------------------------------------------------------------
$ARGOSSCRIPT_DIR/report_metadata_stderror.sh $script_name

# -------------------------------------------------------------------------
#                 Report for PHY_COMPLETE Processing
# -------------------------------------------------------------------------
#    SUMMARY_PHY_COMPLETE (PHY profile, length, not complete and % completion)
#    phy_complete_stderror  (stderror produced by script)
# -------------------------------------------------------------------------
if [ -f $phylist_outfile ]                 # Does PHY output list file exists?
then                                       #   yes, do profile completion test
  $ARGOSSCRIPT_DIR/phy_complete.sh $phylist_outfile
  if [ -f $phy_complete_stderror ]
  then
    if [ -s $phy_complete_stderror ]
    then
      msg="\nERROR detected while executing script phy_complete.sh."
      msg=$msg" For further details look \nat the following error logfile:"
      msg=$msg"\n\t$phy_complete_stderror"
      echo $msg   >> $SUMMARY_PHY_COMPLETE
      echo $msg   >> $SUMMARY_PHY_EMAIL
    else
      rm -f $phy_complete_stderror
    fi
  fi
fi

cat $SUMMARY_PHY_COMPLETE >> $SUMMARY_REPORT
cat $SUMMARY_PHY_EMAIL    >> $EMAIL_MESSAGE

# ----------------------------------------------------------------------
#                      Send Email to OPERATOR
# ----------------------------------------------------------------------
if [ "$mode" = "hex" ]
then
  NPRV_FAILURES=0
else
  NPRV_FAILURES=`cat $nprv_failures_file`
  rm -f $nprv_failures_file
fi

$ARGOSSCRIPT_DIR/send_email.sh $up_script_name $NPRV_FAILURES

# ----------------------------------------------------------------------
#       Move C-programs (CNT2PHY_?.c) output PHY list to ARGO list
# ----------------------------------------------------------------------
if [ -f $phylist_outfile ]
then
  echo "=> phylist_outfile: $phylist_outfile"
  printf "   Move: phylist_outfile -> $ARGOLIST_DIR - "
  mv $phylist_outfile $ARGOLIST_DIR
  echo "OK"
fi

# ----------------------------------------------------------------------
#               Remove working files from ARGOSTMP_DIR 
# ----------------------------------------------------------------------
rm -f $new_or_changed_cntprof_file     # remove working files in ARGOSTMP_DIR
rm -f $sum_success_cnt_file
rm -f $sum_failure_cnt_file

echo "=> Success Hex List ($success_hex_list):"
cat $success_hex_list | awk '{print "   " $0}'

echo "=> Success Cnt List ($success_cnt_list):"
cat $success_cnt_list | awk '{print "   " $0}'

# Note: 
#   success_hex_list - list of hex filenames 
#   success_cnt_list - list of phy filenames

if [ "${MANUALPROCESS_PHY_ANAL_LIST=}" != "" ]
then
  echo "=> Create Manual Success Execution Lists (hex, cnt and phy)"
  echo "   by moving existing lists"
  echo "   MANUAL_HEX_SUCCESS_LIST: $MANUAL_HEX_SUCCESS_LIST"
  echo "   MANUAL_CNT_SUCCESS_LIST: $MANUAL_CNT_SUCCESS_LIST"
  echo "   MANUAL_PHY_SUCCESS_LIST: $MANUAL_PHY_SUCCESS_LIST"
  
   if [ -f $success_hex_list ]
   then 
     mv $success_hex_list $MANUAL_HEX_SUCCESS_LIST     
   fi 
   if [ -f $success_cnt_list ]
   then
     mv $success_cnt_list $MANUAL_CNT_SUCCESS_LIST     
     sed s/cnt/phy/ $MANUAL_CNT_SUCCESS_LIST > $MANUAL_PHY_SUCCESS_LIST # containts CNT filenames 
   fi
else
  rm -f $success_hex_list
  rm -f $success_cnt_list
fi

ret_val=0
MatchExitCode $ret_val
DelExitCodeTable

now=`date +'%h %d, %Y %T %p'`
echo "<+++++++++++++++++++ FINISH $script_name: $now +++++++++++++++++>" 

exit $ret_val 
