#!/bin/sh
###############################################################################################
# Revision Date: 25-Feb-14 (RS - remove commented stataments prefixed with '#rscs')
#                18-Feb-14 (RS,CS - moving phy files from "done" to "done_old" used to be done
#                                   in this script. Because of that old phy files from Iridium
#                                   floats were not moved (i.e. they were lost). Now, an old
#                                   'phy' file is moved to "done_old" in qc_process.sh.)
#                22-May-13 (RS - 'aomlid' directories are not created under 'failure') 
#                29-Oct-12( RS - code changes related to the creation of the 'aomlid' directory
#                                for each float under:
#                                  /a1/ARGO/argos/data/cnt/done argos/data/cnt/done_old
#                                  /a1/ARGO/data/phy/done data/phy/done_old)
#                13-Apr-09 (RS,LL - use table driven decoder)
#                12-Sep-07 (RS - Trap WARNINGS messages in the profile 'trace' file and write 
#                                to 'warning.[timestamp]_log' for email reporting)
#                09-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                07-Mar-06 (RS - touch CNT files in 'done; before moving to 'done_old')
#                22-Feb-06 (CS - changed "cp -p" to "cp" when copying PHY files
#                                from done to done_old)
#                24-Sep-03 (RS - remove write to $error)
#                30-Apr-04 (RS - 4 digits for profile number & 10 char for transmission ID)
#                23-Jul-01 (RS - cp PHY profile if exists in 'done')
#                19-Jul-01 (RS - back to 5 digits)
#                10-Jul-01 (RS - argos id expansion)
#                05-Jul-01 (RS - echo message when moving to 'done' and/or 'done-old'
#                07-Sep-00 (RS - introduce DONEOLD_CNTFILES_DIR)
#                20-Jun-00 (RS - new directory tree)
#                15-Jun-00 (RS - if failure: mv CNT file to FAILURE_CNTFILES_DIR)
#                02-Feb-00 (RS)
#
# Converts counts to physical units conversion program (cnt2phy_t2.x)
###############################################################################################
script_name=`basename $0`
prgname=$1
float_type=$2
float_id=$3

up_prgname=`echo $prgname | tr "[:lower:]" "[:upper:]"`
now=`date +'%h %d, %Y %T %p'`

echo "<====================== Start  $script_name: $now ====================>"
echo "Enter: `basename $0` to execute $prgname float_id: $float_id aomlid: $aomlid"

# -----------------------------------------------------------------------------
# If the 'aomlid' directory does not exist in the following directories
# then create it.
#
#     PHYFILES_DIR/done  PHYFILES_DIR/done_old & PHYFILES_DIR/failure
#
# Following global variables are defined in ARGO/init_scripts/argo.sh 
# PHYFILES_DIR: ARGO/data/phy
# DONE_PHYFILES_DIR: PHYFILES_DIR/done
# DONEOLD_PHYFILES_DIR: PHYFILES_DIR/done_old
# FAILED_PHYFILES_DIR: PHYFILES_DIR/failure
# -----------------------------------------------------------------------------
aomlid=`echo $float_id | awk -F"_" '{print $1}'`

if [ !  -d $DONE_PHYFILES_DIR/$aomlid ]
then
  printf "Mkdir $DONE_PHYFILES_DIR/$aomlid - "
  mkdir $DONE_PHYFILES_DIR/$aomlid
  echo "OK"
fi

if [ ! -d $DONEOLD_PHYFILES_DIR/$aomlid ]
then
  printf "Mkdir $DONEOLD_PHYFILES_DIR/$aomlid - "
  mkdir $DONEOLD_PHYFILES_DIR/$aomlid
  echo "OK"
fi

error=$ARGOSLOG_DIR/$prgname.$TODAY"_"error
if [ -f $error ]
then
  rm -f $error
fi                 
exec 2>> $error

# -------------------------------------------------------------------
#               Process cnt files in hex2cnt_type.*_list
# -------------------------------------------------------------------

# Does the float list file (hex2cnt_type.TODAY_list) exists?
# ----------------------------------------------------------
hex_list_name=hex2cnt.$TODAY"_"list            # Defined in hex2cnt.sh
hex_list_path_filename=$ARGOSLIST_SUCCESS_DIR/$hex_list_name
 
if [ -f $hex_list_path_filename ]
then                                           # Yes, it exissts
  if [ ! -s  $hex_list_path_filename ]         # Is it empty?
  then                                         # Yes, EXIT
    msg="NOTICE: `basename $0`:No Entries in $hex_list_path_filename\n"
    msg=$msg"all cnt profile(s) have been processed  previously"
    echo $msg
    rm -f $hex_list_path_filename
    exit 0
  fi
else
  msg="NOTICE: `basename $0`: List File ($hex_list_path_filename)  DOES NOT EXISTS\n"
  msg=$msg"all cnt profile(s) might have been processed  previously"
  echo $msg
  exit 0 
fi

# ------------------------------------------------------------------------
#                      Processing CNT File(s)
# ------------------------------------------------------------------------
echo "List CNT Files to Process $hex_list_path_filename:"
cat $hex_list_path_filename

phy_outdir=$PHYFILES_DIR
failure=0

for cnt_infile in `cat $hex_list_path_filename`
do
  now=`date +'%h %d, %Y %T %p'`
  profile_id=`echo $cnt_infile | awk -F"." '{print $1}'`
  echo "<----------------- Begin CNT $profile_id $now ------------------->"
  echo "Processing file: $cnt_infile"
  filepathname=$CNTFILES_DIR/$cnt_infile
  if [ ! -f $filepathname ]                # Does the file exist?
  then                                     #   no, report and break
    failure=1
    failure_cnt=`expr $failure_cnt + 1`
    echo $profile_id.cnt >> $failure_cnt_list
    printf '\t%-20s       0         1         %s\n' $profile_id  $float_type >> $SUMMARY_CNT2PHY
    printf '\t%-20s       0         1         %s\n' $profile_id  $float_type >> $SUMMARY_CNT2PHY_EMAIL
    msg="NOTICE: CNT file does not exist. Hint- it is possible that there were "
    msg=$msg"at least two CNT profile\nfiles with same name. If so, the "
    msg=$msg"first processed file was moved to its 'aomlid' dir under DONE_CNTFILES_DIR \n"
    msg=$msg"To verify it, look at the CNT output list generated by "
    msg=$msg"the execution of the HEX2CNT program for\nfloat $float_id."
    echo $msg >> $error
    echo $msg
    continue                             # Skip to next for-loop iteration 
  fi

  echo "PATH_TABLE_DECODER: $TABLE_DECODERS_DIR/$table_name"
  if [ -f $TABLE_DECODERS_DIR/$table_name ]
  then
      echo "<<< Using Table Decoder: $table_name >>>"
      $ARGOSPRG_DIR/ts_all/cnt2phy_tsall.x $cnt_infile $phy_outdir \
                             $trace_outfile $error_outfile $phylist_outfile
  else
      echo "<<< Not Using Table Decoder >>>"
      $ARGOSPRG_DIR/$prgname.x $cnt_infile $phy_outdir \
                             $trace_outfile $error_outfile $phylist_outfile
  fi
  status_x=$?
  if [ "`grep -i warning $trace_outfile 2> /dev/null`" ]
  then
    echo "$prgname $cnt_infile -> see $trace_outfile" >> $warning_log
  fi

  if [ $status_x -eq 0 ]
  then                                   # Success: esqlc program execution ok
    echo ">>> $up_prgname Program Execution OK <<<"
    success_cnt=`expr $success_cnt + 1`
    echo $profile_id.cnt >> $success_cnt_list
    printf '\t%-20s       1         0         %s\n' $profile_id  $float_type >> $SUMMARY_CNT2PHY

    if [ -f $DONE_CNTFILES_DIR/$aomlid/$cnt_infile ]   # Does it exist?
    then                                                #   Yes, move to keep previous version 
      printf "Move: $DONE_CNTFILES_DIR/$aomlid/$cnt_infile -> $DONEOLD_CNTFILES_DIR/$aomlid - "
      touch $DONE_CNTFILES_DIR/$aomlid/$cnt_infile     # ???? WHY NEEDED ???
      mv $DONE_CNTFILES_DIR/$aomlid/$cnt_infile $DONEOLD_CNTFILES_DIR/$aomlid
      echo "OK"
    fi
    printf "Move: $CNTFILES_DIR/$cnt_infile -> $DONE_CNTFILES_DIR/$aomlid - " 
    mv $CNTFILES_DIR/$cnt_infile $DONE_CNTFILES_DIR/$aomlid 
    echo "OK"
  else                                # Failure: Execution terminated abnormally
    failure=1
    failure_cnt=`expr $failure_cnt + 1`
    echo $profile_id.cnt >> $failure_cnt_list 
    printf '\t%-20s       0         1         %s\n' $profile_id  $float_type  >> $SUMMARY_CNT2PHY
    printf '\t%-20s       0         1         %s\n' $profile_id  $float_type  >> $SUMMARY_CNT2PHY_EMAIL
    msg="ERROR: $up_prgname Execution Terminated Abnormally while Processing:"
    msg=$msg" $cnt_infile"
    echo $msg 
    echo $msg >> $error 
    printf "Move: $CNTFILES_DIR/$cnt_infile -> $FAILURE_CNTFILES_DIR - " 
    mv $CNTFILES_DIR/$cnt_infile $FAILURE_CNTFILES_DIR 
    echo "OK"
    now=`date +'%h %d, %Y %T %p'`
    echo "         **********   Count File Terminated: $now  **********       " >> $error
    echo "         **********   Count File Terminated: $now  **********       "
  fi # endif: failure test
  now=`date +'%h %d, %Y %T %p'`
  echo "<----------------- End   CNT $profile_id $now ------------------->"
done

# NOTICE: environmental variables of parent process cannot be changed
# ------------------------------------------------------------------- 
echo "==> CNT2PHY success_cnt: $success_cnt failure_cnt: $failure_cnt"
echo $success_cnt > $sum_success_cnt_file
echo $failure_cnt > $sum_failure_cnt_file

# Report Number of Successes & Failures via Email
# -----------------------------------------------
n_prof=`expr $success_cnt + $failure_cnt`

if [ $failure = "0" ]
then
  ret_val=0
else
  ret_val=1
  echo "$trace_outfile \t$error_outfile" >> $floatfail_trace_error_list 
fi

# Remove float hex2cnt list file
# ------------------------------
  rm -f $hex_list_path_filename
  echo "Float $float_id $hex_list_path_filename Removed"

# Does the list output file exists?  
# ---------------------------------
if [ -f $phylist_outfile ]
then
  echo "List File ($phylist_outfile):" 
  cat  $phylist_outfile
else
  echo ">>> NOTICE($script_name): NO PHY File to Created <<<"
  ret_val=1
fi

# Does script error file exists?
# ------------------------------
if [ -f $error ]                           # does it exist?
then
  if [ -s $error ]
  then                                     # delete if empty
    msg="<----------------------------------- "
    msg=$msg" $script_name: $float_id "
    msg=$msg"----------------------------------->"
    echo "$msg" >> $cnt2phy_stderror
    cat $error  >> $cnt2phy_stderror
  else
    rm -f $error
  fi    
fi

now=`date +'%h %d, %Y %T %p'`
echo "<====================== Finish $script_name: $now ====================>"

exit $ret_val 
