#!/bin/sh
#######################################################################################
# Revision Date: 22-May-13 (RS - 'aomlid' directories are not created under 'failure') 
#                22-Oct-12( RS - code changes related to the creation of the 'aomlid'
#                                directory for each float under:
#                                 /a1/ARGO/argos/data/cnt/done argos/data/cnt/done_old
#                                 /a1/ARGO/argos/data/cnt/failure)
#                22-Jun-11 (ER   Changes due to redesign of DiffCnt program)
#                13-Apr-09 (RS,LL - use table driven decoder)
#                08-Dec-08 (RS - DiffCnt Exit-Status=6 is now disable because of a
#                                more restrict implementation. DiffCnt Exit-Status=5,
#                                now displays "Profile Number Error".  
#                                Fixed DiffCnt Standard-Error produced when the  
#                                position records in the new 'cnt' file and the one
#                                in 'done' were different)
#                21-Mar-06 (RS - changed ERROR to WARNING for message when 
#                                the 'list_outfile' created by 'hex2cnt_[prg]'
#                                is empty)
#                09-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                10-Jan-05 (RS - report float with data before launch time)
#                07-Jul-04 (RS - add documentation for 'success=2')
#                24-Sep-03 (RS - remove write to $error)
#                27-Aug-03 (RS - modifications to deal with not enough data
#                                to create a profile, Exit Status=2 from
#                                hex2cnt_[float].c)
#                04-Aug-03 (RS - use Java program when the new & old cnt file differ)
#                30-Apr-03 (RS - 10 character for program transmission number)
#                08-Nov-01 (CS - parameter name change)
#                26-Jul-01 (RS - add cummulative sums for Existence tagle)
#                10-Jul-01 (RS - argos id expansion)
#                26-Jun-01 (RS - changed FirstDate to allow no position record)
#                18-Jun-01 (CS - changed DownTime)
#                20-Nov-00 (RS - fix for hex2cnt table)
#                13-Nov-00 (RS - change Latitude to LATITUDE in )
#                20-Sep-00 (RS - new meta data)
#                28-Aug-00 (RS - update DownTime)
#                26-Jul-00 (RS - add comments)
#                19-Jul-00 (RS - delete debugging echo line)
#                29-Jun-00 (RS - detect profile problem (today-first_date > down time)
#                20-Jun-00 (RS - new directory tree)
#                02-Feb-00 (RS)
#
# Execute: hexadecimal to counts conversion program (run_hex2cnt_3.x) 
#######################################################################################
# --------------------------------  Main Script  --------------------------------------
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
prgname=$1
hex_infile=$2
float_id=$3

up_prgname=`echo $prgname | tr "[:lower:]" "[:upper:]"`
now=`date +'%h %d, %Y %T %p'`
echo "<====================== Start  $script_name: $now ====================>"
echo "Enter: `basename $0` to excute $prgname float_id: $float_id"

# -----------------------------------------------------------------------------
# If the 'aomlid' directory does not exist in the following directories    
# then create it.
#     CNTFILES_DIR/done  CNTFILES_DIR/done_old & CNTFILES_DIR/failure
#
# Following global variables are defined in ARGO/init_scripts/argo.sh
# CNTFILES_DIR: ARGO/argos/data/phy
# DONE_CNTFILES_DIR: CNTFILES_DIR/done
# DONEOLD_CNTFILES_DIR: CNTFILES_DIR/done_old
# FAILURE_CNTFILES_DIR=CNTFILES_DIR/failure 
# -----------------------------------------------------------------------------
aomlid=`echo $float_id | awk -F"_" '{print $1}'`
echo "aomlid: $aomlid"

if [ ! -d $DONE_CNTFILES_DIR/$aomlid ]
then
  printf "Mkdir $DONE_CNTFILES_DIR/$aomlid - "
  mkdir $DONE_CNTFILES_DIR/$aomlid
  echo "OK"
fi

if [ ! -d $DONEOLD_CNTFILES_DIR/$aomlid ]
then
  printf "Mkdir $DONEOLD_CNTFILES_DIR/$aomlid - "
  mkdir $DONEOLD_CNTFILES_DIR/$aomlid
  echo "OK"
fi

error=$ARGOSLOG_DIR/$prgname.$TODAY"_"error
if [ -f $error ]
then
  rm -f $error
fi
exec 2>> $error

# Define list for CNT profiles successfully processed - also used in CNT2PHY.SH
# -----------------------------------------------------------------------------
list_outfile=$ARGOSLIST_SUCCESS_DIR/$root_script_name.$TODAY"_"list
echo "HEX2CNT list_outfile: $list_outfile"

if [ -f $list_outfile ]
then
  rm -f $list_outfile
fi

cnt_outdir=$CNTFILES_DIR
echo "---> Processing file: $hex_infile"

# -------------------------------------------------------------------------- 
#                    Execute C-Program:  hex2cnt_??.x   
# where: ?? is float type
# -------------------------------------------------------------------------- 
echo "PATH_TABLE_DECODER: $TABLE_DECODERS_DIR/$table_name"
if [ -f $TABLE_DECODERS_DIR/$table_name ]
then
  echo "<<< Using Table Decoder: $table_name >>>" 
  $ARGOSPRG_DIR/ts_all/hex2cnt_tsall.x $hex_infile $NPROFILE $cnt_outdir \
                       $trace_outfile $error_outfile $list_outfile 
else
  echo "<<< Not Using Table Decoder >>>" 
  $ARGOSPRG_DIR/$prgname.x $hex_infile $NPROFILE $cnt_outdir \
                 $trace_outfile $error_outfile $list_outfile 
fi
failure=$?
if [ $failure -eq 0 ]
then                                   # Success: program execution ok
  echo ">>> $up_prgname Program Execution OK <<<"
else                                   # Failure: Execution terminated abnormally
  echo ""
  msg="ERROR ($root_script_name): $failure\n"
  msg=$msg" Program Execution Terminated Abnormally while Processing $hex_infile"
  echo $msg 
fi

# --------------------------------------------------------------------------
#      Does HEX file contains data before the float launch time?
#       Yes, write message to file 
# --------------------------------------------------------------------------
if [ `grep -i "$search_string_in_trace" $trace_outfile | wc -l` -ne 0 ]
then
  echo "$hex_infile" >> $withdata_before_launchtime 
fi

# --------------------------------------------------------------------------
# Compare 'cnt' profile in list generated by hex2cnt_type with 'cnt' profile
# file (if exists) in "$ARGO_DIR/data/cnt/done/aomlid" directory and check
# if files are the same.
#   If yes, delete cnt profile in $ARGO_DIR/data/cnt  
#   If no, add cnt profile file name to profile cnt list
#
# failure=2, insuficient data for creating a 'cnt' file for one/more 
# profile(s) because of a missing frame (SOLO) or one data line in the
# 'hex' file on a given day. 
# --------------------------------------------------------------------------
new_count=0     # cnt profile is new 
same_count=0    # new cnt profile exists in DONE_CNTFILES_DIR float directory 
differ_count=0  # new cnt profile differs from the one in DONE_CNTFILES_DIR float directory 
problem_count=0 # new cnt profile differs from the one in DONE_CNTFILES_DIR float directory
                # but there is a problem. CNT profile not moved to new list.

if [ $failure -eq 0  -o $failure -eq 2 ]
then                                                 # Success
  if [ -f $list_outfile ] 
  then
    if [ ! -s $list_outfile ]                        # Is list empty?
    then                                             #   yes, list_outfile empty
      msg="$up_prgname SCRIPT -> WARNING: Empty List ($list_outfile)" 
      echo $msg
      echo $msg >> $EMAIL_MESSAGE
    else                                             #   no, list is not empty
      echo "List File ($list_outfile):" 
      cat $list_outfile

      profile_new_list=$ARGOSTMP_DIR/profile_new_list.$TODAY
      if [ -f $profile_new_list ]                    # make sure file does not exists
      then                                          
        rm -f $profile_new_list
      fi

      touch $profile_new_list
      for cnt_filename in `cat $list_outfile`
      do
        cnt_path_filename=$CNTFILES_DIR/$cnt_filename
        echo "cnt_path_filename: $cnt_path_filename"
        root_profile=`echo $cnt_filename | awk -F"." '{print $1}'` 
        float_id=`echo $root_profile | awk -F"_" '{print $1 "_" $2}'` 
        meta_file=$float_id.meta

        if [ -f $cnt_path_filename ]
        then
          printf "=> Does '$cnt_filename' exists in DONE_CNTFILES_DIR/$aomlid - "
          done_aomlid=$DONE_CNTFILES_DIR/$aomlid/$cnt_filename
          if [ -f $done_aomlid ]                      # Does it exists in done/aomlid dir?
          then                                        #   Yes, profile exists
            printf "YES ->  check for difference - " 
            diff $cnt_path_filename $done_aomlid > /dev/null 
            status="$?"
            if [ $status -eq 0 ]                      # Is it different?
            then                                      #   No, increment same counter
              echo "NO SAME as in  'done/$aomlid'" 
              same_count=`expr $same_count + 1`
              rm -f $cnt_path_filename 
            else                                      # else - status=0 
              java -classpath $ARGOSPRGJAVA_DIR DiffCnt $done_aomlid $cnt_path_filename \
                              $diffcnt_log $DIFF_JAVA_STATUS_TABLE
              status="$?" 
              case $status in
              1) differ_count=`expr $differ_count + 1`         #replace
                 echo $cnt_filename >> $profile_new_list
                 echo "DiffCnt(status=$status): difference found - REPLACE";;
              2) problem_count=`expr $problem_count + 1`       #do not replace, fix it     
                 printf "DiffCnt(status=$status): Possible surface drifter -> mv to FAILURE_CNTFILES_DIR - "
                 mv $cnt_path_filename $FAILURE_CNTFILES_DIR
                 echo "OK"
                 echo "  $cnt_path_filename -> $FAILURE_CNTFILES_DIR";;
              3) problem_count=`expr $problem_count + 1`   
                 printf "DiffCnt(status=$status): Usage Error -> mv to FAILURE_CNTFILES_DIR - "
                 problem_count=`expr $problem_count + 1`   
                 mv $cnt_path_filename $FAILURE_CNTFILES_DIR
                 echo "OK"
                 echo "  $cnt_path_filename -> $FAILURE_CNTFILES_DIR";;
              4) problem_count=`expr $problem_count + 1`   
                 printf "DiffCnt(status=$status): Date/Time parsing or I/O Error -> mv to FAILURE_CNTFILES_DIR - "
                 mv $cnt_path_filename $FAILURE_CNTFILES_DIR
                 echo "OK"
                 echo "  $cnt_path_filename -> $FAILURE_CNTFILES_DIR";;
              5) problem_count=`expr $problem_count + 1`   #do not replace
                 printf "DiffCnt(status=$status): Profile Number Problem -> mv to FAILURE_CNTFILES_DIR - " 
                 mv $cnt_path_filename $FAILURE_CNTFILES_DIR
                 echo "OK"
                 echo "  $cnt_path_filename -> $FAILURE_CNTFILES_DIR";;
              *) problem_count=`expr $problem_count + 1` 
                 echo "DiffCnt(status=$status): Unexpected exit status in Java Prg -> mv to FAILURE_CNTFILES_DIR - " 
                 mv $cnt_path_filename $FAILURE_CNTFILES_DIR
                 echo "OK"
                 echo "  $cnt_path_filename -> $FAILURE_CNTFILES_DIR";;
              esac
            fi # endif- status=0 
          else                                        # No, file is not in 'done/aomlid' dir
            echo "File does not exist in 'done/$aomlid'" 
            new_count=`expr $new_count + 1`           #   incremet new counter 
            echo $cnt_filename >> $profile_new_list
          fi # endif: does file exist in 'done/aomlid' dir
        fi # endif: cnt_path_filename
      done # endfor - looop for output files
 
# Cummulative sum for all hex files
# ---------------------------------
      totals=`head -n 1  $totals_existance`
      if [ "$totals" = "" ]
      then
        total_new_count=0
        total_differ_count=0
        total_same_count=0
        total_problem_count=0 
      else
        total_new_count=`echo $totals     | awk '{print $1}'`
        total_differ_count=`echo $totals  | awk '{print $2}'`
        total_same_count=`echo $totals    | awk '{print $3}'`
        total_problem_count=`echo $totals | awk '{print $4}'`
      fi

      total_new_count=`expr $total_new_count + $new_count`
      total_differ_count=`expr $total_differ_count + $differ_count`
      total_same_count=`expr $total_same_count + $same_count`
      total_problem_count=`expr $total_problem_count + $problem_count`

      echo $total_new_count $total_differ_count \
           $total_same_count $total_problem_count > $totals_existance

# Any new and/or changed profiles (detected from data/cnt/done/aomlid? 
# --------------------------------------------------------------------
      entry=`expr $new_count + $differ_count`
      printf "\t%-15s   %3s         %3s        %3s          %s\n" \
             $float_id $new_count $differ_count \
             $same_count $problem_count   >> $SUMMARY_HEX2CNT_2
      if [ $problem_count -ne 0 ]                # Filter for Email?
      then                                       #   Yes
        printf "\t%-15s   %3s         %3s        %3s          %s\n" \
               $float_id $new_count $differ_count \
               $same_count $problem_count   >> $SUMMARY_HEX2CNT_2_EMAIL
      fi

      if [ $entry = "0" ]
      then                                        # Yes, remove cnt list file 
        rm -f $list_outfile                       #      and working file
        rm -f $profile_new_list                   
      else                                        # No, copy new/changed prof list  
        echo "<<< New or Changed (NEW_OR_CHANGED_CNTPROFILE) 'cnt' Profile(s) Found: yes >>>"
        echo "    profile_new_list: $profile_new_list"
        echo "    list_outfile: $list_outfile" 
        printf "    Move: profile_new_list -> list_outfile - "
        mv $profile_new_list $list_outfile     
        echo "OK"
        echo "yes" > $new_or_changed_cntprof_file # env var of parent process
      fi                                          # can't be changed. 
    fi # endif: is list_outfile empty? 
  else                                   # Failure:  list_outfile does not exists
    msg="$up_prgname SCRIPT -> ERROR: No list file ($list_outfile)"
    echo $msg
    echo $msg >> $EMAIL_MESSAGE
  fi # endif: does list_outfile exists?
else                                      # Failure: remove list_outfile 
  echo "$trace_outfile \t$error_outfile" >> $floatfail_trace_error_list 
  rm -f $list_outfile
fi # endif: failure=0 - success/failure in hex2cnt_?.x 

# Does script error file exists?    
# ------------------------------
if [ -f $error ]                           # does it exist?
then
  if [ -s $error ]                         #  yes, empty ?
  then                                     #    file not empty 
    msg="<----------------------------------- "
    msg=$msg" $script_name: $float_id "
    msg=$msg"----------------------------------->"
    echo "$msg" >> $hex2cnt_stderror
    cat $error  >> $hex2cnt_stderror
  else
    rm -f $error
  fi
fi
now=`date +'%h %d, %Y %T %p'`

echo "<====================== Finish $script_name: $now ====================>"

exit $failure
