#!/bin/sh
###########################################################################
# Revision Date: 09-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                07-Oct-03 (RS - modified way to obtain data lines)
#                30-Apr-03 (RS - 4 digits for profile number & expand 
#                                transmission ID to 10 )
#                19-Jul-01 (RS - back to 5 digits)
#                10-Jul-01 (RS - argos id expansion)
#                17-Oct-00 (RS - length=0 & incomplete=0)
#                17-Aug-00 (RS - update documentation)
#                19-Jul-00 (RS - percent in floating arithmetid & round off)
#                10-Jul-00 (RS - check denominator to calculate percent)
#                26-Jun-00 (RS - check if WMO exist)
#                20-Jun-00 (RS - new directory tree)
#                19-Apr-00 (RS)
#
# phy_complete.sh
# =============== 
# Generates a profile report with the following headings:
#
#     Processed PHY profile
#     Length - profile length
#     Incomplete - incomplete data records (record with all zeros)
#     Percentage -  percentage of completion
#
# and compute the percentage of complete data records with a non-zero
# value in a profile. 
#
# Usage
# -----
#       phy_complete.sh  cnt2phy.date_time_list
#
# where  
#     
#       phylist_outfile:  cnt2phy.date_time_list, success list generated
#                         by  program cnt2phy.x
###########################################################################
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

now=`date +'%h %d, %Y %T %p'`
cnt2phy_today_list=$1

echo "<================== Start  $script_name: $now ===============>"
echo "Entering Script: $0 "
echo "to determine profile(s) completion of profile(s) "

error=$ARGOSLOG_DIR/$root_script_name.$TODAY"_"error
if [ -f $error ]
then
  rm -f $error
fi                 
exec 2>> $error

# -------------------------------------------------------------------

# Does the input float list file (cnt2phy.date_time_list) exists?
# ---------------------------------------------------------------
if [ -f $cnt2phy_today_list ]
then                                      # Yes, it exissts
  if [ ! -s  $cnt2phy_today_list ]    # Is it empty?
  then                                    # Yes, EXIT
    msg="NOTICE: `basename $0`:No Entries in $cnt2phy_today_list"
    echo $msg
    echo msg >> $error
    exit 0
  fi
else
  msg="NOTICE: `basename $0`: List File ($cnt2phy_today_list)  DOES NOT EXISTS\n"
  echo $msg
  echo $msg  >> $error
  exit 0 
fi

# ------------------------------------------------------------------------
#    Loop over PHY files to determine profiles completion percentage 
# ------------------------------------------------------------------------
echo "List of PHY files to be checked for completion to be sent to SEAS/GTS located"
echo "in $cnt2phy_today_list"

npassed=0
nprof=0

# Heading for PHY2GTS_LIST Summary Report
# ---------------------------------------
head="\n\t+++++ Argos Profiles Completion Report Generated by PHY_COMPLETE Script +++" 
echo $head > $SUMMARY_PHY_COMPLETE
echo $head > $SUMMARY_PHY_EMAIL
head=`echo $HEAD_DATE_TIME | awk '{print $2 " " $3 " " $4}'`
echo "\t\t\t" \ \ \  "+++ $head +++"  >> $SUMMARY_PHY_COMPLETE
head="\n\tLocation: $PHYFILES_DIR"
head=$head"\n\tIncomplete = profile data record with all zero values"
echo $head >> $SUMMARY_PHY_COMPLETE
echo $head >> $SUMMARY_PHY_EMAIL

printf '\n\n\tProcessed PHY Profile       Length    Incomplete    Completion\n' \
       >> $SUMMARY_PHY_COMPLETE
printf '\n\n\tProcessed PHY Profile       Length    Incomplete    Completion\n' \
       >> $SUMMARY_PHY_EMAIL

echo "Processing Files:"


for phy_infile in `cat $cnt2phy_today_list`
do
  profile_id=`echo $phy_infile | awk -F"." '{print $1}'`
  filepathname=$PHYFILES_DIR/$phy_infile
  wmo=`grep "WMO ID NUMBER" $filepathname | awk '{print $4}'`
  echo "\t$filepathname \t WMO: $wmo"

  if [ -f $filepathname ]                               # Does the file exist?
  then                                                  #   yes, calculate % good data 
    nprof=`expr $nprof + 1`

    last=`grep -ni "=====================" $filepathname | tail  -1 | awk -F':' '{print $1}'`
    sed 1,$last'd' $filepathname  > $ARGOSTMP_DIR/profile.phy  # delete 1-last       

    n_all=`wc -l $ARGOSTMP_DIR/profile.phy | awk '{print $1}'` 
    n_zero=`egrep -vc '[1-9]' $ARGOSTMP_DIR/profile.phy`       # Count all lines except those
                                                               # that contains the pattern.
                                                               # Namely, line with only zeros
    n_complete=`expr $n_all - $n_zero`

    if [ $n_all -eq 0 ] 
    then
      if [ $n_zero -eq 0 ]
      then
        percent=100
      else
        percent=0
      fi
    else                                                 # percent complete rounded off
      echo "scale=1; (100*$n_complete/$n_all) + 0.5" | bc > $ARGOSTMP_DIR/percent  
      percent=`cat $ARGOSTMP_DIR/percent | awk -F"." '{print $1}`
      rm -f $ARGOSTMP_DIR/percent
    fi

    printf '\t%-20s         %4d      %4d           %3d%c\n' \
             $profile_id $n_all $n_zero $percent "%"  >> $SUMMARY_PHY_COMPLETE

    if [ $n_complete -le 10 -o $n_zero -ne 0 ]          # Include for Email? 
    then                                                #   Yes
      printf '\t%-20s         %4d      %4d           %3d%c\n' \
              $profile_id $n_all $n_zero $percent "%" >> $SUMMARY_PHY_EMAIL      
    fi
  else                                     #   no, out message 
    msg="NOTICE: PHY file ($filepathname) DOES NOT EXIST"
    echo $msg >> $error
    echo $msg
  fi

done

echo "Total PRV Files Processed: $nprof \tPassed Completion Test: $npassed"

if [ -f $ARGOSTMP_DIR/profile.phy ]
then
  rm -f $ARGOSTMP_DIR/profile.phy
fi

# Does script error file exists?
# ------------------------------
if [ -f $error ]                           # does it exist?
then
  if [ -s $error ]
  then                                     # delete if empty
    msg="<----------------------------------- "
    msg=$msg" $script_name "
    msg=$msg"----------------------------------->"
    echo "$msg" >> $phy_complete_stderror
    cat  $error >> $phy_complete_stderror
  fi
  rm -f $error
fi

now=`date +'%h %d, %Y %T %p'`
echo "<================== Finish $script_name: $now ===============>"

exit 0 
