#!/bin/sh
############################################################################################
# Revision Date: 19-Nov-09 (ER - add to the email text case in which the hexadecimal file
#                                fails)
#                09-Feb-09 (RS - add hex_noaomlid_dir)
#                09-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                21-Feb-06 (XX - compare the position lines between newly generated hex 
#                                with old one make modification properly based on some 
#                                criteria. Initialize success_prv_list & failure_prv_list)
#                06-Dec-05 (RS - reduce prv2hex processing time for sorting the float
#                                hex files by using hex files in HEXFILES_RECENT_DIR)
#                15-May-03 (RS - improve mail subject)
#                19-Mar-02 (JZ - gunzip prv file if necessary)
#                02-Nov-01 (RS - eliminate prv_input_file_names redundancy)
#                10-Jul-01 (RS - argos id expansion)
#                09-Jul-01 (RS - sort cross reference list file)
#                08-Jan-01 (RS - create list for ARGOS ID not found in
#                           cross reference file)
#                08-Nov-00 (RS - prv list too long, use xarg)
#                26-Aug-00 (RS - create PI directory, if necessary)
#                07-Aug-00 (RS - write to file the PRV connection cycle Date/Time)
#                02-Aug-00 (RS -  rm error file correction)
#                14-Jul-00 (RS - correct failure print to summary)
#                20-Jun-00 (RS - new directory tree)
#                02-Feb-00 (RS)
#
# Executes prv2hex.x
############################################################################################
# -------------------------------------------------------------------------
# GetHexFileNamesFromPRVFile (XX)
# ==========================
# Function to extract hex filenames from a prv file
# -------------------------------------------------------------------------
GetHexFileNamesFromPRVFile()
{  
  printf "=> GetHexFileNamesFromPRVFile: $1 - "
  java -classpath $CLASSPATHLIB GetHexFileNamesFromPrv $1 $2 $3
  status=$?
  if [ $status -eq 0 ]
  then
      for hexfile in `cat $3` 
      do 
         if [ -f $hex_outdir/$hexfile ]
	 then
            cp -p $hex_outdir/$hexfile $hex_tmp_dir
	 fi   
      done
      echo "OK"
      return 0
  else
      echo "FAILURE, extracting hex filenames (java status=$status)"
      return 1
  fi
  
}
# -------------------------------------------------------------------------
# CleanFixHex (XX)
# ===========
# Function to compare new hex file with old hex files and make modifcation 
# based on some criteria
# -------------------------------------------------------------------------
CleanFixHex()
{  
   printf "=> CleanFixHex: $1 - "
   java -classpath $CLASSPATHLIB CleanHex $1 $2 $3
   status=$?
   if [ $status -eq 0 ]
   then
      echo "OK"
      echo $1 >> $success_fixhex_list
   else
      echo "FAILURE,  cleaning hex file (status=$status)"
      echo $1 >> $failure_fixhex_list
   fi
}
# ----------------------------------- Main ---------------------------------
script_name=`basename $0`
prgname=`echo $script_name | awk -F"." '{print $1}'`
root_script_name=$prgname
up_prgname=`echo $prgname | tr "[:lower:]" "[:upper:]"`

now=`date +'%h %d, %Y %T %p'`
echo "<====================== Start  $script_name: $now ====================>"
echo "Entering Script: $0 to excute $prgname"

error=$ARGOSLOG_DIR/$root_script_name.$TODAY"_"stderror
if [ -f $error ]
then
  rm -f $error
fi
exec 2>>$error

# -------------------------------------------------------------------
#                PRV Date/Time Global Variables
# Write the PRV date/time for the last PRV connection cycle processed. 
# This datetime stamp is use to determine which QC profiles will be
# converted to GTS format and trasmitted (FTP) to SEAS (GTM gateway)
# --------------------------------------------------------------------
prv_last_pathname=`tail -1 $prv_input_file_names`
prv_last_filename=`basename $prv_last_pathname`
echo "$prgname: prv_last_filename= $prv_last_filename"

prv_date=`echo $prv_last_filename | awk -F"." '{print $2 "/" $3 "/" $4}'`
prv_time=`echo $prv_last_filename | awk -F"." '{print $5 ":" $6}'`

echo $prv_date $prv_time > $argosconnection_datetime_file
echo "$prgname: prv_date= $prv_date prv_time= $prv_time"
echo "\tare written to: $argosconnection_datetime_file"

# -----------------------------------------------------------------
#                   Processing PRV File(s) 
# -----------------------------------------------------------------
trace_outfile=$ARGOSLOG_DIR/$prgname.$TODAY"_"trace
error_outfile=$ARGOSLOG_DIR/$prgname.$TODAY"_"error
list_outfile=$ARGOSLIST_SUCCESS_DIR/$prgname.$TODAY"_"list
list_crossref_noentry=$ARGOSLIST_SUCCESS_DIR/crossref_noentry.$TODAY"_"list
list_crossref_noentry_sorted=$ARGOSTMP_DIR/crossref_noentry_sorted.$TODAY"_"list

floatid_infile=$FLOATIDFILES_DIR/floatno_argoid_list
#RS 06DEC05 hex_outdir=$HEXFILES_DIR
hex_outdir=$HEXFILES_RECENT_DIR

#For hex files without 'aomlid'
hex_noaomlid_dir=$HEXFILES_DIR/noaomlid

########### newly added (XX - 21-Feb-06)
hex_tmp_dir=$HEXFILES_RECENT_DIR/tmp #directory to store the previous hex files before a new run
list_hexfiles_inaprv=$ARGOSTMP_DIR/list_hexfiles_inaprv.$TODAY"_"list
CLASSPATHLIB=$ARGOSPRG_DIR
success_fixhex_list=$ARGOSTMP_DIR/success_fixhex_list.$TODAY"_"list
failure_fixhex_list=$ARGOSTMP_DIR/failure_fixhex_list.$TODAY"_"list

if [ ! -d $hex_tmp_dir  ]
then
    mkdir $hex_tmp_dir
fi
###########

n_bytes=32
success_prv=0
failure_prv=0
success_prv_list=$ARGOSTMP_DIR/$root_script_name.$TODAY"_"success
failure_prv_list=$FAILURE_LIST_DIR/$root_script_name.$TODAY"_"fail
touch $success_prv_list $failure_prv_list

head="\t+++++ "
head=$head"Summary Report of Argos PRV Files Procesed by $up_prgname Script"
head=$head" +++++"
head=$head"\n$HEAD_DATE_TIME"
head=$head"\n\n\tLocation: $PRVFILES_DIR\n"
echo $head >> $SUMMARY_PRV2HEX
printf '\t       PRV File             Success   Failure\n' >> $SUMMARY_PRV2HEX
echo $head >> $SUMMARY_PRV2HEX_EMAIL
printf '\t       PRV File             Success   Failure\n' >> $SUMMARY_PRV2HEX_EMAIL

for prv_infile in `cat $prv_input_file_names`
do
  now=`date +'%h %d, %Y %T %p'`
  echo "<------------------------ Begin PRV $now --------------------------->" 
  $ARGOSCRIPTS_DIR/check_decompress.sh $prv_infile
  STATUS="$?"
  if [ $STATUS -eq 1 ]
  then 
    tmp_file=`echo $prv_infile | awk -F '{print substr($1,0,length($1)-3)}'`
    prv_infile=$tmp_file
  elif [ "$STATUS" = "9" ]
  then
    echo ">>> $up_prgname: Script check_decompress.sh Failed <<<"
    failure_prv=`expr $failure_prv + 1`
    echo `basename $prv_infile` >> $failure_prv_list
    printf '\t%-25s      0         1\n' $prv_name >> $SUMMARY_PRV2HEX
    printf '\t%-25s      0         1\n' $prv_name >> $SUMMARY_PRV2HEX_EMAIL
    continue 
  fi

  prv_name=`basename $prv_infile`
  prgnumber=`echo $prv_name | awk -F"." '{print $1}' | awk -F"v" '{print $2}'` 
  echo "Processing file: $prv_name for Program Number: $prgnumber"   
  echo "PRV2HEX list_outfile: $list_outfile"   # contains only the file names

  # Extract hex filenames from a prv file
  # -------------------------------------
  GetHexFileNamesFromPRVFile $prv_infile $floatid_infile $list_hexfiles_inaprv
  status=$?
  if [ $status -ne 0 ]
  then 
    failure_prv=`expr $failure_prv + 1`
    echo `basename $prv_infile` >> $failure_prv_list
    continue 
  fi
  
  $ARGOSPRG_DIR/$prgname.x $prgnumber $prv_infile $floatid_infile $hex_outdir \
                $trace_outfile $error_outfile \
                $list_outfile $n_bytes $list_crossref_noentry $hex_noaomlid_dir
  STATUS="$?"
  if [  "$STATUS" = "0"  ]
  then                                   # Success: c program execution ok
    echo ">>> $up_prgname Program Execution OK <<<"  
    success_prv=`expr $success_prv + 1`
    name=`basename $prv_infile`
    echo $name >> $success_prv_list   
    year=`echo $name | awk -F"." '{print $2}'`
    printf '\t%-25s      1         0\n' $prv_name >> $SUMMARY_PRV2HEX
    printf '\t%-25s      1         0\n' $prv_name >> $SUMMARY_PRV2HEX_EMAIL

    done_pidir=$DONE_PRVFILES_ARGO_DIR/$prgnumber"_"done
    if [ ! -d $done_pidir ]              # Does PI dir exists?
    then                                 #   no, create it
      mkdir $done_pidir
    fi

#   done_prvdir=$DONE_PRVFILES_ARGO_DIR/$prgnumber"_"done/$year
    done_prvdir=$done_pidir/$year
    if [ ! -d $done_prvdir ]             # Does PI/Year dir exists?
    then                                 #   no, create it
      mkdir $done_prvdir
    fi
    mv $prv_infile $done_prvdir
#?  touch  $done_prvfile                 #? ASK Claudia

  else                                   # Failure: Execution terminated abnormally
    echo ">>> $up_prgname: Execution Failed <<<" 
    failure_prv=`expr $failure_prv + 1`
    echo `basename $prv_infile` >> $failure_prv_list   
    printf '\t%-25s      0         1\n' $prv_name >> $SUMMARY_PRV2HEX
    printf '\t%-25s      0         1\n' $prv_name >> $SUMMARY_PRV2HEX_EMAIL
  fi

  now=`date +'%h %d, %Y %T %p'` 
  echo "<------------------------ End   PRV $now --------------------------->" 
done

# ----------------------------------------------------------------
#   Sort list files: success_prv_list, failure_prv_list & output_list 
#   and eliminates duplicate entries. The list_outfile is produced
#   by prv2hex with hex files.
# ----------------------------------------------------------------
echo $failure_prv > $nprv_failures_file
echo "PRV2HEX - success_prv: $success_prv failure_prv: $failure_prv"

if [ $success_prv -eq 0 ]                               # Any successes?
then                                                    # No => all failures
  ret_val=0
  sort -u  $failure_prv_list -o $failure_prv_list 
  echo "PRV2HEX - SORTED:"
  echo "                  $failure_prv_list"
elif [ $failure_prv -eq 0 ]                             # Yes, any failures?
then                                                       # No  => all successes
  ret_val=$success_prv
  echo "list_outfile: $list_outfile"
  ls -l $list_outfile
  sort -u  $success_prv_list -o $success_prv_list
  sort -u  $list_outfile     -o $list_outfile  
  echo "PRV2HEX - Sorted Success List:"
  echo "\t\t$success_prv_list"
  echo "          Output File Name List is: $list_outfile"
else                                                       # Yes => succeses & failures
  ret_val=$success_prv                                  
  sort -u  $success_prv_list -o $success_prv_list
  sort -u  $failure_prv_list -o $failure_prv_list 
  sort -u  $list_outfile     -o $list_outfile  
  echo "PRV2HEX - Sorted Failure List:"
  echo  "\t\t$success_prv_list"
  echo  "\t\t$failure_prv_list"
  echo "          Output File Name List is: $list_outfile"
fi
rm -f $success_prv_list

# -----------------------------------------------------------------
#   Report Number of Successes & Failures via Email and/or Report
# -----------------------------------------------------------------
n_prv=`expr $success_prv + $failure_prv`
printf '\n\t\t\tSubtotals     %2s        %2s\n' $success_prv $failure_prv \
     >> $SUMMARY_PRV2HEX
printf '\t\t\t   Totals\t\t\t%s\n' $n_prv >> $SUMMARY_PRV2HEX
printf '\n\t\t\tSubtotals     %2s        %2s\n' $success_prv $failure_prv \
     >> $SUMMARY_PRV2HEX_EMAIL
printf '\t\t\t   Totals\t\t\t%s\n' $n_prv >> $SUMMARY_PRV2HEX_EMAIL

if [ $failure_prv -gt 0 ]
then
  msg="\nArgos PRV Files Processed Unsuccessfully:"
  echo $msg               >> $SUMMARY_PRV2HEX
  cat  $failure_prv_list  >> $SUMMARY_PRV2HEX
  echo $msg               >> $SUMMARY_PRV2HEX_EMAIL
  cat  $failure_prv_list  >> $SUMMARY_PRV2HEX_EMAIL

  # add to the email case in which the hexadecimal file fails
  grep "prv2hex.c : compress function return" $error_outfile >>$SUMMARY_PRV2HEX
  grep "prv2hex.c : compress function return" $error_outfile >>$SUMMARY_PRV2HEX_EMAIL

  msg="\nFor details on program $up_prgname.x execution look at the following "
  msg=$msg"Trace & Error logfiles:" 
  echo $msg               >> $SUMMARY_PRV2HEX
  echo "  $trace_outfile" >> $SUMMARY_PRV2HEX
  echo "  $error_outfile" >> $SUMMARY_PRV2HEX
  echo $msg               >> $SUMMARY_PRV2HEX_EMAIL
  echo "  $trace_outfile" >> $SUMMARY_PRV2HEX_EMAIL
  echo "  $error_outfile" >> $SUMMARY_PRV2HEX_EMAIL
fi

# Is list_crossref_noentry file empty?
# ------------------------------------
if [ -s $list_crossref_noentry ]
then
  msg="\nArgos float number missing in cross reference file ($floatid_infile):"
  echo $msg
  sort $list_crossref_noentry -o $list_crossref_noentry_sorted
  cat $list_crossref_noentry_sorted
  echo $msg                         >> $SUMMARY_PRV2HEX
  cat $list_crossref_noentry_sorted >> $SUMMARY_PRV2HEX
  echo $msg                         >> $SUMMARY_PRV2HEX_EMAIL
  cat $list_crossref_noentry_sorted >> $SUMMARY_PRV2HEX_EMAIL
  rm -f $list_crossref_noentry_sorted
fi
rm -f $list_crossref_noentry

# -------------------------------------------------------
#   Add logic to compare the positions lines
# -------------------------------------------------------
if [ -f $list_outfile ]
then
  for hexfile in `cat $list_outfile`
  do
     if [ -f $hex_outdir/$hexfile -a -f $hex_tmp_dir/$hexfile ]
     then
       CleanFixHex $hex_outdir/$hexfile $hex_tmp_dir/$hexfile $hex_outdir/$hexfile
     fi  
  done
fi  

# Does script error file exists?
# ------------------------------
if [ -f $error ]                           # does it exist?
then
  if [ -s $error ]
  then                                     # file not empty
    msg="\nFor details on script $script_name execution look at the following "
    msg=$msg"Error logfile:" 
    echo $msg   >> $SUMMARY_PRV2HEX
    echo $error >> $SUMMARY_PRV2HEX
    echo $msg   >> $SUMMARY_PRV2HEX_EMAIL
    echo $error >> $SUMMARY_PRV2HEX_EMAIL
  else
    rm -f $error
  fi
fi

now=`date +'%h %d, %Y %T %p'`
echo "<====================== Finish $script_name: $now ====================>"

exit $ret_val
