#!/bin/sh
###############################################################################
# Revision Date: 12-Sep-07 (RS - Trap WARNINGS messages in the profile 'trace'
#                                file and write to 'warning.[timestamp]_log'
#                                for email reporting)
#                08-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                02-May-02 (RS -add remove_ghosts.sh messages)
#                20-Mar-00 (RS)
#                09-Jun-00 (RS)
#
# report_metadata_stderror
# ========================
# Reports:  floats with no metadata
#           floats with no float type 
#           error messages produced by 'remove_ghosts.sh'
#           argos_process.sh standard-error (stderr) output file
#           
###############################################################################
script_name=$1

# Are there any metadata file(s) missing?
# ---------------------------------------
if [ -f $no_metadata_list ]
then
  if [ -s $no_metadata_list ]
  then
    msg="\nMetadata file not available for the following float(s):"
    echo $msg             >> $SUMMARY_REPORT
    cat $no_metadata_list >> $SUMMARY_REPORT
    echo $msg             >> $EMAIL_MESSAGE
    cat $no_metadata_list >> $EMAIL_MESSAGE
    echo $msg
    cat $no_metadata_list
  fi
  rm -f $no_metadata_list
fi

# Are there any profile with missing float type in metafile"
# ---------------------------------------------------------
if [ -f $noinfo_in_metadata_list ]
then
  if [ -s $noinfo_in_metadata_list ]
  then
    msg="\nMetadata file(s) with missing float type:"
    echo $msg                    >> $SUMMARY_REPORT
    cat $noinfo_in_metadata_list >> $SUMMARY_REPORT
    echo $msg                    >> $EMAIL_MESSAGE
    cat $noinfo_in_metadata_list >> $EMAIL_MESSAGE
    echo $msg
    cat $noinfo_in_metadata_list
  fi
  rm -f $noinfo_in_metadata_list
fi

# Does the file with remove_ghosts email messages exits?
# -------------------------------------------------------
if [ -f $remove_ghosts_msg ]  
then
  if [ -s $remove_ghosts_msg ]  
  then
    msg="\nError(s) produced when removing data cycles in 'ghost' file(s) are:"
    echo $msg               >> $SUMMARY_REPORT
    cat $remove_ghosts_msg  >> $SUMMARY_REPORT
    echo $msg               >> $EMAIL_MESSAGE
    cat $remove_ghosts_msg  >> $EMAIL_MESSAGE
    echo $msg
    cat $remove_ghosts_msg
    msg="\nSee logfiles:"
    msg=$msg"\n\t$ARGOSLOG_DIR/remove_ghosts.$TODAY""_trace"
    msg=$msg"\n\t$ARGOSLOG_DIR/remove_ghosts.$TODAY""_error"
    echo $msg
    echo $msg              >> $SUMMARY_REPORT
    echo $msg              >> $EMAIL_MESSAGE
  fi
  rm -f $remove_ghosts_msg
fi

# Does script standard_error output file exists?
# ----------------------------------------------
if [ -f $argos_process_stderror ]
then
  if [ -s $argos_process_stderror ]
  then
    msg="\nFor the details on the standard-error output (stderr) generated"
    msg=$msg" by '$script_name' \nlook at the error logfile: $argos_process_stderror"
    echo $msg >> $SUMMARY_REPORT
    echo $msg >> $EMAIL_MESSAGE
    echo $msg
  else
    rm -f $argos_process_stderror
  fi
fi

if [ -f $warning_log ]
then
  if [ -s $warning_log ]
  then
    msg="\nWarning(s) reported:"
    echo $msg               >> $SUMMARY_REPORT
    cat $warning_log >> $SUMMARY_REPORT
    echo $msg               >> $EMAIL_MESSAGE
    cat $warning_log >> $EMAIL_MESSAGE
    echo $msg
    cat $warning_log
  fi
fi

exit
