#!/bin/sh
###################################################################################
# Revision Date: 05-Dec-12 (RS - modified documentation for the location of the
#                                HEXFILES_BEFRE_SPLIT_DIR)
#                10-Aug-06 (RS - improved doc & remove hex file when it does not
#                                contain data for current year)
#                01-Aug-06 (RS - revised)
#                07-DEC-05 (RS) 
#                22-Jun-05 (XXD)
#
# split_hex.sh
# ============
# Splits HEX files into two files, one contains only the current year's data that
# will be stored in HEXFILES_RECENT_DIR and the other contains the previous year's
# data that will be stored in HEXFILES_HISTORICAL_DIR. There are two mode of
# operation as follows: 
#
#  (1) Initial Run - splits the hex files in HEXFILES_DIR 
#  (2) Sequential runs - splits hex files in HEXFILES_RECENT_DIR
# 
# Before spliting, the hex file is copied to HEXFILES_BEFORE_SPLIT_HISTORICAL 
# If split is successfull, the current year file is moved to HEXFILES_RECENT_DIR 
# and file containing previous year(s) data is append to the HEX file in the
# HEXFILES_HISTORICAL_DIR.
# 
# Usage
# -----
#       unix>split_hex.sh
#
# Notes
# -----
# (1) This script should be run once each year (June 1)
# (2) HEXFILES_BEFORE_SPLIT_DIR = /WSmounts/b1/HEXFILES_BEFORE_SPLIT (on 'argo').
#     Variable is initialized in $HOME/.argorc & $HOME/.argorc_sh 
# (3) HEX files in HEXFILES_RECENT_DIR contain data for current year   
# (4) HEX files in HEXFILES_HISTORICAL_DIR contain data for previous years  
###################################################################################
. $HOME/.argorc_sh 

#-------------------------- Initialize Variables -----------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`
TargetRunMonthDay=101

script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

ARGOS_DIR=$ARGO_DIR/argos
ARGOSDATA_DIR=$ARGOS_DIR/data
BEFORE_SPLIT_HIST_DIR=$HEXFILES_BEFORE_SPLIT_DIR/historical 
BEFORE_SPLIT_RECENT_DIR=$HEXFILES_BEFORE_SPLIT_DIR/recent
HEXFILES_DIR=$ARGOSDATA_DIR/hex
HISTORICAL_DIR=$HEXFILES_DIR/historical
RECENT_DIR=$HEXFILES_DIR/recent
SCRIPTS_DIR=$ARGOS_DIR/scripts
LOG_DIR=$ARGOS_DIR/log
TMP_DIR=$ARGOS_DIR/tmp

logfile=$LOG_DIR/$root_script_name.$TODAY"_log"
error=$LOG_DIR/$root_script_name.$TODAY"_error"
exec 1> $logfile
exec 2> $error

echo "=> Start $script_name" > /dev/tty 
echo "=> Start $script_name"  

mm=`date +'%m'`
dd=`date +'%d'`
month=`printf "%2s\n" $mm`

mmdd=`expr $month \* 100`
mmdd=`expr $mmdd + $dd`

echo "   TargetRunMonthDay: $TargetRunMonthDay"
echo "   Today's MonthDay: $mmdd"

if [ $mmdd -lt $TargetRunMonthDay ]
then
  echo "   >>>>> Must Run Job after May 31 - Exit <<<<<"
  echo "          Implement Email for Cron Job "
  echo "=> Job Terminated"
  exit
else
  echo "   OK - Split HEX Files"
fi

echo " "
echo "   Directories & Logfiles:"  
echo "   ARGOS_DIR: $ARGOS_DIR"
echo "   BEFORE_SPLIT_DIR: $HEXFILES_BEFORE_SPLIT_DIR"
echo "   BEFORE_SPLIT_HIST_DIR: $BEFORE_SPLIT_HIST_DIR"
echo "   BEFORE_SPLIT_RECENT_DIR: $BEFORE_SPLIT_RECENT_DIR"
echo "   HEXFILES_DIR: $HEXFILES_DIR"
echo "   HISTORICAL_DIR: $HISTORICAL_DIR"
echo "   RECENT_DIR: $RECENT_DIR"
echo "   SCRIPTS_DIR: $SCRIPTS_DIR"
echo "   LOG_DIR: $LOG_DIR" 
echo "   TMP_DIR: $TMP_DIR" 

echo ""
echo "   logfile: $logfile"
echo "   error: $error"

current_year=`date +'%Y'`

echo "   ------------------------- Split Year: $current_year -------------------------"
if [ "`ls $RECENT_DIR/*.hex`" = "" ]     # Is it empty?
then                                     #   Yes, only for 1st split run
  echo "   First Run - HEX files to split are in $HEXFILES_DIR"
  hexdir2split=$HEXFILES_DIR
else                                     #   No, subsequent split runs
  echo "   Subsequent Run - HEX files to split are in $RECENT_DIR"
  hexdir2split=$RECENT_DIR
fi

# Change to directory with files to split
# --------------------------------------- 
cd $hexdir2split
if [ "`ls *.hex`" = "" ]
then
  echo "   >>>>> No HEX files found - Exit <<<<<"
  echo "   Take appropiate action and run again"
  echo "=> Job Terminated"
  exit
fi

failure=0
for hex_filename in `ls *.hex`
do 
   echo "   <<< $hex_filename >>>"
   if [ $hexdir2split = $HEXFILES_DIR ]
   then
     printf "     Copy to $BEFORE_SPLIT_HIST_DIR - "   # Save in before split historical dir 
     cp -p $hex_filename $BEFORE_SPLIT_HIST_DIR  
     status=$?
   else
     printf "     Copy to $BEFORE_SPLIT_RECENT_DIR - " # Save in before split recent dir 
     cp -p $hex_filename $BEFORE_SPLIT_RECENT_DIR
     status=$?
     if [ -f $HISTORICAL_DIR/$hex_filename ]
     then
       cp -p $HISTORICAL_DIR/$hex_filename $BEFORE_SPLIT_HIST_DIR 
       status=`expr $status + $?`
     fi
   fi
   if [ $status -eq 0 ]
   then
     echo "OK"
   else
     echo "FAILURE -> DO NOT Split"
     failure=1
     continue
   fi

   printf "     Get data PRIOR to $current_year & Append to HISTORICAL_DIR - "
   grep -v " $current_year " $hex_filename >> $HISTORICAL_DIR/$hex_filename
   if [ $? -eq 0 ]                              # Any data?
   then                                         #   Yes, append to production hist file
     echo "OK"
   else
     echo "NO DATA FOUND" 
     if [ ! -s $HISTORICAL_DIR/$hex_filename ]  # Is empty?
     then                                       #  Yes, remove
       printf "       Remove EMPTY output from grep: $HISTORICAL_DIR/$hex_filename - "
       rm -f $HISTORICAL_DIR/$hex_filename 
       echo "OK" 
     fi
   fi

   printf "     Get $current_year Data - "
   grep " $current_year " $hex_filename > $TMP_DIR/$hex_filename
   if [ $? -eq 0 ]                             # Any Data?
   then                                        #   Yes
     printf "OK, Move file to RECENT_DIR  - "
     if [ $hexdir2split = $RECENT_DIR ]        # HEX file in HEXFILES_RECENT_DIR?  
     then                                      #  Yes, current dir is RECENT_DIR
       mv $TMP_DIR/$hex_filename .             #   Move current year file to current dir (RECENT_DIR)  
     else                                      #  No, current dir is HEXFILES_DIR
       mv $TMP_DIR/$hex_filename $RECENT_DIR   #   Move current year data to RECENT_DIR   
     fi
     echo "OK"
   else                                        #  No, remove empty file
     echo "NO DATA FOUND, Remove the following files: "
     printf "       EMPTY output from grep: $TMP_DIR/$hex_filename - "
     rm -f $TMP_DIR/$hex_filename
     echo "OK"
     printf "       NO $current_year DATA: $hexdir2split/hex_filename  - " 
     rm -f $hex_filename
     echo "OK"
   fi

   if [ $hexdir2split = $HEXFILES_DIR ]        # Did argos/data/hex had file(s) to split - 1st run?  
   then                                        #  Yes, for the 1st split run
     if [ -f $hex_filename ]
     then
       printf "     Remove $HEXFILES_DIR/$hex_filename  - "  # Saved BEFORE_SPLIT_HIST_DIR before split  
       rm -f $hex_filename                      
       echo "OK"
     fi
   fi
done

if [ -s $ error ]
then
  failure=2
else
  rm -f $error
fi

if [ $failure -ne 0 ]
then
  echo "***********************************************************************" > /dev/tty
  echo "                       Failure Reported" > /dev/tty
  echo "  See, logfile(s):" > /dev/tty
  echo "    $logfile"       > /dev/tty
  if [ $failure -eq 2 ]
  then
    echo "    $error"       > /dev/tty
  fi 
  echo "***********************************************************************" > /dev/tty
fi

echo "=> Finish $script_name" 
echo "=> Finish $script_name" > /dev/tty

exit
