#!/bin/sh
# ===========================================================================================================
# Revision Date: 04-Dec-13 (RS,CS - The following additions and modifications were done: 
#                                   . Revised doumentation
#                                   . Allowed FTP incoming Iridium data from: 
#                                     UW: srf - surface data while float is drifting  
#                                         sts - surface temperarure and saliity
#                                     NAVO: msg - iridium profile message
#                                   . Changed crontab: rsio to rsio_i_phy  & rwhio to rwhio_phy
#                                   . Remove code to execute on of the following 'kermit' scripts
#                                     based on 'hostname':
#                                       FTPSCRIPT_DIR/get_files.kermitcmd_file - don't delete remote files
#                                       FTPSCRIPT_DIR/get_delete_files.kermitcmd_file - delete remote files
#                                     Now, the second scripts is executed) 
#                28-Aug-13 (RS -  rsio 'phy' is now saved in iridium/data/phy)
#                21-Oct-12 (RS - when running in 'nuargo' do not delete
#                                downloaded files. File needed for running
#                                in 'nuargo' at a later time) 
#                15-Aug-12 (RS - when running in 'nuargo2' do not delete  
#                                downloaded files. File needed for running
#                                in 'nuargo' at a later time) 
#                10-Aug-12 (RS - add Iridium 'msg' files for WHOI)
#                05-Jul-12 (CS - corrected "extension" under ruw_i_log (was 'msg', should have been 'log')
#                20-Jun-11 (RS - add Iridium 'log' files for PMEL & UW)
#                12-Aug-10 (ER - phy files FTP'd from Scripts to be added to
#                                a cnt2phy list in order to be QC'd
#                14-May-10 (ER - add Iridium floats from Scripts (phy files)
#                04-Feb-10 (ER - add Iridium floats from UW (msg files)
#                              - add Iridium floats from PMEL (msg files) )
#                              - change target directory for Iridium floats from UW (phy files)
#                              - change target directory for Iridium floats from PMEL (phy files)
#                18-Oct-06 (RS - add Iridium floats for WHOI)
#                04-Jan-06 (RS - add ftp for Iridium floats from UW)
#                20-Oct-03 (RS - modified email subjects)
#                08-May-03 (RS)
#                   
# ftp_from_aomlincoming.sh
# ------------------------
# Uses KERMIT-FTP to download file(s) from an specified location under the
# AOML FTP Argo incoming directory to an assigned directory.  The assigned
# directory is based on type of data to download (real-time) and its
# source (e.g., NAVO, PEML, SIO, UW or WHOI).
#
# FTPARGO_INCOMING_DIR=/phod/incoming/argo -> /phod/incoming/ARGO_FTP/argo (set in argo.init)
# REALTIME_DIR=FTPARGO_INCOMING_DIR/real-time
# IRIDIUM_DATA_DIR=ARGO_DIR/iridium/data 
# ORBCOMM_DATA_DIR=ARGO_DIR/orbcomm/data 
#
# Data type allow, incoming directory (FROM_DIR) and target directory (TO_DIR):
#
#                                Incoming Directory  Assigned Directory
#  Type   Source     Code            (FROM_DIR)           (TO_DIR)          Cron 
#  ----   ------     ----       -------------------  --------------------   ----
#   r     navo     rnavo_i_msg  REALTIME_DIR/navo    IRIDIUM_DATA_DIR/msg   Yes
#   r     pmel     rpmel_i_log  REALTIME_DIR/pmel    IRIDIUM_DATA_DIR/log   Yes 
#   r     pmel     rpmel_i_msg  REALTIME_DIR/pmel    IRIDIUM_DATA_DIR/msg   Yes 
#   r     sio      rsio_i_phy   REALTIME_DIR/sio     IRIDIUM_DATA_DIR/phy   Yes
#   r     uw       ruw_i_log    REALTIME_DIR/uw      IRIDIUM_DATA_DIR/log   Yes 
#   r     uw       ruw_i_msg    REALTIME_DIR/uw      IRIDIUM_DATA_DIR/msg   Yes 
#   r     uw       ruw_i_phy    REALTIME_DIR/uw      IRIDIUM_DATA_DIR/phy   No (Inactive) 
#   r     uw       ruw_i_srf    REALTIME_DIR/uw      IRIDIUM_DATA_DIR/srf   Yes
#   r     uw       ruw_i_sts    REALTIME_DIR/uw      IRIDIUM_DATA_DIR/sts   Yes
#   r     whoi     rwhoi_o      REALTIME_DIR/whoi    ORBCOMM_DATA_DIR/nc    No (Orbcomm) 
#   r     whoi     rwhoi_i_msg  REALTIME_DIR/whoi    IRIDIUM_DATA_DIR/msg   Yes 
#   r     whoi     rwhoi_i_phy  REALTIME_DIR/whoi    IRIDIUM_DATA_DIR/phy   Yes
#
#   Type
#   ---- 
#   r: real-time
#
#   Source
#   ------
#   navo: Navy Oceanographic Office 
#   pmel: Pacific Marine Environmental Laboratory
#   sio: Scripts Institute of Oceanography
#   uw: University of Washingon
#   whoi: Woods Whole Oceanographic Institute
#
# Tasks involved in the KERMIT_FTP Session
# ----------------------------------------
# 1) Establish KERMIT-FTP connection to the FTP Server - Abort if 
#    it can't make connection.
# 2) Login to an FTP Server (FTP_AOML) - Abort if it can't login 
# 3) Change to FROM_DIR - Abort if can't change directory 
# 4) Check if remote directory (FROM_DIR) is empty - Abort if it empty 
# 5) Get remote directory listing (remote_list) - Abort if it can't get it 
# 6) Loops over the remote list file - Abort if it can't open list file 
# 7) Checks for the following failures:
#      . Can't read the remote file - add failure in 'failure_filename_list'  
#      . Transfer Failure - add filename to 'failure_transfer_list'  
#      . Sizes not Equal (remote vs. downloaded - add filename to 'failure_size_list'
#      . Can't Delete Remote File - add filename to 'failure_delete_list' 
# 8) If no failure occurs, remote filename is added to 'success_list' and 
#    remote file will be deleted.
#
# FTPARGO_INCOMING_DIR=/phod/incoming/argo -> /phod/incoming/ARGO_FTP/argo (set in argo.init)
#
# Note -  Remote users must use the login username (???) and  password 
# provided to them.
#
# Usage
# -----
#        unix>ftp_from_aomlincoming.sh  code 
#
# Exit Status Values
# ------------------
#  0, All file transfers OK
#  1, Can't login (wrong username and/or password)
#  2, No connection to Remote FTP Host available
#  3, Can't CD to FTPARGO_INCOMING_DIR
#  4, Remote directory list file is empty  
#  5, Can't get remote open remote list file 
#  6, Can't read remote list file
#  7, No file successfully transfer  
#  8, At least one file was not successfully transfer 
#  9, Kermit Exit Code NOT DEFINED
# 10, Error creating links for viaually QCed files 
# 11, Data source is not valid
# ===========================================================================================================
savethis=`pwd`                  # Needed for init system varialbes
cd $ARGO_DIR/init_scripts
# --------------------------------------------------------------------
#  Revision Date: 24-Apr-03 (handle 2 or more Email addresses)
#                 19-Feb-02 (RS)
#
#  ReadSet_InitVar
#  =============== 
#  Reads and sets ARGO initialization environmental global 
#  variables which are used to set depedent environmental global variables.
#
#  Usage: ReadSet_InitVar
# --------------------------------------------------------------------
ReadSet_InitVar()
{
  echo "--- Global Variables Initialization with Parameters ---" 
  echo "        --- from Input File (argo.init) ---"

  exec < argo.init
  while read envname  envvalue doc
  do
    case $envname in
       EMAIL_OPERATOR)
          opr_1=`echo $envvalue | awk -F"," '{print $1}'`
          opr_2=`echo $envvalue | awk -F"," '{print $2}'`
          value="$opr_1 $opr_2"
          name=$envname
          eval $name=$value
#?        eval echo $name"="$value
          eval export $name;;
      *)
          name=$envname
          value=$envvalue
          eval $name=$value
          eval export $name
          eval echo $name"="$value;;
   esac
  done

  return
}
# ------------------------------------------------------------------
#
# SendMail_Success
# ----------------
# Compose and sends Email when at least one file transfer was OK.
# Category 7 failures could have been detected for other files.
#
# Usage: SendMail_Success
# ------------------------------------------------------------------
SendMail_Success()
{
  echo "=> Enter SendMail_Success"
  ret_val=$1
  cat $failure_msg  > $email_msg
 
  text=""
  if [ $ret_val -ne 0 -a $ret_val -ne 4 ]
  then
    text="See logfiles:\n"
    text=$text"\t$logfile\n"
    text=$text"\t$log_kermit\n"
  fi

  if [ -s $error_kermit  -o -s $error ]
  then
    subj=$subj" - stderror detected"
    subj=`echo $subj | sed s/OK/FAILURE/` 
    if [ $failure -eq 0 ]
    then
      text="See logfiles:\n"
    fi
  fi

  if [ -s $error_kermit ]
    then
    text=$text"\t$error_kermit\n"
  else
    rm -f $error_kermit
  fi

  if [ -s $error ]
  then
    text=$text"\t$error\n"
  else
    rm -f $error
  fi

  printf "$text"   >> $email_msg

  echo "   [Email Begin]"
  echo "   Subj: $subj"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Text:"
  cat  $email_msg | awk '{print "     " $0}'
  echo "   [Email End]"
  printf "   Sending ... \c"
  mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
  echo "OK"

  rm -f $failure_msg $email_msg
#?rm -f $failure_filename_list $failure_transfer_list $failure_size_list
#?rm -f $failure_delete_list $remote_list
  echo "=> Exit SendMail_Success"
  return
}
# ------------------------------------------------------------------
#
# SendMailAbort_Failure
# --------------------- 
# Compose and sends Email when at least one failue was detected 
# causing execution to terminate.
#
# Usage: SendMailAbort_Failure 
# ------------------------------------------------------------------
SendMailAbort_Failure()
{
  echo "=> Enter SendMailAbort_Failure" 
  ret_val=$1
  cat $failure_msg  > $email_msg

  text="See logfiles:\n"
  text=$text"\t$logfile\n"
  text=$text"\t$log_kermit\n"

  if [ -s $error_kermit  -o -s $error ]
  then
    subj=$subj" - stderror detected"
    subj=`echo $subj | sed s/OK/FAILURE/`
  fi

  if [ -s $error_kermit ]
    then
    text=$text"\t$error_kermit\n"
  else
    rm -f $error_kermit
  fi

  if [ -s $error ]
  then
    text=$text"\t$error\n"
  else
    rm -f $error
  fi
  printf "$text"   >> $email_msg
  echo "   [Email Begin]"
  echo "   Subj: $subj"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Text:"
  cat  $email_msg | awk '{print "     " $0}'
  echo "   [Email End]"
  printf "   Sending ... \c"
  mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
  echo "OK"

  rm -f $failure_msg $email_msg
#?rm -f $failure_filename_list $failure_transfer_list $failure_size_list
#?rm -f $failure_delete_list $success_list $remote_list

  now=`date +'%h %d, %Y %T %p'`

  echo "<=========== Terminated $script_name: $now ============>"
  exit $ret_val
}
# ------------------------------------------------------------------
# DownLoad
# ========
# Download files specifies in parameter argument (type_source) from
# AOML FTP site.
#
# Usage: DownLoad type_source data_type 
#
# type_source: 'rwhoi'
# data_type: 'nc'
# ------------------------------------------------------------------
DownLoad()
{
  type_source=$1
  ext=$2

  up_ext=`echo $ext | tr "[:lower:]" "[:upper:]"`
  code=$type_source"_"$ext

  if [ $ext = "vqc" ]
  then
    ext=qc
  fi

  machine=`hostname`
  now=`date +'%h %d, %Y %T %p'`
  echo "=> Enter Function ... Machine: $machine DownLoad - $code "

# -------------------------------------------------------------------
#  FTP file(s) from AOML incoming directory specified by 'code' 
# -------------------------------------------------------------------
  log_kermit=$FTPLOG_DIR/kermit_$code.$TODAY"_log_kermit"
  error_kermit=$FTPLOG_DIR/log_kermit_$code.$TODAY"_error_kermit"

  remote_list=$FTPTMP_DIR/remote_list_$code.$TODAY
  success_list=$FTPTMP_DIR/success"_"$code.$TODAY"_list"
  failure_filename_list=$FTPTMP_DIR/failure_filename"_"$code.$TODAY"_list"
  failure_transfer_list=$FTPTMP_DIR/failure_transfer"_"$code.$TODAY"_list"
  failure_size_list=$FTPTMP_DIR/failure_size"_"$code.$TODAY"_list"
  failure_delete_list=$FTPTMP_DIR/failure_delete"_"$code.$TODAY"_list"

  touch $log_kermit $error_kermit
  touch $remote_list $success_list $failure_filename_list $failure_transfer_list
  touch $failure_size_list $failure_delete_list

  echo "   Logfiles and Lists:"
  echo "   log_kermit: $log_kermit"
  echo "   error_kermit: $error_kermit"
  printf   "\n"
  echo "   remote_list:  $remote_list"
  echo "   success_list: $success_list"
  echo "   failure_filename_list: $failure_filename_list"
  echo "   failure_transfer_list: $failure_transfer_list"
  echo "   failure_size_list: $failure_size_list"
  echo "   failure_delete_list: $failure_delete_list"

# --------------------------------------------------------------------------
# Creates Kermit command file (kermitcmd) by invoking 'cat' for
# defining the following variables in the corresponding 'kermitcmd_file' 
#   For FTP: FTP_AOML, FTP_USER & FTP_PW     
#   Directories: TO_DIR & FROM_DIR
#   Lists: remote_list, failure_filename_list, failure_transfer_list, 
#          failure_size_list, success_list & failure_delete_list
# --------------------------------------------------------------------------
  kermitcmd=$FTPTMP_DIR/kermitcmd.$TODAY  
  . $FTPSCRIPT_DIR/get_delete_files.kermitcmd_file

  now=`date +'%h %d, %Y %T %p'`
  echo "=> Start KERMIT-FTP Session to $FTP_AOML ('*.$ext') Time: $now"
  text=$text"   From: $FROM_DIR\n"
  text=$text"   To: $TO_DIR"
  printf "$text\n"

  kermit -C "take $kermitcmd,exit"   2> $error_kermit > $log_kermit
  status_kermit="$?"

  now=`date +'%h %d, %Y %T %p'`
  echo "=> End KERMIT-FTP Session to $FTP_AOML with Status: $status_kermit Time: $now"

  echo "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
  echo "                      -  log_kermit -"
  cat  $log_kermit | awk '{print "   " $0}'
  echo "   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"

# Print list files (success and failures)
# -------------------------------------------
  echo "=> Success List Files:"
  if [ -s $success_list ]
  then
     cat $success_list | awk '{print "   " $0}'
  else
    echo "   It's EMPTY"
  fi  

  echo "=> Remote Directory List File:"
  if [ -s $remote_list ]
  then
    cat $remote_list | awk '{print "   " $0}'
  else
    echo "   It's EMPTY"
  fi

  failure_msg=$FTPTMP_DIR/failure_msg.$TODAY
  touch $failure_msg

# --------------------------------------------------------------------
#  CASE - brach according to Kermit Exit Status
# --------------------------------------------------------------------
  case $status_kermit in
  0) echo "   All file tranfer were successful"
     subj="$up_root_script_name $type_source - OK, All File Tranfers Were Successful"
     SendMail_Success $status_kermit;;
  1) echo "   NO Connection to $FTP_AOML"
     subj="$up_root_script_name $type_source - WARNING, Connection To AOML Not Available"
     echo "Connection to $FTP_AOML is not AVAILABLE." > $failure_msg
     SendMailAbort_Failure $status_kermit;;
  2) echo "   Can't login (wrong username/passwd)"
     subj="$up_root_script_name $type_source - FAILURE, Can't Login (wrong username/passwd)"
     echo "" > $failure_msg
     SendMailAbort_Failure $status_kermit;;
  3) echo "   Can't change to $FROM_DIR directory"
     subj="$up_root_script_name $type_source - FAILURE, Can't Change To '$up_ext' Directory "
     echo "Can't change to $FROM_DIR directory." >> $failure_msg
     SendMailAbort_Failure $status_kermit;;
  4) echo  "   Remote directory list is empty"
     subj="$up_root_script_name $type_source - WARNING, Remote Directory List Is Empty"
     SendMail_Success $status_kermit;;
  5) echo "   Can't get remote directory listing"
     subj="$up_root_script_name $type_source - FAILURE, Can't Get Remote Directory Listing"
     echo "Can't get remote directory listing" > $failure_msg
     SendMailAbort_Failure $status_kermit;;
  6) echo "   Can't read remote list file"
     subj="$up_root_script_name $type_source - FAILURE, Can't Read Remote List File"
     echo "Can't read remote list file." > $failure_msg
     SendMailAbort_Failure $status_kermit;;
  7) if [ -s  $failure_filename_list ]
     then
       printf "  Can't read filename ($n_failure_filename times)"
       printf "  Can't read filename ($n_failure_filename times)" >> $failure_msg
     fi
     if [ -s  $failure_transfer_list ]
     then
       echo "=> Files with Transfer Error):"
       cat $failure_transfer_list | awk '{print "   " $0}'
       echo "  Transfer Error detected for files:"          >> $failure_msg
       cat $failure_transfer_list | awk '{print "    " $0}' >> $failure_msg
     fi
     if [ -s  $failure_size_list ]
     then
       echo "=> Files with Sizes not Equal (downloaded vs. remote):"
       cat $failure_size_list | awk '{print "   " $0}'
       echo "  Sizes (downloaded vs. remote) not Equal:"    >> $failure_msg
       cat $failure_size_list | awk '{print "    " $0}'     >> $failure_msg
     fi
     if [ -s  $failure_delete_list ]
     then
       echo "=> Remote Files not Deleted:"
       cat $failure_delete_list | awk '{print "   " $0}'
       echo "  Can't delete remote file"                    >> $failure_delete_list
       cat $failure_delete_list | awk '{print "    " $0}'   >> $failure_msg
     fi
     if [ -s $success_list ]                     # Success list empty?
     then                                        #   No, Email 
       status_kermit=8
       echo "   At least one failure was detected"
       subj="$up_root_script_name $type_source - WARNING, At Least One Transfer Failure Reported"
       echo  "At least one transfer failure was detected" > $failure_msg
       SendMail_Success $status_kermit
     else                                        #    Yes, Email & Abort 
       echo "   All file transfes failed"
       subj="$up_root_script_name $type_source - FAILURE, All File Transfers Failed"
       echo "All file transfes failed" > $failure_msg
       SendMailAbort_Failure $status_kermit
     fi;;

  *) echo "   Kermit Exit Code NOT DEFINED"
     subj="$up_root_script_name $type_source - FAILURE, Kermit Exit Code Not Defined"
     text=""
     status_kermit=9
     SendMailAbort_Failure $status_kermit;;
  esac

  echo "=> Exit Function ... DownLoad - $code with Status: $status_kermit"

  return $status_kermit
}
# ------------------------------------------------------------------
# CreateLinks 
# =============
# Creates links in VQCLINKS_DIR to QC Ftped files only for data type
# 'vqc'
#
# Usage: CreateLinks 
#
# ------------------------------------------------------------------
CreateLinks()
{
  echo "=> Enter CreateLinks"

  failure=0
  status=0

  if [ -s $success_list ]              # Any file Ftped?
  then                                 #   Yes, create links
    this=`pwd`
    cd $QCLINKS_VISUAL_DIR
    for qcfile in `cat $success_list` 
    do
      pathname_qcfile=$QCDATA_DIR/$qcfile
      if [ -f $pathname_qcfile ]        # Does QC file exist?
      then                              #   Yes, create link
        if [ -f $qcfile ]               #   Does link exist?
        then                            #     Remove it
           rm -f $qcfile
        fi
        ln -s $pathname_qcfile .
        echo "   Link to: $pathname_qcfile Created in: $QCLINKS_VISUAL_DIR"
      else
        failure=1
        echo "   Can't Create Link - $pathname_qcfile DOES NOT EXITS"
      fi
    done #endfor - loop over success_list files
  fi #endif - success_list 
  cd $this

  if [ $failure -ne 0 ]
  then
    ret_val=10
    status=$ret_val
    subj="$up_root_script_name $type_source - FAILURE, Creating Links For Visually QCed Files"
    text="Error reported when creating link in \n\t'$QCLINKS_VISUAL_DIR'\n"
    text=$text"for at least one Ftped Visually QCed file.  See logfiles:\n"
    text=$text"\t$logfile"
    echo "   [Email Begin]"
    echo "   Subj: $subj"
    echo "   To: $EMAIL_OPERATOR"
    echo "   Text:"
    printf  "$text" > $email_msg
    cat $email_msg | awk '{print "         " $0}'
    echo "   [Email End]"
    printf "   Sending Email ...\c"
    mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
    echo "OK"
    rm -f $email_msg
  fi

  rm -f $success_list
  echo "=> Exit CreateLinks with Status: $status"
  return
}
# =====================  Init Global System Variable =====================
ReadSet_InitVar > /dev/null     # Set global variables in 'argo.init'
cd $savethis                    # back to entry directory
# =====================  Set Working Directories  ========================
FTP_DIR=$ARGO_DIR/ftp
FTPSCRIPT_DIR=$FTP_DIR/scripts
FTPLOG_DIR=$FTP_DIR/log
FTPTMP_DIR=$FTP_DIR/tmp

REALTIME_DIR=$FTPARGO_INCOMING_DIR/real-time
SCIENTIFIC_DIR=$FTPARGO_INCOMING_DIR/scientific

ORBCOMM_DIR=$ARGO_DIR/orbcomm
NCDATA_DIR=$ORBCOMM_DIR/data/nc

IRIDIUM_DIR=$ARGO_DIR/iridium
IRIDIUM_DATA_DIR=$IRIDIUM_DIR/data

DATA_DIR=$ARGO_DIR/data/
# ==========================  Main script  ===================================
TODAY=`date +'%Y.%m.%d_%H.%M'`

type_source=$1

script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

email_msg=$FTPLOG_DIR/$root_script_name.$TODAY"_email"
logfile=$FTPLOG_DIR/$root_script_name"_"$type_source.$TODAY"_log"
error=$FTPLOG_DIR/$root_script_name"_"$type_source.$TODAY"_error"

touch $error

exec 1> $logfile
exec 2> $error

now=`date +'%h %d, %Y %T %p'`
echo "<================ Start  $script_name: $now ==============>"

FTP_AOML=$FTP_LOCALHOST
FTP_USER=$FTPLOCAL_USER
FTP_PW=$FTPLOCAL_PW

echo "=> Data Source: $type_source"

echo "=> Working  Variables:"
echo "   FTP_AOML: $FTP_AOML"
echo "   FTP_USER: $FTP_USER"
echo "   FTP_DIR: $FTP_DIR"

echo "=> Working Directories:"
echo "   FTPSCRIPT_DIR: $FTPSCRIPT_DIR"
echo "   FTPLOG_DIR: $FTPLOG_DIR"
echo "   FTPTMP_DIR: $FTPTMP_DIR"

echo "=> From Directories:"
echo "   ORBCOMM_DIR=$ORBCOMM_DIR"
echo "   REALTIME_DIR=$REALTIME_DIR"
echo "   SCIENTIFIC_DIR: $SCIENTIFIC_DIR"

echo "=> To Directories:"
echo "   NCDATA_DIR=$NCDATA_DIR"
echo "   IRIDIUM_DATA_DIR=$IRIDIUM_DATA_DIR"

case $type_source in
  rnavo_i_msg) extension='msg'
               FROM_DIR=$REALTIME_DIR/navo
               TO_DIR=$IRIDIUM_DATA_DIR/msg;;
  rpmel_i_log) extension='log'
               FROM_DIR=$REALTIME_DIR/pmel
               TO_DIR=$IRIDIUM_DATA_DIR/log;;
  rpmel_i_msg) extension='msg'
               FROM_DIR=$REALTIME_DIR/pmel
               TO_DIR=$IRIDIUM_DATA_DIR/msg;;
   rsio_i_phy) extension='phy'
               FROM_DIR=$REALTIME_DIR/sio
               TO_DIR=$IRIDIUM_DATA_DIR/phy;;
    ruw_i_log) extension='log'
               FROM_DIR=$REALTIME_DIR/uw
               TO_DIR=$IRIDIUM_DATA_DIR/log;;
    ruw_i_msg) extension='msg'
               FROM_DIR=$REALTIME_DIR/uw
               TO_DIR=$IRIDIUM_DATA_DIR/msg;;
    ruw_i_phy) extension='phy'
               FROM_DIR=$REALTIME_DIR/uw
               TO_DIR=$IRIDIUM_DATA_DIR/phy;;
    ruw_i_srf) extension='srf' 
               FROM_DIR=$REALTIME_DIR/uw
               TO_DIR=$IRIDIUM_DATA_DIR/srf;;
    ruw_i_sts) extension='sts' 
               FROM_DIR=$REALTIME_DIR/uw
               TO_DIR=$IRIDIUM_DATA_DIR/sts;;
  rwhoi_i_msg) extension='msg'
               FROM_DIR=$REALTIME_DIR/whoi
               TO_DIR=$IRIDIUM_DATA_DIR/msg;;
  rwhoi_i_phy) extension='phy'
               FROM_DIR=$REALTIME_DIR/whoi
               TO_DIR=$IRIDIUM_DATA_DIR/phy;;
      rwhoi_o) extension='nc'
               FROM_DIR=$REALTIME_DIR/whoi
               TO_DIR=$NCDATA_DIR;;
            *) echo "   Not a valid data type: $type_source" 
               ret_val=11
               subj="$up_root_script_name $type_source - FAILURE, Not a Valid Data Type"
               text="Not a valid data type ($type_source) - execution terminated" 
               echo "   [Email Begin]"
               echo "   Subj: $subj"
               echo "   To: $EMAIL_OPERATOR"
               echo "   Text:"
               printf  "$text" > $email_msg
               cat $email_msg | awk '{print "         " $0}'
               echo "   [Email End]"
               printf "   Sending Email ...\c"
               mailx  -s "$subj" $EMAIL_OPERATOR < $email_msg
               echo "OK"
               now=`date +'%h %d, %Y %T %p'`
               rm -f $error $email_msg
               echo "<============= Terminated $script_name: $now ============>"
               exit $ret_val;;
esac

echo "   FROM_DIR: $FROM_DIR"
echo "   TO_DIR: $TO_DIR"
echo "   FILE_EXTENSION: $extension"

# ---------------------------------------------------------------------
#       Download Files from FTP AOML incoming  using  KERMIT-FTP  
# ---------------------------------------------------------------------
DownLoad $type_source $extension
ret_val=$?

now=`date +'%h %d, %Y %T %p'`
echo "<=============== Finish $script_name: $now ==============>"

exit $ret_val
