c   Revision Date: 31-Mar-06 (YHD - Use first 99.999 line time for GTS; if the
c                                   first line is not there, then use second 
c                                   line time.)
c                  10-Feb-06 (YHD - Do not allow negative depth line to GTS)
c                  01-Feb-06 (YHD - A variable, 'dep' is defined in the main
c                                   and lmax is no longer needed in this s/r.)
c                                   Prevent this 'gts' run being crashed when 
c                                   a profile has any problem.) 
c                  20-Oct-04 (YHD - GTS transmission rules & Grey list
c                                   interpretation rules applied. Refer to
c                                   tables)
c                  04-Jun-04 (YHD - Don't send the data line to GTS when 
c                                   p-good; t-bad; s-bad case.
c                                   Send the data to GTS when
c                                   p-good; t-bad; s-good (2#### 4####) or
c                                   p-good; t-good; s-bad (2#### 3####).) 
c                  26-Apr-04 (YHD - Error trap added with status=5)
c                  15-Jan-04 (YHD - 24:00:00 case -- make it to 00:00:00
c                                   next day)
c                  11-Dec-03 (YHD - Spike & gross errors are removed before
c                                   sending to GTS.)
c                  02-Jun-03 (YHD - TESAC max. length is now 15,000 bytes.)
c                  30-May-03 (YHD - changed argos to char*16; cargos to char*6
c                                   cpnum to char*4)
c                  23-Jan-02 (YHD - add GTS_DISTRIBUTION_SITE)
c                  13-Jun-01 (YHD - 7 digits wmo number)
c                  17-Aug-00 (RS - implement return status)
c
c   Pala2kkyy_ts.f
c   ==============
c   Writes a palace float data into "KKYY", a new TESAC format
c   as of 05/03/2000.
c   yhd, 02-10-2000
c
c   Inputs:
c    inum:   Internal ID
c    wmo:    WMO number
c    argos:  ARGOS number
c    pnum:   Profile number
c    alat:   latitude
c    alon:   longitude
c    ctime:  palace times while drifting at sea surface
c    wmo_inst: WMO Instrument type number
c    wmo_reco: WMO Recorder type
c    nprs:   number of pres/temp/sal
c    tem:    temp
c    sal:    salinity
c    pres:  pressure
c    ofile: output file
c    mn_cnt: counter for nnumber kkyy files
c    gts_site: GTS sender? AOML or ARGOS
c  Outputs:
c    typ: which type of float? t, ts, or tcs
c    status: exit 0,1,2,0r,3
c-----------------------------------------------------------

      subroutine pala2kkyy(inum,wmo,argos,pnum,alat,alon,ctime,
     *           wmo_inst,wmo_reco,nprs,tem,sal,pres,dep,ofile,
     *           mn_cnt,gts_site,typ,status)

      implicit none
      
      integer  iyr,imon,iday,ihour,imin,isec,wmo_inst,wmo_reco,
     *         nprs,inum,wmo,pnum,status,typ,idbug,lastnb,lnblnk

      integer  ldate(6)
      integer  id,im,iy,ih,i,status
      integer  itime8(8)
      integer  mdays(12)
      data     mdays/31,28,31,30,31,30,31,31,30,31,30,31/

      real     alat,alon,jdays,pdays
      real     tem(*),pres(*),sal(*),dep(*)

      character ans*1,cday*2,cmon*2,cyr*4,cinum*4,cargos*6,cpnum*4
      character mn_cnt*2,gts_site*5
      character ofile*128,ctime*14
      character date8*8,time10*10,zone5*5
      character argos*16
      character PRGMAIN*12
      data      PRGMAIN /'pala2kkyy_ts'/

      equivalence (ldate(1),iyr),(ldate(2),imon),(ldate(3),iday),
     *            (ldate(4),ihour),(ldate(5),imin),(ldate(6),isec)

      data idbug /1/           ! make it to 1 to get a list
  
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from PALA2KKYY_TS >>>>>'
c
c --- Palace Time.
c
      read(ctime(1:4),4,err=9999) iyr
      read(ctime(5:6),2,err=9999) imon
      read(ctime(7:8),2,err=9999) iday
      read(ctime(9:10),2,err=9999)ihour
      read(ctime(11:12),2,err=9999)imin
      read(ctime(13:14),2,err=9999)isec

      ! 
      ! Check for 24:00:00 case. (ex:0329_21828_038.phy)
      !
      if (ihour.eq.24.and.imin.eq.00.and.isec.eq.00) then
        ihour=0
        imin=0
        isec=0
        iday=iday+1
        if (mod(4,iyr).eq.0) then
          mdays(2)=29
        else
          mdays(2)=28
        end if
        if (iday.gt.mdays(imon)) then
          iday=iday-mdays(imon)
          imon=imon+1
          if (imon.gt.12) then
            imon=1
            iyr=iyr+1
          end if
        end if
      end if

      if (inum.lt.10) then
        cinum(1:3)='000'
        write(cinum(4:4),3,err=9999) inum
      else if (inum.ge.10.and.inum.lt.100) then
        cinum(1:2)='00'
        write(cinum(3:4),2,err=9999) inum
      else if (inum.ge.100.and.inum.lt.1000) then
        cinum(1:1)='0'
        write(cinum(2:4),5,err=9999) inum
      else
        write(cinum(1:4),1,err=9999) inum
      end if
c      write(cargos,4) argos  
      if (pnum.lt.10) then
        cpnum(1:2)='00'
        write(cpnum(3:3),3,err=9999) pnum
      else if (pnum.ge.10.and.pnum.lt.100) then
        cpnum(1:1)='0'
        write(cpnum(2:3),2,err=9999) pnum
      else if (pnum.gt.1000) then
        write(cpnum(1:4),1,err=9999) pnum
      end if
 1    format(i4)
 2    format(i2)
 3    format(i1)
 4    format(i5)
 5    format(i3)

      if (iday.lt.10) then
        cday(1:1)='0'
        write(cday(2:2),3,err=9999) iday
      else
        write(cday(1:2),2,err=9999) iday
      end if
      if (imon.lt.10) then
        cmon(1:1)='0'
        write(cmon(2:2),3,err=9999) imon
      else
        write(cmon,2,err=9999) imon
      end if
      write(cyr,1,err=9999)  iyr

      call date_and_time(date8,time10,zone5,itime8)
      ih = itime8(5)
      id = itime8(3)
      im = itime8(2)
      iy = itime8(1)

      call julday(iy,im,id,ih,jdays)        ! returns days for today
      call julday(iyr,imon,iday,ihour,pdays)! returns days for palace

      if (abs(jdays-pdays).gt.30) then ! data should be 30 days or less.
        write(51,41,err=9999) inum,argos,pnum
 41     format(1x,i4.4,'_',a,'_',i3.3,' is more than 30 days old and not
     * going to put on GTS.')
        ans = 'n'
        if (ans.eq.' '.or.ans.eq.'n'.or.ans.eq.'N') then
          write(51,45)                ! No KKYY file created.
          status = 1                   
        else
          write(51,46)                ! Yes KKYY file created.
          write(51,49,err=9999) jdays,pdays
          status = 0                   
        end if
      else                            ! data less than 30 days.
        ans='y'
        if (ans.eq.' '.or.ans.eq.'n'.or.ans.eq.'N') then
          write(51,45)                ! No KKYY file created.
          status = 1                   
        else
          write(51,46)               ! Yes KKYY file created.
          write(51,49,err=9999) jdays,pdays
          status = 0                   
        end if
      end if

 40   format(1x,'Create a "KKYY" file anyway? (y/n) (def:n) ',$)
 43   format(1x,'   "KKYY" format:',i4.4,'_',a,'_',i4.4,1x,a)
 45   format(1x,'   NO  KKYY file created!')
 46   format(1x,'   YES KKYY file created!')
 48   format(a1)
 49   format(1x,'   today(julian days from 1965):',f10.1,
     *' palace days:',f10.1)
c
c --- Write to "KKYY" format.
c
      if (ans.eq.'Y'.or.ans.eq.'y') then
        lastnb = lnblnk(argos)

        write(51,43,err=9999) inum,argos(1:lastnb),pnum,ofile(1:48)
c
c --- Palace float measures pressures and depths in needed in 'TESAC'. 
c --- Convert pressure to depth. (02-22-2001)
c
        do i=1,nprs
          call pres2dep(pres(i),alat,dep(i))
        end do

        call fill_kkyy(nprs,ldate,itime8,alat,alon,inum,argos,pnum,
     *       wmo,wmo_inst,wmo_reco,tem,sal,dep,mn_cnt,gts_site,typ,
     *       idbug)
      end if
      return    
c
c --- Reading/writing error trap
c
 9999 continue
      status=9
      write(66,'(A, ": Reading/Writing Error - STATUS=", I2)') 
     *        PRGMAIN, status
      write(6,'(A, ": Reading/Writing Error - STATUS=", I2)') 
     *        PRGMAIN, status
      call exit(status)  
      
      end
c
c     subroutine fill_kkyy
c
c     inputs : 
c      . np          : number of points
c      . ldate       : yr,mon,day,hour,min,sec for palace observation
c      . itime8      : yr,mon,day,hour,min,sec for today
c      . rlat        : latitude
c      . rlon        : longitude
c      . f_no        : float number
c      . p_no        : profile number
c      . w_no        : wmo number
c      . w_inst      : wmo instrument type (table 1770)
c      . w_reco      : wmo recorder type (table 4770)
c      . depth       : depths
c      . temp        : temperatures
c      . sal         : salinities
c      . mn_cnt      : counter for kkyy files
c      . typ         : 1 for temp float; 2 for temp/salinity float
c
      subroutine fill_kkyy(np,ldate,itime8,rlat,rlon,f_no,ar_no,p_no,
     *                     w_no,w_inst,w_reco,temp,sal,depth,m_cnt,
     *                     gts_site,typ,idbug)

      implicit none
      
      character(len=9), parameter :: PRGMAIN = 'fill_kkyy'
      integer, parameter :: max_byte = 15000

      integer   i,np,ldate(6),iyr,ip,itemp,isal,
     *          nbytes,nrec,nrepeat,w_inst,w_reco,
     *          id,f_no,p_no,w_no,ihour,imin,
     *          jday,jmon,jyear,status,typ,idbug,ist,ic,status
      integer   itime8(8),fv
      real      depth(1),temp(1),sal(1),rlat,rlon
      character qc*1,form*4,sheader*18,string*24,
     *          cw_inst*3,cw_reco*2,rec*15000,cw_no*5,m_cnt*2,cw7_no*7
      character gts_site*5,ar_no*16

      data      form/'KKYY'/
      data      sheader/'SOVX10 KXXX 000000'/    ! sender AOML or ARGOS
      data      nrec,ip,nrepeat /1,0,1/
      data      fv /-99999/

      if (idbug.gt.0) then
        write(66,*)'np=',np
        write(66,10,err=9999) f_no,ar_no
 10     format(1x,'Fill_kkyy: float id=',i4.4,'_',a)
      end if      

c
c --- Want to finish reading a palace data?
c
      if (f_no.eq.0) then ! Want to quit;write out "bull" contents
        return
      end if                        

      !
      ! Start by filling 'rec' array.
      !
      do i = 1, max_byte
        rec(i:i) = ' '
      end do

      if (gts_site=='ARGOS') then
        sheader(9:11)='ARS'                   ! from SERVICE ARGOS
      else if (gts_site=='AOML') then
        sheader(9:11)='WBC'                   ! from SEAS Server
      else
        write(50,*)'<<< ERROR on GTS_DISTRIBUTION_SITE >>>'
        write(50,*)'GTS_SITE:',gts_site,' float=',f_no
        status = 2                                      
        call exit (status)
        stop '2'
      end if
      do i = 1, 18
        rec(i:i)=sheader(i:i)         ! subheader
      end do

      ihour = itime8(5)
      jday  = itime8(3)
      jmon  = itime8(2)
      jyear = itime8(1)
      
      if (jday.eq.0) then
        jday=1
        rec(13:14)='01'
      else
        write(rec(13:14),11,err=9999) jday
      end  if

 11   format(i2.2)
      read(m_cnt,12,err=9999) imin
 12   format(i2)

      if (imin.ge.60) then
        imin = 0
        ihour = ihour + 1
        if (ihour.ge.24) then
          ihour = 0
          jday = jday - 1
          if (id.eq.0) id=1
          rec(15:16) = '00'     ! hour
        end if
        rec(17:18)='00'         ! min
        write(rec(15:16),11,err=9999) ihour
      else
        write(rec(15:16),11,err=9999) ihour
        write(rec(17:18),11,err=9999) imin
      end if

      do i = 13, 18
        sheader(i:i) = rec(i:i)                      ! save a 'sheader'
      end do
      ip=18                                          ! KKYY starts
c                                                    ! if(repeat)then
      imin = imin + 1
c
c --- KKYY,day,month,year,hour,minutes, and /.
c
      rec(ip+1:ip+4) = form                                    ! KKYY
      write(rec(ip+6:ip+7),102,err=9999) ldate(3)              ! day(dd)
      write(rec(ip+8:ip+9),102,err=9999) ldate(2)              ! month(mm)
      iyr = mod(ldate(1),10)                                   ! 01/29/2001
      write(rec(ip+10:ip+10),101,err=9999) iyr                 ! year(y)
      write(rec(ip+12:ip+13),102,err=9999) ldate(4)            ! hour(hh)
      write(rec(ip+14:ip+15),102,err=9999) ldate(5)            ! minutes(mm)
      rec(ip+16:ip+16) = '/'
c
c --- Which Quadrant of the globe? (1:NE, 3:SE, 5:SW, 7:NW)
c
      if (rlat.ge.0.0.and.rlon.ge.0.0) qc='1'
      if (rlat.lt.0.0.and.rlon.ge.0.0) qc='3'
      if (rlat.lt.0.0.and.rlon.lt.0.0) qc='5'
      if (rlat.ge.0.0.and.rlon.lt.0.0) qc='7'
      rec(ip+18:ip+18)=qc
c
c --- latitude. (LaLaLaLaLa)
c     latitude degree 
      write(rec(ip+19:ip+23),105,err=9999) int(abs(rlat)*1000)   

c
c --- longitude.
c     longitude degree
      write(rec(ip+25:ip+30),106,err=9999) int(abs(rlon)*1000)   

c 
c --- Section identifier.
c
      rec(ip+32:ip+34)='888'
      rec(ip+35:ip+35)='7'                             ! k1 = 7; selected dep
      if (typ.eq.1) then
        rec(ip+36:ip+36)='0'                           ! k2=0; no salinity
      else                                             ! k2=1 or k2=2 or k2=3
        rec(ip+36:ip+36)='1'                           
      end if
c
c --- Instrument type and recorder type.
c     Table 1770 and 4770.
c
      write(cw_inst(1:3),107,err=9999) w_inst
      write(cw_reco(1:2),108,err=9999) w_reco
      rec(ip+38:ip+40)=cw_inst
      rec(ip+41:ip+42)=cw_reco
c
c --- Depths/Temp/sal.
c     Remove any bad points/fill values before sending it to GTS.
c     For example, t is good and s is bad, then 2#### 3#### no 4.
c     For example, t is bad and s is good, then 2#### 4####.
c     Refer to Bob Keeley's e-mail on this issue. 11-Dec-2003.
c     06-Jun-04 correction:  when t and s are bad skip this data to GTS.
c
      ic=0                    ! counter for all three variables(p,t,s)
      do i = 1, np
        if (nint(depth(i))<>fv) then
          if (depth(i)>=0.0) then                   ! + depth allowed
            rec(ip+44:ip+44)='2'                    ! depth indicator
            write(rec(ip+45:ip+48),104,err=9999) nint(depth(i))
            ic=ic+1
            ist=50
          else                                         ! - depth skip this 
            go to 33                                   ! data point
          end if
        else
          ist=44
        end if                                       ! if (nint(depth(i)).ne
        itemp=nint(temp(i)*100.0)
        rec(ip+ist:ip+ist)='3'
        if (itemp.ne.fv) then
          ic=ic+1
          if (temp(i).lt.0.0) itemp=abs(itemp)+5000
          write(rec(ip+ist+1:ip+ist+4),104,err=9999) itemp
          if (temp(i).gt.50.0) then
            write(50,*)'<<< ERROR on temp value >>>'
            write(50,*)'Temp=',temp(i),' float=',f_no,' i=',i
            status = 2                                 ! 3 -> Error t value
            call exit (status)
            stop '2'
          end if
        else
          ic=ic+1
          rec(ip+ist+1:ip+ist+4)='////'              ! bad temp.
        end if                               ! if (itemp.ne.9999) then        
        ist=56          

        if (typ.ge.2) then
          isal=nint(sal(i)*100.0)
          rec(ip+ist:ip+ist)='4'                     ! salinity indicator
          if (isal.ne.fv) then
            if (isal.gt.0.0) then
              write(rec(ip+ist+1:ip+ist+4),104,err=9999) isal
              ic=ic+1
              ip=ip+6*ic
            else
              rec(ip+ist+1:ip+ist+4)='////'  ! bad salinity
              ic=ic+1
              ip=ip+6*ic                     ! don't write sal to GTS
            end if
          else 
            rec(ip+ist+1:ip+ist+4)='////'    ! bad salinity
            ic=ic+1
            ip=ip+6*ic                       ! don't write sal to GTS
          end if
        else
          ip=ip+6*ic                         ! if (isal.ne.9999) then
        end if
 33     ic=0                    ! reset the counter
      end do                                          ! do i = 1, np
c
c --- Identifier of who sent the message.
c
c      encode(5,105,cw_no) w_no

      if (w_no.le.99999) then
        rec(ip+44:ip+48)='99999'                       ! from buoy id
        write(cw_no,105,err=9999) w_no
        rec(ip+50:ip+54)=cw_no                         ! 5 digits WMO number
        rec(ip+55:ip+55)='='                           ! end of record
        nbytes=(ip+55)
      else
        rec(ip+44:ip+44)='Q'                
        write(cw7_no,109,err=9999) w_no
        rec(ip+45:ip+51)=cw7_no                        ! 7 digits WMO number
        rec(ip+52:ip+52)='='                           ! end of record
        nbytes=(ip+52)
      end if

      ip=nbytes                                       ! position in rec

      call write_kkyy(ip,rec,idbug)
      nrec = nrec + 1
      
 31   format(1x,76('-'))
 101  format(i1)
 102  format(i2.2)
 103  format(i3.3)
 104  format(i4.4)
 105  format(i5.5)
 106  format(i6.6)
 107  format(i3)
 108  format(i2)
 109  format(i7.7)

      return
c
c --- Reading/writing error trap
c
 9999 continue
      status=9
      write(66,'(A, ": Reading/Writing Error - STATUS=", I2)') 
     *        PRGMAIN, status
      write(6,'(A, ": Reading/Writing Error - STATUS=", I2)') 
     *        PRGMAIN, status
      call exit (status)

      end

      subroutine write_kkyy (nbyt, rec, idbug)
      ! revision date 2-Oct-09 (YHD - only one data point case)
      !
      ! inputs:  
      !   nbyt : number of bytes to write
      !   rec  : array to write
      !   idbug: indicator for debugging

        implicit none
      
        integer, parameter :: max_byte = 15000
        character(len=1), parameter :: LINE_FEED = char(10) 
        character(len=1), parameter :: CARRIAGE_RETURN = char(13)
      
        integer   i,jp,ip,nbyt,icnt,idbug

        character header*20,bul*15000,rec*15000
        data      header(1:17) /'####018000000####'/

        if (idbug.gt.0) then
          write(66,*)'<<< DEBUG from PALA2KKYY_TS: write_kkyy >>>'
        end if

        write(51,10) nbyt,rec(nbyt:nbyt)
 10       format(1x,'Write_kkyy: number of bytes read   =',i5,
     *      ' last ch=',a1)

        do i = 1, max_byte
          bul(i:i)=' '
        end do                         

        do i = 1, 18
          bul(i:i)=rec(i:i)
        end do

        if (rec(19:19)=='R') then
          bul(19:19)=' '
          bul(20:20)=rec(19:19)
          bul(21:21)=rec(20:20)
          bul(22:22)=rec(21:21)
          jp = 23
          ip = 21
        else
          jp = 19
          ip = 18
        end if

        bul(jp:jp)=CARRIAGE_RETURN
        bul(jp+1:jp+1)=CARRIAGE_RETURN
        bul(jp+2:jp+2)=LINE_FEED
        jp = jp + 2

c   ip:  input 'rec' pointer
c   jp:  output 'bul' pointer

        icnt = 0     ! a pointer for 69 bytes to write

        do while(ip<=nbyt)
          icnt=icnt+1
          if (mod(icnt,68)==0) then
            if (idbug.gt.0) then
              write(66,*)' ip=',ip,' jp=',jp
              write(66,*)'Last 6 rec=>',rec(ip-5:ip)
              write(66,*)'Last 6 bul=>',bul(jp-5:jp)
            end if

            if (rec(ip:ip).eq.' ') then
              jp=jp-5
              ip=ip-6
            else if (rec(ip-1:ip-1).eq.' ') then
              bul(jp-1:jp-1)=' '
              ip=ip-1
            else if (rec(ip-2:ip-2).eq.' ') then
              bul(jp-2:jp-2)=' '
              bul(jp-1:jp-1)=' '
              ip=ip-2
            else if (rec(ip-3:ip-3).eq.' ') then
              bul(jp-3:jp-3)=' '
              bul(jp-2:jp-2)=' '
              bul(jp-1:jp-1)=' '
              ip=ip-3
            else if (rec(ip-4:ip-4).eq.' ') then
              bul(jp-4:jp-4)=' '
              bul(jp-3:jp-3)=' '
              bul(jp-2:jp-2)=' '
              bul(jp-1:jp-1)=' '
              ip=ip-4
            else if (rec(ip-5:ip-5).eq.' ') then
              bul(jp-5:jp-5)=' '
              bul(jp-4:jp-4)=' '
              bul(jp-3:jp-3)=' '
              bul(jp-2:jp-2)=' '
              bul(jp-1:jp-1)=' '
              ip=ip-5
            else if (rec(ip-6:ip-6).eq.' ') then
              if (rec(ip:ip).eq.'=') then
                ip=ip-1
                if (idbug==1) then
                  write(6,*)'last point case...',ip
                end if
              else if (rec(ip:ip).ne.'='.and.rec(ip+1:ip+1).eq.' ') then
                ip=ip+1
              else
                ip=ip-1
              end if
            end if 
            if (rec(ip-1:ip-1).eq.'=') ip=ip-1
            if (rec(ip-6:ip-6).eq.'Q') then     ! 7 digits wmo number case
              ip=ip-7
              bul(jp-6:jp-6)=' '
              jp=jp-6
            end if

            bul(jp:jp)=CARRIAGE_RETURN        ! 67th byte
            bul(jp+1:jp+1)=CARRIAGE_RETURN    ! 68th byte
            bul(jp+2:jp+2)=LINE_FEED          ! 69th byte

            jp=jp+2                             ! increment jp pointer by 2
            icnt=0                              ! reset
          else
            jp=jp+1
            ip=ip+1
            bul(jp:jp)=rec(ip:ip)               ! load rec to bul
            if (bul(jp:jp).eq.'=') then
              bul(jp+1:jp+1)=CARRIAGE_RETURN
              bul(jp+2:jp+2)=CARRIAGE_RETURN
              bul(jp+3:jp+3)=LINE_FEED
              jp=jp+3
              icnt=0
            end if
          end if
        end do

        jp = jp - 1
        if (idbug.gt.0) write(66,11) jp
 11       format(1x,'Write_kkyy: number of bytes written=',i5)

        write(header(8:13),110) jp
        header(18:18)=CARRIAGE_RETURN
        header(19:19)=CARRIAGE_RETURN
        header(20:20)=LINE_FEED

        if (idbug.gt.0) write(66,106) header(1:20)

        write(52,'(a)') header(1:19)        ! header (####) unix puts lf.
        write(52,'(a)') bul(1:jp-1)         ! need to open a file
c
c --- Write out to check the content of record.
c
 31     format(1x,76('-'))
 106    format(a21)
 107    format((/,a69))
 110    format(i6.6)

        return

      end
