c------------------------------------------------------------------- 
c     Revised date: 27-Jan-05 (YHD - rd_float_meta.f is all same for 'qc',
c                                    'netcdf' and 'gts' runs.)
c                   07-Jan-05 (YHD - add grey list)
c                   18-Nov-04 (YHD - 'grey list' lines are added meta files) 
c                   08-Jul-03 (YHD - 'proj_name' is passed to main program.)
c                   04-May-04 (YHD - 'max' was never used & got rid off)
c                   10-Apr-03 (YHD - 'inst_ref' is brand,type,serial number.)
c                   26-Mar-03 (YHD - "dlay_down_hours" is defined as real.)
c                   30-Jan-03 (YHD - trans_sys & pos_sys are included)
c                   24-Dec-02 (CS - changed the cycle times to
c                                   double precision)
c                   02-Aug-02 (YHD - cyc_days(41:47) corrected)
c                   17-Jul-02 (YHD - fsnum added)
c   
c     rd_float_meta.f
c     ===============
c     Reads a palace float "META" file.
c     YHD, 20-Apr-2002
c
c-------------------------------------------------------------------

      subroutine rd_float_meta(iunit,int_num,wmo_num,trans_id,
     *  proj_name,
     *  inst_ref,st_time,laun_time,dlay_down_hours,down_days,up_hours,
     *  cyc_days,des_hours,neu_days,asc_hours,surf_hours,laun_pos_la,
     *  laun_pos_lo,fsnum,trans_sys,pos_sys,deep,send_gts,grey_list,
     *  grey_date_st,grey_date_en,grey_flag,grey_parm,grey_reason)
c     
c     iunit:     unit for reading
c     int_num:   internal id number
c     wmo_num:   wmo id
c     trans_id:  transmission id number
c     inst_ref: APEX, SOLO, or PROVOR(brand)+type+serial number
c     st_time:   start time for APEX
c     proj_name: U.S. ARGO project or Steller's Sea Lion Research
c     dlay_down_hours: delay of first down time for APEX
c     down_days:  days in the water for APEX
c     up_hours:    time to ascent to surface for APEX
c     laun_time: launch time for SOLO type
c     cyc_days: cycle time for SOLO type
c     des_hours: descent time for SOLO type
c     neu_days: neutral depth time (=parking) for SOLO type
c     asc_hours: ascent time for SOLO type
c     surf_hours: surface time for SOLO type
c     laun_pos_la: launch position (latitude)
c     laun_pos_lo: launch postion (longitude)
c     fsnum: float serial number (character)
c     trans_sys: transmission type (character*16)
c     pos_sys: positioning system (character*8)
c     deep : deepest pressure (real)
c     send_gts: send this data to GTS? (character*1)
c     grey_list: Is this float in grey list or not? (character*1)
c     grey_reason: Reason for in grey list (character*128)

      implicit none

      integer i,iunit,icnt,iyear,imon,iday,ihr,imin,isec,
     *        int_num,wmo_num,idbug,nb,lchar_occur

      real    dlay_down_hours
      real    tmp,lat,latm,lon,lonm,deep
      double precision down_days,up_hours,cyc_days,des_hours,
     *        neu_days,asc_hours,
     *        surf_hours,laun_pos_la,laun_pos_lo
      character prg_sub*13,cline*80,st_time*64,inst_type*64,laun_time*64
      character fsnum*4,trans_sys*16,pos_sys*8,trans_id*16,inst_ref*64,
     *con_type*3,tem_type*3,con_ser_number*4,tem_ser_number*4,
     *proj_name*64
      character send_gts*1,grey_list*3,grey_reason(*)*80,
     *          grey_parm(*)*10,grey_date_st(*)*11,grey_date_en(*)*11,
     *          grey_flag(*)*3           
      integer   n_grey

      data     idbug/0/              ! make it 1 to get a debugging list
      data     prg_sub/'RD_FLOAT_META'/
c
c---Read header information.
c
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from RD_FLOAT_META >>>>>'
 1    continue
      read(iunit,25,end=100,err=999) cline
      if (idbug.gt.0) write(6,*) cline
      if (cline(1:11).eq.'internal ID') then
        read(cline(41:51),*,err=999) int_num    
      else if (cline(1:15).eq.'transmission ID') then
        read(cline(41:51),25,err=999) trans_id
      else if (cline(1:17).eq.'transmission type') then
        read(cline(41:51),25,err=999) trans_sys
         if (trans_sys(1:5) .eq. 'ARGOS') then
            pos_sys='ARGOS   '
         elseif (trans_sys(1:7) .eq. 'ORBCOMM') then
            pos_sys='GPS     '
         endif
      else if (cline(1:15).eq.'instrument type') then
        read(cline(41:56),25,err=999) inst_type
      else if (cline(1:12).eq.'Project name') then
        read(cline(41:80),25,err=999) proj_name
      else if (cline(1:10).eq.'start time') then
        read(cline(41:80),25,err=999) st_time
      else if (cline(1:24).eq.'delay of first down time') then
        read(cline(41:45),*,err=999) dlay_down_hours
      else if (cline(1:9).eq.'down time') then
        read(cline(41:45),30,err=999) tmp
        down_days=tmp
      else if (cline(1:7).eq.'up time') then
        read(cline(41:45),30,err=999) tmp
        up_hours=tmp
      else if (cline(1:11).eq.'launch time') then
        read(cline(41:80),25,err=999) laun_time
      else if (cline(1:10).eq.'cycle time') then
        read(cline(41:47),*,err=999) tmp
        cyc_days=tmp
      else if (cline(1:12).eq.'descent time') then
        read(cline(41:46),*,err=999) tmp
        des_hours=tmp
      else if (cline(1:11).eq.'ascent time') then
        read(cline(41:46),*,err=999) tmp
        asc_hours=tmp
      else if (cline(1:18).eq.'neutral depth time') then
        read(cline(41:46),*,err=999) tmp
        neu_days=tmp
      else if (cline(1:12).eq.'surface time') then
        read(cline(41:46),*,err=999) tmp
        surf_hours=tmp
      else if (cline(1:25).eq.'launch position [lat latm') then
        read(cline(41:80),*,err=999) lat,latm,lon,lonm
        laun_pos_la=lat+latm/60.0
        laun_pos_lo=lon+(lonm/60.0)
      else if (cline(1:19).eq.'float serial number') then
        read(cline(41:44),25,err=999) fsnum
      else if (cline(1:32).eq.'conductivity sensor manufacturer') then
        read(cline(41:43),25,err=999) con_type
      else if (cline(1:31).eq.'temperature sensor manufacturer') then
        read(cline(41:43),25,err=999) tem_type
      else if (cline(1:33).eq.'conductivity sensor serial number') then
        read(cline(41:44),25,err=999) con_ser_number
      else if (cline(1:32).eq.'temperature sensor serial number') then
        read(cline(41:44),25,err=999) tem_ser_number
      else if (cline(1:31).eq.'nominal profile pressure [dbar]') then
        read(cline(41:44),*,err=999) deep
      else if (cline(1:11).eq.'send to GTS') then
        read(cline(41:41),25,err=999) send_gts
c        write(6,*)'send_gts=',send_gts
      else if (cline(1:16).eq.'add to grey list') then
        read(cline(41:43),25,err=999) grey_list
        read(grey_list(3:3),'(i1)') n_grey          ! # of grey list
        do i=1,n_grey
           read(iunit,25,end=100,err=999) cline     ! grey list date
           grey_date_st(i)=cline(41:50)
           grey_date_en(i)=cline(52:61)
           if (idbug.gt.0) write(66,*)'grey_date_st(i)=',grey_date_st(i)
           if (idbug.gt.0) write(66,*)'grey_date_en(i)=',grey_date_en(i)
           read(iunit,25,end=100,err=999) cline     ! grey list flag
           read(cline(41:43),25,err=999) grey_flag(i)
           if (idbug.gt.0) write(66,*)'grey_flag(i)=',grey_flag(i)
           read(iunit,25,end=100,err=999) cline     ! grey list parameter
           read(cline(41:55),25,err=999) grey_parm(i)
           if (idbug.gt.0) write(66,*)'grey_parm=',grey_parm(i)
           read(iunit,25,end=100,err=999) cline     ! why add to grey list 
           read(cline(41:80),25,err=999) grey_reason(i)
           if (idbug.gt.0) write(66,*)'grey_reason=',grey_reason(i)
        end do
      end if
      if (con_type.eq.'   ') con_type=tem_type
      if (con_ser_number.eq.'    ') con_ser_number=tem_ser_number
        
      nb=lchar_occur(inst_type,'_')
      if (con_ser_number.eq.'n/a') then
        inst_ref=inst_type(1:nb)//con_type
      else
        inst_ref=inst_type(1:nb)//con_type//'_'//con_ser_number
      end if
      go to 1
 25   format(a)
 30   format(f3.0)
 100  continue
      return
 999  write(6,*)'Reading rd_float_meta.f error=999 ',cline
      write(50,*)'Reading rd_float_meta.f error=999 ',cline 
      return
      end 



