c------------------------------------------------------------------- 
c     Revised date: 19-Dec-12 VMH  - Increased size of cline to 256
c                   28-Feb-07 (YHD - Added 'PSAL_ADJ' case)
c                   04-Jan-07 (YHD - Fool prof of reading the position lines.)
c                   23-Jan-06 (YHD -A Recorder type '88' is changed to '64'
c                                   (up) or '65'(down) transit for Iridium.
c                   12-Jan-06 (YHD - For iridium float, wmo recorder type
c                                    is now 88.)
c                   17-Mar-05 (YHD - "SALINITY GRADIENT CHECK" changed to 
c                                    "VERTICAL GRADIENT CHECK_S".
c                   27-Jan-05 (YHD - Added deep,jump,regional,frzn & salg)
c                   27-Jan-05 (YHD - iclass is now character)
c                   14-Jan-05 (YHD - 'ityp' is different from 'netcdf' version)
c                   30-Nov-04 (YHD - 'rd_phy_qc.f' created from rw_palace_qc.f
c                                     I need to come up with something more
c                                     flexible s/r for reading.)
c                   09-Jul-04 (YHD - 'DRIFT OXYGEN' line unit changed from
c                                     (ML/L) to (MICROMOL/KG))
c                   19-Apr-04 (YHD - change factor=1; oxygen unit changed from
c                       end if
c                         ml/l to micromol/kg)
c                   29-Dec-03 (YHD - TECH_PARAM_NAME: upper case & no blank)
c                   16-Dec-03 (YHD - 'max' changed to nmax and incrse to 2500)
c                   21-Jul-03 (YHD - Defined what lines from 'qc' are not
c                                    to store as technical items.)
c                   09-Jul-03 (YHD - Pass 'juld_qc' to main program and it
c                                    can have 'argo' or 'whoi' juld_qc.)
c                   19-May-03 (YHD - More than 2 drift pressure cases)
c                   09-May-03 (YHD - "POSITION_QC" is passed to main program
c                                    in 'whoi_pos_qc' for WHOI float.)
c                   08-May-03 (YHD - Reverse data when the recoder type=60 or
c                                    62 and prof_num .lt. 1000)
c                   04-Apr-03 (YHD - Do not skip -999.0 line for WHOI float)
c                   28-Feb-03 (YHD - "DATE_CREATION" & "JULD_QC" provided
c                                    for 'WHOI" floats)
c                   07-Feb-03 (YHD - s/r/ data_reverse.f not needed anymore;
c                                    check recorder_type(table 4770) to decide
c                   27-Jan-03 (YHD - At least 2 lines provided for the
c                                    positions)
c                   27-Jan-03 (YHD - every read statements handle 'err' case
c                   21-Jan-03 (YHD - read pres flag)
c                   24-Dec-02 (CS -  changed the way alat, alon, jul is
c                                    handled (missing 999.999 cases)
c                   18-Dec-02 (YHD - read line not depend on the 'num_col')
c                   12-Nov-02 (YHD - oxygen factor(=44.6596) added to drift
c                                    oxygen)
c                   08-Nov-02 (YHD - drift_o1,drif_o2 added)
c                   07-Nov-02 (YHD - oxygen parameter added)
c                   27-Aug-02 (YHD - writing qc from MATLAB is different
c                                    from writing qc from 'qc_t(s)'
c                   23-Jul-02 (YHD - st_trans_time added)
c                   16-Jul-02 (YHD - PALACE_TS1 case added)
c                   16-Jul-02 (YHD - num_col 17 case added)
c                   14-Jun-02 (YHD - nparam,tparam,t_value added)
c   
c     rd_phy_qc.f
c     ===============
c     Reads a palace float "PHY" or "QC" file.
c     YHD, 10-29-2001
c
       subroutine rd_phy_qc(iunit,int_num,wmo_num,trans_id,
     *  prof_num,inst_type,ireverse,whoi_c_date,juld_qc,
     *  wmo_inst,wmo_reco,argos_prg_num,pi,ityp,land,
     *  sur_prs,st_trans_time,whoi_pos_qc,
     *  sigma_clim,sigma_anal,adate,
     *  n_drift,drift_t,drift_p,drift_s,drift_o,drift_c,
     *  nparam,t_param,t_value,
     *  num_col,column,num_cal,calib,
     *  iover_t,it_anlz,it_clim,it_gros,it_vert,it_spik,it_vis,it_cntv,
     *  it_jump,it_frzn,it_regi,ideep_flag,
     *  iover_s,is_anlz,is_clim,is_gros,is_dens,is_spik,is_vis,is_cntv,
     *  is_salg,is_jump,is_frzn,is_regi,
     *  ipres,ip_vis,ispd,cksm,
     *  icnt,alat_hist,alon_hist,itime_hist,iclass_hist,itime_first,
     *  nlev,p,t,s,c,o,
     *  psal_adj_exist,sal_adj,is_grosa,is_regia)

c     
c     iunit:     unit for reading
c     int_num:   internal id number
c     wmo_num:   wmo id
c     whoi_c_date: date that nc file was created by WHOI  --- WHOI float only
c     trans_id:  transmission id number
c     prof_num:  profile number
c     wmo_inst:  instrument type (table 1770)
c     wmo_reco:  recorder type (table 4770)
c     st_trans_time: start of transmission
c     nparam  : number of parameters for technical meta file
c     t_param : lists technical parameters
c     t_value : values of technical parameters
c     icnt    :  number of positions reported
c     alat_hist: latitudes of positions
c     alon_hist: longitudes of positions
c     itime_hist:year/mo/day/hour/min/sec
c     nlev:      number of data points
c     t:         temperature
c     s:         salinity
c     p:         pressure
c     cksm:      check-sum flag
c     ispd:      speed check flag 0:can't apply; 1:pass; 4:fail
c     ityp:      1=t1 or t2; 2=ts1,ts2,..or ts7; 3=tcs

      implicit none

      integer nmax,nx
      parameter (nmax=2500,nx=50)
      integer i,iunit,icnt,icnt_start,iyear,imon,iday,ihr,imin,isec,
     *        int_num,wmo_num,prof_num,n_drift,
     *        num_col,cksm(*),
     *        nlev,idbug,wmo_inst,wmo_reco,
     *        icksm(nmax),status,ityp,ispd,num_cal,
     *        icksm_col,ipres_col,icond_col,isal_col,itemp_col,
     *        ioxy_col,ibatt_col,isdens_col,itspik_col,isspik_col,
     *        itgros_col,isgros_col,itclim_col,isclim_col,ipvis_col,
     *        itanlz_col,isanlz_col,itvis_col,isvis_col,itvert_col,
     *        itregi_col,isregi_col,issalg_col,
     *        ideep_col,itjump_col,isjump_col,
     *        ios,lastnb,lnblnk,itcolum,argos_prg_num,
     *        nparam,const_t,const_s,
     *        icol_read,ipres_flag_col,ireverse,lnb,ncnt,land

      integer iover_t,it_anlz(*),it_clim(*),it_gros(*),it_vert(*),
     *        it_vis(*),it_spik(*),it_cntv,ip_vis(*),it_jump(*),
     *        it_frzn,it_regi(*),ideep_flag(*),
     *        iover_s,is_anlz(*),is_clim(*),is_gros(*),is_dens(*),
     *        is_vis(*),is_spik(*),is_cntv,ipres(*),is_jump(*),
     *        is_frzn,is_salg(*),is_regi(*)

      real    alat,alon,bot_tem,bot_prs,sur_prs,
     *        alon_hist(*),alat_hist(*),p(*),t(*),s(*),c(*),o(*),
     *        tem(nmax),pres(nmax),sal(nmax),cond(nmax),batt(nmax),
     *        doxy(nmax),
     *        bat,
     *        sigma_clim,sigma_anal,fill_value,
     *        factor,dummy(50,nmax),pres_check(nmax),
     *        drift_p(nx),drift_t(nx),drift_s(nx),drift_o(nx),
     *        drift_c(nx)

      character cline*256,cline1*256,column(*)*256,typ*3,prg_sub*9,
     *          calib(*)*256,itime_hist(*)*14,outqcfile*80,
     *          manual_qc_dir*29,inst_type*16,pi*64,whoi_c_date*14,
     *          trans_id*16,st_trans_time*19,juld_qc*1,
     *          whoi_pos_qc*1,drift_line*256,pos_sys*3,
     *          iclass*2,iclass_hist(*)*2,itime_first*14
      character bat_cpu*25,bat_pump*25,bat_cnt*25,
     *             bat_cur_sbe*25,prof_term*25
      character t_param(*)*32,t_value(*)*32

      double precision adate
      data     idbug/0/                ! make it 1 to get a listing
      data     prg_sub/'RD_PHY_QC'/
      data     manual_qc_dir/'/b4/qc_test/uniras/visual_qc/'/
      data     fill_value/99999./
c      data     factor /44.6596/
      data     factor /1.0/
      data     ncnt /0/

      data  itemp_col,isal_col,icond_col,ipres_col,icksm_col,
     *      ipres_flag_col,ioxy_col,ibatt_col,itgros_col,
     *      isgros_col,itvert_col,isdens_col,itspik_col,
     *      isspik_col,itanlz_col,isanlz_col,itclim_col,
     *      isclim_col,itvis_col,isvis_col,ipvis_col,ideep_col,
     *      itjump_col,isjump_col /24*0/
c
c--- For PSAL_ADJUSTED
c
      real    sal_adj(*)
      character psal_adj_exist*1
      integer is_regia(*),is_grosa(*),isala_col,isgrosa_col,
     *        isregia_col
      data isala_col,isgrosa_col,isregia_col /3*0/

c
c--- Fool prof reading the position lines case especially for B. Owens floats.
c  
      integer ll,mm,nn,jj,lchar_occur,index
c
      psal_adj_exist='n'       ! initialize 
      icnt_start=icnt+1
c
c---Read header information.
c
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from RD_PHY_QC >>>>>'
      st_trans_time='                   '      ! initialize 

 1    continue
      read(iunit,25,end=100,err=999) cline

      if (idbug.gt.0) write(66,*) cline
      if (cline(1:11).eq.'INTERNAL ID') then
        read(cline(41:51),*,err=999)int_num    
      else if (cline(1:13).eq.'WMO ID NUMBER') then
        if (cline(46:47).eq.'  ') then
          read(cline(41:51),*,err=999) wmo_num
        else
          read(cline(41:51),*,err=999) wmo_num
        end if                    ! if (cline(46:47).eq.'  ') then
      else if (cline(1:15).eq.'TRANSMISSION ID') then
        read(cline(41:56),25,err=999) trans_id
      else if (cline(1:14).eq.'PROFILE NUMBER') then
        read(cline(41:51),*,err=999) prof_num
      else if (cline(1:15).eq.'INSTRUMENT TYPE') then
        read(cline(41:56),25,err=999) inst_type
      else if (cline(1:3).eq.'PI ') then
        read(cline(41:80),25,err=999) pi
      else if (cline(1:18).eq.'POSITIONING SYSTEM') then
        read(cline(41:80),25,err=999) pos_sys
      else if (cline(1:19).eq.'DATE_CREATION OF NC') then
c        nparam=nparam+1
        read(cline(41:54),25,err=999) whoi_c_date
c        read(cline(41:73),25,err=999) t_value(nparam)
c        t_param(nparam)='DATE_CREATION OF NC'
      else if (cline(1:7).eq.'JULD_QC') then
        read(cline(41:41),25,err=999) juld_qc
      else if (cline(1:11).eq.'POSITION_QC') then
        whoi_pos_qc=cline(41:41)
      else if (cline(1:20).eq.'ARGOS PROGRAM NUMBER') then
        read(cline(41:56),25,err=999) argos_prg_num
      else if (cline(1:10).eq.'WMO INSTRU') then
        read(cline(41:43),30,err=999) wmo_inst 
      else if (cline(1:10).eq.'WMO RECORD') then
        read(cline(41:42),41,err=999) wmo_reco
      else if (cline(1:21).eq.'START OF TRANSMISSION') then
        read(cline(41:59),25,err=999) st_trans_time
      else if (cline(1:22).eq.'START SURFACE PRESSURE'.or.
     *         cline(1:16).eq.'SURFACE PRESSURE') then
        read(cline(41:45),42,err=999) sur_prs
      else if (cline(1:17).eq.'DRIFT TEMPERATURE') then
        ncnt=0
        read(cline(41:256),25,err=999) drift_line
        lnb=lnblnk(drift_line)
        do i=1,lnb
          if (drift_line(i:i).eq.'.') ncnt=ncnt+1
        end do
        read(cline(41:256),*,err=999) (drift_t(i),i=1,ncnt)
        n_drift=ncnt
        if (idbug.gt.0) write(66,*) (drift_t(i),i=1,ncnt)
      else if (cline(1:15).eq.'DRIFT PRESSURE') then
        ncnt=0                ! set number of drift pressure to be zero
        read(cline(41:256),25,err=999) drift_line
        lnb=lnblnk(drift_line)
        do i=1,lnb
          if (drift_line(i:i).eq.'.') ncnt=ncnt+1
        end do
        read(cline(41:256),*,err=999) (drift_p(i),i=1,ncnt)
        n_drift=ncnt
        if (idbug.gt.0) write(66,*) (drift_p(i),i=1,ncnt)
      else if (cline(1:20).eq.'DRIFT SALINITY (PSU)') then
        ncnt=0                ! set number of drift pressure to be zero
        read(cline(41:256),25,err=999) drift_line
        lnb=lnblnk(drift_line)
        do i=1,lnb
          if (drift_line(i:i).eq.'.') ncnt=ncnt+1
        end do
        read(cline(41:256),*,err=999) (drift_s(i),i=1,ncnt)
        if (idbug.gt.0) write(66,*) (drift_s(i),i=1,ncnt)
        n_drift=ncnt
c      else if (cline(1:19).eq.'DRIFT OXYGEN (ML/L)') then
      else if (cline(1:12).eq.'DRIFT OXYGEN') then   ! (micromol/kg)
        ncnt=0                ! set number of drift pressure to be zero
        read(cline(41:256),25,err=999) drift_line
        lnb=lnblnk(drift_line)
        do i=1,lnb
           if (drift_line(i:i).eq.'.') ncnt=ncnt+1
        end do
        read(cline(41:256),*,err=999) (drift_o(i),i=1,ncnt)
        do i=1,ncnt
          drift_o(i)=drift_o(i)*factor
        end do
        if (idbug.gt.0) write(66,*) (drift_o(i),i=1,ncnt)
        n_drift=ncnt
      else if (cline(1:26).eq.'DRIFT CONDUCTIVITY (MS/CM)') then
        ncnt=0                ! set number of drift conductivity to be zero
        read(cline(41:256),25,err=999) drift_line
        lnb=lnblnk(drift_line)
        do i=1,lnb
          if (drift_line(i:i).eq.'.') ncnt=ncnt+1
        end do
        read(cline(41:256),*,err=999) (drift_c(i),i=1,ncnt)
        if (idbug.gt.0) write(66,*) (drift_c(i),i=1,ncnt)
        n_drift=ncnt
c
c --- Land indicator.
c
      else if (cline(1:26).eq.'POSITION ON LAND INDICATOR') then
        read(cline(41:41),*,err=999) land
      else if (cline(1:17).eq.'NUMBER OF COLUMNS') then 
        read(cline(41:51),*,err=999) num_col
c
c --- Read column information.
c
        do i = 1, num_col                
          read(iunit,25,end=100,err=999) cline
          lastnb=lnblnk(cline)
          column(i) = cline
          if (idbug.gt.0) write(66,*) column(i)
        end do                                ! do i = 1, num_col
c
c --- Read calib information.
c
        i = 0
 5      read(iunit,26,end=100,err=998) cline1 ! calib line
        lastnb=lnblnk(cline1)
        if (cline1(1:5).eq.'CALIB') then
          i=i+1
          calib(i) = cline1
          if(idbug.gt.0) write(66,*) 'CALIB:',i,' ',calib(i)
          num_cal=i
          go to 5
        else
c
c--- Decide which type of float is?
c   
          typ=''
          ityp=0
          itemp_col=0
          isal_col=0
          icond_col=0
          ipres_col=0
          ioxy_col=0
          ibatt_col=0
          icol_read=0
          itgros_col=0
          isgros_col=0
          itvert_col=0
          isdens_col=0
          itspik_col=0
          isspik_col=0
          itvis_col=0
          isvis_col=0
          itclim_col=0
          isclim_col=0
          itanlz_col=0
          isanlz_col=0
          do i = 1, num_col
            if (column(i)(41:62).eq.'TEMPERATURE-90 (DEG C)') then
              typ(1:1)='t'
              ityp=ityp+1
              itemp_col=i
              icol_read=icol_read+1
            else if (column(i)(41:54).eq.'SALINITY (PSU)') then
              typ(2:2)='s'
              ityp=ityp+1
              isal_col=i
              icol_read=icol_read+1
            else if (column(i)(41:59).eq.'PSAL_ADJUSTED (PSU)') then
              typ(2:2)='s'
              ityp=ityp+1
              isala_col=i
              icol_read=icol_read+1
              psal_adj_exist='y'
c              write(6,*)'psal adjusted read=', icol_read,isala_col
            else if (column(i)(41:48).eq.'CONDUCTI') then
              typ(3:3)='c'
              ityp=ityp+1
              icond_col=i
              icol_read=icol_read+1
            else if (column(i)(41:55).eq.'PRESSURE (DBAR)') then
              ipres_col=i
c              ityp=ityp+1        ! @@@@@@ For 'qc' run comment this 
              icol_read=icol_read+1
            else if (column(i)(41:48).eq.'CHECKSUM') then
              icksm_col=i
              icol_read=icol_read+1
            else if (column(i)(41:59).eq.'PRESSURE INCREASING') then
              ipres_flag_col=i
              icol_read=icol_read+1
            else if (column(i)(41:56).eq.'DISSOLVED OXYGEN') then
              ioxy_col=i
              ityp=ityp+1
              icol_read=icol_read+1
            else if (column(i)(41:54).eq.'BATTERY (VOLT)') then
              ibatt_col=i
              icol_read=icol_read+1
            else if (column(i)(41:58).eq.'GROSS CHECK_T FLAG') then
              itgros_col=i
              icol_read=icol_read+1
            else if (column(i)(41:58).eq.'GROSS CHECK_S FLAG') then
              isgros_col=i
              icol_read=icol_read+1
            else if (column(i)(41:65).eq.'VERTICAL GRADIENT CHECK_T') 
     *      then
              itvert_col=i
              icol_read=icol_read+1
            else if (column(i)(41:56).eq.'DENSITY GRADIENT') then
              isdens_col=i
              icol_read=icol_read+1
            else if (column(i)(41:58).eq.'SPIKE CHECK_T FLAG') then
              itspik_col=i
              icol_read=icol_read+1
            else if (column(i)(41:58).eq.'SPIKE CHECK_S FLAG') then
              isspik_col=i
              icol_read=icol_read+1
            else if (column(i)(41:61).eq.'ANALYSIS CHECK_T FLAG') then
              itanlz_col=i
              icol_read=icol_read+1
            else if (column(i)(41:61).eq.'ANALYSIS CHECK_S FLAG') then
              isanlz_col=i
              icol_read=icol_read+1
            else if (column(i)(41:64).eq.'CLIMATOLOGY CHECK_T FLAG') 
     *      then
              itclim_col=i
              icol_read=icol_read+1
            else if (column(i)(41:64).eq.'CLIMATOLOGY CHECK_S FLAG') 
     *      then
              isclim_col=i
              icol_read=icol_read+1
            else if (column(i)(41:67).eq.'CONSTANT VALUE CHECK_T FLAG') 
     *      then
              icol_read=icol_read+1
            else if (column(i)(41:67).eq.'CONSTANT VALUE CHECK_S FLAG') 
     *      then
              icol_read=icol_read+1
            else if (column(i)(41:56).eq.'VISUAL QC_T FLAG') then
              itvis_col=i
              icol_read=icol_read+1
            else if (column(i)(41:56).eq.'VISUAL QC_S FLAG') then
              isvis_col=i
              icol_read=icol_read+1
            else if (column(i)(41:56).eq.'VISUAL QC_P FLAG') then
              ipvis_col=i
              icol_read=icol_read+1
            else if (column(i)(41:62).eq.'DEEPEST PRESSURE CHECK') then
              ideep_col=i
              icol_read=icol_read+1
            else if (column(i)(41:52).eq.'JUMP CHECK_T') then
              itjump_col=i
              icol_read=icol_read+1
            else if (column(i)(41:52).eq.'JUMP CHECK_S') then
              isjump_col=i
              icol_read=icol_read+1
            else if (column(i)(41:54).eq.'REGION CHECK_T') then
              itregi_col=i
              icol_read=icol_read+1
            else if (column(i)(41:54).eq.'REGION CHECK_S') then
              isregi_col=i
              icol_read=icol_read+1
            else if (column(i)(41:65).eq.'PSAL_ADJUST GROSS CHECK_S') 
     *      then
              isgrosa_col=i
              icol_read=icol_read+1
            else if (column(i)(41:58).eq.'PSAL_ADJUST REGION')then
              isregia_col=i
              icol_read=icol_read+1
            else if (column(i)(41:63).eq.'PSAL_ADJUST CLIMATOLOGY') 
     *      then
              icol_read=icol_read+1
            else if (column(i)(41:60).eq.'PSAL_ADJUST ANALYSIS') then 
              icol_read=icol_read+1
           else if (column(i)(41:65).eq.'VERTICAL GRADIENT CHECK_S') 
     *      then
              issalg_col=i
              icol_read=icol_read+1
            end if              ! if (column(i)(41:48).eq.'TEMPERAT') then
          end do                ! do i = 1, num_col
          
          if (icol_read.ne.num_col) then
            write(6,*)'WARNING on reading column rd_phy_qc.f'
            write(6,*)'A new column has been added.'
            write(6,*)'num_col=',num_col,' icol_read=',icol_read
          end if                 ! if (icol_read.ne.num_col) then
          if (idbug.gt.0) write(66,*)'type=',typ(1:3),ityp
          if (ityp.eq.9) then
            status=9
            write(50,*) prg_sub,': Unknown type encountered.  type=',
     *                  typ
            write(50,*) prg_sub,': STOP-> STATUS=',status
            call exit(status)
            stop '9'
          end if                ! if (ityp.eq.9) then
        end if                  ! if (cline1(1:5).eq.'CALIB') then
c
c --- Read lat/lon information.
c
        if (cline1(1:10).eq.'==========') then
          read(iunit,25,end=100,err=999) cline  ! Latitude Long.. line
c          write(6,*)'lat line reading',cline
          lastnb=lnblnk(cline)
          read(iunit,25,end=100,err=999) cline  ! +99.999 ... line
          if (cline(5:11).eq.'+99.999') then
            itime_first(1:14)= cline(24:27)//cline(29:30)//
     *      cline(32:33)//cline(35:36)//cline(38:39)//cline(41:42)
c            write(6,*)'1) itime_first=',itime_first(1:14)
          else
            itime_first(1:14)='              '
          end if
          lastnb=lnblnk(cline)
          if (cline(5:11).ne.'+99.999') go to 60 ! comment for netcdf
c          if (idbug.gt.0) write(66,*) cline
        else
c
c --- Read Overall flag and individual test flags.
c
c          write(6,*)'cline1=',cline1
c          write(6,*)'ityp=',ityp
          if (ityp.ge.1) then
           if (cline1(1:13).eq.'MAXIMUM SPEED') then
              read(cline1(41:41),41,err=999) ispd
c              write(6,*)'rd_phy_qc: ispd=',ispd
           end if
           if (cline1(1:18).eq.'OVERALL FLAG FOR T') 
     *      read(cline1(41:41),41) iover_t 
 133       read(iunit,25,end=100,err=999) cline
           if (cline(1:18).eq.'OVERALL FLAG FOR T') 
     *     read(cline(41:41),41) iover_t 
           lastnb=lnblnk(cline)
           if (cline(1:13).eq.'MAXIMUM SPEED') then
              read(cline(41:41),41,err=999) ispd
           end if
           if (cline(1:30).eq.'MAXIMUM CONSTANT VALUE CHECK_T')
     *     read(cline(41:41),41,err=999) it_cntv
           if (cline(1:13).eq.'ANALYSIS TIME') then
               read(cline(53:67),'(f15.9)',err=999) adate
           end if
           if (cline(1:18).eq.'OVERALL FLAG FOR S') 
     *     read(cline(41:41),41,err=999) iover_s 
           if (cline(1:30).eq.'MAXIMUM CONSTANT VALUE CHECK_S')
     *     read(cline(41:41),41,err=999) is_cntv
           if (cline(1:10).ne.'==========') go to 133
c            read(iunit,25,end=100) cline  ! =====...... line   ! bug 04-26-02
c            lastnb=lnblnk(cline)                               ! bug 04-26-02

            read(iunit,25,end=100,err=999) cline  ! Latitude Long.. line
            lastnb=lnblnk(cline)

            read(iunit,25,end=100,err=999) cline  ! +99.999 ... line
            if (cline(5:11).eq.'+99.999') then
              itime_first(1:14)= cline(24:27)//cline(29:30)//
     *        cline(32:33)//cline(35:36)//cline(38:39)//cline(41:42)
c              write(6,*)'2)itime_first=',itime_first(1:4)
            else
              itime_first(1:14)='              '
            end if
            lastnb=lnblnk(cline)

            if (cline(5:11).ne.'+99.999') go to 60 ! comment when for netcdf
          end if                           ! if (ityp.ge.1) then
        end if                             ! if (cline1(1:10).eq.'====.....
 40     read(iunit,25,end=100,err=999) cline
        lastnb=lnblnk(cline)

c        if (idbug.gt.0) write(66,*) cline
 60     if (cline(1:10).ne. '==========') then
          ll=index(cline(1:11),'.')
          mm=lchar_occur(cline,'.')
          nn=lchar_occur(cline,'/')
          jj=lchar_occur(cline,':')
          lastnb=lnblnk(cline)

          read(cline(1:ll+3),37,err=999) alat
          read(cline(mm-4:mm+3),38,err=999) alon
          read(cline(nn-7:nn-4),30,err=999) iyear
          read(cline(nn-2:nn-1),41,err=999) imon
          read(cline(nn+1:nn+2),41,err=999) iday
          read(cline(jj-5:jj-4),41,err=999) ihr
          read(cline(jj-2:jj-1),41,err=999) imin
          read(cline(jj+1:jj+2),41,err=999) isec
          iclass=cline(lastnb-1:lastnb)
c          write(6,*)'iclass:',iclass
          if (idbug.gt.0) 
     *    write(66,*) alat,alon,iyear,imon,iday,ihr,imin,isec
          icnt = icnt+ 1
c         The following commented lines are for netcdf only.        
c          if (icnt .eq. icnt_start .and. alat .le. 90) then
c             alon_hist(icnt) = 999.999
c             alat_hist(icnt) = 99.999
c             itime_hist(icnt) = cline(24:27)//cline(29:30)//
c     *       cline(32:33)//cline(35:36)//cline(38:39)//cline(41:42)
c             iclass_hist(icnt) = 0
c             icnt = icnt+ 1
c          endif                  ! if (icnt .eq. icnt_start ...
          alon_hist(icnt) = alon
          alat_hist(icnt) = alat
          itime_hist(icnt) = cline(nn-7:nn-4)//cline(nn-2:nn-1)//
     *      cline(nn+1:nn+2)//cline(jj-5:jj-4)//cline(jj-2:jj-1)//
     *      cline(jj+1:jj+2)
         iclass_hist(icnt) = iclass
          go to 40
        end if                   ! if (cline(1:10).ne. '==...
        if (icnt.ne.1) icnt = icnt - 1
        if (idbug.gt.0) write(66,*)'icnt=',icnt
c
c --- Read pres/tem/sal information.
c
        nlev=0
 50     continue
        nlev=nlev+1
        read(iunit,*,end=100,err=993) (dummy(i,nlev),i=1,num_col)
c        write(6,*)(dummy(i,nlev),i=1,num_col)
c --- Do not skip -999.0 data line in 'WHOI' float
        do i=1,num_col
c          if (trans_id(1:4).eq.'WHOI'.and.dummy(i,nlev).eq.-999.0) then
c            write(6,*)'WHOI float with -999.0 in data line'
c            nlev=nlev-1
c            go to 50
c          end if                  ! if (trans_id(1:4).....
        end do                    ! do i=1,num_col
        if (nlev.gt.nmax) then
          status=9
          write(50,*) prg_sub,': Data points are more than ',nmax
          write(50,*) prg_sub,': Increase a value nmax.'
          write(50,*) prg_sub,': STOP-> STATUS=',status
          call exit(status)
          stop '9'
        end if                    !  if (nlev.gt.nmax) then
        go to 50
c
c --- Profile number '000' case.
c
      else if (cline(1:10).eq.'=========='.and.prof_num.eq.0) then
        read(iunit,25,end=100,err=999) cline  ! Latitude Long.. line
        lastnb=lnblnk(cline)

 80     read(iunit,25,end=100,err=999) cline
        lastnb=lnblnk(cline)

 90     if (cline(1:10).ne. '==========') then
          ll=index(cline(1:11),'.')
          mm=lchar_occur(cline,'.')
          nn=lchar_occur(cline,'/')
          jj=lchar_occur(cline,':')
          lastnb=lnblnk(cline)

          read(cline(1:ll+3),37,err=999) alat
          read(cline(mm-4:mm+3),38,err=999) alon
          read(cline(nn-7:nn-4),30,err=999) iyear
          read(cline(nn-2:nn-1),41,err=999) imon
          read(cline(nn+1:nn+2),41,err=999) iday
          read(cline(jj-5:jj-4),41,err=999) ihr
          read(cline(jj-2:jj-1),41,err=999) imin
          read(cline(jj+1:jj+2),41,err=999) isec
          iclass=cline(lastnb-1:lastnb)
c          write(6,*)'iclass :2 ',iclass
          if (idbug.gt.0) 
     *    write(66,*) alat,alon,iyear,imon,iday,ihr,imin,isec
c
c--- Making sure that the first line in the position is 99.999 999.999 line.
c
          icnt = icnt+ 1
c         Following statements are for netcdf only.        
c          if (icnt .eq. icnt_start .and. alat .le. 90) then
c             alon_hist(icnt) = 999.999
c             alat_hist(icnt) = 99.999
c             itime_hist(icnt) = cline(24:27)//cline(29:30)//
c     *       cline(32:33)//cline(35:36)//cline(38:39)//cline(41:42)
c             iclass_hist(icnt) = '0'
c             icnt = icnt+ 1
c          endif                  ! if (icnt .eq. icnt_start ...
          alon_hist(icnt) = alon
          alat_hist(icnt) = alat
          itime_hist(icnt) = cline(nn-7:nn-4)//cline(nn-2:nn-1)//
     *      cline(nn+1:nn+2)//cline(jj-5:jj-4)//cline(jj-2:jj-1)//
     *      cline(jj+1:jj+2)
          iclass_hist(icnt) = iclass
          go to 80
        end if                 ! if (cline(1:10).ne. '==========') then
c
c--- Making sure that the last line in the position is 99.999 999.999 line.

c        if (alat_hist(icnt).le.90 .or. icnt.eq.icnt_start) then
c          icnt=icnt+1
c          alon_hist(icnt) = 999.999
c          alat_hist(icnt) = 99.999
c          itime_hist(icnt) = itime_hist(icnt-1)
c          iclass_hist(icnt) = '0'
c        endif                      ! if (alat_hist(icnt).le.90 .or. icnt.e...
        close (iunit)
        return
      else
        nparam=nparam+1
        read(cline(41:73),25,err=999) t_value(nparam)
        do i=1,31
         if (cline(i:i).eq.' '.and.cline(i+1:i+1).ne.' ') cline(i:i)='_'
        end do
        t_param(nparam)=cline(1:32)    
c        go to 1                   ! read next line
      end if                       ! if (cline(1:11).eq.'INTERNAL ID') then
      go to 1
 100  continue                     ! end of "phy" or "qc" file
      nlev=nlev-1

c     write(6,*)'End of file.'
c --- Reverse the pres, sal, and tem, cndc, flag values if data was collected
c --- while the float is coming up; otherwise leave them as they are.

      do i = 1, nlev
        pres_check(i)=dummy(ipres_col,i)
      end do                       ! do i = 1, nlev

      if ((wmo_reco.eq.60.or.wmo_reco.eq.62.or.wmo_reco.eq.64).and.
     *prof_num.lt.1000) ireverse=1     ! YES reverse
      if (wmo_reco.eq.61.or.wmo_reco.eq.63.or.wmo_reco.eq.65) 
     *                  ireverse=0     ! NO  reverse

      lastnb=lnblnk(trans_id)
      if (ireverse.eq.1) then
c        write(6,111) int_num,trans_id(1:lastnb),prof_num
      else if (ireverse.eq.0) then
c        write(6,112) int_num,trans_id(1:lastnb),prof_num
      else
        write(6,*) 'Warning!!! Unknown case',ireverse
      end if                       ! if (ireverse.eq.1) then
      
      if (ireverse.eq.1) then     ! YES reverse
        do i = 1, nlev
          if (ipres_col.gt.0) p(nlev-i+1) = dummy(ipres_col,i)
          if (itemp_col.gt.0) t(nlev-i+1) = dummy(itemp_col,i)
          if (isal_col.gt.0) s(nlev-i+1) = dummy(isal_col,i)
          if (isala_col.gt.0) sal_adj(nlev-i+1) = dummy(isala_col,i)
          if (icond_col.gt.0) c(nlev-i+1) = dummy(icond_col,i)
          if (ioxy_col.gt.0) o(nlev-i+1) = dummy(ioxy_col,i)*factor 
c       ! conversion for ml/l to mmols/m**3
          if (icksm_col.gt.0) cksm(nlev-i+1) = dummy(icksm_col,i)
          if (ipres_flag_col.gt.0) 
     *       ipres(nlev-i+1)=dummy(ipres_flag_col,i)
          if (itgros_col.gt.0) it_gros(nlev-i+1)=dummy(itgros_col,i)
          if (itvert_col.gt.0) it_vert(nlev-i+1)=dummy(itvert_col,i)
          if (itclim_col.gt.0) it_clim(nlev-i+1)=dummy(itclim_col,i)
          if (itanlz_col.gt.0) it_anlz(nlev-i+1)=dummy(itanlz_col,i)
          if (itspik_col.gt.0) it_spik(nlev-i+1)=dummy(itspik_col,i)
          if (ideep_col.gt.0)  ideep_flag(nlev-i+1)=dummy(ideep_col,i)
          if (itjump_col.gt.0) it_jump(nlev-i+1)=dummy(itjump_col,i)
          if (isgros_col.gt.0) is_gros(nlev-i+1)=dummy(isgros_col,i)
          if (isdens_col.gt.0) is_dens(nlev-i+1)=dummy(isdens_col,i)
          if (isclim_col.gt.0) is_clim(nlev-i+1)=dummy(isclim_col,i)
          if (isanlz_col.gt.0) is_anlz(nlev-i+1)=dummy(isanlz_col,i)
          if (isspik_col.gt.0) is_spik(nlev-i+1)=dummy(isspik_col,i)
          if (itvis_col.gt.0) it_vis(nlev-i+1)=dummy(itvis_col,i)
          if (isvis_col.gt.0) is_vis(nlev-i+1)=dummy(isvis_col,i)
          if (ipvis_col.gt.0) ip_vis(nlev-i+1)=dummy(ipvis_col,i)
          if (itregi_col.gt.0) it_regi(nlev-i+1)=dummy(itregi_col,i)
          if (isregi_col.gt.0) is_regi(nlev-i+1)=dummy(isregi_col,i)
          if (issalg_col.gt.0) is_salg(nlev-i+1)=dummy(issalg_col,i)
          if (isjump_col.gt.0) is_jump(nlev-i+1)=dummy(isjump_col,i)
          if (isgrosa_col.gt.0) is_grosa(nlev-i+1)=dummy(isgrosa_col,i)
          if (isregia_col.gt.0) is_regia(nlev-i+1)=dummy(isregia_col,i)
          if (idbug.gt.0) write(66,*)i,t(nlev-i+1),s(nlev-i+1),
     *                  p(nlev-i+1),cksm(nlev-i+1)
        end do
      else                  ! NO reverse 
        do i = 1, nlev
          if (ipres_col.gt.0) p(i) = dummy(ipres_col,i)
          if (itemp_col.gt.0) t(i) = dummy(itemp_col,i)
          if (isal_col.gt.0) s(i) = dummy(isal_col,i)
          if (isala_col.gt.0) sal_adj(i) = dummy(isala_col,i)
          if (icond_col.gt.0) c(i) = dummy(icond_col,i)
          if (ioxy_col.gt.0) o(i) = dummy(ioxy_col,i)*factor 
c       ! conversion for ml/l to mmols/m**3
          if (icksm_col.gt.0) cksm(i) = dummy(icksm_col,i)
          if (ipres_flag_col.gt.0)
     *       ipres(i)=dummy(ipres_flag_col,i)
          if (itgros_col.gt.0) it_gros(i)=dummy(itgros_col,i)
          if (itvert_col.gt.0) it_vert(i)=dummy(itvert_col,i)
          if (itclim_col.gt.0) it_clim(i)=dummy(itclim_col,i)
          if (itanlz_col.gt.0) it_anlz(i)=dummy(itanlz_col,i)
          if (itspik_col.gt.0) it_spik(i)=dummy(itspik_col,i)
          if (ideep_col.gt.0) ideep_flag(i)=dummy(ideep_col,i)
          if (itjump_col.gt.0) it_jump(i)=dummy(itjump_col,i)
          if (isgros_col.gt.0) is_gros(i)=dummy(isgros_col,i)
          if (isdens_col.gt.0) is_dens(i)=dummy(isdens_col,i)
          if (isclim_col.gt.0) is_clim(i)=dummy(isclim_col,i)
          if (isanlz_col.gt.0) is_anlz(i)=dummy(isanlz_col,i)
          if (isspik_col.gt.0) is_spik(i)=dummy(isspik_col,i)
          if (itvis_col.gt.0) it_vis(i)=dummy(itvis_col,i)
          if (isvis_col.gt.0) is_vis(i)=dummy(isvis_col,i)
          if (ipvis_col.gt.0) ip_vis(i)=dummy(ipvis_col,i)
          if (itregi_col.gt.0) it_regi(i)=dummy(itregi_col,i)
          if (isregi_col.gt.0) is_regi(i)=dummy(isregi_col,i)
          if (issalg_col.gt.0) is_salg(i)=dummy(issalg_col,i)
          if (isjump_col.gt.0) is_jump(i)=dummy(isjump_col,i)
          if (isgrosa_col.gt.0) is_grosa(i)=dummy(isgrosa_col,i)
          if (isregia_col.gt.0) is_regia(i)=dummy(isregia_col,i)
          if (idbug.gt.0) write(66,*)i,t(i),s(i),p(i),cksm(i)
        end do                   ! do i = 1, nlev
      end if                     ! if (ireverse.eq.1) then
      bot_tem=t(nlev)
      bot_prs=p(nlev)
 120  close (iunit)

c
c--- Making sure that the last line in the position is 99.999 999.999 line.
c
c      if (alat_hist(icnt).le.90 .or. icnt.eq.icnt_start) then
c         icnt=icnt+1
c         alon_hist(icnt) = 999.999
c         alat_hist(icnt) = 99.999
c         itime_hist(icnt) = itime_hist(icnt-1)
c         iclass_hist(icnt) = '0'
c      endif                      ! if (alat_hist(icnt).le.90 .or. icnt.e...
      go to 1000
 25   format(a)
 26   format(a256)
 30   format(i4)
 37   format(f11.3)
 38   format(f8.3)
 41   format(i2)
 42   format(f5.2)
 111  format(1x,'This dataset needs to reverse ---> ',
     *i4.4,'_',a,'_',i3.3)
 112  format(1x,'This dataset needs NO reverse ---> ',
     *i4.4,'_',a,'_',i3.3)
 993  write(6,*)'Reading rd_phy_qc.f(read dummy) error=993'
      write(50,*)'Reading rd_phy_qc.f(read dummy) error=993'
      status=9
      call exit(status)
 998  write(6,*)'Reading rd_phy_qc.f error=998 --->',cline1(1:80)
      write(50,*)'Reading rd_phy_qc.f error=998 --->',cline1(1:80)
      status=9
      call exit(status)
 999  write(6,*)'Reading rd_phy_qc.f error=999 --->',cline(1:80)
      write(50,*)'Reading rd_phy_qc.f error=999 --->',cline(1:80)
      status=9
      call exit(status)
 1000 continue
      close (iunit)
      return
      end 

   
