#!/bin/sh
#######################################################################
# Revision Date: 18-Oct-10 (RS)
#
# any_RFile_left_in_nclinks.sh
# ============================
# Checks if 'R*.nc' files in DATA_BUFR_NCLINKS_DIR are found in right
# after DATA_BUFR_NCLINKS_DIR after the script 'nc_to_gtsbuffer.sh' 
# has been executed.
#
# Alert
# -----
# This script must be scheduled to run after 'nc_to_gtsbuffer.sh'. 
#
# Definitions
# -----------
# DATA_BUFR_NCLINKS_DIR: ARGO_DIR/gts/data/bufr/nc_links
#######################################################################
. $HOME/.argorc_sh
#######################################################################
#----------------------------------------------------------------------
#                          SendEmail
#----------------------------------------------------------------------
SendEmail()
{
  type=$1
  title=$2
  body=$3

  echo "=> SendEmail Enter (type=$type)"

  case $type in
   'FAILURE') title="$up_root_script_name - $type, $title";;
   'FOUND')   title="$up_root_script_name - $type, $title";;
  esac

  echo "   [Email Begin]"
  echo "     To: $EMAIL_OPERATOR"
  echo "     Subj: $title"
  echo "     Text:"
  echo "$body" | awk '{print "   " $0}'
  printf "     Sending ... "
  echo "$body" | mailx  -s"$title" $EMAIL_OPERATOR
  echo "OK"
  echo "   [Email End]"
  
  echo "=> SendEmail Exit"
  return
}
#---------------------------- Main ------------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M.%S'`
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`
up_root_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

GTS_DIR=$ARGO_DIR/gts
DATA_BUFR_NCLINKS_DIR=$GTS_DIR/data/bufr/nc_links
LOG_DIR=$GTS_DIR/log
TMP_DIR=$GTS_DIR/tmp

logfile=$LOG_DIR/$root_script_name"_"$TODAY.log
error=$LOG_DIR/$root_script_name"_"$TODAY.error
anyfile=$TMP_DIR/$root_script_name"_"$TODAY.anyfile
exec 1> $logfile
exec 2> $error

echo "<--------------------------------------   Job Begin   -------------------------------->"
echo "=> Directories"
echo "   ARGO_DIR: $ARGO_DIR"
echo "   DATA_BUFR_NCLINKS_DIR: $DATA_BUFR_NCLINKS_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   TMP_DIR: $TMP_DIR"

echo "=> Logfiles"
echo "   logfile: $logfile"
echo "   error: $error"
echo "   anyfile: $anyfile"

# --------------------------------------------------------------------
#             Find files modified more than 'olderthandays'
# --------------------------------------------------------------------
ls $DATA_BUFR_NCLINKS_DIR/R*.nc > $anyfile 2> /dev/null
printf "=> Any files in $DATA_BUFR_NCLINKS_DIR? "
if  [ -s $anyfile ]
then
  echo "YES"
  type=FOUND   
  ref="'R*.nc' file(s)"
  msg="  'R*.nc' file(s) found in: $DATA_BUFR_NCLINKS_DIR\n"
  msg=$msg"  after the execution of 'nc_to_gtsbuffer.sh'.\n"
  msg=$msg"  See logfiles:\n   $logfile\n   $anyfile" 
  if [ -s $error ]
  then
    ref=$ref" and Standard-Error reported"
    msg=$msg"\n   $error"
  else
    rm -f $error
  fi
  SendEmail $type "$ref" "$msg"
else
  echo "NO"
  rm -f $anyfile
  if [ -s $error ]
  then
    type=FAILURE
    ref="No 'R*.nc' file found  but Standard-error reported"
    msg="  No 'R*.nc' file found in: $DATA_BUFR_NCLINKS_DIR\n"
    msg=$msg"  after the execution of 'nc_to_gtsbuffer.sh'.\n"
    msg=$msg"  See logfiles:\n   $logfile\n   $error"
    SendEmail $type "$ref" "$msg"
  else
    rm -f $error
  fi
fi #endif - anyfiles?

echo "<------------------------------------- Job Finished ---------------------------------->"
exit
