#!/bin/sh
# ==================================================================================
# Revision Date: 07-Nov-12 (RS - code changes related to the creation of the 
#                                'aomlid' directory)
#                19-Oct-09 (RS - improve diagnostic prints and improve documentation
#                                documentation)
#                08-May-06 (RS - fix typo in  in the following line
#                               'while read profile date time lat lon')
#                10-Apr-06 (RS - take profile position and date-time
#                                ,yyyy/mm/dd hh:mm, from last line
#                                of the f90 output file)
#                06-Feb-06 (RS - trap when DIMENSION is exceeded in the
#                                Fortran program)
#                02-Nov-04 (RS - new argument in call to qc2gts_format.x)
#                05-May-04 (RS - new handler for the Fortran exit status) 
#                11-Feb-04 (RS - change to read QC files instead of PHY)
#                19-Aug-03 (RS - rm empty log files)
#                23-Jun-03 (RS - correction to PHY2GTS Format table)
#                01-May-03 (RS - 4 digits for profile number &
#                                10 char for trasmission Id)
#                26-Feb-02 (RS - include in email profiles with no WMO)
#                06-Feb-02 (RS - suppress FTPGTS4EMAIL_TABLE printout)
#                28-Jan-02 (RS - add file name 'to_gtsgateway' to email)
#                15-Jan-02 (CS - added 2 as possible output status for
#                                fortran program)
#                21-Sep-01 (RS - change ARGOSCONNECTION_CYCLE_TIME)
#                19-Sep-01 (RS - add documentation to SUMMARY_PHY2GTSFMT)
#                19-Jul-01 (RS - back to 5 digits)
#                11-Jul-01 (RS - argos id expansion)
#                21-May-01 (RS - update documentation)
#                20-Apr-01 (RS - returns from Fortran program)
#                29-Mar-01 (RS - add FTPGTS4EMAIL_TABLE table)
#                31-Jan-01 (RS - updates for complete field in PHY2GTSFMT)
#                25-Jan-01 (RS - updates to updated_phygtsfmt_table & rm
#                05-Jan-01 (RS - updates to SUMMARY_PHY2GTSFMT)
#                15-Dec-00 (RS - changed PRV_DATE to PHY_DATE, 
#                                PRV_TIME to PHY_TIME and
#                                prv_date/prvtime to phy_date/phy_time)
#                17-Aug-00 (RS - implement retrun status from 
#                                the Fortran program)
#                04-AUg-00 (RS - updates for F77 interface)
#
# qc2gts_format.sh
# ================
# (1) Runs Fortran program (qc2gts_format.f) to convert quality control
#  physical profile(s) to GTS-Format; one file will contain converted profile(s). 
#
# (2) Creates/maintain the FTPGTS4EMAIL_TABLE with the following column headings:
#
#     gts_file   - gts file containing GTS formatted profile(s) under the
#                  'profile_id' column.
#     profile_id - identification of the QC profile (0014_12275_085; aomid, argoid
#                  and profile number) converted to GTS Format.
#     wmo        - 'World Meteorological Organization' float identification number
#     lat        - profile latitude (sxx.xxx) taken from 1st position record 
#     lon        - profile longitude (sxxx.xxx) taken from 1st position record 
#     date       - profile date (yyyy/mm/dd) taken from 1st position record 
#     time       - profile time (hh:mm) taken from 1st position record 
#                   
#
# For a profile to be a candidate for conversion the following conditions
# should be met:
#
#   (1) Elapsed time (hours) should be greater than an Argos connection
#   cycle time. The elpased time is the difference (hours) between the
#   PHY profile first position date/time and today's date/time.
#
#   (2) Flagged by the QC as complete
#
# Possible return Exit Status values from Fortran program
# -------------------------------------------------------
# 0, profile converted successfully - OK to send to GTS 
# 1, profile older than 30 days -  DO NOT send to GTS
# 2, data value error, see 'fill_kkyy' in 'pala2kkyy_ts.f' - DO NOT send to GTS
# 3, WMO identification number missing - DO NOT send to GTS  
# 9, I/O Fortran error - DO NOT send to GTS 
# other value(s), System/Fortran error - Execution Terminated
#
# Notes
# -----
# Fortran progran (qc2gts_format.f) looks for following line in the meta file
#         send to GTS                             y
# If value=y, encodes the profile; if no other error founds, it returns 0     
# If line is not found, encodes the profile; if no other error founds, it returns 0     
# If value=n, does not encodes the profile; returns 2
# ==================================================================================
ElapsedHours()
{
  idate=$1
  iyear=`echo $idate | awk -F"/" '{print $1}'`
  imon=`echo $idate  | awk -F"/" '{print $2}'`
  iday=`echo $idate  | awk -F"/" '{print $3}'`
  ihour=`echo $2 | awk -F":" '{print $1}'`

  ijulday=`$ARGOUNIX_CMD_DIR/ymd2julday.x $iyear $imon $iday`
# echo "idate: $1 iyear: $iyear imonth: $imon iday: $iday julday: $ijulday"

  ihours=`expr $ijulday \* 24`
  ihours=`expr $ihours + $ihour`

  ldate=$3
  lyear=`echo $ldate | awk -F"/" '{print $1}'`
  lmon=`echo $ldate  | awk -F"/" '{print $2}'`
  lday=`echo $ldate  | awk -F"/" '{print $3}'`
  lhour=`echo $4 | awk -F":" '{print $1}'`

  ljulday=`$ARGOUNIX_CMD_DIR/ymd2julday.x $lyear $lmon $lday`

  lhours=`expr $ljulday \* 24`
  lhours=`expr $lhours + $lhour`

  elapsed_hours=`expr $lhours - $ihours`
  echo $elapsed_hours
  
  exit
}
# -------------------------------------------------------------------------------
#  Revision Date: 23-Mar-91 (RS)
# 
#  Update_FTPGTS4EMAIL_TABLE
#  ==========================
#  Creates/maintains the FTPGTS4EMAIL_TABLE with the following column headings:
#
#     gts_file   - gts file containing GTS formatted profile(s) under the
#                  'profile_id' column.
#     profile_id - identification of the QC profile (0014_12275_085; aomid, argoid
#                  and profile number) converted to GTS Format.
#     wmo        - 'World Meteorological Organization' float identification number
#     lat        - profile latitude (sxx.xxx) taken from 1st position record
#     lon        - profile longitude (sxxx.xxx) taken from 1st position record
#     date       - profile date (yyyy/mm/dd) taken from 1st position record
#     time       - profile time (hh:mm) taken from 1st position record
#
# Check if FTPGTS4EMAIL_TABLE exists, if not create it
# -------------------------------------------------------------------------------
Update_FTPGTS4EMAIL_TABLE()
{
   echo "=> Enter Update_FTPGTS4EMAIL_TABLE"
   if [ -f $FTPGTS4EMAIL_TABLE ]
   then
     echo "$script_name:  FTPGTS4EMAIL_TABLE Exists"
     echo "   ----- $FTPGTS4EMAIL_TABLE (only 'head' & 'tail' -----"
     head $FTPGTS4EMAIL_TABLE
     echo "."
     echo "."
     echo "."
     tail $FTPGTS4EMAIL_TABLE
   else
     echo "$script_name:  FTPGTS4EMAIL_TABLE Created"
     touch $FTPGTS4EMAIL_TABLE
   fi

   echo "=> Adding GTS formatted QC profile(s) to FTPGTS4EMAIL_TABLE"
   nameonly=`basename $to_gtsgateway`

# ----------------------------------------------------------------
#        Loop over profile(s) converted to GTS Format
# ----------------------------------------------------------------
   echo "Loop - insert GTS formatted profile(s)"
   exec < $success_gts_list
   while read profile date time lat lon
   do
     aomlid=`echo $profile | awk -F'_' '{print $1}'`
     pathname=$QCFILES_DIR/$aomlid/$profile
     echo "aomlid: $aomlid pathname: $pathname"
     wmo=`grep "WMO ID NUMBER" $pathname | awk '{print $4}'`
     profile_noext=`echo $profile | awk -F"." '{print $1}'` 
     printf "%s %-20s %7d %7.3f %8.3f %s %5s\n" \
            $nameonly $profile_noext $wmo $lat $lon $date $time 
                                                  # used by 'ftpgts_to_gateway.sh'
     printf "%s %-20s %7d %7.3f %8.3f %s %5s\n" \
            $nameonly $profile_noext $wmo $lat $lon $date $time >> $FTPGTS4EMAIL_TABLE 
   done

   echo "   ----- Updated FTPGTS4EMAIL_TABLE (only tail) -----" 
   tail $FTPGTS4EMAIL_TABLE

   echo "=> Exit Update_FTPFTS4EMAIL_TABLE"

   return
}
StatusActions()
{
  echo "=> Enter StatusActions"

  equal="==============================================================="

  if [ $sysfailure -gt 0 ]                     # Did an exec failure occur?
  then                                             #  Yes, Email
    ret_val=10
    echo "Exit Status (Fortran): $status"
    echo "Return Value (ret_val) set to $ret_val"
    subj="QC2GTS_FORMAT - ERROR, System/Fortran Error (Execution Terminated)"
    msg="Program execution terminated during the conversion to GTS-Format\n"
    msg=$msg"of profile '$qc2gtsfmt_failed_prof'.\n"
    msg=$msg"\nFor details on the FORTRAN-program execution failure look at"
    msg=$msg"\nthe following log-files:"
    msg=$msg"\n\t$log_file"
    msg=$msg"\n\t$qc2gts_format_log"
    msg=$msg"\n\t$trace_outfile"
    msg=$msg"\n\t$error_outfile"
    echo $msg > $EMAIL_MESSAGE
    rm -f $tmp_outfile $updated_qc2gtsfmt_table
    echo "=> Exit  StatusActions"
    return
  fi

  if [ $counter -gt 0 ]   # Any successfull conversion? 
  then                    #   Yes
    ret_val=0
    echo "Return Value (ret_val) set to $ret_val"
    printf "Move: $updated_qc2gtsfmt_table -> $PHY2GTSFMT_TABLE - "
    mv $updated_qc2gtsfmt_table $PHY2GTSFMT_TABLE #  move to argo/lists dir
    echo "OK"
    ls -l $tmp_outfile
    printf "Move: $tmp_outfile ->  $to_gtsgateway - "
    mv $tmp_outfile $to_gtsgateway               #  move to gts/data dir
    echo "OK"

    Update_FTPGTS4EMAIL_TABLE    

    echo "\n$equal"                                   > $EMAIL_MESSAGE
    echo "                 GTS-Format Control Table" >> $EMAIL_MESSAGE
    echo "                     After Conversion"     >> $EMAIL_MESSAGE
    echo $equal
    echo "    Profile             First  Position   Complete   GTS-Format" >> $EMAIL_MESSAGE
    echo $equal >> $EMAIL_MESSAGE
    cat $PHY2GTSFMT_TABLE  >> $EMAIL_MESSAGE
    echo $equal >> $EMAIL_MESSAGE
    msg="\nFollowing QC Profile(s) were successfully converted to GTS-Format \n"
    msg=$msg"and store in file '$to_gtsgateway'\nfor FTP to GTS-Gateway:"
    echo $msg >> $EMAIL_MESSAGE
    awk '{print $1}'  $success_gts_list >> $EMAIL_MESSAGE

    if [ -s $notqualify_list ]
    then
      msg="\nFollowing QC Profile(s) did not qualify for conversion to GTS-Format"
      echo $msg >> $EMAIL_MESSAGE
      cat $notqualify_list >> $EMAIL_MESSAGE
    fi

    if [ -s $dimexceeded_list ]
    then
      msg="\nFollowing QC Profile(s) reported Fortran Dimension Exceeded"
      echo $msg >> $EMAIL_MESSAGE
      cat $dimexceeded_list >> $EMAIL_MESSAGE
    fi

    if [ -s $iofailure_list ]
    then
      msg="\nFollowing QC Profile(s) reported Fortran I/O Failure"
      echo $msg >> $EMAIL_MESSAGE
      cat $iofailure_list >> $EMAIL_MESSAGE
    fi

    if [ $notqualify -eq 0 -a $iofailure -eq 0 ]
    then
       subj="QC2GTS_FORMAT - OK, Profile(s) Conversion Done"
    else 
      if [ $notqualify -gt 0 -a $iofailure -gt 0 ]
      then
        subj="QC2GTS_FORMAT - ERROR, Not All Profiles Qualified for Conversion & I/O Failure Reported"
      else
        if [ $iofailure -eq 0 ]
        then 
          subj="QC2GTS_FORMAT - OK, but Not All Profiles Qualified for Conversion"
        else
          subj="QC2GTS_FORMAT - ERROR, I/O Failures (Not All Qualified Profiles were Converted)"
        fi 
      fi # endif: notqualify>0 and iofailure>0 
    fi # endif: notqualify=0 and iofailure=0
  else                       # No, counter = 0 (No qualify profile and/or i/o failure reported)
    ret_val=1
    echo "Return Value (ret_val) set to $ret_val"

    if [ -s $PHY2GTSFMT_TABLE ]
    then
      echo "\n$equal"       >> $EMAIL_MESSAGE
      echo "                 GTS-Format Control Table" >> $EMAIL_MESSAGE
      echo "                    Before Conversion"     >> $EMAIL_MESSAGE
      echo $equal
      echo "    Profile             First  Position   Complete   GTS-Format" >> $EMAIL_MESSAGE
      echo $equal >> $EMAIL_MESSAGE
      cat $PHY2GTSFMT_TABLE  >> $EMAIL_MESSAGE
      echo $equal >> $EMAIL_MESSAGE
    fi

    if [ $iofailure -eq 0 ]  # Any I/O failure reported?
    then                     #   No
      subj="QC2GTS_FORMAT - All Profile(s) did not qualify for conversion"
      msg="\nAll Profile(s) did not qualify for GTS-Format conversion, see logfiles:\n"
      msg=$msg"\n\t$log_file"
      msg=$msg"\n\t$trace_outfile"
      echo $msg >> $EMAIL_MESSAGE
    else                     #   Yes
      subj="QC2GTS_FORMAT - All Profile(s) did not qualify for conversion and/or I/O failure reported"
      if [ -s $notqualify_list ]
      then
        msg="\nProfile(s) that did not qualify for conversion:"
        echo $msg >> $EMAIL_MESSAGE
        cat $notqualify_list >> $EMAIL_MESSAGE
      fi
      msg="\nFollowing QC Profile(s) reported Fortran I/O Failure"
      echo $msg >> $EMAIL_MESSAGE
      cat $iofailure_list >> $EMAIL_MESSAGE
      msg="See logfiles:"
      msg=$msg"\n\t$log_file"
      msg=$msg"\n\t$trace_outfile"
      echo $msg >> $EMAIL_MESSAGE
    fi
    rm $updated_qc2gtsfmt_table
  fi # endif: counter > 0

  echo "=> Exit  StatusActions"
  return
}
#-------------------------- Main (qc2gts_format) --------------------------
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}'`

# Define stderror file
# --------------------
error=$GTSLOG_DIR/$root_script_name.$TODAY"_stderror"   # Error Logfile
touch $error
exec 2>>$error

now=`date +'%h %d, %Y %T %p'`
echo "<++++++++++++ Start  $script_name: $now ++++++++++++>"
echo "TIME_EXEC2GTSFORMAT: $TIME_EXEC2GTSFORMAT"

EMAIL_MESSAGE=$GTSTMP_DIR/$root_script_name.$TODAY"_"email
touch $EMAIL_MESSAGE

success_gts_list=$GTSLIST_SUCCESS_DIR/qc2gtsfmt.$TODAY"_"success
updated_qc2gtsfmt_table=$GTSTMP_DIR/updated_qc2gtsfmt_table.$TODAY

touch $success_gts_list
touch $updated_qc2gtsfmt_table

# Output logfiles are used by Fortran program (qc2gts_format.f) 
# ----------------------------------------------------------------------------
trace_outfile=$GTSLOG_DIR/$root_script_name.$TODAY"_trace"  
error_outfile=$GTSLOG_DIR/$root_script_name.$TODAY"_error"
debug_outfile=$GTSLOG_DIR/$root_script_name.$TODAY"_debug"  

export trace_outfile error_outfile debug_outfile

tmp_outfile=$GTSTMP_DIR/tmp2gtsgateway.$TODAY.gts     # use temporarily for
export tmp_outfile
                                                      # converted profile(s)   
to_gtsgateway=$GTSFILES_DIR/to_gtsgateway.$TODAY.gts  # if no conversion failure
                                                      # moved here  
floatfail_trace_error_list=$GTSLOG_DIR/float_failure.$TODAY"_"trace"_"error.list
qc2gtsfmt_stderror=$GTSLOG_DIR/$root_script_name.$TODAY.stderror    # stderr logfile 

time_zone=`date +%Z`

# Diplay QC to GTS-Format Control Table
# -------------------------------------
equal="==============================================================="
echo "\n$equal"
echo "                 GTS-Format Control Table"
echo "                    Before Conversion"
echo $equal
echo "    Profile             First  Position   Complete   GTS-Format"

if [ -s $PHY2GTSFMT_TABLE ]
then
 cat $PHY2GTSFMT_TABLE
 echo $equal
else
  echo "No profiles to convert - table is empty"
fi

note="\nNote -  profiles with elapsed time greater than an Argos connection "
note=$note"cycle time\nof ($TIME_EXEC2GTSFORMAT) hours will be converted"

note=$note" to GTS-FORMAT for transmission.\nElapsed time (hours) is the "
note=$note"difference between today's date/time and date/time in\n"
note=$note"the PHY profile first position record.\n"
echo $note

SUMMARY_PHY2GTSFMT=$GTSLOG_DIR/SUMMARY_PHY2GTSFMT.$TODAY
equal2='====================================================================================='
printf '%s\n' $equal2 > $SUMMARY_PHY2GTSFMT
printf '               Results from QC2GTS_FORMAT Process of QC Profiles\n' \
       >> $SUMMARY_PHY2GTSFMT
printf '                    Candidate for GTS Format Conversion\n' >> $SUMMARY_PHY2GTSFMT
printf '                         %s %s %s\n' $TODAY_DATE $TODAY_TIME $time_zone \
       >> $SUMMARY_PHY2GTSFMT
printf '%s\n' $equal2 >> $SUMMARY_PHY2GTSFMT

echo   'Profiles older than 30 days will not be converted to GTS-Format.' \
        >> $SUMMARY_PHY2GTSFMT
echo   "\t\tPossible values under 'GTS Format'" >> $SUMMARY_PHY2GTSFMT
echo   "   y - converted    n - not converted" >> $SUMMARY_PHY2GTSFMT
echo   "   d - dimension exceeded in fortran program" >> $SUMMARY_PHY2GTSFMT
echo   "   f - i/o error or system/fortran exec error" >> $SUMMARY_PHY2GTSFMT
echo   "   o - too old or no WMO number available or no position or t/s value error" >> $SUMMARY_PHY2GTSFMT
dash='------------------------------------------------------------------------------' 
echo $dash >> $SUMMARY_PHY2GTSFMT

printf '     QC Profile              First   Position  Complete Elapsed  GTS-Format   Removed\n' \
       >> $SUMMARY_PHY2GTSFMT
printf '                             Date     Hours             Hours\n'  >> $SUMMARY_PHY2GTSFMT 

nowmo_list=$GTSTMP_DIR/nowmo_list.$TODAY"_list"
notqualify_list=$GTSTMP_DIR/notqualify_list.$TODAY"_list"
dimexceeded_list=$GTSTMP_DIR/dimexceeded_list.$TODAY"_list"
iofailure_list=$GTSTMP_DIR/iofailure_list.$TODAY"_list"
touch $nowmo_list $iofailure_list $notqualify_list

counter=0
iofailure=0
sysfailure=0
notqualify=0
dimexceeded=0

qc2gts_format_log=$GTSLOG_DIR/qc2gts_format.$TODAY"_forlog"

dash='-------------------------------------------------------------------------------------'
dash=$dash$dash

# ----------------------------------------------------------------------------
# Profile Loop for GTS-Format conversion:
#   1) Read profile from table
#   2) Check conversion criteria - must meet one of the following contidions
#        - elapsed hours > $TIME_EXEC2GTSFORMAT
#        - profile flagged as 'complete' by QC
#   3) If criteria is met, run Fortran conversion program (qc2gts_format.x) 
#
# Fortran Code Exit Status:
#    0, OK
#    1, too old (over 30 days)
#    2, one or less data points or temperature/salinity value error
#    3, no WMO number
#    4, no position
#    9, I/O errors
#    other, system/fortran execution error (value set by system/compiler)  
# ----------------------------------------------------------------------------
exec < $PHY2GTSFMT_TABLE 
while read profile_ext phy_date phy_time complete gtsfmt 
do
  aomlid=`echo $profile_ext | awk -F'_' '{print $1}'`
  root=`echo $profile_ext | awk -F"." '{print $1}'`
  metafile=`echo $root | awk -F"_" '{print $1 "_" $2}'`
  pathname_metafile=$METAFILES_DIR/$metafile.meta
  profile=$root.qc

  printf '%-23s   %s  %5s       %s    ' \
         $profile $phy_date $phy_time $complete >> $SUMMARY_PHY2GTSFMT
  elapsed_hours=`ElapsedHours $phy_date $phy_time $TODAY_DATE $TODAY_TIME`
  printf ' %05d   ' $elapsed_hours >> $SUMMARY_PHY2GTSFMT
  echo "<<< $profile_ext elapsed_hours=$elapsed_hours gtsfmt=$gtsfmt complete=$complete >>>" 
  if [ $gtsfmt = "n" ]                                    # Is it in GTS-Formated?
  then                                                    #  No, not in GTS-Format 
    if [ $elapsed_hours -gt $TIME_EXEC2GTSFORMAT -o $complete = "y" ] # Is it a candidate?
    then                                                                     #  Yes ->  GTS-Format 
      in_file=$QCFILES_DIR/$aomlid/$profile                                 
      $GTSPRG_DIR/qc2gts_format.x  $in_file $tmp_outfile $pathname_metafile \
                                   $counter >> $qc2gts_format_log
      status="$?"
      last_line=`tail -1 $qc2gts_format_log`
      echo "status_for=$status\nLast lines in $qc2gts_format_log:"
      echo $dash  >> $qc2gts_format_log

      case $status in
            0) counter=`expr $counter + 1` 
               printf '     y           n\n' >> $SUMMARY_PHY2GTSFMT
               printf '%-23s %s %5s      %s          y\n' \
                      $profile $phy_date $phy_time $complete >> $updated_qc2gtsfmt_table 
               date=`echo $last_line | awk '{print $2}'`
               time=`echo $last_line | awk '{print $3}'`
               lat=`echo $last_line | awk '{print $4}'`
               lon=`echo $last_line | awk '{print $5}'`
               printf '%-23s %-10s %5s %s %s\n' \
                      $profile $date $time $lat $lon >> $success_gts_list;; 
        1|2|4) notqualify=`expr $notqualify + 1`
               printf '     o           y\n' >> $SUMMARY_PHY2GTSFMT
               echo  $profile >> $notqualify_list;; 
            3) notqualify=`expr $notqualify + 1` 
               printf '     o           y\n' >> $SUMMARY_PHY2GTSFMT
               echo $profile >> $nowmo_llist 
               echo  $profile >> $notqualify_list;;
            5) dimexceeded=`expr $dimexceeded + 1` 
               printf '     d           n\n' >> $SUMMARY_PHY2GTSFMT 
               echo $profile > $dimexceeded_list;; 
            9) iofailure=`expr $iofailure + 1` 
               printf '     f           n\n' >> $SUMMARY_PHY2GTSFMT 
               qc2gtsfmt_failed_prof=$profile 
               echo $profile > $iofailure_list;; 
            *) sysfailure=`expr $sysfailure + 1` 
               printf '     f           n\n' >> $SUMMARY_PHY2GTSFMT 
               qc2gtsfmt_failed_prof=$profile 
               break;;       
      esac
    else                                                  # Do not qualify for conversion 
      printf '     n           n\n' >> $SUMMARY_PHY2GTSFMT # (elapsed time <= threshold)
      printf '%-23s %s %5s      %s          n\n' \
             $profile $phy_date $phy_time $complete >> $updated_qc2gtsfmt_table
    fi # endif: elapsed hours > threshold -> GTS-Format 
  else                                                    # Yes, already GTS-Formatted
    if [ $elapsed_hours -le $PHY2GTSFMT_TIME2REMOVE ]     # Keep profile in table?
    then                                                  #  Yes, append to table 
    printf '     y           n\n' >> $SUMMARY_PHY2GTSFMT
    printf '%-23s %s %5s      %s          y\n' \
           $profile $phy_date $phy_time $complete >> $updated_qc2gtsfmt_table
    else                                                  #  No, do not append to table
      printf '     %s           y\n' $gtsfmt >> $SUMMARY_PHY2GTSFMT
    fi # endif: elapsed_hours < threshold for removal
  fi # endif: gtsfmt = 'n'
done

echo "<<<notqualify: $notqualify dimexceeded: $dimexceeded iofailure: $iofailure sysfailure: $sysfailure converted: $counter>>>"

echo $equal2 >> $SUMMARY_PHY2GTSFMT
echo $SUMMARY_PHY2GTSFMT
cat  $SUMMARY_PHY2GTSFMT

if [ -s $success_gts_list ]
then
  echo "\nQC Profiles Converted Successfully to GTS Format"
  echo "Location: $success_gts_list"
  awk '{print $1}' $success_gts_list
fi

if [ -s $notqualify_list ]
then
  echo "\nQC Profiles that Did Not Qualify for GTS Format Conversion"
  echo "Location: $notqualify_list"
  cat $notqualify_list
fi

if [ -s $dimexceeded_list ]
then
  echo "\nQC Profiles that reported Fortran Dimension Exceeded"
  echo "Location: $dimexceeded_list"
  cat $dimexceeded_list
fi

if [ -s $iofailure_list ]
then
  echo "\nQC Profiles that reported Fortran I/O Failure"
  echo "Location: $iofailure_list"
  cat $iofailure_list
fi

# ------------------------------------------------------
# Call Exit Status Handler and Email creation
# ------------------------------------------------------
StatusActions

rm -f $iofailure_list $sysfailure_list $notqualify_list

# ------------------------------------------------------
# Note - error/EMAIL_MESSAGE files always exists (touch) 
# ------------------------------------------------------
if [ ! -s $debug_outfile ]
then
  rm -f $debug_outfile
fi

if [ ! -s $error_outfile ]
then
  rm -f $error_outfile
fi

# Is error (stderr) file empty?
# -----------------------------
if [ -s $error ]
then
  if [ $ret_val -eq 0 ]
  then
    msg="\nStandard-Error log-files:\n"
    msg=$msg"\n\t$log_file"
  fi
  msg=$msg"\n\t$error"
  echo $msg
  echo $msg >> $EMAIL_MESSAGE
else
  rm -f $error
fi 

# Is the NO-WMO list empty?
# -------------------------
if [ -s $nowmo_list ]
then
  printf "\nProfile(s) with no WMO number:\n" >> $EMAIL_MESSAGE
  cat $nowmo_list  >> $EMAIL_MESSAGE
else
  rm -f $nowmo_list
fi
 
# Is the Email message file empty?
# -------------------------------- 
if [ -s $EMAIL_MESSAGE ]             # Is it empty? 
then                                 #   No, send email
  echo "=> [Email Begin]"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Subj: $subj"
  echo "   Text:"
  awk '{print "   " $0}' $EMAIL_MESSAGE
  printf "   Sending email ..."
  mailx -s "$subj" $EMAIL_OPERATOR < $EMAIL_MESSAGE
  echo "OK"
  echo "=> [Email End]"
else                                 # Remove empty file
  rm -f $EMAIL_MESSAGE
fi

echo "=> Exit Status: $ret_val"
echo "<++++++++++++ Finish $script_name: $now ++++++++++++>"

exit $ret_val
