#!/bin/sh
#########################################################################################
# Revision Date: 23-Sep-03 (XXD - created function to split FTPGTS4EMAIL_TABLE)        
#
# split_ftpgts4email_table.sh
# ===========================
# Splits table FTPGTS4EMAIL_TABLE located in the following directories:
#
#     ARGO_DIR/lists and ARGO_DIR/mannualprocess/lists at AOML 
#     ARGO_DIR/lists at Services Argos
#
# into file(s) according to year. The year will be appended to the table
# name for each split file (e.g. FTPGTS4EMAIL_TABLE.2001). IF more thatn one split file
# was created the current year  will be renamed to FTPGTS4EMAIL_TABLE.  Split file(s)
# will reside in the same directory as the 'FTPGTS4EMAIL_TABLE'. 
#
# Tasks
# -----
# (1) copy FTPGTS4EMAIL_TABLE to a temporary working area (ARGO_DIR/gts/tmp or
# (2) Split table for each year. Current year is rename to FTPGTS4EMAIL_TABLE
#if more than one split file was created. 
#########################################################################################

# ---------------------------------------------------------------------------------------
# Split_FTPGTS_Table
# ------------------ 
# Function will split the FTPGTS4EMAIL_TABLE table into several files according to year
# The most recent data file will be renamed to FTPGTS4EMAIL_TABLE.temp, later it will
# be renamed to FTPGTS4EMAIL_TABLE
# ---------------------------------------------------------------------------------------
Split_FTPGTS_Table()
{
   echo "=> Enter Split_FTPGTS_Table"

   cnt=0
   split=0
   split_list=$LOG_DIR/$root_script_name.$TODAY"_splitlist"
   touch $split_list

   echo "   Copy  $pathname_table to $TMP_DIR"
   cp -p $pathname_table $TMP_DIR

   exec < $TMP_DIR/$TABLE_NAME
   while read first_field rest_fields
   do 
     cnt=`expr $cnt + 1`  
     table_entry="$first_field $rest_fields" 
    
     year=`echo $first_field | awk -F"." '{print $2}'`

     if [ "$cnt" = "1" ]
     then
       year_old=$year
       splitfile=$LISTS_DIR/$TABLE_NAME.$year
       echo "   splitfile: $splitfile"
       echo $splitfile > $split_list
     fi

     if [ "$year" -eq "$year_old" ]
     then
       echo $table_entry >> $splitfile
     else
       split=`expr $split + 1`  
       splitfile=$LISTS_DIR/$TABLE_NAME.$year
       echo "   splitfile: $splitfile"
       echo $splitfile >> $split_list
       echo $table_entry       >> $splitfile
     fi
     year_old=$year
  done    

  if [ $split -eq 0 ]
  then
    rm -f $splitfile
    text="No split file for $pathname_table\n"
    printf "   $text"
    echo "$text" >> $email_msg
  else
    echo "   Move: $splitfile"
    echo "   To: $LISTS_DIR/$TABLE_NAME"
    mv $splitfile $LISTS_DIR/$TABLE_NAME
    text="Split files for $pathname_table are:\n"
    printf "   $text"
    printf "$text"   >> $email_msg 
    cat $split_list | awk '{print "   " $0}' >> $email_msg
  fi 

  rm -f $TMP_DIR/$TABLE_NAME $split_list
  echo "=> Exit"
  return
}
# ----------------------------    DEBUG   ----------------------------------
#?EMAIL_OPERATOR=Reyna.Sabina@noaa.gov
#---------------------------      Main      --------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`
now=`date +'%h %d, %Y %T %p'`
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

GTS_DIR=$ARGO_DIR/gts
TMP_DIR=$GTS_DIR/tmp
LOG_DIR=$GTS_DIR/log
TMP_DIR=$GTS_DIR/tmp

TABLE_NAME=FTPGTS4EMAIL_TABLE

logfile=$LOG_DIR/$root_script_name.$TODAY"_"log
email_msg=$TMP_DIR/$root_script_name.$TODAY"_"email
error=$LOG_DIR/$root_script_name.$TODAY"_"error

exec 2>> $error
exec 1>> $logfile

echo "<=============== Start  $script_name: $now ==============>"
echo "=> Environmental variables:"
echo "   GTS_DIR: $GTS_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   TMP_DIR: $TMP_DIR"
echo "   TABLE_NAME: $TABLE_NAME"

# -----   Split  ARGO_DIR/lists/FTPGTS4EMAIL_TABLE table -----
LISTS_DIR=$ARGO_DIR/lists
pathname_table=$LISTS_DIR/$TABLE_NAME
echo "=> List Directory and Pathname for Table to be Split:"
echo "   LISTS_DIR: $LISTS_DIR"
echo "   pathname_table: $pathname_table"

table_notfound=0
if [ -f $pathname_table ]
then
  Split_FTPGTS_Table
else
  table_notfound=1
  subj="$up_script_name -  WARNING, Table not found"
  text="$pathname_table - DOES NOT EXIST\n" 
  printf "   $text" 
  printf  "$text" > $email_msg
fi

# -----   Split  ARGO_DIR/manualprocess/lists/FTPGTS4EMAIL_TABLE table -----
LISTS_DIR=$ARGO_DIR/manualprocesses/lists
pathname_table=$LISTS_DIR/$TABLE_NAME
echo "=> List Directory and Pathname for Table to be Split:"
echo "   LISTS_DIR: $LISTS_DIR"
echo "   pathname_table: $pathname_table"

if [ $OPERATIONAL_SITE = "AOML" ]
then
  if [ -f $pathname_table ]
  then
    Split_FTPGTS_Table
  else
    table_notfound=1
    subj="$up_script_name -  WARNING, Table not found"
    text="$pathname_table - DOES NOT EXIST\n" 
    printf "   $text" 
    subj="$up_script_name -  WARNING, Table not found"
    printf "$text" >> $email_msg
  fi
fi

if [ -s "$error"  ]
then
  if [ "$subj" = "" ]
  then
    subj="$up_script_name -  FAILURE, Standard Error"
  else
    subj=$subj" & Standard Error"
  fi
  text="Standard-Error reported, see logfiles:\n"
  text=$text"\n\t$logfile\n\t$error\n"
  printf "$text" >> $email_msg
else
  if [ $table_notfound -eq 0 ]
  then
    subj="$up_script_name -  OK"
  fi
  rm -f $error
fi

echo "   [Email Begin]"      
echo "   Subj: $subj"
echo "   To: $EMAIL_OPERATOR"
echo "   Text:"      
cat $email_msg | awk '{print "         " $0}'
echo "   [Email End]"

printf "   Sending Email ... "
mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
echo "OK"

rm -f $email_msg

now=`date +'%Y.%m.%d_%H.%M'`
echo "<=============== Finish $script_name: $now ==============>"

exit      
