#!/bin/sh
################################################################################################
# Revision Date: 20-Oct-10 (RS)
#
# split_ftpgts_bufrtrack_table.sh
# ===============================
# Splits the table 'GTS_BUFR_TRACK_TABLE' in ARGO_DIR/lists/bufr (LISTS_DIR) into file(s) 
# according to the year the that the packed buffer file(s) was sent to the NWS Gateway for 
# submission to GTS. For each split file, the year (yyyy) is appended to the table name 
# (e.g. GTS_BUFR_TRACK_TABLE.yyyy). The current year is renamed to GTS_BUFR_TRACK_TABLE.
# Split file(s) are located in 'LISTS_DIR'.
#
# Tasks
# -----
# (1) Copy GTS_BUFR_TRACK_TABL_TABLEE to a temporary working directory (ARGO_DIR/gts/tmp) 
# (2) Split table GTS_BUFR_TRACK_TABLE according to each year
################################################################################################
. $HOME/.argorc_sh 
# ---------------------------------------------------------------------------------------
# Split_GTS_BufrTrackTable
# ------------------------ 
# Function will split the GTS_BUFR_TRACK_TABLE table several files according to year
# The most recent data file will be renamed to GTS_BUFR_TRACK_TABLE.temp.  After the split 
# is done it will be renamed to GTS_BUFR_TRACK_TABLE
# ---------------------------------------------------------------------------------------
Split_FTPGTS_BufrTrackTable()
{
   echo "=> Enter Split_FTPGTS_BufrTrackTable"

   cnt=0
   split=0
   split_list=$LOG_DIR/$root_script_name.$TODAY"_splitlist"
   touch $split_list

   echo "   Copy  $pathname_table to $TMP_DIR"
   cp -p $pathname_table $TMP_DIR

   exec < $TMP_DIR/$TABLE_NAME
   while read record
   do
     cnt=`expr $cnt + 1`  
     year=`echo $record | awk '{print $8}' | awk -F"-" '{print $1}'`

     if [ $cnt -eq 1 ]
     then
       year_old=$year
       splitfile=$LISTS_DIR/$TABLE_NAME.$year
       echo "   splitfile: $splitfile"
       echo $splitfile > $split_list
     fi

     if [ "$year" -eq "$year_old" ]
     then
       echo "$record" >> $splitfile
     else
       split=`expr $split + 1`  
       splitfile=$LISTS_DIR/$TABLE_NAME.$year
       echo "   splitfile: $splitfile"
       echo $splitfile >> $split_list
       echo "$record" >> $splitfile
     fi
     year_old=$year
  done    

  if [ $split -eq 0 ]
  then
    rm -f $splitfile
    text="No split file for $pathname_table\n"
    printf "   $text"
    echo "$text" >> $email_msg
  else
    echo "   Move: $splitfile"
    echo "   To: $LISTS_DIR/$TABLE_NAME"
    mv $splitfile $LISTS_DIR/$TABLE_NAME
    text="Split files for $pathname_table:\n"
    printf "$text"   >> $email_msg 
    sort -u $split_list -o $split_list
    awk '{print "   " $0}' $split_list >> $email_msg
    pathname_current_year=`tail -1 $split_list`
    filename=`basename $pathname_current_year`
    echo "\n   $filename - Rename to GTS_BUFR_TRACK_TABLE" >> $email_msg
  fi 

  rm -f $TMP_DIR/$TABLE_NAME $split_list
  echo "=> Exit Split_FTPGTS_BufrTrackTable"
  return
}
# ----------------------------    DEBUG   ----------------------------------
#?EMAIL_OPERATOR=Reyna.Sabina@noaa.gov
#---------------------------      Main      --------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`
now=`date +'%h %d, %Y %T %p'`
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

GTS_DIR=$ARGO_DIR/gts
LISTS_DIR=$GTS_DIR/lists/bufr
LOG_DIR=$GTS_DIR/log
TMP_DIR=$GTS_DIR/tmp

TABLE_NAME=GTS_BUFR_TRACK_TABLE

logfile=$LOG_DIR/$root_script_name.$TODAY"_"log
email_msg=$TMP_DIR/$root_script_name.$TODAY"_"email
error=$LOG_DIR/$root_script_name.$TODAY"_"error

exec 1>> $logfile
exec 2>> $error

echo "<=============== Start  $script_name: $now ==============>"
echo "=> Environmental variables:"
echo "   GTS_DIR: $GTS_DIR"
echo "   LISTS_DIR: $LISTS_DIR"
echo "   LOG_DIR: $LOG_DIR"
echo "   TMP_DIR: $TMP_DIR"
echo " "
echo "   EMAIL_OPERATOR: $EMAIL_OPERATOR"
echo "   TABLE_NAME: $TABLE_NAME"

# -----   Split  ARGO_DIR/gts/lists/bufr/GTS_BUFR_TRACKTABLE  -----
pathname_table=$LISTS_DIR/$TABLE_NAME

table_notfound=0
if [ -f $pathname_table ]
then
  Split_FTPGTS_BufrTrackTable
else
  table_notfound=1
  subj="$up_script_name -  WARNING, Table not found"
  text="$pathname_table - DOES NOT EXIST\n" 
  printf "   $text" 
  printf  "$text" > $email_msg
fi

if [ -s "$error"  ]
then
  if [ "$subj" = "" ]
  then
    subj="$up_script_name -  FAILURE, Standard Error"
  else
    subj=$subj" & Standard Error"
  fi
  text="Standard-Error reported, see logfiles:\n"
  text=$text"\n\t$logfile\n\t$error\n"
  printf "$text" >> $email_msg
else
  if [ $table_notfound -eq 0 ]
  then
    subj="$up_script_name -  OK"
  fi
  rm -f $error
fi

echo "   [Email Begin]"      
echo "   Subj: $subj"
echo "   To: $EMAIL_OPERATOR"
echo "   Text:"      
awk '{print "         " $0}' $email_msg
printf "   Sending Email - "
mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
echo "OK"
echo "   [Email End]"

rm -f $email_msg

now=`date +'%Y.%m.%d_%H.%M'`
echo "<=============== Finish $script_name: $now ==============>"

exit      
