/***************************************************************************** 
  Revision date: 11-Sep-2012  CA: Added Code to handle lines in the meta file
                                  that are not terminated properly by a semicolon
                                  in MetaFileInfo
                 07-Aug-2012  CA Added Variables rows and columns for foilMatrix
		 01-Aug-2012  CA: Added ERR messages for coefficient value Validation
  		 21-Jun-2012  VMH: add dphase2(p0-p3) vars & set/gets; add
                              array foilMatrix2 & section to read
                 24-Mar-2010  ER: Add oxygen coefficients
                 10-Feb-2010  ER: Initial version
  
  MetaFileInfo.java
  =================
  Manage the meta file that will be read by the Java decoders 

*****************************************************************************/

import java.io.*;
import java.util.*;
import java.lang.*;

class Calibrate {

  String coefficient;
  String offset; 
 
  Calibrate( String line ) {
   
    int p1 = line.indexOf("=", 40);
    int length = line.length();
    char lastChar= line.charAt(length-1);
    char charBefLastChar = line.charAt(length-2);
    //System.out.println("lastChar= "+ lastChar);
    int p2 = line.indexOf(";", 40);
    
      if (p1<0 || p2<0) {
       System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), null, 
                            "NO COEFFICIENT AND OFFSET VALUE FOUND") );
       System.exit(Constants.ERR_COEFF_VAL_NOTFOUND);
      this.coefficient = "0";
      this.offset = "0";
      return;
    }
    this.coefficient = line.substring(p1+1, p2);
    // Getting the offset
    p1 = line.indexOf("=", p2+1);
    if(lastChar==';') 
    p2 = line.indexOf(';', p2+1);
    else if(lastChar==' ' && charBefLastChar ==';')
    p2 = length-2; 
    else if(lastChar==' ')
    p2 = length-1; 
    else if(lastChar!=' ' || lastChar !=';')
    p2 = length;

    if (p1<0 || p2<0) {
      System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), null, 
                            "NO OFFSET VALUE FOUND") );
      System.exit(Constants.ERR_COEFF_VAL_NOTFOUND);
      this.offset = "0";
      return;
    }
    this.offset = line.substring(p1+1, p2);
    //System.out.println("Offset = " +this.offset);
  }

  void setCoefficient( String coefficient ) {
    this.coefficient = coefficient;
  }

  String getCoefficient() {
    return this.coefficient;
  }

  void setOffset( String offset ) {
    this.offset = offset;
  }

  String getOffset() {
    return this.offset;
  }
 
}


public class MetaFileInfo
{

  private String filePathname;

  private boolean error;
  private String internalIdNumber;
  private String WMOIdNumber;
  private String transmissionIdNumber;
  private String instrumentType;
  private String WMOInstrumentType1770;
  private String WMORecorderType4770;
  private String PI;
  private String tempCoefficient, tempOffset;
  private String presCoefficient, presOffset;
  private String salCoefficient, salOffset;
  private String voltCoefficient, voltOffset;
  private String currentCoefficient, currentOffset;
  private String vacuumCoefficient, vacuumOffset;
  // to calculate oxsol
  private double soc, fOffset, aa, bb, cc, pCorr; 
  // to get dphase knowning bphase
  private double dphaseP0,  dphaseP1,  dphaseP2,  dphaseP3;
  private double dphase2P0, dphase2P1, dphase2P2, dphase2P3; 
  public double[][] foilMatrix;
  public double[][] foilMatrix2;
 // Variables for Oxygen Coeff of TSO8
  public double[] oca;
  public double[] ocb;
  public double[] occ;
  public double[] ocd;
  public double oce,ocdphdv;
  private int ocCounter = 0;
 
  /****************************************************************
   Class constructor.  
   Read the meta file and set the class variables.
   filePathname is the pathname of the meta file.
  *****************************************************************/  
  public MetaFileInfo( String filePathname, int rows, int columns ) {

    FileReader fileReader;
    BufferedReader buffReader;

    try  {
      //System.out.println("I am in  MetaFileInfo");
      fileReader = new FileReader( filePathname );
      buffReader = new BufferedReader( fileReader );

      String line = buffReader.readLine();
      //System.out.println("line ="+line);
      while (line!=null) {
        if ( line.startsWith("internal ID number" ) ) {
          setInternalIdNumber( line.substring(40, line.length() ).trim() );
        }                      
        else if ( line.startsWith("WMO ID number" ) ) {
          setWMOIdNumber( line.substring(40, line.length() ).trim() );
        }                      
        else if ( line.startsWith("transmission ID number" ) ) {
          setTransmissionIdNumber( line.substring(40, line.length() ).trim() );
        }                      
        else if ( line.startsWith("instrument type" ) ) {
          setInstrumentType( line.substring(40, line.length() ).trim() );
        }                      
        else if ( line.startsWith("WMO instrument type (table 1770)" ) ) {
          setWMOInstrumentType1770( line.substring(40, line.length() ).trim() );
        }                      
        else if ( line.startsWith("WMO recorder type (table 4770)" ) ) {
          setWMORecorderType4770( line.substring(40, line.length() ).trim() );
        }                      
        else if ( line.startsWith("PI    " ) ) {
          setPI( line.substring(40, line.length() ).trim() );
        }
        else if ( line.startsWith("calib coef for temperature" ) ) {
          Calibrate cal = new Calibrate( line );
          setTempCoefficient( cal.getCoefficient() );
      	  //System.out.println("line ="+line);
          setTempOffset( cal.getOffset() );
        }
        else if ( line.startsWith("calib coef for salinity" ) ) {
          Calibrate cal = new Calibrate( line );
          setSalCoefficient( cal.getCoefficient() );
           //System.out.println("Salinity Offset = "+cal.getOffset()); 
          setSalOffset( cal.getOffset() );
        }
        else if ( line.startsWith("calib coef for pressure" ) ) {
          Calibrate cal = new Calibrate( line );
      	   //System.out.println("line ="+line);
          setPresCoefficient( cal.getCoefficient() );
          setPresOffset( cal.getOffset() );
        }
        else if ( line.startsWith("calib coef for current" ) ) {
          Calibrate cal = new Calibrate( line );
          //System.out.println("line ="+line);
	  //System.out.println("TEST:calib coef for coefficient :"+ cal.getCoefficient()); 
          setCurrentCoefficient( cal.getCoefficient() );
	  //System.out.println("TEST:calib coef for current :"+ cal.getOffset()); 
          setCurrentOffset( cal.getOffset() );
        }
        else if ( line.startsWith("calib coef for voltage" ) ) {
          Calibrate cal = new Calibrate( line );
          setVoltCoefficient( cal.getCoefficient() );
          setVoltOffset( cal.getOffset() );
        }
        else if ( line.startsWith("calib coef for vacuum" ) ) {
          Calibrate cal = new Calibrate( line );
          setVacuumCoefficient( cal.getCoefficient() );
          setVacuumOffset( cal.getOffset() );
        }
        else if ( line.startsWith("calib coef for oxygen" ) ) {
          setSoc( getParameterValue( line, "SOC" ) );
          setFOffset( getParameterValue( line, "FOFFSET" ) );
          setAA( getParameterValue( line, "AA" ) );
          setBB( getParameterValue( line, "BB" ) );
          setCC( getParameterValue( line, "CC" ) );
          setPCorr( getParameterValue( line, "PCOR" ) );

        }
        else if ( line.startsWith("calib coef for DPhase(2)" ) ) {
          setDphase2P0( getParameterValue( line, "p0" ) );
          setDphase2P1( getParameterValue( line, "p1" ) );
          setDphase2P2( getParameterValue( line, "p2" ) );
          setDphase2P3( getParameterValue( line, "p3" ) );
        }
        else if ( line.startsWith("calib coef for DPhase" ) ) {
          setDphaseP0( getParameterValue( line, "p0" ) );
          setDphaseP1( getParameterValue( line, "p1" ) );
          setDphaseP2( getParameterValue( line, "p2" ) );
          setDphaseP3( getParameterValue( line, "p3" ) );
        }
        else if ( line.startsWith("calib coef for O2 matrix(2)") ) {
          if (foilMatrix2==null) {
               foilMatrix2 = new double[rows][columns];
          }
          line = line.substring( 40, line.length() ).trim();
          StringTokenizer st = new StringTokenizer(line);
          while (st.hasMoreElements()) {
            String token = st.nextElement().toString();
            int idx_1 = Character.getNumericValue( token.charAt(1) );
            int idx_2 = Character.getNumericValue( token.charAt(2) );
            int p1 = token.indexOf( '=' ) + 1;
            int p2 = token.indexOf( ';' ) ;
            foilMatrix2[idx_1][idx_2] = Double.parseDouble( token.substring( p1, p2 ) ) ;
	    
          }
        }
        else if ( line.startsWith("calib coef for O2 matrix") ) {
          if (foilMatrix==null) {
               foilMatrix = new double[rows][columns];
          }
          line = line.substring( 40, line.length() ).trim();
          StringTokenizer st = new StringTokenizer(line);
          while (st.hasMoreElements()) {
            String token = st.nextElement().toString();
            int idx_1 = Character.getNumericValue( token.charAt(1) );
            int idx_2 = Character.getNumericValue( token.charAt(2) );
            int p1 = token.indexOf( '=' ) + 1;
            int p2 = token.indexOf( ';' ) ;
            foilMatrix[idx_1][idx_2] = Double.parseDouble( token.substring( p1, p2 ) ) ;
          }
        }
        else if ( line.startsWith("calib coef for O2") ) {
             if (instrumentType.equals("APEXIR_TSO8")) {
             
            line = line.substring( 40, line.length() ).trim();
            StringTokenizer st = new StringTokenizer(line);
            while (st.hasMoreElements()) {
	    String token = st.nextElement().toString();
            int p1 = token.indexOf( '=' ) + 1;
            int p2 = token.indexOf( ';' ) ;
            if(token.charAt(0)=='a')
	    {
	      if(oca == null)
		{
		 oca =new double[3];
		}
              oca[ocCounter] = Double.parseDouble( token.substring( p1, p2 ) ) ;
	      //System.out.println("TEST:calib coef for O2 A:"+ oca[ocCounter]);

	      ocCounter++;
	      if(ocCounter == 3)
	      {
		ocCounter = 0;
	      }
	    }
	    /* */
            if(token.charAt(0) =='b')
	    {
	      if(ocb == null)
		{
		 ocb =new double[2];
		}
              ocb[ocCounter] = Double.parseDouble( token.substring( p1, p2 ) ) ;
	      //System.out.println("TEST:calib coef for O2 B:"+ ocb[ocCounter]);
	      ocCounter++;
	      if(ocCounter == 2)
	      {
		ocCounter = 0;
	      }
	    }
	    /* */
            if(token.charAt(0)=='c')
	    {
	      if(occ == null)
		{
		 occ =new double[3];
		}
              occ[ocCounter] = Double.parseDouble( token.substring( p1, p2 ) ) ;
	      //System.out.println("TEST:calib coef for O2 C:"+ occ[ocCounter]);
	      ocCounter++;
	      if(ocCounter == 3)
	      {
		ocCounter = 0;
	      }
	    }
	    /* */
            if((token.charAt(0) =='d') && (token.charAt(1)!='P'))
	    {
	      if(ocd == null)
		{
		 ocd =new double[6];
		}
              ocd[ocCounter] = Double.parseDouble( token.substring( p1, p2 ) ) ;
	      //System.out.println("TEST:calib coef for O2 Di :"+ ocd[ocCounter]);
	      ocCounter++;
	      if(ocCounter == 5)
	      {
		ocCounter = 0;
	      }
	    }
	    if((token.charAt(0) =='d') && (token.charAt(1)=='P'))
	    {
              ocdphdv = Double.parseDouble( token.substring( p1, p2 ) ) ;
	      //System.out.println("TEST:calib coef for O2 OCDPHDV:"+ ocdphdv);
	    }
	    /* */
            if(token.charAt(0)=='E')
	    {
              oce = Double.parseDouble( token.substring( p1, p2 ) ) ;
	      //System.out.println("TEST:calib coef for O2 E:"+ oce);
	    }
	    /* */
           }  
	  }
	}
        line = buffReader.readLine();
      }
      setError(false);
    }
    catch ( IOException e ) {
      setError( true );
    }

  }

  /*******************************************************
    Given a line and a parameter name returns the value
    of the parameter.
    Example:
    line='calib coef for pressure    PSLOPE=0.1; POFF=0.0;'
    parameter='PSLOPE'
    returns 0.1
  ********************************************************/
  double getParameterValue( String line, String name ) {
    int p1, p2, p3;
    p1 = line.indexOf( name + "=" );
    p2 = line.indexOf( "=", p1 );
    p3 = line.indexOf( ";", p2+1 );
    if (p3 >= 0) {
      return Double.parseDouble( line.substring( p2+1, p3 ) ) ;
    }
    else {
      return Double.parseDouble( line.substring( p2+1, line.length() ) );
    }
  }

  public void setInternalIdNumber( String internalIdNumber ) {
    this.internalIdNumber = internalIdNumber;
  }
  public String getInternalIdNumber() {
    return this.internalIdNumber;
  }

  public void setWMOIdNumber( String WMOIdNumber ) {
    this.WMOIdNumber = WMOIdNumber;
  }
  public String getWMOIdNumber() {
    return this.WMOIdNumber;
  }

  public void setTransmissionIdNumber( String transmissionIdNumber ) {
    this.transmissionIdNumber = transmissionIdNumber;
  }
  public String getTransmissionIdNumber() {
    return this.transmissionIdNumber;
  }

  public void setInstrumentType( String instrumentType ) {
    this.instrumentType = instrumentType;
  }
  public String getInstrumentType() {
    return this.instrumentType;
  }

  public void setWMOInstrumentType1770( String WMOInstrumentType1770 ) {
    this.WMOInstrumentType1770 = WMOInstrumentType1770;
  }
  public String getWMOInstrumentType1770() {
    return this.WMOInstrumentType1770;
  }

  public void setWMORecorderType4770( String WMORecorderType4770 ) {
    this.WMORecorderType4770 = WMORecorderType4770;
  }
  public String getWMORecorderType4770() {
    return this.WMORecorderType4770;
  } 

  public void setPI( String PI ) {
    this.PI = PI;
  }
  public String getPI() {
    return this.PI;
  }

  public void setError( boolean error ) {
    this.error = error;
  }
  public boolean hasError() {
    return this.error;
  };

  public void setTempCoefficient( String tempCoefficient ) {
    this.tempCoefficient = tempCoefficient;
  }
  public String getTempCoefficient() {
    return this.tempCoefficient;
  }

  public void setTempOffset( String tempOffset ) {
    this.tempOffset = tempOffset;
  }
  public String getTempOffset() {
    return this.tempOffset;
  }

  public void setPresCoefficient( String presCoefficient ) {
    this.presCoefficient = presCoefficient;
  }
  public String getPresCoefficient() {
    return this.presCoefficient;
  }

  public void setPresOffset( String presOffset ) {
    this.presOffset = presOffset;
  }
  public String getPresOffset() {
    return this.presOffset;
  }

  public void setSalCoefficient( String salCoefficient ) {
    this.salCoefficient = salCoefficient;
  }
  public String getSalCoefficient() {
    return this.salCoefficient;
  }

  public void setSalOffset( String salOffset ) {
    this.salOffset = salOffset;
  }
  public String getSalOffset() {
    return this.salOffset;
  }

  public void setCurrentCoefficient( String currentCoefficient ) {
    this.currentCoefficient = currentCoefficient;
  }
  public String getCurrentCoefficient() {
    return this.currentCoefficient;
  }

  public void setCurrentOffset( String currentOffset ) {
    this.currentOffset = currentOffset;
  }
  public String getCurrentOffset() {
    return this.currentOffset;
  }

  public void setVoltCoefficient( String voltCoefficient ) {
    this.voltCoefficient = voltCoefficient;
  }
  public String getVoltCoefficient() {
    return this.voltCoefficient;
  }

  public void setVoltOffset( String voltOffset ) {
    this.voltOffset = voltOffset;
  }
  public String getVoltOffset() {
    return this.voltOffset;
  }

  public void setVacuumCoefficient( String vacuumCoefficient ) {
    this.vacuumCoefficient = vacuumCoefficient;
  }
  public String getVacuumCoefficient() {
    return this.vacuumCoefficient;
  }

  public void setVacuumOffset( String vacuumOffset ) {
    this.vacuumOffset = vacuumOffset;
  }
  public String getVacuumOffset() {
    return this.vacuumOffset;
  }


  // gets and sets for OxSol calculation
  public double getSoc() { return this.soc; }
  public double getFOffset() { return this.fOffset; }
  double getAA() { return this.aa; }
  double getBB() { return this.bb; }
  double getCC() { return this.cc; }
  double getPCorr() {return this.pCorr; }
  public void setFOffset( double fOffset ) { this.fOffset = fOffset; }
  public void setSoc( double soc ) { this.soc = soc; }
  void setAA( double aa ) { this.aa = aa; }
  void setBB( double bb ) { this.bb = bb; }
  void setCC( double cc ) { this.cc = cc; }
  void setPCorr( double pCorr ) { this.pCorr = pCorr; }

  // gets and sets for dPhase calculation
  double getDphaseP0() { return this.dphaseP0; }
  double getDphaseP1() { return this.dphaseP1; }
  double getDphaseP2() { return this.dphaseP2; }
  double getDphaseP3() { return this.dphaseP3; }
  void setDphaseP0( double dphaseP0 ) { this.dphaseP0 = dphaseP0; }
  void setDphaseP1( double dphaseP1 ) { this.dphaseP1 = dphaseP1; }
  void setDphaseP2( double dphaseP2 ) { this.dphaseP2 = dphaseP2; }
  void setDphaseP3( double dphaseP3 ) { this.dphaseP3 = dphaseP3; }

  // gets and sets for dPhase2 calculation
  double getDphase2P0() { return this.dphase2P0; }
  double getDphase2P1() { return this.dphase2P1; }
  double getDphase2P2() { return this.dphase2P2; }
  double getDphase2P3() { return this.dphase2P3; }
  void setDphase2P0( double dphase2P0 ) { this.dphase2P0 = dphase2P0; }
  void setDphase2P1( double dphase2P1 ) { this.dphase2P1 = dphase2P1; }
  void setDphase2P2( double dphase2P2 ) { this.dphase2P2 = dphase2P2; }
  void setDphase2P3( double dphase2P3 ) { this.dphase2P3 = dphase2P3; }


}
