/************************************************************************************************* 
  Revision date: 27-Dec-2013  CA:  All fixes except for those that have nSat < 3 are retrieved now
                                   Earlier if nSat < 4 all the other GPS fixes were if ignored
                 04-Dec-2013  CA:  fillValueOFreq is modified from 99999 to -999
                 26-Jun-2013  CA:  processGPS function is modified to ignore GPS Fix that has less 
				   than 3 satellites
  		 24-Jan-2013  CA:  processGPS function is modified to accomodate missing GPS Fixes
                                  interpolation
		 12-Dec-2012  CA: HashMap varibale is added to accomodate dynamic reading
  		 		  Low Resolution Profile Header 
  	         12-Dec-2012  CA: Added DecimalFormat with 6 decimal places   
  		 06-Dec-2012  CA: Making changes to the function processGPSFix
  				  to accommodate retrieval of all GPS Fixes.
                 11-Sep-2012  CA:"TOTAL NUM PROF LEVELS SAMPLED AT 1 HZ" is added to the phy file
				  using NSample  from the message file
                 11-Sep-2012  CA: Added Abstracted processParkPtLine function from decoders and 
				  placed it in this Super Class 
  		 21-Jun-2012  VMH: Add arrays for prof2Tphase2,prof2TOpt2,prof2O22
                 13-Jun-2012  VMH: Check ParkObs (p,t,s) for chars>16,
                              substitute fill value (char fill values created)
                 21-May-2012  VMH: Globalize some vars & arrays; add fillValue
                              and booleans for ph,no3,optical;add VMH debugs.
                 06-Jan-2012  Values of latitude and longitude can have any
                              number of decimal points.
                 29-Nov-2011  Values of drift descent pressure converted to dbar
                 29-Aug-2011  Added treatment of case in which technical 
                              parameter lines comes as tech_parm_name= 
                 24-Mar-2011  Create new incomplete check when processing high
                              resolution lines.
                 04-Jan-2011  Create new constants for error & warning messages
                              Account for possibility of other strings where a
                              GPS fix is expected.
                 04-Mar-2010  Initial release 
  
  Msg2Phy_Ir.java
  ===================
  Superclass for decoding a msg file from an Iridium float and creating a 
  correspondent new phy file.
**************************************************************************************************/
import java.util.*;
import java.io.*;
import java.text.*;

public class Msg2Phy_Ir {

   // Constants for exit codes
   static public final int OK = 0;
   static public final int ERR_OPEN_MSG_FILE  = 111;
   static public final int ERR_OPEN_META_FILE = 115;
   static public final int ERR_OPEN_TECH_PARAM_FILE = 117;
   static public final int ERR_OPEN_CONFIG_PARAM_FILE = 118;
   static public final int ERR_WRONG_NBR_PARAM = 131;
   static public final int ERR_CREATE_LOG_FILE = 132;
   static public final int ERR_CREATE_PHY_FILE = 133;
   static public final int ERR_WRONG_META_INFO = 134;
   static public final int WRN_MSG_FILE_INCOMPLETE = 135;
   static public final int ERR_OPEN_ERROR_FILE = 136;
   static public final int WRN_CONF_PARAM_NOTFOUND = 137;
   static public final int ERR_COEFF_NOTFOUND = 138;
   static public final int ERR_COEFF_VAL_NOTFOUND = 139;
   static public final int ERR_IO = 998;
   static public final int ERR_EXECUTION = 999;

   String msgPathname;  // Message file pathname (input)
   String pathName;  // pathname to InterpolatedGpsFixes.java 
   String technicalFilePathname;   // Technical Parameters file pathname (input) 
   String confTableFilePathname; // Mission configuration parameters file pathname (input)
   String phyPathname;           // Phy file pathname (output) 
   String metaPathname;      // Meta file pathname (output) 
   String logJavaFilePathname;   // Java log file pathname (input)
   String errorFilePathname;     // Error file pathname (input)
   int profileNumber;      
   String phyFilename;
   static boolean debugOn;              // Is debug on
   static int debugLev;                 // What level of debug do you want
   int rows;				// Number of Rows for foil matrix co-efficients
   int columns;				// Number of Columns for foil matrx co-efficients
   //int NSamples = 0;		        // NSamples from msg file
   String floatId;

   Logger logger;                    // Used to write to the log file.
   Logger errorLogger;               // Used to write to the error file.
   FileReader inpReader = null;      // Used to read msg file
   BufferedReader buffReader = null; // Used to read msg file
   FileWriter outWriter = null;      // Used to write to the phy file
   BufferedWriter buffWriter = null; // Used to write to the phy file

   MetaFileInfo meta;

   TechnicalTable techTable; // To hold the x-ref technical parameter table
                             // (msg tech parm => phy tech parm)
   // var used for the technical parameter lines
   static final int MAX_NBR_TECH_LINES = 100;
   String[] techParamLines = new String[MAX_NBR_TECH_LINES];    
   int nbrTechParamLines;

   MissionConfigTable confTable; // To hold the x-ref mission conf parameter table
                                 // (msg config parm => phy config parm)
   // var used for the config parameter lines
   static final int MAX_NBR_CONFIG_LINES = 100;
   String[] confParamLines = new String[MAX_NBR_CONFIG_LINES];    
   int nbrConfParamLines;


   // var used for the (Park) Drift phase
   StringBuffer driftPressureLine = new StringBuffer();
   StringBuffer driftTemperatureLine = new StringBuffer();
   StringBuffer driftSalinityLine = new StringBuffer();
   StringBuffer driftMeasurementTimeLine = new StringBuffer();
   static final int MAX_NBR_DRIFT_LINES = 500;
   String[] driftPressureLines = new String[MAX_NBR_DRIFT_LINES];
   String[] driftTemperatureLines = new String[MAX_NBR_DRIFT_LINES];
   String[] driftSalinityLines = new String[MAX_NBR_DRIFT_LINES];
   String[] driftMeasurementTimeLines = new String[MAX_NBR_DRIFT_LINES];
   int nbrDriftLines;

   // var used for the general profile data lines as well as the hig res.
   static final int MAX_NBR_HIGH_RESOLUTION_LINES = 3000;
   String[] profDataLines = new String[MAX_NBR_HIGH_RESOLUTION_LINES];    
   int nbrProfDataLines;
   int nbrHighResolutionLines;
   String[] highResolutionLines = new String[ MAX_NBR_HIGH_RESOLUTION_LINES];
   int nbrProfResolutionLines;
   double[] highResolutionPres = new double[ MAX_NBR_HIGH_RESOLUTION_LINES];

   // var used for the low resolution lines
   static final int MAX_NBR_LOW_RESOL_LINES = 1000;
   int nbrLowResolutionLines;
   String[] lowResolutionLines = new String[ MAX_NBR_LOW_RESOL_LINES ];
   double[] lowResolutionPres = new double[ MAX_NBR_LOW_RESOL_LINES ];

   // Two lines with date info before the profile data block.
   String beforeProfLine1;
  // String beforeProfLine2;
   String[] beforeProfLine2 = new String[50]; //Initializing to 10 , need to Check with Claudia
   String beforeProfLine3;

   int gpsFixCounter = 0; //Counter to keep track of all the GPS Fixes
 
   // var used for profiles 1, 2 
   static final int MAX_NBR_PROFILE_1_LINES = 3000;
   double[] prof1Pres  = new double[MAX_NBR_PROFILE_1_LINES];
   double[] prof1Temp  = new double[MAX_NBR_PROFILE_1_LINES];
   double[] prof1Psal  = new double[MAX_NBR_PROFILE_1_LINES];
   int[]    prof1Ofreq = new int[MAX_NBR_PROFILE_1_LINES];
   double[] prof1O2    = new double[MAX_NBR_PROFILE_1_LINES];
   String NSamples;
   int nbrProfile1Lines;
   int printTSFlag = 0;

   static final int MAX_NBR_PROFILE_2_LINES = 1000;
   double[] prof2Pres = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2Temp = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2Psal = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2Bphase = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2Tphase = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2Rphase = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2TOpt = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2O2 = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2Tphase2 = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2TOpt2 = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2O22 = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2NO3 = new double[MAX_NBR_PROFILE_2_LINES];
   double[] prof2PH = new double[MAX_NBR_PROFILE_2_LINES];
   int nbrProfile2Lines;
   
   double[] prof3Pres  = new double[MAX_NBR_PROFILE_1_LINES];
   double[] prof3Temp  = new double[MAX_NBR_PROFILE_1_LINES];
   double[] prof3Psal  = new double[MAX_NBR_PROFILE_1_LINES];
   int nbrProfile3Lines;


   // fill value constants
   static final double fillValuePress = -999.0;
   static final String fillValuePress_char = "-999.0";
   static final double fillValueTemp  = -999.0;
   static final String fillValueTemp_char  = "-999.0";
   static final double fillValuePSal  = -999.0;
   static final String fillValuePSal_char  = "-999.0";
   static final int    fillValueOFreq = -999;
   static final double fillValueBPhase = -999.0;
   static final double fillValueRPhase = -999.0;
   static final double fillValueO2    = -999.0;
   static final double fillValueNO3   = -999.0;
   static final double fillValuePH    = -999.0;

   // for use in formatting decimal numbers
   DecimalFormat dec1Format = new DecimalFormat("0.0");
   DecimalFormat dec2Format = new DecimalFormat("0.00");
   DecimalFormat dec3Format = new DecimalFormat("0.000");
   DecimalFormat dec4Format = new DecimalFormat("0.0000");
   DecimalFormat dec6Format = new DecimalFormat("0.000000");

   // vars used for no3 and ph
   static boolean hasNitrate; // does the low resolution section has a nitrate column?
   static boolean hasPH;      // does the low resolution section have a ph column?
   static boolean hasOptical; // are the optical variables present? (not currentlykept)
   static boolean has; // are the optical variables present? (not currentlykept)
   static double no3, ph,rphase,p,t,s,tphase,topt;
   
   //HashMap varibale for reading Low Resolution Profile Header Data
   //HashMap<String,Integer> hm = new HashMap<String,Integer>();
   HashMap<Integer,String> hm = new HashMap<Integer,String>();
   boolean hasGPSFix = true;
   InterpolateGpsFixes iGpsFixes = null;


   /****************************************************************
     Class constructor.  
     Set the file path name variables and the debug variable.
   *****************************************************************/  
   public Msg2Phy_Ir( String msgPathname, String phyPathname,
                      String metaPathname, 
                      String technicalFilePathname, String confTableFilePathname,
                      String logJavaFilePathname, String errorFilePathname, 
                      String debug,int rows,int columns)
   {
     this.msgPathname = msgPathname;
     //System.out.println("this.msgPathname = "+this.msgPathname);
     int p = this.msgPathname.indexOf("iridium");
     this.pathName = this.msgPathname.substring(0,p+8);
     //System.out.println("this.pathName = "+this.pathName);
     this.phyPathname = phyPathname;
     p=this.phyPathname.lastIndexOf('/');
     this.phyFilename=this.phyPathname.substring(p+1,this.phyPathname.length());
     /*System.out.println("this.phyPathname= "+this.phyPathname);
     System.out.println("this.phyFilename= "+this.phyFilename);*/
     this.metaPathname = metaPathname;
     this.technicalFilePathname = technicalFilePathname;
     this.confTableFilePathname = confTableFilePathname;
     this.logJavaFilePathname = logJavaFilePathname;
     this.errorFilePathname = errorFilePathname;
     this.debugLev = Integer.parseInt(debug);
     this.debugOn = this.debugLev>0;
     this.rows = rows;
     this.columns = columns;
     p=this.msgPathname.lastIndexOf('/');
     this.floatId = msgPathname.substring(p+1,this.msgPathname.indexOf('.'));

    // System.out.println("Msg2Phy_Ir debug status: " + this.debugOn );
     if(debugOn) System.out.println("M2PM debug  level: " + this.debugLev );
     /* currently, debug levels > 1 will:
      * 2 = includ edf comparison lines n the log file
     */

     // Open log file
     try {
       logger = new Logger( logJavaFilePathname );
       logger.write( this.getClass().getName(), null, "Start processing " + msgPathname );
     }
     catch ( IOException e )
     {
        System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, 
                            "IO Error while opening Java log file: " + logJavaFilePathname ) );
        System.exit(ERR_CREATE_LOG_FILE);
     }

     // Open Error Log file
     try {
       errorLogger = new Logger( errorFilePathname );
     }
     catch ( IOException e )
     {
        System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, 
                            "IO Error while opening error file: " + errorFilePathname ) );
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening error file: " + errorFilePathname );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(ERR_OPEN_ERROR_FILE) );
          logger.close();
        }
        catch (IOException e1) {}
        finally {
          System.exit(ERR_OPEN_ERROR_FILE);
        }
     }

     // Load metafile vars into object meta
     this.meta = new MetaFileInfo( metaPathname,rows,columns );
     if ( this.meta.hasError() ) {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening meta file " + metaPathname  );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(ERR_OPEN_META_FILE) );
          logger.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, "Error opening meta file " + metaPathname  );
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(ERR_OPEN_META_FILE);
     }

     // load tech param x-ref table into object techTable
     this.techTable = new TechnicalTable( this.technicalFilePathname );
     if ( this.techTable.hasError() ) {
       try {
         logger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening technical file: " + this.technicalFilePathname );
         logger.write( this.getClass().getName(), null, 
                       "End processing " + msgPathname + " with code=" + Integer.toString(ERR_OPEN_TECH_PARAM_FILE) );
         logger.close();
       }
       catch (IOException e1) {}
       try {
         errorLogger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening technical file: " + this.technicalFilePathname );
         errorLogger.close();
       }
       catch (IOException e1) {}
       System.exit(ERR_OPEN_TECH_PARAM_FILE);
     }


     // load mission configuration param x-ref table into object confTable
     this.confTable = new MissionConfigTable( this.confTableFilePathname );
     if ( this.confTable.hasError() ) {
       try {
         logger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening mission config file: " + this.confTableFilePathname );
         logger.write( this.getClass().getName(), null, 
                       "End processing " + msgPathname + " with code=" + Integer.toString(ERR_OPEN_CONFIG_PARAM_FILE) );
         logger.close();
       }
       catch (IOException e1) {}
       try {
         errorLogger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening mission config file: " + this.confTableFilePathname );
         errorLogger.close();
       }
       catch (IOException e1) {}
       System.exit(ERR_OPEN_CONFIG_PARAM_FILE);
     }

     if (debugOn) {
       for (int i=0; i<=27; i++) 
         System.out.println( "M2PM D CP i: " + i + " confTable: " + this.confTable.getMsgConfParamName(i)  + "   " + 
                            this.confTable.getPhyConfParamName(i));
     } 
     
     // open input file
     if(debugOn) System.out.println( "M2PM D OP open inp: " + msgPathname);
     try {
       inpReader = new FileReader( msgPathname );
       buffReader = new BufferedReader( inpReader );
     }
     catch ( IOException e )
     {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening msg file: " + msgPathname );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                   " with code=" + Integer.toString(ERR_OPEN_MSG_FILE) );
          logger.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, 
                             "Error opening msg file: " + msgPathname );
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(ERR_OPEN_MSG_FILE);
     }

     // open phy file
     if(debugOn) System.out.println( "M2PM D OP open out: " + phyPathname);
     try {
       outWriter = new FileWriter( phyPathname );
       buffWriter = new BufferedWriter( outWriter );
       try {
         logger.write( this.getClass().getName(), null, "File " + phyPathname + " created. " );
       }
       catch (IOException e1) {}
     }
     catch ( IOException e )
     {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error creating phy file: " + this.phyPathname );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(ERR_CREATE_PHY_FILE) );
          logger.close();
          buffReader.close();
          inpReader.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, 
                             "Error creating phy file: " + this.phyPathname  );
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(ERR_CREATE_PHY_FILE);
     }



   }

   /**********************************************************************
     Get all the meta file info needed for the msg to phy process.
   ***********************************************************************/  
   void processMetaFileInfo( BufferedWriter buffWriter ) 
                           throws IOException {
     if(debugOn) System.out.println( "M2PM D PM processMetaFileInfo writes out meta info");
     buffWriter.write( "INTERNAL ID NUMBER                      " + meta.getInternalIdNumber() );
     buffWriter.newLine();
     buffWriter.write( "WMO ID NUMBER                           " + meta.getWMOIdNumber() );
     buffWriter.newLine();
     buffWriter.write( "TRANSMISSION ID NUMBER                  " + meta.getTransmissionIdNumber() );
     buffWriter.newLine();
     // ...profile number
     int p1 = msgPathname.indexOf('.')+1;
     int p2 = msgPathname.lastIndexOf('.');
     this.profileNumber= Integer.parseInt(msgPathname.substring(p1, p2));
     buffWriter.write( "PROFILE NUMBER                          " + this.profileNumber );
     buffWriter.newLine();
     buffWriter.write( "INSTRUMENT TYPE                         " + meta.getInstrumentType() );
     buffWriter.newLine();
     buffWriter.write( "WMO INSTRUMENT TYPE (TABLE 1770)        " + meta.getWMOInstrumentType1770() );
     buffWriter.newLine();
     buffWriter.write( "WMO RECORDER TYPE (TABLE 4770)          " + meta.getWMORecorderType4770() );
     buffWriter.newLine();
     buffWriter.write( "POSITIONING SYSTEM                      GPS" );
     buffWriter.newLine();
     buffWriter.write( "PI                                      " + meta.getPI() );
     buffWriter.newLine();
     return;
   }


   /**********************************************************************
     Converts a signed hex number to a float (real) number. 
   ***********************************************************************/  
   float signedHexToFloat( String hex ) {
     if (hex.charAt(0) >= '0' && hex.charAt(0) <= '7')  {
       return Float.parseFloat( Integer.toString( Integer.parseInt( hex, 16 ) ) );
       // return Float.valueOf( Integer.toString( Integer.parseInt( hex, 16 ) ) );
     }
     else     
     {
       //
       // Negative number.  Let's reverse complement 2
       // 
       // 
       StringBuffer hex2 = new StringBuffer( "" ); 
       // Rest 1 to the last byte.  This might set a chain reaction.
       boolean rest;
       rest = true;
       for (int i=hex.length()-1; i>=0; i--) {
         if (rest) {
           switch (hex.charAt(i)) {
             case '0': hex2.insert( 0,  'F' ); break;
             case '1': hex2.insert( 0,  '0' ); rest = false; break;
             case '2': hex2.insert( 0,  '1' ); rest = false; break;
             case '3': hex2.insert( 0,  '2' ); rest = false; break;
             case '4': hex2.insert( 0,  '3' ); rest = false; break;
             case '5': hex2.insert( 0,  '4' ); rest = false; break;
             case '6': hex2.insert( 0,  '5' ); rest = false; break;
             case '7': hex2.insert( 0,  '6' ); rest = false; break;
             case '8': hex2.insert( 0,  '7' ); rest = false; break;
             case '9': hex2.insert( 0,  '8' ); rest = false; break;
             case 'A': hex2.insert( 0,  '9' ); rest = false; break;
             case 'B': hex2.insert( 0,  'A' ); rest = false; break;
             case 'C': hex2.insert( 0,  'B' ); rest = false; break;
             case 'D': hex2.insert( 0,  'C' ); rest = false; break;
             case 'E': hex2.insert( 0,  'D' ); rest = false; break;
             case 'F': hex2.insert( 0,  'E' ); rest = false; break;
           }
         }
         else {
           hex2.insert( 0,  hex.charAt(i) );
         }
         
       }
     
       // After resting 1 to the number, each byte will be "flipped"
       StringBuffer hex3 = new StringBuffer( "" );
       for (int i=0; i<hex2.toString().length(); i++) {
         switch (hex2.toString().charAt(i)) {
           case '0': hex3.append( 'F' ); break;
           case '1': hex3.append( 'E' ); break;
           case '2': hex3.append( 'D' ); break;
           case '3': hex3.append( 'C' ); break;
           case '4': hex3.append( 'B' ); break;
           case '5': hex3.append( 'A' ); break;
           case '6': hex3.append( '9' ); break;
           case '7': hex3.append( '8' ); break;
           case '8': hex3.append( '7' ); break;
           case '9': hex3.append( '6' ); break;
           case 'A': hex3.append( '5' ); break;
           case 'B': hex3.append( '4' ); break;
           case 'C': hex3.append( '3' ); break;
           case 'D': hex3.append( '2' ); break;
           case 'E': hex3.append( '1' ); break;
           case 'F': hex3.append( '0' ); break;
         }
       }
       return -Float.parseFloat( Integer.toString( Integer.parseInt( hex3.toString(), 16 ) ) );
       // return -Float.valueOf( Integer.toString( Integer.parseInt( hex3.toString(), 16 ) ) );
     }
   }


   /**********************************************************************
     Given a line of the msg that holds a parameter name and value, 
     return its value.
     Example:
       Given this line: QuiescentVolts=196
       Returns: 196
   ***********************************************************************/  
   String getParameterValue( String line ) {
     int pos1 = line.indexOf("=") + 1;
     int pos2 = line.length();
     if ( (pos1 == -1 ) || (pos2 < pos1) ) {
       return "";
     }
     else {
       return line.substring( pos1, pos2 ); 
     } 
   }


   /*  
      Process a line containing a mission configuration
      parameter.  For example:
      $ AscentTimeOut(479) [min]
    
      return false only when a parameter is not found in 
      the x-ref mission conf parameter table
    */
   boolean processMissionConfiguration( String line, Logger logger ) {

     String name;
     String value;
     int p1 = line.indexOf( "(" );
     int p2 = line.indexOf( ")" );
     if (p1<1 || p2<=p1) {
       return true;
     }


     // Check for 'TimeOfDay' exception
     // (1 parameter in msg file may become 2 in phy)
     if (line.substring(2, p1).equals( "TimeOfDay" )) {  
       name="CONFIG CLOCK PRESET START CYCLE FLAG";
       value = line.substring(p1+1, p2);
       if (value.equals("DISABLE")) {
         this.nbrConfParamLines++;
         confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + "0";       
       }         
       else {
         this.nbrConfParamLines++;
         confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + "1";       
         name="CONFIG CLOCK START CYCLE (MINUTES)";
         this.nbrConfParamLines++;
         confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + value;       
       }
     }
     else {
       int index = this.confTable.indexOf( line.substring(2, p1 ) );
       if (index<0) {
         //  configuration parameter not found in table.
         return false;
       }
       value = line.substring(p1+1, p2);
       name = this.confTable.getPhyConfParamName( index );
       if (debugOn) System.out.println( "M2PM D WR My param msg: " + line.substring(2, p1 ) + " phy: " + name + " value: " + value );
       this.nbrConfParamLines++;
       confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + value;       
     }
     return true;     
   }



   /*
      process msg file segments like this one:
      ParkDescentPCnt=5
      ParkDescentP[0]=5
      ParkDescentP[1]=37
      ParkDescentP[2]=85
      ParkDescentP[3]=86
      ParkDescentP[4]=87

      returns true if the number of drift descent values declared in first line
      (in the example 5) is equal to the valid number of values found in rest
      of the segment.
   */
   boolean processParkDescent( String line, BufferedReader buffReader )
                throws IOException {

     String value = getParameterValue( line );
     if (value.equals("") ) {
       return false;
     }

     int n = Integer.parseInt( getParameterValue( line ) );
     insertOrUpdateTechParameter( "DRIFT DESCENT PCNT                      ", Integer.toString( n ) );
     
     // if there aren't any values next line is not added.
     if ( n==0 ) {
       return true;
     }
     
     StringBuffer pDescentValues = new StringBuffer( "");
     for (int i=1; i<=n; i++ ) {
       try {
         line = buffReader.readLine();
         if (line.startsWith("ParkDescentP[")) {
           int test_is_int = Integer.parseInt( getParameterValue( line ) );
           pDescentValues.append( getParameterValue( line )+"0" );
           pDescentValues.append( " " );
         }
       }
       catch (Exception e) {}
     } 
     insertOrUpdateTechParameter( "DRIFT DESCENT P                         ", pDescentValues.toString() );

     // checking the number of values is equal to the number of values expected.      
     StringTokenizer tok = new StringTokenizer(pDescentValues.toString()); 
     int actual_count = tok.countTokens(); 
     return (actual_count == n);

   }


   /********************************************************************************
     Process a line that precedes the low resolution (Drift) section on the msg file. 
     Examples:
     $ Profile terminated: Sat Sep 17 00:01:20 2005
     $ Profile 6102.054 terminated: Wed Feb 24 08:18:46 2010
   *********************************************************************************/  
   void processBeforeLowResolution( String line )
                throws java.text.ParseException {
     
     int pos = line.indexOf(':');
     SimpleTimeZone tz = new SimpleTimeZone(0, "Out Timezone");             
     TimeZone.setDefault(tz);   
     if (pos>0) {
       pos = pos + 6; 
       if (pos<(line.length()-19) ) {
         SimpleDateFormat formatter = new SimpleDateFormat("MMM dd HH:mm:ss yyyy") ;
         Date date = (Date)formatter.parse(line.substring( pos, pos+20 )); 
         formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss") ;
         this.beforeProfLine1 = "    +99.999   +999.999 " + 
                                formatter.format(date) +
                                "   000   0     0";
       }
     }
   }


   /**********************************************************************
     Process a Low Resolution (Park Sample) line read from the msg file. 
   ***********************************************************************/  
   void processLowResolution( String line, boolean printNumberValuesAveraged ) {
   
     if (line.indexOf("(Park Sample)") >= 0 ) {
       // String "(Park Sample)" found in line; do nothing.
       return;
     }

     StringTokenizer tk = new StringTokenizer(line);
     if (tk.countTokens()<3) {
        // not enough data in line
       return;
     }
     float press = Float.parseFloat( tk.nextElement().toString() );
     float temp  = Float.parseFloat( tk.nextElement().toString() );
     float psal  = Float.parseFloat( tk.nextElement().toString() );

     DecimalFormat presFormat = new DecimalFormat("0.00");
     DecimalFormat tempFormat = new DecimalFormat("0.0000");
     DecimalFormat psalFormat = new DecimalFormat("####0.0000");

     nbrLowResolutionLines++;
     if (printNumberValuesAveraged) {
       lowResolutionLines[ nbrLowResolutionLines ] = Utils2.padLeft(presFormat.format(press), 11) + "  " +
                                                   Utils2.padLeft(tempFormat.format(temp), 11) + " " +
                                                   Utils2.padLeft(psalFormat.format(psal), 11) + 
                                                   "   0" +
                                                   " 0" ; // checksum 
     }
     else {
       lowResolutionLines[ nbrLowResolutionLines ] = Utils2.padLeft(presFormat.format(press), 11) + "  " +
                                                     Utils2.padLeft(tempFormat.format(temp), 11) + " " +
                                                     Utils2.padLeft(psalFormat.format(psal), 11) + 
                                                     " 0" ; // checksum 
     }
   }


   /********************************************************************************
     Process a line that follows the low resolution (Drift) section on the msg file. 
   *********************************************************************************/  
   void processAfterLowResolution( String line ) 
                throws java.text.ParseException {
     if (line.startsWith( "# Attempt to get GPS fix failed" )) {
       return;
     }
     if (line.startsWith( "# Ice-cap evasion" )) {
       return;
     }
     if (line.startsWith( "# Leads or break-up" )) {
       return;
     }
     try {
       SimpleDateFormat formatter = new SimpleDateFormat("MMM dd yyyy HH:mm:ss") ;
       Date date = (Date)formatter.parse(line.substring( 2, 22 )); 
       formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss") ;
       this.beforeProfLine2[0] = "    +99.999   +999.999 " + 
                              formatter.format(date) +
                              "   000   0     0";
       
     //System.out.println(" this.beforeProfLine2 = "+this.beforeProfLine2[0]);
     String NSampleLine = line.substring(42);
     int openBracketPosition = NSampleLine.indexOf('[');
     int closeBracketPosition = 0;
     if (openBracketPosition > 0) 
       closeBracketPosition = NSampleLine.indexOf(']');
       NSamples = (NSampleLine.substring(openBracketPosition+1,closeBracketPosition));
       //System.out.println("NSamples = "+NSamples);
    

          
     }
     catch (Exception e) {
     }
   }


   /**********************************************************************
     Take a line from the msg file and based on its content determines
     whether it is a HighResolution one. 
   ***********************************************************************/  
   boolean isHighResolution(String line) {
     if (line.length()<14) {
       return false;
     }
     for (int i=0; i<14; i++ ) {
       char c = line.charAt(i);
       if ( ! (  (c >= '0' && c<='9')  ||
                 (c >= 'A' && c<='F') ) )  {
         return false;
       }
     }
     return true;
   }


   /**********************************************************************
     Process a High Resolution (profile) line read from the msg file. 

     returns true if no errors in line
              false if the length of the line is not enough to hold the
                    expected data
   ***********************************************************************/  
   boolean processHighResolution( String line, boolean printNumberValuesAveraged )
                               throws Exception {

     int factor; // how many times this line is repeated
     int openBracketPosition = line.indexOf('[');
     int closeBracketPosition ;
     if (openBracketPosition > 0) {
       closeBracketPosition = line.indexOf(']'); 
       factor = Integer.parseInt(line.substring(openBracketPosition+1, closeBracketPosition));
       line = line.substring( 0, openBracketPosition );
     }
     else {
       factor = 1; 
     }
     if (line.equals("00000000000000")) {
       // nothing to do if there is no data in the line.
       return true; 
     }

     if (line.length()<14) {
       // line is incomplete; therefore file is incomplete
       return false;
     }

     float temp = signedHexToFloat( line.substring(4, 8) );
     temp = temp * Float.parseFloat(meta.getTempCoefficient()) + Float.parseFloat(meta.getTempOffset());
     float pres = Float.parseFloat( Integer.toString( Integer.parseInt( line.substring(0, 4), 16 ) ) );
     pres = pres * Float.parseFloat(meta.getPresCoefficient()) + Float.parseFloat(meta.getPresOffset());
     float psal = Float.parseFloat( Integer.toString( Integer.parseInt( line.substring(8, 12), 16 ) ) );
     psal = psal * Float.parseFloat(meta.getSalCoefficient())  + Float.parseFloat(meta.getSalOffset());
     int cnts = Integer.parseInt( line.substring(12,14), 16 );

     DecimalFormat presFormat = new DecimalFormat("0.00");
     DecimalFormat tempFormat = new DecimalFormat("0.0000");
     DecimalFormat psalFormat = new DecimalFormat("####0.0000");

     for (int i=1; i<=factor; i++) {
        nbrHighResolutionLines++;
        if (printNumberValuesAveraged) {
          profDataLines[ nbrHighResolutionLines ] = Utils2.padLeft(presFormat.format(pres), 8) + "  " +
                                              Utils2.padLeft(tempFormat.format(temp), 8) + " " +
                                              Utils2.padLeft(psalFormat.format(psal), 8) + 
                                              Utils2.padLeft(Integer.toString(cnts),  4) +
                                              " 0" ; // checksum 
        }
        else {
          profDataLines[ nbrHighResolutionLines ] = Utils2.padLeft(presFormat.format(pres), 11) + "  " +
                                              Utils2.padLeft(tempFormat.format(temp), 11) + " " +
                                              Utils2.padLeft(psalFormat.format(psal), 11) + 
                                              " 0" ; // checksum 
        }

     }
     return true;
   }


   /**********************************************************************
     Process a GPS Fix line read from the msg file. 
   ***********************************************************************/  
   void processGPSFix( String line ) {
     
    /* if (line.startsWith("Fix: GPS fix not available")) {
       return;
     }*/
     if (line.startsWith("Fix: GPS fix not available")) {
        hasGPSFix = false;
        iGpsFixes = new InterpolateGpsFixes(this.pathName,floatId,profileNumber,this.beforeProfLine1);
        if(iGpsFixes.getInterpolatedValues(0) == null || iGpsFixes.getInterpolatedValues(1) == null){
        return;
        }

     }
    /* if (line.startsWith("Fix: GPS fix not available due to float stuck to bottom")) {
       return;
     }*/
     float latitude ;
     float longitude ;
     String longStr = "";
     String latStr = "";
     String dateStr = "";
     String timeStr ="";
     String nSatStr= "";
     DecimalFormat latLongFormat = new DecimalFormat("0.000");
     DecimalFormat posLatLongFormat = new DecimalFormat("+0.000");
     SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss") ;
    
     if(hasGPSFix == true){
     	StringTokenizer tok = new StringTokenizer(line); 
     	tok.nextElement().toString();    // 'Fix: ' text, useless.
        longStr = tok.nextElement().toString();  // latitude 
     	latStr = tok.nextElement().toString(); // longitude
     	dateStr = tok.nextElement().toString(); // date
     	timeStr = tok.nextElement().toString(); // time
     	nSatStr = tok.nextElement().toString(); // nbr of satelites
     }
     StringBuffer sb;
     // first line. (only if it wasn't been created already )
     if ( this.beforeProfLine1 == null ) {
       sb = new StringBuffer("    +99.999   +999.999 " );
       sb.append(dateStr.substring(6,10));sb.append("/");
       sb.append(dateStr.substring(0,3));
       sb.append(dateStr.substring(3,5));
       sb.append(" ");
       sb.append(timeStr.substring(0,2));sb.append(":");
       sb.append(timeStr.substring(2,4));sb.append(":");
       sb.append(timeStr.substring(4,6));
       sb.append("   000   0     0");
       this.beforeProfLine1 = sb.toString();
       //System.out.println("beforeProfileLine1 = "+this.beforeProfLine1);
     }
     if (hasGPSFix ==false){
	     sb = new StringBuffer("");
	     latitude  =  Float.parseFloat(iGpsFixes.getInterpolatedValues(0));
	     longitude =  Float.parseFloat(iGpsFixes.getInterpolatedValues(1));
	     Date date =  iGpsFixes.getDateProf();
	     if (latitude<0.00) {
	       sb.append( Utils2.padLeft( latLongFormat.format(latitude), 11 ) );
	     }
	     else {
	       sb.append( Utils2.padLeft( posLatLongFormat.format(latitude), 11 ) );
	     }
	     if (longitude<0.00) {
	       sb.append( Utils2.padLeft( latLongFormat.format(longitude), 11 ) );
	     }
	     else {
	       sb.append( Utils2.padLeft( posLatLongFormat.format(longitude), 11 ) );
	     }
	     sb.append(" ");
	     sb.append(formatter.format(date));
	     sb.append("   000   0     8");
	     this.beforeProfLine2[gpsFixCounter] = sb.toString();
	      // third line.
             sb = new StringBuffer("    +99.999   +999.999 " );
	     sb.append(formatter.format(date));
             sb.append("   000   0     0");
             this.beforeProfLine3 = sb.toString();
        //     System.out.println("this.beforeProfLine3 = " +this.beforeProfLine3);


     }
     if (hasGPSFix == true){
	     
	     // second line.
	     if(!(Integer.parseInt(nSatStr)<3)){
		     sb = new StringBuffer("");
		     latitude = Float.parseFloat( latStr );
		     longitude = Float.parseFloat( longStr );
		     if (latitude<0.00) {
		       sb.append( Utils2.padLeft( latLongFormat.format(latitude), 11 ) );
		     }
		     else {
		       sb.append( Utils2.padLeft( posLatLongFormat.format(latitude), 11 ) );
		     }
		     if (longitude<0.00) {
		       sb.append( Utils2.padLeft( latLongFormat.format(longitude), 11 ) );
		     }
		     else {
		       sb.append( Utils2.padLeft( posLatLongFormat.format(longitude), 11 ) );
		     }
		     sb.append(" ");
		     sb.append(dateStr.substring(6,10));sb.append("/");
		     sb.append(dateStr.substring(0,3));
		     sb.append(dateStr.substring(3,5));
		     sb.append(" ");
		     sb.append(timeStr.substring(0,2));sb.append(":");
		     sb.append(timeStr.substring(2,4));sb.append(":");
		     sb.append(timeStr.substring(4,6));
		     sb.append("   000");
		     sb.append(Utils2.padLeft(nSatStr,4));
		     sb.append("     0");
		     this.beforeProfLine2[gpsFixCounter] = sb.toString();
		     gpsFixCounter++;
		     //System.out.println("this.beforeProfLine2[0] = " +this.beforeProfLine2[0]);
	     
	    	     // third line.
		     sb = new StringBuffer("    +99.999   +999.999 " );
		     sb.append(dateStr.substring(6,10));sb.append("/");
		     sb.append(dateStr.substring(0,3));
		     sb.append(dateStr.substring(3,5));
		     sb.append(" ");
		     sb.append(timeStr.substring(0,2));sb.append(":");
		     sb.append(timeStr.substring(2,4));sb.append(":");
		     sb.append(timeStr.substring(4,6));
		     sb.append("   000   0     0");
		     this.beforeProfLine3 = sb.toString();
		     //System.out.println("this.beforeProfLine3 = " +this.beforeProfLine3);
	     }
	     else
	     {
		     this.beforeProfLine3 = this.beforeProfLine1;
		     if(this.beforeProfLine2 == null){
		         this.beforeProfLine2[0]=null;
		     }

	     }

     }//endOf if hasGPS==true

   }


   /**********************************************************************
     Sometimes a msg file profile holds more than one line of the same 
     parameter. 
     This function, given a parameter line will first check whether the 
     parameter has already been read in which case it updates the value; 
     otherwise it will add the line to the list of parameters
   ***********************************************************************/  
   void insertOrUpdateTechParameter( String name, String value ) {
     // first check whether ther parameter already exists
     if(debugOn) System.out.println( "M2PM D TP insertOrUpdateTechParameter for name: " + name);
     for (int i=1; i<=this.nbrTechParamLines; i++) {
       if ( name.equals( techParamLines[i].substring(0,40) ) ) {
         techParamLines[i] = name + value;
         return;
       }         
     }
     this.nbrTechParamLines++;
     techParamLines[ this.nbrTechParamLines ] = name + value;       
   }


   /***********************************************************************
    Given a value and the position of the technical parameter in the tech
    parameter list, return the calibrated value
    **********************************************************************/
     String getCalibratedValue( String value, int index ) {
     String coefficient = "0";
     String offset = "0";
     String calibratedFor = this.techTable.getCalibratedFor( index );
     if(debugOn) System.out.println("calibratedFor = "+calibratedFor+"value ="+value+"index = "+index);
     if (calibratedFor.equals( "current" )) {
       coefficient = meta.getCurrentCoefficient();
       offset = meta.getCurrentOffset();
       if ( (coefficient==null) || (offset==null) ) {
         try {
           logger.write( this.getClass().getName(), this.phyFilename, "Check current slope and offset on meta file" );
           logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                    " with code=" + Integer.toString(ERR_WRONG_META_INFO) );
           logger.close();
           buffReader.close();
           inpReader.close();
           buffWriter.close();
           outWriter.close();
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), this.phyFilename, 
                              "Check current slope and offset on meta file" );
           errorLogger.close();
         }
         catch (IOException e1) {}
         System.exit(ERR_WRONG_META_INFO);
       }
     }
     else {
     if (calibratedFor.equals( "voltage" )) {
       coefficient = meta.getVoltCoefficient();
       offset = meta.getVoltOffset();
       if ( (coefficient==null) || (offset==null) ) {
         try {
           logger.write( this.getClass().getName(), this.phyFilename, "Check voltage slope and offset on meta file" );
           logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                    " with code=" + Integer.toString(ERR_WRONG_META_INFO) );
           logger.close();
           buffReader.close();
           inpReader.close();
           buffWriter.close();
           outWriter.close();
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), this.phyFilename, 
                              "Check current slope and offset on meta file" );
           errorLogger.close();
         }
         catch (IOException e1) {}
         System.exit(ERR_WRONG_META_INFO);
       }
     }
     else {
     if (calibratedFor.equals( "vacuum" )) {
       coefficient = meta.getVacuumCoefficient();
       offset = meta.getVacuumOffset();
     }}} 

     float calValue;
     if (value.equals("0")) {
       calValue=0.0f;
     }
     else {
       calValue = (Float.parseFloat(value)*Float.parseFloat( coefficient )) + 
                         Float.parseFloat(offset);
     }

     //find out how many decimal points in coeff & offset
     //will determine how many decimal points in calibrated value.
     int dec_pts = Math.max( coefficient.length()-coefficient.indexOf('.')-1, 
                             offset.length()-offset.indexOf('.')-1 );
     if (dec_pts < 1) {
       dec_pts = 1;
     }
      
     StringBuffer strFormat = new StringBuffer( "0." );
     for (int i=1; i<=dec_pts; i++ ) {
       strFormat.append( '0');
     }
     DecimalFormat decFormat = new DecimalFormat(strFormat.toString());

     return decFormat.format( calValue );
     
   }


   /**********************************************************************
     Process a line from the msg file that holds a technical parameter
   ***********************************************************************/  
   void processTechnicalParameter( String line, Logger logger ) {

     int index;
     String name = null;

     int p1 = line.indexOf("=");
     if (p1>0) { 
        index = this.techTable.indexOf( line.substring(0, p1 ) );
        if (index==-1) {
          return;
        }
        else
          name = this.techTable.getPhyTechParamName( index );
     }
     else 
       return;  
          
     p1 = line.indexOf("=") + 1;
     int p2 = line.length();
     if ( (p1 == -1 ) || (p2 <= p1) ) {
       return;
     }
     else {
       String value = line.substring( p1, p2 ); 
       if (this.techTable.hasCalibration( index )) {
         value = getCalibratedValue( value, index );
       }
       insertOrUpdateTechParameter( name, value );
     }
   }

   /**********************************************************************
     Given a string, returns all the numbers and the decimal point (.) on 
     it
   ***********************************************************************/  
   String onlyDecNbrChars( String s ) {
     StringBuffer s2 = new StringBuffer("");
     for (int i=0; i<s.length(); i++) {
       char c = s.charAt(i);
       if ( (c >= '0' &&  c <= '9' ) || (c == '.') ) {
         s2.append( s.charAt(i) );
       }
     }
     return s2.toString();
   }


   void processParkObservations( String line ) {

      // get pressure
      int p1 = line.indexOf( " " );
      int p2 = line.indexOf( "," );
      String press = onlyDecNbrChars( line.substring( p1, p2 ) );
      int npchars = press.length();
      // if(debugOn) { System.out.println( "Msg2Phy_Ir D ParkObs length pres: " + npchars); }
      if(npchars > 16) {
         System.out.println( "Msg2Phy_Ir ParkObs pres longer than 16 (" + npchars + "), set to fillValue");
         press = fillValuePress_char;
      }
      insertOrUpdateTechParameter( "DRIFT PRESSURE PRE PROFILE (DBAR)       ", press );
      // get temperature
      p1 = line.indexOf( " ", p2 );
      p2 = line.indexOf( ",", p1 );
      String temp = onlyDecNbrChars( line.substring( p1, p2 ) );
      int ntchars = temp.length();
      // if(debugOn) { System.out.println( "Msg2Phy_Ir D ParkObs length temp: " + ntchars); }
      if(ntchars > 16) {
         System.out.println( "Msg2Phy_Ir ParkObs temp longer than 16 (" + ntchars + "), set to fillValue");
         temp = fillValuePress_char;
      }
      insertOrUpdateTechParameter( "DRIFT TEMPERATURE PRE PROFILE (DEG C)   ", temp );
      // get salinity 
      p1 = line.indexOf( " ", p2 );
      p2 = line.indexOf( ",", p1 );
      if (p2<p1) {
        p2 = line.length();
      }
      String psal = onlyDecNbrChars( line.substring( p1, p2 ) );
      int nschars = psal.length();
      // if(debugOn) { System.out.println( "Msg2Phy_Ir D ParkObs length psal: " + nschars); }
      if(nschars > 16) {
         System.out.println( "Msg2Phy_Ir ParkObs psal longer than 16 (" + nschars + "), set to fillValue");
         psal = fillValuePress_char;
      }
      insertOrUpdateTechParameter( "DRIFT SALINITY PRE PROFILE (PSU)        ", psal );
   }


   /*************************************************************************
     Write the config parameters and their values in the output phy file
   **************************************************************************/  
   void writeConfParameterBlock( BufferedWriter buffWriter ) {
     // write the missiong configuration parameters block
     if(debugOn) System.out.println( "M2PM D WR writeConfParameterBlock");
     try {
       for (int i=1; i<=this.nbrConfParamLines; i++) {
         buffWriter.write( this.confParamLines[i] );
         buffWriter.newLine();
       }
     }
     catch ( IOException e )
     {
       System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() ) );
       System.exit(ERR_IO);
     }
   }


   /*************************************************************************
     Write the technical parameters and their values in the output phy file
   **************************************************************************/  
   void writeTechParameterBlock( BufferedWriter buffWriter ) {
     // write the technical parameters block
     if(debugOn) System.out.println( "M2PM D WR writeTechParameterBlock");
     try {
       for (int i=1; i<=this.nbrTechParamLines; i++) {
         buffWriter.write( this.techParamLines[i] );
         buffWriter.newLine();
       }
     }
     catch ( IOException e )
     {
       System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() ) );
       System.exit(ERR_IO);
     }
   }


   /**********************************************************
     Process a Park (Drift) line read from the msg file.
   ***********************************************************/
   public void processParkPtLine( String line, int driftCnts, boolean isFirst )
   {

     if (isFirst) {
       driftPressureLine = new StringBuffer("DRIFT PRESSURE (DBAR)                   ");
       driftTemperatureLine = new StringBuffer("DRIFT TEMPERATURE (DEG C)               ");
       if (line.startsWith("ParkPts"))
       driftSalinityLine = new StringBuffer("DRIFT SALINITY (PSU)                    ");
       driftMeasurementTimeLine = new StringBuffer("DRIFT MEASUREMENT TIME (YYYYMMDDHHMMSS) ");
     }

     String year = "";
     String month = "";
     String day = "" ;
     int parkPtsTokens = 0;
     StringTokenizer tok = new StringTokenizer(line);
     if (line.startsWith("ParkPts"))
     parkPtsTokens = 10;
     else
     parkPtsTokens = 9;
     
     //System.out.println("Park Pts TOken "+parkPtsTokens);


     if (tok.countTokens()<parkPtsTokens) {
        // line is fragmented
       return;
     }
     for (int n=1; n<=parkPtsTokens; n=n+1) {
       String s = tok.nextElement().toString();
       switch (n)
       {
         case 2:
           if (s.equals("Jan")) {
             month = "01";
           } else if (s.equals("Feb")) {
             month = "02";
           } else if (s.equals("Mar")) {
             month = "03";
           } else if (s.equals("Apr")) {
             month = "04";
           } else if (s.equals("May")) {
             month = "05";
           } else if (s.equals("Jun")) {
             month = "06";
           } else if (s.equals("Jul")) {
             month = "07";
           } else if (s.equals("Aug")) {
             month = "08";
           } else if (s.equals("Sep")) {
             month = "09";
           } else if (s.equals("Oct")) {
             month = "10";
           } else if (s.equals("Nov")) {
             month = "11";
           } else if (s.equals("Dec")) {
             month = "12";
           }
           break;
         case 3: day = s; break;
         case 4: year = s; break;
         case 5: driftMeasurementTimeLine.append( year + month + day +
                                                 s.substring(0,2) +
                                                 s.substring(3,5) +
                                                 s.substring(6,8) );
                 driftMeasurementTimeLine.append( " " );
                 break;
         case 8: if ( s.indexOf("dbars")>0 ) 
                   s = s.substring(0, s.indexOf("dbars") );
                 driftPressureLine.append( s ); driftPressureLine.append( " " ); 
                 break; 
         case 9: if ( s.indexOf("C")>0 ) 
                   s = s.substring(0, s.indexOf("C") );
                 driftTemperatureLine.append( s ); driftTemperatureLine.append( " " ); 
                 break; 
         case 10: driftSalinityLine.append( s ); driftSalinityLine.append( " " ); break;
       }
     }
   }



   /*************************************************************************
     Write the column headers block to the output phy file
   **************************************************************************/  
   void writeColumnHeaderBlock( BufferedWriter buffWriter, boolean printNumberValuesAveraged ) {
     if (this.nbrLowResolutionLines > 0 || this.nbrHighResolutionLines > 0 ) {
       if(debugOn) System.out.println( "M2PM D WR writeColumnHeaderBlock");
       try {
         if (printNumberValuesAveraged) {
           buffWriter.write( "NUMBER OF COLUMNS                       5" );
         }
         else {
           buffWriter.write( "NUMBER OF COLUMNS                       4" );
         }
         buffWriter.newLine();
         buffWriter.write( "1. COLUMN                               PRESSURE (DBAR)" );
         buffWriter.newLine();
         buffWriter.write( "2. COLUMN                               TEMPERATURE-90 (DEG C)" );
         buffWriter.newLine();
         buffWriter.write( "3. COLUMN                               SALINITY (PSU)" );
         buffWriter.newLine();
         if (printNumberValuesAveraged) {
           buffWriter.write( "4. COLUMN                               NUMBER OF VALUES AVERAGED" );
           buffWriter.newLine();
           buffWriter.write( "5. COLUMN                               CHECKSUM FLAG" );
           buffWriter.newLine();
         }
         else {
           buffWriter.write( "4. COLUMN                               CHECKSUM FLAG" );
           buffWriter.newLine();
         }

       }
       catch ( IOException e )
       {
         System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() ) );
         System.exit(ERR_IO);
       }
     }
   }

   /*************************************************************************
     Write the column headers block to the output phy file for STS data
   **************************************************************************/  
    void writeColumnHeaderBlockForSTS( BufferedWriter buffWriter, boolean printNumberValuesAveraged ) {
     if (this.nbrLowResolutionLines > 0 || this.nbrHighResolutionLines > 0 ) {
       if(debugOn) System.out.println( "M2PM D WR writeColumnHeaderBlock");
       try {
         if (printNumberValuesAveraged) {
           buffWriter.write( "NUMBER OF COLUMNS                       13" );
         }
         else {
           buffWriter.write( "NUMBER OF COLUMNS                       12" );
         }
         buffWriter.newLine();
         buffWriter.write( "1. COLUMN                               PRESSURE (DBAR)" );
         buffWriter.newLine();
         buffWriter.write( "2. COLUMN                               TEMPERATURE-90 (DEG C)" );
         buffWriter.newLine();
         buffWriter.write( "3. COLUMN                               SALINITY (PSU)" );
         buffWriter.newLine();
         if (printNumberValuesAveraged) {
           buffWriter.write( "4. COLUMN                               NUMBER OF VALUES AVERAGED" );
           buffWriter.newLine();
           buffWriter.write( "5. COLUMN                               CHECKSUM FLAG" );
           buffWriter.newLine();
           buffWriter.write( "6. COLUMN                               PRESSURE2 (DBAR) ");
           buffWriter.newLine();
           buffWriter.write( "7. COLUMN                               TEMPERATURE2-90 (DEG C) ");
           buffWriter.newLine();
           buffWriter.write( "8. COLUMN                               CONDUCTIVITY2 (MS/CM) ");
           buffWriter.newLine();
           buffWriter.write( "9. COLUMN                               SALINITY2 (PSU) ");
           buffWriter.newLine();
           buffWriter.write( "10. COLUMN                               PRESSURE3 (DBAR) ");
           buffWriter.newLine();
           buffWriter.write( "11. COLUMN                              TEMPERATURE3-90 (DEG C)" );
           buffWriter.newLine();
           buffWriter.write( "12. COLUMN                              CONDUCTIVITY3 (MS/CM) ");
           buffWriter.newLine();
           buffWriter.write( "13. COLUMN                              SALINITY3 (PSU) ");
           buffWriter.newLine();
         }
         else {
           buffWriter.write( "4. COLUMN                               CHECKSUM FLAG" );
           buffWriter.newLine();
           buffWriter.write( "5. COLUMN                               PRESSURE2 (DBAR) ");
           buffWriter.newLine();
           buffWriter.write( "6. COLUMN                               TEMPERATURE2-90 (DEG C) ");
           buffWriter.newLine();
           buffWriter.write( "7. COLUMN                               CONDUCTIVITY2 (MS/CM) ");
           buffWriter.newLine();
           buffWriter.write( "8. COLUMN                               SALINITY2 (PSU) ");
           buffWriter.newLine();
           buffWriter.write( "9. COLUMN                               PRESSURE3 (DBAR) ");
           buffWriter.newLine();
           buffWriter.write( "10. COLUMN                              TEMPERATURE3-90 (DEG C)" );
           buffWriter.newLine();
           buffWriter.write( "11. COLUMN                              CONDUCTIVITY3 (MS/CM) ");
           buffWriter.newLine();
           buffWriter.write( "12. COLUMN                              SALINITY3 (PSU) ");
           buffWriter.newLine();
         }

       }
       catch ( IOException e )
       {
         System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() ) );
         System.exit(ERR_IO);
	}
      }
   }

}
