/***************************************************************************** 
  Revision date: 03-Dec-2012 CA: Accommodated Retrieval of all GPS Fixes from msg file
                 27-Aug-2012 CA: Removed function processParkPtLine and added to Msg2Phy_Ir
                 07-Aug-2012 CA: Added rows and column values(0,0) in the call of super
				 class constructor
  		 13-Jun-2012 VMH: more debug changes
                 21-May-2012 VMH: VMH debugs.
                 24-Mar-2011 Create new incomplete check when processing high
                             resolution lines.  
                 06-Jan-2011 When processing park P,T,S variables discard
                             incomplete lines.
                 05-Jan-2011 New values for error & warning message codes
                 10-Feb-2010   
  
  Msg2Phy_Ir_TS3.java
  ===================
  Decode a msg file from an Iridium float type APEXIR_TS3 message file and 
  create a correspondent new phy file

*****************************************************************************/

import java.util.*;
import java.io.*;
import java.text.*;

public class Msg2Phy_Ir_TS3 extends Msg2Phy_Ir
{
   
   String stsPathname;
   /****************************************************************
     Class constructor.  
     Set the file path name variables and the debug variable.
   *****************************************************************/  
   public Msg2Phy_Ir_TS3( String msgPathname, String phyPathname,String stsPathname,
                          String metaPathname, String technicalFilePathname,
                          String configTableFilePathname, 
                          String logJavaFilePathname,  String errorFilePathname, 
                          String debug )
   {
     super( msgPathname, phyPathname,metaPathname, 
            technicalFilePathname, configTableFilePathname,
            logJavaFilePathname, errorFilePathname, debug,0,0 );
     this.stsPathname = stsPathname;
   }


   /**********************************************************************
     Process all the lines of the msg file that will be converted into a
     phy file.
   ***********************************************************************/  
   void process(boolean stsData,String decoder) 
   {
     // const and vars that indicate in which phase of 
     // the reading of the input file the program is
     final byte NOT_STARTED = 0;
     final byte MISSION_CONFIG = 1;
     final byte PARK = 2;
     final byte LOW_RESOLUTION = 3;
     final byte HIGH_RESOLUTION = 4;
     final byte GPS = 5;
     final byte TECHNICAL = 6;
     byte phase = NOT_STARTED; 

     nbrDriftLines = 0;
     nbrTechParamLines = 0; 
     nbrHighResolutionLines = 0;
     nbrLowResolutionLines = 0;
     double [][] stsDataArray =  new double [2000][8];

     int driftCnts = 0; //  4 Parklines in input file becomes 1 line in output file
                        //  This var. keeps track of it; its values: 1,2,3,4,1,2,3,4,...

     boolean msgFileIncomplete = false;
     boolean confParameterNotFound = false;
     boolean parkPts = false; //parkPts to include the salinity into the PHY file - CA
     int stsLines = 0,stsLineCount=0,j=0;
     boolean stsLinesPrinted = false;

     try
     {
      
       processMetaFileInfo( buffWriter );
      if(decoder == "TS6"){
           DecodeSTSData dsd = new DecodeSTSData(this.stsPathname,stsData);
	   stsLines = dsd.getLineCount();
	   stsDataArray = dsd.getSTSData();
     }


       String line = buffReader.readLine();
       while (line!=null) {
         if (debugOn) System.out.println( "TS3 D " + phase + " " + line );
	

         try {

           if (line.length()==0) {
             line = buffReader.readLine(); 
             continue;
           }
	   if(line.startsWith("ParkPts"))
	   parkPts = true;

           if (line.equals("<EOT>")) {
           }
           else if (line.startsWith("$ Mission configuration")) {
             phase = MISSION_CONFIG;
           }
           else if (line.startsWith("ParkPt")) { 
             phase = PARK;
             if (driftCnts==0) {
               driftCnts = 1;
               processParkPtLine(line, 1, true ); 
             }
             else {
               driftCnts++ ;
               processParkPtLine(line, driftCnts, false ); 
               if (driftCnts==4) {
                 nbrDriftLines = nbrDriftLines + 1;
                 driftPressureLines[nbrDriftLines] = driftPressureLine.toString();
                 driftTemperatureLines[nbrDriftLines] = driftTemperatureLine.toString();
		 //Added By CA
		 if (parkPts) 
		 {
                  driftSalinityLines[nbrDriftLines] = driftSalinityLine.toString();
		 }
                 driftMeasurementTimeLines[nbrDriftLines] = driftMeasurementTimeLine.toString();
                 driftCnts = 0;
               }
             }
           }
           else if (line.startsWith("$ Profile")) {
             processBeforeLowResolution(line);
             phase = LOW_RESOLUTION;
           }
           else if ( line.startsWith("$") && 
                    (phase!=MISSION_CONFIG) ) {
             if (phase==PARK) {
               if ( line.length()==1 || line.startsWith("Pal") ) {
                 line = buffReader.readLine(); 
                 continue;
               }
               else {
                 phase = LOW_RESOLUTION;
               }
             }
           }  
           else if (line.startsWith("Fix: ")) {
             processGPSFix( line );
             phase = TECHNICAL;
           }
           else if (line.startsWith("# GPS fix obtained")) {
             phase = TECHNICAL;
           }
           else if (line.startsWith("#")) {
             if (phase==LOW_RESOLUTION) {
               processAfterLowResolution( line );
               phase = HIGH_RESOLUTION;
             }
           }  
           else if (line.startsWith("ParkDescentPCnt")) {
             if ( ! processParkDescent(line, buffReader)) {
               msgFileIncomplete = true; 
               try {
                 logger.write( this.getClass().getName(), this.phyFilename, "Warning: Number of DRIFT DESCENT declared points not equal to actual number of points." );
               }
               catch (IOException e1) {}
               try {
                 errorLogger.write( this.getClass().getName(), this.phyFilename, 
                                   "Warning: Number of DRIFT DESCENT declared points not equal to actual number of points." ) ;
               }
               catch (IOException e1) {}

             }
           }
           else if (line.startsWith("ParkObs")) {
             processParkObservations( line );
           }
           else {
             if (phase==HIGH_RESOLUTION) {
               if (isHighResolution(line) ) {
                 if ( ! processHighResolution(line, false) ) {
                   msgFileIncomplete = true;
                   try {
                     logger.write( this.getClass().getName(), this.phyFilename, "High Resolution Line: ["+line+"] not long enough.");
                   }
                   catch (IOException e1) {}
                   try {
                     errorLogger.write( this.getClass().getName(), this.phyFilename, 
                                   "Warning: High Resolution Line: ["+line+"] not long enough.");
                   }
                   catch (IOException e1) {}
                 }; 
               }
               else {
                 phase=TECHNICAL;
                 processTechnicalParameter(line, logger); 
               }
             }
             else if (phase==LOW_RESOLUTION) {
               processLowResolution(line, false); 
             }
             else if (phase==TECHNICAL) {
               processTechnicalParameter(line, logger); 
             }
             else if (phase==MISSION_CONFIG) {
               if ( ! processMissionConfiguration(line, logger) ) {
                 confParameterNotFound = true; 
                 try {
                   logger.write( this.getClass().getName(), this.phyFilename, 
                                 "Warning: Configuration parameter not found in line " + line );
                 }
                 catch (IOException e1) {}
                 try {
                   errorLogger.write( this.getClass().getName(), this.phyFilename, 
                                      "Warning: Configuration parameter not found in line " + line );
                 }
                 catch (IOException e1) {}
               }
             }
           }  

           line = buffReader.readLine(); 
         }
         catch (Exception e) {
           e.printStackTrace();
           try {
             logger.write( this.getClass().getName(), this.phyFilename, "Error: " + e.getMessage() + 
                           "\n while processing line: \n" + line);
             logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                       " with code=" + Integer.toString(ERR_EXECUTION) );
             logger.close();
             buffReader.close();
             inpReader.close();
             buffWriter.close();
             outWriter.close();
           }
           catch (IOException e1) {}
           try {
              errorLogger.write( this.getClass().getName(), this.phyFilename, 
                                 "Error: " + e.getMessage() + "\n while processing line: \n" + line );
              errorLogger.close();
           }
           catch (IOException e1) {}
           System.exit(ERR_EXECUTION);
         }
       }

       // flushing possible incomplete lines
       if (driftCnts>0 && driftCnts<4) {
         nbrDriftLines = nbrDriftLines + 1;
         driftPressureLines[nbrDriftLines] = driftPressureLine.toString();
         driftTemperatureLines[nbrDriftLines] = driftTemperatureLine.toString();
	 //Added By CA
	 if (parkPts)  
	 {
	  driftSalinityLines[nbrDriftLines] = driftSalinityLine.toString();
	 }
         driftMeasurementTimeLines[nbrDriftLines] = driftMeasurementTimeLine.toString();
       }

       // Write the Drift Lines block
       for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftPressureLines[i] );
         buffWriter.newLine();
       }
       for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftTemperatureLines[i] );
         buffWriter.newLine();
       }
       //Added By CA
      if (parkPts) 
       {
         for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftSalinityLines[i] );
         buffWriter.newLine();
        }
       }
         for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftMeasurementTimeLines[i] );
         buffWriter.newLine();
       }

       writeConfParameterBlock( buffWriter );

       writeTechParameterBlock( buffWriter );
       if(stsData)
       writeColumnHeaderBlockForSTS( buffWriter, false );
       else
       writeColumnHeaderBlock( buffWriter, false );

       if(debugOn) System.out.println( "TS3 D WR NAV beg");
       buffWriter.write( "============================================================" );
       buffWriter.newLine();
       buffWriter.write( "   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION-QC" );
       buffWriter.newLine();
       if ( beforeProfLine1==null && beforeProfLine2[0]==null && beforeProfLine3==null ) {
         buffWriter.write( "    +99.999   +999.999 9999/99/99 99:99:99   000   0     0");
         buffWriter.newLine();
         msgFileIncomplete = true;
         try {
           logger.write( this.getClass().getName(), this.phyFilename, "Warning: No position data found" );
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), this.phyFilename, 
                             "Warning: No position data found" );
         }
         catch (IOException e1) {}
       }
       if ( beforeProfLine1 != null ) {
         buffWriter.write( beforeProfLine1 );
         buffWriter.newLine();
       }
       /*if ( beforeProfLine2 != null ) {
         buffWriter.write( beforeProfLine2 );
         buffWriter.newLine();
       }*/
       if ( beforeProfLine2[0] != null ) {
          for(int i=0;i<=gpsFixCounter;i++){
	   if((i< gpsFixCounter) || (i ==0) ){
           buffWriter.write( beforeProfLine2[i] );
           buffWriter.newLine();
	  }
	 }
       }
       if ( beforeProfLine3 != null ) {
         buffWriter.write( beforeProfLine3 );
         buffWriter.newLine();
       }
       buffWriter.write( "============================================================" );
       buffWriter.newLine();
       if(debugOn) System.out.println( "TS3 D WR NAV end");
 
       // write the low resolution data block
       for (int i=1; i<=nbrLowResolutionLines; i++) {
         buffWriter.write( lowResolutionLines[i] );
        
        if(stsData && ! stsLinesPrinted && decoder == "TS6"){
         buffWriter.write( " " );

             for (int k=0; k<8;k++){
              if(k == 3 || k == 7)
              buffWriter.write(Utils2.padLeft(String.format("%.6f",stsDataArray[stsLineCount][k]),11)+" ");
              else
              buffWriter.write(Utils2.padLeft(String.format("%.5f",stsDataArray[stsLineCount][k]),11)+" ");
            } 
             stsLineCount++;
	     if(stsLineCount > stsLines){
	       stsLinesPrinted = true;
	     }
           
         }
       /* if(!stsData && ! stsLinesPrinted && decoder =="TS6"){
         buffWriter.write( " " );
             for (int k=0; k<8;k++){
              buffWriter.write(Utils2.padLeft(String.format("%.1f","-999.0"),11)+" ");
            } 

         } */
         buffWriter.newLine();
       }

       // write the profile data block
       for (int i=nbrHighResolutionLines; i>0; i--) {
         buffWriter.write( profDataLines[i] );
	 if(stsData && ! stsLinesPrinted && decoder=="TS6"){
           buffWriter.write( " " );
             for (int k=0; k<8;k++){
              if(k == 3 || k == 7){
               if(stsDataArray[stsLineCount][k] == -999.0)
                 buffWriter.write(Utils2.padLeft(String.format("%.1f",stsDataArray[stsLineCount][k]),11)+" ");
               else 
                 buffWriter.write(Utils2.padLeft(String.format("%.6f",stsDataArray[stsLineCount][k]),11)+" ");
              }
              else{
               if(stsDataArray[stsLineCount][k] == -999.0)
                 buffWriter.write(Utils2.padLeft(String.format("%.1f",stsDataArray[stsLineCount][k]),11)+" ");
               else 
                 buffWriter.write(Utils2.padLeft(String.format("%.5f",stsDataArray[stsLineCount][k]),11)+" ");
              }
	    } 
             stsLineCount++;
             
	     if(stsLineCount > stsLines){
	       stsLinesPrinted = true;
	     }

	 }
/*        if(!stsData && ! stsLinesPrinted && decoder =="TS6"){
         buffWriter.write( " " );
             for (int k=0; k<8;k++){
              buffWriter.write(Utils2.padLeft(String.format("%.1f",Constants.fillValuePress),11)+" ");
            }
        } 

*/


            if(debugLev > 1) System.out.println( "TS3 D WR edf HI i: " + i);

             buffWriter.newLine();
	 }
	 if(! stsLinesPrinted && stsData && decoder=="TS6"){
          for (int ll=stsLineCount;ll<stsLines;ll++){
                buffWriter.write(Utils2.padLeft(String.format("%.1f",Constants.fillValuePress), 11) +" "+ 
                Utils2.padLeft("-999.0", 11) + " "+
                Utils2.padLeft("-999.0", 11) + " "+ 
                                        " 0"  +" " );
                for (int k=0; k<8;k++){
                   if(k == 3 || k == 7){
                       if(stsDataArray[stsLineCount][k] == -999.0)
                           buffWriter.write(Utils2.padLeft(String.format("%.1f",stsDataArray[stsLineCount][k]),11)+" ");
		       else
                           buffWriter.write(Utils2.padLeft(String.format("%.6f",stsDataArray[ll][k]),11)+" ");
                    }
                     else{
                       if(stsDataArray[stsLineCount][k] == -999.0)
                           buffWriter.write(Utils2.padLeft(String.format("%.1f",stsDataArray[stsLineCount][k]),11)+" ");
		       else
                           buffWriter.write(Utils2.padLeft(String.format("%.5f",stsDataArray[ll][k]),11)+" ");
		     }
	         } 
                buffWriter.newLine();
           }

         }

       

       // Warning when no profile data is not found.
       if ( (profileNumber>0) && (nbrLowResolutionLines==0) && 
            (nbrHighResolutionLines==0) ) {
         msgFileIncomplete = true;
         try {
           logger.write( this.getClass().getName(), this.phyFilename, "Warning: No profile data found" );
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), this.phyFilename, 
                             "Warning: No profile data found" );
         }
         catch (IOException e1) {}
       }

       buffReader.close();
       inpReader.close();
       buffWriter.close();
       outWriter.close();

       if (msgFileIncomplete) {
         try {
           logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                    " with code=" + Integer.toString(WRN_MSG_FILE_INCOMPLETE) );
           logger.close();
           errorLogger.close();
         }
         catch (IOException e1) {}
         System.exit(WRN_MSG_FILE_INCOMPLETE);
       }
       else {
         if (confParameterNotFound) {
           try {
             logger.write( this.getClass().getName(), null, 
                           "End processing " + msgPathname + " with code=" + Integer.toString(WRN_CONF_PARAM_NOTFOUND) );
             logger.close();
             errorLogger.close();
           }
           catch (IOException e1) {}
           System.exit(WRN_CONF_PARAM_NOTFOUND);
         }
         else {
           try {
             logger.write( this.getClass().getName(), null, 
                           "End processing " + msgPathname + " with code="+Integer.toString(OK) );
             logger.close();
             errorLogger.close();
           }
           catch (IOException e1) {}
         }
       }
    }
    catch ( IOException e )
    {
      try {
        logger.write( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() );
        logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                 " with code=" + Integer.toString(ERR_IO) );
        logger.close();
        logger.close();
        buffReader.close();
        inpReader.close();
        buffWriter.close();
        outWriter.close();
      }
      catch (IOException e1) {}
      try {
         errorLogger.write( this.getClass().getName(), this.phyFilename, 
                           "IO Error: " + e.getMessage() );
         errorLogger.close();
      }
      catch (IOException e1) {}
      System.exit(ERR_IO);
    }
    catch (Exception e) {
      e.printStackTrace();
      try {
        logger.write( this.getClass().getName(), this.phyFilename, "Error: " + e.getMessage() );
        logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                 " with code=" + Integer.toString(ERR_EXECUTION) );
        logger.close();
        logger.close();
        buffReader.close();
        inpReader.close();
        buffWriter.close();
        outWriter.close();
      }
      catch (IOException e1) {}
      try {
         errorLogger.write( this.getClass().getName(), this.phyFilename, 
                           "Error: " + e.getMessage() );
         errorLogger.close();
      }
      catch (IOException e1) {}
      System.exit(ERR_EXECUTION);
    }

   }
 

   public static void main ( String args[] )
   {
     if(debugOn) System.out.println( "TS3 D beg");
     boolean stsData = false;
     if ( args.length != 9) {
       System.out.println( Utils2.dateFormatMessage( "Msg2Phy_Ir_TS3", null, "Usage:java Msg2File msgPathname phyPathname metaPathname technicalTableFilePathname configTableFilePathname logJavaFilePathname errorFilePathname debug") ); 
       System.exit(ERR_WRONG_NBR_PARAM);
     }

     Msg2Phy_Ir_TS3  msg2Phy = new Msg2Phy_Ir_TS3(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7],args[8]);

     if(! args[2].contains("NONE") )
       stsData = true;
     
     String decoder = "TS3";
     msg2Phy.process(stsData,decoder);  
     if(debugOn) System.out.println( "Ir_TS3 D end");
   }

} 
