/************************************************************************************* 
  Revision date: 10-Feb-2010 ER: Initial version 
  
  HighResTableReader.java
  ===================
  Reads the High Res table for each float type to determine the flow of equations
  it has to process 

**************************************************************************************/
import java.io.*;
import java.util.*;
import java.text.*;

public class HighResTableReader_all extends Msg2Phy_Ir_TSOALL
{
	
    private FileReader fileReader;
    private BufferedReader buffReader;
    boolean error; 
    public String filePathname ;
    public LinkedHashMap lhmHigh;
    List headerNames ;
    String instrumentType;
    int extraVariables=0;
    int eqPos=0;
    int ofreqPos=0;

    LinkedHashMap<String,List> lhm ;
    List<String> equations_high = new ArrayList<String>();
    List<String> lineGen_high = new ArrayList<String>();

  public HighResTableReader_all(String filePathname) {
     
  try {
       this.filePathname=filePathname;
       fileReader = new FileReader( filePathname );
       buffReader = new BufferedReader( fileReader );
       highResHeaders();
     }
     catch ( IOException e ) {
       setError( true );
       return;
     }


  }
  public void setError( boolean error ) {
    this.error = error;
  }

  public boolean hasError() {
    return this.error;
  }



  private void highResHeaders() throws IOException 
  {
      	String line = buffReader.readLine();
        StringTokenizer st1 ;
       	int sep1Count = 0;
       	int sep2Count = 0;
        String sep1String,sep2String;
	String name = "";
	int keyCount=0;
	lhm =  new LinkedHashMap <String,List>();
        List<Object> values = new ArrayList<Object>();
	boolean lineGen_bool = false;

    
       while(line != null)
       {
       		//System.out.println("Line from High Res::"+line);
			 
			 st1= new StringTokenizer(line,"<>");
			 //System.out.println("line"+line);
			while(st1.hasMoreElements())
			{
				if(sep1Count == 0)
				{
				 sep1String = st1.nextElement().toString();
				 //System.out.println("sep1String in If"+sep1String);
				 if(sep1String.equals("gen"))
				 lineGen_bool = true;
				 else
				 lineGen_bool = false;
				 sep1Count++;
				}
				sep1String = st1.nextElement().toString();
				//System.out.println("sep1String = "+sep1String);
				
				if(sep1String.startsWith("TOTAL"))
				 extraVariables++;
				
				if(sep1String.startsWith("NSAMPLE"))
				 extraVariables++;
				
				StringTokenizer st2 = new StringTokenizer(sep1String,",");
			
				while(st2.hasMoreElements())
				{
				   
				    if(sep2Count == 0)
				    {
					name =st2.nextElement().toString();
				
					if(name.startsWith("equation"))
					this.equations_high.add(name);

					if(lineGen_bool){
					this.lineGen_high.add(name);
					}
					
					//System.out.println("name in If :::  "+name);
					sep2Count++;
				    }
				    sep2String =st2.nextElement().toString();
				    //System.out.println("sep2String = "+sep2String);
				    values.add(sep2String);
				
				}
	//	 		   //System.out.println("St1 next Element"+st1.nextElement());
	    


				   sep2Count =0;
				   lhm.put(name,values);
				   values = new ArrayList();
				   if(name.startsWith("equation"))
				   eqPos = lhm.size();
				   
				   if(name.startsWith("ofreq"))
				   ofreqPos = lhm.size();

				   //System.out.println(" Position of  = "+eqPos);
				   //System.out.println(" No of Values in the Hash map = "+lhm.size());
				   //System.out.println(" values from the Hash map = "+lhm.get(name));
				   //System.out.println(" values from the Hash map in high res = "+lhm);

				   setHeaders(lhm);
				   setMap(lhm);

			}
		
	
      	line = buffReader.readLine();
	sep1Count =0;
	}
  }



private void setHeaders(LinkedHashMap lhmKeyValues){
	this.headerNames = new ArrayList(lhmKeyValues.keySet());
	//System.out.println("headerNames in highRes == "+headerNames);
}
public List getHeaders(){
	return this.headerNames;
}

private void setMap(LinkedHashMap lhmKeyValues){
	this.lhmHigh = lhmKeyValues;
}

public LinkedHashMap getMap(){
       return this.lhmHigh;
}
public int getHeadersSize(){

	int headerSize = this.headerNames.size() - extraVariables; 
	//System.out.println("headerNames Size == "+headerSize);
	return headerSize;
}

public List getEquations(){
        //System.out.println("Equations in High Res = "+this.equations_high);
	return this.equations_high;
}

public List getEquationParameters(String str){

	List forParameters = new ArrayList();
	List parameters = new ArrayList();
	LinkedHashMap<String,List> lhm_temp = new LinkedHashMap<String,List>();

         //System.out.println("str == "+str);

	lhm_temp = getMap();
	parameters = lhm_temp.get(str);
	
         //System.out.println("parameters::" +parameters.size());
	
	//parameters = lhm_temp.get(str)
	for(int jj=2;jj<parameters.size();jj++){
	   forParameters.add(parameters.get(jj));
	}
         //System.out.println("forParameters in low res class" +forParameters);
	return forParameters;
 }


public List getLinegen(){
	return this.lineGen_high;
}


public int getEquationPositionHighRes(){

	int eqPosition = this.eqPos - extraVariables;
	//System.out.println(" Position of  = "+eqPosition);
        return eqPosition;

}


public int getOfreqPositionLowRes(){
        //System.out.println("Ofreq positon= "+ofreqPos);
	return ofreqPos;
}

public String getFirstValue(String str){
       // System.out.println("First Value String in high Res= "+str);
	//System.out.println("First Value"+ (lhm.get(str).get(0).toString()));
	return lhm.get(str).get(0).toString();
}

}

