/********************************************************************************************* 

  Revision date: 27-Dec-2013  CA:  All fixes except for those that have nSat < 3 are retrieved now
                                   Earlier if nSat < 4 all the other GPS fixes were if ignored
                 26-Jun-2013  CA:  processGPS function is modified to ignore GPS Fix that has less 
				   than 3 satellites
  		 22-Mar-2013  CA:  Initial release
  
  Msg2Phy_Ir_TSOALL.java
  ===================
  Decode a msg file from an Iridium message file of any Float type



***********************************************************************************************/


import java.util.*;
import java.io.*;
import java.text.*;

public class Msg2Phy_Ir_TSOALL{

   Logger logger;                    // Used to write to the log file.
   Logger errorLogger;               // Used to write to the error file.
   FileReader inpReader = null;      // Used to read msg file
   BufferedReader buffReader = null; // Used to read msg file
   FileWriter outWriter = null;      // Used to write to the phy file
   BufferedWriter buffWriter = null; // Used to write to the phy file

   MetaFileInfo_all meta;

   TechnicalTable techTable; // To hold the x-ref technical parameter table
   MissionConfigTable confTable; // To hold the x-ref mission conf parameter table
   LowResTableReader_all lowResTabReader; // To Hold the LowResTableReader_all values
   HighResTableReader_all highResTabReader; // To Hold the highResTableReader_all values
   
   String msgPathname;  // Message file pathname (input)
   String pathName;  // pathname to InterpolatedGpsFixes.java 
   String technicalFilePathname;   // Technical Parameters file pathname (input) 
   String confTableFilePathname; // Mission configuration parameters file pathname (input)
   String phyPathname;           // Phy file pathname (output) 
   String metaPathname;      // Meta file pathname (output) 
   String lowResTabPathName;      //Low Resolution Table Path Name 
   String highResTabPathName;      //High Resolution Table Path Name 
   String logJavaFilePathname;   // Java log file pathname (input)
   String errorFilePathname;     // Error file pathname (input)
   int profileNumber;      
   String phyFilename;
   static boolean debugOn;              // Is debug on
   static int debugLev;                 // What level of debug do you want
   int rows;				// Number of Rows for foil matrix co-efficients
   int columns;				// Number of Columns for foil matrx co-efficients
   String floatId;
   int nbrTechParamLines;
   String[] confParamLines = new String[Constants.MAX_NBR_CONFIG_LINES];    
   int nbrConfParamLines;
   
   int equationCount;
   List<String> equations = new ArrayList<String>();
   List<String> equationsHigh = new ArrayList<String>();
   
   
   // var used for the (Park) Drift phase
   StringBuffer driftPressureLine = new StringBuffer();
   StringBuffer driftTemperatureLine = new StringBuffer();
   StringBuffer driftSalinityLine = new StringBuffer();
   StringBuffer driftMeasurementTimeLine = new StringBuffer();
   
   String[] driftPressureLines = new String[Constants.MAX_NBR_DRIFT_LINES];
   String[] driftTemperatureLines = new String[Constants.MAX_NBR_DRIFT_LINES];
   String[] driftSalinityLines = new String[Constants.MAX_NBR_DRIFT_LINES];
   String[] driftMeasurementTimeLines = new String[Constants.MAX_NBR_DRIFT_LINES];
   int nbrDriftLines;

   // var used for the general profile data lines as well as the hig res.
   
   String[] profDataLines = new String[Constants.MAX_NBR_HIGH_RESOLUTION_LINES];    
   int nbrProfDataLines;
   int nbrHighResolutionLines;
   String[] highResolutionLines = new String[ Constants.MAX_NBR_HIGH_RESOLUTION_LINES];
   int nbrProfResolutionLines;
   double[] highResolutionPres = new double[Constants.MAX_NBR_HIGH_RESOLUTION_LINES];

   // var used for the low resolution lines
   
   String[] techParamLines = new String[Constants.MAX_NBR_TECH_LINES];    
   int nbrLowResolutionLines;
   String[] lowResolutionLines = new String[ Constants.MAX_NBR_LOW_RESOL_LINES ];
   double[] lowResolutionPres = new double[ Constants.MAX_NBR_LOW_RESOL_LINES ];

   // Two lines with date info before the profile data block.
   String beforeProfLine1;
  // String beforeProfLine2;
   String[] beforeProfLine2 = new String[50]; //Initializing to 10 , need to Check with Claudia
   String beforeProfLine3;

   int gpsFixCounter = 0; //Counter to keep track of all the GPS Fixes
 
   // var used for profiles 1, 2 
   
   double[] prof1Pres  = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   double[] prof1Temp  = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   double[] prof1Psal  = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   int[]    prof1Ofreq = new int[Constants.MAX_NBR_PROFILE_1_LINES];
   
   double[] prof1O2    = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   String NSamples;
   int nbrProfile1Lines;
   int printTSFlag = 0;

   double[] prof2Pres = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2Temp = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2Psal = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2Bphase = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2Tphase = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2Rphase = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2TOpt = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2O2 = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2Tphase2 = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2TOpt2 = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2O22 = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2NO3 = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   double[] prof2PH = new double[Constants.MAX_NBR_PROFILE_2_LINES];
   
    double[][] lowResProfData = new double[Constants.MAX_NBR_PROFILE_2_LINES][Constants.HEADERS];
   int nbrProfile2Lines;
   int headersCount;

   double[] prof3Pres  = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   double[] prof3Temp  = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   double[] prof3Psal  = new double[Constants.MAX_NBR_PROFILE_1_LINES];
   int nbrProfile3Lines;
   

   // for use in formatting decimal numbers
   DecimalFormat dec1Format = new DecimalFormat("0.0");
   DecimalFormat dec2Format = new DecimalFormat("0.00");
   DecimalFormat dec3Format = new DecimalFormat("0.000");
   DecimalFormat dec4Format = new DecimalFormat("0.0000");
   DecimalFormat dec6Format = new DecimalFormat("0.000000");

   // vars used for no3 and ph
   static boolean has; // are the optical variables present? (not currentlykept)
   static double no3, ph,rphase,p,t,s,tphase,topt;
   
   //Linked HashMap varibale for reading Low Resolution Profile Header Data
   //Linked HashMap<String,Integer> hm = new HashMap<String,Integer>();
   LinkedHashMap<String,Integer> lhmLowHeaders = new LinkedHashMap<String,Integer>();
   LinkedHashMap<String,List> lhmHighMap = new LinkedHashMap<String,List>();
   LinkedHashMap<String,List> lhmLowMap = new LinkedHashMap<String,List>();
   boolean hasGPSFix = true;
   InterpolateGpsFixes iGpsFixes = null;
   
   boolean hasNitrate; // does the low resolution section have a nitrate column?
   boolean hasPH;      // does the low resolution section have a ph column?
   boolean hasOptical; // are the optical variables present? (not currentlykept)
   boolean hasRPhase;      // does the low resolution section have a rphase column?
   

   boolean[] t_interpolated;
   boolean t_interpolation_exists = false;
   boolean[] s_interpolated;
   boolean s_interpolation_exists = false;
   boolean ofreq_exists = false;
 
   int pos_interp_t2 = 0;    //Position at which TEMPERATURE2 occurs
   int pos_interp_s2 = 0;    //Position at which SALINITY2 occurs
   static int lowres_prof=0;
   int highResHeader_count=0;// High Resoultion Header Counter
   boolean display_LRProfile_separate=false;
   boolean display_LROfreq=false;
   int ofreqPos_high = 0; 
   int o2_position_high =0; 
   int o2_position_low =0; 
   int ofreqPos_low = 0;
   int matrixCount; 
   
   boolean hrProfile_exists = true; 
   double maxHRProfile = 0.0;
   double minHRProfile = 0.0;
   double maxLRProfile = 0.0; 
   double minLRProfile = 0.0;
   
   boolean profile_overlap = false;
   double o2=0.0;
   
   public Msg2Phy_Ir_TSOALL(){}
   /****************************************************************
     Class constructor.  
     Set the file path name variables and the debug variable.
   *****************************************************************/  
   public Msg2Phy_Ir_TSOALL( String msgPathname, String phyPathname,
                           String metaPathname, String technicalFilePathname,
                           String confTableFilePathname,String lowResTabPathName,
			   String highResTabPathName, 
                           String logJavaFilePathname, String errorFilePathname, String debug )
   {
     this.msgPathname = msgPathname;
     //System.out.println("this.msgPathname = "+this.msgPathname);
     int p = this.msgPathname.indexOf("iridium");
     this.pathName = this.msgPathname.substring(0,p+8);
     //System.out.println("this.pathName = "+this.pathName);
     this.phyPathname = phyPathname;
     p=this.phyPathname.lastIndexOf('/');
     this.phyFilename=this.phyPathname.substring(p+1,this.phyPathname.length());
     //System.out.println("this.phyPathname= "+this.phyPathname);
     //System.out.println("this.phyFilename= "+this.phyFilename);
     this.metaPathname = metaPathname;
     this.technicalFilePathname = technicalFilePathname;
     //System.out.println("this.technicalFilePathname = "+this.technicalFilePathname);
     this.confTableFilePathname = confTableFilePathname;
     //System.out.println("this.confTableFilePathname = "+this.confTableFilePathname);
     this.logJavaFilePathname = logJavaFilePathname;
     this.errorFilePathname = errorFilePathname;
     this.lowResTabPathName = lowResTabPathName;
     this.highResTabPathName = highResTabPathName;
     this.debugLev = Integer.parseInt(debug);
     this.debugOn = this.debugLev>0;
     this.rows = rows;
     this.columns = columns;
     p=this.msgPathname.lastIndexOf('/');
     this.floatId = msgPathname.substring(p+1,this.msgPathname.indexOf('.'));
     // Open log file
     try {
       logger = new Logger( logJavaFilePathname );
       logger.write( this.getClass().getName(), null, "Start processing " + msgPathname );
     }
     catch ( IOException e )
     {
        System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, 
                            "IO Error while opening Java log file: " + logJavaFilePathname ) );
        System.exit(Constants.ERR_CREATE_LOG_FILE);
     }

     // Open Error Log file
     try {
       errorLogger = new Logger( errorFilePathname );
     }
     catch ( IOException e )
     {
        System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, 
                            "IO Error while opening error file: " + errorFilePathname ) );
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening error file: " + errorFilePathname );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(Constants.ERR_OPEN_ERROR_FILE) );
          logger.close();
        }
        catch (IOException e1) {}
        finally {
          System.exit(Constants.ERR_OPEN_ERROR_FILE);
        }
     }

     // Load low Resoulution vars into object
     if (debugOn){ 
     System.out.println("lowResTabPathName"+lowResTabPathName);
     }
     this.lowResTabReader = new LowResTableReader_all( lowResTabPathName );
     if ( this.lowResTabReader.hasError() ) {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening Low Resolution Table  " + lowResTabPathName );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(Constants.ERR_OPEN_LOWRES_TABLE) );
          logger.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, "Error opening Low Resolution Table " +lowResTabPathName);
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(Constants.ERR_OPEN_LOWRES_TABLE);
     }
     // Load High Resoulution vars into object
     if (debugOn){ 
     System.out.println("highResTabPathName"+highResTabPathName);}
     

     // Load metafile vars into object meta                                                            
     this.meta = new MetaFileInfo_all( metaPathname,this.lowResTabReader,this.logger,errorFilePathname );                                                                                                      
     if ( this.meta.hasError() ) {                                                                     
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening meta file " + metaPathname  );                                                                                             
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname +             
                                                   " with code=" + Integer.toString(Constants.ERR_OPEN_META_FILE) ); 
          logger.close();                                                                              
        } 
        catch (IOException e1) {}                                                                      
        try { 
          errorLogger.write( this.getClass().getName(), this.phyFilename, "Error opening meta file " + metaPathname  );          
          errorLogger.close();
        }
        catch (IOException e1) {}                                                                      
        System.exit(Constants.ERR_OPEN_META_FILE);                                                 
     }  

     //this.highResTabReader = new HighResTableReader_all( highResTabPathName,meta.getInstrumentType());
     this.highResTabReader = new HighResTableReader_all( highResTabPathName );
     if ( this.highResTabReader.hasError() ) {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening High Resolution Table  " + highResTabPathName );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(Constants.ERR_OPEN_HIGHRES_TABLE) );
          logger.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, "Error opening Low Resolution Table " +highResTabPathName);
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(Constants.ERR_OPEN_HIGHRES_TABLE);
     }

     // load tech param x-ref table into object techTable
     this.techTable = new TechnicalTable( this.technicalFilePathname );
     if ( this.techTable.hasError() ) {
       try {
         logger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening technical file: " + this.technicalFilePathname );
         logger.write( this.getClass().getName(), null, 
                       "End processing " + msgPathname + " with code=" + Integer.toString(Constants.ERR_OPEN_TECH_PARAM_FILE) );
         logger.close();
       }
       catch (IOException e1) {}
       try {
         errorLogger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening technical file: " + this.technicalFilePathname );
         errorLogger.close();
       }
       catch (IOException e1) {}
       System.exit(Constants.ERR_OPEN_TECH_PARAM_FILE);
     }


     // load mission configuration param x-ref table into object confTable
     this.confTable = new MissionConfigTable( this.confTableFilePathname );
     if ( this.confTable.hasError() ) {
       try {
         logger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening mission config file: " + this.confTableFilePathname );
         logger.write( this.getClass().getName(), null, 
                       "End processing " + msgPathname + " with code=" + Integer.toString(Constants.ERR_OPEN_CONFIG_PARAM_FILE) );
         logger.close();
       }
       catch (IOException e1) {}
       try {
         errorLogger.write( this.getClass().getName(), this.phyFilename, 
                       "Error opening mission config file: " + this.confTableFilePathname );
         errorLogger.close();
       }
       catch (IOException e1) {}
       System.exit(Constants.ERR_OPEN_CONFIG_PARAM_FILE);
     }
     if (debugOn) {
       for (int i=0; i<=27; i++) 
         System.out.println( "M2PM D CP i: " + i + " confTable: " + this.confTable.getMsgConfParamName(i)  + "   " + 
                            this.confTable.getPhyConfParamName(i));
     } 
     
     // open input file
     if(debugOn) System.out.println( "M2PM D OP open inp: " + msgPathname);
     try {
       inpReader = new FileReader( msgPathname );
       buffReader = new BufferedReader( inpReader );
     }
     catch ( IOException e )
     {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error opening msg file: " + msgPathname );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                   " with code=" + Integer.toString(Constants.ERR_OPEN_MSG_FILE) );
          logger.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, 
                             "Error opening msg file: " + msgPathname );
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(Constants.ERR_OPEN_MSG_FILE);
     }
     // open phy file
     if(debugOn) System.out.println( "M2PM D OP open out: " + phyPathname);
     try {
       outWriter = new FileWriter( phyPathname );
       buffWriter = new BufferedWriter( outWriter );
       try {
         logger.write( this.getClass().getName(), null, "File " + phyPathname + " created. " );
       }
       catch (IOException e1) {}
     }
     catch ( IOException e )
     {
        try {
          logger.write( this.getClass().getName(), this.phyFilename, "Error creating phy file: " + this.phyPathname );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                   " with code=" + Integer.toString(Constants.ERR_CREATE_PHY_FILE) );
          logger.close();
          buffReader.close();
          inpReader.close();
        }
        catch (IOException e1) {}
        try {
          errorLogger.write( this.getClass().getName(), this.phyFilename, 
                             "Error creating phy file: " + this.phyPathname  );
          errorLogger.close();
        }
        catch (IOException e1) {}
        System.exit(Constants.ERR_CREATE_PHY_FILE);
     }


   }
   
 /**********************************************************************
     Get all the meta file info needed for the msg to phy process.
     From Msg2Phy_Ir.java -CA
   ***********************************************************************/  
   void processMetaFileInfo( BufferedWriter buffWriter ) 
                           throws IOException {
     if(debugOn) System.out.println( "M2PM D PM processMetaFileInfo_all writes out meta info");
     buffWriter.write( "INTERNAL ID NUMBER                      " + meta.getInternalIdNumber() );
     buffWriter.newLine();
     buffWriter.write( "WMO ID NUMBER                           " + meta.getWMOIdNumber() );
     buffWriter.newLine();
     buffWriter.write( "TRANSMISSION ID NUMBER                  " + meta.getTransmissionIdNumber() );
     buffWriter.newLine();
     // ...profile number
     int p1 = msgPathname.indexOf('.')+1;
     int p2 = msgPathname.lastIndexOf('.');
     this.profileNumber= Integer.parseInt(msgPathname.substring(p1, p2));
     buffWriter.write( "PROFILE NUMBER                          " + this.profileNumber );
     buffWriter.newLine();
     buffWriter.write( "INSTRUMENT TYPE                         " + meta.getInstrumentType() );
     buffWriter.newLine();
     buffWriter.write( "WMO INSTRUMENT TYPE (TABLE 1770)        " + meta.getWMOInstrumentType1770() );
     buffWriter.newLine();
     buffWriter.write( "WMO RECORDER TYPE (TABLE 4770)          " + meta.getWMORecorderType4770() );
     buffWriter.newLine();
     buffWriter.write( "POSITIONING SYSTEM                      GPS" );
     buffWriter.newLine();
     buffWriter.write( "PI                                      " + meta.getPI() );
     buffWriter.newLine();
     return;
   }


   /**********************************************************************
     Converts a signed hex number to a float (real) number. 
   ***********************************************************************/  
   float signedHexToFloat( String hex ) {
     if (hex.charAt(0) >= '0' && hex.charAt(0) <= '7')  {
       return Float.parseFloat( Integer.toString( Integer.parseInt( hex, 16 ) ) );
       // return Float.valueOf( Integer.toString( Integer.parseInt( hex, 16 ) ) );
     }
     else     
     {
       //
       // Negative number.  Let's reverse complement 2
       // 
       // 
       StringBuffer hex2 = new StringBuffer( "" ); 
       // Rest 1 to the last byte.  This might set a chain reaction.
       boolean rest;
       rest = true;
       for (int i=hex.length()-1; i>=0; i--) {
         if (rest) {
           switch (hex.charAt(i)) {
             case '0': hex2.insert( 0,  'F' ); break;
             case '1': hex2.insert( 0,  '0' ); rest = false; break;
             case '2': hex2.insert( 0,  '1' ); rest = false; break;
             case '3': hex2.insert( 0,  '2' ); rest = false; break;
             case '4': hex2.insert( 0,  '3' ); rest = false; break;
             case '5': hex2.insert( 0,  '4' ); rest = false; break;
             case '6': hex2.insert( 0,  '5' ); rest = false; break;
             case '7': hex2.insert( 0,  '6' ); rest = false; break;
             case '8': hex2.insert( 0,  '7' ); rest = false; break;
             case '9': hex2.insert( 0,  '8' ); rest = false; break;
             case 'A': hex2.insert( 0,  '9' ); rest = false; break;
             case 'B': hex2.insert( 0,  'A' ); rest = false; break;
             case 'C': hex2.insert( 0,  'B' ); rest = false; break;
             case 'D': hex2.insert( 0,  'C' ); rest = false; break;
             case 'E': hex2.insert( 0,  'D' ); rest = false; break;
             case 'F': hex2.insert( 0,  'E' ); rest = false; break;
           }
         }
         else {
           hex2.insert( 0,  hex.charAt(i) );
         }
         
       }
     
       // After resting 1 to the number, each byte will be "flipped"
       StringBuffer hex3 = new StringBuffer( "" );
       for (int i=0; i<hex2.toString().length(); i++) {
         switch (hex2.toString().charAt(i)) {
           case '0': hex3.append( 'F' ); break;
           case '1': hex3.append( 'E' ); break;
           case '2': hex3.append( 'D' ); break;
           case '3': hex3.append( 'C' ); break;
           case '4': hex3.append( 'B' ); break;
           case '5': hex3.append( 'A' ); break;
           case '6': hex3.append( '9' ); break;
           case '7': hex3.append( '8' ); break;
           case '8': hex3.append( '7' ); break;
           case '9': hex3.append( '6' ); break;
           case 'A': hex3.append( '5' ); break;
           case 'B': hex3.append( '4' ); break;
           case 'C': hex3.append( '3' ); break;
           case 'D': hex3.append( '2' ); break;
           case 'E': hex3.append( '1' ); break;
           case 'F': hex3.append( '0' ); break;
         }
       }
       return -Float.parseFloat( Integer.toString( Integer.parseInt( hex3.toString(), 16 ) ) );
       // return -Float.valueOf( Integer.toString( Integer.parseInt( hex3.toString(), 16 ) ) );
     }
   }


   /**********************************************************************
     Given a line of the msg that holds a parameter name and value, 
     return its value.
     Example:
       Given this line: QuiescentVolts=196
       Returns: 196
   ***********************************************************************/  
   String getParameterValue( String line ) {
     int pos1 = line.indexOf("=") + 1;
     int pos2 = line.length();
     if ( (pos1 == -1 ) || (pos2 < pos1) ) {
       return "";
     }
     else {
       return line.substring( pos1, pos2 ); 
     } 
   }


   /*  
      Process a line containing a mission configuration
      parameter.  For example:
      $ AscentTimeOut(479) [min]
    
      return false only when a parameter is not found in 
      the x-ref mission conf parameter table
    
   From Msg2Phy_Ir.java - CA
   */
   boolean processMissionConfiguration( String line, Logger logger ) {

     String name;
     String value;
     int p1 = line.indexOf( "(" );
     int p2 = line.indexOf( ")" );
     if (p1<1 || p2<=p1) {
       return true;
     }


     // Check for 'TimeOfDay' exception
     // (1 parameter in msg file may become 2 in phy)
     if (line.substring(2, p1).equals( "TimeOfDay" )) {  
       name="CONFIG CLOCK PRESET START CYCLE FLAG";
       value = line.substring(p1+1, p2);
       if (value.equals("DISABLE")) {
         this.nbrConfParamLines++;
         confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + "0";       
       }         
       else {
         this.nbrConfParamLines++;
         confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + "1";       
         name="CONFIG CLOCK START CYCLE (MINUTES)";
         this.nbrConfParamLines++;
         confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + value;       
       }
     }
     else {
       int index = this.confTable.indexOf( line.substring(2, p1 ) );
       if (index<0) {
         //  configuration parameter not found in table.
         return false;
       }
       value = line.substring(p1+1, p2);
       name = this.confTable.getPhyConfParamName( index );
       if (debugOn) System.out.println( "M2PM D WR My param msg: " + line.substring(2, p1 ) + " phy: " + name + " value: " + value );
       this.nbrConfParamLines++;
       confParamLines[ this.nbrConfParamLines ] = Utils2.padRight(name, 40) + value;       
     }
     return true;     
   }



   /*
      process msg file segments like this one:
      ParkDescentPCnt=5
      ParkDescentP[0]=5
      ParkDescentP[1]=37
      ParkDescentP[2]=85
      ParkDescentP[3]=86
      ParkDescentP[4]=87

      returns true if the number of drift descent values declared in first line
      (in the example 5) is equal to the valid number of values found in rest
      of the segment.
   */
   boolean processParkDescent( String line, BufferedReader buffReader )
                throws IOException {

     String value = getParameterValue( line );
     if (value.equals("") ) {
       return false;
     }

     int n = Integer.parseInt( getParameterValue( line ) );
     insertOrUpdateTechParameter( "DRIFT DESCENT PCNT                      ", Integer.toString( n ) );
     
     // if there aren't any values next line is not added.
     if ( n==0 ) {
       return true;
     }
     
     StringBuffer pDescentValues = new StringBuffer( "");
     for (int i=1; i<=n; i++ ) {
       try {
         line = buffReader.readLine();
         if (line.startsWith("ParkDescentP[")) {
           int test_is_int = Integer.parseInt( getParameterValue( line ) );
           pDescentValues.append( getParameterValue( line )+"0" );
           pDescentValues.append( " " );
         }
       }
       catch (Exception e) {}
     } 
     insertOrUpdateTechParameter( "DRIFT DESCENT P                         ", pDescentValues.toString() );

     // checking the number of values is equal to the number of values expected.      
     StringTokenizer tok = new StringTokenizer(pDescentValues.toString()); 
     int actual_count = tok.countTokens(); 
     return (actual_count == n);

   }

//---------------
   /********************************************************************************
     Process a line that precedes the low resolution (Drift) section on the msg file. 
     Examples:
     $ Profile terminated: Sat Sep 17 00:01:20 2005
     $ Profile 6102.054 terminated: Wed Feb 24 08:18:46 2010
     From Msg2Phy_Ir.java - CA
   *********************************************************************************/  
   void processBeforeLowResolution( String line )
                throws java.text.ParseException {
     int pos = line.indexOf(':');
     
     SimpleTimeZone tz = new SimpleTimeZone(0, "Out Timezone");             
     TimeZone.setDefault(tz);   
     
     if (pos>0) {
       pos = pos + 6; 
       if (pos<(line.length()-19) ) {
         SimpleDateFormat formatter = new SimpleDateFormat("MMM dd HH:mm:ss yyyy") ;
         Date date = (Date)formatter.parse(line.substring( pos, pos+20 )); 
         formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss") ;
         this.beforeProfLine1 = "    +99.999   +999.999 " + 
                                formatter.format(date) +
                                "   000   0     0";
       }
     }
   }




   /********************************************************************************
     Process a line that follows the low resolution (Drift) section on the msg file. 
   *********************************************************************************/  
   void processAfterLowResolution( String line ) 
                throws java.text.ParseException {
     if (line.startsWith( "# Attempt to get GPS fix failed" )) {
       return;
     }
     if (line.startsWith( "# Ice-cap evasion" )) {
       return;
     }
     if (line.startsWith( "# Leads or break-up" )) {
       return;
     }
     try {
       SimpleDateFormat formatter = new SimpleDateFormat("MMM dd yyyy HH:mm:ss") ;
       Date date = (Date)formatter.parse(line.substring( 2, 22 )); 
       formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss") ;
       this.beforeProfLine2[0] = "    +99.999   +999.999 " + 
                              formatter.format(date) +
                              "   000   0     0";
       
     //System.out.println(" this.beforeProfLine2 = "+this.beforeProfLine2[0]);
     String NSampleLine = line.substring(42);
     int openBracketPosition = NSampleLine.indexOf('[');
     int closeBracketPosition = 0;
     if (openBracketPosition > 0) 
       closeBracketPosition = NSampleLine.indexOf(']');
       NSamples = (NSampleLine.substring(openBracketPosition+1,closeBracketPosition));
       //System.out.println("NSamples = "+NSamples);
    

          
     }
     catch (Exception e) {
     }
   }


   /**********************************************************************
     Take a line from the msg file and based on its content determines
     whether it is a HighResolution one. 
   ***********************************************************************/  
   boolean isHighResolution(String line) {
     if (line.length()<14) {
       return false;
     }
     for (int i=0; i<14; i++ ) {
       char c = line.charAt(i);
       if ( ! (  (c >= '0' && c<='9')  ||
                 (c >= 'A' && c<='F') ) )  {
         return false;
       }
     }
     return true;
   }


   /**********************************************************************
     Process a High Resolution (profile) line read from the msg file. 
   ***********************************************************************/  
   boolean processHighResolution( String line, String highResLen ) {
    if (debugOn){ 
    System.out.println("highResLen in process"+highResLen);}
    boolean lineLengthOK = true;
     // getting 'factor', that is, how many times this line should be 
     // repeated in phy file.
     int factor;
     int openBracketPosition = line.indexOf('[');
     int closeBracketPosition ;
     if (openBracketPosition > 0) {
       closeBracketPosition = line.indexOf(']'); 
       factor = Integer.parseInt(line.substring(openBracketPosition+1, closeBracketPosition));
       line = line.substring( 0, openBracketPosition );
     }
     else {
       factor = 1; 
     }

     Integer lineLength = line.length();
     if ( lineLength != Integer.parseInt(highResLen) ) {
     if (debugOn) System.out.println("line is less than the prefered length");
        return false;
     }
    
     // when lines is full of zeros, do nothing
     if (lineLengthOK) {
      String pattern = "^[0]*$";
       if (line.matches(pattern))
	{
         return true;
	}
     }

     // parse actual data (temp, pres, psal)
     double p, t, s;
     int ofreq=0;
       //calculating the p,t,s values using the high res table 
       p = Double.parseDouble( Integer.toString( Integer.parseInt( line.substring(0, (Integer.parseInt(lhmHighMap.get("p").get(2).toString()))), 16 ) ) )*
           Double.parseDouble( meta.getPresCoefficient()) + Double.parseDouble(meta.getPresOffset());
       t = signedHexToFloat( line.substring((Integer.parseInt(lhmHighMap.get("p").get(2).toString())), (Integer.parseInt(lhmHighMap.get("t").get(2).toString()))) ) * Double.parseDouble(meta.getTempCoefficient()) + 
           Double.parseDouble(meta.getTempOffset());
       s = Double.parseDouble( Integer.toString( Integer.parseInt( line.substring((Integer.parseInt(lhmHighMap.get("t").get(2).toString())), (Integer.parseInt(lhmHighMap.get("s").get(2).toString()))), 16 ) ) ) *
           Double.parseDouble(meta.getSalCoefficient())  + Double.parseDouble(meta.getSalOffset());
       ofreq_exists = lhmHighMap.containsKey("ofreq");
      if(ofreq_exists){
       ofreq = Integer.parseInt( line.substring((Integer.parseInt(lhmHighMap.get("s").get(2).toString())),16 ), 16 ) ;
       }
 
      equationsHigh = highResTabReader.getEquations();

     for (int i=1; i<=factor; i++) {
       nbrProfile1Lines++;
       prof1Pres[nbrProfile1Lines] = p;
       prof1Temp[nbrProfile1Lines] = t;
       prof1Psal[nbrProfile1Lines] = s;
      if(ofreq_exists){	
       prof1Ofreq[nbrProfile1Lines]=ofreq;
       if(equationsHigh.get(0).toString().equals("equation_getOxygen_SBE43F")){
	
	  if( ofreq != 0 )
          prof1O2[nbrProfile1Lines]    = O2_all.getOxygen_SBE43F( p, t, s, ofreq,this.meta );
	  else
          prof1O2[nbrProfile1Lines]    = Constants.fillValueTemp; 

          if(debugOn) System.out.println("Oxygen value for HRes" +p+","+t+","+s+","+ofreq+","+prof1O2[nbrProfile1Lines]);

       }
      }


     }

     return true;
   }
   
   /**********************************************************************
     Process a Low Resolution (Park Sample) line read from the msg file. 
   ***********************************************************************/  
   void processLowResolution( String line ) {
    
    if(debugOn) System.out.println("I am in processLOWResolution");
     if (line.indexOf("(Park Sample)") >= 0 ) {
       // String "(Park Sample)" found in line; do nothing.
       return;
     }
     
     StringTokenizer tk = new StringTokenizer(line);
    
       if(debugOn)System.out.println("equationCount == "+equationCount);

     //   1.a check whether the line is complete.
     if(tk.countTokens()!= (lhmLowHeaders.size()))
     {
	//msgFileIncomplete = true; 
       return;
      }
     
     
     //   1.b Read data
     try {
       int count_get = 0;
       this.headersCount = 0; 
       String value;
	 
       //Increment Row Count
       nbrProfile2Lines++;
       
       while(tk.countTokens()>0)
       {
           value = tk.nextElement().toString();
             if (value.equals("nan")) 
                lowResProfData[nbrProfile2Lines][headersCount] = Constants.fillValuePress;
             else
                lowResProfData[nbrProfile2Lines][headersCount] = Double.parseDouble(value);
         if(debugOn)
	System.out.println("lowres Pressure data" +lowResProfData[nbrProfile2Lines][0]);

		//Increment Column Count
		this.headersCount++;
	}
            
     }
      catch(Exception e){
      System.out.println("Exception ="+e.getMessage());
     }
}

   public void calcO2_lowRes(){
	
   //Get the Map from lowResTabReader
   LinkedHashMap<String,List> lhm_map = new LinkedHashMap<String,List>();
   lhm_map = lowResTabReader.getMap();

   //Get names of the equations in the Low Res Table
   equations = lowResTabReader.getEquations();
   equationsHigh = highResTabReader.getEquations();
   
   List forParameters = new ArrayList();
   double o2 = 0.0;
   double p ,t,s,phase63,t63,bphase;
   int ofreq;
   String doxy = "";
   String actual_eqn_name=" ";
   

if(nbrProfile2Lines != 0){
    //Calculate O2 
   for(int i=0; i<equationCount; i++){


             for (int j=1;j<=nbrProfile2Lines;j++){
	     if(equations.get(i).toString().startsWith("equation_getOxygen_Optode4330"))
	     {
	       //variable actual_eqn_name is used to get the exact eqn name in case of 2 same sensors
	       // Ex: equation_getOxygen_Optode4330 or equation_getOxygen_Optode43302
	
	        actual_eqn_name=equations.get(i).toString();	
		// System.out.println("actual_eqn_name = "+ actual_eqn_name);
		forParameters = lowResTabReader.getEquationParameters(actual_eqn_name);
		//doxy=lowResTabReader.getDOXY("equation_getOxygen_Optode4330");
		doxy=lowResTabReader.getFirstValue(actual_eqn_name);

		p = lowResProfData[j][lhmLowHeaders.get(forParameters.get(0).toString())];
		t = lowResProfData[j][lhmLowHeaders.get(forParameters.get(1).toString())];
		s = lowResProfData[j][lhmLowHeaders.get(forParameters.get(2).toString())];
		tphase = lowResProfData[j][lhmLowHeaders.get(forParameters.get(3).toString())];
		topt = lowResProfData[j][lhmLowHeaders.get(forParameters.get(4).toString())];
		
		 //System.out.println("pts values "+p+" " +t+" " +s+" " +tphase+" " +topt+" " +doxy);

		if(tphase != Constants.fillValuePress && topt != Constants.fillValuePress && p != Constants.fillValuePress){
		  o2 = O2_all.getOxygen_Optode4330(p,t,s,tphase,topt,this.meta,doxy);
		  //System.out.println("pts values "+p+" " +t+" " +s+" " +tphase+" " +topt +" "+o2);
		  this.lowResProfData[j][this.headersCount]=o2;
		}
		else 
		 this.lowResProfData[j][this.headersCount]=Constants.fillValuePress;

		  //System.out.println("pts values "+p+" " +t+" " +s+" " +tphase+" " +topt +" "+o2);

	     }
	     if(equations.get(i).toString().equals("equation_getOxygen_Optode3830"))
	     {
	        actual_eqn_name=equations.get(i).toString();	
		forParameters = lowResTabReader.getEquationParameters(actual_eqn_name);
		//doxy=lowResTabReader.getDOXY("equation_getOxygen_Optode3830");
		doxy=lowResTabReader.getFirstValue(actual_eqn_name);
		
		p = lowResProfData[j][lhmLowHeaders.get(forParameters.get(0).toString())];
		t = lowResProfData[j][lhmLowHeaders.get(forParameters.get(1).toString())];
		s = lowResProfData[j][lhmLowHeaders.get(forParameters.get(2).toString())];
		bphase = lowResProfData[j][lhmLowHeaders.get(forParameters.get(3).toString())];
		topt = lowResProfData[j][lhmLowHeaders.get(forParameters.get(4).toString())];
		 // System.out.println("forParameters for equation_getOxygen_Optode3830 "+forParameters);
		//  System.out.println("pts values "+p+" " +t+" " +s+" " +bphase+" " +topt +" "+o2);

		if(bphase != Constants.fillValuePress && topt != Constants.fillValuePress &&  p != Constants.fillValuePress && t != Constants.fillValuePress && s != Constants.fillValuePress){

		  //System.out.println("forParameters for equation_getOxygen_Optode3830 "+forParameters);
		  o2 =O2_all.getOxygen_Optode3830(p,t,s,bphase,topt,this.meta,doxy);
		  //System.out.println("pts values "+p+" " +t+" " +s+" " +bphase+" " +topt +" "+o2);
		  this.lowResProfData[j][this.headersCount]=o2;
		}
		else
		  this.lowResProfData[j][this.headersCount]=Constants.fillValuePress;

	     }
	     if(equations.get(i).toString().equals("equation_getOxygen_SBE63"))
	     {
	        actual_eqn_name=equations.get(i).toString();	
		forParameters = lowResTabReader.getEquationParameters(actual_eqn_name);
		p = lowResProfData[j][lhmLowHeaders.get(forParameters.get(0).toString())];
		t = lowResProfData[j][lhmLowHeaders.get(forParameters.get(1).toString())];
		s = lowResProfData[j][lhmLowHeaders.get(forParameters.get(2).toString())];
		phase63 = lowResProfData[j][lhmLowHeaders.get(forParameters.get(3).toString())];
		t63 = lowResProfData[j][lhmLowHeaders.get(forParameters.get(4).toString())];
		
		//System.out.println("forParameters for equation_getOxygen_SBE63 "+forParameters);
		if(phase63 != Constants.fillValuePress && t63 != Constants.fillValuePress && p != Constants.fillValuePress && t != Constants.fillValuePress && s != Constants.fillValuePress ){
		  o2 = O2_all.getOxygen_SBE63(p,t,s,phase63,t63,this.meta);
		  //System.out.println("pts values "+p+" " +t+" " +s+" " +phase63+" " +t63 +" "+o2);
		  this.lowResProfData[j][this.headersCount]=o2;
		}
		else
		  this.lowResProfData[j][this.headersCount]=Constants.fillValuePress;
		

	     }
	     if(equations.get(i).toString().equals("equation_getOxygen_SBE43F"))
	     {
	        actual_eqn_name=equations.get(i).toString();	
		forParameters = lowResTabReader.getEquationParameters(actual_eqn_name);
		p = lowResProfData[j][lhmLowHeaders.get(forParameters.get(0).toString())];
		t = lowResProfData[j][lhmLowHeaders.get(forParameters.get(1).toString())];
		s = lowResProfData[j][lhmLowHeaders.get(forParameters.get(2).toString())];
		ofreq = (int)(lowResProfData[j][lhmLowHeaders.get(forParameters.get(3).toString())]);
		if(ofreq != 0 && p != Constants.fillValuePress && t != Constants.fillValuePress && s != Constants.fillValuePress ){
          	   o2    = O2_all.getOxygen_SBE43F( p, t, s, ofreq,this.meta );
		   this.lowResProfData[j][this.headersCount]=o2;
		}
		else{
		     this.lowResProfData[j][this.headersCount]=Constants.fillValueTemp;
		}
          	//System.out.println("Oxygen value for SBE43F " +p+","+t+","+s+","+ofreq+","+o2);
             }


	}
		headersCount++;
   
    }
  } 

 }
   
  /**********************************************************************
     Process a High Resolution (profile) line read from the msg file. 

     returns true if no errors in line
              false if the length of the line is not enough to hold the
                    expected data
   ***********************************************************************/  
   boolean processHighResolution( String line, boolean printNumberValuesAveraged )
                               throws Exception {

     int factor; // how many times this line is repeated
     int openBracketPosition = line.indexOf('[');
     int closeBracketPosition ;
     if (openBracketPosition > 0) {
       closeBracketPosition = line.indexOf(']'); 
       factor = Integer.parseInt(line.substring(openBracketPosition+1, closeBracketPosition));
       line = line.substring( 0, openBracketPosition );
     }
     else {
       factor = 1; 
     }
     if (line.equals("00000000000000")) {
       // nothing to do if there is no data in the line.
       return true; 
     }

     if (line.length()<14) {
       // line is incomplete; therefore file is incomplete
       return false;
     }

     float temp = signedHexToFloat( line.substring(4, 8) );
     temp = temp * Float.parseFloat(meta.getTempCoefficient()) + Float.parseFloat(meta.getTempOffset());
     float pres = Float.parseFloat( Integer.toString( Integer.parseInt( line.substring(0, 4), 16 ) ) );
     pres = pres * Float.parseFloat(meta.getPresCoefficient()) + Float.parseFloat(meta.getPresOffset());
     float psal = Float.parseFloat( Integer.toString( Integer.parseInt( line.substring(8, 12), 16 ) ) );
     psal = psal * Float.parseFloat(meta.getSalCoefficient())  + Float.parseFloat(meta.getSalOffset());
     int cnts = Integer.parseInt( line.substring(12,14), 16 );

     DecimalFormat presFormat = new DecimalFormat("0.00");
     DecimalFormat tempFormat = new DecimalFormat("0.0000");
     DecimalFormat psalFormat = new DecimalFormat("####0.0000");

     for (int i=1; i<=factor; i++) {
        nbrHighResolutionLines++;
        if (printNumberValuesAveraged) {
          profDataLines[ nbrHighResolutionLines ] = Utils2.padLeft(presFormat.format(pres), 8) + "  " +
                                              Utils2.padLeft(tempFormat.format(temp), 6) + " " +
                                              Utils2.padLeft(psalFormat.format(psal), 8) + 
                                              Utils2.padLeft(Integer.toString(cnts),  4) +
                                              " 0" ; // checksum 
        }
        else {
          profDataLines[ nbrHighResolutionLines ] = Utils2.padLeft(presFormat.format(pres), 8) + "  " +
                                              Utils2.padLeft(tempFormat.format(temp), 6) + " " +
                                              Utils2.padLeft(psalFormat.format(psal), 7) + 
                                              " 0" ; // checksum 
        }

     }
     return true;
   }

   /**********************************************************************
     If a low resolution line has:
     p, tphase, t_optode but t and s weren't recorded they will be
     interpolated based on the two high resolution lines that bracket p.
   ***********************************************************************/
   void interpolateTempPSal()
   {
    
     t_interpolated = new boolean[nbrProfile2Lines+1];
     s_interpolated = new boolean[nbrProfile2Lines+1];

     for (int i=1; i<=nbrProfile2Lines; i++) {

       
       // temperature
       if (lowResProfData[i][0]!=Constants.fillValuePress && lowResProfData[i][1]==Constants.fillValueTemp){
         for (int j=1; j<nbrProfile1Lines;  j++)
           if (lowResProfData[i][0]>=prof1Pres[j] && lowResProfData[i][0]<=prof1Pres[j+1])
             if ( prof1Pres[j]!=Constants.fillValuePress && prof1Pres[j+1]!=Constants.fillValuePress &&
                  prof1Temp[j]!=Constants.fillValueTemp  && prof1Temp[j+1]!=Constants.fillValueTemp ) {

               lowResProfData[i][1] = Utils2.linearInterpolation( prof1Pres[j], lowResProfData[i][0], prof1Pres[j+1],
                                                          prof1Temp[j],              prof1Temp[j+1] );

		 t_interpolated[i] = true;
   		 t_interpolation_exists = true;
		
             }
	    else
	         t_interpolated[i] =false;
          }

       // salinity
       if (lowResProfData[i][0]!=Constants.fillValuePress && lowResProfData[i][2]==Constants.fillValuePSal){
         for (int j=1; j<nbrProfile1Lines;  j++)
           if (lowResProfData[i][0]>=prof1Pres[j] && lowResProfData[i][0]<=prof1Pres[j+1])
             if ( prof1Pres[j]!=Constants.fillValuePress && prof1Pres[j+1]!=Constants.fillValuePress &&
                  prof1Psal[j]!=Constants.fillValuePSal  && prof1Psal[j+1]!=Constants.fillValuePSal ) {
               lowResProfData[i][2]=Utils2.linearInterpolation( prof1Pres[j], lowResProfData[i][0], prof1Pres[j+1],
                                                        prof1Psal[j],              prof1Psal[j+1] );
		 s_interpolated[i]=true;
   		 s_interpolation_exists = true;
             }
	     else
	         s_interpolated[i] = false;
        }
     }
    
    // Looping for the second time to interpolate any missing Low Resolution 
    // Ex:1961_0116_018 
     for (int i=2; i<=nbrProfile2Lines-1; i++) { 
        // temperature
            if (lowResProfData[i][0]!=Constants.fillValuePress && lowResProfData[i][1]==Constants.fillValueTemp){ 
	    //if value above and below in lowres profile are not fill value - use them for interpolation 
	          
		  lowResProfData[i][1] = Utils2.linearInterpolation( lowResProfData[i-1][0], lowResProfData[i][0], lowResProfData[i+1][0],
	                                                              lowResProfData[i-1][1],           lowResProfData[i+1][1] ); 
                   t_interpolated[i] = true;
	           t_interpolation_exists = true; 
              }
        // salinity
	    if (lowResProfData[i][0]!=Constants.fillValuePress && lowResProfData[i][2]==Constants.fillValuePSal){
	     //if value above and below in lowres profile are not fill value - use them for interpolation 
	     	     lowResProfData[i][2]=Utils2.linearInterpolation( lowResProfData[i-1][0], lowResProfData[i][0], lowResProfData[i+1][0],
		                                                        lowResProfData[i-1][2],         lowResProfData[i+1][2] ); 
                     s_interpolated[i]=true;
	             s_interpolation_exists = true;
	       } 

     } 					    



   }

   private void printOneProfileLine( int i_1, int i_2 ,int j, int i_3)
   {
     StringBuffer s = new StringBuffer( "" );
     //System.out.println("I am in TSO4 printOneProfileLine");
     //printOneProfileLine( -1, i-d_2, i,d_3);
     int pos;
    

     if(debugOn) System.out.println( "TSO4 D POFL beg i_1,i_2: " + i_1 + " " + i_2);
     if(i_3>0){
        List lineGen = lowResTabReader.getLinegen();
        int headerSize_high = highResTabReader.getHeadersSize();

       	for(int i=0; i<lineGen.size(); i++)
	{

       	 if(lowResProfData[j][i]==Constants.fillValuePress)
	 { 
       	   s.append( Utils2.padLeft(dec2Format.format(lowResProfData[j][i]), 10) ); s.append( " " );
	  }
         else
	 {
       	   s.append( Utils2.padLeft(dec4Format.format(lowResProfData[j][i]), 10) ); s.append( " " );
	  }
	}
	//If ofreq exists   
	if(ofreq_exists){
       	  
           //ofreq Positions in High Res and Low Res Tables 
           ofreqPos_high = highResTabReader.getOfreqPositionLowRes();
           ofreqPos_low = lowResTabReader.getOfreqPositionLowRes();
	   o2_position_high = highResTabReader.getEquationPositionHighRes();
	    
	  // to get the position in the array subtract 1
	   o2_position_high= o2_position_high-1;
	   ofreqPos_high= ofreqPos_high-1;
	   o2_position_low= headersCount - 1 ;
	   if(lowResProfData[j][o2_position_low] != Constants.fillValueTemp){
              if(profile_overlap){
               //  if(lowResTabReader.getEquations().size()==2){
	       //     s.append( Utils2.padLeft(dec4Format.format(lowResProfData[j][o2_position_low-1]), 10) ); s.append( " " );
               //  }
               //   else if(lowResTabReader.getEquations().size()==1){
	       //        s.append( Utils2.padLeft(dec4Format.format(lowResProfData[j][o2_position_low]), 10) ); s.append( " " );
	       // }
                  if(equationsHigh.get(0).toString().equals("equation_getOxygen_SBE43F")){
	  	      if( lowResProfData[j][3] != 0 ){
                          o2    = O2_all.getOxygen_SBE43F( lowResProfData[j][0],lowResProfData[j][1],lowResProfData[j][2],lowResProfData[j][3],this.meta );}
                          
	              else{
                          o2    = Constants.fillValueTemp; }
                  }
		   s.append( Utils2.padLeft(dec4Format.format(o2), 10) ); s.append( " " );

                    
		}
             else{
	          s.append( Utils2.padLeft(dec4Format.format(lowResProfData[j][o2_position_low]), 10) ); s.append( " " );
		}
	   }
	   else{
	   s.append( Utils2.padLeft(dec1Format.format(lowResProfData[j][o2_position_low]), 10) ); s.append( " " );
	   }

	   }
     }
     // high resolution part
     if (i_1>0) {
       if(debugOn)System.out.println("i_1 in printOne funciton = "+i_1);
       if (prof1Pres[i_1]==Constants.fillValuePress) {
          s.append( Utils2.padLeft(dec1Format.format(prof1Pres[i_1]), 10) ); s.append( " " );}
       else
          s.append( Utils2.padLeft(dec4Format.format(prof1Pres[i_1]), 10) ); s.append( " " );
       if (prof1Temp[i_1]==Constants.fillValueTemp) {
         s.append( Utils2.padLeft(dec1Format.format(Constants.fillValueTemp), 10) ); s.append( " " ); }
       else
         s.append( Utils2.padLeft(dec4Format.format(prof1Temp[i_1]), 10) ); s.append( " " );
       if (prof1Psal[i_1]==Constants.fillValuePSal) {
         s.append( Utils2.padLeft(dec1Format.format(Constants.fillValuePSal), 10) ); s.append( " " ); }
       else
         s.append( Utils2.padLeft(dec4Format.format(prof1Psal[i_1]), 10) ); s.append( " " );
             
         
       if(ofreq_exists)
       {
	  if(prof1Ofreq[i_1]==Constants.fillValueOFreq)
	  {
              s.append( Utils2.padLeft(dec1Format.format(Constants.fillValueOFreq), 10) ); s.append( " " ); 
              s.append( Utils2.padLeft(dec1Format.format(Constants.fillValueTemp), 10) ); s.append( " " ); 
	  }
	  else
	  {
              s.append( Utils2.padLeft(dec4Format.format(prof1Ofreq[i_1]), 10) ); s.append( " " );
	      if(prof1Ofreq[i_1]==0.0){
              s.append( Utils2.padLeft(dec1Format.format(prof1O2[i_1]), 10) ); s.append( " " );}
	      else{
              s.append( Utils2.padLeft(dec4Format.format(prof1O2[i_1]), 10) ); s.append( " " );}
	  }
      }
     
    }
   //EX: 1901_5055_077 and 4430_7661_034  4069_7593_021
   else if (hrProfile_exists == false && display_LRProfile_separate){

        s.append( dec1Format.format(Constants.fillValuePress) ); s.append( " " );
	s.append( dec1Format.format(Constants.fillValueTemp) ); s.append( " " );
	s.append( dec1Format.format(Constants.fillValuePSal) ); s.append( " " );
        if(ofreq_exists)
	s.append( dec1Format.format(Constants.fillValueOFreq) ); s.append( " " );
   }
     // low resolution part
     if (i_2>0) {
       	for(int i=0 ; i< headersCount; i++)
	{
		if(ofreqPos_low != 0 && !(display_LROfreq) && (i == (ofreqPos_low-1) ) )
			continue;

		if(lowResTabReader.getNaNPositions().contains(String.valueOf(i)))
		        continue;
		
       		if(lowResProfData[i_2][i]==Constants.fillValuePress){ 
       	   		
			s.append( Utils2.padLeft(dec1Format.format(lowResProfData[i_2][i]), 10) ); s.append( " " );
		        	
			if( ((pos_interp_t2-1) == (highResHeader_count+i)) ||((pos_interp_s2-2) == (highResHeader_count+i)) && ((t_interpolation_exists) || (s_interpolation_exists))) 
			{
			  s.append( Utils2.padLeft("9", 10) ); s.append( " " );
			}
			
		}
        	else{

			s.append( Utils2.padLeft(dec4Format.format(lowResProfData[i_2][i]), 10) ); s.append( " " );
			if( (pos_interp_t2-1)== (highResHeader_count+i) && (t_interpolation_exists) )
			{
			  if( t_interpolated[i_2] )
			  {
			   s.append( Utils2.padLeft("8", 10) ); s.append( " " );
			  }
			  else
			  {
			   //s.append( Utils2.padLeft("9", 10) ); s.append( " " );
			   s.append( Utils2.padLeft("0", 10) ); s.append( " " );
			  }
			} 
			if( (pos_interp_s2-2)== (highResHeader_count+i) &&(s_interpolation_exists))
			{
			  if( s_interpolated[i_2] )
			  {
			   s.append( Utils2.padLeft("8", 10) ); s.append( " " );
			  }
			  else
			  {
			   //s.append( Utils2.padLeft("9", 10) ); s.append( " " );
			   s.append( Utils2.padLeft("0", 10) ); s.append( " " );
			  }
			} 
		}
        }
     }
     else {

         //print only if LR profile has to be displayed separate
         
        if(display_LRProfile_separate){
		for(int i=0 ; i< headersCount; i++)
		{
		        if(ofreqPos_low != 0 && !(display_LROfreq) && (i == (ofreqPos_low-1) ) )
			continue;
		        
		        if(lowResTabReader.getNaNPositions().contains(String.valueOf(i)))
		        continue;
			
			s.append( Utils2.padLeft(dec1Format.format(Constants.fillValueTemp), 10) ); s.append( " " );
			if( ((pos_interp_t2-1) == (highResHeader_count+i)) ||((pos_interp_s2-2) == (highResHeader_count+i)) &&((t_interpolation_exists) || (s_interpolation_exists)) ) 
			{
			   s.append( Utils2.padLeft("9", 10) ); s.append( " " );
			}
		
	        }
	}
   }
    // s.append( " 0");  // checksum
     try {
       if(debugOn) System.out.println( "TSO4 D WR " + s.toString() );
       buffWriter.write( s.toString() );
       buffWriter.newLine();
     }
     catch (IOException e) {
       try {
         errorLogger.write( this.getClass().getName(), phyFilename,
                            "Error: " + e.getMessage() );
         errorLogger.close();
       }
       catch (IOException e1) {}
       System.exit(Constants.ERR_EXECUTION);
     }
     if(debugOn) System.out.println( "TSO4 D POFL end i_1,i_2: " + i_1 + " " + i_2);
   }


   private void determineProfilesOverlap(){
     
    try {
	   profile_overlap=false;
	   int max_lines = Math.max(nbrProfile1Lines, nbrProfile2Lines);
	  
           //Check if HR profile Exist 
	   if( nbrProfile1Lines > 0){
	     maxHRProfile = prof1Pres[max_lines];
             minHRProfile = prof1Pres[nbrProfile1Lines];
           }
	   else{

	     maxHRProfile = Constants.fillValuePress;
             minHRProfile = Constants.fillValuePress;
	  }
	   
           //Check if LR profile exist 
	   if(nbrProfile2Lines > 0){
 	     
	     maxLRProfile = lowResProfData[1][0];
	     minLRProfile = lowResProfData[nbrProfile2Lines][0];
	    
	    }
	    else{
	     maxLRProfile = Constants.fillValuePress;
	     minLRProfile = Constants.fillValuePress;
	     //minLRProfile = lowResProfData[nbrProfile2Lines-1][0];
	    }
	   
 	  if(debugOn){
            System.out.println("maxHRProfile in determine"+maxHRProfile);
     	    System.out.println("maxLRProfile"+maxLRProfile);
     	    System.out.println("minLRProfile"+minLRProfile);
           }
          
	  
          if(lowResTabReader.getFirstValue("p").equals(highResTabReader.getFirstValue("p"))){ 
	       display_LRProfile_separate = false;
	      }
          else{
	       display_LRProfile_separate = true;
	      }


	  if(maxHRProfile < maxLRProfile){
            if ( (maxHRProfile < minLRProfile) || (maxHRProfile >= minLRProfile) && ( (minLRProfile != Constants.fillValuePress) && (maxHRProfile != Constants.fillValuePress)) ) 
	      profile_overlap=true;
	    }
	  else if(maxHRProfile > maxLRProfile ){
            if( (maxHRProfile >= minLRProfile) &&((maxLRProfile != 0.0) && (minLRProfile != 0.0)) )
	      profile_overlap = false;	
	   }

          if(lowResTabReader.getOfreqPositionLowRes() != 0){
	   if(nbrProfile2Lines > 0){
	         
		//double minLROfreq = lowResProfData[nbrProfile2Lines-1][3];
		double minLROfreq = lowResProfData[nbrProfile2Lines-1][3];
	        double maxHROfreq = prof1Ofreq[max_lines];
	      
               
	        if(minLROfreq != Constants.fillValuePress)
	         display_LROfreq = true;
                //comment the above temporarily . The below code is to print
		// TSO3 ofreq values  
	        //if(minLROfreq != Constants.fillValuePress || maxHROfreq != Constants.fillValuePress )
	        // display_LROfreq = true;
	    }
	   }

	   buffWriter.write( "PROFILES OVERLAP                        "+profile_overlap );
           buffWriter.newLine();
    }
       catch ( IOException e )
       {
         System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), phyFilename, 
                             "IO Error: " + e.getMessage() ) );
         System.exit(Constants.ERR_IO);
       }


   }

   private void printProfileData(){
     
     int max_lines = Math.max(nbrProfile1Lines, nbrProfile2Lines);
     int d_1 = max_lines - nbrProfile1Lines;
     int d_2 = max_lines - nbrProfile2Lines;
     int d_3 =0;
     int d_4 =0;
     int lrMiniCounter = 0; 
     
     
     if(debugOn){
	     System.out.println("maxHRProfile in print Profile"+maxHRProfile);
	     System.out.println("minHRProfile in print Profile"+minHRProfile);
	     System.out.println("maxLRProfile"+maxLRProfile);
	     System.out.println("minLRProfile"+minLRProfile);
	     System.out.println("nbrProfile2Lines in printProfileData = "+nbrProfile2Lines);
     }
     
     //overlap but LR profile is not displayed separately display_LRProfile_separate = false;
     //Concatenate the Lowres AND High Res Profiles
     if((maxHRProfile < maxLRProfile ) && (maxHRProfile < minLRProfile) ){
         if (maxHRProfile != Constants.fillValuePress && minHRProfile != Constants.fillValuePress){
	d_3 = nbrProfile2Lines;
	//print only LR profile 
        for ( int i=1; i<=nbrProfile2Lines; i++ )
         printOneProfileLine( -1, -1, i,d_3);
	
	 d_3 = 0;
         //print only HR profile after LR
         for ( int i=1; i<=nbrProfile1Lines; i++ )
	 {
           printOneProfileLine( i-d_1, -1, i, d_3);
	 }
       }
     }
     
     //overlap and LR profile is displayed separately display_LRProfile_separate = true; 
     else if((maxHRProfile < maxLRProfile) && (maxHRProfile >= minLRProfile ) &&(( (minLRProfile != Constants.fillValuePress) && (maxHRProfile != Constants.fillValuePress) ))){
     //if((maxHRProfile < maxLRProfile) && (maxHRProfile >= minLRProfile ) &&(( (minLRProfile != Constants.fillValuePress) && (maxHRProfile != Constants.fillValuePress) ))){
       
	d_3 = nbrProfile2Lines;
       for(int i=1;i<=max_lines;i++)
       {
         lrMiniCounter++;
	 //lowResProfData[lrMiniCounter][1]=temperature and lowResProfData[lrMiniCounter][2] = salinity cannot be NaN in msg file 
	 //when P,T,S values from low resolution values are being printed above the shallower high resolution profile 
	 //Ex: 1961_0116_018 profile


         if( lowResProfData[lrMiniCounter][0] > maxHRProfile && lowResProfData[lrMiniCounter][1] == Constants.fillValuePress &&  lowResProfData[lrMiniCounter][2] == Constants.fillValuePress){
                  break;
           }
	 if( lowResProfData[lrMiniCounter][0] < maxHRProfile){
                  break;
	  
	 }
        }
	
	//print LR profile that is deeper than the HR profile
        for ( int j=1;j<=lrMiniCounter-1;j++ )
        printOneProfileLine( -1, -1,j,d_3);
        d_3 = 0;

        //print the HR profile
        for ( int i=1; i<=max_lines; i++ )
         printOneProfileLine( i-d_1, i-d_2, d_3, d_3);

     }
     else if((maxHRProfile > maxLRProfile) && (maxHRProfile > minLRProfile )){
     //if((maxHRProfile > maxLRProfile) && (maxHRProfile > minLRProfile )){
        
	for ( int i=1; i<=max_lines; i++ )
         printOneProfileLine( i-d_1, i-d_2, d_3, d_3);
     
     }
    
       //EX: 4430_7661_034  4069_7593_021 
       //Below if condition cannot be an else-if for the cases mentioned above
      //if(minLRProfile == Constants.fillValuePress || minLRProfile != Constants.fillValuePress){
      if (maxHRProfile == Constants.fillValuePress && minHRProfile == Constants.fillValuePress){
       
        hrProfile_exists = false;
        
	//print LR first
        for ( int i=1; i<=nbrProfile2Lines; i++ )
         printOneProfileLine( -1, i-d_2, i,d_3);

      }




   }


   /**********************************************************************
     Process a GPS Fix line read from the msg file. 
   ***********************************************************************/  
   void processGPSFix( String line ) {
     
    /* if (line.startsWith("Fix: GPS fix not available")) {
       return;
     }*/
     if (line.startsWith("Fix: GPS fix not available")) {
        hasGPSFix = false;
        iGpsFixes = new InterpolateGpsFixes(this.pathName,floatId,profileNumber,this.beforeProfLine1);

        if(iGpsFixes.getInterpolatedValues(0) == null || iGpsFixes.getInterpolatedValues(1) == null)
	{
          return;
        }

     }
    /* if (line.startsWith("Fix: GPS fix not available due to float stuck to bottom")) {
       return;
     }*/
     float latitude ;
     float longitude ;
     String longStr = "";
     String latStr = "";
     String dateStr = "";
     String timeStr ="";
     String nSatStr= "";
     DecimalFormat latLongFormat = new DecimalFormat("0.000");
     DecimalFormat posLatLongFormat = new DecimalFormat("+0.000");
     SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss") ;
     if(hasGPSFix == true){
     	StringTokenizer tok = new StringTokenizer(line); 
     	tok.nextElement().toString();    // 'Fix: ' text, useless.
        longStr = tok.nextElement().toString();  // latitude 
     	latStr = tok.nextElement().toString(); // longitude
     	dateStr = tok.nextElement().toString(); // date
     	timeStr = tok.nextElement().toString(); // time
     	nSatStr = tok.nextElement().toString(); // nbr of satelites
       // System.out.println("nSatStr = "+nSatStr);
     }
     
     StringBuffer sb;
     // first line. (only if it wasn't been created already )
     if ( this.beforeProfLine1 == null ) {
       sb = new StringBuffer("    +99.999   +999.999 " );
       sb.append(dateStr.substring(6,10));sb.append("/");
       sb.append(dateStr.substring(0,3));
       sb.append(dateStr.substring(3,5));
       sb.append(" ");
       sb.append(timeStr.substring(0,2));sb.append(":");
       sb.append(timeStr.substring(2,4));sb.append(":");
       sb.append(timeStr.substring(4,6));
       sb.append("   000   0     0");
       this.beforeProfLine1 = sb.toString();
     }
       //System.out.println("this.beforeProfLine1 = " +this.beforeProfLine1);
     if (hasGPSFix ==false){
	     sb = new StringBuffer("");
	     latitude  =  Float.parseFloat(iGpsFixes.getInterpolatedValues(0));
	     longitude =  Float.parseFloat(iGpsFixes.getInterpolatedValues(1));
	     Date date =  iGpsFixes.getDateProf();
	     if (latitude<0.00) {
	       sb.append( Utils2.padLeft( latLongFormat.format(latitude), 11 ) );
	     }
	     else {
	       sb.append( Utils2.padLeft( posLatLongFormat.format(latitude), 11 ) );
	     }
	     if (longitude<0.00) {
	       sb.append( Utils2.padLeft( latLongFormat.format(longitude), 11 ) );
	     }
	     else {
	       sb.append( Utils2.padLeft( posLatLongFormat.format(longitude), 11 ) );
	     }
	     sb.append(" ");
	     sb.append(formatter.format(date));
	     sb.append("   000   0     8");
	     this.beforeProfLine2[gpsFixCounter] = sb.toString();
	      // third line.
             sb = new StringBuffer("    +99.999   +999.999 " );
	     sb.append(formatter.format(date));
             sb.append("   000   0     0");
             this.beforeProfLine3 = sb.toString();
             //System.out.println("this.beforeProfLine3 = " +this.beforeProfLine3);


     }
     if (hasGPSFix == true){
	     // second line.
	     if(!(Integer.parseInt(nSatStr)<3)){
		     sb = new StringBuffer("");
		     latitude = Float.parseFloat( latStr );
		     longitude = Float.parseFloat( longStr );
		     if (latitude<0.00) {
		       sb.append( Utils2.padLeft( latLongFormat.format(latitude), 11 ) );
		     }
		     else {
		       sb.append( Utils2.padLeft( posLatLongFormat.format(latitude), 11 ) );
		     }
		     if (longitude<0.00) {
		       sb.append( Utils2.padLeft( latLongFormat.format(longitude), 11 ) );
		     }
		     else {
		       sb.append( Utils2.padLeft( posLatLongFormat.format(longitude), 11 ) );
		     }
		     sb.append(" ");
		     sb.append(dateStr.substring(6,10));sb.append("/");
		     sb.append(dateStr.substring(0,3));
		     sb.append(dateStr.substring(3,5));
		     sb.append(" ");
		     sb.append(timeStr.substring(0,2));sb.append(":");
		     sb.append(timeStr.substring(2,4));sb.append(":");
		     sb.append(timeStr.substring(4,6));
		     sb.append("   000");
		     sb.append(Utils2.padLeft(nSatStr,4));
		     sb.append("     0");
		     this.beforeProfLine2[gpsFixCounter] = sb.toString();
		     gpsFixCounter++;
		     //System.out.println("this.beforeProfLine2[0] = " +this.beforeProfLine2[0]);
		     // third line.
		     sb = new StringBuffer("    +99.999   +999.999 " );
		     sb.append(dateStr.substring(6,10));sb.append("/");
		     sb.append(dateStr.substring(0,3));
		     sb.append(dateStr.substring(3,5));
		     sb.append(" ");
		     sb.append(timeStr.substring(0,2));sb.append(":");
		     sb.append(timeStr.substring(2,4));sb.append(":");
		     sb.append(timeStr.substring(4,6));
		     sb.append("   000   0     0");
		     this.beforeProfLine3 = sb.toString();
		     //System.out.println("this.beforeProfLine3 = " +this.beforeProfLine3);
	    }	
	     else
	     {
		     this.beforeProfLine3 = this.beforeProfLine1;
		     //this.beforeProfLine2[0]=null;
		     if(this.beforeProfLine2 == null){
		         this.beforeProfLine2[0]=null;
		     }
	      }
     }//endOf if hasGPS==true

   }


   /**********************************************************************
     Sometimes a msg file profile holds more than one line of the same 
     parameter. 
     This function, given a parameter line will first check whether the 
     parameter has already been read in which case it updates the value; 
     otherwise it will add the line to the list of parameters
   ***********************************************************************/  
   void insertOrUpdateTechParameter( String name, String value ) {
     // first check whether ther parameter already exists
     if(debugOn) System.out.println( "M2PM D TP insertOrUpdateTechParameter for name: " + name);
     for (int i=1; i<=this.nbrTechParamLines; i++) {
       if ( name.equals( techParamLines[i].substring(0,40) ) ) {
         techParamLines[i] = name + value;
         return;
       }         
     }
     this.nbrTechParamLines++;
     techParamLines[ this.nbrTechParamLines ] = name + value;       
   }


   /***********************************************************************
    Given a value and the position of the technical parameter in the tech
    parameter list, return the calibrated value
    **********************************************************************/
     String getCalibratedValue( String value, int index ) {
     String coefficient = "0";
     String offset = "0";
     String calibratedFor = this.techTable.getCalibratedFor( index );
     if(debugOn) System.out.println("calibratedFor = "+calibratedFor+"value ="+value+"index = "+index);
     if (calibratedFor.equals( "current" )) {
       coefficient = meta.getCurrentCoefficient();
       offset = meta.getCurrentOffset();
       if ( (coefficient==null) || (offset==null) ) {
         try {
           logger.write( this.getClass().getName(), this.phyFilename, "Check current slope and offset on meta file" );
           logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                    " with code=" + Integer.toString(Constants.ERR_WRONG_META_INFO) );
           logger.close();
           buffReader.close();
           inpReader.close();
           buffWriter.close();
           outWriter.close();
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), this.phyFilename, 
                              "Check current slope and offset on meta file" );
           errorLogger.close();
         }
         catch (IOException e1) {}
         System.exit(Constants.ERR_WRONG_META_INFO);
       }
     }
     else {
     if (calibratedFor.equals( "voltage" )) {
       coefficient = meta.getVoltCoefficient();
       offset = meta.getVoltOffset();
       if ( (coefficient==null) || (offset==null) ) {
         try {
           logger.write( this.getClass().getName(), this.phyFilename, "Check voltage slope and offset on meta file" );
           logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                    " with code=" + Integer.toString(Constants.ERR_WRONG_META_INFO) );
           logger.close();
           buffReader.close();
           inpReader.close();
           buffWriter.close();
           outWriter.close();
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), this.phyFilename, 
                              "Check current slope and offset on meta file" );
           errorLogger.close();
         }
         catch (IOException e1) {}
         System.exit(Constants.ERR_WRONG_META_INFO);
       }
     }
     else {
     if (calibratedFor.equals( "vacuum" )) {
       coefficient = meta.getVacuumCoefficient();
       offset = meta.getVacuumOffset();
     }}} 

     float calValue;
     if (value.equals("0")) {
       calValue=0.0f;
     }
     else {
       calValue = (Float.parseFloat(value)*Float.parseFloat( coefficient )) + 
                         Float.parseFloat(offset);
     }

     //find out how many decimal points in coeff & offset
     //will determine how many decimal points in calibrated value.
     int dec_pts = Math.max( coefficient.length()-coefficient.indexOf('.')-1, 
                             offset.length()-offset.indexOf('.')-1 );
     if (dec_pts < 1) {
       dec_pts = 1;
     }
      
     StringBuffer strFormat = new StringBuffer( "0." );
     for (int i=1; i<=dec_pts; i++ ) {
       strFormat.append( '0');
     }
     DecimalFormat decFormat = new DecimalFormat(strFormat.toString());

     return decFormat.format( calValue );
     
   }


   /**********************************************************************
     Process a line from the msg file that holds a technical parameter
   ***********************************************************************/  
   void processTechnicalParameter( String line, Logger logger ) {

     int index;
     String name = null;

     int p1 = line.indexOf("=");
     if (p1>0) { 
        index = this.techTable.indexOf( line.substring(0, p1 ) );
        if (index==-1) {
          return;
        }
        else
          name = this.techTable.getPhyTechParamName( index );
     }
     else 
       return;  
          
     p1 = line.indexOf("=") + 1;
     int p2 = line.length();
     if ( (p1 == -1 ) || (p2 <= p1) ) {
       return;
     }
     else {
       String value = line.substring( p1, p2 ); 
       if (this.techTable.hasCalibration( index )) {
         value = getCalibratedValue( value, index );
       }
       insertOrUpdateTechParameter( name, value );
     }
   }

   /**********************************************************************
     Given a string, returns all the numbers and the decimal point (.) on 
     it
   ***********************************************************************/  
   String onlyDecNbrChars( String s ) {
     StringBuffer s2 = new StringBuffer("");
     for (int i=0; i<s.length(); i++) {
       char c = s.charAt(i);
       if ( (c >= '0' &&  c <= '9' ) || (c == '.') ) {
         s2.append( s.charAt(i) );
       }
     }
     return s2.toString();
   }


   void processParkObservations( String line ) {

      // get pressure
      int p1 = line.indexOf( " " );
      int p2 = line.indexOf( "," );
      String press = onlyDecNbrChars( line.substring( p1, p2 ) );
      int npchars = press.length();
      // if(debugOn) { System.out.println( "Msg2Phy_Ir D ParkObs length pres: " + npchars); }
      if(npchars > 16) {
         System.out.println( "Msg2Phy_Ir ParkObs pres longer than 16 (" + npchars + "), set to fillValue");
         press = Constants.fillValuePress_char;
      }
      insertOrUpdateTechParameter( "DRIFT PRESSURE PRE PROFILE (DBAR)       ", press );
      // get temperature
      p1 = line.indexOf( " ", p2 );
      p2 = line.indexOf( ",", p1 );
      String temp = onlyDecNbrChars( line.substring( p1, p2 ) );
      int ntchars = temp.length();
      // if(debugOn) { System.out.println( "Msg2Phy_Ir D ParkObs length temp: " + ntchars); }
      if(ntchars > 16) {
         System.out.println( "Msg2Phy_Ir ParkObs temp longer than 16 (" + ntchars + "), set to fillValue");
         temp = Constants.fillValuePress_char;
      }
      insertOrUpdateTechParameter( "DRIFT TEMPERATURE PRE PROFILE (DEG C)   ", temp );
      // get salinity 
      p1 = line.indexOf( " ", p2 );
      p2 = line.indexOf( ",", p1 );
      if (p2<p1) {
        p2 = line.length();
      }
      String psal = onlyDecNbrChars( line.substring( p1, p2 ) );
      int nschars = psal.length();
      // if(debugOn) { System.out.println( "Msg2Phy_Ir D ParkObs length psal: " + nschars); }
      if(nschars > 16) {
         System.out.println( "Msg2Phy_Ir ParkObs psal longer than 16 (" + nschars + "), set to fillValue");
         psal = Constants.fillValuePress_char;
      }
      insertOrUpdateTechParameter( "DRIFT SALINITY PRE PROFILE (PSU)        ", psal );
   }


   /*************************************************************************
     Write the config parameters and their values in the output phy file
   **************************************************************************/  
   void writeConfParameterBlock( BufferedWriter buffWriter ) {
     // write the missiong configuration parameters block
     if(debugOn) System.out.println( "M2PM D WR writeConfParameterBlock");
     try {
       for (int i=1; i<=this.nbrConfParamLines; i++) {
         buffWriter.write( this.confParamLines[i] );
         buffWriter.newLine();
       }
     }
     catch ( IOException e )
     {
       System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() ) );
       System.exit(Constants.ERR_IO);
     }
   }


   /*************************************************************************
     Write the technical parameters and their values in the output phy file
   **************************************************************************/  
   void writeTechParameterBlock( BufferedWriter buffWriter ) {
     // write the technical parameters block
     if(debugOn) System.out.println( "M2PM D WR writeTechParameterBlock");
     try {
       for (int i=1; i<=this.nbrTechParamLines; i++) {
         buffWriter.write( this.techParamLines[i] );
         buffWriter.newLine();
       }
     }
     catch ( IOException e )
     {
       System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), this.phyFilename, "IO Error: " + e.getMessage() ) );
       System.exit(Constants.ERR_IO);
     }
   }


   /**********************************************************
     Process a Park (Drift) line read from the msg file.
     from Msg2Phy_Ir.java - CA
   ***********************************************************/
   public void processParkPtLine( String line, int driftCnts, boolean isFirst )
   {

     if (isFirst) {
       driftPressureLine = new StringBuffer("DRIFT PRESSURE (DBAR)                   ");
       driftTemperatureLine = new StringBuffer("DRIFT TEMPERATURE (DEG C)               ");
       if (line.startsWith("ParkPts"))
       driftSalinityLine = new StringBuffer("DRIFT SALINITY (PSU)                    ");
       driftMeasurementTimeLine = new StringBuffer("DRIFT MEASUREMENT TIME (YYYYMMDDHHMMSS) ");
     }

     String year = "";
     String month = "";
     String day = "" ;
     int parkPtsTokens = 0;
     StringTokenizer tok = new StringTokenizer(line);
     if (line.startsWith("ParkPts"))
     parkPtsTokens = 10;
     else
     parkPtsTokens = 9;
     
     //System.out.println("Park Pts TOken "+parkPtsTokens);


     if (tok.countTokens()<parkPtsTokens) {
        // line is fragmented
       return;
     }
     for (int n=1; n<=parkPtsTokens; n=n+1) {
       String s = tok.nextElement().toString();
       switch (n)
       {
         case 2:
           if (s.equals("Jan")) {
             month = "01";
           } else if (s.equals("Feb")) {
             month = "02";
           } else if (s.equals("Mar")) {
             month = "03";
           } else if (s.equals("Apr")) {
             month = "04";
           } else if (s.equals("May")) {
             month = "05";
           } else if (s.equals("Jun")) {
             month = "06";
           } else if (s.equals("Jul")) {
             month = "07";
           } else if (s.equals("Aug")) {
             month = "08";
           } else if (s.equals("Sep")) {
             month = "09";
           } else if (s.equals("Oct")) {
             month = "10";
           } else if (s.equals("Nov")) {
             month = "11";
           } else if (s.equals("Dec")) {
             month = "12";
           }
           break;
         case 3: day = s; break;
         case 4: year = s; break;
         case 5: driftMeasurementTimeLine.append( year + month + day +
                                                 s.substring(0,2) +
                                                 s.substring(3,5) +
                                                 s.substring(6,8) );
                 driftMeasurementTimeLine.append( " " );
                 break;
         case 8: if ( s.indexOf("dbars")>0 ) 
                   s = s.substring(0, s.indexOf("dbars") );
                 driftPressureLine.append( s ); driftPressureLine.append( " " ); 
                 break; 
         case 9: if ( s.indexOf("C")>0 ) 
                   s = s.substring(0, s.indexOf("C") );
                 driftTemperatureLine.append( s ); driftTemperatureLine.append( " " ); 
                 break; 
         case 10: driftSalinityLine.append( s ); driftSalinityLine.append( " " ); break;
       }
     }
   }


   /*************************************************************************
     Write the column headers block to the output phy file
   **************************************************************************/  
   void writeColumnHeaderBlock( BufferedWriter buffWriter) {
     if ( nbrProfile1Lines + nbrProfile2Lines > 0 ) {
       try {
	 List highResHeaders = new ArrayList();
	 List lowResHeaders = new ArrayList();
	 highResHeaders = highResTabReader.getHeaders(); 
	 lowResHeaders =  lowResTabReader.getHeaders(); 
         Integer columnCount = 1;
         Integer totalCount = 0;
	 LinkedHashMap<String,List> lhmColumnHeaderMap = new LinkedHashMap<String,List>();
	 List headers = new ArrayList();
	 int index = 0;
	 String name ="";
	 String name_t ="";
	 String name_s ="";
         equations = lowResTabReader.getEquations();
         int matrixCount_meta = meta.getMatrixCount();
        /* matrixCount = meta.getMatrixCount();
         if(lowResTabReader.get("MATRIX2").get(0).toString().equals("NOT_AVAILABLE"))
         matrixCount++;*/
         
	 Integer nanCount=0;
	 if(debugOn)System.out.println("display_LRProfile_separate in column Header = "+display_LRProfile_separate );

         //nbrProfile2Lines != 0 TSO2 has 2 profiles and has to be displayed parallel, but 
	 //incase of 2675_5230_128  profile low res data is missing . In this case , do not print low res columns
	 //or data
	 
	 if(display_LRProfile_separate && nbrProfile2Lines != 0){ 
		 totalCount=lowResTabReader.getHeadersSize()-matrixCount;

		 //highResTabReader.size()-2 = 2 is the number of extra table contents
		 totalCount=totalCount+(highResTabReader.getHeadersSize());
		 
		 if(t_interpolation_exists) 
		 totalCount=totalCount + 1;
		 if(debugOn)System.out.println("TOTAL COUNT "+totalCount); 
		 
		 if(s_interpolation_exists) 
		 totalCount++; 
		 if(debugOn)System.out.println("TOTAL COUNT 2 "+totalCount);

		 if (!( display_LROfreq) && (lowResTabReader.getOfreqPositionLowRes() != 0))
		 totalCount = totalCount - 1 ;
		 if(debugOn)System.out.println("TOTAL COUNT 3 "+totalCount);

	 }
	 else{

		 totalCount=totalCount+(highResTabReader.getHeadersSize());
		 if(debugOn)System.out.println("TOTAL COUNT in else "+totalCount);


	 }
	 nanCount = lowResTabReader.getNaNCount();
	 totalCount = totalCount - nanCount;
	 
	 if(lhmLowMap.containsKey("ALL_NAN_IN_MSG"))
	 totalCount = totalCount - 1;
         if(debugOn)System.out.println("TOTAL COUNT after NANs "+totalCount);
         
	 
	 buffWriter.write( "NUMBER OF COLUMNS                       "+totalCount.toString() );
         buffWriter.newLine();
	 headers = highResTabReader.getHeaders();
	 lhmColumnHeaderMap = highResTabReader.getMap();

        // In case of TSO1 parallel profile does not exist
        // as high res stops before low res starts . So, if low res and high res exists 
        // print these columns only if high res != 0
        // 5048.173.msg - both exists 5048.225.msg - high res exits , no low res (Profile Overlap)
        // In case of 5048.173.msg it should skip the below if condition, where as in case of 
        // 5048.225.msg it should enter the below loop (No profile Overlap) 

        if(display_LRProfile_separate || (nbrProfile1Lines != 0 && (!(display_LRProfile_separate ) ) ) ) {
	 for(int i=0; i< highResTabReader.getHeadersSize(); i++){
	      name = headers.get(i).toString();
	      if(name.contains("NSAMPLE") || name.contains("TOTAL") ){
		continue;
	      }

             buffWriter.write( columnCount.toString()+".  COLUMN                              "+lhmColumnHeaderMap.get(name).get(0).toString()+" "+lhmColumnHeaderMap.get(name).get(1).toString() );
             buffWriter.newLine();
	     columnCount++;
	 }
           if(ofreq_exists){
	   
	   		buffWriter.write( columnCount.toString()+".  COLUMN                              "+lhmColumnHeaderMap.get(equationsHigh.get(0).toString()).get(0)+" "+lhmColumnHeaderMap.get(equationsHigh.get(0).toString()).get(1).toString() );
             buffWriter.newLine();
	     columnCount++;
	 
	 
	 }
	 highResHeader_count= columnCount;

	 }
	 
	 //Display the Low res headers only if it is necessary
	 //in case of TSO1  when lowres exists but high res does not exists, then the low res 
	 //columns have to printed Ex: 1901_5055_077
	if(display_LRProfile_separate ||( nbrProfile1Lines == 0 && (!(display_LRProfile_separate)) ) )
	{

	  headers = lowResTabReader.getHeaders();
	  lhmColumnHeaderMap = lowResTabReader.getMap();

         //nbrProfile2Lines != 0 TSO2 has 2 profiles and has to be displayed parallel, but 
	 //incase of 2675_5230_128  profile low res data is missing . In this case , do not print low res columns
	 //or data
	if(nbrProfile2Lines != 0) {
	  for(int i=0; i< lowResTabReader.getHeadersSize(); i++){
	      name = headers.get(i).toString();
	      if(debugOn){
		System.out.println("column name description  :: "+lhmColumnHeaderMap.get(name).get(0).toString());
	        System.out.println("display_LROfreq  :: "+display_LROfreq);
	      }
	         
	      if(name.contains("MATRIX")|| name.startsWith("equation") || ( (name.contains("ofreq")) && (!(display_LROfreq)) )  || name.contains("ALL_NAN_IN_MSG") || lhmColumnHeaderMap.get(name).get(0).toString().equals("NOT_AVAILABLE")  ){
		continue;
	      }
	     

	     if(columnCount <= 9)
	     {
             
	       buffWriter.write( columnCount.toString()+".  COLUMN                              "+lhmColumnHeaderMap.get(name).get(0).toString()+" "+lhmColumnHeaderMap.get(name).get(1).toString() );
	       
	       if(lhmColumnHeaderMap.get(name).get(0).toString().contains("TEMPERATURE2") && (t_interpolation_exists))
	       {
                
		buffWriter.newLine();
		name_t = headers.get(i+1).toString();
	        
		if(name_t.contains("MATRIX")|| name_t.startsWith("equation")){
		 continue;
	        }
		
		columnCount++;
	        
		if(columnCount <= 9)
	        buffWriter.write( columnCount.toString()+".  COLUMN                              INTERP_STATUS_T2" );
		else
	        buffWriter.write( columnCount.toString()+". COLUMN                               INTERP_STATUS_T2" );
		
		pos_interp_t2=columnCount;
		name_t = " ";

	       }
	       
	       if(lhmColumnHeaderMap.get(name).get(0).toString().contains("SALINITY2") && (s_interpolation_exists))
	       {
                
		buffWriter.newLine();
		name_s = headers.get(i+1).toString();
	        
		if(name_s.contains("MATRIX")|| name_s.startsWith("equation")){
		 continue;
	        }
		columnCount++;
	        if(columnCount <= 9)
	        buffWriter.write( columnCount.toString()+".  COLUMN                              INTERP_STATUS_S2" );
		else
	        buffWriter.write( columnCount.toString()+". COLUMN                              INTERP_STATUS_S2" );

		pos_interp_s2=columnCount;
		name_s = " ";

	       }
	     }
	     else
	     {
             
	       buffWriter.write( columnCount.toString()+". COLUMN                              "+lhmColumnHeaderMap.get(name).get(0).toString()+" "+lhmColumnHeaderMap.get(name).get(1).toString() );
	       if(lhmColumnHeaderMap.get(name).get(0).toString().contains("TEMPERATURE2") && (t_interpolation_exists))
	       {
		
		buffWriter.newLine();
		name_t = headers.get(i+1).toString();
	        
		 if(name_t.contains("MATRIX")|| name_t.startsWith("equation")){
		  continue;
	         }
		columnCount++;
	        buffWriter.write( columnCount.toString()+". COLUMN                               INTERP_STATUS_T2" );
		pos_interp_t2=columnCount;
                buffWriter.newLine();
                name_t = " ";
	       }

	       if(lhmColumnHeaderMap.get(name).get(0).toString().contains("SALINITY2") &&(s_interpolation_exists))
	       {
		
		buffWriter.newLine();
		name_s = headers.get(i+1).toString();
	        
		 if(name_s.contains("MATRIX")|| name_s.startsWith("equation")){
		  continue;
	         }
		
		columnCount++;
	        buffWriter.write( columnCount.toString()+". COLUMN                               INTERP_STATUS_S2" );
		pos_interp_s2=columnCount;
                name_s = " ";
	       }
	     }

           buffWriter.newLine();
	   columnCount++;
	 }
         
	 //nbrProfile2Lines != 0 TSO2 has 2 profiles and has to be displayed parallel, but 
	 //incase of 2675_5230_128  profile low res data is missing . In this case , do not print low res columns
	 //or data
        if(nbrProfile2Lines != 0){
	 for(int i=0; i< equationCount; i++){
	
	   
	   if(columnCount <= 9)
           buffWriter.write( columnCount.toString()+".  COLUMN                              "+lhmColumnHeaderMap.get(equations.get(i).toString()).get(0)+" "+lhmColumnHeaderMap.get(equations.get(i).toString()).get(1).toString() );
	   else
           buffWriter.write( columnCount.toString()+". COLUMN                              "+lhmColumnHeaderMap.get(equations.get(i).toString()).get(0)+" "+lhmColumnHeaderMap.get(equations.get(i).toString()).get(1).toString() );
	   
	   columnCount++;
           
	   buffWriter.newLine();


	}
      }
      }
     }
       }
       catch ( IOException e )
       {
         System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), phyFilename, 
                             "IO Error: " + e.getMessage() ) );
         System.exit(Constants.ERR_IO);
       }
     }
   }



   /**********************************************************************
     Process all the lines of the msg file that will be converted into a
     phy file.
   ***********************************************************************/  
   void process() 
   {

     nbrDriftLines = 0;
     nbrTechParamLines = 0; 
     nbrProfile1Lines = 0;
     nbrProfile2Lines = 0;
     byte phase = Constants.NOT_STARTED; 
     equationCount = lowResTabReader.getEquationSize();
     
     int driftCnts = 0; //  4 Parklines in input file becomes 1 line in output file
                        //  This var. keeps track of it; its values: 1,2,3,4,1,2,3,4,...

     boolean msgFileIncomplete = false;
     boolean confParameterNotFound = false;
     boolean parkPts = false; //parkPts to include the salinity into the PHY file - CA
     hasNitrate = false;
     hasPH = false;
     hasRPhase = false;

     // open output file
     try {
       outWriter = new FileWriter( phyPathname );
       buffWriter = new BufferedWriter( outWriter );
       try {
         logger.write( this.getClass().getName(), null, "File " + phyPathname + " created. " );
       }
       catch (IOException e1) {}
     }
     catch ( IOException e )
     {
        System.out.println( Utils2.dateFormatMessage( this.getClass().getName(), phyFilename, 
                            "Error creating output phy file: " + e.getMessage() ));
        try {
          logger.write( this.getClass().getName(), phyFilename, "Error creating output msg file: " + e.getMessage() );
          logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                   " with code="+Integer.toString(Constants.ERR_CREATE_PHY_FILE) );
  

          logger.close();
          buffReader.close();
          inpReader.close();
        }
        catch (IOException e1) {}
        finally {
          System.exit(Constants.ERR_CREATE_PHY_FILE);
        }
     }

     
     try
     {
      
       processMetaFileInfo( buffWriter );

       String line = buffReader.readLine();
       while (line!=null) {
         if (debugOn) System.out.println( "TSOALL D RD " + phase + " " + line );
         try {

           if (line.length()==0) {
             line = buffReader.readLine(); 
             continue;
           }
	   if(line.startsWith("ParkPts"))
	   parkPts = true;

           if (line.equals("<EOT>")) {
           }
	   else if (line.contains("GasEx")) {
                if(debugOn) System.out.println( "TSOALL D GasEx - line: " + line );
	   }
	   else if (line.startsWith("Gtd")) {
	        if(debugOn) System.out.println( "TSOALL D Gtd - line: " + line );
	   }
           else if (line.startsWith("$ Mission configuration")) {
             phase = Constants.MISSION_CONFIG;
           }
           else if (line.startsWith("ParkPt")) { 
             phase = Constants.PARK;
             if (driftCnts==0) {
               driftCnts = 1;
               processParkPtLine(line, 1, true ); 
	       //System.out.println("after processParkPtLine");
             }
             else {
               driftCnts++ ;
               processParkPtLine(line, driftCnts, false ); 
               if (driftCnts==4) {
                 nbrDriftLines = nbrDriftLines + 1;
                 driftPressureLines[nbrDriftLines] = driftPressureLine.toString();
                 driftTemperatureLines[nbrDriftLines] = driftTemperatureLine.toString();
		 //Added By CA
		 if (parkPts) 
		 {
                  driftSalinityLines[nbrDriftLines] = driftSalinityLine.toString();
		 }
                 driftMeasurementTimeLines[nbrDriftLines] = driftMeasurementTimeLine.toString();
                 driftCnts = 0;
               }
             }
           }
           else if (line.startsWith("$ Profile")) {
             processBeforeLowResolution(line);
             phase = Constants.LOW_RESOLUTION;
           }
           else if ( line.startsWith("$") && 
                    (phase!=Constants.MISSION_CONFIG) ) {
             if (phase==Constants.PARK) {
               phase = Constants.LOW_RESOLUTION;
             }
             if (phase==Constants.LOW_RESOLUTION) {
	
         	if(line.startsWith("$ Discrete")){
			line = buffReader.readLine();
			continue;
		}

               StringTokenizer tk2 = new StringTokenizer(line);
	       String headerName;
	       int count = 0;
	       //System.out.println("count outside while = "+count);
               while(tk2.countTokens()>0)
	       {
	 	 headerName=tk2.nextElement().toString();
		 if(headerName.contains("$"))
		 continue;
		 lhmLowHeaders.put(headerName,count);
		 count++;
	       }
	       
	       lhmLowMap=lowResTabReader.getMap();
	       
	      if(debugOn){ System.out.println("lhmLowHeaders.size() = "+lhmLowHeaders.size());
	       System.out.println("lowResTabReader.getHeadersSize() = "+lowResTabReader.getHeadersSize());
	       System.out.println("meta.getMatrixCount() = "+meta.getMatrixCount());
	       System.out.println("equationCount = "+equationCount);
	       //System.out.println("lowResTabReader.getNaNCount() = "+lowResTabReader.getNaNCount());
	       //System.out.println("lhmLowMap.get(MATRIX2) = "+(lhmLowMap.get("MATRIX2")));
	       //System.out.println("lhmLowMap.get(MATRIX2) = "+(lhmLowMap.get("MATRIX2").get(0).toString()));
	       }
               matrixCount = meta.getMatrixCount();
	       if(lhmLowMap.containsKey("MATRIX2")){
               if(lhmLowMap.get("MATRIX2").get(0).toString().equals("NOT_AVAILABLE"))
               	matrixCount++;
	       }
	       int actualCount = lowResTabReader.getHeadersSize()-matrixCount-equationCount;
	       if(lhmLowMap.containsKey("ALL_NAN_IN_MSG"))
	       {
	        actualCount = actualCount-1;

	       }

	       if(debugOn) {
	       System.out.println("matrixCount =" +matrixCount);
	       System.out.println("actualCount =" +actualCount);
	       }

		if((lhmLowHeaders.size()) != (actualCount))
		{
        		try {
          			errorLogger.write( this.getClass().getName(), this.phyFilename, "Headers in Msg File do not match Low Resolution Table " + msgPathname  );
          			errorLogger.close();
          			  System.exit(Constants.ERR_LOWRES_TABLE);
        		     }
        		 catch (IOException e1) {}

		}

             }
           }  
           else if (line.startsWith("Fix: ")) {
             processGPSFix( line );
             phase = Constants.TECHNICAL;
           }
           else if (line.startsWith("# GPS fix obtained")) {
             phase = Constants.TECHNICAL;
           }
           else if (line.startsWith("#")) {
             if (phase==Constants.LOW_RESOLUTION) {
               processAfterLowResolution( line );
               phase = Constants.HIGH_RESOLUTION;
             }
           }  
           else if (line.startsWith("ParkDescentPCnt")) {
             if ( ! processParkDescent(line, buffReader)) {
               msgFileIncomplete = true; 
               try {
                 logger.write( this.getClass().getName(), phyFilename, "Warning: Number of DRIFT DESCENT declared points not equal to actual number of points." );
               }
               catch (IOException e1) {}
               try {
                 errorLogger.write( this.getClass().getName(), phyFilename, 
                                   "Warning: Number of DRIFT DESCENT declared points not equal to actual number of points." );
               }
               catch (IOException e1) {}
             }           
           }
           else if (line.startsWith("ParkObs")) {
             if(debugOn) System.out.println( "TSO4 D ParkObs process");
             processParkObservations( line );
           }
           else {
             if (phase==Constants.HIGH_RESOLUTION) {
               if (isHighResolution(line) ) {
		 lhmHighMap=highResTabReader.getMap();
                List<String> highResLenList = new ArrayList<String>();  
		highResLenList = lhmHighMap.get("TOTAL");
		String highResLen=highResLenList.get(0);
                 if ( ! processHighResolution(line,highResLen) ) {
                   msgFileIncomplete = true; 
                   try {
                     logger.write( this.getClass().getName(), phyFilename, 
                                  "Warning: High Resolution line (" + line + ") is too short" );
                   }
                   catch (IOException e1) {}
                   try {
                     errorLogger.write( this.getClass().getName(), this.phyFilename, 
                                        "Warning: High Resolution line (" + line + ") is too short" );
                   }
                   catch (IOException e1) {}
                 } 
               }
               else {
                 phase=Constants.TECHNICAL;
                 processTechnicalParameter(line, logger); 
               }
             }
             else if (phase==Constants.LOW_RESOLUTION) {
               processLowResolution(line); 
             }
             else if (phase==Constants.TECHNICAL) {
               processTechnicalParameter(line, logger); 
             }
             else if (phase==Constants.MISSION_CONFIG) {
               if ( ! processMissionConfiguration(line, logger) ) {
                 confParameterNotFound = true; 
                 try {
                   logger.write( this.getClass().getName(), this.phyFilename, 
                                 "Warning: Configuration parameter not found in line " + line );
                 }
                 catch (IOException e1) {}
                 try {
                   errorLogger.write( this.getClass().getName(), this.phyFilename, 
                                      "Warning: Configuration parameter not found in line " + line );
                 }
                 catch (IOException e1) {}
               }
             }
           }  

           line = buffReader.readLine(); 
         }
         catch (Exception e) {
           e.printStackTrace();
           try {
             logger.write( this.getClass().getName(), phyFilename, "Error: " + e.getMessage() + 
                           "\n while processing line: \n" + line);
             logger.write( this.getClass().getName(),  null, "End processing " + msgPathname +
                                                       " with code="+Integer.toString(Constants.ERR_EXECUTION) );
             logger.close();
             buffReader.close();
             inpReader.close();
             buffWriter.close();
             outWriter.close();
           }
           catch (IOException e1) {}
           try {
              errorLogger.write( this.getClass().getName(), phyFilename, 
                                 "Error: " + e.getMessage() + "\n while processing line: \n" + line );
              errorLogger.close();
           }
           catch (IOException e1) {}
           System.exit(Constants.ERR_EXECUTION);
         }
       }

       if(debugOn) System.out.println( "TSO4 D done reading msg file");
       // flushing possible incomplete lines
       if (driftCnts>0 && driftCnts<4) {
         nbrDriftLines = nbrDriftLines + 1;
         driftPressureLines[nbrDriftLines] = driftPressureLine.toString();
         driftTemperatureLines[nbrDriftLines] = driftTemperatureLine.toString();
	 //Added By CA
	 if (parkPts)  
	 {
	  driftSalinityLines[nbrDriftLines] = driftSalinityLine.toString();
	 }
         driftMeasurementTimeLines[nbrDriftLines] = driftMeasurementTimeLine.toString();
       }

       // Write the Drift Lines block
       for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftPressureLines[i] );
         buffWriter.newLine();
       }
       for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftTemperatureLines[i] );
         buffWriter.newLine();
       }
       if (parkPts) 
       {
         for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftSalinityLines[i] );
         buffWriter.newLine();
        }
       }
       for (int i=1; i<=nbrDriftLines; i++) {
         buffWriter.write( driftMeasurementTimeLines[i] );
         buffWriter.newLine();
       }

       writeConfParameterBlock( buffWriter );

       writeTechParameterBlock( buffWriter );

       interpolateTempPSal();
       
       determineProfilesOverlap();

       if(debugOn) System.out.println( "TSOALL writeColumnHeaderBlock");
       writeColumnHeaderBlock( buffWriter);

       if(debugOn) System.out.println( "TSOALL D WR NAV beg");
       buffWriter.write( "============================================================" );
       buffWriter.newLine();
       buffWriter.write( "   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION-QC" );
       buffWriter.newLine();
       if ( beforeProfLine1==null && beforeProfLine2[0]==null && beforeProfLine3==null ) {
         buffWriter.write( "    +99.999   +999.999 9999/99/99 99:99:99   000   0     0");
         buffWriter.newLine();
         msgFileIncomplete = true;
         try {
           logger.write( this.getClass().getName(), phyFilename, "Warning: No position data found" );
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), phyFilename, 
                             "Warning: No position data found" );
         }
         catch (IOException e1) {}
       }
       if ( beforeProfLine1 != null ) {
         buffWriter.write( beforeProfLine1 );
         buffWriter.newLine();
       }
      /* System.out.println(" beforeProfLine2.length  = "+ beforeProfLine2.length );
       System.out.println(" beforeProfLine2[0]  = "+ beforeProfLine2[0] );
       System.out.println(" gpsFixCounter = "+gpsFixCounter);*/
       if ( beforeProfLine2[0] != null ) {
          for(int i=0;i<=gpsFixCounter;i++){
	   if((i< gpsFixCounter) || (i ==0) ){
           buffWriter.write( beforeProfLine2[i] );
           buffWriter.newLine();
	  }
	 }
       }
       if ( beforeProfLine3 != null ) {
         buffWriter.write( beforeProfLine3 );
         buffWriter.newLine();
       }
       /*System.out.println(" beforeProfLine1  = "+ beforeProfLine1 );
       System.out.println(" beforeProfLine2  = "+ beforeProfLine2[0] );
       System.out.println("beforeProfLine3 =" + beforeProfLine3); */
       buffWriter.write( "============================================================" );
       buffWriter.newLine();
       if(debugOn) System.out.println( "TSO4 D WR NAV end");
     
       
       //interpolateTempPSal();
       calcO2_lowRes();

       // order high resolution arrays decrescently 
       for (int k=1; k<nbrProfile1Lines; k++) 
         for (int j=k+1; j<=nbrProfile1Lines; j++) 
           if (prof1Pres[j]>prof1Pres[k]) {
             // swapping pressure values
             double swappie = prof1Pres[j];
             prof1Pres[j] = prof1Pres[k];
             prof1Pres[k] = swappie;
             // swapping temp values
             swappie = prof1Temp[j];
             prof1Temp[j] = prof1Temp[k];
             prof1Temp[k] = swappie;
             // swapping salinity values
             swappie = prof1Psal[j];
             prof1Psal[j] = prof1Psal[k];
             prof1Psal[k] = swappie;
             // swapping frequencies
	    if(ofreq_exists)
	    {
		 int int_swappie = prof1Ofreq[j];
		 prof1Ofreq[j] = prof1Ofreq[k];
		 prof1Ofreq[k] = int_swappie;
		 // swapping O2 values
		 swappie = prof1O2[j];
		 prof1O2[j] = prof1O2[k];
		 prof1O2[k] = swappie;
	     }
           }

       printProfileData();
       //System.out.println("profileNumber ,nbrProfile1Lines,nbrProfile2Lines==="+profileNumber+"'" +nbrProfile1Lines +"'"+nbrProfile2Lines);
       // Warning when no profile data is not found.
       if ( (profileNumber>0) && (nbrProfile1Lines==0) && (nbrProfile2Lines==0) ) {
         msgFileIncomplete = true;
         try {
           logger.write( this.getClass().getName(), phyFilename, "Warning: No profile data found" );
         }
         catch (IOException e1) {}
         try {
           errorLogger.write( this.getClass().getName(), phyFilename, 
                             "Warning: No profile data found" );
         }
         catch (IOException e1) {}
       }

       buffReader.close();
       inpReader.close();
       buffWriter.close();
       outWriter.close();


       if (msgFileIncomplete) {
         try {
           logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                    " with code=" + Integer.toString(Constants.WRN_MSG_FILE_INCOMPLETE) );
           logger.close();
           errorLogger.close();
         }
         catch (IOException e1) {}
         System.exit(Constants.WRN_MSG_FILE_INCOMPLETE);
       }
       else {
         if (confParameterNotFound) {
           try {
             logger.write( this.getClass().getName(), null, 
                           "End processing " + msgPathname + " with code=" + Integer.toString(Constants.WRN_CONF_PARAM_NOTFOUND) );
             logger.close();
             errorLogger.close();
           }
           catch (IOException e1) {}
           System.exit(Constants.WRN_CONF_PARAM_NOTFOUND);
         }
         else {
           try {
             logger.write( this.getClass().getName(), null, 
                           "End processing " + msgPathname + " with code="+Integer.toString(Constants.OK) );
             logger.close();
             errorLogger.close();
           }
           catch (IOException e1) {}
         }
       }
    }
    catch ( IOException e )
    {
      try {
        logger.write( this.getClass().getName(), phyFilename, "IO Error: " + e.getMessage() );
        logger.write( this.getClass().getName(), null, "End processing " + msgPathname + 
                                                 " with code=" + Integer.toString(Constants.ERR_IO) );
        logger.close();
        logger.close();
        buffReader.close();
        inpReader.close();
        buffWriter.close();
        outWriter.close();
      }
      catch (IOException e1) {}
      try {
         errorLogger.write( this.getClass().getName(), phyFilename, 
                           "IO Error: " + e.getMessage() );
         errorLogger.close();
      }
      catch (IOException e1) {}
      System.exit(Constants.ERR_IO);
    }
    catch (Exception e) {
      e.printStackTrace();
      try {
        logger.write( this.getClass().getName(), phyFilename, "Error: " + e.getMessage() );
        logger.write( this.getClass().getName(), null, "End processing " + msgPathname +
                                                 " with code=" + Integer.toString(Constants.ERR_EXECUTION) );
        logger.close();
        logger.close();
        buffReader.close();
        inpReader.close();
        buffWriter.close();
        outWriter.close();
      }
      catch (IOException e1) {}
      try {
         errorLogger.write( this.getClass().getName(), phyFilename, 
                           "Error: " + e.getMessage() );
         errorLogger.close();
      }
      catch (IOException e1) {}
      System.exit(Constants.ERR_EXECUTION);
    }
}
 


   /****************************************************************
      Main of the Program,Entry Point 
   *****************************************************************/  

   public static void main ( String args[] )
   {
     if(debugOn) System.out.println( "APEXIR_ALL D main beg");
     
     if ( args.length != 10) {
       System.out.println(Utils2.dateFormatMessage( "APEXIR_ALL", null, 
                          "Usage:java Msg2File msgPathname phyPathname metaPathname technicalTableFilePathname configTableFilePathname lowResTabPathName highResTabPathName logJavaFilePathname errorFilePathname debug") ); 
       System.exit(Constants.ERR_WRONG_NBR_PARAM);
     }

     Msg2Phy_Ir_TSOALL  msg2Phy = new Msg2Phy_Ir_TSOALL(args[0], args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8] ,args[9]);
     msg2Phy.process();  
     if(debugOn) System.out.println( "TSOALL D main end");
   }
}//End of Msg2Phy_Ir_TSOALL
