/***************************************************************************** 
  Revision date: 10-Feb-2010  ER: Initial version  
  
  Logger.java
  ===========
  Manage the Java log file that will be written by the Java decoders 

*****************************************************************************/

import java.io.*;
import java.util.*;
import java.text.*;

public class Logger 
{

  private String logFileName;
  FileWriter logFileWriter;
  BufferedWriter logBuffWriter;

  /****************************************************************
   Class constructor.  
   Create and open the log file.
   logFileName is the pathname of the log file.
  *****************************************************************/
  public Logger( String logFileName ) throws IOException {
    this.logFileName = logFileName;
    this.logFileWriter = new FileWriter( logFileName, true );
    this.logBuffWriter = new BufferedWriter( this.logFileWriter );
  }

  /****************************************************************
   Write a string in the log file with the following format
   callingProgram date&time string.
   for example:
   msg2phy.sh 2010/02/10 15:54:00: Beg processing 5240.026.msg 
  *****************************************************************/
  public void write( String callingProgram, String phyFilename, String s ) throws IOException
  {
    logBuffWriter.write( Utils2.dateFormatMessage( callingProgram, phyFilename, s ) );
    logBuffWriter.newLine();
  }

  /****************************************************************
   Flush and close log file. 
  *****************************************************************/
  public void close() throws IOException {
    this.logBuffWriter.flush();
    this.logBuffWriter.close();
    this.logFileWriter.close();
  }

}
