/***************************************************************************** 
  Revision date:  13-Jul-2011  ER: Initial release 
  
  MissionConfigTable.java
  ========================
  Manage the mission configuration table that will be read by the Java 
  decoders 

*****************************************************************************/
import java.io.*;
import java.util.*;


public class MissionConfigTable
{

  boolean error;
  
  final static int MAX_NBR_PARAMS = 100;
  String[] msgConfParamName = new String[  MAX_NBR_PARAMS ];
  String[] phyConfParamName = new String[  MAX_NBR_PARAMS ];
  
  int nbrOfParameters;

  /****************************************************************
   Class constructor.  
   Read the mission config parameter file and create two vectors:
    1- with the mission config parameter names as written on msg files
    2- with the mission config parameter names as written on phy files
  *****************************************************************/  
  public MissionConfigTable( String filePathname ) {

     FileReader fileReader;
     BufferedReader buffReader;

     try {
       fileReader = new FileReader( filePathname );
       buffReader = new BufferedReader( fileReader );
     }
     catch ( IOException e ) {
       setError( true );
       return;
     }

     try {
       int i = 0;
       String line = buffReader.readLine();
       while (line!=null) {
         StringTokenizer tk = new StringTokenizer(line, "<>");
         setMsgConfParamName( tk.nextElement().toString(), i );
         setPhyConfParamName( tk.nextElement().toString(), i );
         line = buffReader.readLine();
         i++;
       }
       this.nbrOfParameters = i;
       setError(false);
     }
     catch ( Exception e ) {
       setError( true );
       System.out.println( "Error processing config file: " + e.getMessage() );
     }

  }


  public void setError( boolean error ) {
    this.error = error;
  }

  public boolean hasError() {
    return this.error;
  };

  void setMsgConfParamName( String msgConfParamName, int index ) {
    this.msgConfParamName[ index ] = msgConfParamName;
  }

  String getMsgConfParamName( int index ) {
    return msgConfParamName[ index ] ;
  }

  void setPhyConfParamName( String phyConfParamName, int index ) {
    this.phyConfParamName[ index ] = phyConfParamName;
  }

  String getPhyConfParamName( int index ) {
    return phyConfParamName[ index ];
  }


  /****************************************************************
   Accepts a config parameter name as it is written in the msg
   file and returns its position in the vector
  *****************************************************************/  
  public int indexOf( String msgConfParamName ) {

    for (int i=0; i<nbrOfParameters; i++)  {
      if (this.msgConfParamName[ i ].equals(msgConfParamName)) 
        return i;
    }
    return -1; 
  }

}
