/***************************************************************************** 
  Revision date: 10-Feb-2010 ER: Initial version 
  
  TechnicalTable.java
  ===================
  Manage the technical table that will be read by the Java decoders 

*****************************************************************************/
import java.io.*;

public class TechnicalTable
{

  boolean error;
  
  final static int MAX_NBR_PARAMS = 150;
  String[] msgTechParamName = new String[  MAX_NBR_PARAMS ];
  String[] phyTechParamName = new String[  MAX_NBR_PARAMS ];
  String[] calibratedFor    = new String[  MAX_NBR_PARAMS ];
  boolean[] hasCalibration  = new boolean[ MAX_NBR_PARAMS ];
  
  int nbrOfParameters;

  /****************************************************************
   Class constructor.  
   Read the technical parameter file and create two vectors:
    1- with the technical parameter names as written on msg files
    2- with the technical parameter names as written on phy files
  *****************************************************************/  
  public TechnicalTable( String filePathname ) {

     FileReader fileReader;
     BufferedReader buffReader;

     try {
       fileReader = new FileReader( filePathname );
       buffReader = new BufferedReader( fileReader );
     }
     catch ( IOException e ) {
       setError( true );
       return;
     }

     try {
       int i = 0;
       String line = buffReader.readLine();
       while (line!=null) {
         setMsgTechParamName( line.substring(0, line.indexOf(" ")), i );  
         if (line.charAt(40) != ' ') {
           setHasCalibration( true, i );
           setCalibratedFor( line.substring(40, line.indexOf(" ",40)), i );  
         }
         else {
           setHasCalibration( false, i );
         }
         setPhyTechParamName( line.substring( 50, 90 ), i );
         line = buffReader.readLine();
         i++;
       }
       this.nbrOfParameters = i;
       setError(false);
     }
     catch ( Exception e ) {
       setError( true );
       System.out.println( "Error processing technical file: " + e.getMessage() );
     }

  }


  public void setError( boolean error ) {
    this.error = error;
  }

  public boolean hasError() {
    return this.error;
  };

  void setMsgTechParamName( String msgTechParamName, int index ) {
    this.msgTechParamName[ index ] = msgTechParamName;
  }

  String getMsgTechParamName( int index ) {
    return msgTechParamName[ index ] ;
  }

  void setPhyTechParamName( String phyTechParamName, int index ) {
    this.phyTechParamName[ index ] = phyTechParamName;
  }

  String getPhyTechParamName( int index ) {
    return phyTechParamName[ index ];
  }

  void setCalibratedFor( String calibratedFor, int index ) {
    this.calibratedFor[ index ] = calibratedFor;
  }

  String getCalibratedFor( int index ) {
    return calibratedFor[ index ];
  }

  void setHasCalibration( boolean hasCalibration, int index ) {
    this.hasCalibration[ index ] = hasCalibration;
  }

  boolean hasCalibration( int index ) {
    return hasCalibration[ index ];
  }


  /****************************************************************
   Accepts a technical parameter name as it is written in the msg
   file and returns its position in the vector
  *****************************************************************/  
  public int indexOf( String msgTechParamName ) {

    for (int i=0; i<nbrOfParameters; i++)  {
      if (this.msgTechParamName[ i ].equals(msgTechParamName)) 
        return i;
    }
    return -1; 
  }

}
