#!/bin/sh
#########################################################################################
# Revision Date: 23-Jul-14 (CA, RS - changed ARGO_DIR to ARGOBETA_DIR. 
#                                    Created ./argos/prg/java & copy files from
#                                    /a1/ARGO/argos/prg/java) 
#                03-May-13 (CA, RS - implemented the 'shared' and used in classpath) 
#                16-Apr-13 (CA,RS - incorporated code for the table driven decoder.
#                                   Java code for the table driven decoder is located in:
#                                       /a1/ARGO/iridium/msg2phy/prg/tsoall) 
#                06-Mar-13 (RS - add no meta file found counter (nometa)'nometa' to 
#                                email subject; nometa=err_15_cnt. Also now, the number
#                                of failures excludes the number of meta files not found)
#                01-Mar-13 (RS - modified to improve code structure and documentation)
#                24-Jan-13 (CA,RS - integrated 'GetMsgLst_GPSFixes' for interpolation of
#                                   missing GPS fixes)   
#                18-Dec-12 (RS - introduce check to skip empty 'msg' files. 
#                                Email subject will indicate a WARNING and empty file(s)
#                                will be listed on the email body)  
#                01-Jun-12 (RS - introduced 'whoami' prefixed by "PATH_WHOAMI
#                                to detect 'user'. If user is 'argo' then
#                                Vicki's email address is appened to EMAIL_OPERATOR)
#                30-May-12 VMH - change == to = in ifs, add no mvh for debug>0,
#                                add myself to EMAIL_OPERATOR
#                30-May-12 (RS - change if [ debug == 0 ] to if [ $debug == 0 ]
#                21-May012 VMH - If debug is on, do not sed mai or move msg files
#                25-Apr-12 (RS - added code to remove empty logfiles and
#                                'identicalList' table)
#                20-Mar-12 VMH   Read config_parameters.<float type> and
#                                technical_parameters.<float type> if they exist,
#                                instead of *.txt. No mail if debug=1)
#                15-Feb-12 (RS, CS - add '/' at the end of a path for 'mv')
#                14-Jul-11 (ER - Add processing of mission config params.
#                              - New errors and warnings 18, 118, 137)
#                27-Jun-11 (ER - Call to DiffCnt and posterior process of
#                                results added.)
#                04-Jan-11 (ER - Merge of Iridium & Java msg codes)
#                19-Nov-10 (ER - Do nothing when the decoder for the float
#                                type has not yet been implemented)
#                12-Aug-10 (ER - Program making sure to delete new phy
#                                file in case an error occur. )
#                23-Mar-10 (ER - Change the text of the email for the case
#                                in which they are not files to process. )
#                06-Jan-10 (ER - Initial version)
#
# Note
# ----
# PATH_WHOAMI: defined and exported in ~/.argorc_sh  (not defined in 'cron' PATH)
#########################################################################################
#----------------------------------------------------------------------------------------
# Function WriteJavaExitStatusCodes
# =================================
# Write Java Exit Status Codes to the 'javalogfile' file
#
# Usage: WriteJavaExitStatusCodes
#----------------------------------------------------------------------------------------
WriteJavaExitStatusCodes()
{
  funcname="WriteJavaExitStatusCodes"
  echo "=> Enter $funcname"

  echo "Possible Java Exit Status" >$javalogfile
  echo "    0, OK" >>$javalogfile
  echo "  111, Error opening input msg file" >>$javalogfile
  echo "  115, Error opening meta file" >>$javalogfile
  echo "  117, Error opening technical parameter file" >>$javalogfile
  echo "  118, Error opening configuration parameter file" >>$javalogfile
  echo "  131, Wrong number of parameters in call to Java program" >>$javalogfile
  echo "  132, Error creating Java log file" >>$javalogfile
  echo "  133, Error creating output phy file" >>$javalogfile
  echo "  134, Missing or wrong meta file information" >>$javalogfile
  echo "  135, Warning Incomplete msg file" >>$javalogfile
  echo "  136, Error opening error file" >>$javalogfile
  echo "  137, Warning Configuration parameter not found" >>$javalogfile
  echo "  140, Error opening Low Resolution Table" >>$javalogfile
  echo "  141, Error opening High Resolution Table" >>$javalogfile
  echo "  142, Error Element in the Low Res Table/or High Res is missing" >>$javalogfile
  echo "  143, Error Foil Matrix Does Not Exist, Initialize a new one" >>$javalogfile
  echo "  144, Error Mismatch of Message File Headers with the Low Resolution Table " >>$javalogfile
  echo "  145, Error Error Row Or Column Values missing in Foil Matrix " >>$javalogfile
  echo "  146, Error Row Or Column Values of Foil Matrix of the meta file do not match that of the Table " >>$javalogfile
  echo "  148, Error  Variable not found in the Low Resolution Table " >>$javalogfile
  echo "  998, IO error" >>$javalogfile
  echo "  999, Execution error\n" >>$javalogfile


  echo "=> Return $funcname"
  return
}  
#----------------------------------------------------------------------------------------
# Function InitFlagsAndError 
# ==========================
# Initialize flags and set possible exit status outcome 
#
# Usage: InitFlagsAndError
#----------------------------------------------------------------------------------------
InitFlagsAndError()
{
  fncname="InitFlagsAndError"
  echo "=> Enter $fncname"

  success_cnt=0
  failure_cnt=0
  warning_cnt=0
  cntdiff_same_cnt=0
  cntdiff_different_cnt=0
  cntdiff_failure_cnt=0
  err_11_cnt=0
  err_111_cnt=0
  err_12_cnt=0
  err_13_cnt=0
  err_14_cnt=0
  err_15_cnt=0
  err_115_cnt=0
  err_16_cnt=0
  wrn_116_cnt=0
  err_17_cnt=0
  err_18_cnt=0
  err_117_cnt=0
  err_118_cnt=0
  err_131_cnt=0
  err_132_cnt=0
  err_133_cnt=0
  err_134_cnt=0
  wrn_135_cnt=0
  err_136_cnt=0
  wrn_137_cnt=0
  err_140_cnt=0
  err_141_cnt=0
  err_142_cnt=0
  err_143_cnt=0
  err_144_cnt=0
  err_145_cnt=0
  err_146_cnt=0
  wrn_147_cnt=0
  err_148_cnt=0
  err_998_cnt=0
  err_999_cnt=0
  echo "=> Return $fncname" 
  return
}
#----------------------------------------------------------------------------------------
# Function WriteReportHeading
# ===========================
# Write the 'Status Report Heading' to the 'reportfile'
#
# Usage: WriteReportHeading
#----------------------------------------------------------------------------------------
WriteReportHeading()
{
  echo "=========================================================" >>$reportfile
  echo   "                       Status Report">>$reportfile
  now_zone=`date +'%Y/%m/%d %H:%M %Z'`
  echo "                    $now_zone" >>$reportfile
  echo  "\nPossible exit status" >>$reportfile
  echo "    0, OK" >>$reportfile
  echo "   11, Msg file not found" >>$reportfile
  echo "  111, Java - error opening input msg file" >>$reportfile
  echo "   12, floatno_iridiumid_list not found" >>$reportfile
  echo "   13, AOML internal ID not found in floatno_iridiumid_list" >>$reportfile
  echo "   14, More than one entry found in floatno_iridiumid_list for an iridium_id" >>$reportfile
  echo "   15, Meta file not found" >>$reportfile
  echo "  115, Java - error opening meta file" >>$reportfile
  echo "   16, Float Type not found in meta file" >>$reportfile
  echo "  116, Java - Warning Float Type not implemented" >>$reportfile
  echo "   17, Technical Parameter file not found" >>$reportfile
  echo "   18, Configuration Parameter file not found" >>$reportfile
  echo "  117, Java - error opening technical parameter file" >>$reportfile
  echo "  118, Java - error opening configuration parameter file" >>$reportfile
  echo "  131, Java - error Wrong number of parameters in call to Java program" >>$reportfile
  echo "  132, Java - error creating Java log file" >>$reportfile
  echo "  133, Java - error creating output phy file in Java program" >>$reportfile
  echo "  134, Java - error Missing or wrong meta file information" >>$reportfile
  echo "  135, Java - Warning Incomplete msg file" >>$reportfile
  echo "  136, Java - error opening error file" >>$reportfile
  echo "  137, Java - Warning Configuration parameter not found" >>$reportfile
  echo "  140, Java - error opening low resolution table" >>$reportfile
  echo "  141, Java - Error opening High Resolution Table" >>$reportfile
  echo "  142, Java - Error Element in the Low Res Table/or High Res is missing" >>$reportfile
  echo "  143, Java - Error Foil Matrix Does Not Exist,Initialize a new one " >>$reportfile
  echo "  144, Java - Error Mismatch of Message File Headers with the Low Resolution Table " >>$reportfile
  echo "  145, Java - Error Row Or Column Values missing in Foil Matrix " >>$reportfile
  echo "  146, Java - Error Row Or Column Values of Foil Matrix of the meta file do not match Table " >>$reportfile
  echo "  147, LowRes_Table.{float_type} exists but HighRes_table.{float_type} does not exits">>$reportfile
  echo "  148, Java - Error  Variable not found in the Low Resolution table ">>$reportfile
  echo "  998, I/O error in Java program" >>$reportfile
  echo "  999, Execution error in Java program" >>$reportfile
  echo "\n==============================================" >>$reportfile
  echo   "Summary Report of MSG Files Processed\n" >>$reportfile
  echo "  MSG File          Success   Failure   Warning   Inst. Type     Metafile" >>$reportfile

  return
}

#----------------------------------------------------------------------------------------
# Function ExecDiffCnt
# ====================
# Executes 'Diffcnt' java program. It sets the 'diffCnt_ststus' flag. 
#
# Usage: ExecDiffCnt
#----------------------------------------------------------------------------------------
ExecDiffCnt()
{
  funcname="ExecDiffCnt"
  echo "=> Enter $funcname"

  now=`date +'%Y/%m/%d %H:%M:%S'`
  echo "java -classpath $ARGOSPRGJAVA_DIR DiffCnt $ARGOPHY_DIR/done/$phy_filename $ARGOPHY_DIR/$phy_filename $tracefile $diffStatusTable " 
  java -classpath $ARGOSPRGJAVA_DIR DiffCnt \
  $ARGOPHY_DIR/done/$phy_filename  $ARGOPHY_DIR/$phy_filename \
  $tracefile $diffStatusTable > /dev/null
  diffCnt_status="$?" 
  case $diffCnt_status in
    1) cntdiff_different_cnt=`expr $cntdiff_different_cnt + 1` 
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): difference found"
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): difference found" >> $statusfile;;
    2) cntdiff_failure_cnt=`expr $cntdiff_failure_cnt + 1`
       is_failure=1
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Possible surface drifter"
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Possible surface drifter" >> $statusfile;;
    3) cntdiff_failure_cnt=`expr $cntdiff_failure_cnt + 1`
       is_failure=1
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Usage error"
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Usage error" >> $statusfile;;
    4) cntdiff_failure_cnt=`expr $cntdiff_failure_cnt + 1`
       is_failure=1
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Date/Time parsing or I/O Error"
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Date/Time parsing or I/O Error" >> $statusfile;;
    5) cntdiff_failure_cnt=`expr $cntdiff_failure_cnt + 1`
       is_failure=1
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Profile Number Problem"
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Profile Number Problem" >> $statusfile;;
    *) cntdiff_failure_cnt=`expr $cntdiff_failure_cnt + 1`
       is_failure=1
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Unexpected code found in Java Program"
       echo "$script_name $now: $phy_filename DiffCnt(status=$diffCnt_status): Unexpected code found in Java Program" >> $statusfile;;
    esac

   if [ $is_failure -eq 1 ]
   then
     if [ $debug = 0 ]
     then
       mv $IRIDIUMMSG_DIR/$msg_file $IRIDIUMMSG_DIR/failure/
       mv $ARGOPHY_DIR/$phy_filename $ARGOPHY_DIR/failure/
     else
       echo "Debug is ON, do not move msg/phy files to failure"
     fi

     now=`date +'%Y/%m/%d %H:%M:%S'`
     echo "$script_name $now: $msg_file moved to $IRIDIUMMSG_DIR/failure" 
     echo "$script_name $now: $msg_file moved to $IRIDIUMMSG_DIR/failure"  >> $statusfile
     echo "$script_name $now: $phy_filename moved to $ARGOPHY_DIR/failure" 
     echo "$script_name $now: $phy_filename moved to $ARGOPHY_DIR/failure" >> $statusfile
     echo "$script_name $now: End processing $msg_file with no status"
   fi

   echo "=> Return $funcname"
   return
}
#----------------------------------------------------------------------------------------
# Function GetMsgLst_GPSFixes
# ===========================
#
# Creates a float list(MSG2PHYTMP_DIR/floatname_pos.gpslist for
# each float with those profiles that has a GPS fix. The list 
# is accessed by 'InterpolateGpsFixes' (called from Msg2Phy_Ir.java) 
# for interpolation of the profiles with missing GPS fixes
#
# Usage: GetMsgLst_GPSFixes
#----------------------------------------------------------------------------------------
GetMsgLst_GPSFixes(){

echo "=> Enter GetMsgLst_GPSFixes"
UNIQ_FLOAT_LST=$MSG2PHYTMP_DIR/uniqueFloats.gpslst
echo "   UNIQ_FLOAT_LST: $UNIQ_FLOAT_LST"

rm -f $MSG2PHYTMP_DIR/*.gpslst 2> /dev/null

cd $IRIDIUMMSG_DIR

# Attempt to get GPS fix failed after 300s .In this case the profile might
# or might not have a position

count_GPS=`grep  "GPS fix not available due to surface ice" *.msg|wc -l`

if [ $count_GPS -eq 0 ]
then
  echo "   No profile with missing GPS fix"
  echo "=> Return GetMsgLst_GPSFixex"
  return 
fi

grep "Fix:" *.msg| awk -F. '{print $1}' |sort -u > $UNIQ_FLOAT_LST

for float in `cat $UNIQ_FLOAT_LST`
do
	count_msg=`ls $IRIDIUMMSG_DIR/$float*.msg |wc -l`
	echo ">>> Count Message:" $count_msg
	echo ">>> Float :" $float
	if [ $count_msg -gt 1 ]
	then
		ls $float*.msg > $MSG2PHYTMP_DIR/$float.gpslst

		first_profile=`head -1 $MSG2PHYTMP_DIR/$float.gpslst`
		echo ">>> First Profile : " $first_profile
	        first_prof_msgNo=`echo $first_profile|awk -F"." '{print $2}'`
		echo ">>> First Msg No::"$first_prof_msgNo

 	cd $IRIDIUMMSG_DONE_DIR

		last_profile=`ls $float*.msg 2> /dev/null |tail -1`
		if [ "$last_profile" != "" ]
		then
			echo ">>> Last Profile ::"$last_profile  
			last_prof_msgNo=`echo $last_profile |awk -F"." '{print $2}'`
			echo ">>> Last Msg No::"$last_prof_msgNo
	        
			if [ $first_prof_msgNo -gt $last_prof_msgNo ]
			then
				fix_msg=`grep  "Fix:" $IRIDIUMMSG_DONE_DIR/$last_profile |head -1`
				if [ -n "$fix_msg" ]
				then
			 		cd $IRIDIUMMSG_DIR
			 		echo "$last_profile  $fix_msg Head" >> $MSG2PHYTMP_DIR/$float"_tmp.gpslst"
			 		cd $IRIDIUMMSG_DONE_DIR
				fi
				fix_msg=`grep  "Fix:" $IRIDIUMMSG_DONE_DIR/$last_profile |tail -1` 
				if [ -n "$fix_msg" ]
				then
			 		cd $IRIDIUMMSG_DIR
			 		echo "$last_profile  $fix_msg Tail" >> $MSG2PHYTMP_DIR/$float"_tmp.gpslst"
		 		fi 
			fi
		fi

		cd $IRIDIUMMSG_DIR
		
	        	
               	for msg_file in `cat $MSG2PHYTMP_DIR/$float.gpslst` 
		do
				fix_msg=`grep  "Fix:" $IRIDIUMMSG_DIR/$msg_file |head -1`
				if [ -n "$fix_msg" ]
				then
				 echo "$msg_file  $fix_msg Head" >> $MSG2PHYTMP_DIR/$float"_tmp.gpslst"
				fi
				fix_msg=`grep  "Fix:" $IRIDIUMMSG_DIR/$msg_file |tail -1` 
				if [ -n "$fix_msg" ]
				then
				 echo "$msg_file  $fix_msg Tail" >> $MSG2PHYTMP_DIR/$float"_tmp.gpslst"
			 	fi 
		done
		grep -v "GPS fix not available due to surface ice" $MSG2PHYTMP_DIR/$float"_tmp.gpslst" >> $MSG2PHYTMP_DIR/$float"_pos.gpslst"

	fi
done
echo "=> Return GetMsgLst_GPSFixes"
return
}
# ---------------------------------------------------------------------------------------
# Function GetMetaFileName (Call from 'ProcessOneMsgFile') 
# =========================================================
# Given a msg file name, returns the meta file 
#
# Usage: GetMetaFileName msg_filename
# ---------------------------------------------------------------------------------------
GetMetaFilename() 
{

  # Look up the line in the cross reference table where the iridium id exists, then get 
  # the  correspondent aoml_id
  msg_filename=$1
  iridium_id=`echo $1| awk -F. '{print $1}'`

  meta_file_name=$aoml_id"_"$iridium_id.meta
  if [ ! -f $METAFILES_DIR/floatno_iridiumid_list ]   # Does x-ref table exist?
  then                                                # No
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: ERROR: $METAFILES_DIR/floatno_iridiumid_list not found" 
    echo "$script_name $now: ERROR: $METAFILES_DIR/floatno_iridiumid_list not found" >>$statusfile
    return 12
  fi
  
  #aoml_id=`grep " $iridium_id" $METAFILES_DIR/floatno_iridiumid_list | awk '{print $1}'`
  aoml_id=`grep " $iridium_id"$ $METAFILES_DIR/floatno_iridiumid_list | awk '{print $1}'`

  # Check that there is one, and just one entry for that iridium id in the 
  # cross reference table (iridium id vs. aoml id)
  nbr_lines_returned=0
  for val in `echo $aoml_id`   
  do
    nbr_lines_returned=`expr $nbr_lines_returned + 1`
  done

  if [ $nbr_lines_returned -eq 0 ]  # Any entry?
  then                              # No, ID not found
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: ERROR: Iridium ID $iridium_id not found in $METAFILES_DIR/floatno_iridiumid_list" 
    echo "$script_name $now: ERROR: Iridium ID $iridium_id not found in $METAFILES_DIR/floatno_iridiumid_list" >>$statusfile
    return 13 
  fi
  if [ $nbr_lines_returned -gt 1 ]   # Many error entries? 
  then                               #  Yes 
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: ERROR: more than one entry found in $METAFILES_DIR/floatno_iridiumid_list for Iridium ID=$iridium_id" 
    echo "$script_name $now: ERROR: more than one entry found in $METAFILES_DIR/floatno_iridiumid_list for Iridium ID=$iridium_id" >>$statusfile
    return 14 
  fi

  meta_file_name=$aoml_id"_"$iridium_id.meta
  if [ ! -f "$METAFILES_DIR/$meta_file_name" ]    # Does meta file exists?
  then                                            #  No
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: ERROR: meta file $METAFILES_DIR/$meta_file_name not found" 
    echo "$script_name $now: ERROR: meta file $METAFILES_DIR/$meta_file_name not found" >>$statusfile
    return 15 
  fi

  # Store meta filename in file 'MSG2PHYTMP_DIR/meta_filename'
  echo $meta_file_name > $MSG2PHYTMP_DIR/meta_filename
  return 0
}
# ---------------------------------------------------------------------------------------
# Revision Date: 06-Jan-10 (ER - Initial version)
#
# Function GetFloatType (Call from 'ProcessOneMsgFile')
# =====================================================
# Given a meta filename, returns the float type
#
# Usage: GetFloatType meta_filename
# ---------------------------------------------------------------------------------------
GetFloatType()
{
  #
  # Extract the complete float type from the meta file line that holds it. 
  float_type_complete=`grep "instrument type" $1 | grep -v "WMO" | awk '{print $3}' | awk -F_ '{print $1"_"$2}'`
 
  #
  # Extract the float type from the meta file line that holds it. 
  float_type=`grep "instrument type" $1 | grep -v "WMO" | awk '{print $3}' | awk -F_ '{print $2}'`

  if [ -z "$float_type" ]
  then
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: ERROR: float type not found in $meta_filename" 
    echo "$script_name $now: ERROR: float type not found in $meta_filename" >>$statusfile
    return 16 
  fi

  #
  # Float type name (comlete) is store in file $MSG2PHYTMP_DIR/float_type_complete 
  # Float type name is store in file $MSG2PHYTMP_DIR/float_type 
  echo $float_type_complete > $MSG2PHYTMP_DIR/float_type_complete
  echo $float_type > $MSG2PHYTMP_DIR/float_type 
  return 0
}
# ---------------------------------------------------------------------------------------
# Revision Date: 06-Jan-10 (ER - Initial version)
#
# Function ProcessOneMsgFile() 
# ============================
# Given a message file, creates a correspondent phy file
#
# Usage: ProcessOneMsgFile msg_file
# ---------------------------------------------------------------------------------------
ProcessOneMsgFile() {
 
  msg_file=$1
  if [ -z "$msg_file" ]    # Is message file null? 
  then                     #  Yes 
    echo "Usage: msg2phy.sh message_file" 
    return 11
  fi

  if [ ! -f "$IRIDIUMMSG_DIR/$msg_file" ]     # Does message file exist?
  then                                        #  No
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: $msg_file ERROR: Message file $IRIDIUMMSG_DIR/$msg_file not found" 
    echo "$script_name $now: $msg_file ERROR: Message file $IRIDIUMMSG_DIR/$msg_file not found" >>$statusfile
    return 11
  fi

  GetMetaFilename $msg_file                         # Get name of the meta file
  status_check=$?
  if [ $status_check -ne 0 ] 
  then
    return $status_check 
  fi
  meta_filename=`cat $MSG2PHYTMP_DIR/meta_filename`

  GetFloatType $METAFILES_DIR/$meta_filename        # Get the float type
  status_check=$?
  if [ $status_check -ne 0 ] 
  then
    return $status_check 
  fi
  float_type_complete=`cat $MSG2PHYTMP_DIR/float_type_complete`
  float_type=`cat $MSG2PHYTMP_DIR/float_type`

  sts_filename=`echo $msg_file|awk -F'.' '{print $1"."$2}'`".sts"
  echo "sts_filename = $sts_filename"
  aoml_id=`echo $meta_filename|awk -F"_" '{print $1}'`
  echo "file path = $STSFILES_DIR/$aoml_id/$sts_filename"
  if [ ! -z $aoml_id  ]
  then
       if [ ! -f "$STSFILES_DIR/$aoml_id/$sts_filename" ]     # Does message file exist?
       then
          sts_filename="NONE"
       fi

  else
        sts_filename="NONE"
  fi 
  
  echo "sts_filename = $sts_filename"
  #.......................................................................................
  #
  # (1) TS2, TS3, TS5, TS8, TSO1, TSO2, TSO3, TSO4, TSO6, TSO7 & TSO8
  #     Uses: technical_parameters_file & config_parameters_file 
  # (2) Table driven decoder for: TSO9 ...
  #     Uses: technical_parameters_file & config_parameters_file 
  #             lowRes_table_file.{float_type) & highRes_table_file
  #     Note - there must be an entry for the float instrument type in 'highRes_table_file'
  #.......................................................................................
  lowRes_table_file="lowRes_table."$float_type
  #highRes_table_file="highRes_table.txt"
  highRes_table_file="highRes_table."$float_type

  # 
  # Check technical parameter table exists
  # Mar 2012 - see if a specific table exists for the specific float type
  # if no specific tabes exist, look for generic
  #
  technical_parameters_file="technical_parameters."$float_type
  if [ ! -f "$MSG2PHYMSG_TABLE/$technical_parameters_file" ]
  then
    echo "$script_name specific file: $technical_parameters_file NOT found, look for generic"
    technical_parameters_file="technical_parameters.txt"
    if [ ! -f "$MSG2PHYMSG_TABLE/$technical_parameters_file" ]
    then
      now=`date +'%Y/%m/%d %H:%M:%S'`
      echo "$script_name $now: $msg_file ERROR: Technical parameter table $MSG2PHYMSG_TABLE/$technical_parameters_file not found" 
      echo "$script_name $now: $msg_file ERROR: Technical parameter table $MSG2PHYMSG_TABLE/$technical_parameters_file not found" >>$statusfile
      return 17
    fi
  fi

  echo "$script_name found technical_parameters file: $technical_parameters_file"

  #
  # Check mission configuration parameter table exists
  # Mar 2012 - see if a specific table exists for the specific float type
  # if no specific tabes exist, look for generic
  config_parameters_file="config_parameters."$float_type
  if [ ! -f "$MSG2PHYMSG_TABLE/$config_parameters_file" ]
  then
    echo "$script_name specific file: $config_parameters_file NOT found, look for generic"
    config_parameters_file="config_parameters.txt"
    if [ ! -f "$MSG2PHYMSG_TABLE/$config_parameters_file" ]
    then
      now=`date +'%Y/%m/%d %H:%M:%S'`
      echo "$script_name $now: $msg_file ERROR: Config parameter table $MSG2PHYMSG_TABLE/$config_parameters_file not found" 
      echo "$script_name $now: $msg_file ERROR: Config parameter table $MSG2PHYMSG_TABLE/$config_parameters_file not found" >>$statusfile
      return 18
    fi
  fi
  echo "$script_name found config_parameters file: $config_parameters_file"

  filename_stem=`echo $meta_filename | awk -F. '{print $1}'` # Define name of the phy output file  
  profile_nbr=`echo $msg_file | awk -F. '{print $2}'`
  phy_filename=$filename_stem"_"$profile_nbr".phy"
  echo $phy_filename > $MSG2PHYTMP_DIR/phy_filename

  #............................................................................................
  #                         Calling the Java Decoder
  # (1) Msg2Phy_Ir_TSOALL: Table driven decoder for: TSO9 ...   
  # (2) Msg2Phy_Ir_{float_type}: TS2, TS3, TS5, TS8, TSO1, TSO2, TSO3, TSO4, TSO6, TSO7 & TSO8
  #............................................................................................
  printf "Does $MSG2PHYMSG_TABLE/$lowRes_table_file exist? "
  if [ -f $MSG2PHYMSG_TABLE/$lowRes_table_file ]
  then
      echo "Yes"
      printf "Does  $MSG2PHYMSG_TABLE/$highRes_table_file exist?  "
      if [ -f $MSG2PHYMSG_TABLE/$highRes_table_file ]
      then
        echo "Yes" 
        java -classpath "$MSG2PHYPRG_DIR/all:$MSG2PHYPRG_DIR/shared" Msg2Phy_Ir_TSOALL \
        $IRIDIUMMSG_DIR/$msg_file $ARGOPHY_DIR/$phy_filename  \
        $METAFILES_DIR/$meta_filename $MSG2PHYMSG_TABLE/$technical_parameters_file \
        $MSG2PHYMSG_TABLE/$config_parameters_file \
        $MSG2PHYMSG_TABLE/$lowRes_table_file \
        $MSG2PHYMSG_TABLE/$highRes_table_file \
        $javalogfile $stderrorfile $debug
        status_check=$?
      else
        echo "No" 
        return $wrn_147
      fi 
  else
    if [ ! -f $MSG2PHYPRG_DIR/Msg2Phy_Ir_$float_type.class ]
    then
      now=`date +'%Y/%m/%d %H:%M:%S'`
      echo "$script_name $now: $msg_file Java - Warning Float Type not implemented"
      echo "$script_name $now: $msg_file Java - Warning Float Type not implemented" >>$statusfile
      return 116
    fi

    echo "$STSFILES_DIR/$aoml_id/$sts_filename from shell "

    java -classpath "$MSG2PHYPRG_DIR:$MSG2PHYPRG_DIR/shared" Msg2Phy_Ir_$float_type \
         $IRIDIUMMSG_DIR/$msg_file $ARGOPHY_DIR/$phy_filename  \
         $STSFILES_DIR/$aoml_id/$sts_filename \
         $METAFILES_DIR/$meta_filename $MSG2PHYMSG_TABLE/$technical_parameters_file \
         $MSG2PHYMSG_TABLE/$config_parameters_file \
         $javalogfile $statusfile $debug  
         status_check=$?
  fi

  if [ "$status_check" -ne "0" ] 
  then
    if [ "$status_check" != "`grep $status_check $MSG2PHYMSG_TABLE/java_status_codes_list.txt | awk -F, '{print $1}'`" ]
    then
      status_check=999
      kind_of_error="ERROR"
    else
      kind_of_error=`grep $status_check $MSG2PHYMSG_TABLE/java_status_codes_list.txt | awk -F, '{print $2}'`
    fi
    now=`date +'%Y/%m/%d %H:%M:%S'`
    if [ "$kind_of_error" = "ERROR" ]
    then
      echo "$script_name $now: $phy_filename Error: Java program returns error $status_check"   
      echo "$script_name $now: $phy_filename Error: Java program returns error $status_check" >> $statusfile
    else
      echo "$script_name $now: $phy_filename Warning: Java program returns warning $status_check" 
      echo "$script_name $now: $phy_filename Warning: Java program returns warning $status_check" >>$statusfile
    fi
    return $status_check
  fi

  return 0 
}
#---------------------------------------------------------------------------------------
# CheckWaringProcess_MsgFile
# ==========================
# Checks the exit status warning of the 'ProcessOneMsgFile'
#
# Usage: CheckWaringProcess_MsgFile 
#---------------------------------------------------------------------------------------
CheckWaringProcess_MsgFile()
{
   status_warning=$1
   case $status_warning in
     0) if [ $debug = 0 ]
        then
          mv $IRIDIUMMSG_DIR/$msg_file $IRIDIUMMSG_DIR/done/
        else
          echo "Debug is ON, do not mv msg file to done"
        fi
        echo $phy_filename >> $phy2bprocessed_list
        echo "$msg_file           1        0        0       $float_type_complete     $meta_filename" >>$reportfile
        success_cnt=`expr $success_cnt + 1`;;
      116) echo "$msg_file           0        0      116       $float_type_complete     $meta_filename" >>$reportfile
           wrn_116_cnt=`expr $wrn_116_cnt + 1`
           warning_cnt=`expr $warning_cnt + 1`;;
      135) mv $IRIDIUMMSG_DIR/$msg_file $IRIDIUMMSG_DIR/done/
           echo $phy_filename >> $phy2bprocessed_list
           echo "$msg_file           0        0      135       $float_type_complete     $meta_filename" >>$reportfile
           wrn_135_cnt=`expr $wrn_135_cnt + 1`
           warning_cnt=`expr $warning_cnt + 1`;;
      137) mv $IRIDIUMMSG_DIR/$msg_file $IRIDIUMMSG_DIR/done/
           echo $phy_filename >> $phy2bprocessed_list
           echo "$msg_file           0        0      137       $float_type_complete     $meta_filename" >>$reportfile
           wrn_137_cnt=`expr $wrn_137_cnt + 1`
           warning_cnt=`expr $warning_cnt + 1`;;
      147) echo "$msg_file           0        0      147       $float_type_complete     $meta_filename" >>$reportfile
           wrn_147_cnt=`expr $wrn_147_cnt + 1`
           warning_cnt=`expr $warning_cnt + 1`;; 
      esac
   return
 }
#---------------------------------------------------------------------------------------
# CheckStatusProcess_MsgFile 
# ==========================
# Checks the exit status of the 'ProcessOneMsgFile' to setup appropiate flag(s)  
#
# Usage: CheckStatusProcess_MsgFile
#---------------------------------------------------------------------------------------
CheckStatusProcess_MsgFile()
{
  funcname="CheckStatusProcess_MsgFile"

  case $status_process_msgfile  in
    11) echo "$msg_file           0       11        0       $float_type_complete     $meta_filename" >>$reportfile
        err_11_cnt=`expr $err_11_cnt + 1`
        is_failure=1;;
   111) echo "$msg_file           0      111        0       $float_type_complete     $meta_filename" >>$reportfile
        err_111_cnt=`expr $err_111_cnt + 1`
        is_failure=1;;
    12) echo "$msg_file           0       12        0       $float_type_complete     $meta_filename" >>$reportfile
        err_12_cnt=`expr $err_12_cnt + 1`
        is_failure=1;;
    13) echo "$msg_file           0       13        0       $float_type_complete     $meta_filename" >>$reportfile
        err_13_cnt=`expr $err_13_cnt + 1`
        is_failure=1;;
    14) echo "$msg_file           0       14        0       $float_type_complete     $meta_filename" >>$reportfile
        err_14_cnt=`expr $err_14_cnt + 1`
        is_failure=1;;
    15) echo "$msg_file           0       15        0       $float_type_complete     $meta_filename" >>$reportfile
        err_15_cnt=`expr $err_15_cnt + 1`
        is_failure=1;;
   115) echo "$msg_file           0      115        0       $float_type_complete     $meta_filename" >>$reportfile
        err_115_cnt=`expr $err_115_cnt + 1`
        is_failure=1;;
    16) echo "$msg_file           0       16        0       $float_type_complete     $meta_filename" >>$reportfile
        err_16_cnt=`expr $err_16_cnt + 1`
        is_failure=1;;
    17) echo "$msg_file           0       17        0       $float_type_complete     $meta_filename" >>$reportfile
        err_17_cnt=`expr $err_17_cnt + 1`
        is_failure=1;;
    18) echo "$msg_file           0       18        0       $float_type_complete     $meta_filename" >>$reportfile
        err_18_cnt=`expr $err_18_cnt + 1`
        is_failure=1;;
   117) echo "$msg_file           0      117        0       $float_type_complete     $meta_filename" >>$reportfile
        err_117_cnt=`expr $err_117_cnt + 1`
        is_failure=1;;
   118) echo "$msg_file           0      118        0       $float_type_complete     $meta_filename" >>$reportfile
        err_118_cnt=`expr $err_118_cnt + 1`
        is_failure=1;;
   131) echo "$msg_file           0      131        0       $float_type_complete     $meta_filename" >>$reportfile
        err_131_cnt=`expr $err_131_cnt + 1`
        is_failure=1;;
   132) echo "$msg_file           0      132        0       $float_type_complete     $meta_filename" >>$reportfile
        err_132_cnt=`expr $err_132_cnt + 1`
        is_failure=1;;
   133) echo "$msg_file           0      133        0       $float_type_complete     $meta_filename" >>$reportfile
        err_133_cnt=`expr $err_133_cnt + 1`
        is_failure=1;;
   134) echo "$msg_file           0      134        0       $float_type_complete     $meta_filename" >>$reportfile
        err_134_cnt=`expr $err_134_cnt + 1`
        is_failure=1;;
   136) echo "$msg_file           0      136        0       $float_type_complete     $meta_filename" >>$reportfile
        err_136_cnt=`expr $err_136_cnt + 1`
        is_failure=1;;
    140) echo "$msg_file           0      140        0       $float_type_complete     $meta_filename" >>$reportfile
          err_140_cnt=`expr $err_140_cnt + 1`
          is_failure=1;;
     141) echo "$msg_file           0      141        0       $float_type_complete     $meta_filename" >>$reportfile
          err_141_cnt=`expr $err_141_cnt + 1`
          is_failure=1;;
     142) echo "$msg_file           0      142        0       $float_type_complete     $meta_filename" >>$reportfile
          err_142_cnt=`expr $err_142_cnt + 1`
          is_failure=1;;
     143) echo "$msg_file           0      143        0       $float_type_complete     $meta_filename" >>$reportfile
          err_143_cnt=`expr $err_143_cnt + 1`
          is_failure=1;;
     144) echo "$msg_file           0      144       0       $float_type_complete     $meta_filename" >>$reportfile
          err_144_cnt=`expr $err_144_cnt + 1`
          is_failure=1;;
     145) echo "$msg_file           0      145      0       $float_type_complete     $meta_filename" >>$reportfile
          err_145_cnt=`expr $err_145_cnt + 1`
          is_failure=1;;
     146) echo "$msg_file           0      146      0       $float_type_complete     $meta_filename" >>$reportfile
          err_146_cnt=`expr $err_146_cnt + 1`
          is_failure=1;;
     148) echo "$msg_file           0      148      0       $float_type_complete     $meta_filename" >>$reportfile
          err_148_cnt=`expr $err_148_cnt + 1`
          is_failure=1;;
   998) echo "$msg_file           0      998        0       $float_type_complete     $meta_filename" >>$reportfile
        err_998_cnt=`expr $err_998_cnt + 1`
        is_failure=1;;
   999) echo "$msg_file           0      999        0       $float_type_complete     $meta_filename" >>$reportfile
        err_999_cnt=`expr $err_999_cnt + 1`
        is_failure=1;;
  esac

  return
}
#---------------------------------------------------------------------------------------
# Function WriteFinalReportSummary
# ================================ 
# Write the following Summaries to the 'reportfile' 
#
#  . Subtotals for 'success_cnt', 'failure_cnt' and 'warning_cnt',  and its total
#  , Overview by failure type
#  . Summary of PHY Files Processed by DiffCnt Program
#  . Exit Status Explanation
#  . Difference Status Table and tally for: Identical, Different and Failures
#  . List of PHY Files Identical To Previously Generated Phy Files 
#
# Usage: WriteFinalReportSummary
#---------------------------------------------------------------------------------------
WriteFinalReportSummary()
{
  if [ -s $emptymsg_list ]
  then
    emptymsg=1
    warning_cnt=`expr $warning_cnt + 1`
  else
    emptymsg=0
    rm -f $emptymsg_list
  fi

  total=`expr $success_cnt + $failure_cnt + $warning_cnt`
  echo "\n      Subtotals        $success_cnt        $failure_cnt        $warning_cnt">>$reportfile
  echo   "          Total        $total">>$reportfile

  echo "\nOverview by failure type:"                                  >>$reportfile
  echo "  Msg file not found:                           $err_11_cnt"  >>$reportfile
  echo "  Java - error opening input msg file:          $err_111_cnt" >>$reportfile
  echo "  floatno_iridiumid_list not found:             $err_12_cnt"  >>$reportfile
  echo "  AOML ID not found in floatno_iridiumid_list:  $err_13_cnt"  >>$reportfile
  echo "  More than one entry found in "                              >>$reportfile
  echo "     floatno_iridiumid_list for an iridium_id:  $err_14_cnt"  >>$reportfile
  echo "  Metafile not found:                           $err_15_cnt"  >>$reportfile
  echo "  Java - error opening meta file:               $err_115_cnt" >>$reportfile
  echo "  Float Type not found:                         $err_16_cnt"  >>$reportfile
  echo "  Java - Warning Float Type not implemented:    $wrn_116_cnt" >>$reportfile
  echo "  Technical Parameter file not found:           $err_17_cnt"  >>$reportfile
  echo "  Configuration Parameter file not found:       $err_18_cnt"  >>$reportfile
  echo "  Java - error opening techn parameter file:    $err_117_cnt" >>$reportfile
  echo "  Java - error opening config parameter file:   $err_118_cnt" >>$reportfile
  echo "  Java - error Wrong number of parameters in"                 >>$reportfile
  echo "         call to Java program:                  $err_131_cnt" >>$reportfile
  echo "  Java - error creating Java log file:          $err_132_cnt" >>$reportfile
  echo "  Java - error creating output phy file:        $err_133_cnt" >>$reportfile
  echo "  Java - error Missing/wrong meta file info:    $err_134_cnt" >>$reportfile
  echo "  Java - Warning Incomplete msg file:           $wrn_135_cnt" >>$reportfile
  echo "  Java - error opening error file:              $err_136_cnt" >>$reportfile
  echo "  Java - Configuration parameter not found:     $wrn_137_cnt" >>$reportfile
  echo "  Java - error opening low resolution table:    $err_140_cnt" >>$reportfile
  echo "  Java - Error opening High Resolution Table    $err_141_cnt" >>$reportfile
  echo "  Java - Element in the Low Res Table/or"                     >>$reportfile 
  echo "         High Res is missing                    $err_142_cnt" >>$reprotfile
  echo "  Java - Error Foil Matrix Does Not Exist,"                   >>$reportfile 
  echo "         Initialize a new one                   $err_143_cnt" >>$reportfile
  echo "  Java - Mismatch of Message File Headers with"               >>$reportfile
  echo "         the Low Resolution Table               $err_144_cnt" >>$reportfile
  echo "  Java - Error Error Row Or Column Values"                    >>$reportfile
  echo "         missing in Foil Matrix                 $err_145_cnt" >>$reportfile
  echo "  Java - Error Row Or Column Values of Foil"                  >>$reportfile 
  echo "         Matrix of the meta file do not"                      >>$reportfile         
  echo "         match that of  Table                   $err_146_cnt" >>$reportfile
  echo "  LowRes_Table.{float_type} exists but"                       >>$reportfile
  echo "         HighRes_table.{float_type} does not"                 >>$reportfile 
  echo "         exist                                  $wrn_147_cnt" >>$reportfile
  echo "  Java - Error Variable not Found in Table      $err_148_cnt" >>$reportfile
  echo "  I/O error in Java program:                    $err_998_cnt" >>$reportfile
  echo "  Execution error in Java program:              $err_999_cnt" >>$reportfile

  if [ -s $diffStatusTable ]
  then
    echo "\n\n" >> $reportfile
    echo "  +++++ Summary Report of PHY Files Processed by DiffCnt Program  +++++" >> $reportfile
    echo " " >> $reportfile
    echo " =======================================================================" >> $reportfile
    echo " Exit Status Explanation" >> $reportfile
    echo " 1, files are different" >> $reportfile
    echo " 2, float is potentially a surface drifter" >> $reportfile
    echo " 3, error in passing parameters to DiffCnt" >> $reportfile
    echo " 4, error parsing Date-Time, or IO error" >> $reportfile
    echo " 5, profile number problem" >> $reportfile

    echo " =======================================================================\n" >> $reportfile
    echo "  File Name          Exit Status"   >> $reportfile

    cat $diffStatusTable | sort -k2 | awk '{print " "$0}' >> $reportfile

    echo "\n Subtotals"  >> $reportfile
    echo " Identical:  $cntdiff_same_cnt" >> $reportfile
    echo " Different:  $cntdiff_different_cnt" >> $reportfile
    echo "   Failure:  $cntdiff_failure_cnt" >> $reportfile
    echo "     Total:  `expr $cntdiff_same_cnt + $cntdiff_different_cnt + $cntdiff_failure_cnt`" >> $reportfile
    echo " =======================================================================\n" >> $reportfile
  fi

  if [ -s $identicalList ]
  then
    echo "\n\n" >> $reportfile
    echo "  +++ List of PHY Files Identical To Previously Generated Phy Files +++" >> $reportfile
    echo " =======================================================================" >> $reportfile
    cat $identicalList | awk '{print " "$1}' >> $reportfile
    echo " \n Total: `wc -l <$identicalList`" >> $reportfile
    echo " =======================================================================\n" >> $reportfile
  fi

  echo "\nFor Details:" >>$reportfile
  echo "Log:       $logfile" >>$reportfile
  echo "Java log:  $javalogfile" >>$reportfile
  echo "Error:     $statusfile" >>$reportfile
  echo "Std Error: $stderrorfile" >>$reportfile
  echo "Report:    $reportfile" >>$reportfile

  if [ -s $emptymsg_list ]
  then
    echo "Empty msg: $emptymsg_list" >>$reportfile
  fi

  return
}
#---------------------------------------------------------------------------------------
# Function ComposeAndSendEmail 
# ============================
# Compose and send email for processing the message files. 
# 
# Usge: ComposeAndSendEmail 
#---------------------------------------------------------------------------------------
ComposeAndSendEmail()
{
  true_failure=`expr $failure_cnt - $err_15_cnt` 

  if [ $failure_cnt -eq 0 -a $warning_cnt -eq 0 ] 
  then
    if [ -s $stderrorfile ]
    then
      subj="$script_name - WARNING, $success_cnt msg files successfully processed but"
      subj="$subj standard-error detected"
      if [ $emptymsg -ne 0 ]
      then
        subj="$subj & empty msg"
      fi
    else
      subj="$script_name - SUCCESS, $success_cnt msg files successfully processed"
      if [ $emptymsg -eq 0 ]
      then
        subj="$script_name - SUCCESS, $success_cnt msg files successfully processed"
      else
        subj="$script_name - WARNING, $success_cnt msg files successfully processed"
        subj="$subj but empty msg found" 
      fi
    fi # end - stderrorfle
  else # failure_cnt=0 & warning_cnt=0 
    if [ -s $stderrorfile ]
    then
      outcome="Success=$success_cnt, Failure=$true_failure Warning=$warning_cnt NoMeta=$err_15_cnt"
      subj="$script_name - WARNING, $outcome"
      subj="$subj and standard-error detected"
      if [ $emptymsg -ne 0 ]
      then
        subj="$subj & empty msg"
      fi
    else
      outcome="Success=$success_cnt Failure=$true_failure Warning=$warning_cnt NoMeta=$err_15_cnt"
      subj="$script_name - WARNING, $outcome"
      if [ $emptymsg -ne 0 ]
      then
        subj="$subj & empty msg" 
      fi
    fi # end - stderrorfile 
  fi # end failure_cnt=0 & warning_cnt=0 

  echo "=> [Email Begin]"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Subject: $subj"
  echo "   Text:"

  if [ -s $stderrorfile ]
  then
    cat $stderrorfile >$email_msg_file
  else
    touch $email_msg_file
  fi

  sed s/" Files Processed"/" Files Not Processed"/  $reportfile | grep -v "   1        0       " | grep -v "\.phy       1" >> $email_msg_file
   
  if [ $emptymsg -ne 0 ]
  then
    echo "List of empty 'msg' file(s):"   >> $email_msg_file
    awk '{print "   " $0}' $emptymsg_list >> $email_msg_file
  fi

  awk '{print "   " $0}'  $email_msg_file      # email body

  if [ $debug = 0 ]
  then
    printf "   Sending - "
    mailx -s "$subj" $EMAIL_OPERATOR < $email_msg_file
    echo "OK"
  else
    echo "   debug is ON, do not send mail"
  fi
  echo "=> [Email End]"

  rm -f $email_msg_file

  return
}
# ----------------------------   Global Variables     ---------------------------- 
. $HOME/.argobetarc_sh
#EMAIL_OPERATOR=Reyna.Sabina@noaa.gov
EMAIL_OPERATOR="$EMAIL_OPERATOR Charita.Atluri@noaa.gov"
export EMAIL_OPERATOR
# ---------------------------- Main (msg2phy_process) -----------------------------
script_name=`basename $0`
root_script_name=`echo $script_name | awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

TODAY=`date +'%Y.%m.%d_%H.%M'`
now=`date +'%h %d, %Y %T %p'`

ARGOPHY_DIR=$ARGOBETA_DIR/data/phy             # where the output phy files are going to be left
IRIDIUM_DIR=$ARGOBETA_DIR/iridium              # iridium directory
MSG2PHY_DIR=$IRIDIUM_DIR/msg2phy               # msg2phy directory
MSG2PHYPRG_DIR=$MSG2PHY_DIR/prg                # where the Java programs are
IRIDIUMMSG_DIR=$IRIDIUM_DIR/data/msg           # where the input msg files are
IRIDIUMMSG_DONE_DIR=$IRIDIUMMSG_DIR/done       # where the input msg files are
MSG2PHYMSG_TABLE=$MSG2PHY_DIR/tables           # where the tables are
MSG2PHYLOG_DIR=$MSG2PHY_DIR/log                # where the log, error files are
MSG2PHYTMP_DIR=$MSG2PHY_DIR/tmp                # where the temp files are
METAFILES_DIR=$ARGOBETA_DIR/meta               # where the meta files are
STSFILES_DIR=$IRIDIUM_DIR/data/sts             # where the sts files are
ARGOSPRGJAVA_DIR=$ARGOBETA_DIR/argos/prg/java  # where the DiffCnt.java program is
export ARGOPHY_DIR MSG2PHYPRG_DIR IRIDIUMMSG_DIR MSG2PHYTMP_DIR MSG2PHYMSG_TABLE METAFILES_DIR
export ARGOSPRGJAVA_DIR STSFILES_DIR

logfile=$MSG2PHYLOG_DIR/$root_script_name.$TODAY"_log"            # script logfile
tracefile=$MSG2PHYLOG_DIR/$root_script_name.$TODAY"_trace"        # script tracefile
javalogfile=$MSG2PHYLOG_DIR/$root_script_name.$TODAY"_javalog"    # Java programs log file 
stderrorfile=$MSG2PHYLOG_DIR/$root_script_name.$TODAY"_stderror"  # standard error
statusfile=$MSG2PHYLOG_DIR/$root_script_name.$TODAY"_status"      # error
reportfile=$MSG2PHYLOG_DIR/$root_script_name.$TODAY"_report"      # report file
phy2bprocessed_list=$ARGOBETA_DIR/lists/cnt2phy.$TODAY"_list"     # success files are added to a list to be later QCed.
diffStatusTable=$MSG2PHYLOG_DIR/DiffStatus.$TODAY"_"table         # table with DiffCnt results
identicalList=$MSG2PHYLOG_DIR/Identical.$TODAY"_"table            # list of phy files generated identical to previously
                                                                  # generated phy file in phy/done
exec 1>$logfile
exec 2>$stderrorfile

touch $logfile 
touch $tracefile
touch $javalogfile
touch $statusfile
touch $reportfile
touch $phy2bprocessed_list
touch $identicalList

echo "<++++++++++++++++++++ Start  $script_name: $now +++++++++++++++++++>"
if [ "`$PATH_WHOAMI/whoami`" = "argo" ]
then
  EMAIL_OPERATOR="$EMAIL_OPERATOR Vicki.Halliwell@noaa.gov"
  echo "EMAIL_OPERATOR (expanded): $EMAIL_OPERATOR"
  export EMAIL_OPERATOR
fi

#----------------------------------------------------------------------------------------
#                  Write Java Exit Status Codes to the 'javalogfile' 
#----------------------------------------------------------------------------------------
WriteJavaExitStatusCodes

debug=0   # 0=debug is off, 1=debug is on, 2=debug is on + write edf lines to log

emptymsg_list=$MSG2PHYTMP_DIR/emptymsg_list.$TODAY # empty msg file(s); moved to 'failure & not processed 
msgfiles_list=$MSG2PHYTMP_DIR/msgfiles_list        # msg files to be processed
email_msg_file=$MSG2PHYTMP_DIR/email_msg_file      # email body of the msg 

#----------------------------------------------------------------------------------------
#                       Get list of message files to process
#----------------------------------------------------------------------------------------
ls $IRIDIUMMSG_DIR/*.msg | awk -F/ '{print $7}' > $msgfiles_list
if [ ! -s "$msgfiles_list" ]                       # Any message file to process? 
then                                               #  No, sent email & exit
  ret_val=1
  subj="$script_name - No MSG Files to Process"
  msg="No msg files to process"
  echo $msg > $email_msg_file

  echo "=> [Email Begin]"
  echo "   To: $EMAIL_OPERATOR"
  echo "   Subject: $subj"
  echo "   Text:"
  awk '{print "   " $0}' $email_msg_file

  if [ $debug =  0 ]
  then 
    printf "   Sending - "
    mailx -s "$subj" $EMAIL_OPERATOR < $email_msg_file
    echo "OK"
  else
    echo "debug is ON, do not send mail"
  fi

  rm -f $email_msg_file
  now=`date +'%Y/%m/%d %H:%M:%S'`
  echo "<+++++++++++++++++++ Terminated $script_name: $now +++++++++++++++++>" 
  exit $ret_val   # no MSG file to process
fi

echo "List of 'msg' file(s) to process located in: $msgile_list:"
awk '{print "  " $0}'  $msgfiles_list

#----------------------------------------------------------------------------------------
#                Initialize flags and set possible exit status outcome
#----------------------------------------------------------------------------------------
InitFlagsAndError

#----------------------------------------------------------------------------------------
#            Write the 'Status Report Heading' to the 'reportfile'
#----------------------------------------------------------------------------------------
WriteReportHeading

#----------------------------------------------------------------------------------------
# Creates a float list(MSG2PHYTMP_DIR/floatname_pos.gpslist for
# each float with those profiles that has a GPS fix. The list
# is accessed by 'InterpolateGpsFixes' (called from Msg2Phy_Ir.java) 
# for interpolation of the profiles with missing GPS fixes
#----------------------------------------------------------------------------------------
GetMsgLst_GPSFixes

#----------------------------------------------------------------------------------------
#            Loop over the list of 'msg'  file(s) to process 
#           Empty file is move to 'failure' and not processed 
#----------------------------------------------------------------------------------------
wrn_147=147  # LowRes_Table.{float_type} exists but HighRes_Table.{float_type} 
             # does not exist
touch $emptymsg_list
echo "                <<<<< Loop to  Process 'msg' File(s) >>>"
exec < $msgfiles_list
while read msg_file
do
   now=`date +'%Y/%m/%d %H:%M:%S'`

   if [ ! -s $IRIDIUMMSG_DIR/$msg_file ]   # Is empty?
   then                                    #   Yes
     echo "$msg_file - Empty -> Move to 'failure' - Skip process"  
     mv $IRIDIUMMSG_DIR/$msg_file $IRIDIUMMSG_DIR/failure/
     echo $msg_file >> $emptymsg_list
     echo "$msg_file           0        0        1       Empty Msg      $meta_filename" >> $reportfile
     continue
   else
     echo "$script_name $msg_file -> Begin process at $now"
   fi 

#----------------------------------------------------------------------------------------
#                      Cleanup some temporal files
#----------------------------------------------------------------------------------------
  if [ -f $MSG2PHYTMP_DIR/float_type ]
  then
    rm $MSG2PHYTMP_DIR/float_type
  fi  
  if [ -f $MSG2PHYTMP_DIR/meta_filename ]
  then
    rm $MSG2PHYTMP_DIR/meta_filename
  fi  
  if [ -f $MSG2PHYTMP_DIR/phy_filename ]
  then
    rm $MSG2PHYTMP_DIR/phy_filename
  fi  

#---------------------------------------------------------------------------------------
#                Calling the function that process a message (msg) file
#---------------------------------------------------------------------------------------
  ProcessOneMsgFile $msg_file
  status_process_msgfile=$? 
  echo ">>> status_process_msgfile=$status_process_msgfile <<<"
  if [ $status_process_msgfile -eq $wrn_147 ]
  then
    CheckWaringProcess_MsgFile  $wrn_147
    continue
  fi

  identical=0                       
  is_failure=0
  if [ -f "$ARGOPHY_DIR/done/$phy_filename" -a -f "$ARGOPHY_DIR/done/$phy_filename" ] # Is new phy identical to old phy?   
  then 
    diff $ARGOPHY_DIR/$phy_filename $ARGOPHY_DIR/done/$phy_filename > /dev/null 
    diff_status="$?"
    if [ $diff_status -eq 0 ]                          
    then
      echo $phy_filename >> $identicalList
      identical=1
      cntdiff_same_cnt=`expr $cntdiff_same_cnt + 1`
      now=`date +'%Y/%m/%d %H:%M:%S'`
      echo "$script_name $now: $phy_filename same as in $ARGOPHY_DIR/done."
      echo "$script_name $now: $phy_filename same as in $ARGOPHY_DIR/done." >> $statusfile
      rm $IRIDIUMMSG_DIR/$msg_file
      echo "$script_name $now: $IRIDIUMMSG_DIR/$msg_file removed." 
      echo "$script_name $now: $IRIDIUMMSG_DIR/$msg_file removed."  >> $statusfile
      rm $ARGOPHY_DIR/$phy_filename
      echo "$script_name $now: $ARGOPHY_DIR/$phy_filename removed." 
      echo "$script_name $now: $ARGOPHY_DIR/$phy_filename removed."  >> $statusfile
      echo "$script_name $now: End processing $msg_file with no status"
      echo "$script_name $now: End processing $msg_file with no status" >> $statusfile
    else
      ExecDiffCnt                                                                    # No, Execute DiffCnt 
    fi
  fi

  if [ $identical -eq 0 -a $is_failure -eq 0 ]
  then 
    now=`date +'%Y/%m/%d %H:%M:%S'`
    echo "$script_name $now: End processing $msg_file with status(ProcessOneMsgFile)=$status_process_msgfile" 

    if [ -f $MSG2PHYTMP_DIR/float_type ]
    then
      float_type=`cat $MSG2PHYTMP_DIR/float_type`
    else
      float_type="UNKNOWN"
    fi
    if [ -f $MSG2PHYTMP_DIR/meta_filename ]
    then
      meta_filename=`cat $MSG2PHYTMP_DIR/meta_filename`
    else
      meta_filename="UNKNOWN"
    fi 
    if [ -f $MSG2PHYTMP_DIR/phy_filename ]
    then
      phy_filename=`cat $MSG2PHYTMP_DIR/phy_filename`
    else
      phy_filename="UNKNOWN"
    fi

#---------------------------------------------------------------------------------------
#    Checks the exit status of the 'ProcessOneMsgFile' to setup appropiate flag(s)
#                  status_process_msgfile: exit status        
#---------------------------------------------------------------------------------------
    CheckStatusProcess_MsgFile $status_process_msgfile

    if [ $is_failure -eq 1 ]
    then
      if [ $debug = 0 ]
      then
        mv $IRIDIUMMSG_DIR/$msg_file $IRIDIUMMSG_DIR/failure/
      else
        echo "Debug is ON, do not move msg/phy files to failure"
      fi
      failure_cnt=`expr $failure_cnt + 1`
      if [ -f "$ARGOPHY_DIR/$phy_filename" ]
      then
        rm $ARGOPHY_DIR/$phy_filename
        now=`date +'%Y/%m/%d %H:%M:%S'`
        echo "$script_name $now: $ARGOPHY_DIR/$phy_filename file removed" 
      fi 
    else
      CheckWaringProcess_MsgFile  $status_process_msgfile 
    fi
  fi
done

#---------------------------------------------------------------------------------------
#             Write the following Summaries to the 'reportfile' 
#
#    . Subtotals for 'success_cnt', 'failure_cnt' and 'warning_cnt',  and its total
#    , Overview by failure type
#    . Summary of PHY Files Processed by DiffCnt Program
#    . Exit Status Explanation
#    . Difference Status Table and tally for: Identical, Different and Failures
#    . List of PHY Files Identical To Previously Generated Phy Files 
#---------------------------------------------------------------------------------------
WriteFinalReportSummary

#---------------------------------------------------------------------------------------
# Compose and send email for processing the message files
#---------------------------------------------------------------------------------------
ComposeAndSendEmail

#---------------------------------------------------------------------------------------
#                Remove empty logfiles & Identical Table if Empty 
#---------------------------------------------------------------------------------------
if [ ! -s $tracefile ]
then
  rm -f $tracefile
fi

if [ ! -s $stderrorfile ]
then
  rm -f $stderrorfile
fi

if [ ! -s $statusfile ]
then
  rm -f $statusfile
fi

if [ ! -s $identicalList ]
then
  rm -f $identicalList

fi

now=`date +'%h %d, %Y %T %p'`
echo "<+++++++++++++++++++ Terminated $script_name: $now +++++++++++++++++>" 
exit 0
