#!/bin/sh
##############################################################################################
# Revision Date: 28-Apr-14 (RS - if incoming file alreasy exists then current one is moved
#                                to 'old'. The 'duplicate' director is renamed to 'old') 
#                11-Feb-13 (RS)
#
# mv_Iridium_log_srf_sts2aomlid.sh 
# ================================
# Moves incoming files of type 'log, srf and sts' in DATA_DIR to correspondig 'type'
# subdirectory. 
#
# It performs the following tasks for each 'type':
#
#  1) Checks for the file 'infileid' in the following "Iridium Floats List" to obtain 
#     the 'aomlid'.
#            META_DIR/floatno_iridiumid_list 
#     If the 'infileid' is not found, it is added to the following list:      
#            TMP_DIR/notfound_aomlid.[type]_list
#  2) If DATA_DIR/{aomlid} directory is not found in DATA_TYPE/{aomlid}, it is created   
#  3) Checks if an incoming 'type' file exists in DATA_TYPE/{aomlid}. If it is found 
#     do as follows:
#      . Its name is added to: 
#              TMP_DIR/existing_{TODAY}.[type]_list
#      .  Existing 'type' file is moved to: 
#              DATA_TYPE/OLDFILES_DIR 
#      .  The 'incoming' 'type' file is moved to: 
#              DATA_TYPE_DIR/{aomlid}
#
# Usage
# -----
#           IRIDIUM_DIR/scripts mv_Iridium_log_srf_sts2aomlid.sh
#
# Definitions
# -----------
# ARGO_DIR: defined in ~/.argorc_sh
# META_DIR: ARGO/meta
# IRIDIUM_DIR: ARGO_DIR/iridium
# MSG2PHY_DIR: IRIDIUM_DIR/msg2phy
# DATA_DIR: IRIDIUM_DIR/data
# DATA_TYPE_DIR: DATA_DIR/[type]
# OLDFILES_DIR=DATA_TYPE_DIR/old
# TMP_DIR: MSG2PHY_DIR/tmp 
##############################################################################################
. $HOME/.argorc_sh
#---------------------------------------------------------------------------------------------
# SendEmail
# --------- 
#---------------------------------------------------------------------------------------------
SendEmail()
{
  status_exec=`expr $noaomlid + $existing`
  echo "=> status_exec: $status_exec"

  case $status_exec in
  0) title="$up_script_name - OK, All '$type' file(s) moved"
     body="Files (*.$type) moved to respective 'aomlid'." 
     echo "$body" > $email_msg;;
  1) title="$up_script_name - WARNING, Missing 'aomlid' found for '$type' file(s)" 
     body="For '$type' file(s) with missing 'aomlid' was/were not found in:"
     body="$body\n       $floatno_iridiumid_list\n   See: $notfound_aomlid_list"
     body="$body\nSee Logfile: $logfile"
     echo "$body" > $email_msg;;
  2) title="$up_script_name - WARNING, Existing '$type' file(s) found" 
     body="Existing '$type' file(s) found.\n   See: $existing_list"
     body="$body\nSee Logfile: $logfile"
     echo "$body" > $email_msg;;
  3) title="$up_script_name - WARNING, Missing 'aomlid' found or Existing one(s) found for '$type' file(s)"
     body="For '$type' file(s) missing 'aomlid' was/were found in:"
     body="$body\n       $floatno_iridiumid_list\n   See: $notfound_aomlid_list"
     body="$body\nFor existing file(s) found.\n   See: $existing_list "
     body="$body\nSee Log File:\n    $logfile"
     echo "$body" > $email_msg;;
esac

  if [ -s $error ]
  then
    if [ $status_exec -eq 0 ]
    then
      title=`echo $title | sed s/OK/FAILURE/`
      subj="$title but Standard-Error reported"
    else
      subj="$title. Standard-Error reported"
    fi
    echo "See Error File:\n    $error" >> $email_msg
  else
    subj=$title
    rm -f $error
  fi

  echo "=> [Email Begin]"
  echo "   Subj: $subj"
  echo "   To:   $EMAIL_OPERATOR"
  echo "   Text:"
  cat $email_msg | awk '{print "   " $0}'
  printf "   Sending - "
  mailx -s "$subj" $EMAIL_OPERATOR < $email_msg
  echo "OK"
  echo "=> [Email End]"
  return
}
#------------------------------------------- Main ---------------------------------------------
TODAY=`date +'%Y.%m.%d_%H.%M'`
script_name=`basename $0`
root_script_name=`echo $script_name| awk -F"." '{print $1}`
up_script_name=`echo $root_script_name | tr "[:lower:]" "[:upper:]"`

VALID_DATA_TYPES="log srf sts"

IRIDIUM_DIR=$ARGO_DIR/iridium
DATA_DIR=$IRIDIUM_DIR/data
MSG2PHY_DIR=$IRIDIUM_DIR/msg2phy
SCRIPTS_DIR=$MSG2PHY_DIR/scripts
LOG_DIR=$MSG2PHY_DIR/log
TMP_DIR=$MSG2PHY_DIR/tmp

log_main=$LOG_DIR/$root_script_name"_$TODAY".log
error_maim=$LOG_DIR/$root_script_name."_$TODAY".error

exec 2> $error_main

floatno_iridiumid_list=$ARGO_DIR/meta/floatno_iridiumid_list

echo "<---------------------------------- Begin Main  ----------------------------->"  > $log_main
echo "   ARGO_DIR: $ARGO_DIR"         >> $log_main
echo "   IRIDIUM_DIR: $IRIDIUM_DIR"   >> $log_main
echo "   DATA_DIR: $DATA_DIR"         >> $log_main 
echo "   MSG2PHY_DIR: $MSG2PHY_DIR"   >> $log_main
echo "   SCRIPTS_DIR: $SCRIPTS_DIR"   >> $log_main
echo "   LOG_DIR: $LOG_DIR"           >> $log_main
echo "   TMP_DIR: $TMP_DIR"           >> $log_main

# --------------------------------------------------------------------------------------------"
#                    Data Type Processing Loop"
# --------------------------------------------------------------------------------------------"
for type in `echo $VALID_DATA_TYPES`
do
  noaomlid=0
  existing=0
  warning=0

  DATA_TYPE_DIR=$DATA_DIR/$type
  OLDFILES_DIR=$DATA_TYPE_DIR/old

  incoming_type_list=$TMP_DIR/incoming.$type"_list"

  notfound_aomlid_list=$TMP_DIR/notfound_aomlid"_$TODAY".$type"_list"
  existing_list=$TMP_DIR/existing"_$TODAY".$type"_list"

  email_msg=$LOG_DIR/$root_script_name"_$TODAY".$type"_email"
  logfile=$LOG_DIR/$root_script_name"_$TODAY".$type"_log"
  error=$LOG_DIR/$root_script_name"_$TODAY".$type"_error"

  exec 1> $logfile
  exec 2> $error

  echo "        ********** Begin Move Data Type: $type **********" >> $log_main 
  echo "                   ********** Begin Move Data Type: $type **********" 

  echo "   DATA_TYPE_DIR: $DATA_TYPE_DIR" >> $log_main
  echo "   OLDFILES_DIR: $OLDFILES_DIR"   >> $log_main

  echo "   DATA_TYPE_DIR: $DATA_TYPE_DIR"
  echo "   OLDFILES_DIR: $OLDFILES_DIR"

  echo "=> Logfiles"
  echo "   email_msg: $email_msg"
  echo "   logfile: $logfile"
  echo "   error: $error"

  echo "=> Lists"
  echo "   floatno_iridiumid_list: $floatno_iridiumid_list"
  echo "   incoming_type_list: $incoming_type_list"
  echo "   notfound_aomlid_list: $notfound_aomlid_list"
  echo "   existing_list: $existing_list"

  printf "   Change Dir to '$DATA_TYPE_DIR' - "
  cd $DATA_TYPE_DIR
  echo "OK"

  ls *.$type 2> /dev/null | sort -u  > $incoming_type_list
  n_files=`wc -l $incoming_type_list | awk '{print $1}'`

  echo "   Number of incoming '$type' files: $n_files" 

  if [ $n_files -eq 0 ]
  then
    echo "   <<<<< No Profiles to be moved found -> Exit >>>>>"
    continue 
    echo "=> Finished '$type' data"
    continue
  fi

  echo "   Pocessing Loop for '$type' files" >> $log_main
  echo "=> Pocessing Loop for '$type' files"

  for infile in `cat $incoming_type_list` 
  do
    infileid=`echo $infile | awk -F'.' '{print $1}'`
    aomlid=`grep $infileid $floatno_iridiumid_list | awk '{print $1}'`
    pathname_aomlid=$DATA_TYPE_DIR/$aomlid
    printf "   $infile aomlid:"

    if [ "$aomlid" = "" ]
    then
      warning=`expr $warning + 1` 
      noaomlid=1
      echo " Not Found"
      echo $infileid  >> $notfound_aomlid_list
      continue
    else
      printf $aomlid
    fi

    printf " ./data/$type/$aomlid Exists? "   
    if [ -d $pathname_aomlid ]
    then
      printf "Yes;"
    else
      printf "No, create it - "
      mkdir $pathname_aomlid
      printf "OK;"
    fi

    if [ -f $pathname_aomlid/$infile ]
    then 
      warning=`expr $warning + 1` 

#? duplicate=2
#? printf " mv -> $DUPLICATES_DIR - "
#? mv $infile $DUPLICATES_DIR

      existing=2
      printf " mv Old -> ./data/$type/old  - "
      mv  $pathname_aomlid/$infile  $OLDFILES_DIR
      printf "OK" 
      printf "; mv New -> ./data/$type/$aomlid - "
      mv $infile $pathname_aomlid 
      echo "OK"
    else
      printf " mv New -> ./data/$type/$aomlid - "
      mv $infile $pathname_aomlid 
      echo "OK"
    fi
  done 

  echo "   Number of Warnings=$warning" >> $log_main
  echo "   Number of Warnings=$warning" 

  SendEmail

  echo "          ******** End  Move Data Type: $type **********" >> $log_main 
  echo "                   ********** End  Move Data Type: $type *******" 
done

echo "<---------------------------------- End  Main  ----------------------------->" >> $log_main

exit
