/**
This is the Java program to read NetCDF file and convert it to ARGO PHY file format

Author: Xiangdong Xia
Date  : November 2002

To run it,for example in Windows:
java -Dphy_dir=c:\xxd\netcdf\GetEnv\ NetcdfReader netcdf_file_name meta_data_file phy_file_name_container

in Unix:
java -Dphy_dir=$PHY_DIR NetcdfReader netcdf_file_name meta_data_file phy_file_name_container

Exit Codes
----------

#   1, OK
#   2, WMO ID in 'nc' file is a 'fill' value (0)
#   3, WMO number not found in meta files
#   4, WMO ID in 'nc' file name differs from WMO ID inside the 'nc' file
#   5, WMO ID in 'nc' file can not be aphanumeric
#   6, WMO instrument type from meta file and 'nc' file do not match
#   7, WMO number from 'meta' file and 'nc' file do not match.
#   8, Number of columns is not 3 (p, t, c)
#   9, Profile direction (up/down) is inconsistant with the file name convention
#  10, Magnitude of data value(s) exceed format specifications  
#  11, Not a NetCDF file
*/

import ucar.ma2.*;
import ucar.nc2.*;
import java.io.IOException;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.*;
import java.io.*;

public class NetcdfReader {

  ArrayList pres,temp,cndc,psal,juld,juld_location,reference_date_time,latitude,longitude,direction;
  ArrayList cycle_number,platform_number,wmo_inst_type,positioning_system,date_creation,juld_qc,position_qc;;
  ArrayList Pres,Temp,Cndc;//original P,T,C
  
  String WMO_INST,WMO_REC,FLOAT_NUMBER,FLOAT_WMO_NUMBER;
  String INSTRUMENT_TYPE,PRINCIPAL_INVESTIGATOR,TRANSM_ID;
  String PLATFORM_NUMBER="",WMO_INST_TYPE="",POSITIONING_SYSTEM="",REFERENCE_DATE_TIME="",CYCLE_NUMBER="";
  String DATE_CREATION="",JULD_QC="",DIRECTION="",POSITION_QC="";
  String WMOID_in_Filename="",D_Filename="";  
  String calibration="";//store calibration string lines from meta file to phy output file
  
  float  LATITUDE,LONGITUDE;
  int year1,month1,day1,hour1,minute1,second1;
  int year2,month2,day2,hour2,minute2,second2;
  
  static String phy_dir="";//PHY_DIR directory path environment variable
  String outFileName="";
  static String phyNameFile="";//The filename which contains only one line for the phy file name, to be used in shell script
  String purePhyFileName="";//phy filename without path
    
  int    columns=0;//check to see if there are 3 columns there(P,T,C)

  Array a;


  public static void main(String[] args) {
        if (args.length == 3){
        	System.out.println("Entering program NetcdfReader......"+"\n\n");
        	System.out.println("Reading Netcdf File..."+args[0]+"\n\n"); 
     	    NetcdfReader ncr=new NetcdfReader(args[0]);//retrive P,T,C from NetCDF file
            ncr.ReadMetaDataFile(args[1]);//read information from meta data file
            phy_dir=System.getProperty("phy_dir");
            ncr.getOutFileName();//get output filename
            phyNameFile=args[2];
            //convert to physical unit data            
            if(ncr.Convert()!=0){
				System.out.println("Converting to physical unit goes wrong!,program exit now");
				System.exit(-1);
		    }
		    
            System.out.println("Exiting program NetcdfReader...");
            System.exit(1);//OK 
		}
        else {
			System.out.println("You input wrong command to excute!"+"\n");
            System.out.println("Format: java -Dphy_dir=$PHY_DIR NetcdfReader netcdf_file_name meta_data_file phy_file_name_container"+"\n");
        }
  }

  public NetcdfReader(String fileName) {

    try {
      NetcdfFile nc = new NetcdfFile(fileName); // open it readonly           
      //System.out.println(nc); // output schema in CDL form (like ncdump)
      //System.out.println("data:");
      Iterator vi = nc.getVariableIterator();
      while(vi.hasNext()) {
        Variable v = (Variable) vi.next();
        a = v.read();
        //System.out.print(v.getName() + " = ");
        //System.out.println(a.getElementType());

          if(v.getName().equals("REFERENCE_DATE_TIME")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             reference_date_time=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println(val);
			      reference_date_time.add(i++,new Character(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<reference_date_time.size();j++){
			REFERENCE_DATE_TIME+=reference_date_time.get(j);
    	    }
            REFERENCE_DATE_TIME=REFERENCE_DATE_TIME.trim();
            System.out.println("REFERENCE_DATE_TIME:"+REFERENCE_DATE_TIME);
	      }//end of if


          if(v.getName().equals("CYCLE_NUMBER")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             cycle_number=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      int val = iter.getCharNext();//System.out.println(val);
			      cycle_number.add(i++,new Integer(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<cycle_number.size();j++){
            CYCLE_NUMBER=CYCLE_NUMBER+cycle_number.get(j);            
    	    }
            CYCLE_NUMBER=CYCLE_NUMBER.trim();
            
            System.out.println("CYCLE_NUMBER:"+CYCLE_NUMBER);

	      }//end of if

          if(v.getName().equals("DIRECTION")){
			 
             IndexIterator iter = a.getIndexIterator();
             direction=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println("char direction="+val);
			      direction.add(i++,new Character(val));//System.out.println(al);
			  }
                        
            
            //display the list
            for(int j=0;j<direction.size();j++){
            DIRECTION=DIRECTION+direction.get(j);            
    	    }
    	    
            DIRECTION=DIRECTION.trim();
            
            System.out.println("DIRECTION:"+DIRECTION);

	      }//end of if

          if(v.getName().equals("POSITION_QC")){
			 
             IndexIterator iter = a.getIndexIterator();
             position_qc=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println("char position_qc="+val);
			      position_qc.add(i++,new Character(val));//System.out.println(al);
			  }
                        
            
            //display the list
            for(int j=0;j<position_qc.size();j++){
            POSITION_QC=POSITION_QC+position_qc.get(j);            
    	    }
    	    
            POSITION_QC=POSITION_QC.trim();
            
            System.out.println("POSITION_QC:"+POSITION_QC);

	      }//end of if


          if(v.getName().equals("PLATFORM_NUMBER")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             platform_number=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println(val);
			      platform_number.add(i++,new Character(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<platform_number.size();j++){
            PLATFORM_NUMBER+=platform_number.get(j);
    	    }
    	    PLATFORM_NUMBER=PLATFORM_NUMBER.trim();
            System.out.println("PLATFORM_NUMBER:"+PLATFORM_NUMBER);
	      }//end of if


          if(v.getName().equals("WMO_INST_TYPE")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             wmo_inst_type=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println(val);
			      wmo_inst_type.add(i++,new Character(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<wmo_inst_type.size();j++){
			WMO_INST_TYPE+=wmo_inst_type.get(j);
    	    }
    	    WMO_INST_TYPE=WMO_INST_TYPE.trim();
            System.out.println("WMO_INST_TYPE:"+WMO_INST_TYPE);
	      }//end of if


          if(v.getName().equals("POSITIONING_SYSTEM")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             positioning_system=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println(val);
			      positioning_system.add(i++,new Character(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<positioning_system.size();j++){
			POSITIONING_SYSTEM+=positioning_system.get(j);
    	    }
    	    POSITIONING_SYSTEM=POSITIONING_SYSTEM.trim();
    	    System.out.println("POSITIONING_SYSTEM:"+POSITIONING_SYSTEM);

	      }//end of if

//HERE added DATE_CREATION and JULD_QC requestd by Claudia.
          if(v.getName().equals("DATE_CREATION")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             date_creation=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println(val);
			      date_creation.add(i++,new Character(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<date_creation.size();j++){
			DATE_CREATION+=date_creation.get(j);
    	    }
    	    DATE_CREATION=DATE_CREATION.trim();
            System.out.println("DATE_CREATION:"+DATE_CREATION);
	      }//end of if


          if(v.getName().equals("JULD_QC")){
			 //System.out.print(v.getName() + " =");
             //System.out.println(a.getElementType());
             IndexIterator iter = a.getIndexIterator();
             juld_qc=new ArrayList(); int i=0;
			  while(iter.hasNext()) {

			      char val = iter.getCharNext();//System.out.println(val);
			      juld_qc.add(i++,new Character(val));//System.out.println(al);
			  }

            //display the list
            for(int j=0;j<juld_qc.size();j++){
			JULD_QC+=juld_qc.get(j);
    	    }
    	    JULD_QC=JULD_QC.trim();
            System.out.println("JULD_QC:"+JULD_QC);
	      }//end of if


         if(v.getName().equals("PRES")){
			          columns++;
		 			  //System.out.print(v.getName() + " =");
		              //System.out.println(a.getElementType());
		              IndexIterator iter = a.getIndexIterator();
		              Pres=new ArrayList(); int i=0;
		 			  while(iter.hasNext()) {

		 			      float val = iter.getFloatNext();//System.out.println(val);
		 			      Pres.add(i++,new Float(val));//System.out.println(al);
		 			  }
/*
		             //display the list
		             for(int j=0;j<Pres.size();j++){
		             System.out.println(Pres.get(j));
		     	    }
*/
	      }//end of if


         if(v.getName().equals("TEMP")){
			                  columns++;
		 		 			  //System.out.print(v.getName() + " =");
		 		              //System.out.println(a.getElementType());
		 		              IndexIterator iter = a.getIndexIterator();
		 		              Temp=new ArrayList(); int i=0;
		 		 			  while(iter.hasNext()) {

		 		 			      float val = iter.getFloatNext();//System.out.println(val);
		 		 			      Temp.add(i++,new Float(val));//System.out.println(al);
		 		 			  }
/*
		 		             //display the list
		 		             for(int j=0;j<Temp.size();j++){
		 		             System.out.println(Temp.get(j));
		 		     	    }
*/
	      }//end of if


	      if(v.getName().equals("CNDC")){
			                  columns++;
		  		 			  //System.out.print(v.getName() + " =");
		  		              //System.out.println(a.getElementType());
		  		              IndexIterator iter = a.getIndexIterator();
		  		              Cndc=new ArrayList(); int i=0;
		  		 			  while(iter.hasNext()) {

		  		 			      float val = iter.getFloatNext();//System.out.println(val);
		  		 			      Cndc.add(i++,new Float(val));//System.out.println(al);
		  		 			  }
/*
		  		             //display the list
		  		             for(int j=0;j<Cndc.size();j++){
		  		             System.out.println(Cndc.get(j));
		  		     	    }
*/

	      }//end of if


	      if(v.getName().equals("JULD")){
		  		 			  //System.out.print(v.getName() + " =");
		  		              //System.out.println(a.getElementType());
		  		              IndexIterator iter = a.getIndexIterator();
		  		              juld=new ArrayList(); int i=0;
		  		 			  while(iter.hasNext()) {

		  		 			      double val = iter.getDoubleNext();//System.out.println(val);
		  		 			      juld.add(i++,new Double(val));//System.out.println(al);
		  		 			  }
/*
		  		             //display the list
		  		             for(int j=0;j<juld.size();j++){
		  		             System.out.println(juld.get(j));
		  		     	    }
*/



	      }//end of if

          if(v.getName().equals("JULD_LOCATION")){
		  		 			  //System.out.print(v.getName() + " =");
		   		              //System.out.println(a.getElementType());
		   		              IndexIterator iter = a.getIndexIterator();
		   		              juld_location=new ArrayList(); int i=0;
		   		 			  while(iter.hasNext()) {

		   		 			      double val = iter.getDoubleNext();//System.out.println(val);
		   		 			      juld_location.add(i++,new Double(val));//System.out.println(al);
		   		 			  }
/*
		   		             //display the list
		   		             for(int j=0;j<juld_location.size();j++){
		   		             System.out.println(juld_location.get(j));
		   		     	    }
*/

		   }//end of if

	      if(v.getName().equals("LATITUDE")){
		  		 			  //System.out.print(v.getName() + " =");
		  		              //System.out.println(a.getElementType());
		  		              IndexIterator iter = a.getIndexIterator();
		  		              latitude=new ArrayList(); int i=0;
		  		 			  while(iter.hasNext()) {

		  		 			      double val = iter.getDoubleNext();//System.out.println(val);
		  		 			      latitude.add(i++,new Double(val));//System.out.println(al);
		  		 			  }
/*
		  		             //display the list
		  		             for(int j=0;j<latitude.size();j++){
		  		             System.out.println(latitude.get(j));
		  		     	    }
*/
                            LATITUDE=(float)((Double)latitude.get(0)).doubleValue(); 
                            System.out.println("LATITUDE="+LATITUDE);
	      }//end of if

	      if(v.getName().equals("LONGITUDE")){
		  		 			  //System.out.print(v.getName() + " =");
		  		              //System.out.println(a.getElementType());
		  		              IndexIterator iter = a.getIndexIterator();
		  		              longitude=new ArrayList(); int i=0;
		  		 			  while(iter.hasNext()) {

		  		 			      double val = iter.getDoubleNext();//System.out.println(val);
		  		 			      longitude.add(i++,new Double(val));//System.out.println(al);
		  		 			  }
/*
		  		             //display the list
		  		             for(int j=0;j<longitude.size();j++){
		  		             System.out.println(longitude.get(j));
		  		     	    }
*/		  		     	    LONGITUDE=(float)((Double)longitude.get(0)).doubleValue();
                            System.out.println("LONGITUDE="+LONGITUDE);

	      }//end of if

      }//end of while

     

      System.out.println("There are "+columns+" columns"+"\n\n");
      System.out.println("Reading netcdf file done !"+"\n\n");
      
//extract WMO ID from filename
      int slash_index=fileName.lastIndexOf("/");
      //if(slash_index==-1) slash_index=0;//if no whole pathname, start after "R".
      System.out.println("fileName="+fileName+" slash_index="+slash_index+" under_score index="+fileName.lastIndexOf("_"));      
      
      WMOID_in_Filename=(fileName.substring(slash_index+2,fileName.lastIndexOf("_"))).trim();      
      System.out.println("WMO ID from NC filename is: "+WMOID_in_Filename);
//extract "D" from filename if it has.
      D_Filename=fileName.substring(fileName.indexOf(".")-1,fileName.indexOf(".")); 
      System.out.println("D_Filename= "+D_Filename); 
      if(D_Filename.equals("D")){
      	D_Filename="D";
        System.out.println("The profile is a down profile based on its name"); 
      }else{
      	D_Filename="A";
      	System.out.println("The profile is a up profile based on its name"); 
      }    
      
    } catch (java.io.IOException e) {
        e.printStackTrace();
    } catch(Exception ex){
    	//ex.printStackTrace();
    	System.out.println(ex.getMessage());
        System.exit(-1);    	
    }

  }//end of constructor





  public void ReadMetaDataFile(String meta_file_name){
      String str;//store one line of string from meta data file
      String strUpper;//upper case of str
	  try{
	  	 System.out.println("Reading Meta Data File......"+meta_file_name+"\n\n");
	  	 
		 FileReader fr=new FileReader(meta_file_name);
         BufferedReader in= new BufferedReader(fr);
         //read line by line for the meta data file
         while ((str= in.readLine()) != null){
			 //System.out.println(str);
            strUpper=str.toUpperCase();

            if(strUpper.startsWith(new String("internal ID number").toUpperCase())){
				FLOAT_NUMBER=str.substring(39).trim();
				System.out.println("FLOAT_NUMBER:"+FLOAT_NUMBER);
			}

			else
			if(strUpper.startsWith(new String("WMO ID number").toUpperCase())){
				FLOAT_WMO_NUMBER=str.substring(39).trim();
				System.out.println("FLOAT_WMO_NUMBER:"+FLOAT_WMO_NUMBER);
			}

			else
			if(strUpper.startsWith(new String("instrument type").toUpperCase())){
				INSTRUMENT_TYPE=str.substring(39).trim();
				System.out.println("INSTRUMENT_TYPE:"+INSTRUMENT_TYPE);
			}

			else
			if(strUpper.startsWith(new String("PI").toUpperCase())){
				PRINCIPAL_INVESTIGATOR=str.substring(39).trim();
				System.out.println("PRINCIPAL_INVESTIGATOR:"+PRINCIPAL_INVESTIGATOR+"\n");
			}
			else
			if(strUpper.startsWith(new String("calib").toUpperCase())){
				calibration=calibration+strUpper+"\n";
				//System.out.println("CALIBRATION:"+calibration);
			}

			else
			if(strUpper.startsWith(new String("transmission ID number").toUpperCase())){
				TRANSM_ID=str.substring(39).trim();
				System.out.println("TRANSM_ID:"+TRANSM_ID);
			}

			else
			
			if(strUpper.startsWith(new String("WMO instrument type (table 1770)").toUpperCase())){
				WMO_INST=str.substring(39).trim();
				System.out.println("WMO_INST:"+WMO_INST);
			}
			else
			if(strUpper.startsWith(new String("WMO recorder type (table 4770)").toUpperCase())){
				WMO_REC=str.substring(39).trim();
				System.out.println("WMO_REC:"+WMO_REC);
			}//end of if


		 }//end of while loop
           System.out.println("Reading Meta File done !"+"\n\n");
           
	  }catch (java.io.IOException e) {
        e.printStackTrace();
      }//end of try/catch block




  }//end of ReadMetaDataFile function

//generate output and log file names following ARGO nameing convention convention
  private void getOutFileName(){
  	String aa,bb,bb_d,cc="",dd="",ee="",ff="";
  	int num,i;
  	
        aa=TRANSM_ID.toUpperCase();
/*	
        if(aa.length()<5){
            num=5-aa.length();
            for(i=0;i<num;i++){cc=cc+"0";}
            //System.out.println("cc="+cc);        	
        	aa=cc+aa;
        	//System.out.println("aa="+aa);        	
        }
*/      	
        bb=CYCLE_NUMBER;
        
        
        if(bb.length()<3){
        	num=3-bb.length();
            for(i=0;i<num;i++){dd=dd+"0";}        	
            //System.out.println("dd="+dd);
        	bb=dd+bb;
        	//System.out.println("bb="+bb);        	
        }

          
        ee=FLOAT_NUMBER;
        if(ee.length()<4){
            num=4-ee.length();
            for(i=0;i<num;i++){ff=ff+"0";}
            //System.out.println("ff="+ff);        	
        	ee=ff+ee;
        	//System.out.println("ff="+ff);        	
        }
        
       
               
        if(DIRECTION.equals("D")){
        bb_d=""+(Integer.parseInt(bb)+1000);	
        outFileName=phy_dir+ee+"_"+aa+"_"+bb_d+".phy";
        purePhyFileName=ee+"_"+aa+"_"+bb_d+".phy";
        }else{
        outFileName=phy_dir+ee+"_"+aa+"_"+bb+".phy";	
        purePhyFileName=ee+"_"+aa+"_"+bb+".phy";
        }
        
                          
        System.out.println("outFileName: "+outFileName+"\n\n");
	        
  	
  }//end of getOutFileNames


//convert to physical units logic here

  private int Convert(){
     float zero=0.0f,minus999=-999.0f,sal=0.0f;
     float tref=0.0f,p,t,c,s;
     String aa,bb,cc="",dd="",ee="",ff="";
     
     ArrayList time1=new ArrayList();//time1
     ArrayList time2=new ArrayList();//time2
     int cnt=0;

      
     System.out.println("Converting from netcdf file format to physical format......"+"\n\n");	
     
     if(!DIRECTION.equals(D_Filename)){
     	System.out.println("DIRECTION="+DIRECTION+" D_Filename="+D_Filename);
     	System.out.println("Profile direction(up/down) is inconsistant with the file name convention");
     	System.exit(9);
     }
     
     if(columns!=3){
		 System.out.println("can not decode data if number of columns is not 3 !");
         System.exit(8);
	 }
     
     if(!FLOAT_WMO_NUMBER.equals(PLATFORM_NUMBER)){
		 System.out.println("FLOAT_WMO_NUMBER and data.PLATFORM_NUMBER differ !");
		 System.out.println("FLOAT_WMO_NUMBER="+FLOAT_WMO_NUMBER+" PLATFORM_NUMBER="+PLATFORM_NUMBER);		 
         System.exit(7);
	 }
     
     //if(!WMO_INST.equals(WMO_INST_TYPE)){
     try{
     	          	
      if(Integer.parseInt(WMO_INST) != Integer.parseInt(WMO_INST_TYPE)){	
     	 System.out.println("WMO_INST and WMO_INST_TYPE differ !");
     	 System.out.println("WMO_INST="+Integer.parseInt(WMO_INST)+ " WMO_INST_TYPE="+Integer.parseInt(WMO_INST_TYPE));		 
         System.exit(6);
	  }
     }catch(NumberFormatException e){
     	 System.out.println("WMO_INST or WMO_INST_TYPE is not a number!");
     	 System.exit(6);
     }

//Compare WMO ID(PLATFORM_NUMBER) from filename with that in NC file
     
     if(!PLATFORM_NUMBER.equals(WMOID_in_Filename)){
     	 System.out.println("WMO ID(PLATFORM_NUMBER) in NC filename differs from that inside NC file");
     	 System.exit(4);
     }
 
     if(PLATFORM_NUMBER=="0"){
     	 System.out.println("WMO ID(PLATFORM_NUMBER) in NC file is a 'fill' value(0)");
     	 System.exit(2);
     }
     

//extract useful data values,discard useless data such as all with 99999.
     pres=new ArrayList();
     temp=new ArrayList();
     cndc=new ArrayList();

     
     for(int j=0;j<Pres.size();j++){
     	         p=((Float)Pres.get(j)).floatValue();
		 t=((Float)Temp.get(j)).floatValue();
		 c=((Float)Cndc.get(j)).floatValue();
		 if((p<99999.0f) || (t<99999.0f) || (c< 99999.0f)){
		   pres.add(cnt,new Float(p));	
		   temp.add(cnt,new Float(t));
		   cndc.add(cnt,new Float(c));
		   cnt++;
		 }
     	
     }

//init the psal list to 0
     psal=new ArrayList(cndc.size());

     for(int j=0;j<cndc.size();j++){
       psal.add(j,new Float(zero));
     }//end of for loop


//process dummy values in pressure, temperature and conductivity
     for(int j=0;j<pres.size();j++){
		 p=((Float)pres.get(j)).floatValue();
		 t=((Float)temp.get(j)).floatValue();
		 c=((Float)cndc.get(j)).floatValue();
                  
         if((p>99998.9f) || (t>99998.9f) || (c>99998.9f)){
           psal.set(j,new Float(minus999));
		 }
		 else{

		   sal=sal78((double)p,(double)t,(double)c);
		   //System.out.println("Salinity caculated is: "+sal);
		   psal.set(j,new Float(sal));
		 }//end of if

         if(p>99998.9f){
			 pres.set(j,new Float(minus999));
		 }//end of if

         if(t>99998.9f){
			 temp.set(j,new Float(minus999));
		 }//end of if
         if(c>99998.9f){
			 cndc.set(j,new Float(minus999));
		 }//end of if

     }//end of for
     
//verify whether the data are within the range according to specifications, f6.1,f8.3,f8.3.
     for(int j=0;j<pres.size();j++){
		 p=((Float)pres.get(j)).floatValue();
		 t=((Float)temp.get(j)).floatValue();
		 c=((Float)cndc.get(j)).floatValue();
         s=((Float)psal.get(j)).floatValue();         
         if((p>9999.9f) || (p<-999.9f) 
         || (t>9999.999f) || (t<-999.999f) 
         || (c>9999.999f) || (c<-999.999f) 
         || (s>9999.999f) || (s<-999.999f)){
    	  System.out.println("Magnitude of data value(s) exceed format specifications !");    
          System.exit(10);
         }            
     }
//process julian day stuff here
        time1=getTime(REFERENCE_DATE_TIME,juld);
        time2=getTime(REFERENCE_DATE_TIME,juld_location);
        
        year1=((Integer)time1.get(0)).intValue();
        month1=((Integer)time1.get(1)).intValue();
        day1=((Integer)time1.get(2)).intValue();
        hour1=((Integer)time1.get(3)).intValue();
        minute1=((Integer)time1.get(4)).intValue();
        second1=((Integer)time1.get(5)).intValue();
        
        year2=((Integer)time2.get(0)).intValue();
        month2=((Integer)time2.get(1)).intValue();
        day2=((Integer)time2.get(2)).intValue();
        hour2=((Integer)time2.get(3)).intValue();
        minute2=((Integer)time2.get(4)).intValue();
        second2=((Integer)time2.get(5)).intValue();  
                             	
	 
        PrintResult(outFileName);
        System.out.println("SIZE "+ pres.size()+" "+temp.size()+" "+cndc.size()+" "+psal.size()+"\n");
	
//print phy filename to phyNameFile file	
     
        PrintPhyFileName();
/*        
        out.close();
        fww.close();
        
   }catch(IOException e){
    e.printStackTrace();        	   	
   }
     
*/
	 return 0;
  }//end of convert function

 private void PrintPhyFileName(){
 
    try{
        FileWriter fw=new FileWriter(phyNameFile);
        PrintWriter out=new PrintWriter(fw);
	    out.println(purePhyFileName);
        out.close();
	fw.close();    
    }catch(IOException ioe){
      ioe.printStackTrace();
    }
 
 
 }


 private void PrintResult(String outFileName){

        
        try{
        	
            FileWriter fw=new FileWriter(outFileName);
            PrintWriter out=new PrintWriter(fw);
            System.out.println("Printing result to output file: "+outFileName+"\n\n");
                
        	out.println("INTERNAL ID NUMBER                      "+FLOAT_NUMBER);
        	out.println("WMO ID NUMBER                           "+FLOAT_WMO_NUMBER);
        	out.println("TRANSMISSION ID NUMBER                  "+TRANSM_ID);
        	
        	if(DIRECTION.equals("D")){
        	out.println("PROFILE NUMBER                          "+(Integer.parseInt(CYCLE_NUMBER)+1000));
                }else{
	    	out.println("PROFILE NUMBER                          "+(Integer.parseInt(CYCLE_NUMBER)));
		    }
            
        	out.println("INSTRUMENT TYPE                         "+INSTRUMENT_TYPE);
        	out.println("WMO INSTRUMENT TYPE (TABLE 1770)        "+WMO_INST);
        	out.println("WMO RECORDER TYPE (TABLE 4770)          "+WMO_REC);
        	out.println("POSITIONING SYSTEM                      "+POSITIONING_SYSTEM);
        	out.println("PI                                      "+PRINCIPAL_INVESTIGATOR.toUpperCase());
        	out.println("DATE_CREATION OF NC                     "+DATE_CREATION);
        	out.println("JULD_QC                                 "+JULD_QC);
        	out.println("POSITION_QC                             "+POSITION_QC);
        	out.print(calibration);
        	out.println("NUMBER OF COLUMNS                       "+(columns+2));
        	out.println("1. COLUMN                               "+"PRESSURE (DBAR)");
        	out.println("2. COLUMN                               "+"TEMPERATURE-90 (DEG C)");
        	out.println("3. COLUMN                               "+"CONDUCTIVITY");
        	out.println("4. COLUMN                               "+"SALINITY (PSU)");
        	out.println("5. COLUMN                               "+"CHECKSUM FLAG");
        	out.println("==========================================================");
        	out.println("   LATITUDE  LONGITUDE YEAR/MO/DY HR:MN:SC XMITS SAT POSITION_QC");
        	
        	if(LATITUDE>99998.0f || LONGITUDE>99998.0f){
        	out.print(new PrintfFormat("%+11.3f").sprintf(99.999)+new PrintfFormat("%+11.3f").sprintf(999.999)+" ");
        	out.print(new PrintfFormat("%4d").sprintf(year1)+"/"+new PrintfFormat("%02d").sprintf(month1)+"/"+new PrintfFormat("%02d").sprintf(day1)+" ");
        	out.print(new PrintfFormat("%02d").sprintf(hour1)+":"+new PrintfFormat("%02d").sprintf(minute1)+":"+new PrintfFormat("%02d").sprintf(second1)+"   ");
        	out.println("000   0     0");
            
                out.print(new PrintfFormat("%+11.3f").sprintf(99.999)+new PrintfFormat("%+11.3f").sprintf(999.999)+" ");
        	out.print(new PrintfFormat("%4d").sprintf(year1)+"/"+new PrintfFormat("%02d").sprintf(month1)+"/"+new PrintfFormat("%02d").sprintf(day1)+" ");
        	out.print(new PrintfFormat("%02d").sprintf(hour1)+":"+new PrintfFormat("%02d").sprintf(minute1)+":"+new PrintfFormat("%02d").sprintf(second1)+"   ");
        	out.println("000   0     0");
            	
        	}else{
        	out.print(new PrintfFormat("%+11.3f").sprintf(99.999)+new PrintfFormat("%+11.3f").sprintf(999.999)+" ");
        	out.print(new PrintfFormat("%4d").sprintf(year1)+"/"+new PrintfFormat("%02d").sprintf(month1)+"/"+new PrintfFormat("%02d").sprintf(day1)+" ");
        	out.print(new PrintfFormat("%02d").sprintf(hour1)+":"+new PrintfFormat("%02d").sprintf(minute1)+":"+new PrintfFormat("%02d").sprintf(second1)+"   ");
        	out.println("000   0     0");
            
                out.print(new PrintfFormat("%+11.3f").sprintf(LATITUDE)+new PrintfFormat("%+11.3f").sprintf(LONGITUDE)+" ");
        	out.print(new PrintfFormat("%4d").sprintf(year2)+"/"+new PrintfFormat("%02d").sprintf(month2)+"/"+new PrintfFormat("%02d").sprintf(day2)+" ");
        	out.print(new PrintfFormat("%02d").sprintf(hour2)+":"+new PrintfFormat("%02d").sprintf(minute2)+":"+new PrintfFormat("%02d").sprintf(second2)+"   ");
		out.println("000   0     "+POSITION_QC);//changed from 0 to POSITION_QC
            
                      	
        	out.print(new PrintfFormat("%+11.3f").sprintf(99.999)+new PrintfFormat("%+11.3f").sprintf(999.999)+" ");
                out.print(new PrintfFormat("%4d").sprintf(year2)+"/"+new PrintfFormat("%02d").sprintf(month2)+"/"+new PrintfFormat("%02d").sprintf(day2)+" ");
        	out.print(new PrintfFormat("%02d").sprintf(hour2)+":"+new PrintfFormat("%02d").sprintf(minute2)+":"+new PrintfFormat("%02d").sprintf(second2)+"   ");
        	out.println("000   0     0");
            }//end of if else   	
        	
        	out.println("==========================================================");
           if((WMO_REC.equals("61")) || WMO_REC.equals("63") || DIRECTION.equals("D")){
        	for(int i=0;i<pres.size();i++){
        	
        	out.print(new PrintfFormat("%6.1f").sprintf(((Float)pres.get(i)).floatValue()));
                out.print(new PrintfFormat("%9.3f").sprintf(((Float)temp.get(i)).floatValue()));
                out.print(new PrintfFormat("%9.3f").sprintf(((Float)cndc.get(i)).floatValue()));
                out.print(new PrintfFormat("%9.3f").sprintf(((Float)psal.get(i)).floatValue()));
                out.println(new PrintfFormat("%3d").sprintf(0));
                
                      		
        	}
	       }else{
	   
        	for(int i=pres.size()-1;i>=0;i--){
        	
        	out.print(new PrintfFormat("%6.1f").sprintf(((Float)pres.get(i)).floatValue()));
                out.print(new PrintfFormat("%9.3f").sprintf(((Float)temp.get(i)).floatValue()));
                out.print(new PrintfFormat("%9.3f").sprintf(((Float)cndc.get(i)).floatValue()));
                out.print(new PrintfFormat("%9.3f").sprintf(((Float)psal.get(i)).floatValue()));
                out.println(new PrintfFormat("%3d").sprintf(0));
                
        	}
	   }
	    	
        	System.out.println("Resulting output file created successfully!"+"\n\n");
        	out.close();
        	fw.close();
        	
        }catch(java.io.IOException e) {
          e.printStackTrace();        	
        }

 	
 }//end of PrintResult




  private ArrayList getTime(String ref_date_time,ArrayList list){
	   int year,month,day,hour,minute,second;
           float ht,mt,st;
           double tt,tref;

           int Year,Month,Day,Hour,Minute,Second;

           ArrayList dmy=new ArrayList();
           ArrayList ymdhms=new ArrayList();

	       year=(new Integer(ref_date_time.substring(0,4))).intValue();
	       month=(new Integer(ref_date_time.substring(4,6))).intValue();
	       day=(new Integer(ref_date_time.substring(6,8))).intValue();
	       hour=(new Integer(ref_date_time.substring(8,10))).intValue();
	       minute=(new Integer(ref_date_time.substring(10,12))).intValue();
	       second=(new Integer(ref_date_time.substring(12,14))).intValue();
	       System.out.println("Reference Year="+year+"Month="+month+"Day="+day+"Hour="+hour+"Minute="+minute+"Second="+second);
	       tref=(double)dmy2rd(day,month,year)+hour/24.0f+minute/24.0f/60.0f+second/24.0f/3600.0f;

	       System.out.println("tref="+tref);
	       tt=(new Double(list.get(0).toString())).doubleValue()+tref;
           
           System.out.println("julian day="+tt);
           
	       dmy=rd2dmy((long)Math.floor((double)tt));
	       
	       tt=(float)(tt-Math.floor((double)tt))*24.0f;
	       
	       ht=(float)Math.floor((double)tt);
	       mt=(float)(tt-Math.floor((double)tt))*60.0f;	       
	       st=(float)(mt-Math.floor((double)mt))*60.0f;
	       
	       st=Math.round(st);
	       ht=(float)Math.floor((double)ht);
	       mt=(float)Math.floor((double)mt);
	       if(Math.abs(st-60)<0.000001){
	       	mt=mt+1;
	       	st=0;
	       }
	       if(Math.abs(mt-60)<0.000001){
	       	ht=ht+1;
	       	mt=0;
	       }
	       if(Math.abs(ht-24)<0.000001){
            dmy=rd2dmy((long)Math.floor((double)dmy2rd(((Integer)dmy.get(0)).intValue(),((Integer)dmy.get(1)).intValue(),((Integer)dmy.get(2)).intValue()))+1);
            ht=0;  	       	
	       }
	       
	       
	       System.out.println("ht="+ht);
	       System.out.println("mt="+mt);
	       System.out.println("st="+st);
	       
	       if(st<1.0f) st=0.0f;
           Year=((Integer)dmy.get(2)).intValue();
           Month=((Integer)dmy.get(1)).intValue();
           Day=((Integer)dmy.get(0)).intValue();
           Hour=(int)ht;
           Minute=(int)mt;
           Second=(int)st;
           
           //if it is fill value,then reset the Year,month,day,hour,minute,second to 999..
           if((new Double(list.get(0).toString())).doubleValue()>999998.0){
           Year=9999;
           Month=99;
           Day=99;
           Hour=99;
           Minute=99;
           Second=99;	
           }
           
           System.out.println("Actal Year="+Year+"Month="+Month+"Day="+Day+"Hour="+Hour+"Minute="+Minute+"Second="+Second+"\n\n");

           ymdhms.add(0,new Integer(Year));
           ymdhms.add(1,new Integer(Month));
           ymdhms.add(2,new Integer(Day));
           ymdhms.add(3,new Integer(Hour));
           ymdhms.add(4,new Integer(Minute));
           ymdhms.add(5,new Integer(Second));

	       return ymdhms;
  }



//convert conductivity to salinity function

  private float sal78(double p, double t, double cors){
	double dt,r,t2,t3,t4,rt35,abt,p2,p3,c_p,rt,rt2,rt3,rt4,rt5,srt,sdt;
	float result;
	double a[]={2.7081,-7.0261,14.0941,25.3851,-0.1692,0.0080};
	double b[]={0.0162, -0.0144,  0.0636, -0.0375, -0.0066, -0.0056, 0.0005};
	double c[]={1.0031e-9, -6.9698e-7, 1.104259e-4, 2.00564e-2, 0.6766097};
	double d[]={3.426e-2, 4.464e-4, -3.107e-3, 0.4215};
	double e[]={2.070e-5, -6.370e-10, 3.989e-15};
	double f[]={13.5405, -28.1044, 42.2823, 50.7702, -0.1692};
	double g[]={0.0162, -0.0720, 0.2544, -0.1125, -0.0132, -0.0056};

    if(cors<= 5e-4) {return 0.0f;}

    dt=t-15;
    r=cors/42.914;
  	t2=t*t;
  	t3=t2*t;
  	t4=t3*t;
    rt35= c[0]*t4 + c[1]*t3 + c[2]*t2 + c[3]*t + c[4];
    abt= 1 + d[0]*t + d[1]*t2 + d[2]*t*r + d[3]*r;
  	p2=p*p;
  	p3=p2*p;
    c_p= e[2]*p3 + e[1]*p2 + e[0]*p;
    rt=Math.sqrt((double)Math.abs((double) r/rt35/(1+c_p/abt)));
    rt2=rt*rt;
    rt3=rt2*rt;
    rt4=rt3*rt;
    rt5=rt4*rt;
    srt = a[0]*rt5 + a[1]*rt4 + a[2]*rt3 + a[3]*rt2 + a[4]*rt + a[5];
    sdt = dt/(1+b[0]*dt)* (b[1]*rt5 + b[2]*rt4 + b[3]*rt3 + b[4]*rt2 + b[5]*rt + b[6]);
    result = (float)(srt+sdt);


    return result;
  }//end of sal78 function



  public long dmy2rd(int day,int month,int year){
	     int  ic, iy;
	     long julday;

	     if ( month > 2 )
	       month = month - 3;
	     else if ( month < 3 )
	     {
	       month = month + 9;
	       year = year - 1;
	     }

	     ic = (int) Math.floor((double) (year/100) );
	     iy = year - 100*ic;


	     julday =   (long) Math.floor((double)(146097*ic)/4)
	               + (long) Math.floor((double)(1461*iy)/4)
	               + (long) Math.floor((double)(153*month+2)/5)
	               + day + 1721119;
	     julday = julday - 2440000;

   return julday;

  }

  public ArrayList rd2dmy(long rjul){
	    ArrayList dmy=new ArrayList();

	    long julian;
	    int iyr, month, iday,i;

	    julian = rjul - 1721119 + 2440000;
	    iyr = (int)Math.floor((double)( 4*julian - 1 )/ 146097);
	    julian = 4*julian - 1 - 146097*iyr;
	    iday = (int)Math.floor((double)julian/4);
	    julian = (int)Math.floor((double)( 4*iday + 3 )/ 1461);
	    iday = (int)(4*iday + 3 - 1461*julian);
	    iday = (int)Math.floor((double)(iday + 4 )/ 4);
	    month = (int)Math.floor((double)( 5*iday - 3 )/ 153);
	    iday = 5*iday - 3 - 153*month;
	    iday = (int)Math.floor((double)(iday + 5 )/ 5);
	    iyr = (int)(100*iyr + julian);

	    if (month < 10)
	    {
	      month += 3;
	    }
	    else if (month > 9)
	    {
	      month -= 9;
	      iyr += 1;
	    }

        System.out.println("Year:"+iyr+" Month: "+month+ "Day:"+iday);

	    i=0;
	    dmy.add(i++,new Integer(iday));
	    dmy.add(i++,new Integer(month));
	    dmy.add(i++,new Integer(iyr));

        return dmy;

  }


}//end of class NetcdfReader
