c Revision Date: 19-Feb-13  VMH - Adjust i/o & debug statements
c 05-Oct-09 (ER  - extensive changes to the code of the program to 
c                  avoid go to statements, and make it more readable 
c                  and streamlined. 
c                - new function psal_line_exist, to find out whether 
c                  line PSAL ADJUST exists in PHY file
c                - new subroutine get_salinity_in_meta to get salinity
c                  variables from meta file (as opposed to all meta vars)
c                - new subroutine close_files to close error and default
c                  files.  In several cases they were not close before.
c                - if status=1, exit the program right away
c                - if meta file doesn't exist, exit program with status=9
c                - if Input PHY doesn't exist, exit program with status=9
c                28-Sep-09 (ER  - changes in layout of statements to make
c                                 them easier to read) 
c                12-Dec-08 (YHD - Do not create a new phy file when
c                                 sal_offset=0.0 and sal_std=0.0)
c                05-Jul-07 (YHD - More than one line of 'PSAL ADJUST' line)
c                12-Apr-07 (YHD - Make an exit status=2 when the "PSAL" line
c                                 already exists.)
c                06-Mar-07 (YHD - Make an exit status=1 when prof_num le 
c                                 sal_adj_pnum case)
c                16-Jan-06 (YHD - This is a stand alone program.)
c     
c
c   adjsal2phy.f
c   ============
c   This is a stand alone program to create the phy file with adjusted
c   salinity column added and to write an new *.phy file.
c 
c
c
      implicit none

      integer :: status = 0  ! This program's return value:
                             ! 0: sal_offset=0, sal_std=0 
                             ! 1: Input Prof.# < sal_adj_pnum from meta file 
                             ! 2: PSAL ADJUST line already in PHY file 
                             ! 9: Fatal error, as in error opening a file 

      !
      ! Salinity Adjusted Variables
      ! Values for them will be extracted from meta file 
      ! by subroutine get_salinity_in_meta 
      !
      integer :: sal_adj_dd    ! Sal. Adjusted Day
      integer :: sal_adj_mm    ! Sal. Adjusted Month
      integer :: sal_adj_yyyy  ! Sal. Adjusted Year
      integer :: sal_adj_pnum  ! Sal. Adjusted Profile #
      real :: sal_offset ! Sal. Offset
      real :: sal_std    ! Sal. Offset standard deviation
      character(len=128) :: sal_adj_line ! PSAL ADJUSTED line

      ! File handle constants and file name variables 
      integer :: DBG  = 66  ! Unit to handle debug file
      character(len=128) :: pathname_debug
      integer, parameter :: META   = 25  ! Unit to handle meta file 
      character(len=128) :: pathname_meta
      integer, parameter :: INPUT  = 26  ! Unit to handle input file 
      character(len=128) :: pathname_infile   
      integer, parameter :: OUTPUT = 27  ! Unit to handle output file
      character(len=128) :: pathname_outfile
      integer :: ERR = 50   ! Unit to handle error file
      character(len=128) :: pathname_error
      integer, parameter :: SCREEN = 6

      !
      ! Auxiliar variables
      !
      logical :: DEBUG = .false.   
      character(len=1) :: env_debug ! Debug environment variable 

      integer :: ios  ! Error number of I/O operation
      integer, parameter :: IO_OK  =  0  ! IO Status = 0

      logical :: psal_line_exist ! PSAL ADJUST exists in meta file? 
      logical :: meta_exist      ! Meta file exists?
      logical :: phy_exist       ! PHY file exists?

      integer prof_num ! Prof. Nbr derived from the phy file name      
      character(len=20) :: profile_id = '                    '
      character(len=20) :: profname = '                    '

      character(len=10), parameter :: PRGMAIN = 'adjsal2phy' 
      integer*4 lnblnk,last,lastnb,lastdir,lenstr,lasb,lase,
     *         lchar_occur

      write(SCREEN, '(A, ": Entering Program Execution")') PRGMAIN 
      ! move down if (DEBUG) write(DBG,*)'<<< DEBUG from adjsal2phy >>>'

      ! Get environmental variables.
      call GETENV('DEBUG', env_debug)
      call GETENV('error_outfile', pathname_error)
      call GETENV('debug_outfile', pathname_debug)

      if (env_debug=='y') then
        DEBUG = .true. 
        write(SCREEN, '(A4, ": Debug Option is ON")') PRGMAIN 
      else
        DEBUG = .false.
        write(SCREEN, '(A4, ": Debug Option is OFF")') PRGMAIN 
      endif
     
      ! Get command arguments:
      call GETARG(1, pathname_infile)  ! phy input file
      call GETARG(2, pathname_meta)    ! meta input file	
      call GETARG(3, pathname_outfile) ! phy output file

      print *, "PATHNAME_INFILE: ", pathname_infile
      print *, "PATHNAME_META: ", pathname_meta
      print *, "PATHNAME_OUTFILE: ", pathname_outfile

      lastdir = lchar_occur(pathname_infile,'/')
      lenstr = len(pathname_infile) 

      profname = pathname_infile(lastdir+1:lenstr)
      last = index(pathname_infile, '.')
      lasb = lchar_occur(profname(1:last-lastdir),'_')
      lase = index(profname,'.')
      if (lasb.gt.lase) then
        status=9
        write(SCREEN, '(A,": Profname error - ", A)') PRGMAIN, profname
        write(ERR,'(A,": Profname error - ", A)') PRGMAIN, profname
        write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
        call exit(status)
      end if

      profile_id = profname(1:lase-1)

      write(SCREEN, '(A, ": Profile File Name = ", A)') 
     *          PRGMAIN, profile_id

      read(profile_id(lasb+1:lase-1),*) prof_num
      
      ! Open Error File
      lastnb = lnblnk(pathname_error)
      write(SCREEN, '(A, ": Opening Error file - ", A)') 
     *          PRGMAIN, pathname_error(1:lastnb)
      open(ERR,file=pathname_error,form='formatted',position='append',
     *recl=1000,iostat=ios)
      if (ios<>IO_OK) then
        status=9
        write(SCREEN,'(A,": Opening error file error-", A," ios=",i4)') 
     *          PRGMAIN, pathname_error(1:lastnb),ios
        write(SCREEN,*) PRGMAIN, ': STOP -> STATUS = ', status
        call exit (status)
        stop '9'
      end if

      ! Open Debug File
      lastnb = lnblnk(pathname_debug)
      write(SCREEN, '(A, ": Opening Debug file - ", A)') 
     *          PRGMAIN, pathname_debug(1:lastnb)
      open(DBG,file=pathname_debug,form='formatted',
     *     position='append', recl=1000,iostat=ios)
      if (ios<>IO_OK) then
        status=9
        write(SCREEN,'(A,": Opening debug file error-", A," ios=",i4)') 
     *          PRGMAIN, pathname_debug(1:lastnb),ios
        write(ERR,'(A,": Opening debug file error-", A," ios=",i4)') 
     *          PRGMAIN, pathname_debug(1:lastnb),ios
        write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
        close(DBG)
        call exit (status)
        stop '9'
      end if
      if (DEBUG) write(DBG,*)'<<< DEBUG from adjsal2phy >>>'

      inquire(file=pathname_meta,exist=meta_exist)
      if (meta_exist) then    
        open(META,status='old',file=pathname_meta,
     *    form='formatted',iostat=ios)
        if (ios<>IO_OK) then
          status=9
          write(SCREEN, 
     *  '(A, ": Error opening META file unit 25. ios= ", i4)') 
     *    PRGMAIN, ios
          write(SCREEN,'("      File =", A)') pathname_meta
          write(SCREEN,'(A, ": STOP -> STATUS=", I3)') PRGMAIN, status
          write(ERR,*) PRGMAIN, 
     *              ': Error opening META file unit 25. ios=',ios
          write(ERR,*)'file =', pathname_meta
          write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
          call close_files(DEBUG,DBG,ERR)
          call exit (status)  
          stop '9'
        end if
      else
        status=9
        write(SCREEN, '(A, ": META file does not exist ")')PRGMAIN
        write(SCREEN,'("      File =", A)') pathname_meta
        write(SCREEN,'(A, ": STOP -> STATUS=", I3)') PRGMAIN, status
        write(ERR,*) PRGMAIN,"META file does not exist,ios=", ios
        write(ERR,*)'file =', pathname_meta
        write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
        call close_files(DEBUG,DBG,ERR)
        call exit (status)
      end if     ! if (meta_exist) then

      write(SCREEN,'(A, ": Reading  META file - ", A)') PRGMAIN, 
     *     pathname_meta

      call get_salinity_in_meta(META,prof_num,
     *  sal_adj_dd,sal_adj_mm,sal_adj_yyyy,sal_adj_pnum,sal_offset,
     *  sal_std,sal_adj_line)

        if(DEBUG) then
           write(DBG,*)'sal_adj:dd/mm/yyy ',sal_adj_dd,sal_adj_mm,
     *              sal_adj_yyyy
           write(DBG,*)'sal_adj:pnum',sal_adj_pnum
           write(DBG,*)'sal_adj:offset, std ',sal_offset,sal_std
        endif

      !
      ! This prof. # < Last Delayed Mode's file prof. # for this float
      !
      if (prof_num < sal_adj_pnum) then
        status=1
        lastnb = lnblnk(pathname_infile)
        write(SCREEN,'(A, ": prof_num < sal_adj_pnum ", i4,i4)') 
     *          PRGMAIN, prof_num,sal_adj_pnum
        write(SCREEN,'(A, ": This file already exists - ", A)') PRGMAIN, 
     *          pathname_infile(1:lastnb)
        write(ERR,'(A, ": prof_num < sal_adj_pnum ", i4,i4)') 
     *           PRGMAIN, prof_num,sal_adj_pnum
        write(ERR,'(A, ": This file already exists - ", A)') PRGMAIN, 
     *           pathname_infile(1:lastnb)
        write(SCREEN,'(A, ":  STOP -> STATUS: ", I3)') PRGMAIN, status
        call close_files(DEBUG,DBG,ERR)
        call exit(status)
      end if

      !
      ! sal_offset=0.0000 and sal_std=0.000 
      ! Do not create a new *.phy file.
      !
      if (sal_offset==0.0 .and. sal_std==0.0) then
        write(SCREEN,*) PRGMAIN,':sal_offset and sal_std = 0.0 
     *; DO NOT create a new phy file.'
        if(DEBUG) write(DBG,*) PRGMAIN,':sal_offset and sal_std = 0.0 
     *; DO NOT create a new phy file.'
        call close_files(DEBUG,DBG,ERR)
        status=0
        call exit(status) 
      end if

      !
      ! Open existing phy file as well as a new phy file to write
      ! Read from existing phy file, then write adjusted salinit to new 
      ! phy file
      !
      inquire(file=pathname_infile,exist=phy_exist)
      if (phy_exist) then    
        open(INPUT,status='old',file=pathname_infile,
     *    form='formatted',iostat=ios)
        if (ios<>IO_OK) then
          status=9
          write(SCREEN, 
     *  '(A, ": Error opening PHY file unit 26. ios= ", i4)') 
     *    PRGMAIN, ios
          write(SCREEN,'("      File =", A)') pathname_infile
          write(SCREEN,'(A, ": STOP -> STATUS=", I3)') PRGMAIN, status
          write(ERR,*) PRGMAIN, 
     *              ': Error opening input PHY file unit 26. ios=',ios
          write(ERR,*)'file =', pathname_infile
          write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
          call exit (status)  
        end if
      else
        status=9
        write(SCREEN,
     *  '(A, ": PHY file does not exist")') PRGMAIN
        write(SCREEN,'("      File =", A)') pathname_infile
        write(SCREEN,'(A, ": STOP -> STATUS=", I3)') PRGMAIN, status
        write(ERR,*) PRGMAIN, ': Input PHY file does not exist'
        write(ERR,*)'file =', pathname_infile
        write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
        call exit (status)
      end if    ! if (phy_exist) then
      
      if (psal_line_exist(INPUT)) then
        write(SCREEN,*)'PSAL ADJUST line already exists in phy file.'
        status=2
      else
        if (status==0) then
          open(OUTPUT,status='new',file=pathname_outfile,
     *    form='formatted',iostat=ios)
          if (ios<>IO_OK) then
            status=9
            write(SCREEN, 
     *  '(A, ": Error opening PHY file unit 27. ios= ", i4)') 
     *    PRGMAIN, ios
            write(SCREEN,'("      File =", A)') pathname_outfile
            write(SCREEN,'(A, ": STOP -> STATUS=", I3)') PRGMAIN, status
            write(ERR,*) PRGMAIN, 
     *              ': Error opening output PHY file unit 27. ios=',ios
            write(ERR,*)'file =', pathname_outfile
            write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
            call exit (status)  
            stop '9'
          end if
          !
          ! Call program that actually do the salinity adjustment
          ! on the phy file.
          !
	  if(DEBUG) write(SCREEN,*)'D ==================='
	  if(DEBUG) write(SCREEN,*)'D CALL add_adjsal2phy'
          call add_adjsal2phy(INPUT,OUTPUT,sal_offset, sal_adj_line)
	  if(DEBUG) write(SCREEN,*)'D RTRN add_adjsal2phy'
	  if(DEBUG) write(SCREEN,*)'D ==================='
        end if ! if (status==0) then
      end if

      call close_files(DEBUG,DBG,ERR) 

      if (status==0) then
        write(SCREEN,'(A, ":  STOP -> STATUS: ", I3)') PRGMAIN, status
        call exit(status)
        stop 'nomal exit'
      else if (status==2) then
        write(SCREEN,'(A, ":  STOP -> STATUS: ", I3)') PRGMAIN, status
        call exit(status)
        stop 'phy file exist case exit'
      else
        status=9
        write(SCREEN,'(A, ":  STOP -> STATUS: ", I3)') PRGMAIN, status
        call exit(status)
        stop 'abnormal exit'
      end if                         ! if (status==0) then
      end

! ---------------------------------------------------------------------
! function psal_line_exist 
!    Returns true if there is a line in the .phy file that starts with
!    'PSAL ADJUST' 
! ---------------------------------------------------------------------

      logical function psal_line_exist(INPUT)

      implicit none
      integer, intent(in)  :: INPUT

      integer, parameter :: IO_OK  =  0  ! IO Status = 0
      integer :: status, ios
      character(len=128) :: cline
      integer, parameter :: SCREEN = 6
      integer, parameter :: ERR = 50   ! Unit to handle error file
      psal_line_exist=.false.

      read(INPUT,'(a)',iostat=ios) cline
      do while (ios==IO_OK)
        if (cline(1:39)=='PSAL ADJUST [dd mm yyyy N S_off stddev]') then
          psal_line_exist=.true.
        end if
        read(INPUT,'(a)',iostat=ios) cline
      end do

      if (ios>IO_OK) then
        write(SCREEN,*)'adjsal2phy.f(read cline) error=',ios
        write(ERR,*)'adjsal2phy.f(read cline) error=',ios
        status=9
      end if

      rewind (INPUT)
      return
      end function psal_line_exist


! ---------------------------------------------------------------------
! subroutine get_salinity_in_meta
!    E. Ramos Oct 07, 2009
!    Returns salinity adjusted variables in meta file 
! ---------------------------------------------------------------------

      subroutine get_salinity_in_meta(iunit,prof_num,
     *  sal_adj_dd,sal_adj_mm,sal_adj_yyyy,sal_adj_pnum,sal_offset,
     *  sal_std,sal_adj_line)

      implicit none

      ! Input and Output Parameters
      integer, intent(in)  :: iunit ! unit for reading meta file
      integer, intent(in)  :: prof_num ! profile number
      integer, intent(out) :: sal_adj_dd    ! Sal. Adjusted Day
      integer, intent(out) :: sal_adj_mm    ! Sal. Adjusted Month
      integer, intent(out) :: sal_adj_yyyy  ! Sal. Adjusted Year
      integer, intent(out) :: sal_adj_pnum  ! Sal. Adjusted Profile #
      ! Following 2 output variables will be used to calculate
      ! the salinity in the .phy or the .qc file
      real, intent(out) :: sal_offset ! Sal. Offset
      real, intent(out) :: sal_std    ! Sal. Offset standard deviation
      character(len=128), intent(out) :: sal_adj_line ! PSAL ADJUSTED line

      ! Constants
      logical, parameter :: DEBUG = .false.
      integer, parameter :: IO_OK  =  0  ! IO Status = 0
      integer, parameter :: IO_EOF = -1  ! IO Status = EOF
      integer, parameter :: SCREEN = 6   ! Unit to handle screen
      integer, parameter :: DBG  = 66  ! Unit to handle debug file
      integer, parameter :: ERR = 50   ! Unit to handle error file

      ! Multiple instance Salinity Adjustment Variables
      integer, parameter :: NNS = 50
      integer :: ns = 0  ! # of 'PSAL ADJUST' lines in meta file
                         ! (# of sal. adjustments on the float.)
      integer, dimension(NNS) :: sal_adj_dd_save, sal_adj_mm_save,
     *                           sal_adj_yyyy_save, sal_adj_pnum_save
      real, dimension(NNS) :: sal_offset_save, sal_std_save
      character, dimension(NNS) :: sal_adj_line_save*128
 
      ! Auxiliar variables
      integer :: ios ! Hold error code of last I/O operation
      integer :: i, lnblnk, lastnb, ll, nn, lchar_occur, ibeg,
     *           nb, nlen, ndec
      character (LEN=80) :: cline ! To read a line in the meta file
      character :: cnp*6, cnp0*7

      if (DEBUG) write(DBG,*)'<<< DEBUG get_salinity_in_meta >>>'

      read(iunit,'(a)',iostat=ios) cline
      do while (ios==IO_OK)

        if (cline(1:11)=='PSAL ADJUST') then
          lastnb=lnblnk(cline)
          sal_adj_line=cline(1:lastnb)
          ibeg=41
          nn=index(cline(ibeg:lastnb),' ')+ibeg-1
          if (nn-ibeg==2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_dd
          if (nn-ibeg==1) read(cline(ibeg:nn-1),'(i1.1)') sal_adj_dd
          ibeg=nn+1
          nn=index(cline(ibeg:lastnb),' ')+ibeg-1
          if (nn-ibeg==2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_mm
          if (nn-ibeg==1) read(cline(ibeg:nn-1),'(i1.1)') sal_adj_mm
          ibeg=nn+1
          nn=index(cline(ibeg:lastnb),' ')+ibeg-1
          if (nn-ibeg==4) read(cline(ibeg:nn-1),'(i4.4)') sal_adj_yyyy
          if (nn-ibeg==2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_yyyy
          ibeg=nn+1
          nn=index(cline(ibeg:lastnb),' ')+ibeg-1
          if (nn-ibeg==3) read(cline(ibeg:nn-1),'(i3.3)') sal_adj_pnum
          if (nn-ibeg==2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_pnum
          if (nn-ibeg==1) read(cline(ibeg:nn-1),'(i1.1)') sal_adj_pnum
          ibeg=nn+2  ! There are 2 spaces after sal_adj_pnum.
          if (cline(ibeg-1:ibeg-1)=='-') ibeg=nn+1
          nn=index(cline(ibeg:lastnb),'.')+ibeg-1
                    nn=index(cline(ibeg:lastnb),'.')+ibeg-1
          ll=index(cline(ibeg:lastnb),' ')+ibeg-1
          nlen=ll-ibeg
          ndec=ll-nn-1

          cnp0(1:2)='(f'
          if (nlen<=10) then
            write(cnp0(3:4),'(i2)') nlen
            cnp0(5:5)='.'
            write(cnp0(6:6),'(i1)') ndec
            cnp0(7:7)=')'
          else
            write(cnp0(3:3),'(i1)') nlen
            cnp0(4:4)='.'
            write(cnp0(5:5),'(i1)') ndec
            cnp0(6:6)=')'
          end if

          read(cline(ibeg:ll),cnp0) sal_offset

          if (nlen>11 .or. ndec>7) then
            write(SCREEN,*)'Error: add this format for sal_offset!'
            write(SCREEN,*)'nlen=',nlen,' ndec=',ndec
          end if

          ll=lchar_occur(cline,' ')
          nn=lchar_occur(cline,'.')
          nlen=lastnb-ll
          ndec=lastnb-nn

          cnp(1:2)='(f'
          cnp(4:4)='.'
          cnp(6:6)=')'
          write(cnp(3:3),'(i1)') nlen
          write(cnp(5:5),'(i1)') ndec

          read(cline(ibeg:ll),cnp) sal_std

          !
          ! Add this line's sal. adj. variables to
          ! their correspondent vectors
          !
          ns=ns+1
          sal_adj_dd_save(ns)=sal_adj_dd
          sal_adj_mm_save(ns)=sal_adj_mm
          sal_adj_yyyy_save(ns)=sal_adj_yyyy
          sal_adj_pnum_save(ns)=sal_adj_pnum
          sal_offset_save(ns)=sal_offset
          sal_std_save(ns)=sal_std
          sal_adj_line_save(ns)=sal_adj_line
          if (ns>NNS) then
            write(SCREEN,*)'adjsal2phy=> Dimension error ns=',ns
            write(SCREEN,*)'adjsal2phy=> Increase NNS=',NNS
            write(ERR,*)'adjsal2phy=> Dimension error ns=',ns
            write(ERR,*)'adjsal2phy=> Increase NNS=',NNS
          end if
        end if
        read(iunit,'(a)',iostat=ios) cline
      end do

      if (ios <> IO_EOF) then
        ! There was some IO error in previous do loop
        write(SCREEN,*)'Reading rd_float_meta.f error=999 ',cline
        write(ERR,*)'Reading rd_float_meta.f error=999 ',cline
        return
      end if

      !
      ! If there is more than 1 SAL ADJUST line in
      ! meta file, pick the correct one for the profile Profile Number
      !

      if (ns>1) then
        do i=ns, 1, -1
          if (prof_num >= sal_adj_pnum_save(i) ) then
              sal_adj_dd=sal_adj_dd_save(i)
              sal_adj_mm=sal_adj_mm_save(i)
              sal_adj_yyyy=sal_adj_yyyy_save(i)
              sal_adj_pnum=sal_adj_pnum_save(i)
              sal_offset=sal_offset_save(i)
              sal_std=sal_std_save(i)
              sal_adj_line=sal_adj_line_save(i)
              exit
          end if
        end do
      end if

      return
      end subroutine get_salinity_in_meta

! ---------------------------------------------------------------------
! subroutine close_files
!    E. Ramos Oct 13, 2009
!    Close files before exiting the program 
! ---------------------------------------------------------------------
      subroutine close_files(DEBUG,DBG,ERR)

        implicit none
        logical, intent(in) :: DEBUG ! Is debug on?
        integer, intent(in)  :: DBG ! handler for debug file 
        integer, intent(in)  :: ERR ! handler for error file 

        if (DEBUG) close(DBG) ! close debug file
        close(ERR)               ! close error file

        return

      end subroutine close_files

