c Revision Date: 
c               
c
c* anlz_check_s.f
c  ==============
c  yhd, 23-Jan-2007
c
c  This test checks salinity against the Dave Behringer's weekly analysis.
c
c  inputs:
c    icho - 1) sal or 2) sal_adj
c    tem - temperature array
c    sal - salinity array
c    dep - depth array
c    nlev - number of depth/salinity pairs
c    icksm_flag - check sum flags
c    istatus - 
c    icomplet - profile is complete or not
c    amax - maximum flag of is_anlz array
c  return:
c    is_anlz - flag array containing - 1 good point
c                                    - 4 bad  point
c    iover_anlz - overall flag value
c    anlz_flag - 'P' or 'F'
c
      subroutine anlz_check_s(icho,tem,sal,dep,nlev,icksm_flag,
     *           amin,amax,istatus,icomplet,is_anlz,iover_anlz,
     *           anlz_flag)

      implicit none
      integer n,nlev,icksm_flag(*),idbug,istatus,is_anlz(*),
     *        iover_anlz,amin,amax,icho,icomplet
      real tem(*),dep(*),sal(*),fv
      character anlz_flag*1,var*5

      data idbug /0/      ! make it 1 to debug
      data fv /-999.0/

      if (icho.eq.1) var='S'
      if (icho.eq.2) var='S_ADJ'
      icomplet=1

      if (nlev.eq.0) then
        write(6,*) 'ANLZ. TEST ',var,' can not be applied.'
        write(48,*)'ANLZ. TEST ',var,' can not be applied.'
        anlz_flag='N'
        iover_anlz=0
        return
      end if

      do n=1,nlev
        if (icksm_flag(n).eq.0 .and. tem(n).eq.0. 
     *     .and. dep(n).eq.0. .and. sal(n).eq.0.) then
          is_anlz(n)=0
          icomplet=0                         ! incomplete profile
        end if                          
        if ((dep(n).eq.fv.or.tem(n).eq.fv.or.sal(n).eq.fv).or.
     *    sal(n).eq.fv) then
          is_anlz(n)=0
          icomplet=0
        end if
      end do

      if (istatus.eq.7.or.istatus.eq.8) then
        write(6,*) 'ANLZ. TEST ',var,' can not be applied.'
        write(48,*)'ANLZ. TEST ',var,' can not be applied.'
        anlz_flag='N'
        iover_anlz=0
        do n=1,nlev
          is_anlz(n)=0
        end do
        return
      end if
 
      do n=1,nlev
        if (istatus.ne.7.and.istatus.ne.8) then
          if (is_anlz(n).lt.amin) amin=is_anlz(n)
          if (is_anlz(n).gt.amax) amax=is_anlz(n)
        end if             ! if (status.ne.7.and.status.ne.8) then
      end do

      if (amax.eq.4) then
        write(6,*) 'ANALYSIS CHECK ',var,' failed.'
        write(48,*) 'ANALYSIS CHECK ',var,' failed.'
        anlz_flag='F'                        ! 01-26-2001
        iover_anlz=4
      else if (amax.eq.1) then
        write(6,*) 'ANALYSIS CHECK ',var,' passed.'
        write(48,*) 'ANALYSIS CHECK ',var,' passed.'
        anlz_flag='P'                        ! 01-26-2001
        iover_anlz=1
      end if
      return
      end
