c Revision Date: 05-Jun-03 (YHD - salinity is fillvalue case)
c                11-Oct-01 (YHD - negative depth check added)
c
c* clim_check_s.f
c  ==============
c  yhd, 23-Jan-2007
c  This test checks salinity against the Levitus 2001 climatology.
c
c  inputs:
c    tem - temperature array
c    sal - salinity array
c    dep - depth array
c    nlev - number of depth/salinity pairs
c    icksm_flag - check sum flags
c    icomplet - profile completed or not
c    istatus - 
c    cmax - maximum flag of is_clim array
c  return:
c    is_clim - flag array containing - 1 good point
c                                    - 4 bad  point
c    iover_clim - overall flag value
c    clim_flag - 'P' or 'F'
c
      subroutine clim_check_s(icho,tem,sal,dep,nlev,icksm_flag,
     *           cmin,cmax,istatus,icomplet,is_clim,iover_clim,
     *           clim_flag)

      implicit none
      integer n,nlev,icksm_flag(*),idbug,istatus,is_clim(*),
     *        iover_clim,cmin,cmax,icho,icomplet

      real tem(*),dep(*),sal(*),fv
      character clim_flag*1,var*5

      data idbug /0/      ! make it 1 to debug
      data fv /-999.0/

      if (icho.eq.1) var='S'
      if (icho.eq.2) var='S_ADJ'
      icomplet=1

      if (nlev.eq.0) then
        write(6,*) 'CLIM. TEST ',var,' can not be applied.'
        write(48,*)'CLIM. TEST ',var,' can not be applied.'
        clim_flag='N'
        iover_clim=0
        return
      end if

      do n=1,nlev
        if (icksm_flag(n).eq.0 .and. tem(n).eq.0. 
     *     .and. dep(n).eq.0. .and. sal(n).eq.0.) then
          is_clim(n)=0
          icomplet=0                         ! incomplete profile
        end if                          
        if ((dep(n).eq.fv.or.tem(n).eq.fv.or.sal(n).eq.fv).or.
     *    sal(n).eq.fv) then
          is_clim(n)=0
          icomplet=0
        end if
      end do

      if (istatus.eq.7.or.istatus.eq.8) then
        write(6,*) 'CLIM. TEST ',var,' can not be applied.'
        write(48,*)'CLIM. TEST ',var,' can not be applied.'
        clim_flag='N'
        iover_clim=0
        do n=1,nlev
          is_clim(n)=0
        end do
        return
      end if
 
      do n=1,nlev
        if (istatus.ne.7.and.istatus.ne.8) then
          if (is_clim(n).lt.cmin) cmin=is_clim(n)
          if (is_clim(n).gt.cmax) cmax=is_clim(n)
        end if             ! if (status.ne.7.and.status.ne.8) then
      end do

      if (cmax.eq.4) then
        write(6,*) 'CLIMA. CHECK ',var,'   failed.'
        write(48,*)'CLIMA. CHECK ',var,'   failed.'
        clim_flag='F'                        ! 01-26-2001
        iover_clim=4
      else if (cmax.eq.1) then
        write(6,*) 'CLIMA. CHECK ',var,'   passed.'
        write(48,*)'CLIMA. CHECK ',var,'   passed.'
        clim_flag='P'                        ! 01-26-2001
        iover_clim=1
      end if
      return
      end
