!
! Revision date: 2013-04-02 VMH Iitial version

module communication

  implicit none
 
  integer, parameter :: LMAX = 2200 ! max. nbr of levels in one profile1
 
  integer, parameter :: LOG = 6      ! lun for log file
  integer, parameter :: TRC = 48     ! lun for trace file
  integer, parameter :: ERR = 50     ! lun for error file
  integer, parameter :: TAB = 49     ! lun for results table
  integer, parameter :: INP = 24     ! general input unit
  integer, parameter :: OUT = 52     ! general output unit
  integer, parameter :: DBG = 66     ! debug log file

    logical, parameter :: DEBUG = .false.   ! debug is off
    integer, parameter :: DEBUG_LEV = 0     ! debug is off
  ! logical, parameter :: DEBUG = .true.    ! debug is on
  ! integer, parameter :: DEBUG_LEV = 1     ! inp/out lines
  ! integer, parameter :: DEBUG_LEV = 2     ! where do flags change? 
  ! integer, parameter :: DEBUG_LEV = 3     ! dengrad debugs
  ! integer, parameter :: DEBUG_LEV = 4     ! juldqc debugs
  ! integer, parameter :: DEBUG_LEV = 5     ! column problems
  ! debug level: 0 = off
  !              1 = normal messages/ and i/o lines
  !              2 = where do flags change?
  !              3 & DEBUG=.true.  = normal messages, write out dengrad arrays
  !              3 & DEBUG=.false. = no messages, write out dengrad arrays
  !              4 = juld/qc debugs
  !              5 = column problems (oxygen/long cline truncation)
  character(len=132)  :: msg
  character(len=132) :: msginp
  character(len=132) :: msgout

  public :: LMAX
  public :: LOG, TRC, ERR, TAB, INP, OUT, DBG
  public :: DEBUG, DEBUG_LEV
  public :: msginp, msgout
  public :: LogIt
  public :: TimeStampMsg

contains
  subroutine LogIt(caller,msg)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg
    integer lnblnk
    write(LOG,'(a,": ",a)')caller,msg(1:lnblnk(msg))
  end subroutine LogIt

  subroutine LogTRC(caller,msg)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg
    integer lnblnk
    write(TRC,'(a)')msg(1:lnblnk(msg))               ! do not log who is calling this
    write(LOG,'(a,": ",a)')caller,msg(1:lnblnk(msg)) ! except in the LOG version
  end subroutine LogTRC

  subroutine LogDBG(caller,msg)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg
    integer lnblnk
    write(DBG,'(a,": D ",a)')caller,msg(1:lnblnk(msg))
    write(LOG,'(a,": D ",a)')caller,msg(1:lnblnk(msg))
  end subroutine LogDBG

  subroutine DBGit(caller,msg)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg
    integer lnblnk
    write(DBG,'(a,": D ",a)')caller,msg(1:lnblnk(msg))
  end subroutine DBGit

  subroutine RawDBG(caller,msg) ! Do NOT put the spDsp in the debug file
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg
    integer lnblnk
    write(DBG,'(a)')msg(1:lnblnk(msg))
    ! write(LOG,'(a,": D ",a)')caller,msg(1:lnblnk(msg))
  end subroutine RawDBG


 subroutine LogTAB(caller,msg)
    character(*), intent( in )     :: caller
    character(*), intent( in )     :: msg
    integer lnblnk
    ! NOTEST write(LOG,'(a,": LOGTABTESTTEST",a)')caller,msg(1:lnblnk(msg))
    ! NOTEST write(TAB,'("TABTABTESTTEST ",a)')msg(1:lnblnk(msg))
    write(LOG,'(a)') msg(1:lnblnk(msg))
    write(TAB,'(a)') msg(1:lnblnk(msg))
  end subroutine LogTAB

 function TimeStampMsg(Input_String) result ( Output_String )
    character( * ), intent( in )     :: Input_String
    character( len( Input_String )+19 ) :: Output_String
    character(len=8)  :: date
    character(len=10) :: time
    character(len=19) :: formatted_date
    ! For testing, we have no functionality - just out=in

    call date_and_time( DATE=date, TIME=time )
    formatted_date(1:4) = date(1:4)
    formatted_date(5:5) = '-' 
    formatted_date(6:7) = date(5:6)
    formatted_date(8:8) = '-'
    formatted_date(9:10)= date(7:8)
    formatted_date(11:11) = ' '
    formatted_date(12:13)= time(1:2)
    formatted_date(14:14) = ':'
    formatted_date(15:16)= time(3:4)
    formatted_date(17:17) = ':'
    formatted_date(18:19)= time(5:6)

    Output_String = formatted_date // Input_String
  end function TimeStampMsg

  integer function open_file(unit,pathname,file_description, &
    program,out1_unit,out2_unit)

  !
  ! Function open_file
  ! E. Ramos Oct 20, 2009
  ! Opens a file.
  ! Return values: 0->Success  9->Error opening file
  !
    implicit none
    integer, intent(in) :: unit ! File handler
    character(len=*), intent(in) :: pathname ! file name
    character(len=*), intent(in) :: file_description
    character(len=*),  intent(in) :: program ! caller program
    integer, intent(in) :: out1_unit ! Handler for writing msg(1)
    integer, intent(in) :: out2_unit ! Handler for writing msg(2)
    integer :: ios ! IO result
    integer :: lnblnk
 
    open(unit,file=pathname,form='formatted',position='append', &
      recl=1000,iostat=ios)
    if(ios<>0) then
      open_file = 9
      if(out1_unit<>0) then
        write(out1_unit, '(A,": Error opening ",A, A," ios=",i4)') &
      program,file_description,pathname(1:lnblnk(pathname)),ios
        write(out1_unit,*) program, ': STOP -> STATUS = 9'
      endif
    else
      open_file = 0
    endif
  end function open_file

end module communication
