!  Revision date: 01-Oct-10 (ER  - fixed bug calculating current date
!                 31-Mar-10 (ER  - printing date from profile and now
!                                  for comparison when error future date
!                                  occurs )
!                 23-Mar-10 (ER  - using Zulu date for comparison)
!                 21-Oct-09 (ER  - add date in the future test)
!                 02-Oct-02 (YHD - impossible date exits with status=6)
!
!    Date_check.f
!    ============
!    Checks impossbile date and time.
!    yhd, 01-Jun-01
!
!    output: io=0  good; io=1 bad
!
      subroutine date_check(iyr,imo,idy,ihr,imn,io)

      implicit none

      integer iyr,imo,idy,ihr,imn,idays_mon(12),ndays,io
      character(len=10), parameter :: sub_prog = 'Date_check'
      data idays_mon/31,28,31,30,31,30,31,31,30,31,30,31/
     
      ! variables to be used on 'is in the future test' 
      character(len=12)  :: today 
      character(len=4)   :: c_year
      character(len=2)   :: c_month, c_day, c_hour, c_min
      character(len=1)   :: aux_1_char
      character(len=12)  :: c_input_date ! Input date as a char.

      integer  :: time  ! Function that returns # of sec since 1/1/1900 as an integer
      integer, dimension(9) ::  tarray(9)


      if (iyr<1997) then
        write(6, '(A,": Date check Year error - iyr=",i4)') 
     *          sub_prog, iyr
        write(50, '(A,": Date check Year error - iyr=",i4)') 
     *          sub_prog, iyr
        io=1
        return
      end if

      if (imo.lt.1.or.imo.gt.12) then
        write(6, '(A,": Date check Month error - imo=",i4)') 
     *          sub_prog, imo
        write(50, '(A,": Date check Month error - imo=",i4)') 
     *          sub_prog, imo
        io=1
        return
      end if
      
      ndays=idays_mon(imo)
      if (mod(iyr,4).eq.0.and.imo.eq.2) ndays=idays_mon(imo)+1     
      if (idy.lt.1.or.idy.gt.ndays) then
        write(6, '(A,": Date check Day error - idy=",i4)') 
     *          sub_prog, idy
        write(50, '(A,": Date check Day error - idy=",i4)') 
     *          sub_prog, idy
        io=1
        return
      end if

      if (ihr.lt.0.or.ihr.gt.24) then
        write(6, '(A,": Date check Hour error - ihr=",i4)') 
     *          sub_prog, ihr
        write(50, '(A,": Date check Hour error - ihr=",i4)') 
     *          sub_prog, ihr
        io=1
        return
      end if

      if (imn.lt.0.or.imn.gt.60) then
        write(6, '(A,": Date check Min error - imn=",i4)') 
     *          sub_prog, imn
        write(50, '(A,": Date check Min error - imn=",i4)') 
     *          sub_prog, imn
        io=1
        return
      end if

      !
      ! Date, Time input is valid
      ! Now let's check whether it is in the future
      !
      write(c_year,'(i4)')iyr

      if (imo<10) then
        write(aux_1_char,'(i1)')imo
        c_month = '0'//aux_1_char
      else
        write(c_month,'(i2)')imo
      end if

      if (idy<10) then
        write(aux_1_char,'(i1)')idy
        c_day = '0'//aux_1_char
      else
        write(c_day,'(i2)')idy
      end if

      if (ihr<10) then
        write(aux_1_char,'(i1)')ihr
        c_hour = '0'//aux_1_char
      else
        write(c_hour,'(i2)')ihr
      end if

      if (imn<10) then
        write(aux_1_char,'(i1)')imn
        c_min = '0'//aux_1_char
      else
        write(c_min,'(i2)')imn
      end if

      c_input_date = c_year // c_month // c_day // c_hour // c_min

      !
      ! Conform current date & time
      !
      !   -- Return system time in Zulu
      call gmtime( time(), tarray )  
      !   -- Get year
      write(today(1:4),'(i4)') 1900+tarray(6)
      !   -- Get month
      if (tarray(5)<9) then
        write(today(5:6),'(A1,i1)')"0",tarray(5)+1 
      else
        write(today(5:6),'(i2)')tarray(5)+1    
      end if
      !  -- Get day
      if (tarray(4)<10) then
        write(today(7:8),'(A1,i1)')"0",tarray(4)  
      else
        write(today(7:8),'(i2)')tarray(4) 
      end if
      !  -- Get hour
      if (tarray(3)<10) then
        write(today(9:10),'(A1,i1)')"0",tarray(3) 
      else
        write(today(9:10),'(i2)')tarray(3)
      end if
      ! Get minute
      if (tarray(2)<10) then
        write(today(11:12),'(A1,i1)')"0",tarray(2) 
      else
        write(today(11:12),'(i2)')tarray(2) 
      end if

      if (c_input_date < today) then
        io = 0
      else                     
        write(6, 100)sub_prog,": Date check in the future!",
     *    c_input_date, " Now: ",today
        write(50, 100)sub_prog,": Date check in the future!",
     *    c_input_date, " Now: ",today
 100    format(A10,A27,1x,A12,A6,A12)
        io = 1
      end if

      return

      end subroutine date_check

