!***************************************************************************
!     revision data : 2013-04-02 VMH Replacement of old dengrad_check.f
!                                    To be kept for comparison or emergency replacement
!                     01-Mar-13  VMH - renamed to old in favor of density gradient/
!                                      threhold check
!                     02-Oct-03 (YHD - check dimension & parameter)
!                     10-Feb-03 (YHD - fill value(-999.0) in data line case)
!
!+
!      inputs: npts_in  --- Number of depth/temp/salinity
!              JWNDO    --- Numbers used to calculate the density
!                        gradient (ex: 3 means one previous and one after
!                        point; higher this number indicates the smoother)
!              Maxdim   -- Maximum size of t,s,p arraies
!              s_in     -- Salinity array
!              t_in     -- Temperature array
!              p_in     -- Pressure array
!              xlon     -- Longitude
!              xlat     -- Latitude
!      output: id_flag   - array that holding the flag values
!                        (0:cannot calculate; 1:good; 4:bad)
!      YHD, 29-Nov-2000 Molly O. Baringer is author of this s/r.
!
!      Use odd value for jwndo to have regularily gridded array
!
!      array returns bvfrq scaled by 100**2  (N**2)
!      array_e returns the PV scaled by 1e14
!-
subroutine dengrad_check_old (npts_in, JWNDO, &
    Maxdim,s_in,t_in,p_in,xlon,xlat,id_flag)

use communication                       ! module

implicit none

character(len=7), parameter :: SUB  = 'dengold'
character(len=10), parameter :: SUBI = 'dengrad I '
character(len=10), parameter :: SUBD = 'dengrad D '
integer JWNDO,kobs
integer status
integer, parameter :: max_zerogroup = 20;
integer max_level,Maxdim,id_flag(*),i,j,igood, &
    icnt,iend,imany,iend_pck(max_zerogroup), &
    imany_pck(max_zerogroup), &
    ibeg,k,npts,npts_in
real s_in(*),t_in(*),p_in(*),s(lmax),p(lmax),t(lmax)
real array(lmax),array_e(lmax),xlon,xlat,rlon,rlat,fcor,G,eps,fv, &
    array_new(lmax)
real old_bvfrq,pav,dum			! associated with bvfrq function
data eps /1.0e-01/
data fv /-999.0/

write(msg,*)'<<<<< DEBUG from DENGRAD_CHECK_OLD DEBUG_LEV is ',DEBUG_LEV, '>>>>>'; call LogDBG(SUB,msg)

!
!* In case when pres, tem, or sal values are -999.0, find out indices 
!  for beginning and ending.

npts=npts_in
icnt=0                           ! number of -999.0 lines
iend=0
imany=0                          ! 
j=0                              ! a new counter w/o -999.0 data line
2001 format('Bad line: ',i4,f12.3,f12.3,f12.3)
do i=1,npts
  if(p_in(i).eq.fv.or.t_in(i).eq.fv.or.s_in(i).eq.fv) then
    iend=i
    imany=imany+1
    if(DEBUG_LEV==3) write(msg,2001) i,p_in(i),t_in(i),s_in(i); call LogDBG(SUB,msg)
  else
    if(iend.ne.0) then
      icnt=icnt+1
      if(icnt.gt.max_zerogroup) then
        status=9
        write(msg, '(A,": icnt gt max_zerogroup - ", i4,i4)') SUBI, icnt,max_zerogroup

        write(LOG, '(A,": icnt gt max_zerogroup - ", i4,i4)')  &
    SUBI, icnt,max_zerogroup
        write(ERR, '(A,": icnt gt max_zerogroup - ", i4,i4)')  &
    SUBI, icnt,max_zerogroup
        write(ERR,*) SUBI, ': STOP -> STATUS = ', status
        call exit(status)
      endif
      iend_pck(icnt)=iend
      imany_pck(icnt)=imany
      iend=0
      imany=0
    endif               ! if(iend.ne.0) then
  endif                 ! if(p(i).eq.fv.or.t(i)...
enddo                   ! do i=1,npts
if(icnt.ne.0.and.DEBUG) then
  write(DBG,*)SUBD,'The number of -999.0 gaps is ',icnt
  do k=1,icnt
    write(DBG,*)SUBD,'-999.0 line :',iend_pck(k),imany_pck(k)
  enddo
endif                    ! if(icnt.ne.0.and.idb....
     
if(icnt.ne.0) then       ! found -999.0 line
  do k=1,icnt
    ibeg=iend_pck(k)-imany_pck(k)+1
    if(DEBUG) write(DBG,*)SUBD,'-999.0 data line case.'
    if(DEBUG) write(DBG,*)SUBD,'How many -999.0 lines exist? ',imany_pck(k)
    if(DEBUG) write(DBG,*)SUBD,'ibeg,iend=',ibeg,iend_pck(k)
  enddo                         ! do k=1,icnt
  do i=1,npts
    if(p_in(i).eq.fv.or.t_in(i).eq.fv.or.s_in(i).eq.fv) then
    else                         ! loading p,t,s without -999.0
      j=j+1
      p(j)=p_in(i)
      t(j)=t_in(i)
      s(j)=s_in(i)
    endif                       ! if((p(i).eq.fv....
  enddo                         ! do i=1,npt
  npts = j
else                             ! data contains No -999.0 line
  do i = 1, npts
    p(i)=p_in(i)
    t(i)=t_in(i)
    s(i)=s_in(i)
  enddo
endif                           ! if(icnt.ne.0) then
!
RLAT=ABS(XLAT)/57.29578
G= 978.0318*(1.+5.3024E-3*SIN(RLAT)**2 &
    -5.9E-6*SIN(2*RLAT)**2)
fcor=1.458422E-4*SIN(RLAT)
!
if(npts.lt.jwndo) then
  do i=1,npts
    id_flag(i)=0
  enddo
  return
endif
DO 10 I=1,npts
  if(i.LE.(JWNDO/2))then
    array(i)=-999.
    array_e(i)=-999.
  elseif( (i+JWNDO/2).GT.npts)then
    array(i)=array(i-1)
    array_e(i)=array_e(i-1)
  else
!         continue with calculation
    kobs=I-JWNDO/2
    dum=old_bvfrq(S(kobs),T(kobs),P(kobs),JWNDO,pav,array(i))
!         for potential vorticity if(NV.EQ.35) V(i)=E*fcor/G*1E14
!          array_e(i) = arrAy(i)*fcor/g*1e14
    array(i) = array(i) * 10000.
  endif
  10  continue
! copy first good value to the surface
igood=1+jwndo/2
do i= 1, jwndo/2
 19      if(array(igood).lt.0.0) then
    igood=igood+1
    go to 19
  endif
  array(i) = array(igood)
 array_e(i) = array_e(igood)
enddo
! copy last good value to the bottom
do 30 i = npts-jwndo/2, npts
  array(i) = array(npts-jwndo/2)
  array_e(i) = array_e(npts-jwndo/2)
   30 continue
!
!  Is it pass density gradient test (=1), not(=4), or can't perform(=0)?
!------------------------------------------------------------------------
!   "array" has no -999.0 and now put this array values back into 
!   "array_new".

if(icnt.gt.0) then         ! -999.0 case exist
!
!*    Beginning to the first -999.0 data line
!
  iend=iend_pck(1)-imany_pck(1)
  j=0
  do i=1,iend
    j=j+1
    array_new(i)=array(j)
  enddo
! 
!*   -999.0 lines
!
  do k=1,icnt
    ibeg=iend_pck(k)-imany_pck(k)+1
    iend=iend_pck(k)
    do i=ibeg,iend
      array_new(i)=-1.0
    enddo
  enddo
!
!*   Between the -999.0 blocks
!
  do k=1,icnt-1
    ibeg=iend_pck(k)+1
    iend=iend_pck(k+1)-imany_pck(k+1)
    do i =ibeg ,iend
      j=j+1
      array_new(i)=array(i)
    enddo
  enddo                  ! do k=1,icnt-1
!
!*      Last gap to the end of data
!
  iend=iend_pck(icnt)
  do i=iend+1,npts_in
    j=j+1
    array_new(i)=array(i)
  enddo
else
  do i=1,npts_in
    array_new(i)=array(i)
  enddo
endif                                ! if(icnt.ne.0) then
!----------------------------------------------------------------------

do i=1,npts_in-1
    id_flag(i)=1
    if(array_new(i).lt.0.0) then        ! potentially bad salinity?
        if(abs(t_in(i)-t_in(i+1)).le..05.and.abs(s_in(i)-s_in(i+1)).le..05) then ! Is it inside of mixed layer?
              id_flag(i)=1                 !
        else if(s_in(i).eq.fv.or.t_in(i).eq.fv.or.p_in(i).eq.fv) then
              id_flag(i)=0
        else
            id_flag(i)=4 
        endif                         ! if(t(1)-t(i).le.1.0) then
    endif
enddo
id_flag(npts_in)=id_flag(npts_in-1)
if(DEBUG_LEV==3) then
    write(msg,*) ' lev flag      pres        temp        psal             array         array_new   9999.000'; call RawDBG(SUB,msg)
    do i=1,npts_in
	write(DBG,1001) i,id_flag(i),p_in(i),t_in(i),s_in(i),array(i),array_new(i)
    enddo
endif
1001 format(i4,i4,f12.3,f12.4,f12.4,f18.14,f18.14,'  -9999.000')
return
end
! end 
!*************************************
! brval ***** brunt-vaisala freq *****
!  uses 1980 equation of state
! ************************************
! units:      
!       pressure        p0       decibars
!       temperature     t        deg celsius (ipts-68)
!       salinity        s        (ipss-78)
!       bouyancy freq   bvfrq    cph
!       n**2            e        radians/second
! checkvalue: bvfrq=14.57836 cph e=6.4739928e-4 rad/sec.
!            s(1)=35.0, t(1)=5.0, p(1)=1000.0
!            s(2)=35.0, t(2)=4.0, p(2)=1002.0
!  ********note result centered at pav=1001.0 dbars **********
!  r millard 
! july 12 1982
! computes n in cycles per hour, and e=n**2 in rad/sec**2
! after formulation of breck owen's & n.p. fofonoff

function old_bvfrq(s,t,p,nobs,pav,e)
real*4 p(1),t(1),s(1)
real pav
e=0.0
old_bvfrq=0.0
if(nobs.lt.2) return
cxx=0.0
cx=0.0
cxy=0.0
cy=0.0
! compute least squares estimate of specific volume anamoly gradient
do k=1,nobs
!      write(LOG,*) k,p(k),t(k),s(k)
 cx =cx+p(k)
enddo
pav=cx/nobs
do 35 k=1,nobs
data= svan(s(k),old_theta(s(k),t(k),p(k),pav),pav,sig)*1.0e-8
cxy=cxy+data*(p(k)-pav)
cy =cy+data
cxx=cxx+(p(k)-pav)**2
   35 continue
if(cxx.eq.0.0) return
a0=cxy/cxx
v350p=(1./(sig+1000.))-data
vbar=v350p+cy/nobs
dvdp=a0
!  
if(vbar.eq.0.0) return
e   = -.96168423e-2*dvdp/(vbar)**2
old_bvfrq = 572.9578*sign(sqrt(abs(e)),e) 
!      write(LOG,*) 'b2   ',old_bvfrq,'  e',e
return
end
! ***********************************************************
! to compute local potential temperature at pr
! using bryden 1973 polynomial for adiabatic lapse rate
! and runge-kutta 4-th order integration algorithm.
! ref: bryden,h.,1973,deep-sea res.,20,401-408
! fofonoff,n.,1977,deep-sea res.,24,489-491
! units:      
!       pressure        p0       decibars
!       temperature     t0       deg celsius (ipts-68)
!       salinity        s        (ipss-78)
!       reference prs   pr       decibars
!       potential tmp.  theta    deg celsius 
! checkvalue: theta= 36.89073 c,s=40 (ipss-78),t0=40 deg c,
! p0=10000 decibars,pr=0 decibars
!             
!      set-up intermediate temperature and pressure variables

real function old_theta(s,t0,p0,pr)

p=p0
t=t0
h = pr - p
xk = h*old_atg(s,t,p) 
t = t + 0.5*xk
q = xk  
p = p + 0.5*h 
xk = h*old_atg(s,t,p) 
t = t + 0.29289322*(xk-q) 
q = 0.58578644*xk + 0.121320344*q 
xk = h*old_atg(s,t,p) 
t = t + 1.707106781*(xk-q)
q = 3.414213562*xk - 4.121320344*q
p = p + 0.5*h 
xk = h*old_atg(s,t,p) 
old_theta = t + (xk-2.0*q)/6.0
return  
end     
! **************************** 
real function old_atg(s,t,p) 
! **************************** 
! adiabatic temperature gradient deg c per decibar
! ref: bryden,h.,1973,deep-sea res.,20,401-408
! units:      
!       pressure        p        decibars
!       temperature     t        deg celsius (ipts-68)
!       salinity        s        (ipss-78)
!       adiabatic       atg      deg. c/decibar
! checkvalue: atg=3.255976e-4 c/dbar for s=40 (ipss-78),
! t=40 deg c,p0=10000 decibars
ds = s - 35.0 
old_atg = (((-2.1687e-16*t+1.8676e-14)*t-4.6206e-13)*p &
    +((2.7759e-12*t-1.1351e-10)*ds+((-5.4481e-14*t &
    +8.733e-12)*t-6.7795e-10)*t+1.8741e-8))*p  &
    +(-4.2393e-8*t+1.8932e-6)*ds &
    +((6.6228e-10*t-6.836e-8)*t+8.5258e-6)*t+3.5803e-5
return
end
!=============================================================================
	real function svan(S,T,P0,SIGMA)
!
!  	modified rcm
! 
! 	Specific volume anomaly (steric anomaly) based on 1980 equation
! 	    of state for seawater and 1978 practical salinity scale.
! 	References:
! 	Millero, et al (1980) Deep-Sea Res.,27a,255-264
! 	Millero and Poisson 1981, Deep-Sea Res.,28a pp 625-629.
! 	Both above references are also found in UNESCO Report 38 (1981)
!
! 	units:      
!       pressure        p0       decibars
!       temperature     t        deg celsius (ipts-68)
!       salinity        s        (ipss-78)
!       spec. vol. ana. svan     m**3/kg *1.0e-8
!       density ana.    sigma    kg/m**3
! 
! 	check value: svan=981.3021 e-8 m**3/kg.  for s = 40 (ipss-78) ,
! 	t = 40 deg c, p0= 10000 decibars.
! 	check value: sigma = 59.82037  kg/m**3 for s = 40 (ipss-78) ,
! 	t = 40 deg c, p0= 10000 decibars.
! 
	real p,t,s,sig,sr,r1,r2,r3,r4
	real a,b,c,d,e,a1,b1,aw,bw,k,k0,kw,k35
! 	
	equivalence (e,d,b1),(bw,b,r3),(c,a1,r2) 
	equivalence (aw,a,r1),(kw,k0,k)
! 	
	data r3500,r4/1028.1063,4.8314e-4/
	data dr350/28.106331/
!
!   	r4 is refered to as  c  in Millero and Poisson 1981
! 	convert pressure to bars and take square root salinity.
!
	p=p0/10.
	sr = sqrt(abs(s)) 
! 
! 	Pure water density at atmospheric pressure
!   	Bigg P.H.,(1967) Br. J. Applied Physics 8 pp 521-537.
!
	r1 = ((((6.536332e-9*t-1.120083e-6)*t+1.001685e-4)*t  &
    		-9.095290e-3)*t+6.793952e-2)*t-28.263737
!
! 	Seawater density atm press. 
!  	coefficients involving salinity
!  	r2 = a   in notation of Millero and Poisson 1981
!
	r2 = (((5.3875e-9*t-8.2467e-7)*t+7.6438e-5)*t-4.0899e-3)*t &
    		+8.24493e-1 
!
!  	r3 = b  in notation of Millero and Poisson 1981
!
	r3 = (-1.6546e-6*t+1.0227e-4)*t-5.72466e-3
!
!  	International one-atmosphere equation of state of seawater
!
	sig = (r4*s + r3*sr + r2)*s + r1 
!
! 	specific volume at atmospheric pressure
!
	v350p = 1.0/r3500
	sva = -sig*v350p/(r3500+sig)
	sigma=sig+dr350
!
!  	scale specific vol. anamoly to normally reported units
!
	svan=sva*1.0e+8
	if(p.eq.0.0) return
! 
! 	new high pressure equation of state for seawater 
! 
!   	Millero, et al , 1980 dsr 27a, pp 255-264
!               constant notation follows article
!
! 	compute compression terms
	e = (9.1697e-10*t+2.0816e-8)*t-9.9348e-7
	bw = (5.2787e-8*t-6.12293e-6)*t+3.47718e-5
	b = bw + e*s
!             
	d = 1.91075e-4
	c = (-1.6078e-6*t-1.0981e-5)*t+2.2838e-3
	aw = ((-5.77905e-7*t+1.16092e-4)*t+1.43713e-3)*t  &
    		-0.1194975
	a = (d*sr + c)*s + aw 
!             
	b1 = (-5.3009e-4*t+1.6483e-2)*t+7.944e-2
	a1 = ((-6.1670e-5*t+1.09987e-2)*t-0.603459)*t+54.6746 
	kw = (((-5.155288e-5*t+1.360477e-2)*t-2.327105)*t  &
    		+148.4206)*t-1930.06
	k0 = (b1*sr + a1)*s + kw
!
! 	evaluate pressure polynomial 
! 
!   	k equals the secant bulk modulus of seawater
!   	dk=k(s,t,p)-k(35,0,p)
!  	k35=k(35,0,p)
! 
	dk = (b*p + a)*p + k0
	k35  = (5.03217e-5*p+3.359406)*p+21582.27
	gam=p/k35
	pk = 1.0 - gam
	sva = sva*pk + (v350p+sva)*p*dk/(k35*(k35+dk))
!
!  	scale specific vol. anamoly to normally reported units
!
	svan=sva*1.0e+8
	v350p = v350p*pk
!  
! 	compute density anamoly with respect to 1000.0 kg/m**3
!  	1) dr350: density anamoly at 35 (ipss-78), 0 deg. c and 0 decibars
!  	2) dr35p: density anamoly 35 (ipss-78), 0 deg. c ,  pres.variation
!  	3) dvan : density anamoly variations involving specfic vol.anamoly
! 
! 	check value: sigma = 59.82037  kg/m**3 for s = 40 (ipss-78),
! 	             t = 40 deg c, p0= 10000 decibars.
! 
	dr35p=gam/v350p
	dvan=sva/(v350p*(v350p+sva))
	sigma=dr350+dr35p-dvan
	return  
	end     

