! Revision date: 2013-04-02 VMH - use communication for LUNS, array sizes and parameters
!                18-Dec-12 VMH  - increase size of cline to 256
!                14-Aug-12 VMH    Accept ahd write out st_trans_time,st_trans_time_qc,
!                                 ascent_end_time,ascent_end_time_qc
!                17-May-11 (ER  - Conversion of flags2qcfile to Fortran 90.
!                               - Correction of bug causing program to 
!                                 fail when processing profiles with 2 
!                                 oxygen sensors.
!                               - Substantial simplification of code 
!                               - Variable itype replaced by hasTemp, 
!                                 hasPsal, hasCond, hasO2  
!                05-Nov-10 (ER  - Changes to deal with new gross pressure
!                                 check
!                               - Vertical temp and density salinity flag
!                                 not sharing the same spot in subroutine
!                                 signature
!                               - Changes in variable names to make it more
!                                 readable
!                16-Jul-10 (ER  - Changes dealing with surface 
!                                 pressure = null
!                15-Jun-10 (ER  - When number of column=10, columns were
!                                 being repeated.flags2qcfile.f
!                07-Jun-10 (ER  - Treatment of pressure adjusted
!                05-Nov-09 (ER  - Changes in grey list parameter type and
!                                 name) 
!                21-May-09 (YHD - Do not write s2 flags when ts_32 type)
!                08-May-09 (YHD - PRESSURE2 and TEMPERATURE2 case)
!                30-Apr-09 (YHD - No data line case with columns info exist.)
!                                 ex:1300_52730_149 or 1572_57405_135
!                30-Jan-08 (YHD - Breck Owens float case: 2586_08580)
!                                 Make sure "NUMBER OF COLUMNS" matches
!                                 actual number of column lines.) 
!                27-Nov-07 (YHD - (0.0,0.0) case POSITION_QC changes from
!                          '0' to '-4')
!                26-Nov-07 (YHD - 'SOLOIR_TS2' type with no CHECKSUM column)
!                12-Sep-07 (YHD - A bug on line 678 fixed.)
!                10-Sep-07 (YHD - Based on speed check result, 'iclass' can 
!                                 be set '-' in front of it.)
!                07-May-07 (YHD - For bounce float, even number profile
!                                 case, write only 'T' section)
!                29-Mar-07 (YHD - GREY list correction)
!                07-Mar-07 (YHD - More correction regarding PSAL_ADJ)
!                07-Feb-07 (YHD - Separated into two parts (T and S)).
!                                
!  flags2qcfile.f
! ===============
!  Creates 'qc' file with flags from 'qc_t' and 'qc_s' run.
!  yhd, 07-Feb-07 
!
!  icho : 1-writing T; 2-writing S
!  nf   : number of depth/temp/sal pairs
!  mx_n : overall flag for constant value test
!  mx_k : overall flag for frozen test
!  aflag: analysis flags
!  cflag: climatology flags
!  deepest_pres_flag: deepest pressure check flag
!  gross_pres_flag
!  gross_temp_flag
!  gross_temp_sal
!  jflag: jump flag
!  vert_grad_sal_flag: vertical grad for salinity flag
!  pflag: spike flag
!  rflag: pressure increasing flag
!  sflag: speed flag
!  vflag: vertical grad
!  qflag: regional flag
!  aaflag: analysis for sal_adj flag
!  caflag: climatology for sal_adj flag
!  gaflag: gross for sal_adj flag
!  raflag: regional for sal_adj flag
!  surface_pressure: surface pressure value.
!  density_grad_flag
 
subroutine flags2qcfile(icho, hasTemp, hasPsal, hasCond, hasO2,    & ! 0
  phyfil, nf, iov_all, pres_in_gl, temp_in_gl, psal_in_gl,         & ! 1
  pres_gl_flag, temp_gl_flag, psal_gl_flag,                        & ! 2
  mx_n,mx_k, iregion,ireverse,no_pos,land,                         & ! 3
  j_qc,sig_c,sig_a,atime,ana_version,                              & ! 4
  sflag,aflag,cflag,                                               & ! 5
  gross_pres_flag, gross_temp_flag, gross_sal_flag,                & ! 6
  pflag, vflag, rflag,                                             & ! 7
  deepest_pres_flag, deepest_pres_adj_flag,                        & ! 8
  gross_pres_adj_flag,jflag,qflag,                                 & ! 9
  density_grad_flag, vert_grad_sal_flag, aaflag,                   & ! *
  caflag,gaflag,raflag,                                            & ! 1
  type_ts32,ipres2_flag,ideep2_flag,itemp2_flag,                   & ! 2
  gross_pres2_flag, gross_temp2_flag,                              & ! 3
  st_trans_time, st_trans_time_qc,                                 & ! 4
  ascent_end_time, ascent_end_time_qc,                             & ! 5
  ispik2_flag,bad_row,surface_pressure)                              ! 6

  use communication

  implicit none

  character(len=10), parameter :: SUBI = 'flg2qcf I '
  character(len=10), parameter :: SUBD = 'flg2qcf D '

  integer, parameter :: IO_OK  =  0  ! IO Status = 0
  integer, parameter :: IO_EOF = -1  ! IO Status = EOF
 
  integer, parameter :: TEMP = 1
  integer, parameter :: SAL = 2 
  integer :: icho  ! Which file it is writing to (TEMP or SAL)
 
  ! Has pressure/temperature grey-listed?
  logical :: pres_in_gl, temp_in_gl, psal_in_gl
  ! If so, what are the values?
  integer :: pres_gl_flag, temp_gl_flag, psal_gl_flag 

  ! QC flag vectors
  integer :: deepest_pres_flag(*), deepest_pres_adj_flag(*),         &
    aflag(*), cflag(*), gross_pres_flag(*), gross_pres_adj_flag(*),  &
    gross_temp_flag(*), gross_sal_flag(*),                           &
    pflag(*), vflag(*), rflag(*), jflag(*),                          &
    density_grad_flag(*), vert_grad_sal_flag(*), qflag(*),           &
    gaflag(*), raflag(*), aaflag(*), caflag(*)
  ! Max values for each QC flag vector
  integer :: mx_deepest_pres_flag, mx_deepest_pres_adj_flag,         &
    mx_a, mx_c, max_gross_pres_flag, max_gross_pres_adj_flag,        &
    max_gross_temp_flag, max_gross_sal_flag,                         &
    mx_p, mx_v, mx_r, mx_j,                                          &
    max_density_grad_flag, max_vert_grad_sal_flag, mx_q,             &
    max_gross_pres2_flag, max_gross_temp2_flag, mx_s, mx_n, mx_k,    &
    mx_ga, mx_ca, mx_aa, mx_ra
  data mx_a, mx_c, max_gross_pres_flag, max_gross_temp_flag,         & 
    max_gross_sal_flag, max_gross_pres_adj_flag, mx_s, mx_p, mx_v,   &
    mx_r, mx_j, max_density_grad_flag, max_vert_grad_sal_flag, mx_q, &
    mx_aa, mx_ca, mx_ga, mx_ra, mx_deepest_pres_flag,                &
    mx_deepest_pres_adj_flag /20*0/

                                  
  integer :: nf, sflag, aux_flag(lmax), iov_all, its, num_col, col_cnt, num_test

  integer :: no_pos, land, ireverse, j_qc, ilen, iregion, iadd
  logical :: hasTemp, hasPsal, hasCond, hasO2
  character (len=256) :: cline, cline_before
  character (len=128) :: phyfil, pathname_outfile
  integer :: i, kk, ncnt, ios, status
  character what_flag*1,fm*9

  character(len=4)  :: PRGMAIN
  character(len=12) :: prg_sub 
  character sflag_write*1,clim_ver*16, ana_version*50
  integer*4 lastnb,lnblnk,last,ldate(3),year,month,day
  real      sig_c,sig_a
  double precision atime
  equivalence (ldate(1),year),(ldate(2),month),(ldate(3),day)
  logical :: juld_qc_written = .false.  

  data clim_ver/'LEVITUS WOA-2001'/
  data fm/'formatted'/

  real :: surface_pressure 
  character(len=10) :: surf_pres_aux

  logical :: pos_prev_exist = .false.  ! For IRIDIUM float

  logical :: psal_adj_exist = .false.  ! Is there salinity adjusted?

  ! Pressure Adjusted
  logical :: pres_adj_exist = .false.  ! Is there pressure adjusted?
  integer :: p_adj_cnt  ! 0 if pres_adj_exist=false, 1 if pres_adj_exist=true
                        ! used to calculate column nbr for the columns after pres_adj

  ! A bad position gets '-' in front of class.
  integer :: bad_row(*)
  integer :: ipos_cnt = 0
      
 
  ! Breck Owens float with wrong "NUMBER OF COLUMN" case.
  character(len=2) :: cnum_col
  integer :: ncol = 0

  ! Optode float
  integer num_len

  ! start_of_transmission & ascend_end and their qc variables
  character(len=19) :: st_trans_time
  character(len=1) :: st_trans_time_qc
  character(len=19) :: ascent_end_time
  character(len=1) :: ascent_end_time_qc

  !  pressure2(sea surface pressure) and temperature2
  !
  integer :: ipres2_flag(*), itemp2_flag(*), ideep2_flag(*),  &
    gross_pres2_flag(*), gross_temp2_flag(*), ispik2_flag(*)  
  integer :: mx_d2, mx_r2, mx_v2, mx_p2
  character(len=1) :: type_ts32
  data mx_d2, mx_r2, mx_v2, mx_p2, max_gross_pres2_flag, max_gross_temp2_flag /6*0/

  ! Start of execution
  mx_s=max(sflag,mx_s)
  if(nf<0) nf=0

  do i=1, nf
    mx_a=max(aflag(i),mx_a)
    mx_c=max(cflag(i),mx_c)
    max_gross_pres_flag=max(gross_pres_flag(i),max_gross_pres_flag)
    max_gross_pres_adj_flag=max(gross_pres_adj_flag(i),max_gross_pres_adj_flag)
    max_gross_temp_flag=max(gross_temp_flag(i),max_gross_temp_flag)
    max_gross_sal_flag=max(gross_sal_flag(i),max_gross_sal_flag)
    mx_p=max(pflag(i),mx_p)
    mx_v=max(vflag(i),mx_v)
    mx_r=max(rflag(i),mx_r)
    mx_deepest_pres_flag=max(deepest_pres_flag(i), mx_deepest_pres_flag)
    mx_deepest_pres_adj_flag=max(deepest_pres_adj_flag(i), mx_deepest_pres_adj_flag)
    mx_j=max(jflag(i),mx_j)
    max_density_grad_flag=max(density_grad_flag(i), max_density_grad_flag)
    max_vert_grad_sal_flag=max(vert_grad_sal_flag(i), max_vert_grad_sal_flag)
    mx_q=max(qflag(i),mx_q)
    mx_aa=max(aaflag(i),mx_aa)
    mx_ca=max(caflag(i),mx_ca)
    mx_ga=max(gaflag(i),mx_ga)
    mx_ra=max(raflag(i),mx_ra)
    mx_d2=max(ideep2_flag(i),mx_d2)
    mx_r2=max(ipres2_flag(i),mx_r2)
    max_gross_pres2_flag=max(gross_pres2_flag(i), max_gross_pres2_flag)
    max_gross_temp2_flag=max(gross_temp2_flag(i), max_gross_temp2_flag)
    mx_v2=max(itemp2_flag(i),mx_v2)
    mx_p2=max(ispik2_flag(i),mx_p2)
  enddo

  if(icho==TEMP) then
    PRGMAIN = 'qc_t'
  else
    PRGMAIN = 'qc_s'
  endif
  prg_sub   = 'flags2qcfile'

  ! Convert Analysis time(atime) to year/month/day.
  call j2arr3(int(atime),ldate)

  ! Open file to be QC
  lastnb = lnblnk(phyfil)
  if(nf==0) then                         ! no data point case
    its = 0
    if(icho==TEMP) then
      sflag_write='N'
    else
      sflag_write='Y'
    endif
  else                                    ! has profile data case
    if(icho==TEMP) then
      its = 1
    else
      its = 2
    endif                               
  endif                                   

  write(LOG, '(A, ", ", A, ": Opening Profile input file - ", A)')   &
    PRGMAIN, prg_sub, phyfil(1:lastnb)

  open(INP,file=phyfil(1:lastnb),status='old',form='formatted',iostat=ios)
  if(ios<>0) then
    status=9
    write(LOG, '(A,", ",A,": Error opening PHY input file unit 24.ios=",I5)')  &
      PRGMAIN, prg_sub, ios 
    write(LOG,'(A, ", ", A, ": STOP -> STATUS= ", I5)') PRGMAIN, prg_sub, status
    write(ERR,*) PRGMAIN, '-', prg_sub, ': STOP -> STATUS= ', status
    call exit (status)
    stop '9'
  endif

  ! Open output file for QC profile with flags 
  call GETARG(3, pathname_outfile)
  lastnb = lnblnk(pathname_outfile)
  last = lastnb
  write(LOG, '(A, ", ", A, ": Opening Output QC file - ", A)')       &
    PRGMAIN, prg_sub, pathname_outfile(1:lastnb)
  open(unit=OUT,file=pathname_outfile,status='replace',form=fm, iostat=ios)
  if(ios<>0) then
    status=9
    write(LOG, '(A, ", ", A, ": Error opening qc file unit 52. ios=", I5)')  &
      PRGMAIN, prg_sub, ios 
    write(ERR,*) PRGMAIN, ',', prg_sub, 'Error opening qc file unit 52. ios=', ios
    if(ios==1017) then
      write(ERR,*) PRGMAIN, ',', prg_sub, 'Qc file already exists! Delete them and run it again.'
    endif 

    write(LOG,'(A, ", ", A, ": STOP -> STATUS= ", I5)') PRGMAIN, prg_sub, status

    write(ERR,*) PRGMAIN,',',prg_sub,': STATUS=',status,' -> STOP'
    call exit (status)
    stop '9'
  endif

  ncnt = 0
  ! 
  ! Reverse (if needed) the flags to write to qc file
  if(ireverse==1) then
    do kk = 1, 24
      do i = 1, nf
        select case (kk)
        case (1)  
          aux_flag(i)=aflag(i)   ! analys  
        case (2)  
          aux_flag(i)=gross_pres_flag(i)   
        case (3)  
          aux_flag(i)=gross_pres_adj_flag(i)   
        case (4)  
          aux_flag(i)=gross_temp_flag(i)   
        case (5)  
          aux_flag(i)=gross_sal_flag(i)   
        case (6)  
          aux_flag(i)=vflag(i)   ! vertical
        case (7)  
          aux_flag(i)=cflag(i)   ! clim
        case (8)  
          aux_flag(i)=pflag(i)   ! press
        case (9)  
          aux_flag(i)=rflag(i)   ! press increasing
        case (10)  
          aux_flag(i)=deepest_pres_flag(i)     ! deepest
        case (11) 
          aux_flag(i)=deepest_pres_adj_flag(i) ! deepest (adj) 
        case (12) 
          aux_flag(i)=jflag(i)   ! jump
        case (13) 
          aux_flag(i)=vert_grad_sal_flag(i)   ! vertical gradient salinity
        case (14) 
          aux_flag(i)=qflag(i)  ! regional
        case (15) 
          aux_flag(i)=gaflag(i) ! psal adjust
        case (16) 
          aux_flag(i)=ipres2_flag(i) ! pressure2
        case (17) 
          aux_flag(i)=itemp2_flag(i) ! temperature2
        case (18) 
          aux_flag(i)=ideep2_flag(i) ! deepest flag2
        case (19) 
          aux_flag(i)=gross_pres2_flag(i) ! pres2 flag
        case (20) 
          aux_flag(i)=gross_temp2_flag(i) ! gros2 flag
        case (21) 
          aux_flag(i)=ispik2_flag(i) ! spik2 flag
        case (22) 
          aux_flag(i)=aaflag(i)      ! aaflag
        case (23) 
          aux_flag(i)=caflag(i)      ! caflag
        case (24) 
          aux_flag(i)=density_grad_flag(i) 
        end select
      enddo
      do i = 1, nf
        select case (kk)
        case (1)
          aflag(i)=aux_flag(nf+1-i)
        case (2)
          gross_pres_flag(i)=aux_flag(nf+1-i)
        case (3)
          gross_pres_adj_flag(i)=aux_flag(nf+1-i)
        case (4)
          gross_temp_flag(i)=aux_flag(nf+1-i)
        case (5)
          gross_sal_flag(i)=aux_flag(nf+1-i)
        case (6)
          vflag(i)=aux_flag(nf+1-i)
        case (7)
          cflag(i)=aux_flag(nf+1-i)
        case (8)
          pflag(i)=aux_flag(nf+1-i)
        case (9)
          rflag(i)=aux_flag(nf+1-i)
        case (10)
          deepest_pres_flag(i)=aux_flag(nf+1-i)
        case (11)
          deepest_pres_adj_flag(i)=aux_flag(nf+1-i)
        case (12)
          jflag(i)=aux_flag(nf+1-i)
        case (13)
          vert_grad_sal_flag(i)=aux_flag(nf+1-i)
        case (14)
          qflag(i)=aux_flag(nf+1-i)
        case (15)
          gaflag(i)=aux_flag(nf+1-i)
        case (16)
          ipres2_flag(i)=aux_flag(nf+1-i)
        case (17)
          itemp2_flag(i)=aux_flag(nf+1-i)
        case (18)
          ideep2_flag(i)=aux_flag(nf+1-i)
        case (19)
          gross_pres2_flag(i)=aux_flag(nf+1-i)
        case (20)
          gross_temp2_flag(i)=aux_flag(nf+1-i)
        case (21)
          ispik2_flag(i)=aux_flag(nf+1-i)
        case (22)
          aaflag(i)=aux_flag(nf+1-i)      
        case (23)
          caflag(i)=aux_flag(nf+1-i)  
        case (24) 
          density_grad_flag(i)=aux_flag(nf+1-i)  
        end select
      enddo
    enddo
  endif

  ! Read header information for the first time to:
  !   - check whether "JULD_QC" was written.
  !   - get the number of columns
  !   - check whether salinity was adjusted
  !   - check whether pressure was adjusted
  !  
  if(DEBUG_LEV==1) write(DBG,*)SUBD,'TEST time qc values st trnstimqc: ',st_trans_time_qc, &
            ' endasctimeqc: ',ascent_end_time_qc

  read(INP,'(a)',iostat=ios) cline
  if(DEBUG_LEV==1) write(DBG,*)SUBD,'rd cline at 1: ',cline
  do while (ios==IO_OK)
    if(cline(1:5)=='======') then  ! end of the header section
      exit
    endif
    if(cline(1:7)=='JULD_QC') then
      juld_qc_written=.true.
    else if(cline(1:17)=='NUMBER OF COLUMNS') then
      read(cline(41:42),41) num_col
    else if(cline(4:9)=='COLUMN' .or. cline(5:10)=='COLUMN') then
      ncol=ncol+1
      if(.not. pres_adj_exist) then
        if(cline(41:59)=='PRESSURE_ADJ (DBAR)') then
          pres_adj_exist=.true.
        endif
      endif
    else if(cline(1:11)=='PSAL ADJUST') then
      psal_adj_exist=.true.
    endif
    read(INP,'(a)',iostat=ios) cline
    if(DEBUG_LEV==1) write(DBG,*)SUBD,'rd cline at 2: ',cline
  enddo


  ! Read header information for the second time.
  !
  rewind(INP)
1 continue
  read(INP,25,end=100) cline
  if(DEBUG_LEV==1) write(DBG,*)SUBD,'rd cline at 3: ',cline
      
  ! Julian date line (if we need to add it)
  if(cline_before(1:5)=='PI   '.and.PRGMAIN=='qc_t') then
    if(.not. juld_qc_written) then     ! (non WHOI floats)
       if(DEBUG_LEV==1) write(DBG,*)SUBD,'write out juld_qc: ',j_qc  ! DEBUG date_check
       write(OUT,79) j_qc 
    endif
  endif

  ! Number of columns line
  !   It will be affected by whether there is salinity adj 
  !   and/or pressure adjusted
  if(cline(1:17)=='NUMBER OF COLUMNS') then
    ! a.) Get the original number of columns
    read(cline(41:42),41) num_col

    ! b.) If there is no profile data just take
    !     the original number of columns
    if(nf==0) then                    
      write(OUT,60) num_col
      go to 1
    endif

    if(num_col<10) then
      write(cnum_col(1:1),'(i1)') num_col
    else
      write(cnum_col(1:2),'(i2)') num_col
    endif

    if(its==1) then
      ! qc_t is calling program
      num_test=3                 ! pressure tests.
      if(pres_adj_exist) then   ! pres. adj. tests
        num_test=num_test+2 
      endif
      if(hasTemp) then          ! temp tests
        num_test=num_test+5
      endif
      if(iregion>0) then        ! region test
        num_test=num_test+1
      endif
      if(type_ts32=='y') then   ! ts32 tests
        num_test=num_test+6
      endif
    else
      ! qc_s is calling program
      if(hasPsal) then
        num_test=6               ! salinity tests
        if(iregion>0) then
          num_test=num_test+1    ! salinity-region test
        endif
        if(psal_adj_exist) then
          num_test=num_test+3    ! sal. adj. tests
          if(iregion>0) then
            num_test=num_test+1  ! salinity-region test
          endif
        endif      
      endif
    endif

    if(num_col+num_test<10) then
      write(OUT,60) num_col+num_test
    else
      write(OUT,61) num_col+num_test
    endif
    cline_before=cline
    go to 1
  endif

  if(ncol<>num_col) then
    status=9
    write(LOG, '(A: ",", A, ": ncol not equals num_col case: ncol=", I5, " num_col=", I5)') &
      PRGMAIN, prg_sub, ncol, num_col
    write(LOG,'(A: ",", A, ": STOP -> STATUS= ",I5)') PRGMAIN, prg_sub, status
    write(ERR,'(A: ",", A, ": ncol not equals num_col case: ncol=", I5, " num_col=", I5)') &
      PRGMAIN, prg_sub, ncol, num_col
    write(ERR,*) PRGMAIN, ': STOP -> STATUS = ', status
    call exit (status)
    stop '9'
  endif

  ! Beginning of Column writing.
  if(num_col>=10) then
    num_len=2
  else
    num_len=1
  endif

  ! It is the last column of the input phy file
  ! Let's add the qc columns generated by qc_t
  if(cnum_col(1:num_len)==cline(1:num_len) .and. its==1) then
    what_flag='T'
    write(OUT,'(a)') cline(1:lnblnk(cline))
    col_cnt=num_col+1
    if(col_cnt<10) then
      write(OUT,42) col_cnt     ! pressure increasing
    else
      write(OUT,45) col_cnt     ! pressure increasing
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then
      write(OUT,80) col_cnt     ! deepest pressure
    else
      write(OUT,81) col_cnt     ! deepest pressure
    endif
    if(pres_adj_exist) then
      col_cnt=col_cnt+1
      if(col_cnt<10) then
        write(OUT,280) col_cnt  ! deepest pressure adjusted
      else
        write(OUT,281) col_cnt  ! deepest pressure adjusted 
      endif
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then
      write(OUT,56) col_cnt, 'P' ! gross press
    else
      write(OUT,156) col_cnt,'P' ! gross press 
    endif
    if(pres_adj_exist) then
      col_cnt=col_cnt+1
      if(col_cnt<10) then
        write(OUT,282) col_cnt   ! gross pressure adjusted
      else
        write(OUT,283) col_cnt   ! gross pressure adjusted
      endif
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then
      write(OUT,56) col_cnt,'T'  ! gross temp
    else
      write(OUT,66) col_cnt,'T'  ! gross temp
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then
      write(OUT,57) col_cnt,what_flag  ! vertical
    else
      write(OUT,88) col_cnt,what_flag  ! vertical
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then 
      write(OUT,58) col_cnt,what_flag  ! clima
    else
      write(OUT,68) col_cnt,what_flag  ! clima
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then
      write(OUT,59) col_cnt,what_flag   ! analysis
    else
      write(OUT,69) col_cnt,what_flag   ! analysis
    endif
    col_cnt=col_cnt+1
    if(col_cnt<10) then
      write(OUT,70) col_cnt,what_flag   ! spike
    else
      write(OUT,71) col_cnt,what_flag   ! spike
    endif
    if(iregion>0) then 
      col_cnt=col_cnt+1
      write(OUT,83) col_cnt,what_flag   ! region
    endif
    if(type_ts32=='y') then                    
      col_cnt=col_cnt+1
      write(OUT,35) col_cnt        ! pres increasing
      col_cnt=col_cnt+1
      write(OUT,33) col_cnt        ! deepest pressure
      col_cnt=col_cnt+1
      write(OUT,36) col_cnt        ! vertical gradient
      col_cnt=col_cnt+1
      write(OUT,34) col_cnt, 'P'   ! gross_pres2_flag
      col_cnt=col_cnt+1
      write(OUT,34) col_cnt, 'T'   ! gross_temp2_flag
      col_cnt=col_cnt+1
      write(OUT,32) col_cnt        ! spike2_flag   
    endif
    cline_before=cline
    go to 1
  endif

  ! It is the last column of the input qc_t file
  ! Let's add the qc columns generated by qc_s
  if(cnum_col(1:num_len)==cline(1:num_len) .and. its==2) then
    what_flag='S'
    write(OUT,25) cline(1:lnblnk(cline))
    if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 1: ',cline
    col_cnt=num_col+1
    write(OUT,66) col_cnt,what_flag      ! gross
    col_cnt=col_cnt+1
    write(OUT,67) col_cnt,what_flag      ! density
    col_cnt=col_cnt+1
    write(OUT,68) col_cnt,what_flag      ! clima
    col_cnt=col_cnt+1
    write(OUT,69) col_cnt,what_flag      ! analys
    col_cnt=col_cnt+1
    write(OUT,71) col_cnt,what_flag      ! spike
    if(iregion>0) then
      col_cnt=col_cnt+1
      write(OUT,83) col_cnt,what_flag    ! region
    endif
    col_cnt=col_cnt+1
    write(OUT,88) col_cnt,what_flag    ! salinity gradient
    if(psal_adj_exist) then        ! psal adjusted
      col_cnt=col_cnt+1
      write(OUT,97) col_cnt,what_flag  ! psal gross
      col_cnt=col_cnt+1
      write(OUT,98) col_cnt,what_flag  ! psal clima
      col_cnt=col_cnt+1
      write(OUT,99) col_cnt,what_flag ! psal analysis
      if(iregion>=1) then
        col_cnt=col_cnt+1
        write(OUT,101) col_cnt,what_flag! psal region
      endif
    endif
    cline_before=cline
    go to 1
  endif
  ! End of Column writing.

  
  select case(its)
  case (0)
    what_flag='N' 
  case (1) 
    what_flag='T'
  case (2) 
    what_flag='S'
  end select

  if(cline(1:30)=='==============================') then
    if(nf==0) then
      if(no_pos==1 .and. sflag_write=='N') write(OUT,74) 
      if(land==1 .and. sflag_write=='N') write(OUT,75) 
      if(what_flag=='N' .and. sflag_write=='N') write(OUT,52) mx_s  ! Max speed check flag

      ! Grey List
      if(icho==TEMP) then
        write(OUT,93) 'P',max(1,pres_gl_flag)
        write(OUT,93) 'T',max(1,temp_gl_flag)
      else
        write(OUT,93) 'S',max(1,psal_gl_flag)
      endif

      lastnb = lnblnk(cline)
      write(OUT,25) cline(1:lastnb)
      if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 2: ',cline
      sflag_write='Y'
      go to 2
    endif  

    if(PRGMAIN=='qc_t') then
      if(no_pos==1)  write(OUT,74) 
      if(land==1)    write(OUT,75) 
      if(iregion==1) write(OUT,90) 
      if(iregion==2) write(OUT,91) 
    endif

    if(icho==TEMP) then
      if(nf<10) then 
        write(OUT,37) nf
      elseif(nf<100) then 
        write(OUT,38) nf
      elseif(nf<1000) then 
        write(OUT,39) nf
      elseif(nf<10000) then 
        write(OUT,40) nf
      endif
      write(OUT,51) iov_all      
      what_flag='T'
    else
      write(OUT,53) iov_all
      what_flag='S'
      its=2
    endif
    write(OUT,47) what_flag,mx_a         ! max analysis
    write(OUT,48) what_flag,mx_c         ! max climatology
    if(what_flag=='T') then
      write(OUT,49) 'P', max_gross_pres_flag  
      write(OUT,49) 'T', max_gross_temp_flag  
    else
      write(OUT,49) 'S', max_gross_sal_flag  
    endif
    if(type_ts32=='y' .and. its<=1)   then
      write(OUT,108) 'P', max_gross_pres2_flag 
      write(OUT,108) 'T', max_gross_temp2_flag 
    endif
    if(its.ne.2) then
      write(OUT,50) mx_v                 ! max vertical gradient
      if(type_ts32.eq.'y') write(OUT,109) mx_v2  
    else 
      write(OUT,54) max_density_grad_flag ! max density gradient
    endif

    write(OUT,64) what_flag,mx_p         ! max spike check
    if(type_ts32.eq.'y'.and.its.le.1) write(OUT,110) what_flag,mx_p2
    write(OUT,73) what_flag,mx_n         ! max constant value check
    write(OUT,86) what_flag,mx_j         ! max jump check
    if(iregion.ge.1) write(OUT,43) what_flag,mx_q ! max region check
    if(what_flag=='S') then
      write(OUT,89) max_vert_grad_sal_flag     ! max vertical gradient salinity check
    endif
    if(psal_adj_exist .and. its>=2) then
      write(OUT,102) what_flag,mx_ga         ! max psal_gros check
      write(OUT,103) what_flag,mx_ca         ! max psal_clim check
      write(OUT,104) what_flag,mx_aa         ! max psal_anlz check
      if(iregion.ge.1) write(OUT,105) what_flag,mx_ra ! max psal_regi
    endif
    if(mx_k.eq.4) then
      write(OUT,84) what_flag ! frozen test
    else
      write(OUT,87) what_flag ! frozen test
    endif

    ! Grey List
    if(icho==TEMP) then
      write(OUT,93) 'P',max(1,pres_gl_flag)
      write(OUT,93) 'T',max(1,temp_gl_flag)
    else
      write(OUT,93) 'S',max(1,psal_gl_flag)
    endif 

    ! Deepest prssure, speed, pressure increasing need to be written only once.
    if(what_flag=='N' .or. what_flag=='T') then
      write(OUT,85)  mx_deepest_pres_flag              ! max deep pres check
      if(pres_adj_exist) then
        write(OUT,185) mx_deepest_pres_adj_flag        ! max deep  pres adj check
        write(OUT,187) max_gross_pres_adj_flag         ! max gross pres adj check
      endif
      if(type_ts32.eq.'y')  write(OUT,107) mx_d2  
    endif
    if(what_flag.eq.'T') write(OUT,52) mx_s ! speed check

    if(what_flag.eq.'N'.or.what_flag.eq.'T') then
      write(OUT,76) mx_r         ! max pressure
      if(type_ts32.eq.'y') write(OUT,106) mx_r2      ! max pressure2
      write(OUT,62) sig_c        ! sigma used for clim
      write(OUT,63) sig_a        ! sigma used for analysis
      write(OUT,77) clim_ver     ! climatology version used
      if(land.ne.1.and.no_pos.ne.1) then
        if(year.eq.1900.and.month.eq.01.and.day.eq.01) then
        else
          write(OUT,72) year,month,day,atime ! sigmas and atime once only
          lastnb=lnblnk(ana_version)
          write(OUT,78) ana_version(1:lastnb)  ! Behringer's mpp_io_version
        endif ! if(year.eq.1900.
      endif   ! if(land.ne.1
    endif     ! if(what_flag.eq.'N'
    lastnb = lnblnk(cline)
    write(OUT,25) cline(1:lastnb)
    if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 3: ',cline
    go to 2
  else if(cline(1:22)=='START SURFACE PRESSURE' .or. cline(1:16)=='SURFACE PRESSURE') then
    write(surf_pres_aux,'(F10.2)') surface_pressure
    surf_pres_aux=ADJUSTL(surf_pres_aux)
    lastnb = lnblnk(surf_pres_aux)
    write(OUT,'(A40,A)') cline(1:40),surf_pres_aux(1:lastnb)
    go to 1
  else
    ! Rest: Mostly technical parameter
    lastnb = lnblnk(cline)
    if(cline(1:24)=='START OF TRANSMISSION QC') then
       if(icho==TEMP) then
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC TEMP SOT do not write unevaluated st_trans_time_qc from phy file to output file'
       else
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC PSAL SOT write st_trans_time_qc from qct file'
          write(OUT,25) cline(1:lastnb)
       endif
    else if(cline(1:23)=='START OF TRANSMISSION  ') then
       write(OUT,25) cline(1:lastnb)
       if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 9a: ',cline
       if(icho==TEMP) then
          write(OUT,111) st_trans_time_qc
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC TEMP SOT wr st_trans_time_qc at 9b: ',st_trans_time_qc
       else
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC PSAL SOT do not overwrite the st_trans_time_qc from qct'
       endif
    else if(cline(1:17)=='TIME ASCENT END QC') then
        if(icho==TEMP) then
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC TEMP ASE do not write unevaluated ascent_end_time_qc from phy file to output file'
       else
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC PSAL ASE write ascent_end_time_qc from qct file'
          write(OUT,25) cline(1:lastnb)
       endif
    else if(cline(1:21)=='TIME ASCENT END (YYYY') then
       write(OUT,25) cline(1:lastnb)
       if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 9b: ',cline
       if(icho==TEMP) then
          write(OUT,112) ascent_end_time_qc
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC TEMP ASE wr ascent_end_time_qc at 9d: ',ascent_end_time_qc
       else
          if(DEBUG_LEV==1) write(DBG,*)SUBD,'QC PSAL ASE do not write verwrite the ascent_end_time_qc from qct'
       endif
    else       
       write(OUT,25) cline(1:lastnb)
       if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 4: ',cline
    endif
    cline_before=cline
    if(cline(1:28)=='POSITION LINE PREVIOUS CYCLE') pos_prev_exist=.true.
    go to 1
  endif      ! if(cline(1:30).eq.'=====

2 read(INP,25,end=100) cline
  if(DEBUG_LEV==1) write(DBG,*)SUBD,'rd cline at 4: ',cline
  if(cline(1:10).eq.'==========') then
    lastnb = lnblnk(cline)
    write(OUT,25) cline(1:lastnb)
    if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 5: ',cline
    go to 3
  endif
  lastnb = lnblnk(cline)
  if(cline(1:22).eq.'    +99.999   +999.999' .or. cline(1:22).eq.'   LATITUDE  LONGITUDE') then
    !skip these lines
  else
    ipos_cnt=ipos_cnt+1
    if(bad_row(ipos_cnt).ge.4) then 
      cline(lastnb-1:lastnb-1)='-'
      if(cline(lastnb:lastnb).eq.'0') cline(lastnb-1:lastnb)='-4'
    endif

    ! (0.0,0.0) case, change position_qc from '0' to '-4'.
    if(cline(7:11).eq.'0.000'.and.cline(18:22).eq.'0.000') cline(lastnb-1:lastnb)='-4'
        
  endif
  write(OUT,25) cline(1:lastnb)        ! writing position
  if(DEBUG_LEV==1) write(DBG,*)SUBD,'wr cline at 6: ',cline
  go to 2                                ! go read another line
3 continue

  ! Read pres, sal, and tem information and add flags.
  !
55 read(INP,25,end=100)  cline
   if(DEBUG_LEV==1) write(DBG,*)SUBD,'rd cline at 5: ',cline
  ncnt=ncnt+1
  if(ncnt.gt.lmax) then
    status=9
    write(LOG, '(A, "-", A, ": ncnt gt lmax case: ncnt=", I5, " lmax=", I5)')   &
      PRGMAIN, prg_sub, ncnt, lmax
    write(LOG,'(A, "-", A, ": STOP -> STATUS= ",I5)') PRGMAIN, prg_sub, status
    write(ERR,'(A, "-", A, ": ncnt gt lmax case: ncnt=", I5, " lmax=", I5)')   &
      PRGMAIN, prg_sub, ncnt, lmax
    write(ERR,*) PRGMAIN, '-', prg_sub, ': STOP'
    call exit (status)
    stop '9'
  endif                            ! if(ncnt.gt.lmax) then
  lastnb = lnblnk(cline)

  if(its<=1) then
    write(cline(lastnb+2:lastnb+2),44) rflag(ncnt)! T run only press
    write(cline(lastnb+4:lastnb+4),44) deepest_pres_flag(ncnt)! T run only deep
    if(pres_adj_exist) then
      write(cline(lastnb+6:lastnb+6),44) deepest_pres_adj_flag(ncnt)! 
      lastnb=lastnb+2
    endif  
    write(cline(lastnb+6:lastnb+6),44) gross_pres_flag(ncnt) 
    if(pres_adj_exist) then
      write(cline(lastnb+8:lastnb+8),44) gross_pres_adj_flag(ncnt)  
      lastnb=lastnb+2
    endif  
    write(cline(lastnb+8:lastnb+8),44) gross_temp_flag(ncnt) 
    lastnb=lastnb+2
  else
    lastnb=lastnb-4                  ! do not write pres flag for S run
    write(cline(lastnb+6:lastnb+6),44) gross_sal_flag(ncnt) ! gross salinity
  endif
  if(its<=1) then
    write(cline(lastnb+8:lastnb+8),44) vflag(ncnt) ! vertical grad
  else
    write(cline(lastnb+8:lastnb+8),44) density_grad_flag(ncnt) 
  endif
  write(cline(lastnb+10:lastnb+10),44) cflag(ncnt) ! clim
  write(cline(lastnb+12:lastnb+12),44) aflag(ncnt) ! analysis
  write(cline(lastnb+14:lastnb+14),44) pflag(ncnt) ! spike
  iadd=16
  if(iregion>1) then
    write(cline(lastnb+iadd:lastnb+iadd),44) qflag(ncnt)   ! region
    iadd=18
  endif
  if(type_ts32=='y' .and. its<=1) then
    write(cline(lastnb+iadd:lastnb+iadd),44) ipres2_flag(ncnt)          ! ipres2
    write(cline(lastnb+iadd+2:lastnb+iadd+2),44) ideep2_flag(ncnt)      ! ideep2
    write(cline(lastnb+iadd+4:lastnb+iadd+4),44) itemp2_flag(ncnt)      ! itemp2
    write(cline(lastnb+iadd+6:lastnb+iadd+6),44) gross_pres2_flag(ncnt) ! gross pres2
    write(cline(lastnb+iadd+8:lastnb+iadd+8),44) gross_temp2_flag(ncnt) ! gross temp2
    write(cline(lastnb+iadd+10:lastnb+iadd+10),44) ispik2_flag(ncnt)    ! ispik2
    iadd=iadd+4
  endif
  if(its>=2) then
    write(cline(lastnb+iadd:lastnb+iadd),44) vert_grad_sal_flag(ncnt)
  endif
  if(psal_adj_exist .and. its>=2) then
    iadd=iadd+2
    write(cline(lastnb+iadd:lastnb+iadd),44) gaflag(ncnt) ! psal gross
    write(cline(lastnb+iadd+2:lastnb+iadd+2),44) caflag(ncnt) ! psal clim
    write(cline(lastnb+iadd+4:lastnb+iadd+4),44) aaflag(ncnt) ! psal anal
    if(iregion.ge.1) write(cline(lastnb+iadd+6:lastnb+iadd+6),44) raflag(ncnt)
  endif
  lastnb = lnblnk(cline)
  write(OUT,25) cline(1:lastnb)

  go to 55
100  continue
  if(ncnt.ne.nf) then
    status=9
    write(LOG, '(A, "-", A, ": Number of dep/temp values differ - ncnt=", I5, " nf=", I5)') &
      PRGMAIN, prg_sub, ncnt, nf 
    write(LOG,'(A, "-", A, ": STOP -> STATUS= ",I5)') PRGMAIN, prg_sub, status
    write(ERR,*) PRGMAIN, '-', prg_sub, 'Number of dep/temp values differ - ',             &
      'ncnt=',ncnt,' nf=',nf  
    write(ERR,*) PRGMAIN, '-', prg_sub, ': STOP'
    call exit (status)
    stop '9'
  endif

  write(TRC,46)  pathname_outfile(1:last)
  write(TRC,*)' '
  close (INP)  
  close (OUT)

25   format(a)
32   format(i2,'. COLUMN',30x,'SPIKE CHECK_T2 FLAG')
33   format(i2,'. COLUMN',30x,'DEEPEST PRESSURE2 CHECK FLAG')
34   format(i2,'. COLUMN',30x,'GROSS CHECK_',a1,'2 FLAG')
35   format(i2,'. COLUMN',30x,'PRESSURE INCREASING CHECK_P2 FLAG')
36   format(i2,'. COLUMN',30x,'VERTICAL GRADIENT CHECK_T2 FLAG')
37   format('PROFILE LEVELS',26x,i1)
38   format('PROFILE LEVELS',26x,i2)
39   format('PROFILE LEVELS',26x,i3)
40   format('PROFILE LEVELS',26x,i4)
41   format(i2)
42   format(i1,'. COLUMN',31x,'PRESSURE INCREASING CHECK_P FLAG')
43   format('MAXIMUM REGION CHECK_',a1,' FLAG',13x,i1,39x)
44   format(i1)
46   format(1x,'Flag file created:',a)
47   format('MAXIMUM ANALYSIS CHECK_',a1,' FLAG',11x,i1,39x)
48   format('MAXIMUM CLIMATOLOGY_',a1,' FLAG',14x,i1,39x)
49   format('MAXIMUM GROSS CHECK_',a1,' FLAG ',13x,i1,39x)
50   format('MAXIMUM VERTICAL GRADIENT_T FLAG',8x,i1,39x)
51   format('OVERALL FLAG FOR T',22x,i1,39x)
52   format('MAXIMUM SPEED CHECK FLAG',16x,i1,39x)
53   format('OVERALL FLAG FOR S',22x,i1,39x)
54   format('MAXIMUM DENSITY GRADIENT_S FLAG',9x,i1,39x)
45   format(i2,'. COLUMN',30x,'PRESSURE INCREASING CHECK_P FLAG')
56   format(i1,'. COLUMN',31x,'GROSS CHECK_',a1,' FLAG')
 156  format(i2,'. COLUMN',30x,'GROSS CHECK_',a1,' FLAG')
 57   format(i1,'. COLUMN',31x,'VERTICAL GRADIENT CHECK_',a1,' FLAG')
 58   format(i1,'. COLUMN',31x,'CLIMATOLOGY CHECK_',a1,' FLAG')
 158  format(i2,'. COLUMN',30x,'CLIMATOLOGY CHECK_',a1,' FLAG')
 59   format(i1,'. COLUMN',31x,'ANALYSIS CHECK_',a1,' FLAG')
 60   format('NUMBER OF COLUMNS',23x,i1)
 61   format('NUMBER OF COLUMNS',23x,i2)
 62   format('SIGMA USED FOR CLIMATOLOGY',13x,f6.2)
 63   format('SIGMA USED FOR ANALYSIS',16x,f6.2)
 64   format('MAXIMUM SPIKE CHECK_',a1,' FLAG',14x,i1,39x)
 66   format(i2,'. COLUMN',30x,'GROSS CHECK_',a1,' FLAG')
 67   format(i2,'. COLUMN',30x,'DENSITY GRADIENT CHECK_',a1,' FLAG')
 68   format(i2,'. COLUMN',30x,'CLIMATOLOGY CHECK_',a1,' FLAG')
 69   format(i2,'. COLUMN',30x,'ANALYSIS CHECK_',a1,' FLAG')
 70   format(i1,'. COLUMN',31x,'SPIKE CHECK_',a1,' FLAG')
 71   format(i2,'. COLUMN',30x,'SPIKE CHECK_',a1,' FLAG')
 72   format('ANALYSIS TIME USED',22x,i4,'-',i2.2,'-',i2.2,1x,'(',f15.9,')')
 73   format('MAXIMUM CONSTANT VALUE CHECK_',a1,' FLAG',5x,i1,39x)
 74   format('NO POSITION INDICATOR',19x,'1')
 75   format('POSITION ON LAND INDICATOR',14x,'1')
 76   format('MAXIMUM PRESSURE INCREASING FLAG',8x,i1,39x)
 77   format('CLIMATOLOGY VERSION USED',16x,a16)
 78   format('ANALYSIS VERSION USED',19x,a)
 79   format('JULD_QC',33x,i1)
 80   format(i1,'. COLUMN',31x,'DEEPEST PRESSURE CHECK FLAG')
 81   format(i2,'. COLUMN',30x,'DEEPEST PRESSURE CHECK FLAG')
280   format(i1,'. COLUMN',31x,'DEEPEST PRESSURE ADJ CHECK FLAG')
281   format(i2,'. COLUMN',30x,'DEEPEST PRESSURE ADJ CHECK FLAG')
282   format(i1,'. COLUMN',31x,'GROSS CHECK_P ADJ FLAG')
283   format(i2,'. COLUMN',30x,'GROSS CHECK_P ADJ FLAG')
 82   format(i1,'. COLUMN',30x,'REGION CHECK_',a1,' FLAG')
 83   format(i2,'. COLUMN',30x,'REGION CHECK_',a1,' FLAG')
 84   format('FROZEN PROFILE INDICATOR FOR ',a1,10x,'1')
 85   format('MAXIMUM DEEPEST PRESSURE CHECK FLAG',5x,i1,39x)
 185  format('MAXIMUM DEEPEST PRESSURE ADJ CHECK FLAG',1x,i1,39x)
 187  format('MAXIMUM GROSS CHECK_P ADJ FLAG',10x,i1,39x)

 86   format('MAXIMUM JUMP CHECK_',a1,' FLAG',15x,i1,39x)
 87   format('FROZEN PROFILE INDICATOR FOR ',a1,10x,'0')
 88   format(i2,'. COLUMN',30x,'VERTICAL GRADIENT CHECK_',a1,' FLAG')
 89   format('MAXIMUM SALINITY GRADIENT FLAG',10x,i1,39x)
 90   format('REGION USED ',28x,'RED SEA')
 91   format('REGION USED ',28x,'MEDITERRAREAN SEA')
 93   format('GREY LIST FLAG FOR ',a1,20x,i1)
 97   format(i2,'. COLUMN',30x,'PSAL_ADJUST GROSS CHECK_',a1,' FLAG')
 98   format(i2,'. COLUMN',30x,'PSAL_ADJUST CLIMATOLOGY CHECK_',a1,' FLAG')
 99   format(i2,'. COLUMN',30x,'PSAL_ADJUST ANALYSIS CHECK_',a1,' FLAG')
 101  format(i2,'. COLUMN',30x,'PSAL_ADJUST REGION CHECK_',a1,' FLAG')
 102  format('MAXIMUM PSAL_ADJUST GROS CHECK_',a1,' FLAG',3x,i1,39x)
 103  format('MAXIMUM PSAL_ADJUST CLIM CHECK_',a1,' FLAG',3x,i1,39x) 
 104  format('MAXIMUM PSAL_ADJUST ANAL CHECK_',a1,' FLAG',3x,i1,39x)
 105  format('MAXIMUM PSAL_ADJUST REGI CHECK_',a1,' FLAG',3x,i1,39x)
 106  format('MAXIMUM PRESSURE2 INCREASING FLAG',7x,i1,39x)
 107  format('MAXIMUM DEEPEST PRESSURE2 CHECK FLAG',4x,i1,39x)
 108  format('MAXIMUM GROSS CHECK_',a1,'2 FLAG ',12x,i1,39x)
 109  format('MAXIMUM VERTICAL GRADIENT_T2 FLAG',7x,i1,39x)
 110  format('MAXIMUM SPIKE CHECK_',a1,'2 FLAG',13x,i1,39x)
 111  format('START OF TRANSMISSION QC',16x,a1,31xx)
 112  format('TIME ASCENT END QC',22x,a1,31x)

  return
  end 
