c    Revision Date: 02-Dec-04 (YHD - created)
c
c    frozen_s.f
c    ==========
c    This test detects a float that reproduces the same profile (with
c    small deviations) over and over again.
c    The profile requires that max(delta S) < 0.3
c                              min(delta S) < 0.001
c                              mean(delta S) < 0.004
c    otherwise, it is defined as failed the frozen test.
c
c    inputs:
c    s1: salinity array of present profile
c    p1: pressure array of present profile
c    n1: number of data points of present profile
c    s0: salinity array of previous profile
c    p0: pressure array of previous profile
c    n0: number of data points of previous profile
c    output:
c    ifrzn_flag=1  good     frozen test passed.
c              =4  bad      frozen test failed.
c
c    otherwise, it is defined as failed the frozen test.
c    author: CS; modified by YHD
c
      subroutine frozen_s(s1,p1,n1,s0,p0,n0,ifrzn_flag)

      implicit none
      integer, parameter :: lmax = 2200;
      integer n1,n0,ifrzn_flag,i,m,icnt,idbug
      real    s1(*),s0(*),p1(*),p0(*),sal1(lmax),sal0(lmax),pre(lmax),
     *        ds(lmax),sum,mx_s,mn_s,mean_s,max_ds,min_ds,mean_ds,
     *        pre_before,pre_after,fill_val
      data    mx_s,mn_s,mean_s/0.3,0.001,0.004/
      data    fill_val /-999.0/
      data    idbug /0/

      if (idbug.gt.0) write(66,*)'<<< DEBUG from FROZEN_S CHECK >>>'

      m=0                       ! counter for 50 dbar slabs
      m=int(p1(n1)/50) 
      if (mod(p1(n1),50).ne.0) m=m+1
      do i=1,m
        pre(i)=50.0+(i-1)*50.0
      end do
c
c--- Present profile.
c      
      m=1
      sum=0.0
      icnt=0
      pre_before=0.0
      pre_after=pre(1)

      do i=1,n1
        if (s1(i).eq.fill_val.or.p1(i).eq.fill_val) go to 10
        if (p1(i).ge.pre_before.and.p1(i).lt.pre_after) then
          icnt=icnt+1
          sum=sum+s1(i)
c          write(6,*) 'present:',icnt,s1(i),p1(i),sum
        else
          if (icnt.eq.0) then
            sal1(m)=0.0
          else
            sal1(m)=sum/icnt
          end if
c          write(6,*)'--------------------------------'
          if (idbug.gt.0) write(66,*)'present:',pre_before,pre_after,
     *    icnt,m,sal1(m)
          sum=0.0
          icnt=0
          m=m+1
          pre_before=pre_after
          pre_after=pre(m)
        end if
  10   end do
c
c--- Previous profile.
c
      m=1
      sum=0.0
      icnt=0
      pre_before=0.0
      pre_after=pre(1)

      do i=1,n0
        if (s0(i).eq.fill_val.or.p0(i).eq.fill_val) go to 20
        if (p0(i).ge.pre_before.and.p0(i).lt.pre_after) then
          icnt=icnt+1
          sum=sum+s0(i)
c          write(6,*) 'prev:',icnt,s0(i),p0(i),sum
        else
          if (icnt.eq.0) then
            sal0(m)=0.0
          else
            sal0(m)=sum/icnt
          end if
c          write(6,*)'--------------------------------'
          if (idbug.gt.0) write(66,*)'previous:',pre_before,pre_after,
     *    icnt,m,sal0(m)
          sum=0.0
          icnt=0
          m=m+1
          pre_before=pre_after
          pre_after=pre(m)
        end if
 20   end do
c
c--- Find the absolute difference between the present and previous profiles.
c
      do i=1,n1
        ds(i)=abs(sal1(i)-sal0(i))
      end do
c
c--- Find Max(ds), Min(ds) & mean(ds).
c
      sum=0.0
      max_ds=0.0
      min_ds=1000.0
      do i=1,n1
        max_ds=max(ds(i),max_ds)
        min_ds=min(ds(i),min_ds)
        sum=sum+ds(i)
      end do
      mean_ds=sum/n1
c
c--- Determine if a profile is too much like the previous profile.
c
      if (max_ds.lt.mx_s.and.min_ds.lt.mn_s.and.mean_ds.lt.mean_s)
     *then
        ifrzn_flag=4 
        if (idbug.gt.0) then
          write(66,*)'YES, this profile is FROZEN.'
          write(66,*)'max_ds=',max_ds,' less than ',mx_s
          write(66,*)'min_ds=',min_ds,' less than ',mn_s
          write(66,*)'mean_ds=',mean_ds,' less than ',mean_s
        end if
      else
        ifrzn_flag=1 
        if (idbug.gt.0) then
          write(66,*)'NO, this profile is NOT FROZEN.'
          write(66,*)'max_ds=',max_ds,' mx_s=',mx_s
          write(66,*)'min_ds=',min_ds,' mn_s=',mn_s
          write(66,*)'mean_ds=',mean_ds,' mean_s=',mean_s
        end if
      end if    
      return
      end
