c    Revision Date: 02-Dec-04 (YHD - created)
c
c    frozen_t.f
c    ==========
c    This test detects a float that reproduces the same profile (with
c    small deviations) over and over again.
c    The profile requires that max(delta T) < 0.3
c                              min(delta T) < 0.001
c                              mean(delta T) < 0.02
c    otherwise, it is defined as failed the frozen test.
c
c    inputs:
c    t1: teperature array of present profile
c    p1: pressure array of present profile
c    n1: number of data points of present profile
c    t0: temperature array of previous profile
c    p0: pressure array of previous profile
c    n0: number of data points of previous profile
c    output:
c    ifrzn_flag=1  good     frozen test passed.
c              =4  bad      frozen test failed.
c
c    otherwise, it is defined as failed the frozen test.
c    author: CS; modified by YHD
c
      subroutine frozen_t(t1,p1,n1,t0,p0,n0,ifrzn_flag)

      implicit none
      integer, parameter :: lmax = 2200;
      integer n1,n0,ifrzn_flag,i,m,icnt,idbug
      real    t1(*),t0(*),p1(*),p0(*),tem1(lmax),tem0(lmax),pre(lmax),
     *        dt(lmax),sum,mx_t,mn_t,mean_t,max_dt,min_dt,mean_dt,
     *        pre_before,pre_after,fill_val
      data    mx_t,mn_t,mean_t/0.3,0.001,0.02/
      data    fill_val /-999.0/
      data    idbug /0/

      if (idbug.gt.0) write(66,*)'<<< DEBUG from FROZEN_T CHECK >>>'

      m=0                       ! counter for 50 dbar slabs
      m=int(p1(n1)/50) 
      if (mod(p1(n1),50).ne.0) m=m+1
      do i=1,m
        pre(i)=50.0+(i-1)*50.0
      end do
c
c--- Present profile.
c      
      m=1
      sum=0.0
      icnt=0
      pre_before=0.0
      pre_after=pre(1)

      do i=1,n1
        if (t1(i).eq.fill_val.or.p1(i).eq.fill_val) go to 10
        if (p1(i).ge.pre_before.and.p1(i).lt.pre_after) then
          icnt=icnt+1
          sum=sum+t1(i)
c          write(6,*) 'present:',icnt,t1(i),p1(i),sum
        else
          if (icnt.eq.0) then
            tem1(m)=0.0
          else
            tem1(m)=sum/icnt
          end if
c          write(6,*)'--------------------------------'
          if (idbug.gt.0) write(66,*)'present:',pre_before,pre_after,
     *    icnt,m,tem1(m)
          sum=0.0
          icnt=0
          m=m+1
          pre_before=pre_after
          pre_after=pre(m)
        end if
  10   end do
c
c--- Previous profile.
c
      m=1
      sum=0.0
      icnt=0
      pre_before=0.0
      pre_after=pre(1)

      do i=1,n0
        if (t0(i).eq.fill_val.or.p0(i).eq.fill_val) go to 20
        if (p0(i).ge.pre_before.and.p0(i).lt.pre_after) then
          icnt=icnt+1
          sum=sum+t0(i)
c          write(6,*) 'prev:',icnt,t0(i),p0(i),sum
        else
          if (icnt.eq.0) then
            tem0(m)=0.0
          else
            tem0(m)=sum/icnt
          end if
c          write(6,*)'--------------------------------'
          if (idbug.gt.0) write(66,*)'previous:',pre_before,pre_after,
     *    icnt,m,tem0(m)
          sum=0.0
          icnt=0
          m=m+1
          pre_before=pre_after
          pre_after=pre(m)
        end if
 20   end do
c
c--- Find the absolute difference between the present and previous profiles.
c
      do i=1,n1
        dt(i)=abs(tem1(i)-tem0(i))
      end do
c
c--- Find Max(dt), Min(dt) & mean(dt).
c
      sum=0.0
      max_dt=0.0
      min_dt=1000.0
      do i=1,n1
        max_dt=max(dt(i),max_dt)
        min_dt=min(dt(i),min_dt)
        sum=sum+dt(i)
      end do
      mean_dt=sum/n1
c
c--- Determine if a profile is too much like the previous profile.
c
      if (max_dt.lt.mx_t.and.min_dt.lt.mn_t.and.mean_dt.lt.mean_t)
     *then
        ifrzn_flag=4 
        if (idbug.gt.0) then
          write(66,*)'YES, this profile is FROZEN.'
          write(66,*)'max_dt=',max_dt,' less than ',mx_t
          write(66,*)'min_dt=',min_dt,' less than ',mn_t
          write(66,*)'mean_dt=',mean_dt,' less than ',mean_t
        end if
      else
        ifrzn_flag=1
        if (idbug.gt.0) then
          write(66,*)'NO, this profile is NOT FROZEN.'
          write(66,*)'max_dt=',max_dt,' mx_t=',mx_t
          write(66,*)'min_dt=',min_dt,' mn_t=',mn_t
          write(66,*)'mean_dt=',mean_dt,' mean_t=',mean_t
        end if
      end if    
      return
      end
