!
! Revision date: 2013-04-02 VMH   Minor changes
!                26-Oct-10  ER: Initial release.
!
! gross_check_pressure
! ====================
! Check that the pressure value for each level of the profile
! stay between certain limits
!
! Inputs:
!   pres - pressure array of the profile
!   nbr_levels - how many levels there are in the profile
!
! Output:
!   flag - array holding how each pressure value 'fared' in the test
!
subroutine gross_check_pressure( pres, nbr_levels, flag )

  implicit none
  real,    intent(in) :: pres(*)  
  integer, intent(in) :: nbr_levels
  integer, intent(inout) :: flag(*)
  real, parameter :: MIN_ACCEPTABLE_PRESSURE = -5.0 ! Pressure cannot be < -5 dbar
  integer, parameter :: GOOD = 1
  integer, parameter :: BAD = 4
  integer :: i

  do i=1, nbr_levels
    if(pres(i) < MIN_ACCEPTABLE_PRESSURE) then
      flag(i)=BAD
    else
      flag(i)=GOOD
    endif
  enddo

  return
end subroutine gross_check_pressure  
