!
! Revision date: 2013-04-02 VMH use communication for LUNS
!                26-Oct-10  ER: Based on previous program
!                               by YHD
!
! gross_check_salinity
! ====================
! Tests that the salinity value for each level of the profile
! stays between given limits
!
! inputs:
!   adjusted - is the salinity adjusted or not?
!   sal - salinity array or adjusted salinity array
!   nbr_levels - number of levels in profile
!   cksum_flag = checksum flag array
!
! return:
!   gross_sal_flag - array holding results of test for each level
!                    (0=can't test, 1=OK, 4=BAD)              
!   overall_gros_flag_int - worse individual test is the overall test
!   overall_gros_flag_chr - pass 'P', fail 'F', or can't test 'N'
!
subroutine gross_check_salinity(                            &
  adjusted, sal, nbr_levels, cksum_flag, gross_sal_flag,    &
  overall_gros_flag_int, overall_gros_flag_chr)

  use communication                       ! module

  implicit none

  logical, intent(in)  :: adjusted 
  real,    intent(in)  :: sal(*)  
  integer, intent(in)  :: nbr_levels
  integer, intent(in)  :: cksum_flag(*)
  integer, intent(out) :: gross_sal_flag(*)
  integer, intent(out) :: overall_gros_flag_int
  character(len=1), intent(out) :: overall_gros_flag_chr
  
  real, parameter :: MIN_ACCEPTABLE_SALINITY =  2.0 
  real, parameter :: MAX_ACCEPTABLE_SALINITY = 41.0 
  integer, parameter :: NOT_APPLIED = 0
  integer, parameter :: GOOD = 1
  integer, parameter :: BAD = 4
  real, parameter :: FILLVALUE = -999.0 

  integer :: gmin = 1000
  integer :: gmax = 0
  integer i

  if(nbr_levels==0) then
    if(adjusted) then 
      write(LOG,*) 'GROSS CHECK TEST S_ADJ can not be applied.'
      write(TRC,*)'GROSS CHECK TEST S_ADJ can not be applied.'
    else
      write(LOG,*) 'GROSS CHECK TEST S can not be applied.'
      write(TRC,*)'GROSS CHECK TEST S can not be applied.'
    endif
    overall_gros_flag_chr='N'
    overall_gros_flag_int=NOT_APPLIED
    return
  endif

  do i = 1, nbr_levels
    gross_sal_flag(i)=1
    if(cksum_flag(i)==0 .and. sal(i)==0.) then 
      gross_sal_flag(i)=NOT_APPLIED
    else if(sal(i)==FILLVALUE) then
      gross_sal_flag(i)=NOT_APPLIED
    else if(sal(i)<MIN_ACCEPTABLE_SALINITY .or. sal(i)>MAX_ACCEPTABLE_SALINITY) then 
      gross_sal_flag(i)=BAD
    else 
      gross_sal_flag(i)=GOOD
    endif
    if(gross_sal_flag(i)>gmax) gmax=gross_sal_flag(i)
  enddo

  if(gmax==NOT_APPLIED) then
    if(adjusted) then 
      write(LOG,*) 'GROS CHECK TEST S_ADJ can not be applied.'
      write(TRC,*)'GROS CHECK TEST S_ADJ can not be applied.'
    else
      write(LOG,*) 'GROS CHECK TEST S can not be applied.'
      write(TRC,*)'GROS CHECK TEST S can not be applied.'
    endif
    overall_gros_flag_chr='N'
    overall_gros_flag_int=NOT_APPLIED
  else if(gmax==BAD) then
    if(adjusted) then 
      write(LOG,*) 'GROS CHECK TEST S_ADJ failed.'
      write(TRC,*)'GROS CHECK TEST S_ADJ failed.'
    else
      write(LOG,*) 'GROS CHECK TEST S failed.'
      write(TRC,*)'GROS CHECK TEST S failed.'
    endif
    overall_gros_flag_chr='F'
    overall_gros_flag_int=BAD
  else
    if(adjusted) then 
      write(LOG,*) 'GROS CHECK TEST S_ADJ passed.'
      write(TRC,*)'GROS CHECK TEST S_ADJ passed.'
    else
      write(LOG,*) 'GROS CHECK TEST S passed.'
      write(TRC,*)'GROS CHECK TEST S passed.'
    endif
    overall_gros_flag_chr='P'
    overall_gros_flag_int=GOOD
  endif

  return
end
