!
! Revision date: 2013-12-26 CS   for some reason the test was using -2.0 instead
!                                of -2.5 as minimum -> corrected ...
!                2013-04-02 VMH   Minor changes
!                31-Aug-11  ER: Checking for fillvalue
!                26-Oct-10  ER: Based on previous program
!                               by YHD
!
! gross_check_temperature
! =======================
! Check that the temperature value for each level of the profile
! stays between given limits
!
! Inputs:
!   temp - temperature array of the profile
!   nbr_levels - how many levels there are in the profile
!
! Output:
!   flag - array holding how each temperature value 'fared' in the test
!
subroutine gross_check_temperature( temp, nbr_levels, flag )

  implicit none
  real,    intent(in) :: temp(*)  
  integer, intent(in) :: nbr_levels
  integer, intent(inout) :: flag(*)
  real, parameter :: FILLVALUE = -9.0 
  real, parameter :: MIN_ACCEPTABLE_TEMPERATURE = -2.5 ! Temperature cannot be < -2.5 C
  real, parameter :: MAX_ACCEPTABLE_TEMPERATURE = 40.0 ! Temperature cannot be > 40 C
  integer, parameter :: NOT_APPLIED = 0
  integer, parameter :: GOOD = 1
  integer, parameter :: BAD = 4
  integer :: i

  do i=1, nbr_levels
    if(temp(i)==FILLVALUE) then
      flag(i)=NOT_APPLIED
    else if(temp(i) < MIN_ACCEPTABLE_TEMPERATURE .or.    &
           temp(i) > MAX_ACCEPTABLE_TEMPERATURE ) then
      flag(i)=BAD
    else
      flag(i)=GOOD
    endif
  enddo

  return
end subroutine gross_check_temperature  
