c Revision Date:20-Aug-09 (YHD - It needs to have abs(bottom-p(i)) to check
c                                bottom 100 dbars) 
c               25-Nov-04 (YHD)
c
c* jump_check_t.f
c  ===============
c  yhd, 25-Nov-2004
c  This test checks the sudden sensor drift.
c  It calculates the bottom 100 dbars of the present profile and the previous
c  profile.  Only good QC measurements are allowed.
c  If the difference between the 2 average values is more than 1 degree
c  then all measurements for this parameter are flagged as bad(flag3).
c
c  inputs:
c    t1 - temp. array for present profile
c    p1 - pres. array for present profile
c    igros_flag:
c    iverg_flag:
c    ispik_flag:
c    ipres_flag: flags for present profile
c    n1 - number of pres/temp pairs for present profile
c    t0 - temp. array for previous profile
c    p0 - pres. array for previous profile
c    igros_flag_qc:
c    iverg_flag_qc:
c    ispik_flag_qc:
c    ipres_flag_qc: flags for previous profile
c    n0 - number of pres/temp pairs for previous profile
c
c  return:
c    iflag - flag array containing - 1 good point
c                                  - 3 bad  point
c
      subroutine jump_check_t(t1,p1,n1,t0,p0,n0,
     *     igrot_flag,iverg_flag,ispik_flag,
     *     ipres_flag,iover_t_qc,igrot_flag_qc,iverg_flag_qc,
     *     ispik_flag_qc,ipres_flag_qc,iflag)

      implicit none
      integer i,n0,n1,iflag(*),idbug,icnt,jcnt,max_f
      integer igrot_flag(*),iverg_flag(*),ispik_flag(*),ipres_flag(*)
      integer igrot_flag_qc(*),iverg_flag_qc(*),ispik_flag_qc(*),
     *        ipres_flag_qc(*),iover_t_qc
      real t0(*),t1(*),p0(*),p1(*),t_allow,fv,sum0,sum1,avg0,avg1,
     *     pbottom
      data t_allow/1.0/       ! 
      data idbug /0/          ! make 1 to debug
      data fv /-999.0/
      data icnt,jcnt/0,0/
      data sum0,sum1,avg0,avg1/4*0.0/

      if (idbug.gt.0) write(66,*)'<<< DEBUG from JUMP T CHECK >>>'
c
c--- Initialize iflag to 1 (good).
c
      do i=1,n1
        iflag(i)=1
      end do
c
c---  It has to be deeper than 500 dbars to apply this test.
c
      if (p1(n1) .lt. 500.0) then
         do i=1,n1
           iflag(i)=0
         end do
         if (idbug.gt.0) then
           write(66,*)'Jump check can not be applied.'
           write(66,*)'Profile is NOT deep enough.',' Pres=',p1(n1) 
         end if
         return
      end if

      pbottom=int(p1(n1))                   ! 

      do i = n1,1,-1
        if (abs(pbottom-int(p1(i))).le.100.0) then    ! 20-Aug-09 bug fixed
          icnt=icnt+1
          sum1=sum1+t1(i)
          if (idbug.gt.0) write(66,*)i,icnt,' sum=',sum1,' t=',t1(i),
     *' p=',p1(i)
        end if
      end do
      if (icnt.eq.0) then
        write(66,*)'Can not apply the drift test on this profile.'
        write(66,*)'Because there is no data point.'
        write(66,*)'icnt, sum1:',icnt,sum1
      else
        avg1=sum1/icnt
      end if
      if (idbug.gt.0) 
     *  write(66,*) 'average tempererature for this profile=',avg1

      if (n0.eq.0) then
        if (idbug.gt.0) write(66,*)'No Previous profile exists.'
        do i=1,n1
          iflag(i)=0
        end do
        return
      end if
c
c--- For previous profile:
c
      if (iover_t_qc.eq.4) then    ! if previous profile is bad, can't apply
        do i=1,n1
          iflag(i)=0               
        end do
        if (idbug.gt.0) then
          write(66,*)'Can not apply the drift test on previous profile.'
          write(66,*)'Because the previous profile is bad.'
        end if
        return
      end if 

      max_f=0
      do i = n0,1,-1
c        if (idbug.gt.0)
c     *  write(66,*)igrot_flag_qc(i),iverg_flag_qc(i),ispik_flag_qc(i),
c     * ipres_flag_qc(i)
        max_f=max(igrot_flag_qc(i),iverg_flag_qc(i),ispik_flag_qc(i),
     *            ipres_flag_qc(i))
        if (int(p0(i)).le.pbottom.and.pbottom-int(p0(i)).le.100.0.and.
     *      max_f.eq.1) then
          jcnt=jcnt+1
          sum0=sum0+t0(i)
          if (idbug.gt.0) write(66,*)i,jcnt,' sum=',sum0,' t=',t0(i)
        end if
      end do
      if (jcnt.eq.0) then
        max_f=1
        if (idbug.gt.0) then
          write(66,*)'Can not apply the drift test on previous profile.'
          write(66,*)'jcnt, sum0:',jcnt,sum0
        end if
c        do i=n1-icnt+1,n1
        do i=1,n1
          iflag(i)=0               
        end do
        return
      else
        avg0=sum0/jcnt
      end if
      if (idbug.gt.0) 
     *   write(66,*) 'average tempererature for prev profile=',avg0
c
c---The difference between the p1 and p0 average value is more than 1 degree
c--- then mark the all measurements are flagged as bad data (flag 3).
c
      if (abs(avg1-avg0).gt.t_allow) then
        do i=n1-icnt+1,n1
          iflag(i)=4               ! ask Claudia about this?
        end do
      end if
c
c--- When temperature is fill value, iflag is 0
c  
      do i=1,n1
         if (t1(i).eq.fv) iflag(i)=0
      end do

      return
      end
