c    
c  Revision Date: 29-Mar-06 (YHD - Positions & 2 lines of data with one pres=
c                                  0.0 case)
c                 28-Mar-06 (YHD - No position & 1 line of data with many 0.0
c                                  lines of data case (i.e 0533_41353_093.phy)
c                 09-Aug-05 (YHD - A bug found; do not reinitialize 
c                                  iflag(i)=1)
c                 05-Apr-05 (YHD - A bug found in line 148, 
c                                  pres_work(i).eq.pres_work(i-1)) ...
c                                  should be .le. not .eq.) 
c                 27-Apr-04 (YHD - pres(n+1) has to be > pres(n))
c                 14-Oct-03 (YHD - 'pres_tmp1' dimension check)
c                 07-Oct-03 (YHD - Indices check- make sure not go over the
c                                  defined limits)
c                 03-Oct-03 (YHD - All dimensions check)
c                 05-Jun-03 (YHD - Pres is fillvalue --> iflag=0)
c                 03-Jun-03 (YHD - WHOI float with t, s fill value case)
c                 02-Apr-03 (YHD - WHOI float 0242_001 0245_007 cases)
c                 10-Feb-03 (YHD - fill value in data line case)
c                 25-Sep-02 (YHD - median filter, spike) 
c
c     pres_check.f
c     ============
c     This program checks pressure values and finds out any points
c     that is not in increasing order using the median filter, the spike
c     "down", and spike "up".
c
c     inputs:
c       pres  - pressure array
c       tem   - temperature array
c       sal   - salinity array
c       cksm_flag - check sum flags
c       nlev  - number of pressure points
c      output:
c       iflag  - 1     good data point
c                4     bad data point
c
      subroutine pres_check (tem,sal,pres,nlev,cksm_flag,iflag)

      implicit none
      integer i,j,n,k,nlev,iflag(*),mwin,mcnt,idbug,status,
     *        cksm_flag(*),nw1,nw2,nw3,icnt,np,nx,mx,lx,kx,ll
      integer, parameter :: lmax = 2200;
      real    tem(*),sal(*),pres(*),xmed,xmed_save(lmax),
     *        x(11),pres_work(lmax),pres_bad(lmax),pres_tmp(lmax),
     *        pres_tmp1(lmax),pres_tmp2(lmax),
     *        d_bef,d_aft,tol,tol1,tol2,tol3,pmin,p1,dp,fv
      character prg_sub*10
      data    tol,tol1,tol2,tol3/20.,102.,120.0,200.0/
      data    dp /5.0/
      data    prg_sub /'pres_check'/
      data    idbug /0/                ! make it 1 to debug
      data    nw1,nw2,nw3 /3,7,11/     ! nw points per window for median
      data    fv /-999./
c
c*    Use a window of 3 data points and find a median.
c
      nx=0                             ! a counter for diff(data-median)>120
      mx=0                             ! a counter for good presure point
      lx=0
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from PRES_CHECK >>>>>'
      if (idbug.gt.0) write(66,*)'pres_check:nlev =',nlev,
     *' Number of pts per window used=',nw1,nw2,nw3

      np=0
c
c --- Eliminate zero or fill value data lines.
c
      do i=1,nlev
        if (pres(i).eq.0.0.and.cksm_flag(i).eq.0.and.tem(i).eq.0.0.and.
     *      sal(i).eq.0.0) then
          iflag(i)=0
c        else if (pres(i).eq.fv.and.tem(i).eq.fv.and.sal(i).eq.fv) then
        else if (pres(i).eq.fv) then
          iflag(i)=0
        else
          iflag(i)=1
        end if
      end do

      do 11 i = 1, nlev
        if (pres(i).eq.0.0.and.cksm_flag(i).eq.0.and.tem(i).eq.0.0.and.
     *     sal(i).eq.0.0) go to 11
c        if (pres(i).eq.fv.or.tem(i).eq.fv.or.sal(i).eq.fv) go to 11
        if (pres(i).eq.fv) go to 11
        np=np+1
        pres_work(np)=pres(i)
        if (idbug.gt.0) write(66,*) np,pres_work(np)
 11   end do

      if (np.lt.nw1) then    ! less than 3
        if (np.eq.0) then
          do i=1,nlev
            iflag(i)=0
          end do
          return
        end if
        if (np.eq.1) then
          iflag(np)=1
        end if
        if (np.eq.2) then
          if (pres(2).gt.pres(1)) then     ! changed from .ge. to .gt.
            iflag(1)=1
            iflag(2)=1
          else
            iflag(1)=1
            iflag(2)=4
          end if
         nlev=np
        end if
c        nlev=np
        if (idbug.gt.0) write(66,*)'iflag(1)=',iflag(1),' iflag(2)=',
     *  iflag(2)
        return
      end if
c
c --- Apply First Median Filter with a width(nw=3).
c     Eliminate any abs(pressure - median) > 102.0 and loop until
c     there is no abs(pressure - median) > 102.0
c
      mcnt=0
 10   continue
      if (mcnt.ge.np) go to 100
      if (np-mcnt.ge.nw1) then      ! Enough data points?
        mwin=nw1
      else
        go to 100
      end if
      do i=1,mwin
        x(i)=pres_work(mcnt+i)
      end do
      if (mwin.ne.nw1) then
        status=9
        write(6,*) prg_sub,' mwin is not 3. Check mwin value.'
        write(6,*) prg_sub, ': STOP -> STATUS = ', status
        call exit (status)
      end if

      call median(x,mwin,xmed)
      mcnt = mcnt + 1
      if (mcnt.gt.lmax) then
         status=9
         write(6, '(A,": mcnt gt lmax case: - ", i4,i4)') 
     *   prg_sub, mcnt, lmax
         write(50, '(A,": mcnt gt lmax case: - ", i4,i4)') 
     *   prg_sub, mcnt, lmax
         write(50,*) prg_sub, ': STOP -> STATUS = ', status
         call exit(status)
      end if
      xmed_save(mcnt+(nw1/2))=xmed
      if (idbug.gt.0) write(66,14) mcnt+nw1/2,x(1),x(2),x(3),xmed
 14   format(1x,i4,' x''s',3(f6.1),' median=',f7.2)
      go to 10
 100  continue
      if (idbug.gt.0) write(66,*)'nlev=',nlev,' mcnt=',mcnt
      if (idbug.gt.0) write(66,*)'<<< First Median Filter Loop >>>'
      if (idbug.gt.0) write(66,*)'<<< width=',nw1,' tol=',tol1,' >>>'

      mx=1                   ! a new counter for data point
      pres_tmp(mx)=pres_work(1)
      do 22 i=2,np-1
c        if (abs(pres_work(i)-xmed_save(i)).gt.tol1) then
        if (abs(pres_work(i)-xmed_save(i)).gt.tol1.or.
c     *     pres_work(i).eq.pres_work(i-1)) then   ! a bug found 05-apr-05
     *     pres_work(i).le.pres_work(i-1)) then  
c          write(6,*)'Median filter 1:',i,pres_work(i),xmed_save(i)
          nx=nx+1
          pres_bad(nx)=pres_work(i)
        else
          mx=mx+1
          pres_tmp(mx)=pres_work(i)
        end if
 22   end do
      if (pres_work(np-1).eq.pres_work(np)) then     ! pres has to increase
         nx=nx+1
         pres_bad(nx)=pres_work(i)
      else
        mx=mx+1
        pres_tmp(mx)=pres_work(np)
      end if

      if (nx.ge.1) then            ! Mark bad pressure if there is any
        do j=1,nx
        do i=1,nlev
c          iflag(i)=1              ! a bug 08-aug-05
          if (pres(i).eq.pres_bad(j)) iflag(i)=4
        end do
        end do
      end if
c
c --- Spike "up" and "down" for 2 times to identify any spikes.
c
c      write(6,*)'Number of data point that pressure > 120.0 nx=',nx,mx
c
c --- Spike "Up" case
c
      do k=1,2                 ! do twice spike Up and Down
        if (idbug.gt.0) write(66,*)'<<< Spike up case >>>'
        kx=1
        pres_tmp1(kx)=pres_tmp(1)
        do i=2,mx-1
          d_bef=pres_tmp(i)-pres_tmp(i-1)
          d_aft=pres_tmp(i+1)-pres_tmp(i)
          if (d_bef.le.0.0.and.abs(d_aft).gt.tol) then
            write(6,*)'Found it up 1:',i,pres_tmp(i)
            nx=nx+1
            pres_bad(nx)=pres_tmp(i)
          else
            kx=kx+1
            if (kx.gt.lmax) then
               status=9
               write(6, '(A,": kx gt lmax case: - ", i4,i4)') 
     *         prg_sub, kx, lmax
               write(50, '(A,": kx gt lmax case: - ", i4,i4)') 
     *         prg_sub, kx, lmax
               write(50,*) prg_sub, ': STOP -> STATUS = ', status
               call exit(status)
            end if
            pres_tmp1(kx)=pres_tmp(i)
          end if
        end do
c
c --- Spike "Down" case
c
        if (idbug.gt.0) write(66,*)'<<< Spike down case >>>'
        kx=1
        pres_tmp1(kx)=pres_tmp(1)
        do i=2,mx-1
          d_bef=pres_tmp(i)-pres_tmp(i-1)
          d_aft=pres_tmp(i+1)-pres_tmp(i)
          if (d_aft.le.0.0.and.abs(d_bef).gt.tol) then
            write(6,*)'Found it down:',i,pres_tmp(i-1),pres_tmp(i),tol
            nx=nx+1
            if (nx.gt.lmax) then
               status=9
               write(6, '(A,": nx gt lmax case: - ", i4,i4)') 
     *         prg_sub, nx, lmax
               write(50, '(A,": nx gt lmax case: - ", i4,i4)') 
     *         prg_sub, nx, lmax
               write(50,*) prg_sub, ': STOP -> STATUS = ', status
               call exit(status)
            end if
            pres_bad(nx)=pres_tmp(i)
          else
            kx=kx+1
            if (kx.gt.lmax) then
               status=9
               write(6, '(A,": kx gt lmax case: - ", i4,i4)') 
     *         prg_sub, kx, lmax
               write(50, '(A,": kx gt lmax case: - ", i4,i4)') 
     *         prg_sub, kx, lmax
               write(50,*) prg_sub, ': STOP -> STATUS = ', status
               call exit(status)
            end if
            pres_tmp1(kx)=pres_tmp(i)
          end if
        end do
        kx=kx+1
        if (kx.gt.lmax) then
          status=9
          write(6,'(A,": kx gt lmax case: - ", i4,i4)') prg_sub,nx,lmax
          write(50, '(A,": kx gt lmax case: - ", i4,i4)') 
     *    prg_sub, kx, lmax
          write(50,*) prg_sub, ': STOP -> STATUS = ', status
          call exit(status)
        end if
        pres_tmp1(kx)=pres_tmp(mx)
      end do                   ! do k=1,2
c
c --- Second Median Filter with a width=7 and tol > 120.0
c
      if (kx.lt.nw2) then      ! less than 7 points case return
        do j=1,nx
          do i=1,nlev
c            iflag(i)=1        ! a bug 09-aug-05
            if (pres(i).eq.pres_bad(j)) iflag(i)=4
          end do
        end do
        return
      end if
      if (idbug.gt.0) write(66,*)'<<< Second Median Filter Loop >>>'
      if (idbug.gt.0) write(66,*)'<<< width=',nw2,' tol=',tol2,' >>>'
c      do i=1,kx
c        write(6,*) ' second inputs:',i,pres_tmp1(i)
c      end do
      mcnt=0
 20   continue
      if (mcnt.ge.kx) go to 100
      if (kx-mcnt.ge.nw2) then      ! Enough data points?
        mwin=nw2
      else
        go to 200
      end if
      do i=1,mwin
        x(i)=pres_tmp1(mcnt+i)
      end do
      if (mwin.ne.nw2) then
        status=9
        write(6,*) prg_sub,' mwin is not 7 Check mwin value.'
        write(6,*) prg_sub, ': STOP -> STATUS = ', status
        call exit (status)
      end if

      call median(x,mwin,xmed)
      mcnt = mcnt + 1
      if (mcnt.gt.lmax) then
         status=9
         write(6, '(A,": mcnt gt lmax case: - ", i4,i4)') 
     *   prg_sub, mcnt, lmax
         write(50, '(A,": mcnt gt lmax case: - ", i4,i4)') 
     *   prg_sub, mcnt, lmax
         write(50,*) prg_sub, ': STOP -> STATUS = ', status
         call exit(status)
      end if
      xmed_save(mcnt+(nw2/2))=xmed
      if (idbug.gt.0) write(66,15) mcnt+nw2/2,x(1),x(2),x(3),x(4),x(5),
     *x(6),x(7),xmed
 15   format(1x,i4,' x''s',7(f8.2),' median=',f8.2)
      go to 20
 200  continue

      pres_tmp2(1)=pres_tmp1(1)
      pres_tmp2(2)=pres_tmp1(2)
      pres_tmp2(3)=pres_tmp1(3)
      lx=3
      do i=4,kx-3
        if (abs(pres_tmp1(i)-xmed_save(i)).gt.tol2) then
          write(6,*)'Median filter 2:',i,pres_tmp1(i),xmed_save(i)
          nx=nx+1
          if (nx.gt.lmax) then
            status=9
            write(6, '(A,": nx gt lmax case: - ", i4,i4)') 
     *      prg_sub, nx, lmax
            write(50, '(A,": nx gt lmax case: - ", i4,i4)') 
     *      prg_sub, nx, lmax
            write(50,*) prg_sub, ': STOP -> STATUS = ', status
            call exit(status)
          end if
          pres_bad(nx)=pres_tmp1(i)
        else
          lx=lx+1
          if (lx.gt.lmax) then
             status=9
             write(6, '(A,": lx gt lmax case: - ", i4,i4)') 
     *       prg_sub, lx, lmax
             write(50, '(A,": lx gt lmax case: - ", i4,i4)') 
     *       prg_sub, lx, lmax
             write(50,*) prg_sub, ': STOP -> STATUS = ', status
             call exit(status)
          end if
          pres_tmp2(lx)=pres_tmp1(i)
        end if
      end do

C CS  seems to me that kx-3+i may sometimes be less than 1
      if (kx.lt.nw2.or.kx-3.le.0) then
          status=9
          write(6, '(A,": kx lt nw2 or kx-3 le 0 case: - ", i4,i4)') 
     *    prg_sub, kx, nw2
          write(50, '(A,": kx lt nw2 or kx-3 le 0 case: - ", i4,i4)') 
     *    prg_sub, kx, nw2
          write(50,*) prg_sub, ': STOP -> STATUS = ', status
          call exit(status)
      end if
      do i=1,3
        pres_tmp2(lx+i)=pres_tmp1(kx-3+i)
      end do
c
c --- Third Median Filter with a width(nw3=11) > 200.0
c
      if (lx.lt.nw3) then          ! less than nw3 points return
        do j=1,nx
          do i=1,nlev
c            iflag(i)=1            ! a bug 09-aug-05 
            if (pres(i).eq.pres_bad(i)) iflag(i)=4
          end do
        end do
        return
      end if
      if (idbug.gt.0) write(66,*)'<<< Third Median Filter Loop >>>'
      if (idbug.gt.0) write(66,*)'<<< width=',nw3,' tol=',tol3,' >>>'
      mcnt=0
 30   continue
      if (mcnt.ge.lx) go to 300
      if (lx-mcnt.ge.nw3) then      ! Enough data points?
        mwin=nw3
      else
        go to 300
      end if
      do i=1,mwin
        x(i)=pres_tmp2(mcnt+i)
      end do
      if (mwin.ne.nw3) then
        status=9
        write(6,*) prg_sub,' mwin is not 11 Check mwin value.'
        write(6,*) prg_sub, ': STOP -> STATUS = ', status
        call exit (status)
      end if

      call median(x,mwin,xmed)
      if (mcnt.gt.lmax) then
         status=9
         write(6, '(A,": mcnt gt lmax case: - ", i4,i4)') 
     *   prg_sub, mcnt, lmax
         write(50, '(A,": mcnt gt lmax case: - ", i4,i4)') 
     *   prg_sub, mcnt, lmax
         write(50,*) prg_sub, ': STOP -> STATUS = ', status
         call exit(status)
      end if
      mcnt = mcnt + 1
      xmed_save(mcnt+(nw3/2))=xmed
      if (idbug.gt.0) write(66,16) mcnt+nw3/2,x(1),x(2),x(3),x(4),x(5),
     *x(6),x(7),x(8),x(9),x(10),x(11),xmed
 16   format(1x,i4,' x''s:',11(f6.1),' median=',f7.2)
      go to 30
 300  continue
      
      do i=6,lx-5
        if (abs(pres_tmp2(i)-xmed_save(i)).gt.tol3) then
          write(6,*)'Median filter 3:',i,pres_tmp2(i),xmed_save(i)
          nx=nx+1
          pres_bad(nx)=pres_tmp2(i)
        end if
      end do

      do k=1,nx
        do i=1,nlev
          if (pres(i).eq.pres_bad(k)) then
            iflag(i)=4
            go to 150
          end if
        end do              ! do i=1,nlev
 150    continue 
      end do                ! do k=1,nx
c
c--- Surface points check case.
c
      pmin=pres_work(np)
      do i=1,np
         pmin=MIN(pmin,pres_work(i))
      end do
c      write(6,*)'pmin=',pmin
      ll=NINT((MIN(nw1,nw2,nw3))/2.0)
c      write(6,*)'ll=',ll
      do i=1,ll
        p1=pmin+dp*(i-1) 
        if (abs(p1-pres(i)).le.tol) then
          if (idbug.gt.0) write(66,*)'Surface points check OK'
        else
          if (idbug.gt.0) write(66,*)'Surface points check NOT OK'
          write(6,*)'Surface points check NOT OK'
          iflag(i)=4
        end if
      end do
      
      do i=1,nlev
        if (pres(i).eq.0.0.and.cksm_flag(i).eq.0.and.tem(i).eq.0.0
     *      .and.sal(i).eq.0.0) iflag(i)=0
        if (pres(i).eq.fv) iflag(i)=0
c        write(66,*)i,pres(i),iflag(i)
        if (iflag(i).eq.4) write(6,*) i,pres(i),iflag(i)
      end do
      return
      end
