c------------------------------------------------------------------- 
c     Revised date: 04-Jan-13 VMH  - Increase cline,sal_adj_line to 256 chars
c                   14-Aug-12 VMH  - Eliminate most_likely_frame_status,
c                                    add space to cnp0 if only 6ch
c                   23-Aug-10 (ER  - Enabled a grey_flag value to have
c                                 leading blanks
c                   26-Oct-09 (ER  - gray_flag length increased to 5
c                                    so app. can deal with multiple grey list
c                                    parameters)
c                   13-Oct-09 (ERH - Add Most Likely Frame Status var
c                   05-Jul-07 (YHD - More than one line of 'PSAL ADJUST' line)
c                   03-Jul-07 (YHD - A character variables, cnp & cnp0 intro-
c                                    duced)
c                   16-Jan-07 (YHD - salinity offset is read)
c                   23-Feb-05 (YHD - Pick a larger value for nominal profile 
c                                    pressure when there are more than one)
c                   07-Jan-05 (YHD - add grey list)
c                   18-Nov-04 (YHD - 'grey list' lines are added meta files) 
c                   08-Jul-03 (YHD - 'proj_name' is passed to main program.)
c                   04-May-04 (YHD - 'max' was never used & got rid off)
c                   10-Apr-03 (YHD - 'inst_ref' is brand,type,serial number.)
c                   26-Mar-03 (YHD - "dlay_down_hours" is defined as real.)
c                   30-Jan-03 (YHD - trans_sys & pos_sys are included)
c                   24-Dec-02 (CS - changed the cycle times to
c                                   double precision)
c                   02-Aug-02 (YHD - cyc_days(41:47) corrected)
c                   17-Jul-02 (YHD - fsnum added)
c   
c     rd_float_meta.f
c     ===============
c     Reads a float "META" file.
c     YHD, 20-Apr-2002
c
c-------------------------------------------------------------------

      subroutine rd_float_meta(iunit,int_num,prof_num,wmo_num,trans_id,
     *  proj_name,
     *  inst_ref,st_time,laun_time,dlay_down_hours,down_days,up_hours,
     *  cyc_days,des_hours,neu_days,asc_hours,surf_hours,laun_pos_la,
     *  laun_pos_lo,fsnum,trans_sys,pos_sys,deep,send_gts,grey_list,
     *  grey_date_st,grey_date_en,grey_flag,grey_parm,grey_reason,
     *  sal_adj_dd,sal_adj_mm,sal_adj_yyyy,sal_adj_pnum,sal_offset,
     *  sal_std,sal_adj_line)
c     
c     iunit:     unit for reading
c     int_num:   internal id number
c     prof_num:  profile number
c     wmo_num:   wmo id
c     trans_id:  transmission id number
c     inst_ref: APEX, SOLO, or PROVOR(brand)+type+serial number
c     st_time:   start time for APEX
c     proj_name: U.S. ARGO project or Steller's Sea Lion Research
c     dlay_down_hours: delay of first down time for APEX
c     down_days:  days in the water for APEX
c     up_hours:    time to ascent to surface for APEX
c     laun_time: launch time for SOLO type
c     cyc_days: cycle time for SOLO type
c     des_hours: descent time for SOLO type
c     neu_days: neutral depth time (=parking) for SOLO type
c     asc_hours: ascent time for SOLO type
c     surf_hours: surface time for SOLO type
c     laun_pos_la: launch position (latitude)
c     laun_pos_lo: launch postion (longitude)
c     fsnum: float serial number (character)
c     trans_sys: transmission type (character*16)
c     pos_sys: positioning system (character*8)
c     deep : deepest pressure (real)
c     send_gts: send this data to GTS? (character*1)
c     grey_list: Is this float in grey list or not? (character*1)
c     grey_reason: Reason for in grey list (character*128)
c     sal_adj_dd: salinity adjusted day
c     sal_adj_mm: salinity adjusted month
c     sal_adj_yyyy: salinity adjusted year
c     sal_adj_pnum: salinity adjusted profile number
c     sal_offset: salinity offset that will apply to salinity
c     sal_std: salinity offset standard deviation
c     sal_adj_line: psal adjusted line

      implicit none

      integer, parameter :: idbug = 0

      integer i,iunit,icnt,iyear,imon,iday,ihr,imin,isec,
     *        int_num,wmo_num,idbug,nb,lchar_occur,ll,nn,ibeg,
     *        nlen,ndec,prof_num

      real    dlay_down_hours
      real    tmp,lat,latm,lon,lonm
      real    deep,deep_nominal(5)
      double precision down_days,up_hours,cyc_days,des_hours,
     *        neu_days,asc_hours,
     *        surf_hours,laun_pos_la,laun_pos_lo
      character cline*256
      character prg_sub*13,st_time*64,inst_type*64,laun_time*64
      character fsnum*4,trans_sys*16,pos_sys*8,trans_id*16,inst_ref*64,
     *con_type*3,tem_type*3,con_ser_number*4,tem_ser_number*4,
     *proj_name*64
      character send_gts*1,grey_list*3,grey_reason(*)*80,
     *          grey_parm(*)*15,grey_date_st(*)*11,grey_date_en(*)*11,
     *          grey_flag(*)*5           
      integer   n_grey,ncnt,lnb,lnblnk

      data     prg_sub/'RD_FLOAT_META'/
c
c--- Salinity offset.
c
      integer nns
      parameter (nns=50)
      integer ns,sal_adj_dd_save(nns),sal_adj_mm_save(nns),
     *        sal_adj_yyyy_save(nns),sal_adj_pnum_save(nns)
      integer sal_adj_dd,sal_adj_mm,sal_adj_yyyy,sal_adj_pnum,
     *        lastnb,ll,nn

      real    sal_offset,sal_std
      real    sal_offset_save(nns),sal_std_save(nns)
      character sal_adj_line*(*), cnp*6, cnp0*7
      character sal_adj_line_save(nns)*256
      data    ns /0/                ! counter for 'PSAL ADJUST' line

c
c---Read header information.
c
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from RD_FLOAT_META >>>>>'
 1    continue
      read(iunit,25,end=100,err=999) cline
      if (idbug.gt.0) write(6,*) cline
      if (cline(1:11).eq.'internal ID') then
        read(cline(41:51),*,err=999) int_num    
      else if (cline(1:15).eq.'transmission ID') then
        read(cline(41:51),25,err=999) trans_id
      else if (cline(1:17).eq.'transmission type') then
        read(cline(41:51),25,err=999) trans_sys
         if (trans_sys(1:5) .eq. 'ARGOS') then
            pos_sys='ARGOS   '
         elseif (trans_sys(1:7) .eq. 'ORBCOMM') then
            pos_sys='GPS     '
         endif
      else if (cline(1:15).eq.'instrument type') then
        read(cline(41:56),25,err=999) inst_type
      else if (cline(1:12).eq.'Project name') then
        read(cline(41:80),25,err=999) proj_name
      else if (cline(1:10).eq.'start time') then
        read(cline(41:80),25,err=999) st_time
      else if (cline(1:24).eq.'delay of first down time') then
        read(cline(41:45),*,err=999) dlay_down_hours
      else if (cline(1:9).eq.'down time') then
        read(cline(41:45),30,err=999) tmp
        down_days=tmp
      else if (cline(1:7).eq.'up time') then
        read(cline(41:45),30,err=999) tmp
        up_hours=tmp
      else if (cline(1:11).eq.'launch time') then
        read(cline(41:80),25,err=999) laun_time
      else if (cline(1:10).eq.'cycle time') then
        read(cline(41:47),*,err=999) tmp
        cyc_days=tmp
      else if (cline(1:12).eq.'descent time') then
        read(cline(41:46),*,err=999) tmp
        des_hours=tmp
      else if (cline(1:11).eq.'ascent time') then
        read(cline(41:46),*,err=999) tmp
        asc_hours=tmp
      else if (cline(1:18).eq.'neutral depth time') then
        read(cline(41:46),*,err=999) tmp
        neu_days=tmp
      else if (cline(1:12).eq.'surface time') then
        read(cline(41:46),*,err=999) tmp
        surf_hours=tmp
      else if (cline(1:11).eq.'PSAL ADJUST') then
        lastnb=lnblnk(cline)
        sal_adj_line=cline(1:lastnb)
        ibeg=41
        nn=index(cline(ibeg:lastnb),' ')+ibeg-1
        if (nn-ibeg.eq.2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_dd
        if (nn-ibeg.eq.1) read(cline(ibeg:nn-1),'(i1.1)') sal_adj_dd
c        write(6,*)'sal_adj_dd=',sal_adj_dd
        ibeg=nn+1
        nn=index(cline(ibeg:lastnb),' ')+ibeg-1
        if (nn-ibeg.eq.2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_mm
        if (nn-ibeg.eq.1) read(cline(ibeg:nn-1),'(i1.1)') sal_adj_mm
c        write(6,*)'sal_adj_mm=',sal_adj_mm
        ibeg=nn+1
        nn=index(cline(ibeg:lastnb),' ')+ibeg-1
        if (nn-ibeg.eq.4) read(cline(ibeg:nn-1),'(i4.4)') sal_adj_yyyy
        if (nn-ibeg.eq.2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_yyyy
c        write(6,*)'sal_adj_yyyy=',sal_adj_yyyy
        ibeg=nn+1
        nn=index(cline(ibeg:lastnb),' ')+ibeg-1
        if (nn-ibeg.eq.3) read(cline(ibeg:nn-1),'(i3.3)') sal_adj_pnum
        if (nn-ibeg.eq.2) read(cline(ibeg:nn-1),'(i2.2)') sal_adj_pnum
        if (nn-ibeg.eq.1) read(cline(ibeg:nn-1),'(i1.1)') sal_adj_pnum
        ibeg=nn+2          ! There are 2 spaces after sal_adj_pnum. 03-jul-07
        if (cline(ibeg-1:ibeg-1).eq.'-') ibeg=nn+1   ! 03-jul-07
        nn=index(cline(ibeg:lastnb),'.')+ibeg-1
        ll=index(cline(ibeg:lastnb),' ')+ibeg-1
        nlen=ll-ibeg
        ndec=ll-nn-1

        cnp0(1:2)='(f'
        if (nlen.ge.10) then
          write(cnp0(3:4),'(i2)') nlen
          cnp0(5:5)='.'
          write(cnp0(6:6),'(i1)') ndec
          cnp0(7:7)=')'
        else
          write(cnp0(3:3),'(i1)') nlen
          cnp0(4:4)='.'
          write(cnp0(5:5),'(i1)') ndec
          cnp0(6:6)=')'
          cnp0(7:7)=' '
        end if

c        if (nlen.eq.11.and.ndec.eq.7) read(cline(ibeg:ll-1),'(f11.7)')
c     *     sal_offset
c        if (nlen.eq.10.and.ndec.eq.7) read(cline(ibeg:ll-1),'(f10.7)')
c     *     sal_offset
c        if (nlen.eq.9.and.ndec.eq.7) read(cline(ibeg:ll-1),'(f9.7)')
c     *     sal_offset
c        if (nlen.eq.7.and.ndec.eq.5) read(cline(ibeg:ll-1),'(f7.5)')
c     *     sal_offset
c        if (nlen.eq.7.and.ndec.eq.4) read(cline(ibeg:ll-1),'(f7.4)') 
c     *     sal_offset
c        if (nlen.eq.6.and.ndec.eq.4) read(cline(ibeg:ll-1),'(f6.4)') 
c     *     sal_offset
c        if (nlen.eq.6.and.ndec.eq.3) read(cline(ibeg:ll-1),'(f6.3)') 
c     *     sal_offset
c        if (nlen.eq.5.and.ndec.eq.2) read(cline(ibeg:ll-1),'(f5.2)') 
c     *     sal_offset
c        if (nlen.eq.4.and.ndec.eq.1) read(cline(ibeg:ll-1),'(f4.1)') 
c     *     sal_offset

        write(6,*)'sal_offset fomat: cnp0=',cnp0
        read(cline(ibeg:ll),cnp0) sal_offset
        
        if (nlen.gt.11.or.ndec.gt.7) then
         write(6,*)'Error I need to add this format for sal_offset...'
          write(6,*)'nlen=',nlen,' ndec=',ndec
        end if
        if (idbug.gt.0) write(66,*)'sal_offset:nlen=',nlen,' ndec=',ndec

        ll=lchar_occur(cline,' ')
        nn=lchar_occur(cline,'.')
        nlen=lastnb-ll
        ndec=lastnb-nn

        cnp(1:2)='(f'
        cnp(4:4)='.'
        cnp(6:6)=')'
        write(cnp(3:3),'(i1)') nlen
        write(cnp(5:5),'(i1)') ndec

c        if(nlen.eq.7.and.ndec.eq.5)read(cline(ibeg:ll),'(f7.5)') sal_std
c        if(nlen.eq.7.and.ndec.eq.4)read(cline(ibeg:ll),'(f7.4)') sal_std
c        if(nlen.eq.6.and.ndec.eq.4)read(cline(ibeg:ll),'(f6.4)') sal_std
c        if(nlen.eq.6.and.ndec.eq.3)read(cline(ibeg:ll),'(f6.3)') sal_std
c        if(nlen.eq.5.and.ndec.eq.3)read(cline(ibeg:ll),'(f5.3)') sal_std
c        if(nlen.eq.5.and.ndec.eq.2)read(cline(ibeg:ll),'(f5.2)') sal_std
c        if(nlen.eq.4.and.ndec.eq.2)read(cline(ibeg:ll),'(f4.2)') sal_std

        write(6,*)'sal_std format: cnp=',cnp
        read(cline(ibeg:ll),cnp) sal_std
c
c--- Is there more than one line of "PSAL ADJUST" line?
c
        ns=ns+1
        sal_adj_dd_save(ns)=sal_adj_dd
        sal_adj_mm_save(ns)=sal_adj_mm
        sal_adj_yyyy_save(ns)=sal_adj_yyyy
        sal_adj_pnum_save(ns)=sal_adj_pnum
        sal_offset_save(ns)=sal_offset
        sal_std_save(ns)=sal_std
        sal_adj_line_save(ns)=sal_adj_line
        if (ns.gt.50) then
          write(6,*)'Reading rd_float_meta.f dimension error ns=',ns 
          write(6,*)'Reading rd_float_meta.f Increase nns nns=',nns
          write(50,*)'Reading rd_float_meta.f dimension error ns=',ns
          write(50,*)'Reading rd_float_meta.f Increase nns nns=',nns
        end if

      else if (cline(1:25).eq.'launch position [lat latm') then
        read(cline(41:80),*,err=999) lat,latm,lon,lonm
        laun_pos_la=lat+latm/60.0
        laun_pos_lo=lon+(lonm/60.0)
      else if (cline(1:19).eq.'float serial number') then
        read(cline(41:44),25,err=999) fsnum
      else if (cline(1:32).eq.'conductivity sensor manufacturer') then
        read(cline(41:43),25,err=999) con_type
      else if (cline(1:31).eq.'temperature sensor manufacturer') then
        read(cline(41:43),25,err=999) tem_type
      else if (cline(1:33).eq.'conductivity sensor serial number') then
        read(cline(41:44),25,err=999) con_ser_number
      else if (cline(1:32).eq.'temperature sensor serial number') then
        read(cline(41:44),25,err=999) tem_ser_number
      else if (cline(1:31).eq.'nominal profile pressure [dbar]') then
        lnb=lnblnk(cline(41:80))      ! more than one pressure case
        ncnt=0
        do i=41,41+lnb
          if (cline(i:i).eq.' ') ncnt=ncnt+1
        end do
        read(cline(41:80),*,err=999) (deep_nominal(i),i=1,ncnt)
        deep=deep_nominal(1)
        do i=1,ncnt
          if (deep_nominal(i).gt.deep) deep=deep_nominal(i)
        end do
      else if (cline(1:11).eq.'send to GTS') then
        read(cline(41:41),25,err=999) send_gts
        write(6,*)'send_gts=',send_gts
        if (idbug.gt.0) write(66,*)'send_gts=',send_gts
      else if (cline(1:16).eq.'add to grey list') then
        read(cline(41:43),25,err=999) grey_list
        read(grey_list(3:3),'(i1)') n_grey          ! # of grey list
        do i=1,n_grey
           read(iunit,25,end=100,err=999) cline     ! grey list date
           grey_date_st(i)=cline(41:50)
           grey_date_en(i)=cline(52:61)
           if (idbug.gt.0) write(66,*)'grey_date_st(i)=',grey_date_st(i)
           if (idbug.gt.0) write(66,*)'grey_date_en(i)=',grey_date_en(i)
           read(iunit,25,end=100,err=999) cline     ! grey list flag
           read(cline(41:45),25,err=999) grey_flag(i)
           grey_flag(i)=adjustl(grey_flag(i))
           if (idbug.gt.0) write(66,*)'grey_flag(i)=',grey_flag(i)
           read(iunit,25,end=100,err=999) cline     ! grey list parameter
           read(cline(41:55),25,err=999) grey_parm(i)
           if (idbug.gt.0) write(66,*)'grey_parm=',grey_parm(i)
           read(iunit,25,end=100,err=999) cline     ! why add to grey list 
           read(cline(41:80),25,err=999) grey_reason(i)
           if (idbug.gt.0) write(66,*)'grey_reason=',grey_reason(i)
        end do               
      end if
      if (con_type.eq.'   ') con_type=tem_type
      if (con_ser_number.eq.'    ') con_ser_number=tem_ser_number
        
      nb=lchar_occur(inst_type,'_')
      if (con_ser_number.eq.'n/a') then
        inst_ref=inst_type(1:nb)//con_type
      else
        inst_ref=inst_type(1:nb)//con_type//'_'//con_ser_number
      end if
      go to 1
 25   format(a)
 30   format(f3.0)
 100  continue
      if (ns.gt.1) then
        do i=1,ns-1
          if (prof_num.ge.sal_adj_pnum_save(i).and.
     *        prof_num.lt.sal_adj_pnum_save(i+1)) then
              sal_adj_dd=sal_adj_dd_save(i)
              sal_adj_mm=sal_adj_mm_save(i)
              sal_adj_yyyy=sal_adj_yyyy_save(i)
              sal_adj_pnum=sal_adj_pnum_save(i)
              sal_offset=sal_offset_save(i)
              sal_std=sal_std_save(i)
              sal_adj_line=sal_adj_line_save(i)
           else if (prof_num.gt.sal_adj_pnum_save(i).and.
     *              prof_num.ge.sal_adj_pnum_save(i+1)) then
              sal_adj_dd=sal_adj_dd_save(i+1)
              sal_adj_mm=sal_adj_mm_save(i+1)
              sal_adj_yyyy=sal_adj_yyyy_save(i+1)
              sal_adj_pnum=sal_adj_pnum_save(i+1)
              sal_offset=sal_offset_save(i+1)
              sal_std=sal_std_save(i+1)
              sal_adj_line=sal_adj_line_save(i+1)
           else if (prof_num.lt.sal_adj_pnum_save(1)) then
              sal_adj_dd=sal_adj_dd_save(1)
              sal_adj_mm=sal_adj_mm_save(1)
              sal_adj_yyyy=sal_adj_yyyy_save(1)
              sal_adj_pnum=sal_adj_pnum_save(1)
              sal_offset=sal_offset_save(1)
              sal_std=sal_std_save(1)
              sal_adj_line=sal_adj_line_save(1)
           end if
        end do
      end if
      return
 999  write(6,*)'Reading rd_float_meta.f error=999 ',cline
      write(50,*)'Reading rd_float_meta.f error=999 ',cline 
      return
      end 



