c Revision Date: 30-Jan-07 (YHD - adjusted salinity check on region test)
c
c* regi_check_s.f
c  ================
c  yhd, 01-30-2007
c  This test checks 'Med' or 'Red' sea region check.
c
c  inputs:
c    icho - 0 - region test for temp; 1 - region test for salinity; 
c           2 - region test for adjusted salinity
c    sa - salinity array or adjusted salinity array
c    de - depth array
c    te - temperature array
c    nl - number of depth/salinity pairs
c    rlon - longitude
c    rlat - latitude
c    ireds_s
c    imeds_s
c    rmin -
c    rmax -
c
c  return:
c    iregion -
c    iregi_flag - flag array containing - 1 good point
c                                       - 4 bad  point
c    iover_flag - overall flag
c    regi_flag - pass 'P' or fail 'F' 
c
      subroutine regi_check_s(icho,nl,te,sa,de,rlon,rlat,
     *           ireds_s,imeds_s,rmin,rmax,
     *           iregion,imeds_region,ireds_region,
     *           iregi_s,regi_flag,iover_regi)

      implicit none
      integer n,nl,iregi_s(*),ireds_s(*),imeds_s(*),idbug,iover_regi,
     *        rmin,rmax,icho,iregion,ireds_region,imeds_region
      real sa(*),de(*),te(*),fv,rlon,rlat
      character regi_flag*1,var*5
      data idbug /0/      ! make it 1 to debug
      data fv /-999.0/

      if (icho.eq.1) var='S'
      if (icho.eq.2) var='S_ADJ'

      if (rlat.eq.99.999.and.rlon.eq.999.999) then
        write(6,*) 'REGI CHECK TEST ',var,'can not be determined.'
        write(48,*)'REGI CHECK TEST ',var,'can not be determined.'
        iover_regi=0
        regi_flag='N'
        return
      end if

      do n=1,nl
        iregi_s(n)=1
        if (de(n).eq.0..and.sa(n).eq.0..and.te(n).eq.0) 
     *   iregi_s(n)=0
        if ((de(n).eq.fv.and.sa(n).eq.fv).or.te(n).eq.fv) then
          iregi_s(n)=0
        end if
      end do

      rmax=0
      rmin=1000
      if (ireds_region.eq.1) then
        iregion=1      
        do n=1,nl
          if (ireds_s(n).eq.4) then 
	    iregi_s(n)=4
          else
            iregi_s(n)=1
          end if
          if (iregi_s(n).gt.rmax) rmax=iregi_s(n)
          if (iregi_s(n).lt.rmin) rmin=iregi_s(n)
        end do
      else if (imeds_region.eq.1) then
        iregion=2
        do n=1,nl
          if (imeds_s(n).eq.4) then 
	    iregi_s(n)=4
          else
            iregi_s(n)=1
          end if
          if (iregi_s(n).gt.rmax) rmax=iregi_s(n)
          if (iregi_s(n).lt.rmin) rmin=iregi_s(n)
          if (te(n).eq.0.0.and.de(n).eq.0.0.and.sa(n).eq.0.0) then
            iregi_s(n)=0
          end if
        end do
      end if

      if (nl.eq.0.or.rmax.eq.0) then
        write(6,*) 'REGI CHECK TEST ',var,'can not be applied.'
        write(48,*)'REGI CHECK TEST ',var,'can not be applied.'
        regi_flag='N'
        iover_regi=0
        return
      end if

      do n=1,nl
        if (iregi_s(n).eq.4) then
          write(6,*) 'REGI CHECK TEST ',var,'failed.'
          write(48,*)'REGI CHECK TEST ',var,'failed.'
          regi_flag='F'
          iover_regi=4
          return
        else
          regi_flag='P'
          iover_regi=1
        end if
      end do

      write(6,*) 'REGI CHECK TEST ',var,'passed.'
      write(48,*)'REGI CHECK TEST ',var,'passed.'
 71   continue
      return
      end

