c  Revision date: 13-Dec-08 (YHD - low salinity range changed from 0 to 2 psu)
c                 03-Apr-07 (YHD - Output produced only when idbug is on)
c                 10-Feb-04 (HY - check mediterranean sea)
c    meditsea_check.f
c    ================
c    Checks impossible temperature and salinity in red and mediterranean seas.
c    The mediterranean sea:30N,6W;30N,40E;40N,35E;42N,20E;50N,15E;
c                          40N,5E;30N,6W
c                Temperature in range 10.0 to 40.0 degrees C
c                Salinity in range 2.0 to 40.0 in psu
c                -- from GOSUD REAL-TIME QC
c
c       output: imeditsea_flag_t(or imeditsea_flag_s)=1 means OK
c               imeditsea_flag_t(or imeditsea_flag_s)=4 means NOT ok.
c     
C Sample driver to test inpolygon
C  ftp://anusf.anu.edu.au/mld900
C check for real lat and lon

      subroutine meditsea_check(lon,lat,tem,sal,n1,
     * imeditsea_region,imeditsea_flag_t,imeditsea_flag_s)
      
      implicit none

      integer n1,N,i,nmax
      parameter (nmax=6,N=6)
      integer imeditsea_region,imeditsea_flag_t(*)
      integer imeditsea_flag_s(*)
      real X,Y,Xpoly(nmax),Ypoly(nmax)
      real tem(*),sal(*),lat,lon
      integer idbug
      logical INPOLYGON

      data xpoly/-6,40,35,20,15,5/
      data ypoly/30,30,40,42,50,40/
      
      data idbug /0/          ! make 1 to debug
   
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from MEDITSEA_CHECK >>>>>'
      do i = 1, n1
        imeditsea_flag_t(i)=1
        imeditsea_flag_s(i)=1
      end do
      imeditsea_region=0

      X=lon
      Y=lat

      if (INPOLYGON(X, Y, Xpoly, Ypoly, N)) then
        imeditsea_region=1
        if (idbug.gt.0) write(66,*)'1)INPOLYGON:',INPOLYGON(X, Y, 
     *  Xpoly, Ypoly, N)
      else
        imeditsea_region=-1
        if (idbug.gt.0) write(66,*)'2)INPOLYGON:',INPOLYGON(X, Y, 
     *  Xpoly, Ypoly, N)
      end if

      if (idbug.gt.0) then
        if (imeditsea_region.eq.1) 
     *   write(66,*)'lat=',Y,' lon=',X,' is INside of Meditsea region.'
        if (imeditsea_region.eq.-1)
     *   write(66,*)'lat=',Y,' lon=',X,' is OUTside of Meditsea region'
      end if

      do i = 1, n1
       if (imeditsea_region.ne.-1) then
          if (tem(i).lt.10.0.or.tem(i).gt.40.0) then
	    imeditsea_flag_t(i)=4
	    if (idbug.gt.0) then
	      write(66,*) 'Region check mediterranean sea 
     *	      temperature error--',i,' t=',tem(i)
            end if 	
	  end if 
	  
	  if (sal(i).lt.0.0.or.sal(i).gt.40.0) then
	    imeditsea_flag_s(i)=4
	    if (idbug.gt.0) then
	      write(66,*) 'Region check mediterranean sea 
     *	      salinity error--',i,' s=',sal(i)
            end if
	  end if

      end if !if (imeditsea_region.ne.-1)

      end do ! end_of_do i = 1, n1
      return    

      end

c--------------------------------------------------------------------------
c Revision date: 13-Dec-08 (YHD - low salinity range changed from 0 to 2 psu)
c                10-Feb-04 (HY - check red sea)
c    redsea_check.f
c    ============
c    Checks impossbile temperature and salinity in red sea.
c    The red sea:10N,40E;20N,50E;30N,30E;10N,40E
c                Temperature in range 21.7 to 40.0 degrees C
c                Salinity in range 0.0 to 41.0
c                -- from GOSUD REAL-TIME QC
c
c    output: iredsea_flag_t(or iredsea_flag_s)=1 means OK
c            iredsea_flag_t(or iredsea_flag_s)=4 means NOT ok.
c            iredsea_region=1 means in red sea
      
      subroutine redsea_check(lon,lat,tem,sal,n1,
     * iredsea_region,iredsea_flag_t,iredsea_flag_s)
      
      implicit none
      
      integer nmax,N,n1,i,idbug
      
      parameter(nmax=3,N=3)
      real Xpoly(nmax),Ypoly(nmax),X,Y
      real tem(*),sal(*),lon,lat,sal_lo,sal_hi,tem_lo,tem_hi
      integer iredsea_region,iredsea_flag_t(*)
      integer iredsea_flag_s(*)
      logical INPOLYGON
      data xpoly/40,30,50/
      data ypoly/10,30,20/
      data idbug /0/          ! make 1 to debug

      data xpoly/40,30,50/
      data ypoly/10,30,20/
      data idbug /0/          ! make 1 to debug

      data sal_lo,sal_hi /2.0,41.0/
      data tem_lo,tem_hi /21.7,40.0/
    
      if (idbug.gt.0) write(66,*)'<<<<< DEBUG from REDSEA_CHECK >>>>>'

      do i = 1, n1
        iredsea_flag_t(i)=1
        iredsea_flag_s(i)=1
      end do
      iredsea_region=0

      X=lon
      Y=lat
       
      if (INPOLYGON(X, Y, Xpoly, Ypoly, N)) then
         if (idbug.gt.0) write(66,*)'1)INPOLYGON:',INPOLYGON(X, Y, 
     *   Xpoly, Ypoly, N)
         iredsea_region=1
      else
         if (idbug.gt.0) write(66,*)'2)INPOLYGON:',INPOLYGON(X, Y, 
     *   Xpoly, Ypoly, N)
         iredsea_region=-1
      end if
      
      if (idbug.gt.0) then
        if (iredsea_region.eq.1) 
     *   write(66,*)'lat=',Y,' lon=',X,' is INside of Redsea region.'
        if (iredsea_region.eq.-1)
     *   write(66,*)'lat=',Y,' lon=',X,' is OUTside of Redsea region.'
      end if
 
      do i = 1, n1
      if (iredsea_region.eq.1) then

       if (tem(i).lt.tem_lo.or.tem(i).gt.tem_hi) then
	  iredsea_flag_t(i)=4
       if (idbug.eq.1) write(66,*) 'Region check red sea temperature err
     *or --> t=',tem(i)
        end if  ! end if (tem(i).lt.21.7.or.tem(i).gt.40.0) 
	  
	if (sal(i).lt.sal_lo.or.sal(i).gt.sal_hi) then
	  iredsea_flag_s(i)=4
       if (idbug.eq.1) write(66,*) 'Region check red sea salinity error 
     *-->s=',sal(i)
        end if ! if (sal(i).lt.sal_lo.or.sal(i).gt.sal_hi 
      end if !if (iredsea_region.eq.1)

      end do ! end_of_do i = 1, n1
      return
      end
      
C  Revison date: Jan-05-04 (YHD - DATA statement does not work)
c
       LOGICAL FUNCTION INPOLYGON(XIN, YIN, XCONV, YCONV, NCONV)
C
C  SUBROUTINE TO DETERMINE WHETHER OR NOT A POINT IS IN A POLYGON
C
C  INPUT PARAMETERS UNCHANGED IN THE ROUTINE
C  CONV CONTAINS THE COORDS OF THE POLYGON XCONV, YCONV
C  X AND Y ARE THE COORDINATES OF THE POINT TO BE CHECKED
C  NCONV IS THE NUMBER OF POINTS IN THE POLYGON
C
C  OUTPUT PARAMETERS
C  INPOLYGON IS UNSET ON ENTRY BUT IS RETURNED AS .TRUE. IF THE POINT
C  LIES INSIDE THE POLYGON, OTHERWISE RETURNED AS .FALSE.
C
      REAL XCONV(NCONV), YCONV(NCONV),  XIN, YIN
      REAL Z(4)
      REAL X,Y
c      DATA Z/-32701,-32701,32701,32701/
      X=XIN
      Y=YIN
      Z(1)=-32701.
      Z(2)=Z(1)
      Z(3)=32701.
      Z(4)=Z(3)

c      IF (Z(1).NE.-32701) GO TO 20
      IF ((Z(1)+32701).GT.0.001) GO TO 20
      DO 10 I = 1,NCONV
         Z(1)=MAX(Z(1),XCONV(I))
         Z(2)=MAX(Z(2),YCONV(I))
         Z(3)=MIN(Z(3),XCONV(I))
         Z(4)=MIN(Z(4),YCONV(I))
   10    CONTINUE
   20 INPOLYGON=.TRUE.
      IF(X .LT. Z(3) .OR. X .GT. Z(1)) INPOLYGON=.FALSE.
      IF(Y .LT. Z(4) .OR. Y .GT. Z(2)) INPOLYGON=.FALSE.
      IF(.NOT. INPOLYGON) RETURN
      J=0
      DO 90 I = 2,NCONV
         M=0
      IF ((YCONV(I-1)-Y)*(Y-YCONV(I))) 90,30,70
   30 IF (YCONV(I-1)-YCONV(I)) 60,40,50
   40 IF ((XCONV(I-1)-X)*(X-XCONV(I))) 90,100,100
   50    M=M-2
   60    M=M-1
   70    M=M+2
      IF ((Y-YCONV(I-1))*(XCONV(I)-XCONV(I-1))/(YCONV(I)-
     *YCONV(I-1))+XCONV(I-1)-X) 90,100,80
   80    J=J+M
   90    CONTINUE
      INPOLYGON=.FALSE.
      IF(J/4*4 .NE. J) INPOLYGON=.TRUE.
  100 RETURN
      END
