c   Revision date : 06-May013 VMH   - made number of speed check results (speed_flag array) into
c                                     a parameter, npp, and check if it goes out of bounds
c                   17-Oct-11 (ER   - Fixed this bug: if a position fails the previous profile
c                                     test, the rest were getting a '4' even if they were good
c                                     positions
c                   14-Oct-11 (ER   - Fixed incorrect assignment of bad_previous_row vector
c                   07-Sep-11 (ER   - Fixed this bug: when there is only 1 position and it
c                                     fails the previous profile test it wasn't adding a '-'
c                                     before the POSITION-QC column in qc file.
c                   24-Sep-09 (ER   - ihr=hour(jst)*100+mn(1) became ihr=hour(jst)*100+mn(k) )
c                                   - stop doing the no-positions-in-profile  
c                                     test since it is already done in qc_t
c                                   - algorith to determine bad positions changed
c                                   - new subroutine flag_worse_positions
c                                   - new subroutine pick_bad_pos
c                                   - new subroutine print_flag_table
c                                   - new subroutine print_nbr_bad_flags_per_pos
c                                   - new subroutine print_bad_positions
c                                   - new function distance: returns distance between 
c                                     2 coordinates.  It allowed the standardization of
c                                     the calculus of distance. (Method to calculate distance 
c                                     was not uniform: pos i v. pos j and pos i v.
c                                     previous profile used the same method, while pos i and 
c                                     pos j v. mean position used a different one.)
c                                   - adding many comments for readability.
c                                   - changes in layout of statements to make
c                                     them easier to read.)
c                    09-Sep-09 (YHD - a bug fixed on line 469)
c                    03-Jun-09 (YHD - Loosen the restriction on speed check.)
c                                     It requires at least one 4 to consider
c                                     a position bad.)
c                    10-Jun-08 (YHD - Decide which location is bad if there is
c                                     only one '4' left in the flag_table.
c                                     Refer to June 10, 2008 note.)
c                    24-Oct-07 (YHD - 3 loops on flag_table; any 4's left in
c                                     flag_table gets a status=7).
c                    04-Oct_07 (YHD - Go thru loops 2 times for best results)
c                    02-Oct-07 (YHD - Count 4's in hori/vert in flag_table)
c                    26-Sep-07 (YHD - Check a horizontal sum of flag table).
c                    17-Sep-07 (YHD - More correction on spd table)
c                    06-Sep-07 (YHD - 'mxs'=100 introduced for maximum
c                                     speed table array size & print out speed
c                                     table in trace output.)
c                    08-May-07 (YHD - A brand new approach for the speed check
c                                     a) cycle 0 or 1 case
c                                     b) cycle N against cycle N-1
c                                     c) within cycle N
c                    23-Nov-05 (YHD - Make sure that bounce float does not
c                                     check the distance only case.)
c                    22-Nov-05 (YHD - 'longdiff' is wrong if a position is
c                                     skipped case.
c                    23-Mar-05 (YHD - print out the which part and location
c                                     of positions when the speed check fails;
c                                     write this to *.trace file')
c                    01-Feb-05 (YHD - The criteria for speed check is still
c                                     tight (i.e. checks internal & external 
c                                     for 3 m/s) to catch any bad position or
c                                     time errors.  Internal speed check fails
c                                     always the first position so far.)
c                    27-Jan-05 (YHD - Make time2-time1 to be more than 30 min)
c                    25-Jan-05 (YHD - Use time1 to be i; time2 to be i+1 for
c                                     within position check and make sure that
c                                     time2-time1 be gt 10 min.
c                                     time1 used be the time for the first 
c                                     position.)
c                    21-Jan-05 (YHD - For '000' profile, use second time in 
c                                     the position & make sure its time is
c                                     larger than the lauch time).
c                    12-May-04 (YHD - More change on the restriction;
c                                     See 05/11/04 note)
c                    11-May-04 (YHD - Loosen the restriction on speed check
c                                     between the meta and the first profile)
c                    15-Mar-04 (YHD - For profile '000' case, use launch time
c                                     position as previous profile.)
c                    02-Oct-03 (YHD - All dimensions check)
c                    20-May-03 (YHD - At least 10 min. difference needed
c                                     in surface positions.)
c                    14-May-03 (YHD - if ispeed_flag=0 make ispeed_flag=1)
c                    25-Jun-02 (YHD - For '000' or '001' profiles, apply
c                                     within speed checks only)
c                    10-Aug-01 (YHD - dateline crossing case)
c                    19-Jun-01 (YHD - speed between the profiles)
c                    01-Jun-01 (YHD - impossible speed test added)
c    Speed_check.f
c    =============
c
c    Looks for outliers in the ARGOS position for one surface cycle.
c    yhd, 07-21-2000
c
      subroutine speed_check(profile_id,pnum,np,year,month,day,hour,mn,
     *           sec,xlon,ylat,itime_prev,xlon_prev,ylat_prev,
     *           ispeed_flag,speed,bad_row)

      implicit none

      interface
        subroutine pick_bad_pos(np,flag_table,bad_row,xlon,ylat,pos1,
     *    pos2)
          integer  :: np, pos1, pos2
          real, dimension(*) :: xlon, ylat 
          integer, dimension(*) :: bad_row
          integer, dimension(:,:) :: flag_table
        end subroutine pick_bad_pos 
      end interface

      double precision :: distance  ! function distance

      ! Constants
      real, parameter :: max_speed = 3.0  ! m/s
      integer, parameter :: idbug = 0  ! idbug=1 ==> in debug mode
      real, parameter :: max_dist = 10000  ! maximum distance in meters
      real, parameter :: max_dista = 50000 ! maximum distance in meters 
      integer, parameter :: mxs = 100 ! max. nbr of position in profile
      real, parameter :: pi = 3.14159265358979  
      real, parameter :: daysec = 24.*60.*60. ! seconds in a day
      real, parameter :: factor = 1.852*1.e3*60.
 
      
      integer status
      integer, parameter :: nnp = 3000; ! nnp=max number of positions
      integer np,year(*),month(*),day(*),hour(*),mn(*),sec(*),
     *        ntime,i,ispeed_flag,ihr,pnum,ip,speed_flag(nnp),
     *        flag_table(mxs,mxs)

      integer iyr_prev,imo_prev,ida_prev,ihr_prev,imin_prev,isec_prev,
     *        icnt,ibad_cnt,ibad_wthin,ibad_prv,ist,jst,k

      integer :: bad_row(*)  ! bad_row(i)=4 when pos i fails speed check
      integer :: bad_previous_row(mxs) !bad_previous_row(i)=4 when 
                                       !pos(i) fails speed check against
                                       !previous profile or launch   

      integer poss_bad_cnt(mxs),hori_row_cnt(mxs),
     *        vert_col_cnt(mxs),nloop
      integer max_bad_cnt 
      data    ibad_cnt,ibad_wthin,ibad_prv /3*0/

      real    xlon(*),ylat(*),speed(*),dx,dy,u,v,longdiff,
     *        longdiff_pro,sumh,sumv
      double precision :: d_dist  ! hold distancia between two pairs of coordinates
      real    time1,time2
      real    xlon_prev,ylat_prev,speed_prev,xlon1,xlon2,ylat1,ylat2
      double precision t1,t2,t_sec,dt
      character itime_prev*14,cnp*14,cnp1*14
      character profile_id*(*)
      character(len=50) :: title
      character(len=10) :: str_i

      data    nloop /0/
      
      if (idbug==1) write(66,*)'<<<<< DEBUG from SPEED_CHECK >>>>>'

      if (np.gt.mxs) then
        status=9
        write(50,*)'speed_check.f positions np > mxs case:',np,mxs
        write(50,*)'speed_check.f : STOP -> STATUS = ', status
        call exit(status)
      end if

      ! initializing variables that will hold results of tests 
      do k=1,np
        bad_row(k)=0 
        bad_previous_row(k)=0 
        do i=1,np
          flag_table(k,i)=0
        end do
      end do

c======================================================================= 
c  part a) profile number is 0 or 1 case:
c          All positions/times are compared against the launch postion/time.
c  part b) profile number > 1 and compared against the previous cycle
c          postion/time.
c
      if (idbug.gt.0) write(66,*)'profile:',profile_id

      if (pnum.le.1) then
        if (idbug.gt.0) write(66,*)'Launch date/position:'
      else
        if (idbug.gt.0) write(66,*)'Previous profile date/position:'
      end if

      if (idbug.gt.0) write(66,*)'Total number of positions:',np
      
      if (itime_prev(1:14).eq.'              ') then
        itime_prev(1:14)='99999999999999'
      else
        read(itime_prev(1:4),'(i4)') iyr_prev
        read(itime_prev(5:6),'(i2)') imo_prev
        read(itime_prev(7:8),'(i2)') ida_prev
        read(itime_prev(9:10),'(i2)') ihr_prev
        read(itime_prev(11:12),'(i2)') imin_prev
        read(itime_prev(13:14),'(i2)') isec_prev
      end if

      if (idbug>0) then
        write(66,90) iyr_prev,imo_prev,ida_prev,ihr_prev,
     *               imin_prev,isec_prev,xlon_prev,ylat_prev
 90    format('time1:yr/mon/day=',i4.4,'/',i2.2,'/',i2.2,2x,'hh:mm:ss=',
     *         i2.2,':',i2.2,':',i2.2,2x,'lon/lat=',f10.4,f10.4)
      end if

      ! Calculate t1 -- Time of previous profile or launch
      ihr=ihr_prev*100+imin_prev
      call julday(iyr_prev,imo_prev,ida_prev,ihr,time1)!launch or previous tim
      t1=time1+float(isec_prev)/daysec

      ! Latitude, Longitude of previous profile or launch
      xlon1=xlon_prev
      ylat1=ylat_prev
      
      icnt=0      ! a counter of number of speed check test
      do i=1,np

        icnt=icnt+1
        if(icnt > nnp) then
            status=9
            write( 6,*)'speed_check.f number of position tests (',
     *                 icnt,') exceed maximum (',nnp,') at sc1'
            write(50,*)'speed_check.f number of position tests (',
     *                 icnt,') exceed maximum (',nnp,') at sc1'
            write( 6,*)'speed_check.f : STOP -> STATUS = ', status
            write(50,*)'speed_check.f : STOP -> STATUS = ', status
            call exit(status)
        endif

        ! Time of position i 
        ihr = hour(i)*100+mn(i)
        call julday(year(i),month(i),day(i),ihr,time2)
        t2=time2+float(sec(i))/daysec

        ! Time elapsed betw previous profile or launch and pos i (in secs)
        dt=(t2-t1)*daysec

        ! Distance betw pos i and previous profile or launch
        xlon2=xlon(i)
        ylat2=ylat(i)
        d_dist = distance(ylat1, xlon1, ylat2, xlon2) 
             
        if (dt .gt. 0) then
          speed(icnt)=d_dist/dt
        else
          ! Time elapsed <= 0 !!!! 
          speed(icnt)=0
          if (pnum.le.1) then
            write(6,*)'----> speed_check (part a), warning: dt <= zero'
          else
            write(6,*)'----> speed_check (part b), warning: dt <= zero'
          end if
        end if

        if (speed(icnt)>max_speed) then
          speed_flag(icnt)=4
          flag_table(i,np+1)=4
          bad_previous_row(i)=4  ! Flag pos. as bad related to 
                                 ! previous profile or launch
        else
          speed_flag(icnt)=1
          flag_table(i,np+1)=1
        end if

      end do   ! do i=1,np

      ! Print results of this test for debugging purposes 
      if (idbug.gt.0) then
        write(66,93) 
        do i=1,np
          if (pnum.le.1) then
             write(66,91)i,speed(i),year(i),month(i), day(i),hour(i),
     *                   mn(i),sec(i),xlon(i),ylat(i),speed_flag(i)
          else
             write(66,92)i,speed(i),year(i),month(i), day(i),hour(i),
     *                   mn(i),sec(i),xlon(i),ylat(i),speed_flag(i)
          end if
        end do
      end if
 91   format('pos-laun',i2,' sp=',f7.4,1x,i4.4,'/',i2.2,'/',
     *i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
 92   format('pos-prev',i2,' sp=',f7.4,1x,i4.4,'/',i2.2,'/',
     *i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
 93   format('First part of speed check against launch or previous profil
     *e:')

c======================================================================= 
c    part c) Internal within profile positions check
c    velocity from point i to point i+1
c    Date-line crossing case
c    longdiff(xlon(i)-xlon(1)) < 180.0 (west -> east)
c    longdiff(xlon(i)-xlon(1)) > 180.0 (east -> west)
c
      if (idbug.gt.0) write(66,94)
      do k=1,np
        ist=k+1
        jst=k
        icnt=icnt+1

        if(icnt > nnp) then
            status=9
            write( 6,*)'speed_check.f number of position tests (',
     *                 icnt,') exceed maximum (',nnp,') at sc2'
            write(50,*)'speed_check.f number of position tests (',
     *                 icnt,') exceed maximum (',nnp,') at sc2'
            write( 6,*)'speed_check.f : STOP -> STATUS = ', status
            write(50,*)'speed_check.f : STOP -> STATUS = ', status
            call exit(status)
        endif



        if (np.le.1) speed_flag(icnt)=0

        ! Calculate t1, time of position k                 
        ihr=hour(jst)*100+mn(k)
        call julday(year(jst),month(jst),day(jst),ihr,time1)
        t1=time1+float(sec(jst))/daysec

        ! Latitude, Longitude of position k
        xlon1=xlon(jst)
        ylat1=ylat(jst)

        speed(icnt)=0
        speed_flag(icnt)=1

        do i=ist,np

          ! Latitude, Longitude of position i
          xlon2=xlon(i)
          ylat2=ylat(i)

          ! Distance betw positions k and i 
          d_dist = distance(ylat1, xlon1, ylat2, xlon2)

          ! Position i time
          ihr=hour(i)*100+mn(i)
          call julday(year(i),month(i),day(i),ihr,time2)    ! time2
          t_sec=float(sec(i))/daysec
          t2=time2+t_sec

          if ((time2-time1).le.30.0/(24.0*60.0)) then

            ! Time elapsed betw positions k and i <= 30 min
            ! Speed Check cannot be done for these positions

            speed_flag(icnt)=5
            if (idbug.gt.0) then 
              write(66,95)i-1,i,speed(icnt),year(i),month(i),day(i),
     *                    hour(i),mn(i),sec(i),xlon(i),ylat(i),
     *                    speed_flag(icnt)
              write(66,*)'-->i=',i,' time2-time1<30 min case: time1=',
     *                   time1,' time2=',time2
            end if
            flag_table(k,i)=5
          else 

            icnt=icnt+1

            if(icnt > nnp) then
                status=9
                write( 6,*)'speed_check.f number of position tests (',
     *                 icnt,') exceed maximum (',nnp,') at sc3'
                write(50,*)'speed_check.f number of position tests (',
     *                 icnt,') exceed maximum (',nnp,') at sc3'
                write( 6,*)'speed_check.f : STOP -> STATUS = ', status
                write(50,*)'speed_check.f : STOP -> STATUS = ', status
                call exit(status)
            endif


            ! Time elapsed betw previous profile and pos i (in seconds)
            dt=(t2-t1)*daysec  
            if (dt .gt. 0) then
              speed(icnt)=d_dist/dt
            else
              speed(icnt)=0
              print *,'----> speed_check (part c), warning: dt <= zero'
            endif
            speed_flag(icnt)=1
            flag_table(k,i)=1
            if (speed(icnt) .gt. max_speed) then
              speed_flag(icnt)=4
              flag_table(k,i)=4
c             bad_previous_row(icnt)=4  ! Flag pos. as bad related to 
c                                       ! previous profile or launch
            end if
            if (idbug.gt.0) then
              write(66,95)k,i,speed(icnt),year(i),month(i),day(i),
     *                    hour(i),mn(i),sec(i),xlon(i),ylat(i),
     *                    speed_flag(icnt)
            end if
          end if
        end do  ! do i=ist,np
      end do  ! do k=1,np

 94   format('Second part of speed check within positions:')
 95   format('pos',i2,'-pos',i2,' sp=',f8.4,1x,i4.4,'/',
     *i2.2,'/',i2.2,1x,i2.2,':',i2.2,':',i2.2,1x,f9.4,f9.4,' flag=',i1)
      if (idbug.gt.0) write(66,*)'icnt=',icnt

      ! Only one position on the profile (np=1)
      ! Only need to check against the previous profile (launch)
      if (np.eq.1) then
        if (bad_previous_row(1).eq.4) then
          bad_row(1)=4
          write(48,*)'Only 1 pos. and failed against previous profile.'
          ispeed_flag=4
          return
        else
          write(48,*)'Only 1 position & passed.'
          ispeed_flag=1
          return
        end if
      end if

      ! Only two positions (np=2) and test between positions failed 
      if ( (np.eq.2) .AND. (bad_row(1).eq.4) ) then 
        write(48,*)'Only 2 positions & failed within positions.'
        ispeed_flag=4
        return
      end if

      ! Print the speed flag table 
      title = 'Speed Flag Table: ' // profile_id 
      call print_flag_table(np,flag_table,title)

      ! Print bad positions (related to previous profile or launch) 
      title="Failed speed check against previous profile:"
      call print_bad_positions(np, bad_previous_row, title)

      ! Check between positions only (doesn't include check against
      ! previous profile or launch) 
      ! In each iteration, the position(s) with most bad flags
      ! are flagged as bad.
      ! This process is repeated until no position has more 
      ! than 1 bad flag.
      i=0
      do 
        i=i+1 
        write(str_i,*)i
        title="Speed Flag Table Loop # " // trim(adjustl(str_i)) 
        call flag_worse_positions(np,flag_table,bad_row,max_bad_cnt)
        write(48,*)'Loop=',i
        write(48,*)'max_bad_cnt=',max_bad_cnt
        call print_flag_table(np,flag_table,title)
        call print_nbr_bad_flags_per_pos(np,bad_row) 
        if (max_bad_cnt.le.1) then
          exit 
        end if
      end do

      if (max_bad_cnt.eq.1) then
        ! After previous loop, all positions have at most 1 bad flag. 
        ! But each bad flag involves 2 positions, so we need to pick 
        ! the bad one.  Criterion used: The bad position is the one  
        ! further away from the median of all positions --excluding  
        ! the pair in the "dispute" 
        do k=1,np
          do i=1,np
            if (flag_table(k,i).eq.4) then
              call pick_bad_pos(np,flag_table,bad_row,xlon,ylat,k,i)
            end if;
          end do;
        end do;
      end if;

      ispeed_flag=4 ! Initially assuming every position fails one of 
                    ! the two tests. If just one position is allright  
                    ! ispeed_flag will be changed to 1
      do k=1,np 
        if ((bad_row(k).eq.4) .OR. (bad_previous_row(k).eq.4))  then
          bad_row(k) = 4 ! for the case that bad_previous_row(k)=4 and
                         ! bad_row(i) <> 4
          write(48,*)'SPEED CHECK: Pos ',k,' gets a "
     *"-" sign front of class.'
          write(6,*)'SPEED CHECK: Pos ',k,' gets a "
     *-"  sign front of class.'
        else
          ispeed_flag=1
        end if
      end do 

      return
      end


! --------------------------------------------------------------------
! subroutine flag_worse_positions
!    E. Ramos 09/2009
!    Flag the 'worse' positions inside the profile
!    'Worse' defined as those positions (p) with the biggest count of:
!      flag_table(p,1..n+1)=4 and flag_table(1..p-1,p)=4   
! --------------------------------------------------------------------
      subroutine flag_worse_positions(np,flag_table,bad_row,max_bad_cnt)
      implicit none
      integer, parameter :: mxs = 100
      integer np, flag_table(100,100), bad_row(*) 
      integer i, k
      integer pos_bad_cnt(mxs) ! how many bad points in a given pos.
      integer max_bad_cnt ! maximum # of bad counts of any position 

      ! counting bad points of each position 
      do k=1,np
         pos_bad_cnt(k)=0   
      end do 
      do k=1,np
        do i=1,np
          if (flag_table(k,i).eq.4) then
            pos_bad_cnt(k)=pos_bad_cnt(k)+1
            pos_bad_cnt(i)=pos_bad_cnt(i)+1
          end if
        end do
      end do

      ! find out the maximum number of bad counts of any position 
      max_bad_cnt = 0
      do k=1,np
         if (pos_bad_cnt(k).gt.max_bad_cnt) then  
           max_bad_cnt = pos_bad_cnt(k)
         end if
      end do 

      if (max_bad_cnt.gt.1) then 
        ! flag positions with the max number of bad counts (the worse!)
        do k=1,np
          if (pos_bad_cnt(k).eq.max_bad_cnt) then
            bad_row(k)=4
            ! reverse the position's bad flags 
            ! first vertically
            do i=1,k-1
              if (flag_table(i,k).eq.4) then
                 flag_table(i,k)=-4
              end if
            end do
            ! then horizontally
            do i=k+1,np
              if (flag_table(k,i).eq.4) then
                 flag_table(k,i)=-4
              end if
            end do
          end if;
        end do
      end if

      return
      end subroutine flag_worse_positions


! ---------------------------------------------------------------------
! subroutine print_flag_table
!    E. Ramos 09/2009
!    Print the Flag Table
! ---------------------------------------------------------------------
      subroutine print_flag_table(np,flag_table,title)
      implicit none

      integer np, flag_table(100,100)
      integer i, k 
      character cnp*14,cnp1*14
      character(len=50) :: title

      cnp(1:4)='(3x,'
      cnp(7:14)='(i2,1x))'
      write(cnp(5:6),'(i2.2)') np+1
      cnp1(1:4)='(i3,'
      cnp1(7:14)='(i2,1x))'
      write(cnp1(5:6),'(i2.2)') np+1
      write(48,115)
 115  format(59('-'))
      write(48,*)title 
      write(48,cnp) (i,i=1,np+1)
      do k=1,np
        write(48,cnp1) k,(flag_table(k,i),i=1,np+1)
      end do
      write(48,115)

      end subroutine print_flag_table


! ---------------------------------------------------------------------
! subroutine print_nbr_bad_flags_per_pos 
!    E. Ramos 09/2009
!    Print the number of bad flags per position 
! ---------------------------------------------------------------------
      subroutine print_nbr_bad_flags_per_pos(np,bad_row)
      implicit none

      integer np, bad_row(*)
      integer k 
      
      write(48,*)'bad flags per position: ',(bad_row(k),k=1,np)

      return
      end subroutine print_nbr_bad_flags_per_pos

! ---------------------------------------------------------------------
! subroutine print_bad_positions 
!    E. Ramos 09/2009
!    Print the positions that failed the test against the previous 
!    profile or launch
!---------------------------------------------------------------------

      subroutine print_bad_positions(np, bad_pos, title)

      implicit none

      integer np, bad_pos(*)
      integer k, i, cnt_bad
      character(len=50) :: title
      character(len=80) :: line
      character(len=3) :: ci

      line=title 
      cnt_bad=0
      do k=1,np
         if (bad_pos(k).eq.4) then
           write(ci,*)k
           ci=trim(adjustl(ci))
           if (cnt_bad.eq.0) then
             line = trim(line) // ci
           else
             if (len(trim(line)) + len(ci).gt.80) then
                write(48,*)line
                line=ci
             else 
                line=trim(line) // "," // ci
             end if
           end if 
           cnt_bad=cnt_bad+1 
         end if
      end do
      if (cnt_bad.eq.0) then
        line=trim(line)//" NONE"
      end if
      write(48,*)line

      return
      end subroutine print_bad_positions


! ---------------------------------------------------------------------
! subroutine pick_bad_pos
!    E. Ramos 09/2009
!    Decides which position is the bad one based on it being further 
!    away of the mean of all positions excluding the 2 "contendents"
! ---------------------------------------------------------------------
      subroutine pick_bad_pos(np,flag_table,bad_row,xlon,ylat,pos1,
     *pos2)

      implicit none

      double precision :: distance ! function distance

      integer :: np, pos1, pos2
      real, dimension(*) :: xlon, ylat
      integer, dimension(*) :: bad_row
      integer, dimension(:,:) :: flag_table

      real :: sum_lat, sum_lon, mean_lat, mean_lon, d1, d2
      integer :: k, n 

      write(6,*)'Potentially bad positions:',pos1,pos2
      write(48,*)'Potentially bad positions:',pos1,pos2

      sum_lat=0.0
      sum_lon=0.0
      n=0
      do k=1,np
        if ((bad_row(k).ne.4) .AND. (k.ne.pos1) .AND. (k.ne.pos2)) then    
          sum_lat=sum_lat+ylat(k)
          sum_lon=sum_lon+xlon(k)
          n=n+1
        end if
      end do
      if (n.ge.1) then
        mean_lon=sum_lon/n
        mean_lat=sum_lat/n
        d1=distance(mean_lat, mean_lon, ylat(pos1), xlon(pos1))
        d2=distance(mean_lat, mean_lon, ylat(pos2), xlon(pos2))
        !d1=(xlon(pos1)-mean_lon)**2+(ylat(pos1)-mean_lat)**2
        !d2=(xlon(pos2)-mean_lon)**2+(ylat(pos2)-mean_lat)**2
        write(6,*)'mean_lat,mean_lon,d1,d2:',mean_lat,mean_lon,d1,d2
        write(48,*)'mean_lat,mean_lon,d1,d2:',mean_lat,mean_lon,d1,d2 
        if (d1.gt.d2) then
          write(6,*)'Choosen bad position:',pos1
          write(48,*)'Choosen bad position:',pos1
          bad_row(pos1)=4
        else
          write(6,*)'Choosen bad position:',pos2
          write(48,*)'Choosen bad position:',pos2
          bad_row(pos2)=4
        end if
      else
        write(6,*)'Cannot calculate mean. Both pos.flagged as bad'
        write(48,*)'Cannot calculate mean. Both pos.flagged as bad'
        bad_row(pos1)=4
        bad_row(pos2)=4
      end if

      return
      end subroutine pick_bad_pos

! ---------------------------------------------------------------------
! function distance 
!    E. Ramos 09/2009
!    Calculate distance between 2 pairs of coordinates 
! ---------------------------------------------------------------------
      double precision function distance( lat1, long1, lat2, long2 )

        implicit none
        real :: lat1, long1, lat2, long2 
        real :: longdiff, dx, dy
        real, parameter :: factor = 1.852*1.e3*60.
        real, parameter :: pi = 3.14159265358979

        longdiff=long2-long1
        if (longdiff.ge.180.0) longdiff=longdiff-360.0
        if (longdiff.le.-180.0) longdiff=longdiff+360.0
        dx=(longdiff)*factor*cos(pi/180*(lat1+lat2)/2.0)
        dy=(lat2-lat1)*factor

        distance = sqrt(dx*dx + dy*dy)
 
      end function distance 
