c    Revision Date: 23-Dec-13 (CS - improve initialization of isal_flag)
c                   19-Dec-13 (CS - improve handling of parallel profiles)
c                   03-Nov-10 (ERH - changed name from salgrad_check.f to
c                                    vert_gradient_check_s.f
c                   22-Jun-07 (YHD - Incomplete data case; 0.0 0.0 0.0 line)
c                   07-Dec-04 (YHD - created)
c
c    vert_gradient_check_s.f
c    =======================
c    This test detects when the difference between vertically adjacent
c    measurements is too steep.
c
c    Test value=|v2-(v3+v1)/2|
c
c    The test values exceeds 1.5 psu for pressures < 500 db or
c                    exceeds 0.5 psu for pressures >=500 db 
c    then flag it as bad.
c
c    inputs:
c    s1: salinity array of present profile
c    p1: pressure array of present profile
c    n1: number of data points of present profile
c    output:
c    isalg_flag=1  good     salinity gradient test passed.
c              =4  bad      salinity gradient test failed.
c
c
      subroutine vert_gradient_check_s(s1,p1,n1,isal_flag)

      implicit none
      integer n1,n0,isal_flag(*),i,m,icnt,idbug
      real    s1(*),p1(*),tv,fill_val,up_psu_allow,dn_psu_allow
      data    up_psu_allow,dn_psu_allow/1.5,0.5/
      data    fill_val /-999.0/
      data    idbug /0/

      if (idbug.gt.0) write(66,*)'<<< DEBUG from SALGRAD CHECK >>>'

c--- initialize
      do i=2,n1-1
        isal_flag(i)=1
      end do
      isal_flag(1)=0
      if (n1.gt.0) isal_flag(n1)=0

      do i=2,n1-1
        if (s1(i+1)-s1(i-1).eq.0.0) then
          tv=0.0
        else
          if (s1(i-1).eq.0.0.and.s1(i).eq.0.0.and.s1(i+1).ne.0.0) then
            tv=0.0
          else if (s1(i-1).ne.0.0.and.s1(i).ne.0.0.and.s1(i+1).eq.0.0) 
     *    then
            tv=abs(s1(i)-s1(i-1))
          else if (s1(i-1).ne.0.0.and.s1(i).eq.0.0.and.s1(i+1).eq.0.0) 
     *    then
            tv=0.0
          else
            tv=abs(s1(i)-(s1(i+1)+s1(i-1))/2.0)
          end if
        end if
        if (tv.gt.up_psu_allow.and.p1(i).lt.500.0) isal_flag(i)=4
        if (tv.gt.dn_psu_allow.and.p1(i).ge.500.0) isal_flag(i)=4
c         write(6,*)i,tv,s1(i-1),s1(i),s1(i+1)
      end do
      
c--- fill value cases
      do i=2,n1-1
        if (s1(i).eq.fill_val) then
          if (isal_flag(i-1).eq.4.and.i.gt.2) isal_flag(i-1)=1
          isal_flag(i)=0
          if (isal_flag(i+1).eq.4.and.i.lt.n1-1) isal_flag(i+1)=1
        end if
      end do
c
c--- For first point case.
c
      if (s1(1).eq.fill_val) isal_flag(1)=0
      if (s1(1).eq.0.0) isal_flag(1)=0

c
c--- Last point case.
c
      if (n1.gt.0) then
         if (s1(n1).eq.fill_val) isal_flag(n1)=0
         if (s1(n1).eq.0.0) isal_flag(1)=0
      end if

      if (idbug.gt.0) then
        do i=1,n1
         write(66,*)'i=',i,' p=',p1(i),' s=',s1(i),' flag=',isal_flag(i)
        end do
      end if

      return
      end
