c    Revision Date: 23-Dec-13 (CS - improve initialization of itemp_flag)
c                   19-Dec-13 (CS - improve handling of parallel profiles)
c                   18-Dec-13 (CS,RS - replaced the unofficial with official tests
C                                      based on the vert_gradient_check_s)
C                   03-Nov-10 (ERH - changed name from salgrad_check.f to
c                                    vert_gradient_check_t.f
c                   22-Jun-07 (YHD - Incomplete data case; 0.0 0.0 0.0 line)
c                   07-Dec-04 (YHD - created)
c
c    vert_gradient_check_t.f
c    =======================
c    This test detects when the difference between vertically adjacent
c    measurements is too steep.
c
c    Test value=|v2-(v3+v1)/2|
c
c    The test values exceeds 9.0 degrees for pressures < 500 db or
c                    exceeds 3.0 degrees for pressures >=500 db 
c    then flag it as bad.
c
c    inputs:
c    t1: temperature array of present profile
c    p1: pressure array of present profile
c    n1: number of data points of present profile
c    output:
c    itemp_flag=1  good     temperature gradient test passed.
c              =4  bad      temperature gradient test failed.
c
c
      subroutine vert_gradient_check_t(t1,p1,n1,itemp_flag)

      implicit none
      integer n1,n0,itemp_flag(*),i,m,icnt,idbug
      real    t1(*),p1(*),tv,fill_val,up_degrees_allow,dn_degrees_allow
      data    up_degrees_allow,dn_degrees_allow/9.0,3.0/
      data    fill_val /-9.0/
      data    idbug /0/

      if (idbug.gt.0) write(66,*)'<<< DEBUG from TEMPGRAD CHECK >>>'

c--- initialize
      do i=2,n1-1
        itemp_flag(i)=1
      end do
      itemp_flag(1)=0
      if (n1.gt.0) itemp_flag(n1)=0

      do i=2,n1-1
        if (t1(i+1)-t1(i-1).eq.0.0) then
          tv=0.0
        else
          if (t1(i-1).eq.0.0.and.t1(i).eq.0.0.and.t1(i+1).ne.0.0) then
            tv=0.0
          else if (t1(i-1).ne.0.0.and.t1(i).ne.0.0.and.t1(i+1).eq.0.0) 
     *    then
            tv=abs(t1(i)-t1(i-1))
          else if (t1(i-1).ne.0.0.and.t1(i).eq.0.0.and.t1(i+1).eq.0.0) 
     *    then
            tv=0.0
          else
            tv=abs(t1(i)-(t1(i+1)+t1(i-1))/2.0)
          end if
        end if
        if (tv.gt.up_degrees_allow.and.p1(i).lt.500.0) itemp_flag(i)=4
        if (tv.gt.dn_degrees_allow.and.p1(i).ge.500.0) itemp_flag(i)=4
c         write(6,*)i,tv,t1(i-1),t1(i),t1(i+1)
      end do
c--- fill value cases
      do i=2,n1-1
        if (t1(i).eq.fill_val) then
          if (itemp_flag(i-1).eq.4.and.i.gt.2) itemp_flag(i-1)=1
          itemp_flag(i)=0
          if (itemp_flag(i+1).eq.4.and.i.lt.n1-1) itemp_flag(i+1)=1
        end if
      end do
c
c--- For first point case.
c
      if (t1(1).eq.fill_val) itemp_flag(1)=0
      if (t1(1).eq.0.0) itemp_flag(1)=0
c
c--- Last point case.
c
      if (n1.gt.0) then
         if (t1(n1).eq.fill_val) itemp_flag(n1)=0
         if (t1(n1).eq.0.0) itemp_flag(n1)=0
      end if

      if (idbug.gt.0) then
        do i=1,n1
         write(66,*)'i=',i,' p=',p1(i),' t=',t1(i),' flag=',
     *             itemp_flag(i)
        end do
      end if

      return
      end
