#!/bin/sh
###########################################################################
# Revision Date: 19-Ju-08 (RS - add execf90QC failures to email subject)
#                25-Feb-08 (RS - add profiles without metafile)
#                29-Mar-07 (RS - in case of error create string for the
#                                email subject)
#                28-Feb-07 (RS - add adjsal failures)
#                08-Mar-06 (RS - streamline email text. Skip profile tests
#                                in email when a problem)
#                23-Aug-00 (RS - add prints)
#                09-Jun-00 (RS)
#
# report_metadata_stderror
# ========================
# Reports:  floats with no metadata
#           floats with no information in metafile
#           floats with no float type 
#           qc_process.sh standard-error (stderr) output file
#           
###########################################################################
script_name=$1
add2subj_file=$2

now=`date +'%h %d, %Y %T %p'`

echo "<============== Start  $script_name: $now ==============>"
echo "Entering Script: $0"

# Are there any profile with missing float type in metafile"
# ---------------------------------------------------------
add2subj=""

if [ -f $missing_metafile_list ]
then
  if [ -s $missing_metafile_list ]
  then
    add2subj="Missing meta file"
    msg="\nPHY profiles(s) without meta file"
    echo $msg                    >> $SUMMARY_REPORT
    cat $missing_metafile_list   >> $SUMMARY_REPORT
    echo $msg                    >> $EMAIL_MESSAGE
    cat $missing_metafile_list   >> $EMAIL_MESSAGE
    echo $msg
    cat $missing_metafile_list
  fi
  rm -f $missing_metafile_list
fi

if [ -f $noinfo_in_metadata_list ]
then
  if [ -s $noinfo_in_metadata_list ]
  then
    add2subj="Missing info in meta file"
    msg="\nMetadata file(s) with missing float type:"
    echo $msg                    >> $SUMMARY_REPORT
    cat $noinfo_in_metadata_list >> $SUMMARY_REPORT
    echo $msg                    >> $EMAIL_MESSAGE
    cat $noinfo_in_metadata_list >> $EMAIL_MESSAGE
    echo $msg
    cat $noinfo_in_metadata_list
  fi
  rm -f $noinfo_in_metadata_list
fi

# Any error reported by f90 (qc_t.x and/or qc_s.x)
# ------------------------------------------------
if [ $execf90_failures -ne 0 ]
then
  if [ "$add2subj" = "" ]
  then
    add2subj="F90QC Exec Failure"
  else
    add2subj="$add2subj,  F90QC Exec Failure"
  fi
fi

# Any error reported by 'adjsal2phy'? 
# -----------------------------------
if [ -f $error_adjsal2phy_list ]
then
  if [ -s $error_adjsal2phy_list ]
  then
    if [ "$add2subj" = "" ]
    then
      add2subj="adjsal2phy Error"
    else
      add2subj="$add2subj,  adjsal2phy Error"
    fi
    msg="\nError reported by 'adjsal2phy' for the following files:"
    echo $msg                  >> $SUMMARY_REPORT
    cat $error_adjsal2phy_list >> $SUMMARY_REPORT
    echo $msg                  >> $EMAIL_MESSAGE
    cat $error_adjsal2phy_list >> $EMAIL_MESSAGE
    echo $msg
    cat $error_adjsal2phy_list
  fi
  rm -f $error_adjsal2phy_list
fi

# Does script standard_error output file exists?
# ----------------------------------------------
if [ -f $qc_process_stderror -a -s $qc_process_stderror ]
then
  if [ "$add2subj" = "" ]
  then
    add2subj="Standard-Error"
  else
    add2subj="$add2subj, Standard-Error"
  fi
  msg="\nStandard errors were generated by $script_name. For further\n"
  msg=$msg"details look at the following error file: \n"
  msg=$msg"\t$qc_process_stderror"
  echo $msg >> $SUMMARY_REPORT
  echo $msg >> $EMAIL_MESSAGE
  echo $msg
else
  rm -f $qc_process_stderror
fi

echo "add2subj: $add2subj"

if [ "$add2subj" != "" ]
then
  echo $add2subj > $add2subj_file
fi
echo "<============== Finish $script_name: $now ==============>"

exit
